@@ -130,7 +130,7 @@ nsAccessibilityService::~nsAccessibilityService()
 }
 
 NS_IMPL_THREADSAFE_ISUPPORTS5(nsAccessibilityService, nsIAccessibilityService, nsIAccessibleRetrieval,
-                              nsIObserver, nsIWebProgressListener, nsISupportsWeakReference);
+                              nsIObserver, nsIWebProgressListener, nsISupportsWeakReference)
 
 // nsIObserver
 
@@ -161,7 +161,9 @@ NS_IMETHODIMP nsAccessibilityService::OnStateChange(nsIWebProgress *aWebProgress
 
   nsCOMPtr<nsIDOMWindow> domWindow;
   aWebProgress->GetDOMWindow(getter_AddRefs(domWindow));
-  NS_ASSERTION(domWindow, "No dom window for nsIWebProgress::OnStateChange");
+  // Bug 214049 OnStateChange can come from non UI threads.
+  if (!domWindow)
+    return NS_OK;
 
   nsCOMPtr<nsIDOMDocument> domDoc;
   domWindow->GetDocument(getter_AddRefs(domDoc));
@@ -215,16 +217,14 @@ nsAccessibilityService::GetInfo(nsISupports* aFrame, nsIFrame** aRealFrame, nsIW
 {
   NS_ASSERTION(aFrame,"Error -- 1st argument (aFrame) is null!!");
   *aRealFrame = NS_STATIC_CAST(nsIFrame*, aFrame);
-  nsCOMPtr<nsIContent> content;
-  (*aRealFrame)->GetContent(getter_AddRefs(content));
+  nsCOMPtr<nsIContent> content = (*aRealFrame)->GetContent();
   nsCOMPtr<nsIDOMNode> node(do_QueryInterface(content));
   if (!content || !node)
     return NS_ERROR_FAILURE;
   *aNode = node;
   NS_IF_ADDREF(*aNode);
 
-  nsCOMPtr<nsIDocument> document;
-  content->GetDocument(getter_AddRefs(document));
+  nsCOMPtr<nsIDocument> document = content->GetDocument();
   if (!document)
     return NS_ERROR_FAILURE;
 
@@ -234,10 +234,8 @@ nsAccessibilityService::GetInfo(nsISupports* aFrame, nsIFrame** aRealFrame, nsIW
 #endif
 
   // do_GetWR only works into a |nsCOMPtr| :-(
-  nsCOMPtr<nsIPresShell> tempShell;
-  nsCOMPtr<nsIWeakReference> weakShell;
-  document->GetShellAt(0, getter_AddRefs(tempShell));
-  weakShell = do_GetWeakReference(tempShell);
+  nsCOMPtr<nsIWeakReference> weakShell =
+    do_GetWeakReference(document->GetShellAt(0));
   NS_IF_ADDREF(*aShell = weakShell);
 
   return NS_OK;
@@ -253,8 +251,7 @@ nsAccessibilityService::GetShellFromNode(nsIDOMNode *aNode, nsIWeakReference **a
     return NS_ERROR_INVALID_ARG;
 
   // ---- Get the pres shell ----
-  nsCOMPtr<nsIPresShell> shell;
-  doc->GetShellAt(0, getter_AddRefs(shell));
+  nsIPresShell *shell = doc->GetShellAt(0);
   if (!shell)
     return NS_ERROR_FAILURE;
 
@@ -301,12 +298,11 @@ nsAccessibilityService::CreateRootAccessible(nsIPresShell *aShell,
   nsCOMPtr<nsIDOMNode> rootNode(do_QueryInterface(aDocument));
   NS_ENSURE_TRUE(rootNode, NS_ERROR_FAILURE);
 
-  nsCOMPtr<nsIDocument> parentDoc;
-  aDocument->GetParentDocument(getter_AddRefs(parentDoc));
+  nsIDocument *parentDoc = aDocument->GetParentDocument();
 
-  nsCOMPtr<nsIPresShell> presShell(aShell);
+  nsIPresShell *presShell = aShell;
   if (!presShell) {
-    aDocument->GetShellAt(0, getter_AddRefs(presShell));
+    presShell = aDocument->GetShellAt(0);
   }
   nsCOMPtr<nsIWeakReference> weakShell(do_GetWeakReference(presShell));
 
@@ -319,11 +315,6 @@ nsAccessibilityService::CreateRootAccessible(nsIPresShell *aShell,
     *aRootAcc = new nsRootAccessibleWrap(rootNode, weakShell);
   }
 
-  nsCOMPtr<nsIAccessibleEventReceiver> eventReceiver = 
-    do_QueryInterface(*aRootAcc);
-  NS_ASSERTION(eventReceiver, "Doc accessible does not receive events");
-  eventReceiver->AddEventListeners();
-
   nsCOMPtr<nsPIAccessNode> privateAccessNode(do_QueryInterface(*aRootAcc));
   privateAccessNode->Init();
 
@@ -354,10 +345,10 @@ nsAccessibilityService::CreateHTML4ButtonAccessible(nsISupports *aFrame, nsIAcce
 }
 
 NS_IMETHODIMP
-nsAccessibilityService::CreateHTMLAreaAccessible(nsIWeakReference *aShell, nsIDOMNode *aDOMNode, nsIAccessible *aAccParent, 
+nsAccessibilityService::CreateHTMLAreaAccessible(nsIWeakReference *aShell, nsIDOMNode *aDOMNode, nsIAccessible *aParent, 
                                                                nsIAccessible **_retval)
 {
-  *_retval = new nsHTMLAreaAccessible(aDOMNode, aAccParent, aShell);
+  *_retval = new nsHTMLAreaAccessible(aDOMNode, aParent, aShell);
 
   if (! *_retval) 
     return NS_ERROR_OUT_OF_MEMORY;
@@ -567,7 +558,7 @@ nsAccessibilityService::CreateHTMLObjectFrameAccessible(nsObjectFrame *aFrame,
   nsCOMPtr<nsIDOMNode> node;
   nsCOMPtr<nsIWeakReference> weakShell;
   nsIFrame *frame;
-  nsresult rv = GetInfo(NS_STATIC_CAST(nsIFrame*, aFrame), &frame, getter_AddRefs(weakShell), getter_AddRefs(node));
+  GetInfo(NS_STATIC_CAST(nsIFrame*, aFrame), &frame, getter_AddRefs(weakShell), getter_AddRefs(node));
 
   // 1) for object elements containing either HTML or TXT documents
   nsCOMPtr<nsIDOMDocument> domDoc;
@@ -651,9 +642,9 @@ nsAccessibilityService::CreateHTMLRadioButtonAccessibleXBL(nsIDOMNode *aNode, ns
 
 NS_IMETHODIMP 
 nsAccessibilityService::CreateHTMLSelectOptionAccessible(nsIDOMNode* aDOMNode, 
-																												 nsIAccessible *aAccParent, 
-																												 nsISupports* aPresContext, 
-																												 nsIAccessible **_retval)
+                                                         nsIAccessible *aParent, 
+                                                         nsISupports* aPresContext, 
+                                                         nsIAccessible **_retval)
 {
   nsCOMPtr<nsIPresContext> presContext(do_QueryInterface(aPresContext));
   NS_ASSERTION(presContext,"Error non prescontext passed to accessible factory!!!");
@@ -768,6 +759,7 @@ nsAccessibilityService::CreateHTMLTextAccessible(nsISupports *aFrame, nsIAccessi
 
   *_retval = nsnull;
 
+#ifndef MOZ_ACCESSIBILITY_ATK
   nsCOMPtr<nsITextContent> textContent(do_QueryInterface(node));
   if (textContent) {
     // If empty text string, don't include in accessible tree
@@ -791,6 +783,29 @@ nsAccessibilityService::CreateHTMLTextAccessible(nsISupports *aFrame, nsIAccessi
   }
     
   *_retval = new nsHTMLTextAccessible(node, weakShell);
+#else
+  // In ATK, we are only creating the accessible object for the text frame that is the FIRST
+  //   text frame in its block.
+  // A depth-first traversal from its nearest parent block frame will produce a frame sequence like
+  //   TTTBTTBTT... (B for block frame, T for text frame), so every T frame which is the immediate 
+  //   sibiling of B frame will be the FIRST text frame.
+  nsIFrame* parentFrame = nsAccessible::GetParentBlockFrame(frame);
+  if (! parentFrame)
+    return NS_ERROR_FAILURE; 
+
+  nsCOMPtr<nsIPresShell> presShell(do_QueryReferent(weakShell));
+  nsCOMPtr<nsIPresContext> presContext;
+  presShell->GetPresContext(getter_AddRefs(presContext));
+  nsIFrame* childFrame = nsnull;
+  parentFrame->FirstChild(presContext, nsnull, &childFrame);
+  PRInt32 index = 0;
+  nsIFrame* firstTextFrame = nsnull;
+  PRBool ret = nsAccessible::FindTextFrame(index, presContext, childFrame, &firstTextFrame, frame);
+  if (!ret || index != 0)
+    return NS_ERROR_FAILURE; 
+
+  *_retval = new nsHTMLBlockAccessible(node, weakShell);
+#endif //MOZ_ACCESSIBILITY_ATK
   if (! *_retval) 
     return NS_ERROR_OUT_OF_MEMORY;
 
@@ -1489,7 +1504,7 @@ NS_IMETHODIMP nsAccessibilityService::GetAccessibleFor(nsIDOMNode *aNode,
   nsCOMPtr<nsIContent> content(do_QueryInterface(aNode));
   nsCOMPtr<nsIDocument> doc;
   if (content) {
-    content->GetDocument(getter_AddRefs(doc));
+    doc = content->GetDocument();
   }
   else {// Could be document node
     doc = do_QueryInterface(aNode);
@@ -1497,8 +1512,7 @@ NS_IMETHODIMP nsAccessibilityService::GetAccessibleFor(nsIDOMNode *aNode,
   if (!doc)
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIPresShell> presShell;
-  doc->GetShellAt(0, getter_AddRefs(presShell));
+  nsIPresShell *presShell = doc->GetShellAt(0);
   return GetAccessibleInShell(aNode, presShell, aAccessible);
 }
 
@@ -1584,10 +1598,10 @@ nsresult nsAccessibilityService::GetAccessible(nsIDOMNode *aNode,
     if (! newAcc)
       return NS_ERROR_FAILURE;
     PRUint32 role, state;
-    newAcc->GetAccRole(&role);
+    newAcc->GetRole(&role);
     // don't create the accessible object for popup widget when it's not visible
     if (role == nsIAccessible::ROLE_MENUPOPUP) {
-      newAcc->GetAccState(&state);
+      newAcc->GetState(&state);
       if (state & (nsIAccessible::STATE_INVISIBLE | nsIAccessible::STATE_OFFSCREEN))
         return NS_ERROR_FAILURE;
     }
