@@ -37,32 +37,38 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "nsDocAccessible.h"
+#include "nsAccessibleEventData.h"
+#include "nsIAccessibilityService.h"
+#include "nsICommandManager.h"
+#include "nsIDocShell.h"
+#include "nsIDocShellTreeItem.h"
 #include "nsIDocument.h"
-#include "nsIPresShell.h"
-#include "nsIFrame.h"
+#include "nsIDOMCharacterData.h"
 #include "nsIDOMDocument.h"
 #include "nsIDOMEventTarget.h"
 #include "nsIDOMEvent.h"
 #include "nsIDOMDocument.h"
 #include "nsIDOMDocumentType.h"
 #include "nsIDOMNSDocument.h"
+#include "nsIDOMNSHTMLDocument.h"
 #include "nsIDOMMutationEvent.h"
+#include "nsIDOMWindow.h"
+#include "nsIEditorDocShell.h"
+#include "nsIEditingSession.h"
+#include "nsIFrame.h"
+#include "nsIInterfaceRequestorUtils.h"
+#include "nsINameSpaceManager.h"
+#include "nsIObserverService.h"
+#include "nsIPlaintextEditor.h"
+#include "nsIPresShell.h"
+#include "nsIScriptGlobalObject.h"
+#include "nsIServiceManager.h"
+#include "nsIScrollableView.h"
+#include "nsIURI.h"
+#include "nsIWebNavigation.h"
 #ifdef MOZ_XUL
 #include "nsIXULDocument.h"
 #endif
-#include "nsINameSpaceManager.h"
-#include "nsIWebNavigation.h"
-#include "nsIURI.h"
-#include "nsIInterfaceRequestorUtils.h"
-#include "nsIServiceManager.h"
-#include "nsHashtable.h"
-#include "nsIDocShell.h"
-#include "nsIDocShellTreeItem.h"
-#include "nsIScrollableView.h"
-#include "nsIAccessibilityService.h"
-#ifdef MOZ_ACCESSIBILITY_ATK
-#include "nsAccessibleEventData.h"
-#endif
 
 //=============================//
 // nsDocAccessible  //
@@ -72,10 +78,11 @@
 // construction 
 //-----------------------------------------------------
 nsDocAccessible::nsDocAccessible(nsIDOMNode *aDOMNode, nsIWeakReference* aShell):
-  nsAccessibleWrap(aDOMNode, aShell), 
+  nsBlockAccessible(aDOMNode, aShell), 
   mAccessNodeCache(nsnull), mWnd(nsnull), 
   mScrollWatchTimer(nsnull), mDocLoadTimer(nsnull), 
-  mWebProgress(nsnull), mBusy(eBusyStateUninitialized), 
+  mWebProgress(nsnull), mEditor(nsnull), 
+  mBusy(eBusyStateUninitialized), 
   mScrollPositionChangedTicks(0), mIsNewDocument(PR_FALSE)
 {
   // Because of the way document loading happens, the new nsIWidget is created before
@@ -86,26 +93,22 @@ nsDocAccessible::nsDocAccessible(nsIDOMNode *aDOMNode, nsIWeakReference* aShell)
   nsCOMPtr<nsIPresShell> shell(do_QueryReferent(mWeakShell));
   if (shell) {
     shell->GetDocument(getter_AddRefs(mDocument));
-    nsCOMPtr<nsIViewManager> vm;
-    shell->GetViewManager(getter_AddRefs(vm));
-    nsCOMPtr<nsIWidget> widget;
-    vm->GetWidget(getter_AddRefs(widget));
-    mWnd = widget->GetNativeData(NS_NATIVE_WINDOW);
+    nsIViewManager* vm = shell->GetViewManager();
+    if (vm) {
+      nsCOMPtr<nsIWidget> widget;
+      vm->GetWidget(getter_AddRefs(widget));
+      if (widget) {
+        mWnd = widget->GetNativeData(NS_NATIVE_WINDOW);
+      }
+    }
   }
   
   NS_ASSERTION(gGlobalDocAccessibleCache, "No global doc accessible cache");
-  PutCacheEntry(gGlobalDocAccessibleCache, NS_STATIC_CAST(void*, mWeakShell), this);
-#ifdef DEBUG
-  printf("\nATTENTION: New doc accessible for weak shell %x\n", mWeakShell);
-#endif
+  PutCacheEntry(*gGlobalDocAccessibleCache, mWeakShell, this);
 
   // XXX aaronl should we use an algorithm for the initial cache size?
-#ifdef OLD_HASH
-  mAccessNodeCache = new nsSupportsHashtable(kDefaultCacheSize);
-#else
   mAccessNodeCache = new nsInterfaceHashtable<nsVoidHashKey, nsIAccessNode>;
   mAccessNodeCache->Init(kDefaultCacheSize);
-#endif
 }
 
 //-----------------------------------------------------
@@ -115,45 +118,52 @@ nsDocAccessible::~nsDocAccessible()
 {
 }
 
-NS_IMPL_ISUPPORTS_INHERITED6(nsDocAccessible, nsAccessible, nsIAccessibleDocument, 
-                             nsIAccessibleEventReceiver, nsIWebProgressListener, 
-                             nsIDOMMutationListener,
-                             nsIScrollPositionListener, nsISupportsWeakReference)
+NS_INTERFACE_MAP_BEGIN(nsDocAccessible)
+  NS_INTERFACE_MAP_ENTRY(nsIAccessibleDocument)
+  NS_INTERFACE_MAP_ENTRY(nsPIAccessibleDocument)
+  NS_INTERFACE_MAP_ENTRY(nsIWebProgressListener)
+  NS_INTERFACE_MAP_ENTRY(nsIDOMMutationListener)
+  NS_INTERFACE_MAP_ENTRY(nsIScrollPositionListener)
+  NS_INTERFACE_MAP_ENTRY(nsISupportsWeakReference)
+  NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIAccessibleDocument)
+  NS_INTERFACE_MAP_ENTRY(nsIObserver)
+NS_INTERFACE_MAP_END_INHERITING(nsBlockAccessible)
 
-NS_IMETHODIMP nsDocAccessible::AddEventListeners()
-{
-  AddContentDocListeners();
-  return NS_OK;
-}
+NS_IMPL_ADDREF_INHERITED(nsDocAccessible, nsBlockAccessible)
+NS_IMPL_RELEASE_INHERITED(nsDocAccessible, nsBlockAccessible)
 
-/* void removeAccessibleEventListener (); */
-NS_IMETHODIMP nsDocAccessible::RemoveEventListeners()
-{
-  RemoveContentDocListeners();
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsDocAccessible::GetAccName(nsAString& aAccName) 
+NS_IMETHODIMP nsDocAccessible::GetName(nsAString& aName) 
 { 
-  return GetTitle(aAccName);
+  return GetTitle(aName);
 }
 
-NS_IMETHODIMP nsDocAccessible::GetAccRole(PRUint32 *_retval)
+NS_IMETHODIMP nsDocAccessible::GetRole(PRUint32 *_retval)
 {
   *_retval = ROLE_PANE;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsDocAccessible::GetAccValue(nsAString& aAccValue)
+NS_IMETHODIMP nsDocAccessible::GetValue(nsAString& aValue)
 {
-  return GetURL(aAccValue);
+  return GetURL(aValue);
 }
 
-NS_IMETHODIMP nsDocAccessible::GetAccState(PRUint32 *aAccState)
+NS_IMETHODIMP nsDocAccessible::GetState(PRUint32 *aState)
 {
-  *aAccState = STATE_FOCUSABLE;
+  *aState = STATE_FOCUSABLE;
   if (mBusy == eBusyStateLoading)
-    *aAccState |= STATE_BUSY;
+    *aState |= STATE_BUSY;
+
+#ifdef DEBUG
+  PRBool isEditable;
+  GetIsEditable(&isEditable);
+
+  if (isEditable) {
+    // Just for debugging, to show we're in editor on pane object
+    // We don't use STATE_MARQUEED for anything else
+    *aState |= STATE_MARQUEED; 
+  }
+#endif
   return NS_OK;
 }
 
@@ -164,8 +174,7 @@ NS_IMETHODIMP nsDocAccessible::GetURL(nsAString& aURL)
   if (!mDocument) {
     return NS_ERROR_FAILURE; // Document has been shut down
   }
-  nsCOMPtr<nsISupports> container;
-  mDocument->GetContainer(getter_AddRefs(container));
+  nsCOMPtr<nsISupports> container = mDocument->GetContainer();
   nsCOMPtr<nsIWebNavigation> webNav(do_GetInterface(container));
   nsCAutoString theURL;
   if (webNav) {
@@ -182,7 +191,12 @@ NS_IMETHODIMP nsDocAccessible::GetURL(nsAString& aURL)
 
 NS_IMETHODIMP nsDocAccessible::GetTitle(nsAString& aTitle)
 {
-  return mDocument? mDocument->GetDocumentTitle(aTitle): NS_ERROR_FAILURE;
+  if (mDocument) {
+    aTitle = mDocument->GetDocumentTitle();
+    return NS_OK;
+  }
+
+  return NS_ERROR_FAILURE;
 }
 
 NS_IMETHODIMP nsDocAccessible::GetMimeType(nsAString& aMimeType)
@@ -224,41 +238,129 @@ NS_IMETHODIMP nsDocAccessible::GetNameSpaceURIForID(PRInt16 aNameSpaceID, nsAStr
   return NS_ERROR_FAILURE;
 }
 
-NS_IMETHODIMP nsDocAccessible::GetCaretAccessible(nsIAccessibleCaret **aCaretAccessible)
+NS_IMETHODIMP nsDocAccessible::GetCaretAccessible(nsIAccessible **aCaretAccessible)
 {
   // We only have a caret accessible on the root document
   *aCaretAccessible = nsnull;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsDocAccessible::GetWindow(void **aWindow)
+NS_IMETHODIMP nsDocAccessible::GetWindowHandle(void **aWindow)
 {
   *aWindow = mWnd;
   return NS_OK;
 }
 
+NS_IMETHODIMP nsDocAccessible::GetWindow(nsIDOMWindow **aDOMWin)
+{
+  *aDOMWin = nsnull;
+  if (!mDocument) {
+    return NS_ERROR_FAILURE;  // Accessible is Shutdown()
+  }
+  nsCOMPtr<nsIDOMWindow> domWindow(do_QueryInterface(mDocument->GetScriptGlobalObject()));
+
+  if (!domWindow)
+    return NS_ERROR_FAILURE;  // No DOM Window
+
+  NS_ADDREF(*aDOMWin = domWindow);
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsDocAccessible::GetDocument(nsIDOMDocument **aDOMDoc)
+{
+  nsCOMPtr<nsIDOMDocument> domDoc(do_QueryInterface(mDocument));
+  *aDOMDoc = domDoc;
+
+  if (domDoc) {
+    NS_ADDREF(*aDOMDoc);
+    return NS_OK;
+  }
+
+  return NS_ERROR_FAILURE;
+}
+
+void nsDocAccessible::CheckForEditor()
+{
+  if (!mDocument) {
+    return;  // No document -- we've been shut down
+  }
+  nsCOMPtr<nsIDOMWindow> domWindow(do_QueryInterface(mDocument->GetScriptGlobalObject()));
+  if (!domWindow)
+    return;  // No DOM Window
+
+  nsCOMPtr<nsISupports> container = mDocument->GetContainer();
+  nsCOMPtr<nsIEditingSession> editingSession(do_GetInterface(container));
+  if (!editingSession)
+    return; // No editing session interface
+
+  editingSession->GetEditorForWindow(domWindow, getter_AddRefs(mEditor));
+}
+
+NS_IMETHODIMP nsDocAccessible::GetIsEditable(PRBool *aIsEditable)
+{
+  *aIsEditable = PR_FALSE;
+  if (mEditor) {
+    PRUint32 flags;
+    mEditor->GetFlags(&flags);
+    *aIsEditable = (flags & nsIPlaintextEditor::eEditorReadonlyMask) == 0;
+  }
+  return NS_OK;
+}
+
 NS_IMETHODIMP nsDocAccessible::GetCachedAccessNode(void *aUniqueID, nsIAccessNode **aAccessNode)
 {
   NS_ASSERTION(mAccessNodeCache, "No accessibility cache for document");
-  GetCacheEntry(mAccessNodeCache, aUniqueID, aAccessNode); // Addrefs for us
+  GetCacheEntry(*mAccessNodeCache, aUniqueID, aAccessNode); // Addrefs for us
   return NS_OK;
 }
 
 NS_IMETHODIMP nsDocAccessible::CacheAccessNode(void *aUniqueID, nsIAccessNode *aAccessNode)
 {
   NS_ASSERTION(mAccessNodeCache, "No accessibility cache for document");
-  PutCacheEntry(mAccessNodeCache, aUniqueID, aAccessNode);
+  PutCacheEntry(*mAccessNodeCache, aUniqueID, aAccessNode);
   return NS_OK;
 }
 
+NS_IMETHODIMP nsDocAccessible::Init()
+{
+  // Hook up our new accessible with our parent
+  if (!mParent) {
+    nsIDocument *parentDoc = mDocument->GetParentDocument();
+    if (parentDoc) {
+      nsIContent *ownerContent = parentDoc->FindContentForSubDocument(mDocument);
+      nsCOMPtr<nsIDOMNode> ownerNode(do_QueryInterface(ownerContent));
+      if (ownerNode) {
+        nsCOMPtr<nsIAccessibilityService> accService = 
+          do_GetService("@mozilla.org/accessibilityService;1");
+        if (accService) {
+          // XXX aaronl: ideally we would traverse the presshell chain
+          // Since there's no easy way to do that, we cheat and use
+          // the document hierarchy. GetAccessibleFor() is bad because
+          // it doesn't support our concept of multiple presshells per doc.
+          // It should be changed to use GetAccessibleInWeakShell()
+          nsCOMPtr<nsIAccessible> accParent;
+          accService->GetAccessibleFor(ownerNode, getter_AddRefs(accParent));
+          nsCOMPtr<nsPIAccessible> privateParent(do_QueryInterface(accParent));
+          if (privateParent) {
+            SetParent(accParent);
+            privateParent->SetFirstChild(this);
+          }
+        }
+      }
+    }
+  }
+  AddEventListeners();
+  return nsBlockAccessible::Init();
+}  
+
+
 NS_IMETHODIMP nsDocAccessible::Destroy()
 {
-#ifdef OLD_HASH
-  nsVoidKey key(NS_STATIC_CAST(void*, mWeakShell));
-  gGlobalDocAccessibleCache->Remove(&key);
-#else
-  gGlobalDocAccessibleCache->Remove(NS_STATIC_CAST(void*, mWeakShell));
-#endif
+  NS_ASSERTION(gGlobalDocAccessibleCache, "No global doc accessible cache");
+  if (gGlobalDocAccessibleCache) {
+    gGlobalDocAccessibleCache->Remove(NS_STATIC_CAST(void*, mWeakShell));
+  }
   return Shutdown();
 }
 
@@ -268,9 +370,12 @@ NS_IMETHODIMP nsDocAccessible::Shutdown()
     return NS_OK;  // Already shutdown
   }
 
+  RemoveEventListeners();
+
   mWeakShell = nsnull;  // Avoid reentrancy
 
-  RemoveEventListeners();
+  mEditor = nsnull;
+
   if (mScrollWatchTimer) {
     mScrollWatchTimer->Cancel();
     mScrollWatchTimer = nsnull;
@@ -282,19 +387,13 @@ NS_IMETHODIMP nsDocAccessible::Shutdown()
   mWebProgress = nsnull;
 
   if (mAccessNodeCache) {
-#ifdef OLD_HASH
-    nsSupportsHashtable *hashToClear = mAccessNodeCache; // Avoid reentrency
-#else
-    nsInterfaceHashtable<nsVoidHashKey, nsIAccessNode> *hashToClear = mAccessNodeCache; // Avoid reentrency
-#endif
+    ClearCache(*mAccessNodeCache);
     mAccessNodeCache = nsnull;
-    ClearCache(hashToClear);
-    delete hashToClear;
   }
 
-  NS_ASSERTION(gGlobalDocAccessibleCache, "Global doc cache does not exist");
   mDocument = nsnull;
-  return nsAccessibleWrap::Shutdown();
+
+  return nsBlockAccessible::Shutdown();
 }
 
 nsIFrame* nsDocAccessible::GetFrame()
@@ -308,14 +407,53 @@ nsIFrame* nsDocAccessible::GetFrame()
   return root;
 }
 
-void nsDocAccessible::GetBounds(nsRect& aBounds, nsIFrame** aRelativeFrame)
+void nsDocAccessible::GetBoundsRect(nsRect& aBounds, nsIFrame** aRelativeFrame)
 {
   *aRelativeFrame = GetFrame();
-  if (*aRelativeFrame)
-    (*aRelativeFrame)->GetRect(aBounds);
+
+  nsIDocument *document = mDocument;
+  nsIDocument *parentDoc = nsnull;
+
+  while (document) {
+    nsIPresShell *presShell = document->GetShellAt(0);
+    if (!presShell) {
+      return;
+    }
+    nsIViewManager* vm = presShell->GetViewManager();
+
+    nsIScrollableView* scrollableView = nsnull;
+    if (vm)
+      vm->GetRootScrollableView(&scrollableView);
+
+    nsRect viewBounds(0, 0, 0, 0);
+    if (scrollableView) {
+      const nsIView *view = nsnull;
+      scrollableView->GetClipView(&view);
+      if (view) {
+        viewBounds = view->GetBounds();
+      }
+    }
+    else {
+      nsIView *view;
+      vm->GetRootView(view);
+      if (view) {
+        viewBounds = view->GetBounds();
+      }
+    }
+
+    if (parentDoc) {  // After first time thru loop
+      aBounds.IntersectRect(viewBounds, aBounds);
+    }
+    else {  // First time through loop
+      aBounds = viewBounds;
+    }
+
+    document = parentDoc = document->GetParentDocument();
+  }
 }
 
-void nsDocAccessible::AddContentDocListeners()
+
+nsresult nsDocAccessible::AddEventListeners()
 {
   // 1) Set up scroll position listener
   // 2) Set up web progress listener - we need to know 
@@ -326,62 +464,69 @@ void nsDocAccessible::AddContentDocListeners()
   //    Do this only for top level content documents
 
   nsCOMPtr<nsIPresShell> presShell(GetPresShell());
-  if (!presShell)
-    return;
-
-  AddScrollListener(presShell);
-
-  nsCOMPtr<nsISupports> container;
-  mDocument->GetContainer(getter_AddRefs(container));
+  NS_ENSURE_TRUE(presShell, NS_ERROR_FAILURE);
 
+  nsCOMPtr<nsISupports> container = mDocument->GetContainer();
   nsCOMPtr<nsIDocShellTreeItem> docShellTreeItem(do_QueryInterface(container));
-  if (!docShellTreeItem)
-    return;
+  NS_ENSURE_TRUE(docShellTreeItem, NS_ERROR_FAILURE);
 
   // Make sure we're a content docshell
   // We don't want to listen to chrome progress
   PRInt32 itemType;
   docShellTreeItem->GetItemType(&itemType);
 
-  if (itemType != nsIDocShellTreeItem::typeContent)
-    return;
+  PRBool isContent = (itemType == nsIDocShellTreeItem::typeContent);
+  PRBool isLoading = isContent;
 
-  // Make sure we're the top content doc shell 
-  // We don't want to listen to iframe progress
-  nsCOMPtr<nsIDocShellTreeItem> topOfContentTree;
-  docShellTreeItem->GetSameTypeRootTreeItem(getter_AddRefs(topOfContentTree));
-  if (topOfContentTree != docShellTreeItem)
-    return;
+  if (isContent) {
+    AddScrollListener(presShell);
+    CheckForEditor();
+  
+    if (!mEditor) {
+      // We're not an editor yet, but we might become one
+      nsCOMPtr<nsICommandManager> commandManager = do_GetInterface(docShellTreeItem);
+      if (commandManager) {
+        commandManager->AddCommandObserver(this, "obs_documentCreated");
+      }
+    }
+
+    // Make sure we're the top content doc shell 
+    // We don't want to listen to iframe progress
+    nsCOMPtr<nsIDocShellTreeItem> topOfContentTree;
+    docShellTreeItem->GetSameTypeRootTreeItem(getter_AddRefs(topOfContentTree));
+    if (topOfContentTree != docShellTreeItem) {
+      mBusy = eBusyStateDone;
+      return NS_OK;
+    }
+  }
   
   nsCOMPtr<nsIPresContext> context; 
   presShell->GetPresContext(getter_AddRefs(context));
-  if (!context)
-    return;
+  NS_ENSURE_TRUE(context, NS_ERROR_FAILURE);
 
   mWebProgress = do_GetInterface(docShellTreeItem);
-  if (!mWebProgress)
-    return;
+  NS_ENSURE_TRUE(mWebProgress, NS_ERROR_FAILURE);
 
   mWebProgress->AddProgressListener(this, nsIWebProgress::NOTIFY_LOCATION | 
                                     nsIWebProgress::NOTIFY_STATE_DOCUMENT);
 
 
+  mWebProgress->GetIsLoadingDocument(&isLoading);
+
   mIsNewDocument = PR_TRUE;
   mBusy = eBusyStateLoading;
-  PRBool isLoading;
 
-  mWebProgress->GetIsLoadingDocument(&isLoading);
   if (!isLoading) {
     // If already loaded, fire "done loading" event after short timeout
     // If we fired the event here, we'd get reentrancy problems
-    // Otherwise it will be fired from OnStateChange when the load is done
+    // Otherwise, if the doc is still loading, 
+    // the event will be fired from OnStateChange when the load is done
     mDocLoadTimer = do_CreateInstance("@mozilla.org/timer;1");
     if (mDocLoadTimer) {
       mDocLoadTimer->InitWithFuncCallback(DocLoadCallback, this, 1,
                                           nsITimer::TYPE_ONE_SHOT);
     }
   }
-
   // add ourself as a mutation event listener 
   // (this slows down mozilla about 3%, but only used when accessibility APIs active)
   nsCOMPtr<nsIDOMEventTarget> target(do_QueryInterface(mDocument));
@@ -404,9 +549,11 @@ void nsDocAccessible::AddContentDocListeners()
   rv = target->AddEventListener(NS_LITERAL_STRING("DOMNodeRemovedFromDocument"), 
     this, PR_TRUE);
   NS_ASSERTION(NS_SUCCEEDED(rv), "failed to register listener");
+
+  return rv;
 }
 
-void nsDocAccessible::RemoveContentDocListeners()
+nsresult nsDocAccessible::RemoveEventListeners()
 {
   // Remove listeners associated with content documents
 
@@ -418,7 +565,9 @@ void nsDocAccessible::RemoveContentDocListeners()
 
   // Remove scroll position listener
   nsCOMPtr<nsIPresShell> presShell(do_QueryReferent(mWeakShell));
-  RemoveScrollListener(presShell);
+  if (presShell) {
+    RemoveScrollListener(presShell);
+  }
 
   nsCOMPtr<nsIDOMEventTarget> target(do_QueryInterface(mDocument));
   NS_ASSERTION(target, "No dom event target for document");
@@ -428,6 +577,31 @@ void nsDocAccessible::RemoveContentDocListeners()
   target->RemoveEventListener(NS_LITERAL_STRING("DOMNodeRemoved"), this, PR_TRUE);
   target->RemoveEventListener(NS_LITERAL_STRING("DOMNodeInsertedIntoDocument"), this, PR_TRUE);
   target->RemoveEventListener(NS_LITERAL_STRING("DOMNodeRemovedFromDocument"), this, PR_TRUE);
+
+  if (mScrollWatchTimer) {
+    mScrollWatchTimer->Cancel();
+    mScrollWatchTimer = nsnull;
+  }
+
+  if (mDocLoadTimer) {
+    mDocLoadTimer->Cancel();
+    mDocLoadTimer = nsnull;
+  }
+
+  nsCOMPtr<nsISupports> container = mDocument->GetContainer();
+  nsCOMPtr<nsIDocShellTreeItem> docShellTreeItem(do_QueryInterface(container));
+  NS_ENSURE_TRUE(docShellTreeItem, NS_ERROR_FAILURE);
+
+  PRInt32 itemType;
+  docShellTreeItem->GetItemType(&itemType);
+  if (itemType == nsIDocShellTreeItem::typeContent) {
+    nsCOMPtr<nsICommandManager> commandManager = do_GetInterface(docShellTreeItem);
+    if (commandManager) {
+      commandManager->RemoveCommandObserver(this, "obs_documentCreated");
+    }
+  }
+
+  return NS_OK;
 }
 
 void nsDocAccessible::FireDocLoadFinished()
@@ -435,45 +609,17 @@ void nsDocAccessible::FireDocLoadFinished()
   if (!mDocument || !mWeakShell)
     return;  // Document has been shut down
 
-  // Hook up our new accessible with our parent
-  if (!mParent) {
-    nsCOMPtr<nsIDocument> parentDoc;
-    mDocument->GetParentDocument(getter_AddRefs(parentDoc));
-    if (parentDoc) {
-      nsCOMPtr<nsIContent> ownerContent;
-      parentDoc->FindContentForSubDocument(mDocument, 
-                                           getter_AddRefs(ownerContent));
-      nsCOMPtr<nsIDOMNode> ownerNode(do_QueryInterface(ownerContent));
-      if (ownerNode) {
-        nsCOMPtr<nsIAccessibilityService> accService = 
-          do_GetService("@mozilla.org/accessibilityService;1");
-        if (accService) {
-          // XXX aaronl: ideally we would traverse the presshell chain
-          // Since there's no easy way to do that, we cheat and use
-          // the document hierarchy. GetAccessibleFor() is bad because
-          // it doesn't support our concept of multiple presshells per doc.
-          // It should be changed to use GetAccessibleInWeakShell()
-          nsCOMPtr<nsIAccessible> accParent;
-          accService->GetAccessibleFor(ownerNode, getter_AddRefs(accParent));
-          SetAccParent(accParent);
-          if (accParent) {
-            accParent->SetAccFirstChild(this);
-          }
-        }
-      }
-    }
-  }
-  
   if (mIsNewDocument) {
     mIsNewDocument = PR_FALSE;
 
     if (mBusy != eBusyStateDone) {
-      mBusy = eBusyStateDone;
 #ifndef MOZ_ACCESSIBILITY_ATK
-      FireToolkitEvent(nsIAccessibleEventReceiver::EVENT_STATE_CHANGE, this, nsnull);
+      FireToolkitEvent(nsIAccessibleEvent::EVENT_STATE_CHANGE, this, nsnull);
 #endif
     }
   }
+
+  mBusy = eBusyStateDone;
 }
 
 void nsDocAccessible::DocLoadCallback(nsITimer *aTimer, void *aClosure)
@@ -498,7 +644,7 @@ void nsDocAccessible::ScrollTimerCallback(nsITimer *aTimer, void *aClosure)
     // We only want to fire accessibilty scroll event when scrolling stops or pauses
     // Therefore, we wait for no scroll events to occur between 2 ticks of this timer
     // That indicates a pause in scrolling, so we fire the accessibilty scroll event
-    docAcc->FireToolkitEvent(nsIAccessibleEventReceiver::EVENT_SCROLLINGEND, docAcc, nsnull);
+    docAcc->FireToolkitEvent(nsIAccessibleEvent::EVENT_SCROLLINGEND, docAcc, nsnull);
     docAcc->mScrollPositionChangedTicks = 0;
     if (docAcc->mScrollWatchTimer) {
       docAcc->mScrollWatchTimer->Cancel();
@@ -509,32 +655,30 @@ void nsDocAccessible::ScrollTimerCallback(nsITimer *aTimer, void *aClosure)
 
 void nsDocAccessible::AddScrollListener(nsIPresShell *aPresShell)
 {
-  nsCOMPtr<nsIViewManager> vm;
-
+  nsIViewManager* vm = nsnull;
   if (aPresShell)
-    aPresShell->GetViewManager(getter_AddRefs(vm));
+    vm = aPresShell->GetViewManager();
 
   nsIScrollableView* scrollableView = nsnull;
   if (vm)
     vm->GetRootScrollableView(&scrollableView);
 
   if (scrollableView)
-    scrollableView->AddScrollPositionListener(NS_STATIC_CAST(nsIScrollPositionListener *, this));
+    scrollableView->AddScrollPositionListener(this);
 }
 
 void nsDocAccessible::RemoveScrollListener(nsIPresShell *aPresShell)
 {
-  nsCOMPtr<nsIViewManager> vm;
-
+  nsIViewManager* vm = nsnull;
   if (aPresShell)
-    aPresShell->GetViewManager(getter_AddRefs(vm));
+    vm = aPresShell->GetViewManager();
 
   nsIScrollableView* scrollableView = nsnull;
   if (vm)
     vm->GetRootScrollableView(&scrollableView);
 
   if (scrollableView)
-    scrollableView->RemoveScrollPositionListener(NS_STATIC_CAST(nsIScrollPositionListener *, this));
+    scrollableView->RemoveScrollPositionListener(this);
 }
 
 NS_IMETHODIMP nsDocAccessible::ScrollPositionWillChange(nsIScrollableView *aView, nscoord aX, nscoord aY)
@@ -565,8 +709,9 @@ NS_IMETHODIMP nsDocAccessible::ScrollPositionDidChange(nsIScrollableView *aScrol
 NS_IMETHODIMP nsDocAccessible::OnStateChange(nsIWebProgress *aWebProgress,
   nsIRequest *aRequest, PRUint32 aStateFlags, nsresult aStatus)
 {
-  if ((aStateFlags & STATE_IS_DOCUMENT) && (aStateFlags & STATE_STOP))
+  if ((aStateFlags & STATE_IS_DOCUMENT) && (aStateFlags & STATE_STOP)) {
     FireDocLoadFinished();   // Doc is ready!
+  }
 
   return NS_OK;
 }
@@ -594,13 +739,13 @@ NS_IMETHODIMP nsDocAccessible::OnLocationChange(nsIWebProgress *aWebProgress,
     mBusy = eBusyStateLoading; 
     // Fire a "new doc has started to load" event
 #ifndef MOZ_ACCESSIBILITY_ATK
-    FireToolkitEvent(nsIAccessibleEventReceiver::EVENT_STATE_CHANGE, this, nsnull);
+    FireToolkitEvent(nsIAccessibleEvent::EVENT_STATE_CHANGE, this, nsnull);
 #else
     AtkChildrenChange childrenData;
     childrenData.index = -1;
     childrenData.child = 0;
     childrenData.add = PR_FALSE;
-    FireToolkitEvent(nsIAccessibleEventReceiver::EVENT_REORDER , this, &childrenData);
+    FireToolkitEvent(nsIAccessibleEvent::EVENT_REORDER , this, &childrenData);
 #endif
   }
 
@@ -623,10 +768,52 @@ NS_IMETHODIMP nsDocAccessible::OnSecurityChange(nsIWebProgress *aWebProgress,
   return NS_OK;
 }
 
+NS_IMETHODIMP nsDocAccessible::Observe(nsISupports *aSubject, const char *aTopic,
+                                       const PRUnichar *aData)
+{
+  if (!nsCRT::strcmp(aTopic,"obs_documentCreated")) {
+    CheckForEditor();
+    NS_ASSERTION(mEditor, "Should have editor if we see obs_documentCreated");
+  }
+
+  return NS_OK;
+}
+
+void nsDocAccessible::GetEventShell(nsIDOMNode *aNode, nsIPresShell **aEventShell)
+{
+  // XXX aaronl - this is not ideal.
+  // We could avoid this whole section and the fallible 
+  // doc->GetShellAt(0) by putting the event handler
+  // on nsDocAccessible instead.
+  // The disadvantage would be that we would be seeing some events
+  // for inner documents that we don't care about.
+  *aEventShell = nsnull;
+  nsCOMPtr<nsIDOMDocument> domDocument;
+  aNode->GetOwnerDocument(getter_AddRefs(domDocument));
+  nsCOMPtr<nsIDocument> doc(do_QueryInterface(domDocument));
+  if (doc) {
+    *aEventShell = doc->GetShellAt(0);
+    NS_IF_ADDREF(*aEventShell);
+  }
+}
+
+void nsDocAccessible::GetEventDocAccessible(nsIDOMNode *aNode, 
+                                            nsIAccessibleDocument **aAccessibleDoc)
+{
+  *aAccessibleDoc = nsnull;
+  nsCOMPtr<nsIPresShell> eventShell;
+  GetEventShell(aNode, getter_AddRefs(eventShell));
+  nsCOMPtr<nsIWeakReference> weakEventShell(do_GetWeakReference(eventShell));
+  if (!weakEventShell) {
+    return;
+  }
+  GetDocAccessibleFor(weakEventShell, aAccessibleDoc);
+}
+
 // ---------- Mutation event listeners ------------
 NS_IMETHODIMP nsDocAccessible::NodeInserted(nsIDOMEvent* aEvent)
 {
-  HandleMutationEvent(aEvent, nsIAccessibleEventReceiver::EVENT_CREATE);
+  HandleMutationEvent(aEvent, nsIAccessibleEvent::EVENT_CREATE);
 
   return NS_OK;
 }
@@ -635,14 +822,14 @@ NS_IMETHODIMP nsDocAccessible::NodeRemoved(nsIDOMEvent* aEvent)
 {
   // The related node for the event will be the parent of the removed node or subtree
 
-  HandleMutationEvent(aEvent, nsIAccessibleEventReceiver::EVENT_DESTROY);
+  HandleMutationEvent(aEvent, nsIAccessibleEvent::EVENT_DESTROY);
 
   return NS_OK;
 }
 
 NS_IMETHODIMP nsDocAccessible::SubtreeModified(nsIDOMEvent* aEvent)
 {
-  HandleMutationEvent(aEvent, nsIAccessibleEventReceiver::EVENT_REORDER);
+  HandleMutationEvent(aEvent, nsIAccessibleEvent::EVENT_REORDER);
  
   return NS_OK;
 }
@@ -684,7 +871,7 @@ NS_IMETHODIMP nsDocAccessible::CharacterDataModified(nsIDOMEvent* aMutationEvent
   return NS_OK;
 }
 
-void nsDocAccessible::InvalidateCacheSubtree(nsIDOMNode *aStartNode)
+NS_IMETHODIMP nsDocAccessible::InvalidateCacheSubtree(nsIDOMNode *aStartNode)
 {
   // Invalidate cache subtree
   // We have to check for accessibles for each dom node by traversing DOM tree
@@ -692,18 +879,26 @@ void nsDocAccessible::InvalidateCacheSubtree(nsIDOMNode *aStartNode)
   // Otherwise we might miss the nsAccessNode's that are not nsAccessible's.
 
   if (!aStartNode)
-    return;
+    return NS_ERROR_FAILURE;
   nsCOMPtr<nsIDOMNode> iterNode(aStartNode), nextNode;
   nsCOMPtr<nsIAccessNode> accessNode;
 
   do {
-    GetCachedAccessNode(NS_STATIC_CAST(void*, iterNode), 
-                        getter_AddRefs(accessNode));
+    GetCachedAccessNode(iterNode, getter_AddRefs(accessNode));
     if (accessNode) {
       // XXX aaronl todo: accessibles that implement their own subtrees,
       // like html combo boxes and xul trees, need to shutdown all of their own
       // children when they override Shutdown()
-      accessNode->Shutdown();
+
+      // Don't shutdown our doc object!
+      if (accessNode != NS_STATIC_CAST(nsIAccessNode*, this)) { 
+        void *uniqueID;
+        accessNode->GetUniqueID(&uniqueID);
+        nsCOMPtr<nsPIAccessNode> privateAccessNode(do_QueryInterface(accessNode));
+        privateAccessNode->Shutdown();
+        // Remove from hash table as well
+        mAccessNodeCache->Remove(uniqueID);
+      }
     }
 
     iterNode->GetFirstChild(getter_AddRefs(nextNode));
@@ -723,7 +918,7 @@ void nsDocAccessible::InvalidateCacheSubtree(nsIDOMNode *aStartNode)
     do {
       iterNode->GetParentNode(getter_AddRefs(nextNode));
       if (!nextNode || nextNode == aStartNode) {
-        return;
+        return NS_OK;
       }
       nextNode->GetNextSibling(getter_AddRefs(iterNode));
       if (iterNode)
@@ -732,49 +927,72 @@ void nsDocAccessible::InvalidateCacheSubtree(nsIDOMNode *aStartNode)
     } while (PR_TRUE);
   }
   while (iterNode && iterNode != aStartNode);
+
+  return NS_OK;
 }
 
-void nsDocAccessible::HandleMutationEvent(nsIDOMEvent *aEvent, PRUint32 aAccessibleEventType) 
+NS_IMETHODIMP 
+nsDocAccessible::GetAccessibleInParentChain(nsIDOMNode *aNode, 
+                                            nsIAccessible **aAccessible)
 {
-  if (mBusy != eBusyStateDone)
-    return;  // We don't want mutation events until doc is fully loaded
+  nsCOMPtr<nsIAccessibilityService> accService = 
+    do_GetService("@mozilla.org/accessibilityService;1");
+  if (!accService)
+    return nsnull;
 
-  nsCOMPtr<nsIDOMEvent> domEvent(do_QueryInterface(aEvent));
-  nsCOMPtr<nsIDOMEventTarget> domEventTarget;
-  if (domEvent) {
-    domEvent->GetTarget(getter_AddRefs(domEventTarget));
+  nsCOMPtr<nsIDOMNode> currentNode(aNode), parentNode;
+
+  while (NS_FAILED(accService->GetAccessibleInWeakShell(currentNode, mWeakShell, 
+                                                        aAccessible))) {
+    currentNode->GetParentNode(getter_AddRefs(parentNode));
+    NS_ASSERTION(parentNode, "Impossible! Crawled up parent chain without "
+                             "finding accessible. There should have at least "
+                             "been a document accessible at the root.");
+    if (!parentNode) {
+      // XXX Todo We need to figure out why this is happening.
+      // For now, return safely.
+      return NS_ERROR_FAILURE;
+    }
+    currentNode = parentNode;
   }
+
+  return NS_OK;
+}
+
+void nsDocAccessible::HandleMutationEvent(nsIDOMEvent *aEvent, PRUint32 aAccessibleEventType)
+{
+  if (mBusy == eBusyStateLoading) {
+    // We need this unless bug 90983 is fixed -- 
+    // We only want mutation events after the doc is finished loading
+    return; 
+  }
+  nsCOMPtr<nsIDOMMutationEvent> mutationEvent(do_QueryInterface(aEvent));
+  NS_ASSERTION(mutationEvent, "Not a mutation event!");
+
+  nsCOMPtr<nsIDOMEventTarget> domEventTarget;
+  mutationEvent->GetTarget(getter_AddRefs(domEventTarget));
+
   nsCOMPtr<nsIDOMNode> targetNode(do_QueryInterface(domEventTarget));
-  if (!targetNode)
+  nsCOMPtr<nsIDOMNode> subTreeToInvalidate;
+  mutationEvent->GetRelatedNode(getter_AddRefs(subTreeToInvalidate));
+  NS_ASSERTION(subTreeToInvalidate, "No old sub tree being replaced in DOMSubtreeModified");
+
+  if (!targetNode) {
+    targetNode = subTreeToInvalidate;
+  }
+  else if (aAccessibleEventType == nsIAccessibleEvent::EVENT_REORDER) {
+    subTreeToInvalidate = targetNode; // targetNode is parent for DOMNodeRemoved event
+  }
+
+  nsCOMPtr<nsIAccessibleDocument> docAccessible;
+  GetEventDocAccessible(subTreeToInvalidate, getter_AddRefs(docAccessible));
+  if (!docAccessible)
     return;
 
-  nsCOMPtr<nsIDOMNode> subTreeNode;
-  if (aAccessibleEventType == nsIAccessibleEventReceiver::EVENT_CREATE) {
-    // DOMNodeInserted
-    // We'll need to invalidate the child count and cached children of
-    // the direct parent, otherwise no new accessibles will appear at this
-    // place in the tree.
-    // However, don't invalidate the entire subtree - most of it is still good.
-    targetNode->GetParentNode(getter_AddRefs(subTreeNode));
-    NS_ASSERTION(subTreeNode, "No valid parent of insertion");
-    targetNode = subTreeNode;
-  }
-  else {
-    if (aAccessibleEventType == nsIAccessibleEventReceiver::EVENT_REORDER) {
-      // DOMSubtreeModified
-      // The event target is the parent of the reordered tree
-      // We don't want to invalidate that entire subtree.
-      nsCOMPtr<nsIDOMMutationEvent> mutationEvent(do_QueryInterface(aEvent));
-      NS_ASSERTION(mutationEvent, "Not a mutation event!");
-      mutationEvent->GetRelatedNode(getter_AddRefs(subTreeNode));
-      NS_ASSERTION(subTreeNode, "No old sub tree being replaced in DOMSubtreeModified");
-    }
-    else {
-      // nsIAccessibleEventReceiver::EVENT_DESTROY / DOMNodeRemoved
-      subTreeNode = targetNode;
-    }
-    InvalidateCacheSubtree(subTreeNode);
-  }
+  nsCOMPtr<nsPIAccessibleDocument> privateDocAccessible =
+    do_QueryInterface(docAccessible);
+
+  privateDocAccessible->InvalidateCacheSubtree(subTreeToInvalidate);
 
   // We need to get an accessible for the mutation event's target node
   // If there is no accessible for that node, we need to keep moving up the parent
@@ -782,28 +1000,32 @@ void nsDocAccessible::HandleMutationEvent(nsIDOMEvent *aEvent, PRUint32 aAccessi
   // We will use this accessible to fire the accessible mutation event.
   // We're guaranteed success, because we will eventually end up at the doc accessible,
   // and there is always one of those.
-  // XXX aaronl todo: are MSAA mutation events always fired on the container?
-  //                  if so, we always need to get the dom parent at least once
-
-  nsCOMPtr<nsIAccessibilityService> accService = 
-    do_GetService("@mozilla.org/accessibilityService;1");
-  if (!accService)
-    return;
 
   nsCOMPtr<nsIAccessible> accessible;
-  while (NS_FAILED(accService->GetAccessibleInWeakShell(targetNode, mWeakShell, 
-                                                        getter_AddRefs(accessible)))) {
-    targetNode->GetParentNode(getter_AddRefs(subTreeNode));
-    NS_ASSERTION(subTreeNode, "Crawled up parent chain without finding accessible");
-    targetNode = subTreeNode;
-  }
-
-  accessible->InvalidateChildren();
+  docAccessible->GetAccessibleInParentChain(targetNode, getter_AddRefs(accessible));
+  nsCOMPtr<nsPIAccessible> privateAccessible(do_QueryInterface(accessible));
+  if (!privateAccessible)
+    return;
+  privateAccessible->InvalidateChildren();
 
 #ifdef XP_WIN
-  // Windows MSAA clients crash if the listen to create or destroy events
-  aAccessibleEventType = nsIAccessibleEventReceiver::EVENT_REORDER;
+  // Windows MSAA clients crash if they listen to create or destroy events
+  aAccessibleEventType = nsIAccessibleEvent::EVENT_REORDER;
 #endif
-  accessible->FireToolkitEvent(aAccessibleEventType, accessible, nsnull);
+  privateAccessible->FireToolkitEvent(aAccessibleEventType, accessible, nsnull);
+}
+
+NS_IMETHODIMP nsDocAccessible::FireToolkitEvent(PRUint32 aEvent, nsIAccessible* aAccessible, void* aData)
+{
+  nsCOMPtr<nsIObserverService> obsService =
+    do_GetService("@mozilla.org/observer-service;1");
+  if (!obsService) {
+    return NS_ERROR_FAILURE;
+  }
+
+  nsCOMPtr<nsIAccessibleEvent> accEvent = new nsAccessibleEventData(aEvent, aAccessible, this, aData);
+  NS_ENSURE_TRUE(accEvent, NS_ERROR_OUT_OF_MEMORY);
+
+  return obsService->NotifyObservers(accEvent, NS_ACCESSIBLE_EVENT_TOPIC, nsnull);
 }
 
