@@ -125,8 +125,7 @@ NS_IMETHODIMP nsRootAccessible::GetRole(PRUint32 *aRole)
   *aRole = ROLE_PANE;
 
   // If it's a <dialog>, use ROLE_DIALOG instead
-  nsCOMPtr<nsIContent> rootContent;
-  mDocument->GetRootContent(getter_AddRefs(rootContent));
+  nsIContent *rootContent = mDocument->GetRootContent();
   if (rootContent) {
     nsCOMPtr<nsIDOMElement> rootElement(do_QueryInterface(rootContent));
     if (rootElement) {
@@ -270,7 +269,7 @@ void nsRootAccessible::GetEventShell(nsIDOMNode *aNode, nsIPresShell **aEventShe
 {
   // XXX aaronl - this is not ideal.
   // We could avoid this whole section and the fallible 
-  // doc->GetShellAt(0, ...) by putting the event handler
+  // doc->GetShellAt(0) by putting the event handler
   // on nsDocAccessible instead.
   // The disadvantage would be that we would be seeing some events
   // for inner documents that we don't care about.
@@ -280,8 +279,10 @@ void nsRootAccessible::GetEventShell(nsIDOMNode *aNode, nsIPresShell **aEventShe
   if (!doc) {   // This is necessary when the node is the document node
     doc = do_QueryInterface(aNode);
   }
-  if (doc)
-    doc->GetShellAt(0, aEventShell);
+  if (doc) {
+    *aEventShell = doc->GetShellAt(0);
+    NS_IF_ADDREF(*aEventShell);
+  }
 }
 
 // --------------- nsIDOMEventListener Methods (3) ------------------------
@@ -307,19 +308,19 @@ NS_IMETHODIMP nsRootAccessible::HandleEvent(nsIDOMEvent* aEvent)
     optionTargetNode = do_QueryInterface(selectItem);
   }
 
+  nsCOMPtr<nsIPresShell> eventShell;
+  GetEventShell(targetNode, getter_AddRefs(eventShell));
+
 #ifdef MOZ_ACCESSIBILITY_ATK
   nsCOMPtr<nsIDOMHTMLAnchorElement> anchorElement(do_QueryInterface(targetNode));
   if (anchorElement) {
     nsCOMPtr<nsIDOMNode> blockNode;
     // For ATK, we don't create any individual object for hyperlink, use its parent who has block frame instead
-    nsAccessible::GetParentBlockNode(targetNode, getter_AddRefs(blockNode));
-    targetNode = blockNode;
+    if (NS_SUCCEEDED(nsAccessible::GetParentBlockNode(eventShell, targetNode, getter_AddRefs(blockNode))))
+      targetNode = blockNode;
   }
 #endif
 
-  nsCOMPtr<nsIPresShell> eventShell;
-  GetEventShell(targetNode, getter_AddRefs(eventShell));
-
   nsCOMPtr<nsIAccessible> accessible;
   if (!eventShell ||
       NS_FAILED(mAccService->GetAccessibleInShell(targetNode, eventShell,
@@ -483,10 +484,12 @@ NS_IMETHODIMP nsRootAccessible::HandleEvent(nsIDOMEvent* aEvent)
       FireAccessibleFocusEvent(accessible, targetNode);
     }
   }
-  else if (eventType.EqualsIgnoreCase("popupshowing")) 
-    FireAccessibleFocusEvent(accessible, targetNode);
-  else if (eventType.EqualsIgnoreCase("popuphiding")) 
+  else if (eventType.EqualsIgnoreCase("popupshowing")) {
     FireAccessibleFocusEvent(accessible, targetNode);
+  }
+  else if (eventType.EqualsIgnoreCase("popuphiding")) {
+    //FireAccessibleFocusEvent(accessible, targetNode);
+  }
 #endif
   return NS_OK;
 }
