@@ -33,6 +33,10 @@
  * the terms of any one of the MPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
+ 
+ /* This file implements an XPCOM object which represents a calendar object. It is a container
+ of events and prepares responses for queries made upon the calendar in whole. The task of reading from
+ and writing to the calendar data storage is currently performed by this object */
 
 #ifndef WIN32
 #include <unistd.h>
@@ -58,6 +62,8 @@
 #include "nsIDOMWindowInternal.h"
 #include "nsIDOMDocument.h"
 #include "nsIDOMElement.h"
+#include "nsIPrefBranch.h"
+#include "nsIPrefService.h"
 
 
 extern "C" {
@@ -298,7 +304,7 @@ oeICalImpl::~oeICalImpl()
  * Notice that the second parameter to the macro is the static IID accessor
  * method, and NOT the #defined IID.
  */
-NS_IMPL_ISUPPORTS1(oeICalImpl, oeIICal);
+NS_IMPL_ISUPPORTS1(oeICalImpl, oeIICal)
 
 EventList *oeICalImpl::GetEventList()
 {
@@ -951,6 +957,14 @@ NS_IMETHODIMP oeICalImpl::ModifyEvent(oeIICalEvent *icalevent, char **retid)
 #endif
     icalset *stream;
     icalcomponent *vcalendar;
+    nsresult rv;
+
+    //This might be a TODO object. If so then call the appropriate function.
+    nsCOMPtr<oeIICalTodo> icaltodo;
+    rv = icalevent->QueryInterface(NS_GET_IID(oeIICalTodo), (void **)&icaltodo);
+    if( NS_SUCCEEDED( rv ) ) {
+        return ModifyTodo( icaltodo, retid );
+    }
 
     stream = icalfileset_new(serveraddr);
     if ( !stream ) {
@@ -980,7 +994,6 @@ NS_IMETHODIMP oeICalImpl::ModifyEvent(oeIICalEvent *icalevent, char **retid)
                 icalfileset_remove_component( stream, fetchedvcal );
                 icalcomponent_free( fetchedvcal );
             }
-            nsresult rv;
             if( NS_FAILED( rv = NS_NewICalEvent((oeIICalEvent**) &oldevent ))) {
                 nsMemory::Free( *retid );
                 *retid = nsnull;
@@ -1043,7 +1056,6 @@ NS_IMETHODIMP oeICalImpl::ModifyEvent(oeIICalEvent *icalevent, char **retid)
     for( unsigned int i=0; i<observercount; i++ ) {
         nsCOMPtr<oeIICalObserver>observer;
         m_observerlist->QueryElementAt( i, NS_GET_IID(oeIICalObserver), getter_AddRefs(observer));
-        nsresult rv;
         rv = observer->OnModifyItem( icalevent, oldevent );
         #ifdef ICAL_DEBUG
         if( NS_FAILED( rv ) ) {
@@ -1653,7 +1665,7 @@ oeICalImpl::RemoveTodoObserver(oeIICalTodoObserver *observer)
         PRUint32 observercount;
         m_todoobserverlist->Count( &observercount );
         for( unsigned int i=0; i<observercount; i++ ) {
-            nsCOMPtr<oeIICalObserver>tmpobserver;
+            nsCOMPtr<oeIICalTodoObserver>tmpobserver;
             m_todoobserverlist->QueryElementAt( i, NS_GET_IID(oeIICalTodoObserver), getter_AddRefs(tmpobserver));
             if( observer == tmpobserver ) {
                 m_todoobserverlist->RemoveElementAt( i );
@@ -1739,6 +1751,8 @@ void oeICalImpl::SetupAlarmManager() {
         return;
     }
 
+    static icaltimetype lastcheck = icaltime_null_time();
+
     UpdateCalendarIcon( false );
 
     PRTime todayinms = PR_Now();
@@ -1750,6 +1764,9 @@ void oeICalImpl::SetupAlarmManager() {
 
     icaltimetype nextalarm = icaltime_null_time();
     EventList *tmplistptr = &m_eventlist;
+
+    int processmissed = -1; // -1 means the value has not been read from the preferences. 0 or 1 are valid values.
+
     while( tmplistptr ) {
         oeICalEventImpl *event = (oeICalEventImpl *)(tmplistptr->event);
         if( event ) {
@@ -1765,9 +1782,24 @@ void oeICalImpl::SetupAlarmManager() {
                     printf( "ALARM WENT OFF: %s\n", icaltime_as_ical_string( alarmtime ) );
                     #endif
                     
+                    nsresult rv;
+                    if( processmissed == -1 ) {
+                        nsCOMPtr<nsIPrefBranch> prefBranch = do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
+                        if ( NS_SUCCEEDED(rv) && prefBranch ) {
+                            rv = prefBranch->GetBoolPref("calendar.alarms.showmissed", &processmissed);
+                        } else {
+                            processmissed = true; //if anything goes wrong just consider the default setting
+                        }
+                    }
+
+                    if( !processmissed ) {
+                        time_t timediff = icaltime_as_timet( now ) - icaltime_as_timet( alarmtime );
+                        if( timediff > 30 ) //if alarmtime is older than 30 seconds it won't be processed.
+                            continue;
+                    }
+
                     UpdateCalendarIcon( true );
 
-                    nsresult rv;
                     oeIICalEventDisplay* eventDisplay;
                     rv = NS_NewICalEventDisplay( event, &eventDisplay );
                     #ifdef ICAL_DEBUG
@@ -1802,6 +1834,82 @@ void oeICalImpl::SetupAlarmManager() {
         }
         tmplistptr = tmplistptr->next;
     }
+
+    TodoList *tmptodolistptr = &m_todolist;
+    while( tmptodolistptr ) {
+        oeIICalTodo *todo = tmptodolistptr->todo;
+        if( todo ) {
+            oeICalEventImpl *event = ((oeICalTodoImpl *)todo)->GetBaseEvent();
+            icaltimetype begin=icaltime_null_time();
+            begin.year = 1970; begin.month=1; begin.day=1;
+            icaltimetype alarmtime = begin;
+            do {
+                alarmtime = event->GetNextAlarmTime( alarmtime );
+                if( icaltime_is_null_time( alarmtime ) )
+                    break;
+                if( icaltime_compare( alarmtime, now ) <= 0 ) {
+                    #ifdef ICAL_DEBUG
+                    printf( "ALARM WENT OFF: %s\n", icaltime_as_ical_string( alarmtime ) );
+                    #endif
+                    
+                    nsresult rv;
+                    if( processmissed == -1 ) {
+                        nsCOMPtr<nsIPrefBranch> prefBranch = do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
+                        if ( NS_SUCCEEDED(rv) && prefBranch ) {
+                            rv = prefBranch->GetBoolPref("calendar.alarms.showmissed", &processmissed);
+                        } else {
+                            processmissed = true; //if anything goes wrong just consider the default setting
+                        }
+                    }
+
+                    if( !processmissed ) {
+                        time_t timediff = icaltime_as_timet( now ) - icaltime_as_timet( alarmtime );
+                        if( timediff > 30 ) //if alarmtime is older than 30 seconds it won't be processed.
+                            continue;
+                    }
+
+                    UpdateCalendarIcon( true );
+
+                    oeIICalEventDisplay* eventDisplay;
+                    rv = NS_NewICalEventDisplay( todo, &eventDisplay );
+                    #ifdef ICAL_DEBUG
+                    if( NS_FAILED( rv ) ) {
+                        printf( "oeICalImpl::SetupAlarmManager() : WARNING Cannot create oeIICalEventDisplay instance: %x\n", rv );
+                    }
+                    #endif
+                    icaltimetype eventtime = event->CalculateEventTime( alarmtime );
+                    eventDisplay->SetDisplayDate( ConvertToPrtime( eventtime ) );
+                    PRUint32 observercount;
+                    //Here we should be using the todo observer list but nothing implements 
+                    //alarm handling for todos yet so we'll just use the one for events
+                    m_observerlist->Count( &observercount ); 
+                    for( unsigned int i=0; i<observercount; i++ ) {
+                        nsCOMPtr<oeIICalObserver>observer;
+                        m_observerlist->QueryElementAt( i, NS_GET_IID(oeIICalObserver), getter_AddRefs(observer));
+                        rv = observer->OnAlarm( eventDisplay );
+                        #ifdef ICAL_DEBUG
+                        if( NS_FAILED( rv ) ) {
+                            printf( "oeICalImpl::SetupAlarmManager() : WARNING Call to observer's onAlarm() unsuccessful: %x\n", rv );
+                        }
+                        #endif
+                    }
+                    NS_RELEASE( eventDisplay );
+                }
+                else {
+                    if( icaltime_is_null_time( nextalarm ) )
+                        nextalarm = alarmtime;
+                    else if( icaltime_compare( nextalarm, alarmtime ) > 0 )
+                        nextalarm = alarmtime;
+                    break;
+                }
+            } while ( 1 );
+        }
+        tmptodolistptr = tmptodolistptr->next;
+    }
+
+
+    lastcheck = now;
+
     if( m_alarmtimer  ) {
         PRUint32 delay = 0;
         #ifdef NS_INIT_REFCNT //A temporary way of keeping backward compatibility with Mozilla 1.0 source compile
@@ -1900,7 +2008,7 @@ NS_IMETHODIMP oeICalImpl::AddTodo(oeIICalTodo *icaltodo,char **retid)
         #endif
     }
 
-//    SetupAlarmManager();
+    SetupAlarmManager();
     return NS_OK;
 }
 
@@ -1915,14 +2023,14 @@ oeICalImpl::DeleteTodo( const char *id )
     stream = icalfileset_new(serveraddr);
     if ( !stream ) {
         #ifdef ICAL_DEBUG
-        printf( "oeICalImpl::DeleteEvent() failed: Cannot open stream: %s!\n", serveraddr );
+        printf( "oeICalImpl::DeleteTodo() failed: Cannot open stream: %s!\n", serveraddr );
         #endif
         return NS_OK;
     }
     
     if( id == nsnull ) {
         #ifdef ICAL_DEBUG
-        printf( "oeICalImpl::DeleteEvent() - Invalid Id.\n" );
+        printf( "oeICalImpl::DeleteTodo() - Invalid Id.\n" );
         #endif
         icalfileset_free(stream);
         return NS_OK;
@@ -1933,7 +2041,7 @@ oeICalImpl::DeleteTodo( const char *id )
     if( !fetchedvcal ) {
         icalfileset_free(stream);
         #ifdef ICAL_DEBUG
-        printf( "oeICalImpl::DeleteEvent() - WARNING Event not found.\n" );
+        printf( "oeICalImpl::DeleteTodo() - WARNING Event not found.\n" );
         #endif
         return NS_OK;
     }
@@ -1942,7 +2050,7 @@ oeICalImpl::DeleteTodo( const char *id )
     if( !fetchedvevent ) {
         icalfileset_free(stream);
         #ifdef ICAL_DEBUG
-        printf( "oeICalImpl::DeleteEvent() - WARNING Event not found.\n" );
+        printf( "oeICalImpl::DeleteTodo() - WARNING Event not found.\n" );
         #endif
         return NS_OK;
     }
@@ -1957,7 +2065,7 @@ oeICalImpl::DeleteTodo( const char *id )
     icalfileset_mark( stream ); //Make sure stream is marked as dirty
 	if( icalfileset_commit(stream) != ICAL_NO_ERROR ) {
         #ifdef ICAL_DEBUG
-	    printf( "oeICalImpl::DeleteEvent() : WARNING icalfileset_commit() unsuccessful\n" );
+	    printf( "oeICalImpl::DeleteTodo() : WARNING icalfileset_commit() unsuccessful\n" );
         #endif
     }
     icalfileset_free(stream);
@@ -1983,7 +2091,7 @@ oeICalImpl::DeleteTodo( const char *id )
 
     icalevent->Release();
 
-//    SetupAlarmManager();
+    SetupAlarmManager();
 	return NS_OK;
 }
 
@@ -1995,7 +2103,7 @@ NS_IMETHODIMP oeICalImpl::FetchTodo( const char *id, oeIICalTodo **ev)
 
     if( id == nsnull ) {
         #ifdef ICAL_DEBUG
-        printf( "oeICalImpl::FetchEvent() - Invalid Id.\n" );
+        printf( "oeICalImpl::FetchTodo() - Invalid Id.\n" );
         #endif
         *ev = nsnull;
         return NS_OK;
@@ -2012,7 +2120,7 @@ NS_IMETHODIMP oeICalImpl::FetchTodo( const char *id, oeIICalTodo **ev)
 NS_IMETHODIMP oeICalImpl::ModifyTodo(oeIICalTodo *icalevent, char **retid)
 {
 #ifdef ICAL_DEBUG
-    printf( "oeICalImpl::ModifyEvent()\n" );
+    printf( "oeICalImpl::ModifyTodo()\n" );
 #endif
     icalset *stream;
     icalcomponent *vcalendar;
@@ -2020,7 +2128,7 @@ NS_IMETHODIMP oeICalImpl::ModifyTodo(oeIICalTodo *icalevent, char **retid)
     stream = icalfileset_new(serveraddr);
     if ( !stream ) {
         #ifdef ICAL_DEBUG
-        printf( "oeICalImpl::ModifyEvent() failed: Cannot open stream: %s!\n", serveraddr );
+        printf( "oeICalImpl::ModifyTodo() failed: Cannot open stream: %s!\n", serveraddr );
         #endif
         return NS_OK;
     }
@@ -2028,7 +2136,7 @@ NS_IMETHODIMP oeICalImpl::ModifyTodo(oeIICalTodo *icalevent, char **retid)
     icalevent->GetId( retid );
     if( *retid == nsnull ) {
         #ifdef ICAL_DEBUG
-        printf( "oeICalImpl::ModifyEvent() - Invalid Id.\n" );
+        printf( "oeICalImpl::ModifyTodo() - Invalid Id.\n" );
         #endif
         icalfileset_free(stream);
         return NS_OK;
@@ -2056,7 +2164,7 @@ NS_IMETHODIMP oeICalImpl::ModifyTodo(oeIICalTodo *icalevent, char **retid)
             icalcomponent_free( fetchedvevent );
         } else {
             #ifdef ICAL_DEBUG
-            printf( "oeICalImpl::ModifyEvent() - WARNING Event not found.\n" );
+            printf( "oeICalImpl::ModifyTodo() - WARNING Event not found.\n" );
             #endif
             nsMemory::Free( *retid );
             *retid = nsnull;
@@ -2065,7 +2173,7 @@ NS_IMETHODIMP oeICalImpl::ModifyTodo(oeIICalTodo *icalevent, char **retid)
         }
     } else {
         #ifdef ICAL_DEBUG
-        printf( "oeICalImpl::ModifyEvent() - WARNING Event not found.\n" );
+        printf( "oeICalImpl::ModifyTodo() - WARNING Event not found.\n" );
         #endif
         nsMemory::Free( *retid );
         *retid = nsnull;
@@ -2078,7 +2186,7 @@ NS_IMETHODIMP oeICalImpl::ModifyTodo(oeIICalTodo *icalevent, char **retid)
     
 	if( icalfileset_commit(stream) != ICAL_NO_ERROR ) {
         #ifdef ICAL_DEBUG
-	    printf( "oeICalImpl::ModifyEvent() : WARNING icalfileset_commit() unsuccessful\n" );
+	    printf( "oeICalImpl::ModifyTodo() : WARNING icalfileset_commit() unsuccessful\n" );
         #endif
     }
     icalfileset_free(stream);
@@ -2099,7 +2207,7 @@ NS_IMETHODIMP oeICalImpl::ModifyTodo(oeIICalTodo *icalevent, char **retid)
 
     oldevent->Release();
 
-//    SetupAlarmManager();
+    SetupAlarmManager();
     return NS_OK;
 }
 
@@ -2236,6 +2344,11 @@ oeICalFilter::~oeICalFilter()
         m_completed->Release();
 }
 
+NS_IMETHODIMP oeICalFilter::GetType(Componenttype *aRetVal)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 NS_IMETHODIMP oeICalFilter::GetId(char **aRetVal)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
@@ -2635,3 +2748,11 @@ NS_IMETHODIMP oeICalFilter::ReportError( PRInt16 severity, PRUint32 errorid, con
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
+NS_IMETHODIMP oeICalFilter::SetParameter( const char *name, const char *value ) {
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP oeICalFilter::GetParameter( const char *name, char **value ) {
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
