@@ -35,11 +35,14 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+/* Derived from nsICmdLineHandler this file provides the "-calendar" option 
+for launching calendar */
+
 #include "plstr.h"
 #include "nsString.h"
 #include "oeICalStartupHandler.h"
 
-NS_IMPL_ISUPPORTS1(oeICalStartupHandler, nsICmdLineHandler);
+NS_IMPL_ISUPPORTS1(oeICalStartupHandler, nsICmdLineHandler)
 
 oeICalStartupHandler::oeICalStartupHandler()
 {
