@@ -72,9 +72,9 @@ public:
 
   /**
    * Get the document for this content.
-   * @param aResult the document [OUT]
+   * @return the document
    */
-  NS_IMETHOD GetDocument(nsIDocument** aResult) const = 0;
+  NS_IMETHOD_(nsIDocument*) GetDocument() const = 0;
 
   /**
    * Set the document for this content.
@@ -88,9 +88,9 @@ public:
 
   /**
    * Get the parent content for this content.
-   * @param aResult the parent, or null if no parent [OUT]
+   * @return the parent, or null if no parent
    */
-  NS_IMETHOD GetParent(nsIContent** aResult) const = 0;
+  NS_IMETHOD_(nsIContent*) GetParent() const = 0;
 
   /**
    * Set the parent content for this content.  (This does not add the child to
@@ -129,33 +129,33 @@ public:
    * Get the NodeInfo for this element
    * @param aResult the tag [OUT]
    */
-  NS_IMETHOD GetNodeInfo(nsINodeInfo** aResult) const = 0;
+  NS_IMETHOD_(nsINodeInfo *) GetNodeInfo() const = 0;
 
   /**
    * Tell whether this element can contain children
    * @param aResult whether this element can contain children [OUT]
    */
-  NS_IMETHOD CanContainChildren(PRBool& aResult) const = 0;
+  NS_IMETHOD_(PRBool) CanContainChildren() const = 0;
 
   /**
    * Get the number of children
    * @param aResult the number of children [OUT]
    */
-  NS_IMETHOD ChildCount(PRInt32& aResult) const = 0;
+  NS_IMETHOD_(PRUint32) GetChildCount() const = 0;
 
   /**
    * Get a child by index
    * @param aIndex the index of the child to get, or null if index out of bounds
    * @param aResult the child [OUT]
    */
-  NS_IMETHOD ChildAt(PRInt32 aIndex, nsIContent** aResult) const = 0;
+  NS_IMETHOD_(nsIContent *) GetChildAt(PRUint32 aIndex) const = 0;
 
   /**
    * Get the index of a child within this content
    * @param aPossibleChild the child to get the index
    * @param aIndex the index of the child, or -1 if not a child [OUT]
    */
-  NS_IMETHOD IndexOf(nsIContent* aPossibleChild, PRInt32& aIndex) const = 0;
+  NS_IMETHOD_(PRInt32) IndexOf(nsIContent* aPossibleChild) const = 0;
 
   /**
    * Insert a content node at a particular index.
@@ -167,7 +167,7 @@ public:
    *        occurred
    * @param aDeepSetDocument whether to set document on all children of aKid
    */
-  NS_IMETHOD InsertChildAt(nsIContent* aKid, PRInt32 aIndex,
+  NS_IMETHOD InsertChildAt(nsIContent* aKid, PRUint32 aIndex,
                            PRBool aNotify, PRBool aDeepSetDocument) = 0;
 
   /**
@@ -179,7 +179,7 @@ public:
    *        occurred
    * @param aDeepSetDocument whether to set document on all children of aKid
    */
-  NS_IMETHOD ReplaceChildAt(nsIContent* aKid, PRInt32 aIndex,
+  NS_IMETHOD ReplaceChildAt(nsIContent* aKid, PRUint32 aIndex,
                             PRBool aNotify, PRBool aDeepSetDocument) = 0;
 
   /**
@@ -200,7 +200,21 @@ public:
    * @param aNotify whether to notify the document that the replace has
    *        occurred
    */
-  NS_IMETHOD RemoveChildAt(PRInt32 aIndex, PRBool aNotify) = 0;
+  NS_IMETHOD RemoveChildAt(PRUint32 aIndex, PRBool aNotify) = 0;
+
+  /**
+   * Returns an atom holding the name of the attribute of type ID on
+   * this content node (if applicable).  Returns null for non-element
+   * content nodes.
+   */
+  NS_IMETHOD_(nsIAtom*) GetIDAttributeName() const = 0;
+
+  /**
+   * Returns an atom holding the name of the "class" attribute on this
+   * content node (if applicable).  Returns null for non-element
+   * content nodes.
+   */
+  NS_IMETHOD_(nsIAtom*) GetClassAttributeName() const = 0;
 
   /**
    * Normalizes an attribute string into an atom that represents the
@@ -229,8 +243,7 @@ public:
    *        notified of the attribute change.
    */
   NS_IMETHOD SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
-                     const nsAString& aValue,
-                     PRBool aNotify) = 0;
+                     const nsAString& aValue, PRBool aNotify) = 0;
 
   /**
    * Set attribute values. All attribute values are assumed to have a
@@ -245,8 +258,7 @@ public:
    * @param aNotify specifies whether or not the document should be
    *        notified of the attribute change.
    */
-  NS_IMETHOD SetAttr(nsINodeInfo* aNodeInfo,
-                     const nsAString& aValue,
+  NS_IMETHOD SetAttr(nsINodeInfo* aNodeInfo, const nsAString& aValue,
                      PRBool aNotify) = 0;
 
   /**
@@ -314,7 +326,7 @@ public:
    * @param aPrefix the attribute prefix [OUT]
    *
    */
-  NS_IMETHOD GetAttrNameAt(PRInt32 aIndex,
+  NS_IMETHOD GetAttrNameAt(PRUint32 aIndex,
                            PRInt32* aNameSpaceID,
                            nsIAtom** aName,
                            nsIAtom** aPrefix) const = 0;
@@ -324,7 +336,7 @@ public:
    *
    * @param aCountResult the number of attributes [OUT]
    */
-  NS_IMETHOD GetAttrCount(PRInt32& aCountResult) const = 0;
+  NS_IMETHOD_(PRUint32) GetAttrCount() const = 0;
 
   /**
    * Inform content of range ownership changes.  This allows content
@@ -402,7 +414,7 @@ public:
    * These methods are DEPRECATED, DON'T USE THEM!!!
    *
    */
-  NS_IMETHOD GetContentID(PRUint32* aID) = 0;
+  NS_IMETHOD GetContentID(PRUint32 *aId) = 0;
   /**
    * Set the unique content ID for this content.
    * @param aID the ID to set
@@ -447,9 +459,9 @@ public:
    * existence).  Used by anonymous content (XBL-generated). null for all
    * explicit content.
    *
-   * @param aContent the binding parent [OUT]
+   * @return the binding parent
    */
-  NS_IMETHOD GetBindingParent(nsIContent** aContent) = 0;
+  NS_IMETHOD_(nsIContent*) GetBindingParent() const = 0;
 
   /**
    * Bit-flags to pass (or'ed together) to IsContentOfType()
@@ -464,7 +476,9 @@ public:
     /** form controls */
     eHTML_FORM_CONTROL   = 0x00000008,
     /** XUL elements */
-    eXUL                 = 0x00000010
+    eXUL                 = 0x00000010,
+    /** xml processing instructions */
+    ePROCESSING_INSTRUCTION = 0x00000020
   };
 
   /**
