@@ -450,7 +450,7 @@ nsContentUtils::GetDocumentAndPrincipal(nsIDOMNode* aNode,
       // manager
       nsCOMPtr<nsINodeInfo> ni;
       if (content) {
-        content->GetNodeInfo(getter_AddRefs(ni));
+        ni = content->GetNodeInfo();
       }
       else {
         attr->GetNodeInfo(getter_AddRefs(ni));
@@ -481,7 +481,7 @@ nsContentUtils::GetDocumentAndPrincipal(nsIDOMNode* aNode,
   }
 
   if (!*aPrincipal) {
-    (*aDocument)->GetPrincipal(aPrincipal);
+    NS_IF_ADDREF(*aPrincipal = (*aDocument)->GetPrincipal());
   }
 
   return NS_OK;
@@ -539,9 +539,8 @@ nsContentUtils::CheckSameOrigin(nsIDOMNode *aTrustedNode,
 
       nsCOMPtr<nsIContent> cont = do_QueryInterface(aTrustedNode);
       NS_ENSURE_TRUE(cont, NS_ERROR_UNEXPECTED);
-      
-      nsCOMPtr<nsINodeInfo> ni;
-      cont->GetNodeInfo(getter_AddRefs(ni));
+
+      nsCOMPtr<nsINodeInfo> ni = cont->GetNodeInfo();
       NS_ENSURE_TRUE(ni, NS_ERROR_UNEXPECTED);
       
       ni->GetDocumentPrincipal(getter_AddRefs(trustedPrincipal));
@@ -586,7 +585,7 @@ nsContentUtils::CheckSameOrigin(nsIDOMNode *aTrustedNode,
   }
 
   if (!trustedPrincipal) {
-    trustedDoc->GetPrincipal(getter_AddRefs(trustedPrincipal));
+    trustedPrincipal = trustedDoc->GetPrincipal();
 
     if (!trustedPrincipal) {
       // If the trusted node doesn't have a principal we can't check
@@ -678,12 +677,11 @@ nsContentUtils::InProlog(nsIDOMNode *aNode)
 
   // Check that there are no elements before aNode to make sure we are not
   // in the epilog
-  PRInt32 pos;
-  doc->IndexOf(cont, pos);
+  PRInt32 pos = doc->IndexOf(cont);
+
   while (pos > 0) {
     --pos;
-    nsCOMPtr<nsIContent> sibl;
-    doc->ChildAt(pos, getter_AddRefs(sibl));
+    nsIContent *sibl = doc->GetChildAt(pos);
     if (sibl->IsContentOfType(nsIContent::eELEMENT)) {
       return PR_FALSE;
     }
@@ -717,9 +715,7 @@ nsContentUtils::doReparentContentWrapper(nsIContent *aChild,
   }
 
   if (aOldDocument) {
-    nsCOMPtr<nsISupports> old_ref;
-
-    aOldDocument->RemoveReference(aChild, getter_AddRefs(old_ref));
+    nsCOMPtr<nsISupports> old_ref = aOldDocument->RemoveReference(aChild);
 
     if (old_ref) {
       // Transfer the reference from aOldDocument to aNewDocument
@@ -732,13 +728,10 @@ nsContentUtils::doReparentContentWrapper(nsIContent *aChild,
   rv = old_wrapper->GetJSObject(&old);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsIContent> child;
-  PRInt32 count = 0, i;
-
-  aChild->ChildCount(count);
+  PRUint32 i, count = aChild->GetChildCount();
 
   for (i = 0; i < count; i++) {
-    aChild->ChildAt(i, getter_AddRefs(child));
+    nsIContent *child = aChild->GetChildAt(i);
     NS_ENSURE_TRUE(child, NS_ERROR_UNEXPECTED);
 
     rv = doReparentContentWrapper(child, aNewDocument, aOldDocument, cx, old);
@@ -753,8 +746,7 @@ nsresult GetContextFromDocument(nsIDocument *aDocument, JSContext **cx)
 {
   *cx = nsnull;
 
-  nsCOMPtr<nsIScriptGlobalObject> sgo;
-  aDocument->GetScriptGlobalObject(getter_AddRefs(sgo));
+  nsIScriptGlobalObject *sgo = aDocument->GetScriptGlobalObject();
 
   if (!sgo) {
     // No script global, no context.
@@ -787,15 +779,13 @@ nsContentUtils::ReparentContentWrapper(nsIContent *aContent,
     return NS_OK;
   }
 
-  nsCOMPtr<nsIDocument> old_doc(aOldDocument);
+  nsIDocument* old_doc = aOldDocument;
 
   if (!old_doc) {
-    nsCOMPtr<nsINodeInfo> ni;
-
-    aContent->GetNodeInfo(getter_AddRefs(ni));
+    nsCOMPtr<nsINodeInfo> ni = aContent->GetNodeInfo();
 
     if (ni) {
-      ni->GetDocument(getter_AddRefs(old_doc));
+      old_doc = ni->GetDocument();
     }
 
     if (!old_doc) {
@@ -811,10 +801,7 @@ nsContentUtils::ReparentContentWrapper(nsIContent *aContent,
   nsCOMPtr<nsISupports> new_parent;
 
   if (!aNewParent) {
-    nsCOMPtr<nsIContent> root;
-    old_doc->GetRootContent(getter_AddRefs(root));
-
-    if (root.get() == aContent) {
+    if (old_doc->GetRootContent() == aContent) {
       new_parent = old_doc;
     }
   } else {
@@ -938,16 +925,10 @@ nsContentUtils::InSameDoc(nsIDOMNode* aNode, nsIDOMNode* aOther)
   nsCOMPtr<nsIContent> other(do_QueryInterface(aOther));
 
   if (content && other) {
-    nsCOMPtr<nsIDocument> contentDoc;
-    nsCOMPtr<nsIDocument> otherDoc;
-    content->GetDocument(getter_AddRefs(contentDoc));
-    other->GetDocument(getter_AddRefs(otherDoc));
     // XXXcaa Don't bother to check that either node is in a
     // document.  Editor relies on us returning true if neither
     // node is in a document.  See bug 154401.
-    if (contentDoc == otherDoc) {
-      return PR_TRUE;
-    }
+    return content->GetDocument() == other->GetDocument();
   }
 
   return PR_FALSE;
@@ -961,12 +942,10 @@ nsContentUtils::ContentIsDescendantOf(nsIContent* aPossibleDescendant,
   NS_PRECONDITION(aPossibleDescendant, "The possible descendant is null!");
   NS_PRECONDITION(aPossibleAncestor, "The possible ancestor is null!");
 
-  nsCOMPtr<nsIContent> parent;
   do {
     if (aPossibleDescendant == aPossibleAncestor)
       return PR_TRUE;
-    aPossibleDescendant->GetParent(getter_AddRefs(parent));
-    aPossibleDescendant = parent;
+    aPossibleDescendant = aPossibleDescendant->GetParent();
   } while (aPossibleDescendant);
 
   return PR_FALSE;
@@ -1002,7 +981,6 @@ nsContentUtils::GetAncestorsAndOffsets(nsIDOMNode* aNode,
   NS_ENSURE_ARG_POINTER(aNode);
 
   PRInt32 offset = 0;
-  nsCOMPtr<nsIContent> ancestor;
   nsCOMPtr<nsIContent> content(do_QueryInterface(aNode));
 
   if (!content) {
@@ -1024,13 +1002,14 @@ nsContentUtils::GetAncestorsAndOffsets(nsIDOMNode* aNode,
   aAncestorOffsets->AppendElement(NS_INT32_TO_PTR(aOffset));
 
   // insert all the ancestors
-  content->GetParent(getter_AddRefs(ancestor));
-  while (ancestor) {
-    ancestor->IndexOf(content, offset);
-    aAncestorNodes->AppendElement(ancestor.get());
+  nsIContent* child = content;
+  nsIContent* parent = child->GetParent();
+  while (parent) {
+    offset = parent->IndexOf(child);
+    aAncestorNodes->AppendElement(parent);
     aAncestorOffsets->AppendElement(NS_INT32_TO_PTR(offset));
-    content.swap(ancestor);
-    content->GetParent(getter_AddRefs(ancestor));
+    child = parent;
+    parent = parent->GetParent();
   }
 
   return NS_OK;
@@ -1422,9 +1401,7 @@ nsContentUtils::GenerateStateKey(nsIContent* aContent,
     return NS_OK;
   }
 
-  nsCOMPtr<nsIDocument> doc;
-  aContent->GetDocument(getter_AddRefs(doc));
-  nsCOMPtr<nsIHTMLDocument> htmlDocument(do_QueryInterface(doc));
+  nsCOMPtr<nsIHTMLDocument> htmlDocument(do_QueryInterface(aContent->GetDocument()));
 
   PRBool generatedUniqueKey = PR_FALSE;
 
@@ -1468,7 +1445,7 @@ nsContentUtils::GenerateStateKey(nsIContent* aContent,
 
         // Append the index of the form in the document
         nsCOMPtr<nsIContent> formContent(do_QueryInterface(formElement));
-        htmlForms->IndexOf(formContent, index, PR_FALSE);
+        index = htmlForms->IndexOf(formContent, PR_FALSE);
         if (index <= -1) {
           //
           // XXX HACK this uses some state that was dumped into the document
@@ -1513,7 +1490,7 @@ nsContentUtils::GenerateStateKey(nsIContent* aContent,
         // causes a signficant pageload performance hit. See bug
         // 166636. Doing this wrong means you will see the assertion
         // below being hit.
-        htmlFormControls->IndexOf(aContent, index, PR_FALSE);
+        index = htmlFormControls->IndexOf(aContent, PR_FALSE);
         NS_ASSERTION(index > -1,
                      "nsFrameManager::GenerateStateKey didn't find content "
                      "by type! See bug 139568");
@@ -1541,20 +1518,95 @@ nsContentUtils::GenerateStateKey(nsIContent* aContent,
   return NS_OK;
 }
 
-/* static */ nsresult
+// static
+nsresult
 nsContentUtils::NewURIWithDocumentCharset(nsIURI** aResult,
                                           const nsAString& aSpec,
                                           nsIDocument* aDocument,
                                           nsIURI* aBaseURI)
 {
   nsCAutoString originCharset;
-  if (aDocument && NS_FAILED(aDocument->GetDocumentCharacterSet(originCharset)))
-    originCharset.Truncate();
+  if (aDocument)
+    originCharset = aDocument->GetDocumentCharacterSet();
 
   return NS_NewURI(aResult, NS_ConvertUCS2toUTF8(aSpec), originCharset.get(),
                    aBaseURI, sIOService);
 }
 
+// static
+PRBool
+nsContentUtils::BelongsInForm(nsIDOMHTMLFormElement *aForm,
+                              nsIContent *aContent)
+{
+  NS_PRECONDITION(aForm, "Must have a form");
+  NS_PRECONDITION(aContent, "Must have a content node");
+
+  nsCOMPtr<nsIContent> form(do_QueryInterface(aForm));
+
+  if (!form) {
+    NS_ERROR("This should not happen, form is not an nsIContent!");
+
+    return PR_TRUE;
+  }
+
+  if (form == aContent) {
+    // A form does not belong inside itself, so we return false here
+
+    return PR_FALSE;
+  }
+
+  nsIContent* content = aContent->GetParent();
+
+  while (content) {
+    if (content == form) {
+      // aContent is contained within the form so we return true.
+
+      return PR_TRUE;
+    }
+
+    nsCOMPtr<nsIAtom> tag;
+
+    content->GetTag(getter_AddRefs(tag));
+
+    if (tag == nsHTMLAtoms::form) {
+      // The child is contained within a form, but not the right form
+      // so we ignore it.
+
+      return PR_FALSE;
+    }
+
+    content = content->GetParent();
+  }
+
+  if (form->GetChildCount() > 0) {
+    // The form is a container but aContent wasn't inside the form,
+    // return false
+
+    return PR_FALSE;
+  }
+
+  // The form is a leaf and aContent wasn't inside any other form so
+  // we check whether the content comes after the form.  If it does,
+  // return true.  If it does not, then it couldn't have been inside
+  // the form in the HTML.
+  nsCOMPtr<nsIDOM3Node> contentAsDOM3(do_QueryInterface(aContent));
+  PRUint16 comparisonFlags = 0;
+  nsresult rv = NS_OK;
+  if (contentAsDOM3) {
+    rv = contentAsDOM3->CompareDocumentPosition(aForm, &comparisonFlags);
+  }
+  if (NS_FAILED(rv) ||
+      comparisonFlags & nsIDOM3Node::DOCUMENT_POSITION_PRECEDING) {
+    // We could be in this form!
+    // In the future, we may want to get document.forms, look at the
+    // form after aForm, and if aContent is after that form after
+    // aForm return false here....
+    return PR_TRUE;
+  }
+  
+  return PR_FALSE;
+}
+
 void
 nsCxPusher::Push(nsISupports *aCurrentTarget)
 {
@@ -1569,7 +1621,7 @@ nsCxPusher::Push(nsISupports *aCurrentTarget)
   nsCOMPtr<nsIDocument> document;
 
   if (content) {
-    content->GetDocument(getter_AddRefs(document));
+    document = content->GetDocument();
   }
 
   if (!document) {
@@ -1577,7 +1629,7 @@ nsCxPusher::Push(nsISupports *aCurrentTarget)
   }
 
   if (document) {
-    document->GetScriptGlobalObject(getter_AddRefs(sgo));
+    sgo = document->GetScriptGlobalObject();
   }
 
   if (!document && !sgo) {
