@@ -44,9 +44,9 @@
 #include "nsIDocument.h"
 #include "nsWeakReference.h"
 #include "nsWeakPtr.h"
-#include "nsIArena.h"
 #include "nsVoidArray.h"
 #include "nsIDOMXMLDocument.h"
+#include "nsIDOM3Document.h"
 #include "nsIDOMDocumentView.h"
 #include "nsIDOMDocumentXBL.h"
 #include "nsIDOMNSDocument.h"
@@ -62,6 +62,7 @@
 #include "nsIContent.h"
 #include "nsIEventListenerManager.h"
 #include "nsGenericDOMNodeList.h"
+#include "nsIDOM3Node.h"
 #include "nsIPrincipal.h"
 #include "nsIBindingManager.h"
 #include "nsINodeInfo.h"
@@ -148,8 +149,12 @@ public:
 
   NS_DECL_NSIDOMSTYLESHEETLIST
 
-  NS_IMETHOD BeginUpdate(nsIDocument *aDocument) { return NS_OK; }
-  NS_IMETHOD EndUpdate(nsIDocument *aDocument) { return NS_OK; }
+  NS_IMETHOD BeginUpdate(nsIDocument *aDocument, nsUpdateType aUpdateType) {
+    return NS_OK;
+  }
+  NS_IMETHOD EndUpdate(nsIDocument *aDocument, nsUpdateType aUpdateType) {
+    return NS_OK;
+  }
   NS_IMETHOD BeginLoad(nsIDocument *aDocument) { return NS_OK; }
   NS_IMETHOD EndLoad(nsIDocument *aDocument) { return NS_OK; }
   NS_IMETHOD BeginReflow(nsIDocument *aDocument,
@@ -167,8 +172,7 @@ public:
                               nsIContent*  aContent,
                               PRInt32      aNameSpaceID,
                               nsIAtom*     aAttribute,
-                              PRInt32      aModType,
-                              nsChangeHint aHint) { return NS_OK; }
+                              PRInt32      aModType) { return NS_OK; }
   NS_IMETHOD ContentAppended(nsIDocument *aDocument,
                              nsIContent* aContainer,
                              PRInt32     aNewIndexInContainer)
@@ -195,8 +199,8 @@ public:
                                               PRBool aApplicable) { return NS_OK; }
   NS_IMETHOD StyleRuleChanged(nsIDocument *aDocument,
                               nsIStyleSheet* aStyleSheet,
-                              nsIStyleRule* aStyleRule,
-                              nsChangeHint aHint) { return NS_OK; }
+                              nsIStyleRule* aOldStyleRule,
+                              nsIStyleRule* aNewStyleRule) { return NS_OK; }
   NS_IMETHOD StyleRuleAdded(nsIDocument *aDocument,
                             nsIStyleSheet* aStyleSheet,
                             nsIStyleRule* aStyleRule) { return NS_OK; }
@@ -231,7 +235,7 @@ class nsDocument : public nsIDocument,
                    public nsIDOMDocumentRange,
                    public nsIDOMDocumentTraversal,
                    public nsIDOMDocumentXBL,
-                   public nsIDOM3Node,
+                   public nsIDOM3Document,
                    public nsSupportsWeakReference,
                    public nsIDOMEventReceiver,
                    public nsIDOM3EventTarget,
@@ -241,12 +245,8 @@ class nsDocument : public nsIDocument,
 public:
   NS_DECL_ISUPPORTS
 
-  NS_DECL_AND_IMPL_ZEROING_OPERATOR_NEW
-
-  NS_IMETHOD GetArena(nsIArena** aArena);
-
-  NS_IMETHOD Reset(nsIChannel* aChannel, nsILoadGroup* aLoadGroup);
-  NS_IMETHOD ResetToURI(nsIURI *aURI, nsILoadGroup *aLoadGroup);
+  virtual void Reset(nsIChannel *aChannel, nsILoadGroup *aLoadGroup);
+  virtual void ResetToURI(nsIURI *aURI, nsILoadGroup *aLoadGroup);
 
   NS_IMETHOD StartDocumentLoad(const char* aCommand,
                                nsIChannel* aChannel,
@@ -258,26 +258,15 @@ public:
 
   NS_IMETHOD StopDocumentLoad();
 
-  /**
-   * Return the title of the document. May return null.
-   */
-  NS_IMETHOD GetDocumentTitle(nsAString& aTitle) const;
-
-  /**
-   * Return the URL for the document. May return null.
-   */
-  NS_IMETHOD GetDocumentURL(nsIURI** aURI) const;
-
   /**
    * Return the principal responsible for this document.
    */
-  NS_IMETHOD GetPrincipal(nsIPrincipal **aPrincipal);
+  virtual nsIPrincipal* GetPrincipal();
 
   /**
-   * Update principal responsible for this document to the intersection
-   * of its previous value and aPrincipal, and return its new value.
+   * Set the principal responsible for this document.
    */
-  NS_IMETHOD AddPrincipal(nsIPrincipal *aPrincipal);
+  NS_IMETHOD SetPrincipal(nsIPrincipal *aPrincipal);
 
   /**
    * Get the Content-Type of this document.
@@ -288,39 +277,24 @@ public:
   /**
    * Set the Content-Type of this document.
    */
-  NS_IMETHOD SetContentType(const nsAString& aContentType);
-
-  /**
-   * Return the content language of this document.
-   */
-  NS_IMETHOD GetContentLanguage(nsAString& aContentLanguage) const;
-
-  /**
-   * Return the LoadGroup for the document. May return null.
-   */
-  NS_IMETHOD GetDocumentLoadGroup(nsILoadGroup **aGroup) const;
+  virtual void SetContentType(const nsAString& aContentType);
 
   /**
    * Return the base URL for relative URLs in the document. May return null (or the document URL).
    */
-  NS_IMETHOD GetBaseURL(nsIURI*& aURL) const;
   NS_IMETHOD SetBaseURL(nsIURI* aURL);
 
   /**
    * Get/Set the base target of a link in a document.
    */
-  NS_IMETHOD GetBaseTarget(nsAString &aBaseTarget);
-  NS_IMETHOD SetBaseTarget(const nsAString &aBaseTarget);
+  virtual void GetBaseTarget(nsAString &aBaseTarget) const;
+  virtual void SetBaseTarget(const nsAString &aBaseTarget);
 
   /**
    * Return a standard name for the document's character set. This will
    * trigger a startDocumentLoad if necessary to answer the question.
    */
-  NS_IMETHOD GetDocumentCharacterSet(nsAString& oCharsetID);
-  NS_IMETHOD SetDocumentCharacterSet(const nsAString& aCharSetID);
-
-  NS_IMETHOD GetDocumentCharacterSetSource(PRInt32* aCharsetSource);
-  NS_IMETHOD SetDocumentCharacterSetSource(PRInt32 aCharsetSource);
+  virtual void SetDocumentCharacterSet(const nsACString& aCharSetID);
 
   /**
    * Add an observer that gets notified whenever the charset changes.
@@ -332,32 +306,21 @@ public:
    */
   NS_IMETHOD RemoveCharSetObserver(nsIObserver* aObserver);
 
-  /**
-   *  Check if the document contains bidi data.
-   *  If so, we have to apply the Unicode Bidi Algorithm.
-   */
-  NS_IMETHOD GetBidiEnabled(PRBool* aBidiEnabled) const;
-  /**
-   *  Indicate the document contains RTL characters.
-   */
-  NS_IMETHOD SetBidiEnabled(PRBool aBidiEnabled);
-
   /**
    * Return the Line Breaker for the document
    */
-  NS_IMETHOD GetLineBreaker(nsILineBreaker** aResult)  ;
-  NS_IMETHOD SetLineBreaker(nsILineBreaker* aLineBreaker) ;
-  NS_IMETHOD GetWordBreaker(nsIWordBreaker** aResult)  ;
-  NS_IMETHOD SetWordBreaker(nsIWordBreaker* aWordBreaker) ;
+  virtual nsILineBreaker* GetLineBreaker();
+  virtual void SetLineBreaker(nsILineBreaker* aLineBreaker);
+  virtual nsIWordBreaker* GetWordBreaker();
+  virtual void SetWordBreaker(nsIWordBreaker* aWordBreaker);
 
   /**
    * Access HTTP header data (this may also get set from other sources, like
    * HTML META tags).
    */
-  NS_IMETHOD GetHeaderData(nsIAtom* aHeaderField,
-                           nsAString& aData) const;
-  NS_IMETHOD SetHeaderData(nsIAtom* aheaderField,
-                           const nsAString& aData);
+  virtual void GetHeaderData(nsIAtom* aHeaderField, nsAString& aData) const;
+  virtual void SetHeaderData(nsIAtom* aheaderField,
+                             const nsAString& aData);
 
   /**
    * Create a new presentation shell that will use aContext for
@@ -368,50 +331,37 @@ public:
                          nsIViewManager* aViewManager,
                          nsIStyleSet* aStyleSet,
                          nsIPresShell** aInstancePtrResult);
-  virtual PRBool DeleteShell(nsIPresShell* aShell);
-  virtual PRInt32 GetNumberOfShells();
-  NS_IMETHOD GetShellAt(PRInt32 aIndex, nsIPresShell** aShell);
-
-  /**
-   * Return the parent document of this document. Will return null
-   * unless this document is within a compound document and has a parent.
-   */
-  NS_IMETHOD GetParentDocument(nsIDocument** aParent);
-  NS_IMETHOD SetParentDocument(nsIDocument* aParent);
+  NS_IMETHOD_(PRBool) DeleteShell(nsIPresShell* aShell);
+  NS_IMETHOD_(PRUint32) GetNumberOfShells() const;
+  NS_IMETHOD_(nsIPresShell *) GetShellAt(PRUint32 aIndex) const;
 
   NS_IMETHOD SetSubDocumentFor(nsIContent *aContent, nsIDocument* aSubDoc);
-  NS_IMETHOD GetSubDocumentFor(nsIContent *aContent, nsIDocument** aSubDoc);
-  NS_IMETHOD FindContentForSubDocument(nsIDocument *aDocument,
-                                       nsIContent **aContent);
+  virtual nsIDocument* GetSubDocumentFor(nsIContent *aContent) const;
+  virtual nsIContent* FindContentForSubDocument(nsIDocument *aDocument) const;
 
-  /**
-   * Return the root content object for this document.
-   */
-  NS_IMETHOD GetRootContent(nsIContent** aRoot);
-  NS_IMETHOD SetRootContent(nsIContent* aRoot);
+  virtual void SetRootContent(nsIContent* aRoot);
 
   /**
    * Get the direct children of the document - content in
    * the prolog, the root content and content in the epilog.
    */
-  NS_IMETHOD ChildAt(PRInt32 aIndex, nsIContent*& aResult) const;
-  NS_IMETHOD IndexOf(nsIContent* aPossibleChild, PRInt32& aIndex) const;
-  NS_IMETHOD GetChildCount(PRInt32& aCount);
+  NS_IMETHOD_(nsIContent *) GetChildAt(PRUint32 aIndex) const;
+  NS_IMETHOD_(PRInt32) IndexOf(nsIContent* aPossibleChild) const;
+  NS_IMETHOD_(PRUint32) GetChildCount() const;
 
   /**
    * Get the style sheets owned by this document.
    * These are ordered, highest priority last
    */
-  NS_IMETHOD GetNumberOfStyleSheets(PRBool aIncludeSpecialSheets,
-                                    PRInt32* aCount);
-  NS_IMETHOD GetStyleSheetAt(PRInt32 aIndex, PRBool aIncludeSpecialSheets,
-                             nsIStyleSheet** aSheet);
-  NS_IMETHOD GetIndexOfStyleSheet(nsIStyleSheet* aSheet, PRInt32* aIndex);
+  virtual PRInt32 GetNumberOfStyleSheets(PRBool aIncludeSpecialSheets) const;
+  virtual nsIStyleSheet* GetStyleSheetAt(PRInt32 aIndex,
+                                         PRBool aIncludeSpecialSheets) const;
+  virtual PRInt32 GetIndexOfStyleSheet(nsIStyleSheet* aSheet) const;
   virtual void AddStyleSheet(nsIStyleSheet* aSheet, PRUint32 aFlags);
   virtual void RemoveStyleSheet(nsIStyleSheet* aSheet);
 
-  NS_IMETHOD UpdateStyleSheets(nsCOMArray<nsIStyleSheet>& aOldSheets,
-                               nsCOMArray<nsIStyleSheet>& aNewSheets);
+  virtual void UpdateStyleSheets(nsCOMArray<nsIStyleSheet>& aOldSheets,
+                                 nsCOMArray<nsIStyleSheet>& aNewSheets);
   virtual void AddStyleSheetToStyleSets(nsIStyleSheet* aSheet);
   virtual void RemoveStyleSheetFromStyleSets(nsIStyleSheet* aSheet);
 
@@ -424,13 +374,13 @@ public:
    * This is the context within which all scripts (during document
    * creation and during event handling) will run.
    */
-  NS_IMETHOD GetScriptGlobalObject(nsIScriptGlobalObject** aGlobalObject);
-  NS_IMETHOD SetScriptGlobalObject(nsIScriptGlobalObject* aGlobalObject);
+  virtual nsIScriptGlobalObject* GetScriptGlobalObject() const;
+  virtual void SetScriptGlobalObject(nsIScriptGlobalObject* aGlobalObject);
 
   /**
    * Get the script loader for this document
    */
-  NS_IMETHOD GetScriptLoader(nsIScriptLoader** aScriptLoader);
+  virtual nsIScriptLoader* GetScriptLoader();
 
   /**
    * Add a new observer of document change notifications. Whenever
@@ -445,64 +395,59 @@ public:
    */
   virtual PRBool RemoveObserver(nsIDocumentObserver* aObserver);
 
-  // Observation hooks used by content nodes to propagate
-  // notifications to document observers.
-  NS_IMETHOD BeginUpdate();
-  NS_IMETHOD EndUpdate();
-  NS_IMETHOD BeginLoad();
-  NS_IMETHOD EndLoad();
-  NS_IMETHOD ContentChanged(nsIContent* aContent,
-                            nsISupports* aSubContent);
-  NS_IMETHOD ContentStatesChanged(nsIContent* aContent1,
-                                  nsIContent* aContent2,
-                                  PRInt32 aStateMask);
+  // Observation hooks used to propagate notifications to document
+  // observers.
+  virtual void BeginUpdate(nsUpdateType aUpdateType);
+  virtual void EndUpdate(nsUpdateType aUpdateType);
+  virtual void BeginLoad();
+  virtual void EndLoad();
+  virtual void ContentChanged(nsIContent* aContent,
+                              nsISupports* aSubContent);
+  virtual void ContentStatesChanged(nsIContent* aContent1,
+                                    nsIContent* aContent2,
+                                    PRInt32 aStateMask);
 
-  NS_IMETHOD AttributeWillChange(nsIContent* aChild,
-                                 PRInt32 aNameSpaceID,
-                                 nsIAtom* aAttribute);
-  NS_IMETHOD AttributeChanged(nsIContent* aChild,
-                              PRInt32 aNameSpaceID,
-                              nsIAtom* aAttribute,
-                              PRInt32 aModType,
-                              nsChangeHint aHint);
-  NS_IMETHOD ContentAppended(nsIContent* aContainer,
-                             PRInt32 aNewIndexInContainer);
-  NS_IMETHOD ContentInserted(nsIContent* aContainer,
-                             nsIContent* aChild,
-                             PRInt32 aIndexInContainer);
-  NS_IMETHOD ContentReplaced(nsIContent* aContainer,
-                             nsIContent* aOldChild,
-                             nsIContent* aNewChild,
-                             PRInt32 aIndexInContainer);
-  NS_IMETHOD ContentRemoved(nsIContent* aContainer,
-                            nsIContent* aChild,
-                            PRInt32 aIndexInContainer);
+  virtual void AttributeWillChange(nsIContent* aChild,
+                                   PRInt32 aNameSpaceID,
+                                   nsIAtom* aAttribute);
+  virtual void AttributeChanged(nsIContent* aChild,
+                                PRInt32 aNameSpaceID,
+                                nsIAtom* aAttribute,
+                                PRInt32 aModType);
+  virtual void ContentAppended(nsIContent* aContainer,
+                               PRInt32 aNewIndexInContainer);
+  virtual void ContentInserted(nsIContent* aContainer,
+                               nsIContent* aChild,
+                               PRInt32 aIndexInContainer);
+  virtual void ContentReplaced(nsIContent* aContainer,
+                               nsIContent* aOldChild,
+                               nsIContent* aNewChild,
+                               PRInt32 aIndexInContainer);
+  virtual void ContentRemoved(nsIContent* aContainer,
+                              nsIContent* aChild,
+                              PRInt32 aIndexInContainer);
 
-  NS_IMETHOD StyleRuleChanged(nsIStyleSheet* aStyleSheet,
-                              nsIStyleRule* aStyleRule,
-                              nsChangeHint aHint); // See nsStyleConsts fot hint values
-  NS_IMETHOD StyleRuleAdded(nsIStyleSheet* aStyleSheet,
-                            nsIStyleRule* aStyleRule);
-  NS_IMETHOD StyleRuleRemoved(nsIStyleSheet* aStyleSheet,
+  virtual void StyleRuleChanged(nsIStyleSheet* aStyleSheet,
+                                nsIStyleRule* aOldStyleRule,
+                                nsIStyleRule* aNewStyleRule);
+  virtual void StyleRuleAdded(nsIStyleSheet* aStyleSheet,
                               nsIStyleRule* aStyleRule);
+  virtual void StyleRuleRemoved(nsIStyleSheet* aStyleSheet,
+                                nsIStyleRule* aStyleRule);
 
-  NS_IMETHOD FlushPendingNotifications(PRBool aFlushReflows = PR_TRUE,
-                                       PRBool aUpdateViews = PR_FALSE);
-  NS_IMETHOD GetAndIncrementContentID(PRInt32* aID);
-  NS_IMETHOD GetBindingManager(nsIBindingManager** aResult);
-  NS_IMETHOD GetNodeInfoManager(nsINodeInfoManager*& aNodeInfoManager);
-  NS_IMETHOD AddReference(void *aKey, nsISupports *aReference);
-  NS_IMETHOD RemoveReference(void *aKey, nsISupports **aOldReference);
-  NS_IMETHOD SetContainer(nsISupports *aContainer);
-  NS_IMETHOD GetContainer(nsISupports **aContainer);
-  NS_IMETHOD GetScriptEventManager(nsIScriptEventManager **aResult);
-  NS_IMETHOD SetXMLDeclaration(const nsAString& aVersion,
+  virtual void FlushPendingNotifications(PRBool aFlushReflows = PR_TRUE,
+                                         PRBool aUpdateViews = PR_FALSE);
+  virtual void AddReference(void *aKey, nsISupports *aReference);
+  virtual already_AddRefed<nsISupports> RemoveReference(void *aKey);
+  virtual nsIScriptEventManager* GetScriptEventManager();
+  virtual void SetXMLDeclaration(const nsAString& aVersion,
                                const nsAString& aEncoding,
                                const nsAString& Standalone);
-  NS_IMETHOD GetXMLDeclaration(nsAString& aVersion,
-                               nsAString& aEncoding,
-                               nsAString& Standalone);
+  virtual void GetXMLDeclaration(nsAString& aVersion,
+                                 nsAString& aEncoding,
+                                 nsAString& Standalone);
   NS_IMETHOD_(PRBool) IsCaseSensitive();
+  NS_IMETHOD_(PRBool) IsScriptEnabled();
 
   // nsIRadioGroupContainer
   NS_IMETHOD WalkRadioGroup(const nsAString& aName,
@@ -529,6 +474,9 @@ public:
   // nsIDOMDocument
   NS_DECL_NSIDOMDOCUMENT
 
+  // nsIDOM3Document
+  NS_DECL_NSIDOM3DOCUMENT
+
   // nsIDOMXMLDocument
   NS_DECL_NSIDOMXMLDOCUMENT
 
@@ -571,12 +519,13 @@ public:
   // nsIDOM3EventTarget
   NS_DECL_NSIDOM3EVENTTARGET
 
+  // nsIScriptObjectPrincipal
+  NS_IMETHOD GetPrincipal(nsIPrincipal **aPrincipal);
+
   NS_IMETHOD HandleDOMEvent(nsIPresContext* aPresContext, nsEvent* aEvent,
                             nsIDOMEvent** aDOMEvent, PRUint32 aFlags,
                             nsEventStatus* aEventStatus);
 
-  NS_IMETHOD SetDocumentURL(nsIURI* aURI);
-
   virtual nsresult Init();
 
 protected:
@@ -585,8 +534,8 @@ protected:
                                      PRUint32 aFlags);
   virtual void InternalInsertStyleSheetAt(nsIStyleSheet* aSheet,
                                           PRInt32 aIndex);
-  virtual already_AddRefed<nsIStyleSheet> InternalGetStyleSheetAt(PRInt32 aIndex);
-  virtual PRInt32 InternalGetNumberOfStyleSheets();
+  virtual nsIStyleSheet* InternalGetStyleSheetAt(PRInt32 aIndex) const;
+  virtual PRInt32 InternalGetNumberOfStyleSheets() const;
 
   virtual void RetrieveRelevantHeaders(nsIChannel *aChannel);
 
@@ -598,22 +547,10 @@ protected:
   nsDocument();
   virtual ~nsDocument();
 
-  nsCOMPtr<nsIArena> mArena;
-  nsString mDocumentTitle;
   nsCString mLastModified;
-  nsCOMPtr<nsIURI> mDocumentURL;
-  nsCOMPtr<nsIURI> mDocumentBaseURL;
   nsCOMPtr<nsIPrincipal> mPrincipal;
 
-  nsWeakPtr mDocumentLoadGroup;
-
-  nsWeakPtr mDocumentContainer;
-
-  nsString mCharacterSet;
-  PRInt32 mCharacterSetSource;
-
   nsVoidArray mCharSetObservers;
-  nsIDocument* mParentDocument;
 
   PLDHashTable *mSubDocuments;
 
@@ -622,10 +559,6 @@ protected:
   // Array of owning references to all children
   nsCOMArray<nsIContent> mChildren;
 
-  // A weak reference to the only element in mChildren, or null if no
-  // such element exists.
-  nsIContent* mRootContent;
-
   nsCOMArray<nsIStyleSheet> mStyleSheets;
 
   // Basically always has at least 1 entry
@@ -640,21 +573,11 @@ protected:
 
   nsRefPtr<nsDocumentChildNodes> mChildNodes;
 
-  // A content ID counter used to give a monotonically increasing ID
-  // to the content objects in the document's content model
-  PRInt32 mNextContentID;
-
   nsHashtable mRadioGroups;
 
-  nsCOMPtr<nsIBindingManager> mBindingManager;
-  nsCOMPtr<nsINodeInfoManager> mNodeInfoManager;
-
   // True if the document is being destroyed.
   PRPackedBool mIsGoingAway;
 
-  // True if BIDI is enabled.
-  PRPackedBool mBidiEnabled;
-
   // True if the document is being destroyed.
   PRPackedBool mInDestructor;
 
@@ -666,12 +589,11 @@ protected:
 
   nsCOMPtr<nsICSSLoader> mCSSLoader;
 
-  nsXPIDLCString mContentLanguage;
-  nsCString mContentType;
-
   nsCOMPtr<nsIScriptEventManager> mScriptEventManager;
 
 private:
+  nsresult IsAllowedAsChild(PRUint16 aNodeType, nsIContent* aRefContent);
+
   // These are not implemented and not supported.
   nsDocument(const nsDocument& aOther);
   nsDocument& operator=(const nsDocument& aOther);
