@@ -61,7 +61,6 @@
 #include "nsIDOMRange.h"
 #include "nsIDOMDocument.h"
 #include "nsICharsetConverterManager.h"
-#include "nsICharsetConverterManager2.h"
 #include "nsHTMLAtoms.h"
 #include "nsITextContent.h"
 #include "nsIEnumerator.h"
@@ -74,9 +73,7 @@
 #include "nsIScriptSecurityManager.h"
 #include "nsContentUtils.h"
 #include "nsUnicharUtils.h"
-
-static NS_DEFINE_CID(kCharsetConverterManagerCID,
-                     NS_ICHARSETCONVERTERMANAGER_CID);
+#include "nsReadableUtils.h"
 
 nsresult NS_NewDomSelection(nsISelection **aDomSelection);
 
@@ -109,7 +106,7 @@ public:
   NS_IMETHOD SetRange(nsIDOMRange* aRange);
   NS_IMETHOD SetNode(nsIDOMNode* aNode);
   NS_IMETHOD SetWrapColumn(PRUint32 aWC);
-  NS_IMETHOD SetCharset(const nsAString& aCharset);
+  NS_IMETHOD SetCharset(const nsACString& aCharset);
   NS_IMETHOD GetMimeType(nsAString& aMimeType);
   NS_IMETHOD EncodeToStream(nsIOutputStream* aStream);
   NS_IMETHOD EncodeToString(nsAString& aOutputString);
@@ -150,10 +147,10 @@ protected:
   nsCOMPtr<nsIUnicodeEncoder>    mUnicodeEncoder;
   nsCOMPtr<nsIDOMNode>           mCommonParent;
   nsCOMPtr<nsIDocumentEncoderNodeFixup> mNodeFixup;
-  nsCOMPtr<nsICharsetConverterManager2> mCharsetConverterManager;
+  nsCOMPtr<nsICharsetConverterManager> mCharsetConverterManager;
 
   nsString          mMimeType;
-  nsString          mCharset;
+  nsCString         mCharset;
   PRUint32          mFlags;
   PRUint32          mWrapColumn;
   PRUint32          mStartDepth;
@@ -198,34 +195,6 @@ nsDocumentEncoder::~nsDocumentEncoder()
 {
 }
 
-static PRBool
-IsScriptEnabled(nsIDocument *aDoc)
-{
-  NS_ENSURE_TRUE(aDoc, PR_TRUE);
-
-  nsCOMPtr<nsIScriptSecurityManager> sm(do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID));
-  NS_ENSURE_TRUE(sm, PR_TRUE);
-
-  nsCOMPtr<nsIPrincipal> principal;
-  aDoc->GetPrincipal(getter_AddRefs(principal));
-  NS_ENSURE_TRUE(principal, PR_TRUE);
-
-  nsCOMPtr<nsIScriptGlobalObject> globalObject;
-  aDoc->GetScriptGlobalObject(getter_AddRefs(globalObject));
-  NS_ENSURE_TRUE(globalObject, PR_TRUE);
-
-  nsCOMPtr<nsIScriptContext> scriptContext;
-  globalObject->GetContext(getter_AddRefs(scriptContext));
-  NS_ENSURE_TRUE(scriptContext, PR_TRUE);
-
-  JSContext* cx = (JSContext *) scriptContext->GetNativeContext();
-  NS_ENSURE_TRUE(cx, PR_TRUE);
-
-  PRBool enabled = PR_TRUE;
-  sm->CanExecuteScripts(cx, principal, &enabled);
-  return enabled;
-}
-
 NS_IMETHODIMP
 nsDocumentEncoder::Init(nsIDocument* aDocument,
                         const nsAString& aMimeType,
@@ -273,7 +242,7 @@ nsDocumentEncoder::SetNode(nsIDOMNode* aNode)
 }
 
 NS_IMETHODIMP
-nsDocumentEncoder::SetCharset(const nsAString& aCharset)
+nsDocumentEncoder::SetCharset(const nsACString& aCharset)
 {
   mCharset = aCharset;
   return NS_OK;
@@ -424,7 +393,7 @@ nsDocumentEncoder::IsTag(nsIDOMNode* aNode, nsIAtom* aAtom)
     nsCOMPtr<nsIAtom> atom;
     nsCOMPtr<nsIContent> content = do_QueryInterface(aNode);
     if (content)
-      content->GetTag(*getter_AddRefs(atom));
+      content->GetTag(getter_AddRefs(atom));
     if (atom)
     {
       if (atom.get() == aAtom)
@@ -525,14 +494,13 @@ nsDocumentEncoder::FlushText(nsAString& aString, PRBool aForce)
       // there are problems with it so we don't use it now, maybe later...
 static nsresult ChildAt(nsIDOMNode* aNode, PRInt32 aIndex, nsIDOMNode*& aChild)
 {
-  nsCOMPtr<nsIContent> node(do_QueryInterface(aNode));
-  nsCOMPtr<nsIContent> child;
+  nsCOMPtr<nsIContent> content(do_QueryInterface(aNode));
 
   aChild = nsnull;
 
-  NS_ENSURE_TRUE(node, NS_ERROR_FAILURE);
+  NS_ENSURE_TRUE(content, NS_ERROR_FAILURE);
 
-  node->ChildAt(aIndex, *getter_AddRefs(child));
+  nsIContent *child = content->GetChildAt(aIndex);
 
   if (child)
     return CallQueryInterface(child, &aChild);
@@ -545,14 +513,10 @@ static PRInt32 IndexOf(nsIDOMNode* aParent, nsIDOMNode* aChild)
   nsCOMPtr<nsIContent> parent(do_QueryInterface(aParent));
   nsCOMPtr<nsIContent> child(do_QueryInterface(aChild));
 
-  if (!parent || !child)
+  if (!parent)
     return -1;
 
-  PRInt32 indx = 0;
-
-  parent->IndexOf(child, indx);
-
-  return indx;
+  return parent->IndexOf(child);
 }
 
 static inline PRInt32 GetIndex(nsVoidArray& aIndexArray)
@@ -735,8 +699,8 @@ nsDocumentEncoder::SerializeRangeNodes(nsIDOMRange* aRange,
         NS_ENSURE_SUCCESS(rv, rv);
       }
       
-      // do some calculations that will tell us which children of this node are in the range.
-      nsCOMPtr<nsIContent> child;
+      // do some calculations that will tell us which children of this
+      // node are in the range.
       nsCOMPtr<nsIDOMNode> childAsNode;
       PRInt32 startOffset = 0, endOffset = -1;
       if (startNode == content && mStartRootIndex >= aDepth)
@@ -744,9 +708,9 @@ nsDocumentEncoder::SerializeRangeNodes(nsIDOMRange* aRange,
       if (endNode == content && mEndRootIndex >= aDepth)
         endOffset = NS_PTR_TO_INT32(mEndOffsets[mEndRootIndex - aDepth]) ;
       // generated content will cause offset values of -1 to be returned.  
-      PRInt32 j, childCount=0;
-      rv = content->ChildCount(childCount);
-      NS_ENSURE_SUCCESS(rv, rv);
+      PRInt32 j;
+      PRUint32 childCount = content->GetChildCount();
+
       if (startOffset == -1) startOffset = 0;
       if (endOffset == -1) endOffset = childCount;
       else
@@ -767,13 +731,13 @@ nsDocumentEncoder::SerializeRangeNodes(nsIDOMRange* aRange,
       // serialize the children of this node that are in the range
       for (j=startOffset; j<endOffset; j++)
       {
-        rv = content->ChildAt(j, *getter_AddRefs(child));
-        childAsNode = do_QueryInterface(child);
-        NS_ENSURE_SUCCESS(rv, rv);
+        childAsNode = do_QueryInterface(content->GetChildAt(j));
+
         if ((j==startOffset) || (j==endOffset-1))
           rv = SerializeRangeNodes(aRange, childAsNode, aString, aDepth+1);
         else
           rv = SerializeToStringRecursive(childAsNode, aString);
+
         NS_ENSURE_SUCCESS(rv, rv);
       }
 
@@ -926,10 +890,8 @@ nsDocumentEncoder::EncodeToString(nsAString& aOutputString)
       mCharsetConverterManager = do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
       NS_ENSURE_SUCCESS(rv, rv);
     }
-    rv = mCharsetConverterManager->GetCharsetAtom(mCharset.get(), getter_AddRefs(charsetAtom));
-    NS_ENSURE_SUCCESS(rv, rv);
   }
-  mSerializer->Init(mFlags, mWrapColumn, charsetAtom, mIsCopying);
+  mSerializer->Init(mFlags, mWrapColumn, mCharset.get(), mIsCopying);
 
   if (mSelection) {
     nsCOMPtr<nsIDOMRange> range;
@@ -983,11 +945,7 @@ nsDocumentEncoder::EncodeToStream(nsIOutputStream* aStream)
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
-  nsCOMPtr<nsIAtom> charsetAtom;
-  rv = mCharsetConverterManager->GetCharsetAtom(mCharset.get(), getter_AddRefs(charsetAtom));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  rv = mCharsetConverterManager->GetUnicodeEncoder(charsetAtom,
+  rv = mCharsetConverterManager->GetUnicodeEncoder(mCharset.get(),
                                                    getter_AddRefs(mUnicodeEncoder));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1118,7 +1076,7 @@ nsHTMLCopyEncoder::Init(nsIDocument* aDocument,
   // (see related bugs #57296, #41924, #58646, #32768)
   mFlags = aFlags | OutputAbsoluteLinks;
 
-  if (!IsScriptEnabled(mDocument))
+  if (!mDocument->IsScriptEnabled())
     mFlags |= OutputNoScriptContent;
 
   return NS_OK;
@@ -1154,12 +1112,13 @@ nsHTMLCopyEncoder::SetSelection(nsISelection* aSelection)
     return NS_ERROR_NULL_POINTER;
   range->GetCommonAncestorContainer(getter_AddRefs(commonParent));
 
-  nsCOMPtr<nsIContent> tmp, selContent( do_QueryInterface(commonParent) );
-  while (selContent)
+  for (nsCOMPtr<nsIContent> selContent(do_QueryInterface(commonParent));
+       selContent;
+       selContent = selContent->GetParent())
   {
     // checking for selection inside a plaintext form widget
     nsCOMPtr<nsIAtom> atom;
-    selContent->GetTag(*getter_AddRefs(atom));
+    selContent->GetTag(getter_AddRefs(atom));
     if (atom.get() == nsHTMLAtoms::input ||
         atom.get() == nsHTMLAtoms::textarea)
     {
@@ -1180,8 +1139,6 @@ nsHTMLCopyEncoder::SetSelection(nsISelection* aSelection)
         break;
       }
     }
-    selContent->GetParent(*getter_AddRefs(tmp));
-    selContent = tmp;
   }
   
   // also consider ourselves in a text widget if we can't find an html document
@@ -1309,7 +1266,7 @@ nsHTMLCopyEncoder::IncludeInContext(nsIDOMNode *aNode)
 
   nsCOMPtr<nsIAtom> tag;
 
-  content->GetTag(*getter_AddRefs(tag));
+  content->GetTag(getter_AddRefs(tag));
 
   if (tag.get() == nsHTMLAtoms::b        ||
       tag.get() == nsHTMLAtoms::i        ||
@@ -1497,16 +1454,16 @@ nsHTMLCopyEncoder::GetPromotedPoint(Endpoint aWhere, nsIDOMNode *aNode, PRInt32
       {
         if (bResetPromotion)
         {
-          nsAutoString tag;
-          nsCOMPtr<nsIAtom> atom;
           nsCOMPtr<nsIContent> content = do_QueryInterface(parent);
           if (content)
           {
-            PRBool isBlock = PR_FALSE;
+            nsCOMPtr<nsIAtom> atom;
+            content->GetTag(getter_AddRefs(atom));
+
             PRInt32 id;
-            content->GetTag(*getter_AddRefs(atom));
-            atom->ToString(tag);
-            parserService->HTMLStringTagToId(tag, &id);
+            parserService->HTMLAtomTagToId(atom, &id);
+
+            PRBool isBlock = PR_FALSE;
             parserService->IsBlock(id, isBlock);
             if (isBlock)
             {
@@ -1586,16 +1543,16 @@ nsHTMLCopyEncoder::GetPromotedPoint(Endpoint aWhere, nsIDOMNode *aNode, PRInt32
       {
         if (bResetPromotion)
         {
-          nsAutoString tag;
-          nsCOMPtr<nsIAtom> atom;
           nsCOMPtr<nsIContent> content = do_QueryInterface(parent);
           if (content)
           {
-            PRBool isBlock = PR_FALSE;
+            nsCOMPtr<nsIAtom> atom;
+            content->GetTag(getter_AddRefs(atom));
+
             PRInt32 id;
-            content->GetTag(*getter_AddRefs(atom));
-            atom->ToString(tag);
-            parserService->HTMLStringTagToId(tag, &id);
+            parserService->HTMLAtomTagToId(atom, &id);
+
+            PRBool isBlock = PR_FALSE;
             parserService->IsBlock(id, isBlock);
             if (isBlock)
             {
@@ -1642,13 +1599,10 @@ nsHTMLCopyEncoder::GetChildAt(nsIDOMNode *aParent, PRInt32 aOffset)
     return resultNode;
   
   nsCOMPtr<nsIContent> content = do_QueryInterface(aParent);
-  nsCOMPtr<nsIContent> cChild;
   NS_PRECONDITION(content, "null content in nsHTMLCopyEncoder::GetChildAt");
-  
-  if (NS_FAILED(content->ChildAt(aOffset, *getter_AddRefs(cChild))))
-    return resultNode;
-  
-  resultNode = do_QueryInterface(cChild);
+
+  resultNode = do_QueryInterface(content->GetChildAt(aOffset));
+
   return resultNode;
 }
 
@@ -1673,7 +1627,9 @@ nsHTMLCopyEncoder::IsMozBR(nsIDOMNode* aNode)
 }
 
 nsresult 
-nsHTMLCopyEncoder::GetNodeLocation(nsIDOMNode *inChild, nsCOMPtr<nsIDOMNode> *outParent, PRInt32 *outOffset)
+nsHTMLCopyEncoder::GetNodeLocation(nsIDOMNode *inChild,
+                                   nsCOMPtr<nsIDOMNode> *outParent,
+                                   PRInt32 *outOffset)
 {
   NS_ASSERTION((inChild && outParent && outOffset), "bad args");
   nsresult result = NS_ERROR_NULL_POINTER;
@@ -1684,8 +1640,10 @@ nsHTMLCopyEncoder::GetNodeLocation(nsIDOMNode *inChild, nsCOMPtr<nsIDOMNode> *ou
     {
       nsCOMPtr<nsIContent> content = do_QueryInterface(*outParent);
       nsCOMPtr<nsIContent> cChild = do_QueryInterface(inChild);
-      if (!cChild || !content) return NS_ERROR_NULL_POINTER;
-      result = content->IndexOf(cChild, *outOffset);
+      if (!cChild || !content)
+        return NS_ERROR_NULL_POINTER;
+
+      *outOffset = content->IndexOf(cChild);
     }
   }
   return result;
