@@ -48,7 +48,8 @@
 #include "nsReadableUtils.h"
 #include "nsMutationEvent.h"
 #include "nsINameSpaceManager.h"
-
+#include "nsIDOM3Node.h"
+#include "nsIURI.h"
 #include "nsIPrivateDOMEvent.h"
 #include "nsIDOMEvent.h"
 #include "nsIDOMText.h"
@@ -145,24 +146,22 @@ nsGenericDOMDataNode::GetParentNode(nsIDOMNode** aParentNode)
 nsresult
 nsGenericDOMDataNode::GetPreviousSibling(nsIDOMNode** aPrevSibling)
 {
-  nsCOMPtr<nsIContent> sibling;
   nsresult rv = NS_OK;
 
   nsIContent *parent_weak = GetParentWeak();
+  nsIContent *sibling = nsnull;
 
   if (parent_weak) {
-    PRInt32 pos;
-    parent_weak->IndexOf(this, pos);
-    if (pos > 0 ) {
-      parent_weak->ChildAt(--pos, *getter_AddRefs(sibling));
+    PRInt32 pos = parent_weak->IndexOf(this);
+    if (pos > 0) {
+      sibling = parent_weak->GetChildAt(pos - 1);
     }
   } else if (mDocument) {
     // Nodes that are just below the document (their parent is the
     // document) need to go to the document to find their next sibling.
-    PRInt32 pos;
-    mDocument->IndexOf(this, pos);
-    if (pos > 0 ) {
-      mDocument->ChildAt(--pos, *getter_AddRefs(sibling));
+    PRInt32 pos = mDocument->IndexOf(this);
+    if (pos > 0) {
+      sibling = mDocument->GetChildAt(pos - 1);
     }
   }
 
@@ -179,25 +178,23 @@ nsGenericDOMDataNode::GetPreviousSibling(nsIDOMNode** aPrevSibling)
 nsresult
 nsGenericDOMDataNode::GetNextSibling(nsIDOMNode** aNextSibling)
 {
-  nsCOMPtr<nsIContent> sibling;
   nsresult rv = NS_OK;
 
   nsIContent *parent_weak = GetParentWeak();
+  nsIContent *sibling = nsnull;
 
   if (parent_weak) {
-    PRInt32 pos;
-    parent_weak->IndexOf(this, pos);
+    PRInt32 pos = parent_weak->IndexOf(this);
     if (pos > -1 ) {
-      parent_weak->ChildAt(++pos, *getter_AddRefs(sibling));
+      sibling = parent_weak->GetChildAt(pos + 1);
     }
   }
   else if (mDocument) {
     // Nodes that are just below the document (their parent is the
     // document) need to go to the document to find their next sibling.
-    PRInt32 pos;
-    mDocument->IndexOf(this, pos);
+    PRInt32 pos = mDocument->IndexOf(this);
     if (pos > -1 ) {
-      mDocument->ChildAt(++pos, *getter_AddRefs(sibling));
+      sibling = mDocument->GetChildAt(pos + 1);
     }
   }
 
@@ -286,30 +283,21 @@ nsGenericDOMDataNode::IsSupported(const nsAString& aFeature,
 nsresult
 nsGenericDOMDataNode::GetBaseURI(nsAString& aURI)
 {
-  aURI.Truncate();
-  nsresult rv = NS_OK;
+  nsCOMPtr<nsIURI> baseURI;
+  nsresult rv = GetBaseURL(getter_AddRefs(baseURI));
 
-  // DOM Data Node inherits the base from its parent element/document
-  nsCOMPtr<nsIDOM3Node> node;
-
-  nsIContent *parent_weak = GetParentWeak();
-
-  if (parent_weak) {
-    node = do_QueryInterface(parent_weak);
-  } else if (mDocument) {
-    node = do_QueryInterface(mDocument);
-  }
-
-  if (node) {
-    rv = node->GetBaseURI(aURI);
+  nsCAutoString spec;
+  if (NS_SUCCEEDED(rv) && baseURI) {
+    baseURI->GetSpec(spec);
   }
+  CopyUTF8toUTF16(spec, aURI);
 
   return rv;
 }
 
 nsresult
-nsGenericDOMDataNode::LookupNamespacePrefix(const nsAString& aNamespaceURI,
-                                            nsAString& aPrefix)
+nsGenericDOMDataNode::LookupPrefix(const nsAString& aNamespaceURI,
+                                   nsAString& aPrefix)
 {
   aPrefix.Truncate();
 
@@ -317,9 +305,8 @@ nsGenericDOMDataNode::LookupNamespacePrefix(const nsAString& aNamespaceURI,
 
   // DOM Data Node passes the query on to its parent
   nsCOMPtr<nsIDOM3Node> node(do_QueryInterface(parent_weak));
-
   if (node) {
-    return node->LookupNamespacePrefix(aNamespaceURI, aPrefix);
+    return node->LookupPrefix(aNamespaceURI, aPrefix);
   }
 
   return NS_OK;
@@ -462,10 +449,10 @@ nsGenericDOMDataNode::AppendData(const nsAString& aData)
     if (NS_SUCCEEDED(rv)) {
       tccd->SetData(nsITextContentChangeData::Append, length,
                     aData.Length());
-      rv = mDocument->ContentChanged(this, tccd);
+      mDocument->ContentChanged(this, tccd);
       NS_RELEASE(tccd);
     } else {
-      rv = mDocument->ContentChanged(this, nsnull);
+      mDocument->ContentChanged(this, nsnull);
     }
   }
 
@@ -637,12 +624,10 @@ nsGenericDOMDataNode::ToCString(nsAString& aBuf, PRInt32 aOffset,
 }
 #endif
 
-NS_IMETHODIMP
-nsGenericDOMDataNode::GetDocument(nsIDocument*& aResult) const
+NS_IMETHODIMP_(nsIDocument*)
+nsGenericDOMDataNode::GetDocument() const
 {
-  aResult = mDocument;
-  NS_IF_ADDREF(mDocument);
-  return NS_OK;
+  return mDocument;
 }
 
 
@@ -659,13 +644,10 @@ nsGenericDOMDataNode::SetDocument(nsIDocument* aDocument, PRBool aDeep,
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsGenericDOMDataNode::GetParent(nsIContent*& aResult) const
+NS_IMETHODIMP_(nsIContent*)
+nsGenericDOMDataNode::GetParent() const
 {
-  aResult = GetParentWeak();
-  NS_IF_ADDREF(aResult);
-
-  return NS_OK;;
+  return GetParentWeak();
 }
 
 NS_IMETHODIMP
@@ -694,17 +676,29 @@ nsGenericDOMDataNode::SetNativeAnonymous(PRBool aAnonymous)
 }
 
 NS_IMETHODIMP
-nsGenericDOMDataNode::GetNameSpaceID(PRInt32& aID) const
+nsGenericDOMDataNode::GetNameSpaceID(PRInt32* aID) const
 {
-  aID = kNameSpaceID_None;
+  *aID = kNameSpaceID_None;
   return NS_OK;
 }
 
+NS_IMETHODIMP_(nsIAtom*)
+nsGenericDOMDataNode::GetIDAttributeName() const
+{
+  return nsnull;
+}
+
+NS_IMETHODIMP_(nsIAtom*)
+nsGenericDOMDataNode::GetClassAttributeName() const
+{
+  return nsnull;
+}
+
 NS_IMETHODIMP
 nsGenericDOMDataNode::NormalizeAttrString(const nsAString& aStr,
-                                          nsINodeInfo*& aNodeInfo)
+                                          nsINodeInfo** aNodeInfo)
 {
-  aNodeInfo = nsnull;
+  *aNodeInfo = nsnull;
   return NS_OK;
 }
 
@@ -740,10 +734,10 @@ nsGenericDOMDataNode::GetAttr(PRInt32 aNameSpaceID, nsIAtom *aAttr,
 
 NS_IMETHODIMP
 nsGenericDOMDataNode::GetAttr(PRInt32 aNameSpaceID, nsIAtom *aAttr,
-                              nsIAtom*& aPrefix,
+                              nsIAtom** aPrefix,
                               nsAString& aResult) const
 {
-  aPrefix = nsnull;
+  *aPrefix = nsnull;
   aResult.Truncate();
 
   return NS_CONTENT_ATTR_NOT_THERE;
@@ -756,20 +750,20 @@ nsGenericDOMDataNode::HasAttr(PRInt32 aNameSpaceID, nsIAtom *aAttribute) const
 }
 
 NS_IMETHODIMP
-nsGenericDOMDataNode::GetAttrNameAt(PRInt32 aIndex, PRInt32& aNameSpaceID,
-                                    nsIAtom*& aName, nsIAtom*& aPrefix) const
+nsGenericDOMDataNode::GetAttrNameAt(PRUint32 aIndex, PRInt32* aNameSpaceID,
+                                    nsIAtom** aName, nsIAtom** aPrefix) const
 {
-  aNameSpaceID = kNameSpaceID_None;
-  aName = nsnull;
-  aPrefix = nsnull;
+  *aNameSpaceID = kNameSpaceID_None;
+  *aName = nsnull;
+  *aPrefix = nsnull;
+
   return NS_ERROR_ILLEGAL_VALUE;
 }
 
-NS_IMETHODIMP
-nsGenericDOMDataNode::GetAttrCount(PRInt32& aResult) const
+NS_IMETHODIMP_(PRUint32)
+nsGenericDOMDataNode::GetAttrCount() const
 {
-  aResult = 0;
-  return NS_OK;
+  return 0;
 }
 
 NS_IMETHODIMP
@@ -870,51 +864,45 @@ nsGenericDOMDataNode::SetContentID(PRUint32 aID)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsGenericDOMDataNode::GetNodeInfo(nsINodeInfo*& aResult) const
+NS_IMETHODIMP_(nsINodeInfo *)
+nsGenericDOMDataNode::GetNodeInfo() const
 {
-  aResult = nsnull;
-  return NS_OK;
+  return nsnull;
+}
+
+NS_IMETHODIMP_(PRBool)
+nsGenericDOMDataNode::CanContainChildren() const
+{
+  return PR_FALSE;
+}
+
+NS_IMETHODIMP_(PRUint32)
+nsGenericDOMDataNode::GetChildCount() const
+{
+  return 0;
+}
+
+NS_IMETHODIMP_(nsIContent *)
+nsGenericDOMDataNode::GetChildAt(PRUint32 aIndex) const
+{
+  return nsnull;
+}
+
+NS_IMETHODIMP_(PRInt32)
+nsGenericDOMDataNode::IndexOf(nsIContent* aPossibleChild) const
+{
+  return -1;
 }
 
 NS_IMETHODIMP
-nsGenericDOMDataNode::CanContainChildren(PRBool& aResult) const
-{
-  aResult = PR_FALSE;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsGenericDOMDataNode::ChildCount(PRInt32& aResult) const
-{
-  aResult = 0;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsGenericDOMDataNode::ChildAt(PRInt32 aIndex, nsIContent*& aResult) const
-{
-  aResult = nsnull;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsGenericDOMDataNode::IndexOf(nsIContent* aPossibleChild,
-                              PRInt32& aResult) const
-{
-  aResult = -1;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsGenericDOMDataNode::InsertChildAt(nsIContent* aKid, PRInt32 aIndex,
+nsGenericDOMDataNode::InsertChildAt(nsIContent* aKid, PRUint32 aIndex,
                                     PRBool aNotify, PRBool aDeepSetDocument)
 {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsGenericDOMDataNode::ReplaceChildAt(nsIContent* aKid, PRInt32 aIndex,
+nsGenericDOMDataNode::ReplaceChildAt(nsIContent* aKid, PRUint32 aIndex,
                                      PRBool aNotify, PRBool aDeepSetDocument)
 {
   return NS_OK;
@@ -928,7 +916,7 @@ nsGenericDOMDataNode::AppendChildTo(nsIContent* aKid, PRBool aNotify,
 }
 
 NS_IMETHODIMP
-nsGenericDOMDataNode::RemoveChildAt(PRInt32 aIndex, PRBool aNotify)
+nsGenericDOMDataNode::RemoveChildAt(PRUint32 aIndex, PRBool aNotify)
 {
   return NS_OK;
 }
@@ -1018,9 +1006,9 @@ nsGenericDOMDataNode::RangeRemove(nsIDOMRange* aRange)
 
 
 NS_IMETHODIMP
-nsGenericDOMDataNode::GetRangeList(nsVoidArray*& aResult) const
+nsGenericDOMDataNode::GetRangeList(nsVoidArray** aResult) const
 {
-  aResult = LookupRangeList();
+  *aResult = LookupRangeList();
   return NS_OK;
 }
 
@@ -1036,10 +1024,10 @@ nsGenericDOMDataNode::RemoveFocus(nsIPresContext* aPresContext)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsGenericDOMDataNode::GetBindingParent(nsIContent** aContent)
+NS_IMETHODIMP_(nsIContent*)
+nsGenericDOMDataNode::GetBindingParent() const
 {
-  return NS_OK;
+  return nsnull;
 }
 
 NS_IMETHODIMP
@@ -1069,6 +1057,24 @@ nsGenericDOMDataNode::DumpContent(FILE* out, PRInt32 aIndent,
 }
 #endif
 
+NS_IMETHODIMP
+nsGenericDOMDataNode::GetBaseURL(nsIURI** aURI) const
+{
+  // DOM Data Node inherits the base from its parent element/document
+  nsIContent* parent_weak = GetParentWeak();
+  if (parent_weak) {
+    return parent_weak->GetBaseURL(aURI);
+  }
+
+  if (mDocument) {
+    NS_IF_ADDREF(*aURI = mDocument->GetBaseURL());
+  } else {
+    *aURI = nsnull;
+  }
+
+  return NS_OK;
+}
+
 //----------------------------------------------------------------------
 
 // Implementation of the nsIDOMText interface
@@ -1117,18 +1123,14 @@ nsGenericDOMDataNode::SplitText(PRUint32 aOffset, nsIDOMText** aReturn)
     return rv;
   }
 
-  nsCOMPtr<nsIContent> parentNode;
-  GetParent(*getter_AddRefs(parentNode));
+  nsIContent* parentNode = GetParentWeak();
 
   if (parentNode) {
-    PRInt32 index;
+    PRInt32 index = parentNode->IndexOf(this);
 
-    rv = parentNode->IndexOf(this, index);
-    if (NS_SUCCEEDED(rv)) {
-      nsCOMPtr<nsIContent> content(do_QueryInterface(newNode));
+    nsCOMPtr<nsIContent> content(do_QueryInterface(newNode));
 
-      rv = parentNode->InsertChildAt(content, index+1, PR_TRUE, PR_FALSE);
-    }
+    parentNode->InsertChildAt(content, index+1, PR_TRUE, PR_FALSE);
   }
 
   return CallQueryInterface(newNode, aReturn);
@@ -1186,9 +1188,7 @@ nsGenericDOMDataNode::SetText(const PRUnichar* aBuffer,
     return NS_ERROR_NULL_POINTER;
   }
 
-  if (aNotify && mDocument) {
-    mDocument->BeginUpdate();
-  }
+  mozAutoDocUpdate updateBatch(mDocument, UPDATE_CONTENT_MODEL, aNotify);
 
   mText.SetTo(aBuffer, aLength);
 
@@ -1213,7 +1213,6 @@ nsGenericDOMDataNode::SetText(const PRUnichar* aBuffer,
   // Trigger a reflow
   if (aNotify && mDocument) {
     mDocument->ContentChanged(this, nsnull);
-    mDocument->EndUpdate();
   }
   return NS_OK;
 }
@@ -1230,9 +1229,7 @@ nsGenericDOMDataNode::SetText(const char* aBuffer, PRInt32 aLength,
     return NS_ERROR_NULL_POINTER;
   }
 
-  if (aNotify && mDocument) {
-    mDocument->BeginUpdate();
-  }
+  mozAutoDocUpdate updateBatch(mDocument, UPDATE_CONTENT_MODEL, aNotify);
 
   mText.SetTo(aBuffer, aLength);
 
@@ -1255,7 +1252,6 @@ nsGenericDOMDataNode::SetText(const char* aBuffer, PRInt32 aLength,
   // Trigger a reflow
   if (aNotify && mDocument) {
     mDocument->ContentChanged(this, nsnull);
-    mDocument->EndUpdate();
   }
 
   return NS_OK;
@@ -1265,9 +1261,7 @@ NS_IMETHODIMP
 nsGenericDOMDataNode::SetText(const nsAString& aStr,
                               PRBool aNotify)
 {
-  if (aNotify && mDocument) {
-    mDocument->BeginUpdate();
-  }
+  mozAutoDocUpdate updateBatch(mDocument, UPDATE_CONTENT_MODEL, aNotify);
 
   mText = aStr;
 
@@ -1292,7 +1286,6 @@ nsGenericDOMDataNode::SetText(const nsAString& aStr,
   // Trigger a reflow
   if (aNotify && mDocument) {
     mDocument->ContentChanged(this, nsnull);
-    mDocument->EndUpdate();
   }
 
   return NS_OK;
@@ -1346,6 +1339,23 @@ nsGenericDOMDataNode::CloneContent(PRBool aCloneText, nsITextContent** aClone)
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
+NS_IMETHODIMP
+nsGenericDOMDataNode::AppendTextTo(nsAString& aResult)
+{
+  if (mText.Is2b()) {
+    aResult.Append(mText.Get2b(), mText.GetLength());
+  } else {
+
+    // XXX we would like to have a AppendASCIItoUCS2 here
+
+    aResult.Append(NS_ConvertASCIItoUCS2(mText.Get1b(),
+                                         mText.GetLength()).get(),
+                   mText.GetLength());
+  }
+
+  return NS_OK;
+}
+
 void
 nsGenericDOMDataNode::LookupListenerManager(nsIEventListenerManager **aListenerManager) const
 {
@@ -1388,16 +1398,9 @@ nsGenericDOMDataNode::LookupRangeList() const
 
 void nsGenericDOMDataNode::SetBidiStatus()
 {
-  if (mDocument) {
-    PRBool isBidiDocument = PR_FALSE;
-
-    mDocument->GetBidiEnabled(&isBidiDocument);
-
-    if (isBidiDocument) {
-      // OK, we already know it's Bidi, so we won't test again
-
-      return;
-    }
+  if (mDocument && mDocument->GetBidiEnabled()) {
+    // OK, we already know it's Bidi, so we won't test again
+    return;
   }
 
   mText.SetBidiFlag();
