@@ -91,7 +91,7 @@ nsNodeInfo::Init(nsIAtom *aName, nsIAtom *aPrefix, PRInt32 aNamespaceID,
 
 // nsISupports
 
-NS_IMPL_ISUPPORTS1(nsNodeInfo, nsINodeInfo);
+NS_IMPL_ISUPPORTS1(nsNodeInfo, nsINodeInfo)
 
 
 // nsINodeInfo
@@ -149,13 +149,10 @@ nsNodeInfo::GetNamespaceURI(nsAString& aNameSpaceURI) const
 }
 
 
-NS_IMETHODIMP
-nsNodeInfo::GetIDAttributeAtom(nsIAtom** aResult) const
+NS_IMETHODIMP_(nsIAtom*)
+nsNodeInfo::GetIDAttributeAtom() const
 {
-  *aResult = mIDAttributeAtom;
-  NS_IF_ADDREF(*aResult);
-
-  return NS_OK;
+  return mIDAttributeAtom;
 }
 
 NS_IMETHODIMP
@@ -288,10 +285,10 @@ nsNodeInfo::PrefixChanged(nsIAtom *aPrefix, nsINodeInfo** aResult)
                                     aResult);
 }
 
-NS_IMETHODIMP
-nsNodeInfo::GetDocument(nsIDocument** aDocument) const
+nsIDocument*
+nsNodeInfo::GetDocument() const
 {
-  return mOwnerManager->GetDocument(aDocument);
+  return mOwnerManager->GetDocument();
 }
 
 NS_IMETHODIMP
