@@ -92,7 +92,7 @@ nsStyleContext::~nsStyleContext()
 {
   NS_ASSERTION((nsnull == mChild) && (nsnull == mEmptyChild), "destructing context with children");
 
-  nsIPresContext *presContext = mRuleNode->PresContext();
+  nsIPresContext *presContext = mRuleNode->GetPresContext();
 
   nsCOMPtr<nsIPresShell> shell;
   presContext->GetShell(getter_AddRefs(shell));
@@ -172,14 +172,6 @@ void nsStyleContext::RemoveChild(nsStyleContext* aChild)
   aChild->mPrevSibling = aChild;
 }
 
-already_AddRefed<nsIAtom>
-nsStyleContext::GetPseudoType() const
-{
-  nsIAtom* pseudoTag = mPseudoTag;
-  NS_IF_ADDREF(pseudoTag);
-  return pseudoTag;
-}
-
 already_AddRefed<nsStyleContext>
 nsStyleContext::FindChildWithRules(const nsIAtom* aPseudoTag, 
                                    nsRuleNode* aRuleNode)
@@ -347,14 +339,13 @@ nsStyleContext::SetStyle(nsStyleStructID aSID, nsStyleStruct* aStruct)
   char* resetOrInherit = NS_REINTERPRET_CAST(char*,
       *NS_REINTERPRET_CAST(void**, resetOrInheritSlot));
   if (!resetOrInherit) {
-    nsCOMPtr<nsIPresContext> presContext;
-    mRuleNode->GetPresContext(getter_AddRefs(presContext));
+    nsIPresContext *presContext = mRuleNode->GetPresContext();
     if (mCachedStyleData.IsReset(aSID)) {
-      mCachedStyleData.mResetData = new (presContext.get()) nsResetStyleData;
+      mCachedStyleData.mResetData = new (presContext) nsResetStyleData;
       resetOrInherit = NS_REINTERPRET_CAST(char*, mCachedStyleData.mResetData);
     } else {
       mCachedStyleData.mInheritedData =
-          new (presContext.get()) nsInheritedStyleData;
+          new (presContext) nsInheritedStyleData;
       resetOrInherit =
           NS_REINTERPRET_CAST(char*, mCachedStyleData.mInheritedData);
     }
@@ -396,27 +387,20 @@ nsStyleContext::ApplyStyleFixups(nsIPresContext* aPresContext)
 }
 
 void
-nsStyleContext::ClearStyleData(nsIPresContext* aPresContext, nsIStyleRule* aRule)
+nsStyleContext::ClearStyleData(nsIPresContext* aPresContext)
 {
-  PRBool matched = PR_TRUE;
-  if (aRule)
-    mRuleNode->PathContainsRule(aRule, &matched);
-  
-  if (matched) {
-    // First we need to clear out all of our style data.
-    if (mCachedStyleData.mResetData || mCachedStyleData.mInheritedData)
-      mCachedStyleData.Destroy(mBits, aPresContext);
+  // First we need to clear out all of our style data.
+  if (mCachedStyleData.mResetData || mCachedStyleData.mInheritedData)
+    mCachedStyleData.Destroy(mBits, aPresContext);
 
-    mBits = 0; // Clear all bits.
-    aRule = nsnull; // Force all structs to be blown away in the children.
-  }
+  mBits = 0; // Clear all bits.
 
   ApplyStyleFixups(aPresContext);
 
   if (mChild) {
     nsStyleContext* child = mChild;
     do {
-      child->ClearStyleData(aPresContext, aRule);
+      child->ClearStyleData(aPresContext);
       child = child->mNextSibling;
     } while (mChild != child);
   }
@@ -424,7 +408,7 @@ nsStyleContext::ClearStyleData(nsIPresContext* aPresContext, nsIStyleRule* aRule
   if (mEmptyChild) {
     nsStyleContext* child = mEmptyChild;
     do {
-      child->ClearStyleData(aPresContext, aRule);
+      child->ClearStyleData(aPresContext);
       child = child->mNextSibling;
     } while (mEmptyChild != child);
   }
@@ -542,6 +526,23 @@ nsStyleContext::Mark()
 }
 
 #ifdef DEBUG
+
+class URICString : public nsCAutoString {
+public:
+  URICString(nsIURI* aURI) {
+    if (aURI) {
+      aURI->GetSpec(*this);
+    } else {
+      Assign("[none]");
+    }
+  }
+
+  URICString& operator=(const URICString& aOther) {
+    Assign(aOther);
+    return *this;
+  }
+};
+
 void nsStyleContext::List(FILE* out, PRInt32 aIndent)
 {
   // Indent
@@ -560,8 +561,7 @@ void nsStyleContext::List(FILE* out, PRInt32 aIndent)
     fputs("{\n", out);
     nsRuleNode* ruleNode = mRuleNode;
     while (ruleNode) {
-      nsCOMPtr<nsIStyleRule> styleRule;
-      ruleNode->GetRule(getter_AddRefs(styleRule));
+      nsIStyleRule *styleRule = ruleNode->GetRule();
       if (styleRule) {
         styleRule->List(out, aIndent + 1);
       }
@@ -623,7 +623,7 @@ void nsStyleContext::DumpRegressionData(nsIPresContext* aPresContext, FILE* out,
     (long)bg->mBackgroundColor,
     (long)bg->mBackgroundXPosition.mCoord, // potentially lossy on some platforms
     (long)bg->mBackgroundYPosition.mCoord, // potentially lossy on some platforms
-    NS_ConvertUCS2toUTF8(bg->mBackgroundImage).get());
+    URICString(bg->mBackgroundImage).get());
  
   // SPACING (ie. margin, padding, border, outline)
   IndentBy(out,aIndent);
@@ -657,7 +657,7 @@ void nsStyleContext::DumpRegressionData(nsIPresContext* aPresContext, FILE* out,
   fprintf(out, "<list data=\"%d %d %s\" />\n",
     (int)list->mListStyleType,
     (int)list->mListStyleType,
-    NS_ConvertUCS2toUTF8(list->mListStyleImage).get());
+    URICString(list->mListStyleImage).get());
 
   // POSITION
   IndentBy(out,aIndent);
@@ -724,7 +724,7 @@ void nsStyleContext::DumpRegressionData(nsIPresContext* aPresContext, FILE* out,
     (long)disp->mClip.y,
     (long)disp->mClip.width,
     (long)disp->mClip.height,
-    NS_ConvertUCS2toUTF8(disp->mBinding).get()
+    URICString(disp->mBinding).get()
     );
   
   // VISIBILITY
@@ -790,7 +790,7 @@ void nsStyleContext::DumpRegressionData(nsIPresContext* aPresContext, FILE* out,
     (int)ui->mUserModify,
     (int)ui->mUserFocus, 
     (int)ui->mCursor,
-    NS_ConvertUCS2toUTF8(ui->mCursorImage).get());
+    URICString(ui->mCursorImage).get());
 
   // UIReset
   IndentBy(out,aIndent);
@@ -845,8 +845,7 @@ void
 nsStyleContext::Destroy()
 {
   // Get the pres context from our rule node.
-  nsCOMPtr<nsIPresContext> presContext;
-  mRuleNode->GetPresContext(getter_AddRefs(presContext));
+  nsCOMPtr<nsIPresContext> presContext = mRuleNode->GetPresContext();
 
   // Call our destructor.
   this->~nsStyleContext();
