@@ -388,9 +388,7 @@ nsSyncLoader::LoadDocument(nsIChannel* aChannel,
 
     NS_ENSURE_TRUE(mLoadSuccess, NS_ERROR_FAILURE);
 
-    nsCOMPtr<nsIContent> rootContent;
-    document->GetRootContent(getter_AddRefs(rootContent));
-    NS_ENSURE_TRUE(rootContent, NS_ERROR_FAILURE);
+    NS_ENSURE_TRUE(document->GetRootContent(), NS_ERROR_FAILURE);
 
     return CallQueryInterface(document, aResult);
 }
