@@ -71,9 +71,8 @@ NS_NewTreeWalker(nsIDOMNode *aRoot,
                                             aFilter,
                                             aEntityReferenceExpansion);
     NS_ENSURE_TRUE(walker, NS_ERROR_OUT_OF_MEMORY);
-    
-    return walker->QueryInterface(NS_GET_IID(nsIDOMTreeWalker),
-                                  (void**) aInstancePtrResult);
+
+    return CallQueryInterface(walker, aInstancePtrResult);
 }
 
 nsTreeWalker::nsTreeWalker(nsIDOMNode *aRoot,
@@ -87,7 +86,6 @@ nsTreeWalker::nsTreeWalker(nsIDOMNode *aRoot,
     mCurrentNode(aRoot),
     mPossibleIndexesPos(-1)
 {
-    NS_INIT_ISUPPORTS();
 
     NS_ASSERTION(aRoot, "invalid root in call to nsTreeWalker constructor");
 }
@@ -159,6 +157,11 @@ NS_IMETHODIMP nsTreeWalker::GetCurrentNode(nsIDOMNode * *aCurrentNode)
 NS_IMETHODIMP nsTreeWalker::SetCurrentNode(nsIDOMNode * aCurrentNode)
 {
     NS_ENSURE_TRUE(aCurrentNode, NS_ERROR_DOM_NOT_SUPPORTED_ERR);
+
+    nsresult rv = nsContentUtils::CheckSameOrigin(mRoot, aCurrentNode);
+    if(NS_FAILED(rv))
+        return rv;
+
     mCurrentNode = aCurrentNode;
     return NS_OK;
 }
@@ -619,15 +622,13 @@ nsresult nsTreeWalker::IndexOf(nsIDOMNode* aParent,
         nsCOMPtr<nsIContent> child(do_QueryInterface(aChild));
 
         if (possibleIndex >= 0) {
-            nsCOMPtr<nsIContent> tmp;
-            contParent->ChildAt(possibleIndex, *getter_AddRefs(tmp));
-            if (tmp == child) {
+            if (child == contParent->GetChildAt(possibleIndex)) {
                 *_childNum = possibleIndex;
                 return NS_OK;
             }
         }
 
-        contParent->IndexOf(child, *_childNum);
+        *_childNum = contParent->IndexOf(child);
 
         return *_childNum >= 0 ? NS_OK : NS_ERROR_UNEXPECTED;
     }
@@ -637,15 +638,13 @@ nsresult nsTreeWalker::IndexOf(nsIDOMNode* aParent,
         nsCOMPtr<nsIContent> child(do_QueryInterface(aChild));
 
         if (possibleIndex >= 0) {
-            nsCOMPtr<nsIContent> tmp;
-            docParent->ChildAt(possibleIndex, *getter_AddRefs(tmp));
-            if (tmp == child) {
+            if (child == docParent->GetChildAt(possibleIndex)) {
                 *_childNum = possibleIndex;
                 return NS_OK;
             }
         }
 
-        docParent->IndexOf(child, *_childNum);
+        *_childNum = docParent->IndexOf(child);
 
         return *_childNum >= 0 ? NS_OK : NS_ERROR_UNEXPECTED;
     }
