@@ -74,9 +74,10 @@ nsresult NS_NewXMLContentSerializer(nsIContentSerializer** aSerializer)
 }
 
 nsXMLContentSerializer::nsXMLContentSerializer()
+  : mPrefixIndex(0),
+    mInAttribute(PR_FALSE),
+    mAddNewline(PR_FALSE)
 {
-  mPrefixIndex = 0;
-  mInAttribute = PR_FALSE;
 }
  
 nsXMLContentSerializer::~nsXMLContentSerializer()
@@ -87,7 +88,7 @@ NS_IMPL_ISUPPORTS1(nsXMLContentSerializer, nsIContentSerializer)
 
 NS_IMETHODIMP 
 nsXMLContentSerializer::Init(PRUint32 flags, PRUint32 aWrapColumn,
-                             nsIAtom* aCharSet, PRBool aIsCopying)
+                             const char* aCharSet, PRBool aIsCopying)
 {
   return NS_OK;
 }
@@ -174,6 +175,8 @@ nsXMLContentSerializer::AppendProcessingInstruction(nsIDOMProcessingInstruction*
   nsresult rv;
   nsAutoString target, data;
 
+  MaybeAddNewline(aStr);
+
   rv = aPI->GetTarget(target);
   if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
@@ -187,6 +190,7 @@ nsXMLContentSerializer::AppendProcessingInstruction(nsIDOMProcessingInstruction*
     AppendToString(data, aStr);
   }
   AppendToString(NS_LITERAL_STRING("?>"), aStr);
+  MaybeFlagNewline(aPI);
   
   return NS_OK;
 }
@@ -204,6 +208,8 @@ nsXMLContentSerializer::AppendComment(nsIDOMComment* aComment,
   rv = aComment->GetData(data);
   if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
+  MaybeAddNewline(aStr);
+
   AppendToString(NS_LITERAL_STRING("<!--"), aStr);
   if (aStartOffset || (aEndOffset != -1)) {
     PRInt32 length = (aEndOffset == -1) ? data.Length() : aEndOffset;
@@ -217,6 +223,7 @@ nsXMLContentSerializer::AppendComment(nsIDOMComment* aComment,
     AppendToString(data, aStr);
   }
   AppendToString(NS_LITERAL_STRING("-->"), aStr);
+  MaybeFlagNewline(aComment);
   
   return NS_OK;
 }
@@ -238,6 +245,8 @@ nsXMLContentSerializer::AppendDoctype(nsIDOMDocumentType *aDoctype,
   rv = aDoctype->GetInternalSubset(internalSubset);
   if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
+  MaybeAddNewline(aStr);
+
   AppendToString(NS_LITERAL_STRING("<!DOCTYPE "), aStr);
   AppendToString(name, aStr);
   PRUnichar quote;
@@ -286,6 +295,7 @@ nsXMLContentSerializer::AppendDoctype(nsIDOMDocumentType *aDoctype,
   }
     
   AppendToString(PRUnichar('>'), aStr);
+  MaybeFlagNewline(aDoctype);
 
   return NS_OK;
 }
@@ -487,19 +497,19 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
 
   PRInt32 namespaceID;
     
-  PRInt32 index, count;
+  PRUint32 index, count;
   nsAutoString nameStr, prefixStr, uriStr, valueStr;
   nsCOMPtr<nsIAtom> attrName, attrPrefix;
 
-  content->GetAttrCount(count);
-  
+  count = content->GetAttrCount();
+
   // First scan for namespace declarations, pushing each on the stack
   for (index = 0; index < count; index++) {
     
-    content->GetAttrNameAt(index, 
-                           namespaceID,
-                           *getter_AddRefs(attrName),
-                           *getter_AddRefs(attrPrefix));
+    content->GetAttrNameAt(index,
+                           &namespaceID,
+                           getter_AddRefs(attrName),
+                           getter_AddRefs(attrPrefix));
     
     if (namespaceID == kNameSpaceID_XMLNS) {
       content->GetAttr(namespaceID, attrName, uriStr);
@@ -516,6 +526,8 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
 
   PRBool addNSAttr;
     
+  MaybeAddNewline(aStr);
+
   addNSAttr = ConfirmPrefix(tagPrefix, tagNamespaceURI);
   // Serialize the qualified name of the element
   AppendToString(NS_LITERAL_STRING("<"), aStr);
@@ -536,10 +548,10 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
   // XXX Unfortunately we need a namespace manager to get
   // attribute URIs.
   for (index = 0; index < count; index++) {
-    content->GetAttrNameAt(index, 
-                           namespaceID,
-                           *getter_AddRefs(attrName),
-                           *getter_AddRefs(attrPrefix));
+    content->GetAttrNameAt(index,
+                           &namespaceID,
+                           getter_AddRefs(attrName),
+                           getter_AddRefs(attrPrefix));
 
     if (attrPrefix) {
       attrPrefix->ToString(prefixStr);
@@ -566,10 +578,10 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
 
     if (namespaceID == kNameSpaceID_None) {
       PRInt32 elementNsID;
-      content->GetNameSpaceID(elementNsID);
+      content->GetNameSpaceID(&elementNsID);
       if (elementNsID == kNameSpaceID_XHTML) {
         nsCOMPtr<nsIAtom> elementName;
-        content->GetTag(*getter_AddRefs(elementName));
+        content->GetTag(getter_AddRefs(elementName));
         if (IsShorthandAttr(attrName, elementName) &&
             valueStr.IsEmpty()) {
           valueStr = nameStr;
@@ -587,6 +599,7 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
   // We don't output a separate end tag for empty element
   if (!aHasChildren) {
     AppendToString(NS_LITERAL_STRING("/>"), aStr);    
+    MaybeFlagNewline(aElement);
   } else {
     AppendToString(NS_LITERAL_STRING(">"), aStr);    
   }
@@ -625,6 +638,7 @@ nsXMLContentSerializer::AppendElementEnd(nsIDOMElement *aElement,
   }
   AppendToString(tagLocalName, aStr);
   AppendToString(NS_LITERAL_STRING(">"), aStr);
+  MaybeFlagNewline(aElement);
   
   PopNameSpaceDeclsFor(aElement);
   
@@ -648,7 +662,7 @@ nsXMLContentSerializer::AppendToString(const PRUnichar aChar,
   aOutputStr.Append(aChar);
 }
 
-static PRUint16 kGTVal = 62;
+static const PRUint16 kGTVal = 62;
 static const char* kEntities[] = {
   "", "", "", "", "", "", "", "", "", "",
   "", "", "", "", "", "", "", "", "", "",
@@ -809,6 +823,27 @@ nsXMLContentSerializer::IsShorthandAttr(const nsIAtom* aAttrName,
   return PR_FALSE;
 }
 
+void
+nsXMLContentSerializer::MaybeAddNewline(nsAString& aStr)
+{
+  if (mAddNewline) {
+    aStr.Append((PRUnichar)'\n');
+    mAddNewline = PR_FALSE;
+  }
+}
+
+void
+nsXMLContentSerializer::MaybeFlagNewline(nsIDOMNode* aNode)
+{
+  nsCOMPtr<nsIDOMNode> parent;
+  aNode->GetParentNode(getter_AddRefs(parent));
+  if (parent) {
+    PRUint16 type;
+    parent->GetNodeType(&type);
+    mAddNewline = type == nsIDOMNode::DOCUMENT_NODE;
+  }
+}
+
 NS_IMETHODIMP
 nsXMLContentSerializer::AppendDocumentStart(nsIDOMDocument *aDocument,
                                             nsAString& aStr)
@@ -839,6 +874,7 @@ nsXMLContentSerializer::AppendDocumentStart(nsIDOMDocument *aDocument,
   }
 
   aStr += NS_LITERAL_STRING("?>");
+  mAddNewline = PR_TRUE;
 
   return NS_OK;
 }
