@@ -1072,11 +1072,9 @@ SendJSWarning(nsIHTMLContent* aContent,
   //
   nsCAutoString documentURLSpec;
   {
-    nsCOMPtr<nsIDocument> document;
-    aContent->GetDocument(getter_AddRefs(document));
+    nsIDocument* document = aContent->GetDocument();
     if (document) {
-      nsCOMPtr<nsIURI> documentURL;
-      document->GetDocumentURL(getter_AddRefs(documentURL));
+      nsIURI *documentURL = document->GetDocumentURL();
       NS_ENSURE_TRUE(documentURL, NS_ERROR_UNEXPECTED);
       documentURL->GetPath(documentURLSpec);
     }
@@ -1273,10 +1271,9 @@ nsFormSubmission::GetSubmitCharset(nsIHTMLContent* aForm,
   }
   // if there are no accept-charset or all the charset are not supported
   // Get the charset from document
-  nsCOMPtr<nsIDocument> doc;
-  aForm->GetDocument(getter_AddRefs(doc));
+  nsIDocument* doc = aForm->GetDocument();
   if (doc) {
-    rv = doc->GetDocumentCharacterSet(oCharset);
+    oCharset = doc->GetDocumentCharacterSet();
   }
 
   if (aCtrlsModAtSubmit==IBMBIDI_CONTROLSTEXTMODE_VISUAL
