@@ -192,11 +192,11 @@ public:
   NS_IMETHOD_(PRBool) HasAttr(PRInt32 aNameSpaceID, nsIAtom* aName) const;
   NS_IMETHOD UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
                        PRBool aNotify);
-  NS_IMETHOD GetAttrNameAt(PRInt32 aIndex,
+  NS_IMETHOD GetAttrNameAt(PRUint32 aIndex,
                            PRInt32* aNameSpaceID,
                            nsIAtom** aName,
                            nsIAtom** aPrefix) const;
-  NS_IMETHOD GetAttrCount(PRInt32& aResult) const;
+  NS_IMETHOD_(PRUint32) GetAttrCount() const;
 #ifdef DEBUG
   NS_IMETHOD List(FILE* out, PRInt32 aIndent) const;
   NS_IMETHOD DumpContent(FILE* out, PRInt32 aIndent,PRBool aDumpAll) const;
@@ -223,6 +223,8 @@ public:
   NS_IMETHOD GetHTMLAttribute(nsIAtom* aAttribute, nsHTMLValue& aValue) const;
   NS_IMETHOD GetID(nsIAtom** aResult) const;
   NS_IMETHOD GetClasses(nsVoidArray& aArray) const;
+  NS_IMETHOD_(nsIAtom*) GetIDAttributeName() const;
+  NS_IMETHOD_(nsIAtom*) GetClassAttributeName() const;
   NS_IMETHOD_(PRBool) HasClass(nsIAtom* aClass, PRBool aCaseSensitive) const;
   NS_IMETHOD WalkContentStyleRules(nsRuleWalker* aRuleWalker);
   NS_IMETHOD GetInlineStyleRule(nsIStyleRule** aStyleRule);
@@ -233,8 +235,7 @@ public:
   NS_IMETHOD AttributeToString(nsIAtom* aAttribute,
                                const nsHTMLValue& aValue,
                                nsAString& aResult) const;
-  NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
-                                      nsChangeHint& aHint) const;
+  NS_IMETHOD_(PRBool) HasAttributeDependentStyle(const nsIAtom* aAttribute) const;
   NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
 
 #ifdef DEBUG
@@ -493,32 +494,28 @@ public:
    */
   static void MapCommonAttributesInto(const nsIHTMLMappedAttributes* aAttributes, 
                                       nsRuleData* aRuleData);
-  struct AttributeImpactEntry {
+  struct AttributeDependenceEntry {
     nsIAtom** attribute;
-    nsChangeHint hint;
   };
 
-  static const AttributeImpactEntry sCommonAttributeMap[];
-  static const AttributeImpactEntry sImageAttributeMap[];
-  static const AttributeImpactEntry sImageBorderAttributeMap[];
-  static const AttributeImpactEntry sImageAlignAttributeMap[];
-  static const AttributeImpactEntry sBackgroundAttributeMap[];
+  static const AttributeDependenceEntry sCommonAttributeMap[];
+  static const AttributeDependenceEntry sImageMarginSizeAttributeMap[];
+  static const AttributeDependenceEntry sImageBorderAttributeMap[];
+  static const AttributeDependenceEntry sImageAlignAttributeMap[];
+  static const AttributeDependenceEntry sDivAlignAttributeMap[];
+  static const AttributeDependenceEntry sBackgroundAttributeMap[];
+  static const AttributeDependenceEntry sScrollingAttributeMap[];
   
   /**
-   * A common method where you can just pass in a list of maps to
-   * check for impact. Most implementations of GetMappedAttributeImpact
-   * should use this function as a default handler.
-   *
-   * @param aAttribute   attribute that we care about
-   * @param aHint the    resulting hint
-   * @param aImpactFlags the types of attributes that we care about - see the
-   *                     NS_*_ATTRIBUTE_IMPACT flags
+   * A common method where you can just pass in a list of maps to check
+   * for attribute dependence. Most implementations of
+   * HasAttributeDependentStyle should use this function as a default
+   * handler.
    */
-
-  static void
-  FindAttributeImpact(const nsIAtom* aAttribute, nsChangeHint& aHint,
-                      const AttributeImpactEntry* const aMaps[],
-                      PRUint32 aMapCount);
+  static PRBool
+  FindAttributeDependence(const nsIAtom* aAttribute,
+                          const AttributeDependenceEntry* const aMaps[],
+                          PRUint32 aMapCount);
   /**
    * Helper to map the align attribute into a style struct.
    *
@@ -526,8 +523,8 @@ public:
    * @param aData the returned rule data [INOUT]
    * @see GetAttributeMappingFunction
    */
-  static void MapAlignAttributeInto(const nsIHTMLMappedAttributes* aAttributes,
-                                    nsRuleData* aData);
+  static void MapImageAlignAttributeInto(const nsIHTMLMappedAttributes* aAttributes,
+                                         nsRuleData* aData);
 
   /**
    * Helper to map the align attribute into a style struct for things
@@ -565,8 +562,8 @@ public:
    * @param aData the returned rule data [INOUT]
    * @see GetAttributeMappingFunction
    */
-  static void MapImagePositionAttributeInto(const nsIHTMLMappedAttributes* aAttributes,
-                                            nsRuleData* aData);
+  static void MapImageSizeAttributesInto(const nsIHTMLMappedAttributes* aAttributes,
+                                         nsRuleData* aData);
   /**
    * Helper to map the background attributes (currently background and bgcolor)
    * into a style struct.
@@ -577,6 +574,16 @@ public:
    */
   static void MapBackgroundAttributesInto(const nsIHTMLMappedAttributes* aAttributes,
                                           nsRuleData* aData);
+  /**
+   * Helper to map the scrolling attribute on FRAME and IFRAME
+   * into a style struct.
+   *
+   * @param aAttributes the list of attributes to map
+   * @param aData the returned rule data [INOUT]
+   * @see GetAttributeMappingFunction
+   */
+  static void MapScrollingAttributeInto(const nsIHTMLMappedAttributes* aAttributes,
+                                        nsRuleData* aData);
   /**
    * Get the primary frame for a piece of content.
    *
@@ -815,27 +822,23 @@ public:
   NS_IMETHOD Compact() {
     return NS_OK;
   }
-  NS_IMETHOD CanContainChildren(PRBool& aResult) const {
-    aResult = PR_FALSE;
-    return NS_OK;
+  NS_IMETHOD_(PRBool) CanContainChildren() const {
+    return PR_FALSE;
   }
-  NS_IMETHOD ChildCount(PRInt32& aResult) const {
-    aResult = 0;
-    return NS_OK;
+  NS_IMETHOD_(PRUint32) GetChildCount() const {
+    return 0;
   }
-  NS_IMETHOD ChildAt(PRInt32 aIndex, nsIContent** aResult) const {
-    aResult = nsnull;
-    return NS_OK;
+  NS_IMETHOD_(nsIContent *) GetChildAt(PRUint32 aIndex) const {
+    return nsnull;
   }
-  NS_IMETHOD IndexOf(nsIContent* aPossibleChild, PRInt32& aResult) const {
-    aResult = -1;
-    return NS_OK;
+  NS_IMETHOD_(PRInt32) IndexOf(nsIContent* aPossibleChild) const {
+    return -1;
   }
-  NS_IMETHOD InsertChildAt(nsIContent* aKid, PRInt32 aIndex, PRBool aNotify,
+  NS_IMETHOD InsertChildAt(nsIContent* aKid, PRUint32 aIndex, PRBool aNotify,
                            PRBool aDeepSetDocument) {
     return NS_OK;
   }
-  NS_IMETHOD ReplaceChildAt(nsIContent* aKid, PRInt32 aIndex, PRBool aNotify,
+  NS_IMETHOD ReplaceChildAt(nsIContent* aKid, PRUint32 aIndex, PRBool aNotify,
                             PRBool aDeepSetDocument) {
     return NS_OK;
   }
@@ -843,7 +846,7 @@ public:
                            PRBool aDeepSetDocument) {
     return NS_OK;
   }
-  NS_IMETHOD RemoveChildAt(PRInt32 aIndex, PRBool aNotify) {
+  NS_IMETHOD RemoveChildAt(PRUint32 aIndex, PRBool aNotify) {
     return NS_OK;
   }
 };
@@ -890,17 +893,17 @@ public:
 
   // Remainder of nsIHTMLContent (and nsIContent)
   NS_IMETHOD Compact();
-  NS_IMETHOD CanContainChildren(PRBool& aResult) const;
-  NS_IMETHOD ChildCount(PRInt32& aResult) const;
-  NS_IMETHOD ChildAt(PRInt32 aIndex, nsIContent** aResult) const;
-  NS_IMETHOD IndexOf(nsIContent* aPossibleChild, PRInt32& aResult) const;
-  NS_IMETHOD InsertChildAt(nsIContent* aKid, PRInt32 aIndex, PRBool aNotify,
+  NS_IMETHOD_(PRBool) CanContainChildren() const;
+  NS_IMETHOD_(PRUint32) GetChildCount() const;
+  NS_IMETHOD_(nsIContent *) GetChildAt(PRUint32 aIndex) const;
+  NS_IMETHOD_(PRInt32) IndexOf(nsIContent* aPossibleChild) const;
+  NS_IMETHOD InsertChildAt(nsIContent* aKid, PRUint32 aIndex, PRBool aNotify,
                            PRBool aDeepSetDocument);
-  NS_IMETHOD ReplaceChildAt(nsIContent* aKid, PRInt32 aIndex, PRBool aNotify,
+  NS_IMETHOD ReplaceChildAt(nsIContent* aKid, PRUint32 aIndex, PRBool aNotify,
                             PRBool aDeepSetDocument);
   NS_IMETHOD AppendChildTo(nsIContent* aKid, PRBool aNotify,
                            PRBool aDeepSetDocument);
-  NS_IMETHOD RemoveChildAt(PRInt32 aIndex, PRBool aNotify);
+  NS_IMETHOD RemoveChildAt(PRUint32 aIndex, PRBool aNotify);
 
   /** The list of children */
   nsSmallVoidArray mChildren;
@@ -1256,12 +1259,6 @@ NS_NewHTMLAreaElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo);
 nsresult
 NS_NewHTMLBRElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo);
 
-inline nsresult
-NS_NewHTMLBaseElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo)
-{
-  return NS_NewHTMLSharedLeafElement(aResult, aNodeInfo);
-}
-
 nsresult
 NS_NewHTMLBaseFontElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo);
 
@@ -1283,12 +1280,6 @@ NS_NewHTMLDirectoryElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo);
 nsresult
 NS_NewHTMLDivElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo);
 
-inline nsresult
-NS_NewHTMLEmbedElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo)
-{
-  return NS_NewHTMLSharedLeafElement(aResult, aNodeInfo);
-}
-
 nsresult
 NS_NewHTMLFieldSetElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo);
 
@@ -1329,12 +1320,6 @@ NS_NewHTMLInputElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo,
 nsresult
 NS_NewHTMLInsElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo);
 
-inline nsresult
-NS_NewHTMLIsIndexElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo)
-{
-  return NS_NewHTMLSharedLeafElement(aResult, aNodeInfo);
-}
-
 nsresult
 NS_NewHTMLLIElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo);
 
@@ -1371,12 +1356,6 @@ NS_NewHTMLOptionElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo);
 nsresult
 NS_NewHTMLParagraphElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo);
 
-inline nsresult
-NS_NewHTMLParamElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo)
-{
-  return NS_NewHTMLSharedLeafElement(aResult, aNodeInfo);
-}
-
 nsresult
 NS_NewHTMLPreElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo);
 
@@ -1390,12 +1369,6 @@ nsresult
 NS_NewHTMLSelectElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo,
                         PRBool aFromParser);
 
-inline nsresult
-NS_NewHTMLSpacerElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo)
-{
-  return NS_NewHTMLSharedLeafElement(aResult, aNodeInfo);
-}
-
 nsresult
 NS_NewHTMLSpanElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo);
 
@@ -1411,13 +1384,6 @@ NS_NewHTMLTableCellElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo);
 nsresult
 NS_NewHTMLTableColElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo);
 
-inline nsresult
-NS_NewHTMLTableColGroupElement(nsIHTMLContent** aResult,
-                               nsINodeInfo *aNodeInfo)
-{
-  return NS_NewHTMLTableColElement(aResult, aNodeInfo);
-}
-
 nsresult
 NS_NewHTMLTableElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo);
 
@@ -1448,11 +1414,4 @@ NS_NewHTMLUListElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo);
 nsresult
 NS_NewHTMLUnknownElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo);
 
-inline nsresult
-NS_NewHTMLWBRElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo)
-{
-  return NS_NewHTMLSharedLeafElement(aResult, aNodeInfo);
-}
-
-
 #endif /* nsGenericHTMLElement_h___ */
