@@ -121,8 +121,7 @@ public:
                          PRBool aCompileEventHandlers);
   NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
   NS_IMETHOD WalkContentStyleRules(nsRuleWalker* aRuleWalker);
-  NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
-                                      nsChangeHint& aHint) const;
+  NS_IMETHOD_(PRBool) HasAttributeDependentStyle(const nsIAtom* aAttribute) const;
 
 protected:
   BodyRule* mContentStyleRule;
@@ -156,9 +155,7 @@ BodyRule::MapRuleInfoInto(nsRuleData* aData)
   if (!aData || (aData->mSID != eStyleStruct_Margin) || !aData->mMarginData || !mPart)
     return NS_OK; // We only care about margins.
 
-  nsHTMLValue   value;
-  PRInt32       attrCount;
-  mPart->GetAttrCount(attrCount);
+  nsHTMLValue value;
   
   PRInt32 bodyMarginWidth  = -1;
   PRInt32 bodyMarginHeight = -1;
@@ -173,7 +170,7 @@ BodyRule::MapRuleInfoInto(nsRuleData* aData)
   aData->mPresContext->GetCompatibilityMode(&mode);
 
 
-  if (attrCount > 0) {
+  if (mPart->GetAttrCount() > 0) {
     // if marginwidth/marginheight are set, reflect them as 'margin'
     mPart->GetHTMLAttribute(nsHTMLAtoms::marginwidth, value);
     if (eHTMLUnit_Pixel == value.GetUnit()) {
@@ -414,7 +411,7 @@ nsHTMLBodyElement::Set##func_(const nsAString& aColor)              \
 }
 
 NS_IMPL_HTMLBODY_COLOR_ATTR(vlink, VLink, VisitedLinkColor)
-NS_IMPL_HTMLBODY_COLOR_ATTR(alink, ALink, LinkColor)
+NS_IMPL_HTMLBODY_COLOR_ATTR(alink, ALink, ActiveLinkColor)
 NS_IMPL_HTMLBODY_COLOR_ATTR(link, Link, LinkColor)
 // XXX Should text check the body frame's style struct for color,
 // like we do for bgColor?
@@ -435,8 +432,7 @@ nsHTMLBodyElement::GetBgColor(nsAString& aBgColor)
     // XXX This should just use nsGenericHTMLElement::GetPrimaryFrame()
     if (mDocument) {
       // Make sure the presentation is up-to-date
-      rv = mDocument->FlushPendingNotifications();
-      NS_ENSURE_SUCCESS(rv, rv);
+      mDocument->FlushPendingNotifications();
     }
 
     nsCOMPtr<nsIPresContext> context;
@@ -486,11 +482,8 @@ nsHTMLBodyElement::StringToAttribute(nsIAtom* aAttribute,
       (aAttribute == nsHTMLAtoms::link) ||
       (aAttribute == nsHTMLAtoms::alink) ||
       (aAttribute == nsHTMLAtoms::vlink)) {
-    nsCOMPtr<nsIDocument> doc(mDocument);
-    if (!doc) {
-      mNodeInfo->GetDocument(getter_AddRefs(doc));
-    }
-    if (aResult.ParseColor(aValue, doc)) {
+    if (aResult.ParseColor(aValue,
+                           nsGenericHTMLContainerElement::GetOwnerDocument())) {
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
   }
@@ -512,7 +505,7 @@ NS_IMETHODIMP
 nsHTMLBodyElement::SetDocument(nsIDocument* aDocument, PRBool aDeep,
                                PRBool aCompileEventHandlers)
 {
-  if (nsnull != mContentStyleRule) {
+  if (aDocument != mDocument && mContentStyleRule) {
     mContentStyleRule->mPart = nsnull;
     mContentStyleRule->mSheet = nsnull;
 
@@ -526,10 +519,7 @@ nsHTMLBodyElement::SetDocument(nsIDocument* aDocument, PRBool aDeep,
 static 
 void MapAttributesIntoRule(const nsIHTMLMappedAttributes* aAttributes, nsRuleData* aData)
 {
-  if (!aAttributes || !aData)
-    return;
-  
-  if (aData->mDisplayData && aData->mSID == eStyleStruct_Display) {
+  if (aData->mSID == eStyleStruct_Display) {
     // When display if first asked for, go ahead and get our colors set up.
     nsHTMLValue value;
     
@@ -568,7 +558,7 @@ void MapAttributesIntoRule(const nsIHTMLMappedAttributes* aAttributes, nsRuleDat
     }
   }
 
-  if (aData->mColorData && aData->mSID == eStyleStruct_Color) {
+  if (aData->mSID == eStyleStruct_Color) {
     if (aData->mColorData->mColor.GetUnit() == eCSSUnit_Null) {
       // color: color
       nsHTMLValue value;
@@ -627,27 +617,28 @@ nsHTMLBodyElement::WalkContentStyleRules(nsRuleWalker* aRuleWalker)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsHTMLBodyElement::GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
-                                            nsChangeHint& aHint) const
+NS_IMETHODIMP_(PRBool)
+nsHTMLBodyElement::HasAttributeDependentStyle(const nsIAtom* aAttribute) const
 {
-  static const AttributeImpactEntry attributes[] = {
-    { &nsHTMLAtoms::link,  NS_STYLE_HINT_VISUAL },
-    { &nsHTMLAtoms::vlink, NS_STYLE_HINT_VISUAL },
-    { &nsHTMLAtoms::alink, NS_STYLE_HINT_VISUAL },
-    { &nsHTMLAtoms::text,  NS_STYLE_HINT_VISUAL },
-    { &nsHTMLAtoms::marginwidth, NS_STYLE_HINT_REFLOW },
-    { &nsHTMLAtoms::marginheight, NS_STYLE_HINT_REFLOW },
-    { nsnull, NS_STYLE_HINT_NONE },
+  static const AttributeDependenceEntry attributes[] = {
+    { &nsHTMLAtoms::link },
+    { &nsHTMLAtoms::vlink },
+    { &nsHTMLAtoms::alink },
+    { &nsHTMLAtoms::text },
+    // These aren't mapped through attribute mapping, but they are
+    // mapped through a style rule, so it is attribute dependent style.
+    // XXXldb But we don't actually replace the body rule when we have
+    // dynamic changes...
+    { &nsHTMLAtoms::marginwidth },
+    { &nsHTMLAtoms::marginheight },
+    { nsnull },
   };
 
-  static const AttributeImpactEntry* const map[] = {
+  static const AttributeDependenceEntry* const map[] = {
     attributes,
     sCommonAttributeMap,
     sBackgroundAttributeMap,
   };
 
-  FindAttributeImpact(aAttribute, aHint, map, NS_ARRAY_LENGTH(map));
-
-  return NS_OK;
+  return FindAttributeDependence(aAttribute, map, NS_ARRAY_LENGTH(map));
 }
