@@ -56,8 +56,7 @@
 #include "nsIScriptGlobalObject.h"
 #include "nsDOMError.h"
 #include "nsContentUtils.h"
-#include "nsHashtable.h"
-#include "nsDoubleHashtable.h"
+#include "nsInterfaceHashtable.h"
 #include "nsContentList.h"
 #include "nsGUIEvent.h"
 #include "nsSupportsArray.h"
@@ -90,21 +89,6 @@ class nsFormControlList;
 
 // nsHTMLFormElement
 
-//
-// PLDHashTable entry for radio button
-//
-class PLDHashStringRadioEntry : public PLDHashStringEntry
-{
-public:
-  PLDHashStringRadioEntry(const void* key) : PLDHashStringEntry(key), mVal(nsnull) { }
-  ~PLDHashStringRadioEntry() { }
-
-  nsCOMPtr<nsIDOMHTMLInputElement> mVal;
-};
-
-DECL_DHASH_WRAPPER(nsDoubleHashtableStringRadio, PLDHashStringRadioEntry, nsAString&)
-DHASH_WRAPPER(nsDoubleHashtableStringRadio, PLDHashStringRadioEntry, nsAString&)
-
 class nsHTMLFormElement : public nsGenericHTMLContainerElement,
                           public nsSupportsWeakReference,
                           public nsIDOMHTMLFormElement,
@@ -280,7 +264,7 @@ protected:
   /** The list of controls (form.elements as well as stuff not in elements) */
   nsFormControlList *mControls;
   /** The currently selected radio button of each group */
-  nsDoubleHashtableStringRadio mSelectedRadioButtons;
+  nsInterfaceHashtable<nsStringHashKey,nsIDOMHTMLInputElement> mSelectedRadioButtons;
   /** Whether we are currently processing a submit event or not */
   PRPackedBool mGeneratingSubmit;
   /** Whether we are currently processing a reset event or not */
@@ -318,6 +302,8 @@ public:
   nsFormControlList(nsIDOMHTMLFormElement* aForm);
   virtual ~nsFormControlList();
 
+  nsresult Init();
+
   void Clear();
   void SetForm(nsIDOMHTMLFormElement* aForm);
 
@@ -357,7 +343,7 @@ protected:
   // holds on to a list of named form controls the list has weak
   // references to the form control.
 
-  nsSupportsHashtable mNameLookupTable;
+  nsInterfaceHashtable<nsStringHashKey,nsISupports> mNameLookupTable;
 };
 
 
@@ -456,10 +442,20 @@ nsHTMLFormElement::Init(nsINodeInfo *aNodeInfo)
   if (!mControls) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
+
+  rv = mControls->Init();
+  
+  if (NS_FAILED(rv))
+  {
+    delete mControls;
+    mControls = nsnull;
+    return rv;
+  }
+  
   NS_ADDREF(mControls);
 
-  rv = mSelectedRadioButtons.Init(1);
-  NS_ENSURE_SUCCESS(rv, rv);
+  NS_ENSURE_TRUE(mSelectedRadioButtons.Init(4),
+                 NS_ERROR_OUT_OF_MEMORY);
 
   return NS_OK;
 }
@@ -788,10 +784,8 @@ nsHTMLFormElement::DoSubmitOrReset(nsIPresContext* aPresContext,
   NS_ENSURE_ARG_POINTER(aPresContext);
 
   // Make sure the presentation is up-to-date
-  nsCOMPtr<nsIDocument> doc;
-  GetDocument(getter_AddRefs(doc));
-  if (doc) {
-    doc->FlushPendingNotifications();
+  if (mDocument) {
+    mDocument->FlushPendingNotifications();
   }
 
   // JBK Don't get form frames anymore - bug 34297
@@ -1004,9 +998,7 @@ nsHTMLFormElement::NotifySubmitObservers(nsIURI* aActionURL,
     nsCOMPtr<nsISupports> inst;
     *aCancelSubmit = PR_FALSE;
 
-    nsCOMPtr<nsIScriptGlobalObject> globalObject;
-    mDocument->GetScriptGlobalObject(getter_AddRefs(globalObject));
-    nsCOMPtr<nsIDOMWindowInternal> window = do_QueryInterface(globalObject);
+    nsCOMPtr<nsIDOMWindowInternal> window = do_QueryInterface(mDocument->GetScriptGlobalObject());
 
     PRBool loop = PR_TRUE;
     while (NS_SUCCEEDED(theEnum->HasMoreElements(&loop)) && loop) {
@@ -1053,8 +1045,7 @@ nsHTMLFormElement::CompareNodes(nsIDOMNode* a, nsIDOMNode* b, PRInt32* retval)
     if (!parentA || !contentA) {
       return NS_ERROR_UNEXPECTED;
     }
-    rv = parentA->IndexOf(contentA, indexA);
-    NS_ENSURE_SUCCESS(rv, rv);
+    indexA = parentA->IndexOf(contentA);
   }
 
   nsCOMPtr<nsIDOMNode> parentBNode;
@@ -1073,8 +1064,7 @@ nsHTMLFormElement::CompareNodes(nsIDOMNode* a, nsIDOMNode* b, PRInt32* retval)
     if (!parentB || !bContent) {
       return NS_ERROR_UNEXPECTED;
     }
-    rv = parentB->IndexOf(bContent, indexB);
-    NS_ENSURE_SUCCESS(rv, rv);
+    indexB = parentB->IndexOf(bContent);
   }
 
   *retval = ComparePoints(parentANode, indexA, parentBNode, indexB);
@@ -1282,8 +1272,7 @@ nsHTMLFormElement::GetActionURL(nsIURI** aActionURL)
   }
 
   // Get base URL
-  nsCOMPtr<nsIURI> docURL;
-  mDocument->GetDocumentURL(getter_AddRefs(docURL));
+  nsIURI *docURL = mDocument->GetDocumentURL();
   NS_ENSURE_TRUE(docURL, NS_ERROR_UNEXPECTED);
 
   // If an action is not specified and we are inside
@@ -1450,9 +1439,9 @@ NS_IMETHODIMP
 nsHTMLFormElement::SetCurrentRadioButton(const nsAString& aName,
                                          nsIDOMHTMLInputElement* aRadio)
 {
-  PLDHashStringRadioEntry* entry = mSelectedRadioButtons.AddEntry(aName);
-  NS_ENSURE_TRUE(entry, NS_ERROR_OUT_OF_MEMORY);
-  entry->mVal = aRadio;
+  NS_ENSURE_TRUE(mSelectedRadioButtons.Put(aName, aRadio),
+                 NS_ERROR_OUT_OF_MEMORY);
+
   return NS_OK;
 }
 
@@ -1460,11 +1449,8 @@ NS_IMETHODIMP
 nsHTMLFormElement::GetCurrentRadioButton(const nsAString& aName,
                                          nsIDOMHTMLInputElement** aRadio)
 {
-  PLDHashStringRadioEntry* entry = mSelectedRadioButtons.GetEntry(aName);
-  if (entry) {
-    *aRadio = entry->mVal;
-    NS_IF_ADDREF(*aRadio);
-  }
+  mSelectedRadioButtons.Get(aName,aRadio);
+
   return NS_OK;
 }
 
@@ -1566,9 +1552,8 @@ nsHTMLFormElement::RemoveFromRadioGroup(const nsAString& aName,
 // nsFormControlList implementation, this could go away if there were
 // a lightweight collection implementation somewhere
 
-nsFormControlList::nsFormControlList(nsIDOMHTMLFormElement* aForm)
-  : mForm(aForm),
-    mNameLookupTable(NS_FORM_CONTROL_LIST_HASHTABLE_SIZE)
+nsFormControlList::nsFormControlList(nsIDOMHTMLFormElement* aForm) :
+  mForm(aForm)
 {
 }
 
@@ -1578,6 +1563,15 @@ nsFormControlList::~nsFormControlList()
   Clear();
 }
 
+nsresult nsFormControlList::Init()
+{
+  NS_ENSURE_TRUE(
+    mNameLookupTable.Init(NS_FORM_CONTROL_LIST_HASHTABLE_SIZE),
+    NS_ERROR_OUT_OF_MEMORY);
+
+  return NS_OK;
+}
+
 void
 nsFormControlList::SetForm(nsIDOMHTMLFormElement* aForm)
 {
@@ -1607,7 +1601,7 @@ nsFormControlList::Clear()
   }
   mNotInElements.Clear();
 
-  mNameLookupTable.Reset();
+  mNameLookupTable.Clear();
 }
 
 
@@ -1658,9 +1652,7 @@ nsFormControlList::GetNamedObject(const nsAString& aName,
   }
   
   // Get the hash entry
-  nsStringKey key(aName);
-
-  *aResult = mNameLookupTable.Get(&key);
+  mNameLookupTable.Get(aName,aResult);
 
   return NS_OK;
 }
@@ -1674,9 +1666,10 @@ nsFormControlList::NamedItem(const nsAString& aName,
 
   nsresult rv = NS_OK;
 
-  nsStringKey key(aName);
-
-  nsCOMPtr<nsISupports> supports(dont_AddRef(mNameLookupTable.Get(&key)));
+  nsCOMPtr<nsISupports> supports;
+  
+  if (!mNameLookupTable.Get(aName,getter_AddRefs(supports))) // key not found
+     return rv;
 
   if (supports) {
     // We found something, check if it's a node
@@ -1704,9 +1697,7 @@ nsFormControlList::NamedItem(const nsAString& aName,
 {
   NS_ENSURE_ARG_POINTER(aReturn);
 
-  nsStringKey key(aName);
-
-  *aReturn = mNameLookupTable.Get(&key);
+  mNameLookupTable.Get(aName,aReturn);
 
   return NS_OK;
 }
@@ -1719,16 +1710,14 @@ nsFormControlList::AddElementToTable(nsIFormControl* aChild,
     return NS_OK;
   }
 
-  nsStringKey key(aName);
-
   nsCOMPtr<nsISupports> supports;
-  supports = dont_AddRef(mNameLookupTable.Get(&key));
+  mNameLookupTable.Get(aName,getter_AddRefs(supports));
 
   if (!supports) {
     // No entry found, add the form control
     nsCOMPtr<nsISupports> child(do_QueryInterface(aChild));
 
-    mNameLookupTable.Put(&key, child);
+    NS_ENSURE_TRUE( mNameLookupTable.Put(aName, child), NS_ERROR_FAILURE );
   } else {
     // Found something in the hash, check its type
     nsCOMPtr<nsIContent> content(do_QueryInterface(supports));
@@ -1756,7 +1745,8 @@ nsFormControlList::AddElementToTable(nsIFormControl* aChild,
       nsCOMPtr<nsISupports> listSupports = do_QueryInterface(list);
 
       // Replace the element with the list.
-      mNameLookupTable.Put(&key, listSupports);
+      NS_ENSURE_TRUE(mNameLookupTable.Put(aName, listSupports),
+                     NS_ERROR_FAILURE);
     } else {
       // There's already a list in the hash, add the child to the list
       nsCOMPtr<nsIDOMNodeList> nodeList(do_QueryInterface(supports));
@@ -1803,18 +1793,16 @@ nsFormControlList::RemoveElementFromTable(nsIFormControl* aChild,
     return NS_OK;
   }
 
-  nsStringKey key(aName);
+  nsCOMPtr<nsISupports> supports;
 
-  nsCOMPtr<nsISupports> supports(dont_AddRef(mNameLookupTable.Get(&key)));
-
-  if (!supports)
+  if (!mNameLookupTable.Get(aName,getter_AddRefs(supports)))
     return NS_OK;
 
   nsCOMPtr<nsIFormControl> fctrl(do_QueryInterface(supports));
 
   if (fctrl) {
     // Single element in the hash, just remove it...
-    mNameLookupTable.Remove(&key);
+    mNameLookupTable.Remove(aName);
 
     return NS_OK;
   }
@@ -1834,7 +1822,7 @@ nsFormControlList::RemoveElementFromTable(nsIFormControl* aChild,
   if (!length) {
     // If the list is empty we remove if from our hash, this shouldn't
     // happen tho
-    mNameLookupTable.Remove(&key);
+    mNameLookupTable.Remove(aName);
   } else if (length == 1) {
     // Only one element left, replace the list in the hash with the
     // single element.
@@ -1843,7 +1831,7 @@ nsFormControlList::RemoveElementFromTable(nsIFormControl* aChild,
 
     if (node) {
       nsCOMPtr<nsISupports> tmp(do_QueryInterface(node));
-      mNameLookupTable.Put(&key, tmp.get());
+      NS_ENSURE_TRUE(mNameLookupTable.Put(aName, tmp),NS_ERROR_FAILURE);
     }
   }
 
@@ -1851,7 +1839,7 @@ nsFormControlList::RemoveElementFromTable(nsIFormControl* aChild,
 }
 
 // nsFormControlEnumerator
-NS_IMPL_ISUPPORTS1(nsFormControlEnumerator, nsISimpleEnumerator);
+NS_IMPL_ISUPPORTS1(nsFormControlEnumerator, nsISimpleEnumerator)
 
 nsFormControlEnumerator::nsFormControlEnumerator(nsHTMLFormElement* aForm)
   : mForm(aForm), mElementsIndex(0), mNotInElementsIndex(0)
