@@ -88,6 +88,8 @@ public:
   NS_IMETHOD AttributeToString(nsIAtom* aAttribute,
                                const nsHTMLValue& aValue,
                                nsAString& aResult) const;
+  NS_IMETHOD_(PRBool) HasAttributeDependentStyle(const nsIAtom* aAttribute) const;
+  NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
 };
 
 nsresult
@@ -126,8 +128,8 @@ nsHTMLFrameElement::~nsHTMLFrameElement()
 }
 
 
-NS_IMPL_ADDREF_INHERITED(nsHTMLFrameElement, nsGenericElement);
-NS_IMPL_RELEASE_INHERITED(nsHTMLFrameElement, nsGenericElement);
+NS_IMPL_ADDREF_INHERITED(nsHTMLFrameElement, nsGenericElement)
+NS_IMPL_RELEASE_INHERITED(nsHTMLFrameElement, nsGenericElement)
 
 
 // QueryInterface implementation for nsHTMLFrameElement
@@ -189,9 +191,7 @@ nsHTMLFrameElement::GetContentDocument(nsIDOMDocument** aContentDocument)
     return NS_OK;
   }
 
-  nsCOMPtr<nsIDocument> content_document;
-
-  mDocument->GetSubDocumentFor(this, getter_AddRefs(content_document));
+  nsIDocument* content_document = mDocument->GetSubDocumentFor(this);
 
   if (!content_document) {
     return NS_OK;
@@ -217,10 +217,7 @@ nsHTMLFrameElement::GetContentWindow(nsIDOMWindow** aContentWindow)
     return NS_OK;
   }
 
-  nsCOMPtr<nsIScriptGlobalObject> globalObj;
-  doc->GetScriptGlobalObject(getter_AddRefs(globalObj));
-
-  nsCOMPtr<nsIDOMWindow> window (do_QueryInterface(globalObj));
+  nsCOMPtr<nsIDOMWindow> window (do_QueryInterface(doc->GetScriptGlobalObject()));
 
   *aContentWindow = window;
   NS_IF_ADDREF(*aContentWindow);
@@ -234,11 +231,8 @@ nsHTMLFrameElement::StringToAttribute(nsIAtom* aAttribute,
                                       nsHTMLValue& aResult)
 {
   if (aAttribute == nsHTMLAtoms::bordercolor) {
-    nsCOMPtr<nsIDocument> doc(mDocument);
-    if (!doc) {
-      mNodeInfo->GetDocument(getter_AddRefs(doc));
-    }
-    if (aResult.ParseColor(aValue, doc)) {
+    if (aResult.ParseColor(aValue,
+                           nsGenericHTMLLeafElement::GetOwnerDocument())) {
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
   }
@@ -288,6 +282,32 @@ nsHTMLFrameElement::AttributeToString(nsIAtom* aAttribute,
                                                      aResult);
 }
 
+static void
+MapAttributesIntoRule(const nsIHTMLMappedAttributes* aAttributes,
+                      nsRuleData* aData)
+{
+  nsGenericHTMLElement::MapScrollingAttributeInto(aAttributes, aData);
+  nsGenericHTMLElement::MapCommonAttributesInto(aAttributes, aData);
+}
+
+NS_IMETHODIMP_(PRBool)
+nsHTMLFrameElement::HasAttributeDependentStyle(const nsIAtom* aAttribute) const
+{
+  static const AttributeDependenceEntry* const map[] = {
+    sScrollingAttributeMap,
+    sCommonAttributeMap,
+  };
+  
+  return FindAttributeDependence(aAttribute, map, NS_ARRAY_LENGTH(map));
+}
+
+NS_IMETHODIMP
+nsHTMLFrameElement::GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const
+{
+  aMapRuleFunc = &MapAttributesIntoRule;
+  return NS_OK;
+}
+
 
 //*****************************************************************************
 // nsHTMLFrameElement::nsIChromeEventHandler
