@@ -59,7 +59,6 @@
 #include "nsIServiceManager.h"
 #include "nsNetUtil.h"
 #include "nsContentUtils.h"
-#include "nsIWebShell.h"
 #include "nsIFrame.h"
 #include "nsIImageFrame.h"
 #include "nsLayoutAtoms.h"
@@ -123,9 +122,10 @@ public:
   NS_IMETHOD AttributeToString(nsIAtom* aAttribute,
                                const nsHTMLValue& aValue,
                                nsAString& aResult) const;
-  NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute,
-                                      PRInt32 aModType,
-                                      nsChangeHint& aHint) const;
+  NS_IMETHOD GetAttributeChangeHint(const nsIAtom* aAttribute,
+                                    PRInt32 aModType,
+                                    nsChangeHint& aHint) const;
+  NS_IMETHOD_(PRBool) HasAttributeDependentStyle(const nsIAtom* aAttribute) const;
   NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
   NS_IMETHOD HandleDOMEvent(nsIPresContext* aPresContext, nsEvent* aEvent,
                             nsIDOMEvent** aDOMEvent, PRUint32 aFlags,
@@ -137,10 +137,14 @@ public:
                      const nsAString& aValue, PRBool aNotify);
   NS_IMETHOD SetAttr(nsINodeInfo* aNodeInfo, const nsAString& aValue,
                      PRBool aNotify);
-  
+
   // XXXbz What about UnsetAttr?  We don't seem to unload images when
   // that happens...
-  
+
+  NS_IMETHOD SetDocument(nsIDocument* aDocument, PRBool aDeep,
+                         PRBool aCompileEventHandlers);  
+  NS_IMETHOD SetParent(nsIContent* aParent);  
+
 protected:
   void GetImageFrame(nsIImageFrame** aImageFrame);
   nsresult GetXY(PRInt32* aX, PRInt32* aY);
@@ -167,13 +171,12 @@ NS_NewHTMLImageElement(nsIHTMLContent** aInstancePtrResult,
     nsCOMPtr<nsIDocument> doc(do_QueryInterface(dom_doc));
     NS_ENSURE_TRUE(doc, NS_ERROR_UNEXPECTED);
 
-    nsCOMPtr<nsINodeInfoManager> nodeInfoManager;
-    doc->GetNodeInfoManager(*getter_AddRefs(nodeInfoManager));
+    nsINodeInfoManager *nodeInfoManager = doc->GetNodeInfoManager();
     NS_ENSURE_TRUE(nodeInfoManager, NS_ERROR_UNEXPECTED);
 
     rv = nodeInfoManager->GetNodeInfo(nsHTMLAtoms::img, nsnull,
                                       kNameSpaceID_None,
-                                      *getter_AddRefs(nodeInfo));
+                                      getter_AddRefs(nodeInfo));
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
@@ -258,7 +261,9 @@ NS_IMPL_STRING_ATTR(nsHTMLImageElement, Alt, alt)
 NS_IMPL_STRING_ATTR(nsHTMLImageElement, Border, border)
 NS_IMPL_PIXEL_ATTR(nsHTMLImageElement, Hspace, hspace)
 NS_IMPL_BOOL_ATTR(nsHTMLImageElement, IsMap, ismap)
-NS_IMPL_STRING_ATTR(nsHTMLImageElement, LongDesc, longdesc)
+NS_IMPL_URI_ATTR(nsHTMLImageElement, LongDesc, longdesc)
+NS_IMPL_STRING_ATTR(nsHTMLImageElement, Lowsrc, lowsrc)
+NS_IMPL_URI_ATTR_GETTER(nsHTMLImageElement, Src, src)
 NS_IMPL_STRING_ATTR(nsHTMLImageElement, UseMap, usemap)
 NS_IMPL_PIXEL_ATTR(nsHTMLImageElement, Vspace, vspace)
 
@@ -281,7 +286,7 @@ NS_IMETHODIMP
 nsHTMLImageElement::GetComplete(PRBool* aComplete)
 {
   NS_PRECONDITION(aComplete, "Null out param!");
-  *aComplete = PR_FALSE;
+  *aComplete = PR_TRUE;
 
   if (!mCurrentRequest) {
     return NS_OK;
@@ -289,7 +294,9 @@ nsHTMLImageElement::GetComplete(PRBool* aComplete)
 
   PRUint32 status;
   mCurrentRequest->GetImageStatus(&status);
-  *aComplete = ((status & imgIRequest::STATUS_LOAD_COMPLETE) != 0);
+  *aComplete =
+    (status &
+     (imgIRequest::STATUS_LOAD_COMPLETE | imgIRequest::STATUS_ERROR)) != 0;
 
   return NS_OK;
 }
@@ -310,9 +317,7 @@ nsHTMLImageElement::GetXY(PRInt32* aX, PRInt32* aY)
   }
 
   // Get Presentation shell 0
-  nsCOMPtr<nsIPresShell> presShell;
-  mDocument->GetShellAt(0, getter_AddRefs(presShell));
-
+  nsIPresShell *presShell = mDocument->GetShellAt(0);
   if (!presShell) {
     return NS_OK;
   }
@@ -400,8 +405,7 @@ nsHTMLImageElement::GetWidthHeight(PRInt32* aWidth, PRInt32* aHeight)
   if (frame) {
     // XXX we could put an accessor on nsIImageFrame to return its
     // mComputedSize.....
-    nsSize size;
-    frame->GetSize(size);
+    nsSize size = frame->GetSize();
 
     nsMargin margin;
     frame->CalcBorderPadding(margin);
@@ -540,35 +544,39 @@ static void
 MapAttributesIntoRule(const nsIHTMLMappedAttributes* aAttributes,
                       nsRuleData* aData)
 {
-  if (!aData || !aAttributes)
-    return;
-  nsGenericHTMLElement::MapAlignAttributeInto(aAttributes, aData);
+  nsGenericHTMLElement::MapImageAlignAttributeInto(aAttributes, aData);
   nsGenericHTMLElement::MapImageBorderAttributeInto(aAttributes, aData);
   nsGenericHTMLElement::MapImageMarginAttributeInto(aAttributes, aData);
-  nsGenericHTMLElement::MapImagePositionAttributeInto(aAttributes, aData);
+  nsGenericHTMLElement::MapImageSizeAttributesInto(aAttributes, aData);
   nsGenericHTMLElement::MapCommonAttributesInto(aAttributes, aData);
 }
 
-
 NS_IMETHODIMP
-nsHTMLImageElement::GetMappedAttributeImpact(const nsIAtom* aAttribute,
-                                             PRInt32 aModType,
-                                             nsChangeHint& aHint) const
+nsHTMLImageElement::GetAttributeChangeHint(const nsIAtom* aAttribute,
+                                           PRInt32 aModType,
+                                           nsChangeHint& aHint) const
 {
-  if ((aAttribute == nsHTMLAtoms::usemap) ||
-      (aAttribute == nsHTMLAtoms::ismap)  ||
-      (aAttribute == nsHTMLAtoms::align)) {
-    aHint = NS_STYLE_HINT_FRAMECHANGE;
-  }
-  else if (!GetCommonMappedAttributesImpact(aAttribute, aHint)) {
-    if (!GetImageMappedAttributesImpact(aAttribute, aHint)) {
-      if (!GetImageBorderAttributeImpact(aAttribute, aHint)) {
-        aHint = NS_STYLE_HINT_CONTENT;
-      }
-    }
+  nsresult rv =
+    nsGenericHTMLLeafElement::GetAttributeChangeHint(aAttribute,
+                                                     aModType, aHint);
+  if (aAttribute == nsHTMLAtoms::usemap ||
+      aAttribute == nsHTMLAtoms::ismap) {
+    NS_UpdateHint(aHint, NS_STYLE_HINT_FRAMECHANGE);
   }
+  return rv;
+}
 
-  return NS_OK;
+NS_IMETHODIMP_(PRBool)
+nsHTMLImageElement::HasAttributeDependentStyle(const nsIAtom* aAttribute) const
+{
+  static const AttributeDependenceEntry* const map[] = {
+    sCommonAttributeMap,
+    sImageMarginSizeAttributeMap,
+    sImageBorderAttributeMap,
+    sImageAlignAttributeMap
+  };
+
+  return FindAttributeDependence(aAttribute, map, NS_ARRAY_LENGTH(map));
 }
 
 
@@ -608,9 +616,13 @@ NS_IMETHODIMP
 nsHTMLImageElement::SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
                             const nsAString& aValue, PRBool aNotify)
 {
-  // Call ImageURIChanged first so that the image load kicks off
-  // _before_ the reflow triggered by the SetAttr
-  if (aNameSpaceID == kNameSpaceID_None && aName == nsHTMLAtoms::src) {
+  // If we plan to call ImageURIChanged, we want to do it first so that the
+  // image load kicks off _before_ the reflow triggered by the SetAttr.  But if
+  // aNotify is false, we are coming from the parser or some such place; we'll
+  // get our parent set after all the attributes have been set, so we'll do the
+  // image load from SetParent.  Skip the ImageURIChanged call in that case.
+  if (aNotify &&
+      aNameSpaceID == kNameSpaceID_None && aName == nsHTMLAtoms::src) {
     ImageURIChanged(aValue);
   }
     
@@ -625,6 +637,42 @@ nsHTMLImageElement::SetAttr(nsINodeInfo* aNodeInfo, const nsAString& aValue,
   return nsGenericHTMLLeafElement::SetAttr(aNodeInfo, aValue, aNotify);
 }
 
+NS_IMETHODIMP
+nsHTMLImageElement::SetDocument(nsIDocument* aDocument, PRBool aDeep,
+                                PRBool aCompileEventHandlers)
+{
+  PRBool documentChanging = aDocument && (aDocument != mDocument);
+  
+  nsresult rv = nsGenericHTMLLeafElement::SetDocument(aDocument, aDeep,
+                                                      aCompileEventHandlers);
+  if (documentChanging && mParent) {
+    // Our base URI may have changed; claim that our URI changed, and the
+    // nsImageLoadingContent will decide whether a new image load is warranted.
+    nsAutoString uri;
+    nsresult result = GetAttr(kNameSpaceID_None, nsHTMLAtoms::src, uri);
+    if (result == NS_CONTENT_ATTR_HAS_VALUE) {
+      ImageURIChanged(uri);
+    }
+  }
+  return rv;
+}
+
+NS_IMETHODIMP
+nsHTMLImageElement::SetParent(nsIContent* aParent)
+{
+  nsresult rv = nsGenericHTMLLeafElement::SetParent(aParent);
+  if (aParent && mDocument) {
+    // Our base URI may have changed; claim that our URI changed, and the
+    // nsImageLoadingContent will decide whether a new image load is warranted.
+    nsAutoString uri;
+    nsresult result = GetAttr(kNameSpaceID_None, nsHTMLAtoms::src, uri);
+    if (result == NS_CONTENT_ATTR_HAS_VALUE) {
+      ImageURIChanged(uri);
+    }
+  }
+  return rv;
+}
+
 NS_IMETHODIMP
 nsHTMLImageElement::Initialize(JSContext* aContext, JSObject *aObj,
                                PRUint32 argc, jsval *argv)
@@ -658,34 +706,6 @@ nsHTMLImageElement::Initialize(JSContext* aContext, JSObject *aObj,
   return rv;
 }
 
-NS_IMETHODIMP
-nsHTMLImageElement::GetSrc(nsAString& aSrc)
-{
-  // Resolve url to an absolute url
-  nsresult rv = NS_OK;
-  nsAutoString relURLSpec;
-  nsCOMPtr<nsIURI> baseURL;
-
-  // Get base URL.
-  GetBaseURL(*getter_AddRefs(baseURL));
-
-  // Get href= attribute (relative URL).
-  nsGenericHTMLLeafElement::GetAttr(kNameSpaceID_None, nsHTMLAtoms::src,
-                                    relURLSpec);
-  relURLSpec.Trim(" \t\n\r");
-
-  if (baseURL && !relURLSpec.IsEmpty()) {
-    // Get absolute URL.
-    rv = NS_MakeAbsoluteURI(aSrc, relURLSpec, baseURL);
-  }
-  else {
-    // Absolute URL is same as relative URL.
-    aSrc = relURLSpec;
-  }
-
-  return rv;
-}
-
 NS_IMETHODIMP
 nsHTMLImageElement::SetSrc(const nsAString& aSrc)
 {
