@@ -82,6 +82,15 @@ nsTableCellCollection::~nsTableCellCollection()
 {
 }
 
+static PRBool
+IsCell(nsIContent *aContent)
+{
+  nsINodeInfo *ni = aContent->GetNodeInfo();
+
+  return (ni && (ni->Equals(nsHTMLAtoms::td) || ni->Equals(nsHTMLAtoms::th)) &&
+          aContent->IsContentOfType(nsIContent::eHTML));
+}
+
 NS_IMETHODIMP 
 nsTableCellCollection::GetLength(PRUint32* aLength)
 {
@@ -94,22 +103,13 @@ nsTableCellCollection::GetLength(PRUint32* aLength)
   nsresult result = NS_OK;
 
   if (mParent) {
-    nsCOMPtr<nsIContent> child;
+    nsIContent *child;
     PRUint32 childIndex = 0;
 
-    mParent->ChildAt(childIndex, *getter_AddRefs(child));
-
-    while (child) {
-      nsCOMPtr<nsIAtom> childTag;
-      child->GetTag(*getter_AddRefs(childTag));
-
-      if ((nsHTMLAtoms::td == childTag.get()) ||
-          (nsHTMLAtoms::th == childTag.get())) {
+    while ((child = mParent->GetChildAt(childIndex++))) {
+      if (IsCell(child)) {
         (*aLength)++;
       }
-
-      childIndex++;
-      mParent->ChildAt(childIndex, *getter_AddRefs(child));
     }
   }
 
@@ -125,29 +125,20 @@ nsTableCellCollection::Item(PRUint32     aIndex,
   nsresult rv = NS_OK;
 
   if (mParent) {
-    nsCOMPtr<nsIContent> child;
+    nsIContent *child;
     PRUint32 childIndex = 0;
 
-    mParent->ChildAt(childIndex, *getter_AddRefs(child));
-
-    while (child) {
-      nsCOMPtr<nsIAtom> childTag;
-
-      child->GetTag(*getter_AddRefs(childTag));
-
-      if ((nsHTMLAtoms::td == childTag.get()) ||
-          (nsHTMLAtoms::th == childTag.get())) {
+    while ((child = mParent->GetChildAt(childIndex++))) {
+      if (IsCell(child)) {
         if (aIndex == theIndex) {
           CallQueryInterface(child, aReturn);
           NS_ASSERTION(aReturn, "content element must be an nsIDOMNode");
 
           break;
         }
+
         theIndex++;
       }
-
-      childIndex++;
-      mParent->ChildAt(childIndex, *getter_AddRefs(child));
     }
   }
 
@@ -185,8 +176,7 @@ public:
                                const nsHTMLValue& aValue,
                                nsAString& aResult) const;
   NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
-  NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
-                                      nsChangeHint& aHint) const;
+  NS_IMETHOD_(PRBool) HasAttributeDependentStyle(const nsIAtom* aAttribute) const;
 
 protected:
   nsresult GetSection(nsIDOMHTMLTableSectionElement** aSection);
@@ -200,15 +190,14 @@ void DebugList(nsIDOMHTMLTableElement* aTable) {
   nsCOMPtr<nsIHTMLContent> content = do_QueryInterface(aTable);
   if (content) {
     nsCOMPtr<nsIDocument> doc;
-    result = content->GetDocument(*getter_AddRefs(doc));
+    result = content->GetDocument(getter_AddRefs(doc));
     if (doc) {
       nsCOMPtr<nsIContent> root;
       doc->GetRootContent(getter_AddRefs(root));
       if (root) {
         root->List();
       }
-      nsCOMPtr<nsIPresShell> shell;
-      doc->GetShellAt(0, getter_AddRefs(shell));
+      nsIPresShell *shell = doc->GetShellAt(0);
       if (shell) {
         nsIFrame* rootFrame;
         shell->GetRootFrame(rootFrame);
@@ -327,11 +316,15 @@ nsHTMLTableRowElement::GetTable(nsIDOMHTMLTableElement** aTable)
 
   nsCOMPtr<nsIDOMNode> sectionNode;
   nsresult rv = GetParentNode(getter_AddRefs(sectionNode));
-  NS_ENSURE_TRUE(sectionNode, rv);
+  if (!sectionNode) {
+    return rv;
+  }
 
   nsCOMPtr<nsIDOMNode> tableNode;
   rv = sectionNode->GetParentNode(getter_AddRefs(tableNode));
-  NS_ENSURE_TRUE(tableNode, rv);
+  if (!tableNode) {
+    return rv;
+  }
   
   return CallQueryInterface(tableNode, aTable);
 }
@@ -439,7 +432,7 @@ nsHTMLTableRowElement::InsertCell(PRInt32 aIndex, nsIDOMHTMLElement** aValue)
 
   // create the cell
   nsCOMPtr<nsINodeInfo> nodeInfo;
-  mNodeInfo->NameChanged(nsHTMLAtoms::td, *getter_AddRefs(nodeInfo));
+  mNodeInfo->NameChanged(nsHTMLAtoms::td, getter_AddRefs(nodeInfo));
 
   nsCOMPtr<nsIHTMLContent> cellContent;
   nsresult rv = NS_NewHTMLTableCellElement(getter_AddRefs(cellContent),
@@ -552,7 +545,8 @@ nsHTMLTableRowElement::StringToAttribute(nsIAtom* aAttribute,
     }
   }
   else if (aAttribute == nsHTMLAtoms::bgcolor) {
-    if (aResult.ParseColor(aValue, mDocument)) {
+    if (aResult.ParseColor(aValue,
+                           nsGenericHTMLContainerElement::GetOwnerDocument())) {
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
   }
@@ -594,10 +588,7 @@ nsHTMLTableRowElement::AttributeToString(nsIAtom* aAttribute,
 static 
 void MapAttributesIntoRule(const nsIHTMLMappedAttributes* aAttributes, nsRuleData* aData)
 {
-  if (!aAttributes || !aData)
-    return;
-
-  if (aData->mPositionData) {
+  if (aData->mSID == eStyleStruct_Position) {
     // height: value
     nsHTMLValue value;
     if (aData->mPositionData->mHeight.GetUnit() == eCSSUnit_Null) {
@@ -608,24 +599,22 @@ void MapAttributesIntoRule(const nsIHTMLMappedAttributes* aAttributes, nsRuleDat
         aData->mPositionData->mHeight.SetPercentValue(value.GetPercentValue());
     }
   }
-  else if (aData->mTextData) {
-    if (aData->mSID == eStyleStruct_Text) {
-      if (aData->mTextData->mTextAlign.GetUnit() == eCSSUnit_Null) {
-        // align: enum
-        nsHTMLValue value;
-        aAttributes->GetAttribute(nsHTMLAtoms::align, value);
-        if (value.GetUnit() == eHTMLUnit_Enumerated)
-          aData->mTextData->mTextAlign.SetIntValue(value.GetIntValue(), eCSSUnit_Enumerated);
-      }
+  else if (aData->mSID == eStyleStruct_Text) {
+    if (aData->mTextData->mTextAlign.GetUnit() == eCSSUnit_Null) {
+      // align: enum
+      nsHTMLValue value;
+      aAttributes->GetAttribute(nsHTMLAtoms::align, value);
+      if (value.GetUnit() == eHTMLUnit_Enumerated)
+        aData->mTextData->mTextAlign.SetIntValue(value.GetIntValue(), eCSSUnit_Enumerated);
     }
-    else {
-      if (aData->mTextData->mVerticalAlign.GetUnit() == eCSSUnit_Null) {
-        // valign: enum
-        nsHTMLValue value;
-        aAttributes->GetAttribute(nsHTMLAtoms::valign, value);
-        if (value.GetUnit() == eHTMLUnit_Enumerated) 
-          aData->mTextData->mVerticalAlign.SetIntValue(value.GetIntValue(), eCSSUnit_Enumerated);
-      }
+  }
+  else if (aData->mSID == eStyleStruct_TextReset) {
+    if (aData->mTextData->mVerticalAlign.GetUnit() == eCSSUnit_Null) {
+      // valign: enum
+      nsHTMLValue value;
+      aAttributes->GetAttribute(nsHTMLAtoms::valign, value);
+      if (value.GetUnit() == eHTMLUnit_Enumerated) 
+        aData->mTextData->mVerticalAlign.SetIntValue(value.GetIntValue(), eCSSUnit_Enumerated);
     }
   }
 
@@ -633,26 +622,23 @@ void MapAttributesIntoRule(const nsIHTMLMappedAttributes* aAttributes, nsRuleDat
   nsGenericHTMLElement::MapCommonAttributesInto(aAttributes, aData);
 }
 
-NS_IMETHODIMP
-nsHTMLTableRowElement::GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
-                                                nsChangeHint& aHint) const
+NS_IMETHODIMP_(PRBool)
+nsHTMLTableRowElement::HasAttributeDependentStyle(const nsIAtom* aAttribute) const
 {
-  static const AttributeImpactEntry attributes[] = {
-    { &nsHTMLAtoms::align, NS_STYLE_HINT_REFLOW },
-    { &nsHTMLAtoms::valign, NS_STYLE_HINT_REFLOW }, 
-    { &nsHTMLAtoms::height, NS_STYLE_HINT_REFLOW },
-    { nsnull, NS_STYLE_HINT_NONE }
+  static const AttributeDependenceEntry attributes[] = {
+    { &nsHTMLAtoms::align },
+    { &nsHTMLAtoms::valign }, 
+    { &nsHTMLAtoms::height },
+    { nsnull }
   };
 
-  static const AttributeImpactEntry* const map[] = {
+  static const AttributeDependenceEntry* const map[] = {
     attributes,
     sCommonAttributeMap,
     sBackgroundAttributeMap,
   };
 
-  FindAttributeImpact(aAttribute, aHint, map, NS_ARRAY_LENGTH(map));
-  
-  return NS_OK;
+  return FindAttributeDependence(aAttribute, map, NS_ARRAY_LENGTH(map));
 }
 
 NS_IMETHODIMP
