@@ -68,10 +68,6 @@ NS_INTERFACE_MAP_BEGIN(nsDOMCSSDeclaration)
   NS_INTERFACE_MAP_ENTRY_CONTENT_CLASSINFO(CSSStyleDeclaration)
 NS_INTERFACE_MAP_END
 
-NS_IMPL_ADDREF(nsDOMCSSDeclaration)
-NS_IMPL_RELEASE(nsDOMCSSDeclaration)
-
-
 NS_IMETHODIMP
 nsDOMCSSDeclaration::GetCssText(nsAString& aCssText)
 {
