@@ -75,7 +75,9 @@ nsInspectorCSSUtils::GetRuleNodeParent(nsRuleNode *aNode, nsRuleNode **aParent)
 NS_IMETHODIMP
 nsInspectorCSSUtils::GetRuleNodeRule(nsRuleNode *aNode, nsIStyleRule **aRule)
 {
-    return aNode->GetRule(aRule);
+    *aRule = aNode->GetRule();
+    NS_IF_ADDREF(*aRule);
+    return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -88,31 +90,30 @@ nsInspectorCSSUtils::IsRuleNodeRoot(nsRuleNode *aNode, PRBool *aIsRoot)
 NS_IMETHODIMP
 nsInspectorCSSUtils::AdjustRectForMargins(nsIFrame* aFrame, nsRect& aRect)
 {
-  const nsStyleMargin* margins;
-  ::GetStyleData(aFrame, &margins);
-  
-  // adjust coordinates for margins
-  nsStyleCoord coord;
-  if (margins->mMargin.GetTopUnit() == eStyleUnit_Coord) {
-    margins->mMargin.GetTop(coord);
-    aRect.y -= coord.GetCoordValue();
-    aRect.height += coord.GetCoordValue();
-  }
-  if (margins->mMargin.GetLeftUnit() == eStyleUnit_Coord) {
-    margins->mMargin.GetLeft(coord);
-    aRect.x -= coord.GetCoordValue();
-    aRect.width += coord.GetCoordValue();
-  }
-  if (margins->mMargin.GetRightUnit() == eStyleUnit_Coord) {
-    margins->mMargin.GetRight(coord);
-    aRect.width += coord.GetCoordValue();
-  }
-  if (margins->mMargin.GetBottomUnit() == eStyleUnit_Coord) {
-    margins->mMargin.GetBottom(coord);
-    aRect.height += coord.GetCoordValue();
-  }
+    const nsStyleMargin* margins = aFrame->GetStyleMargin();
 
-  return NS_OK;
+    // adjust coordinates for margins
+    nsStyleCoord coord;
+    if (margins->mMargin.GetTopUnit() == eStyleUnit_Coord) {
+        margins->mMargin.GetTop(coord);
+        aRect.y -= coord.GetCoordValue();
+        aRect.height += coord.GetCoordValue();
+    }
+    if (margins->mMargin.GetLeftUnit() == eStyleUnit_Coord) {
+        margins->mMargin.GetLeft(coord);
+        aRect.x -= coord.GetCoordValue();
+        aRect.width += coord.GetCoordValue();
+    }
+    if (margins->mMargin.GetRightUnit() == eStyleUnit_Coord) {
+        margins->mMargin.GetRight(coord);
+        aRect.width += coord.GetCoordValue();
+    }
+    if (margins->mMargin.GetBottomUnit() == eStyleUnit_Coord) {
+        margins->mMargin.GetBottom(coord);
+        aRect.height += coord.GetCoordValue();
+    }
+
+    return NS_OK;
 }
 
 nsStyleContext*
@@ -151,8 +152,7 @@ nsInspectorCSSUtils::GetStyleContextForContent(nsIContent* aContent,
 
     // No frame has been created, so resolve the style ourselves
     nsRefPtr<nsStyleContext> parentContext;
-    nsCOMPtr<nsIContent> parent;
-    aContent->GetParent(*getter_AddRefs(parent));
+    nsCOMPtr<nsIContent> parent = aContent->GetParent();
     if (parent)
         parentContext = GetStyleContextForContent(parent, aPresShell);
 
@@ -173,15 +173,14 @@ nsInspectorCSSUtils::GetRuleNodeForContent(nsIContent* aContent,
 {
     *aRuleNode = nsnull;
 
-    nsCOMPtr<nsIDocument> doc;
-    aContent->GetDocument(*getter_AddRefs(doc));
+    nsIDocument* doc = aContent->GetDocument();
     NS_ENSURE_TRUE(doc, NS_ERROR_UNEXPECTED);
 
-    nsCOMPtr<nsIPresShell> presShell;
-    doc->GetShellAt(0, getter_AddRefs(presShell));
+    nsIPresShell *presShell = doc->GetShellAt(0);
     NS_ENSURE_TRUE(presShell, NS_ERROR_UNEXPECTED);
 
-    nsRefPtr<nsStyleContext> sContext = GetStyleContextForContent(aContent, presShell);
-    sContext->GetRuleNode(aRuleNode);
+    nsRefPtr<nsStyleContext> sContext =
+        GetStyleContextForContent(aContent, presShell);
+    *aRuleNode = sContext->GetRuleNode();
     return NS_OK;
 }
