@@ -427,7 +427,7 @@ nsSVGAttributes::Create(nsIContent* aContent, nsSVGAttributes** aResult)
 
 //----------------------------------------------------------------------
 // nsISupports interface
-NS_IMPL_ISUPPORTS1(nsSVGAttributes,nsIDOMNamedNodeMap);
+NS_IMPL_ISUPPORTS1(nsSVGAttributes,nsIDOMNamedNodeMap)
 
 //----------------------------------------------------------------------
 // Implementation Helpers
@@ -584,7 +584,7 @@ nsSVGAttributes::SetAttr(nsINodeInfo* aNodeInfo,
 
   nsCOMPtr<nsIDocument> document;
   if (mContent)
-    mContent->GetDocument(getter_AddRefs(document));
+    document = mContent->GetDocument();
   
   nsSVGAttribute* attr = nsnull;
   PRInt32 index;
@@ -607,8 +607,8 @@ nsSVGAttributes::SetAttr(nsINodeInfo* aNodeInfo,
   nsCOMPtr<nsIAtom> name = aNodeInfo->GetNameAtom();
 
   // Send the notification before making any updates
+  mozAutoDocUpdate updateBatch(document, UPDATE_CONTENT_MODEL, aNotify);
   if (aNotify && document) {
-    document->BeginUpdate();
     document->AttributeWillChange(mContent, nameSpaceID, name);
   }
 
@@ -636,8 +636,7 @@ nsSVGAttributes::SetAttr(nsINodeInfo* aNodeInfo,
   }
 
   if (document && NS_SUCCEEDED(rv)) {
-    nsCOMPtr<nsIBindingManager> bindingManager;
-    document->GetBindingManager(getter_AddRefs(bindingManager));
+    nsIBindingManager *bindingManager = document->GetBindingManager();
     nsCOMPtr<nsIXBLBinding> binding;
     bindingManager->GetBinding(mContent, getter_AddRefs(binding));
     if (binding)
@@ -670,8 +669,7 @@ nsSVGAttributes::SetAttr(nsINodeInfo* aNodeInfo,
       PRInt32 modHint = modification ? PRInt32(nsIDOMMutationEvent::MODIFICATION)
                                      : PRInt32(nsIDOMMutationEvent::ADDITION);
       document->AttributeChanged(mContent, nameSpaceID, name,
-                                 modHint, NS_STYLE_HINT_UNKNOWN);
-      document->EndUpdate();
+                                 modHint);
     }
   }
 
@@ -687,58 +685,62 @@ nsSVGAttributes::UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
     return NS_ERROR_NULL_POINTER;
   }
   
-  nsresult rv = NS_OK;
-
   nsCOMPtr<nsIDocument> document;
   if (mContent)
-    mContent->GetDocument(getter_AddRefs(document));
+    document = mContent->GetDocument();
   
   PRInt32 count = Count();
   PRInt32 index;
   PRBool  found = PR_FALSE;
+  nsSVGAttribute* attr = nsnull;
   for (index = 0; index < count; index++) {
-    nsSVGAttribute* attr = ElementAt(index);
+    attr = ElementAt(index);
     if ((aNameSpaceID == kNameSpaceID_Unknown ||
          attr->GetNodeInfo()->NamespaceEquals(aNameSpaceID)) &&
         attr->GetNodeInfo()->Equals(aName) &&
         !attr->IsRequired() &&
         !attr->IsFixed()) {
-      if (aNotify && document) {
-        document->BeginUpdate();
-      }
-      
-      if (mContent && nsGenericElement::HasMutationListeners(mContent, NS_EVENT_BITS_MUTATION_ATTRMODIFIED)) {
-        nsCOMPtr<nsIDOMEventTarget> node(do_QueryInterface(mContent));
-        nsMutationEvent mutation;
-        mutation.eventStructType = NS_MUTATION_EVENT;
-        mutation.message = NS_MUTATION_ATTRMODIFIED;
-        mutation.mTarget = node;
-        
-        CallQueryInterface(attr,
-                           NS_STATIC_CAST(nsIDOMNode**,
-                                          getter_AddRefs(mutation.mRelatedNode)));
-        mutation.mAttrName = aName;
-        nsAutoString str;
-        attr->GetValue()->GetValueString(str);
-        if (!str.IsEmpty())
-          mutation.mPrevAttrValue = do_GetAtom(str);
-        mutation.mAttrChange = nsIDOMMutationEvent::REMOVAL;
-        
-        nsEventStatus status = nsEventStatus_eIgnore;
-        nsCOMPtr<nsIDOMEvent> domEvent;
-        mContent->HandleDOMEvent(nsnull, &mutation, getter_AddRefs(domEvent),
-                             NS_EVENT_FLAG_INIT, &status);
-      }
-      
-      RemoveElementAt(index);
       found = PR_TRUE;
       break;
     }
   }
+
+  if (!found) {
+    return NS_OK;
+  }
+
+  mozAutoDocUpdate updateBatch(document, UPDATE_CONTENT_MODEL, aNotify);
+  if (document && aNotify) {
+    document->AttributeWillChange(mContent, aNameSpaceID, aName);
+  }
   
-  if (NS_SUCCEEDED(rv) && found && document) {
-    nsCOMPtr<nsIBindingManager> bindingManager;
-    document->GetBindingManager(getter_AddRefs(bindingManager));
+  if (mContent && nsGenericElement::HasMutationListeners(mContent, NS_EVENT_BITS_MUTATION_ATTRMODIFIED)) {
+    nsCOMPtr<nsIDOMEventTarget> node(do_QueryInterface(mContent));
+    nsMutationEvent mutation;
+    mutation.eventStructType = NS_MUTATION_EVENT;
+    mutation.message = NS_MUTATION_ATTRMODIFIED;
+    mutation.mTarget = node;
+        
+    CallQueryInterface(attr,
+                       NS_STATIC_CAST(nsIDOMNode**,
+                                      getter_AddRefs(mutation.mRelatedNode)));
+    mutation.mAttrName = aName;
+    nsAutoString str;
+    attr->GetValue()->GetValueString(str);
+    if (!str.IsEmpty())
+      mutation.mPrevAttrValue = do_GetAtom(str);
+    mutation.mAttrChange = nsIDOMMutationEvent::REMOVAL;
+        
+    nsEventStatus status = nsEventStatus_eIgnore;
+    nsCOMPtr<nsIDOMEvent> domEvent;
+    mContent->HandleDOMEvent(nsnull, &mutation, getter_AddRefs(domEvent),
+                             NS_EVENT_FLAG_INIT, &status);
+  }
+      
+  RemoveElementAt(index);
+  
+  if (document) {
+    nsIBindingManager *bindingManager = document->GetBindingManager();
     nsCOMPtr<nsIXBLBinding> binding;
     bindingManager->GetBinding(mContent, getter_AddRefs(binding));
     if (binding)
@@ -746,13 +748,11 @@ nsSVGAttributes::UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
     
     if (aNotify) {
       document->AttributeChanged(mContent, aNameSpaceID, aName,
-                                 nsIDOMMutationEvent::REMOVAL,
-                                 NS_STYLE_HINT_UNKNOWN);
-      document->EndUpdate();
+                                 nsIDOMMutationEvent::REMOVAL);
     }
   }
   
-  return rv;
+  return NS_OK;
 }
 
 NS_IMETHODIMP_(PRBool)
@@ -791,11 +791,8 @@ nsSVGAttributes::NormalizeAttrString(const nsAString& aStr,
   NS_ASSERTION(mContent,"no owner content");
   if (!mContent) return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsINodeInfo> contentNodeInfo;
-  mContent->GetNodeInfo(getter_AddRefs(contentNodeInfo));
-  
   nsCOMPtr<nsINodeInfoManager> nimgr;
-  contentNodeInfo->GetNodeInfoManager(getter_AddRefs(nimgr));
+  mContent->GetNodeInfo()->GetNodeInfoManager(getter_AddRefs(nimgr));
   NS_ENSURE_TRUE(nimgr, NS_ERROR_FAILURE);
   
   return nimgr->GetNodeInfo(aStr, nsnull, kNameSpaceID_None, aNodeInfo);
@@ -834,11 +831,8 @@ nsSVGAttributes::AddMappedSVGValue(nsIAtom* name, nsISupports* value)
   NS_ASSERTION(mContent,"no owner content");
   if (!mContent) return NS_ERROR_FAILURE;
   
-  nsCOMPtr<nsINodeInfo> contentNodeInfo;
-  mContent->GetNodeInfo(getter_AddRefs(contentNodeInfo));
-  
   nsCOMPtr<nsINodeInfoManager> nimgr;
-  contentNodeInfo->GetNodeInfoManager(getter_AddRefs(nimgr));
+  mContent->GetNodeInfo()->GetNodeInfoManager(getter_AddRefs(nimgr));
   NS_ENSURE_TRUE(nimgr, NS_ERROR_FAILURE);
 
   nsCOMPtr<nsINodeInfo> ni;
