@@ -127,48 +127,39 @@ nsSVGElement::Init()
 //----------------------------------------------------------------------
 // nsIContent methods
 
-NS_IMETHODIMP
-nsSVGElement::CanContainChildren(PRBool& aResult) const
+NS_IMETHODIMP_(PRBool)
+nsSVGElement::CanContainChildren() const
 {
-  aResult = PR_TRUE;
-  return NS_OK;
+  return PR_TRUE;
+}
+
+NS_IMETHODIMP_(PRUint32)
+nsSVGElement::GetChildCount() const
+{
+  return mChildren.Count();
+}
+
+NS_IMETHODIMP_(nsIContent *)
+nsSVGElement::GetChildAt(PRUint32 aIndex) const
+{
+  return (nsIContent *)mChildren.SafeElementAt(aIndex);
+}
+
+NS_IMETHODIMP_(PRInt32)
+nsSVGElement::IndexOf(nsIContent* aPossibleChild) const
+{
+  return mChildren.IndexOf(aPossibleChild);
 }
 
 NS_IMETHODIMP
-nsSVGElement::ChildCount(PRInt32& aResult) const
-{
-  aResult = mChildren.Count();
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsSVGElement::ChildAt(PRInt32 aIndex, nsIContent** aResult) const
-{
-  nsIContent *child = (nsIContent *)mChildren.SafeElementAt(aIndex);
-
-  *aResult = child;
-  NS_IF_ADDREF(*aResult);
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsSVGElement::IndexOf(nsIContent* aPossibleChild, PRInt32& aResult) const
-{
-  NS_PRECONDITION(nsnull != aPossibleChild, "null ptr");
-  aResult = mChildren.IndexOf(aPossibleChild);
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsSVGElement::InsertChildAt(nsIContent* aKid, PRInt32 aIndex,
+nsSVGElement::InsertChildAt(nsIContent* aKid, PRUint32 aIndex,
                             PRBool aNotify,
                             PRBool aDeepSetDocument)
 {
   NS_PRECONDITION(nsnull != aKid, "null ptr");
   nsIDocument* doc = mDocument;
-  if (aNotify && (nsnull != doc)) {
-    doc->BeginUpdate();
-  }
+  mozAutoDocUpdate updateBatch(doc, UPDATE_CONTENT_MODEL, aNotify);
+  
   PRBool rv = mChildren.InsertElementAt(aKid, aIndex);/* XXX fix up void array api to use nsresult's*/
   if (rv) {
     NS_ADDREF(aKid);
@@ -196,22 +187,19 @@ nsSVGElement::InsertChildAt(nsIContent* aKid, PRInt32 aIndex,
       }
     }
   }
-  if (aNotify && (nsnull != doc)) {
-    doc->EndUpdate();
-  }
+  
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSVGElement::ReplaceChildAt(nsIContent* aKid, PRInt32 aIndex,
+nsSVGElement::ReplaceChildAt(nsIContent* aKid, PRUint32 aIndex,
                              PRBool aNotify,
                              PRBool aDeepSetDocument)
 {
   NS_PRECONDITION(nsnull != aKid, "null ptr");
   nsIDocument* doc = mDocument;
-  if (aNotify && (nsnull != mDocument)) {
-    doc->BeginUpdate();
-  }
+  mozAutoDocUpdate updateBatch(doc, UPDATE_CONTENT_MODEL, aNotify);
+
   nsIContent* oldKid = (nsIContent *)mChildren.ElementAt(aIndex);
   nsRange::OwnerChildReplaced(this, aIndex, oldKid);
   PRBool rv = mChildren.ReplaceElementAt(aKid, aIndex);
@@ -228,9 +216,6 @@ nsSVGElement::ReplaceChildAt(nsIContent* aKid, PRInt32 aIndex,
     oldKid->SetParent(nsnull);
     NS_RELEASE(oldKid);
   }
-  if (aNotify && (nsnull != mDocument)) {
-    doc->EndUpdate();
-  }
   return NS_OK;
 }
 
@@ -240,9 +225,8 @@ nsSVGElement::AppendChildTo(nsIContent* aKid, PRBool aNotify,
 {
   NS_PRECONDITION(nsnull != aKid && this != aKid, "null ptr");
   nsIDocument* doc = mDocument;
-  if (aNotify && (nsnull != doc)) {
-    doc->BeginUpdate();
-  }
+  mozAutoDocUpdate updateBatch(doc, UPDATE_CONTENT_MODEL, aNotify);
+
   PRBool rv = mChildren.AppendElement(aKid);
   if (rv) {
     NS_ADDREF(aKid);
@@ -270,21 +254,18 @@ nsSVGElement::AppendChildTo(nsIContent* aKid, PRBool aNotify,
       }
     }
   }
-  if (aNotify && (nsnull != doc)) {
-    doc->EndUpdate();
-  }
+  
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSVGElement::RemoveChildAt(PRInt32 aIndex, PRBool aNotify)
+nsSVGElement::RemoveChildAt(PRUint32 aIndex, PRBool aNotify)
 {
   nsIDocument* doc = mDocument;
-  if (aNotify && (nsnull != doc)) {
-    doc->BeginUpdate();
-  }
+
   nsIContent* oldKid = (nsIContent *)mChildren.ElementAt(aIndex);
   if (nsnull != oldKid ) {
+    mozAutoDocUpdate updateBatch(doc, UPDATE_CONTENT_MODEL, aNotify);
 
     if (nsGenericElement::HasMutationListeners(this, NS_EVENT_BITS_MUTATION_NODEREMOVED)) {
       nsCOMPtr<nsIDOMEventTarget> node(do_QueryInterface(oldKid));
@@ -314,13 +295,16 @@ nsSVGElement::RemoveChildAt(PRInt32 aIndex, PRBool aNotify)
     oldKid->SetParent(nsnull);
     NS_RELEASE(oldKid);
   }
-  if (aNotify && (nsnull != doc)) {
-    doc->EndUpdate();
-  }
 
   return NS_OK;  
 }
 
+NS_IMETHODIMP_(nsIAtom*)
+nsSVGElement::GetIDAttributeName() const
+{
+  return nsSVGAtoms::id;
+}
+
 NS_IMETHODIMP
 nsSVGElement::NormalizeAttrString(const nsAString& aStr,
                                   nsINodeInfo** aNodeInfo)
@@ -387,7 +371,7 @@ nsSVGElement::HasAttr(PRInt32 aNameSpaceID, nsIAtom* aName) const
 }
 
 NS_IMETHODIMP
-nsSVGElement::GetAttrNameAt(PRInt32 aIndex,
+nsSVGElement::GetAttrNameAt(PRUint32 aIndex,
                             PRInt32* aNameSpaceID,
                             nsIAtom** aName,
                             nsIAtom** aPrefix) const
@@ -395,11 +379,10 @@ nsSVGElement::GetAttrNameAt(PRInt32 aIndex,
   return mAttributes->GetAttrNameAt(aIndex, aNameSpaceID, aName, aPrefix);
 }
 
-NS_IMETHODIMP
-nsSVGElement::GetAttrCount(PRInt32& aResult) const
+NS_IMETHODIMP_(PRUint32)
+nsSVGElement::GetAttrCount() const
 {
-  aResult = mAttributes->Count();
-  return NS_OK;
+  return mAttributes->Count();
 }
 
 #ifdef DEBUG
@@ -450,26 +433,6 @@ nsSVGElement::GetInlineStyleRule(nsIStyleRule** aStyleRule)
   return mStyle->GetStyleRule(this, aStyleRule);
 }
 
-NS_IMETHODIMP
-nsSVGElement::GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
-                                       nsChangeHint& aHint) const
-{
-  // we don't rely on the cssframeconstructor to map attribute changes
-  // to changes in our frames. an exception is css.
-  // style_hint_content will trigger a re-resolve of the style context
-  // if the attribute is used in a css selector:
-  aHint = NS_STYLE_HINT_CONTENT;
-
-  // ... and we special case the style attribute
-  if (aAttribute == nsSVGAtoms::style) {
-    aHint = NS_STYLE_HINT_VISUAL;
-//    aHint = NS_STYLE_HINT_FRAMECHANGE;
-  }
-  
-  return NS_OK;
-}
-
-
 //----------------------------------------------------------------------
 // nsIDOMNode methods
 
@@ -482,14 +445,17 @@ nsSVGElement::GetNodeName(nsAString& aNodeName)
 NS_IMETHODIMP
 nsSVGElement::GetNodeValue(nsAString& aNodeValue)
 {
-  aNodeValue.Truncate();
+  SetDOMStringToNull(aNodeValue);
+
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsSVGElement::SetNodeValue(const nsAString& aNodeValue)
 {
-  return NS_ERROR_DOM_NO_MODIFICATION_ALLOWED_ERR;
+  // The DOM spec says that when nodeValue is defined to be null "setting it
+  // has no effect", so we don't throw an exception.
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -525,10 +491,11 @@ nsSVGElement::GetChildNodes(nsIDOMNodeList** aChildNodes)
     if (!slots->mChildNodes) {
       return NS_ERROR_OUT_OF_MEMORY;
     }
-    NS_ADDREF(slots->mChildNodes);
   }
 
-  return CallQueryInterface(slots->mChildNodes, aChildNodes);
+  NS_ADDREF(*aChildNodes = slots->mChildNodes);
+
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -669,12 +636,8 @@ nsSVGElement::HasAttributes(PRBool* aReturn)
 {
   NS_ENSURE_ARG_POINTER(aReturn);
   
-  PRInt32 attrCount = 0;
-  
-  GetAttrCount(attrCount);
-  
-  *aReturn = (attrCount > 0);
-  
+  *aReturn = GetAttrCount() > 0;
+
   return NS_OK;
 }
 
@@ -704,9 +667,9 @@ nsSVGElement::GetOwnerSVGElement(nsIDOMSVGSVGElement * *aOwnerSVGElement)
 {
   *aOwnerSVGElement = nsnull;
 
-  nsCOMPtr<nsIBindingManager> bindingManager;
+  nsIBindingManager *bindingManager = nsnull;
   if (mDocument) {
-    mDocument->GetBindingManager(getter_AddRefs(bindingManager));
+    bindingManager = mDocument->GetBindingManager();
   }
 
   nsCOMPtr<nsIContent> parent;
@@ -737,7 +700,7 @@ nsSVGElement::GetOwnerSVGElement(nsIDOMSVGSVGElement * *aOwnerSVGElement)
 
     if (!next) {
       // no anonymous parent, so use explicit one
-      parent->GetParent(getter_AddRefs(next));
+      next = parent->GetParent();
     }
     
     parent = next;
