@@ -228,10 +228,11 @@ ClearObjectEntry(PLDHashTable* table, PLDHashEntryHdr *entry)
   objEntry->~ObjectEntry();
 }
 
-PR_STATIC_CALLBACK(void)
+PR_STATIC_CALLBACK(PRBool)
 InitObjectEntry(PLDHashTable* table, PLDHashEntryHdr* entry, const void* key)
 {
   new (entry) ObjectEntry;
+  return PR_TRUE;
 }
   
 
@@ -297,14 +298,16 @@ RemoveObjectEntry(PLDHashTable& table, nsISupports* aKey)
 static nsresult
 SetOrRemoveObject(PLDHashTable& table, nsISupports* aKey, nsISupports* aValue)
 {
-  // lazily create the table, but don't create it just to remove a
-  // non-existent element!
-  if (!table.ops && aValue)
-    PL_DHashTableInit(&table, &ObjectTableOps, nsnull,
-                      sizeof(ObjectEntry), 16);
-
-  if (aValue)
+  if (aValue) {
+    // lazily create the table, but only when adding elements
+    if (!table.ops &&
+        !PL_DHashTableInit(&table, &ObjectTableOps, nsnull,
+                           sizeof(ObjectEntry), 16)) {
+      table.ops = nsnull;
+      return NS_ERROR_OUT_OF_MEMORY;
+    }
     return AddObjectEntry(table, aKey, aValue);
+  }
 
   // no value, so remove the key from the table
   if (table.ops)
@@ -349,8 +352,8 @@ public:
   NS_IMETHOD GetSingleInsertionPoint(nsIContent* aParent, nsIContent** aResult, PRUint32* aIndex,  
                                      PRBool* aMultipleInsertionPoints);
 
-  NS_IMETHOD AddLayeredBinding(nsIContent* aContent, const nsAString& aURL);
-  NS_IMETHOD RemoveLayeredBinding(nsIContent* aContent, const nsAString& aURL);
+  NS_IMETHOD AddLayeredBinding(nsIContent* aContent, nsIURI* aURL);
+  NS_IMETHOD RemoveLayeredBinding(nsIContent* aContent, nsIURI* aURL);
   NS_IMETHOD LoadBindingDocument(nsIDocument* aBoundDoc, const nsAString& aURL,
                                  nsIDocument** aResult);
 
@@ -392,8 +395,10 @@ protected:
                                      nsIDOMNodeList** aResult,
                                      PRBool* aIsAnonymousContentList);
 
-  void GetEnclosingScope(nsIContent* aContent, nsIContent** aParent);
-  void GetOutermostStyleScope(nsIContent* aContent, nsIContent** aParent);
+  nsIContent* GetEnclosingScope(nsIContent* aContent) {
+    return aContent->GetBindingParent();
+  }
+  nsIContent* GetOutermostStyleScope(nsIContent* aContent);
 
   void WalkRules(nsISupportsArrayEnumFunc aFunc, RuleProcessorData* aData,
                  nsIContent* aParent, nsIContent* aCurrContent);
@@ -447,7 +452,8 @@ protected:
 
   // A queue of binding attached event handlers that are awaiting
   // execution.
-  nsCOMPtr<nsISupportsArray> mAttachedQueue;
+  nsCOMPtr<nsISupportsArray> mAttachedStack;
+  PRBool mProcessingAttachedStack;
 };
 
 // Implementation /////////////////////////////////////////////////////////////////
@@ -459,15 +465,13 @@ NS_IMPL_ISUPPORTS3(nsBindingManager, nsIBindingManager, nsIStyleRuleSupplier, ns
 
 // Constructors/Destructors
 nsBindingManager::nsBindingManager(void)
+: mProcessingAttachedStack(PR_FALSE)
 {
-
   mBindingTable.ops = nsnull;
   mContentListTable.ops = nsnull;
   mAnonymousNodesTable.ops = nsnull;
   mInsertionParentTable.ops = nsnull;
   mWrapperTable.ops = nsnull;
-
-  mAttachedQueue = nsnull;
 }
 
 nsBindingManager::~nsBindingManager(void)
@@ -569,11 +573,8 @@ nsBindingManager::ChangeDocumentFor(nsIContent* aContent, nsIDocument* aOldDocum
   if (binding) {
     binding->ChangeDocument(aOldDocument, aNewDocument);
     SetBinding(aContent, nsnull);
-    if (aNewDocument) {
-      nsCOMPtr<nsIBindingManager> otherManager;
-      aNewDocument->GetBindingManager(getter_AddRefs(otherManager));
-      otherManager->SetBinding(aContent, binding);
-    }
+    if (aNewDocument)
+      aNewDocument->GetBindingManager()->SetBinding(aContent, binding);
   }
 
   // Clear out insertion parents and content lists.
@@ -581,10 +582,11 @@ nsBindingManager::ChangeDocumentFor(nsIContent* aContent, nsIDocument* aOldDocum
   SetContentListFor(aContent, nsnull);
   SetAnonymousNodesFor(aContent, nsnull);
 
-  for (PRInt32 i = aOldDocument->GetNumberOfShells() - 1; i >= 0; --i) {
-    nsCOMPtr<nsIPresShell> shell;
-    aOldDocument->GetShellAt(i, getter_AddRefs(shell));
-    NS_ASSERTION(shell != nsnull, "Zoiks! nsIPresShell::ShellAt() broke");
+  PRUint32 count = aOldDocument->GetNumberOfShells();
+
+  for (PRUint32 i = 0; i < count; ++i) {
+    nsIPresShell *shell = aOldDocument->GetShellAt(i);
+    NS_ASSERTION(shell != nsnull, "Zoiks! nsIDocument::GetShellAt() broke");
 
     // See if the element has nsIAnonymousContentCreator-created
     // anonymous content...
@@ -628,8 +630,8 @@ nsBindingManager::ResolveTag(nsIContent* aContent, PRInt32* aNameSpaceID, nsIAto
     }
   }
 
-  aContent->GetNameSpaceID(*aNameSpaceID);
-  return aContent->GetTag(*aResult);
+  aContent->GetNameSpaceID(aNameSpaceID);
+  return aContent->GetTag(aResult);
 }
 
 NS_IMETHODIMP
@@ -796,7 +798,7 @@ nsBindingManager::GetSingleInsertionPoint(nsIContent* aParent, nsIContent** aRes
 }
 
 NS_IMETHODIMP
-nsBindingManager::AddLayeredBinding(nsIContent* aContent, const nsAString& aURL)
+nsBindingManager::AddLayeredBinding(nsIContent* aContent, nsIURI* aURL)
 {
   // First we need to load our binding.
   nsresult rv;
@@ -818,7 +820,7 @@ nsBindingManager::AddLayeredBinding(nsIContent* aContent, const nsAString& aURL)
 }
 
 NS_IMETHODIMP
-nsBindingManager::RemoveLayeredBinding(nsIContent* aContent, const nsAString& aURL)
+nsBindingManager::RemoveLayeredBinding(nsIContent* aContent, nsIURI* aURL)
 {
   nsCOMPtr<nsIXBLBinding> binding;
   GetBinding(aContent, getter_AddRefs(binding));
@@ -839,11 +841,8 @@ nsBindingManager::RemoveLayeredBinding(nsIContent* aContent, const nsAString& aU
   nsresult rv = NS_NewURI(getter_AddRefs(bindingUri), bindingUriStr);
   NS_ENSURE_SUCCESS(rv, rv);
   
-  nsCOMPtr<nsIURI> uri;
-  rv = NS_NewURI(getter_AddRefs(uri), aURL);
-  NS_ENSURE_SUCCESS(rv, rv);
   PRBool equalUri;
-  rv = uri->Equals(bindingUri, &equalUri);
+  rv = aURL->Equals(bindingUri, &equalUri);
   NS_ENSURE_SUCCESS(rv, rv);
   if (!equalUri) {
     return NS_OK;
@@ -874,15 +873,15 @@ nsBindingManager::RemoveLayeredBinding(nsIContent* aContent, const nsAString& aU
   // ...and recreate it's frames. We need to do this since the frames may have
   // been removed and style may have changed due to the removal of the
   // anonymous children.
-  nsCOMPtr<nsIPresShell> presShell;
-  rv = doc->GetShellAt(0, getter_AddRefs(presShell));
+  nsIPresShell *presShell = doc->GetShellAt(0);
   NS_ENSURE_TRUE(presShell, NS_ERROR_FAILURE);
 
   return presShell->RecreateFramesFor(aContent);;
 }
 
 NS_IMETHODIMP
-nsBindingManager::LoadBindingDocument(nsIDocument* aBoundDoc, const nsAString& aURL,
+nsBindingManager::LoadBindingDocument(nsIDocument* aBoundDoc,
+                                      const nsAString& aURL,
                                       nsIDocument** aResult)
 {
   NS_ConvertUCS2toUTF8 url(aURL);
@@ -892,10 +891,8 @@ nsBindingManager::LoadBindingDocument(nsIDocument* aBoundDoc, const nsAString& a
   if (!ioService) return NS_ERROR_FAILURE;
   ioService->ExtractScheme(url, otherScheme);
 
-  nsCOMPtr<nsIURI> docURL;
-  aBoundDoc->GetDocumentURL(getter_AddRefs(docURL));
   nsCAutoString scheme;
-  docURL->GetScheme(scheme);
+  aBoundDoc->GetDocumentURL()->GetScheme(scheme);
 
   // First we need to load our binding.
   *aResult = nsnull;
@@ -920,10 +917,10 @@ nsBindingManager::LoadBindingDocument(nsIDocument* aBoundDoc, const nsAString& a
 NS_IMETHODIMP
 nsBindingManager::AddToAttachedQueue(nsIXBLBinding* aBinding)
 {
-  if (!mAttachedQueue)
-    NS_NewISupportsArray(getter_AddRefs(mAttachedQueue)); // This call addrefs the array.
+  if (!mAttachedStack)
+    NS_NewISupportsArray(getter_AddRefs(mAttachedStack)); // This call addrefs the array.
 
-  mAttachedQueue->AppendElement(aBinding);
+  mAttachedStack->AppendElement(aBinding);
 
   return NS_OK;
 }
@@ -931,29 +928,29 @@ nsBindingManager::AddToAttachedQueue(nsIXBLBinding* aBinding)
 NS_IMETHODIMP
 nsBindingManager::ClearAttachedQueue()
 {
-  if (mAttachedQueue)
-    mAttachedQueue->Clear();
+  if (mAttachedStack)
+    mAttachedStack->Clear();
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsBindingManager::ProcessAttachedQueue()
 {
-  if (!mAttachedQueue)
+  if (!mAttachedStack || mProcessingAttachedStack)
     return NS_OK;
 
+  mProcessingAttachedStack = PR_TRUE;
+
   PRUint32 count;
-  mAttachedQueue->Count(&count);
-  for (PRUint32 i = 0; i < count; i++) {
-    nsCOMPtr<nsISupports> supp;
-    mAttachedQueue->GetElementAt(0, getter_AddRefs(supp));
-    mAttachedQueue->RemoveElementAt(0);
+  while (NS_SUCCEEDED(mAttachedStack->Count(&count)) && count--) {
+    nsCOMPtr<nsIXBLBinding> binding = do_QueryElementAt(mAttachedStack, count);
+    mAttachedStack->RemoveElementAt(count);
 
-    nsCOMPtr<nsIXBLBinding> binding(do_QueryInterface(supp));
     if (binding)
       binding->ExecuteAttachedHandler();
   }
 
+  mProcessingAttachedStack = PR_FALSE;
   ClearAttachedQueue();
   return NS_OK;
 }
@@ -986,10 +983,8 @@ nsBindingManager::PutXBLDocumentInfo(nsIXBLDocumentInfo* aDocumentInfo)
   nsCOMPtr<nsIDocument> doc;
   aDocumentInfo->GetDocument(getter_AddRefs(doc));
 
-  nsCOMPtr<nsIURI> uri;
-  doc->GetDocumentURL(getter_AddRefs(uri));
   nsCAutoString str;
-  uri->GetSpec(str);
+  doc->GetDocumentURL()->GetSpec(str);
 
   StringToObjectEntry* entry = mDocumentTable.AddEntry(str);
   if (!entry) return NS_ERROR_OUT_OF_MEMORY;
@@ -1007,10 +1002,8 @@ nsBindingManager::RemoveXBLDocumentInfo(nsIXBLDocumentInfo* aDocumentInfo)
   nsCOMPtr<nsIDocument> doc;
   aDocumentInfo->GetDocument(getter_AddRefs(doc));
 
-  nsCOMPtr<nsIURI> uri;
-  doc->GetDocumentURL(getter_AddRefs(uri));
   nsCAutoString str;
-  uri->GetSpec(str);
+  doc->GetDocumentURL()->GetSpec(str);
 
   mDocumentTable.Remove(str);
   return NS_OK;
@@ -1175,13 +1168,11 @@ nsBindingManager::GetBindingImplementation(nsIContent* aContent, REFNSIID aIID,
       // We have never made a wrapper for this implementation.
       // Create an XPC wrapper for the script object and hand it back.
 
-      nsCOMPtr<nsIDocument> doc;
-      aContent->GetDocument(*getter_AddRefs(doc));
+      nsIDocument* doc = aContent->GetDocument();
       if (!doc)
         return NS_NOINTERFACE;
 
-      nsCOMPtr<nsIScriptGlobalObject> global;
-      doc->GetScriptGlobalObject(getter_AddRefs(global));
+      nsIScriptGlobalObject *global = doc->GetScriptGlobalObject();
       if (!global)
         return NS_NOINTERFACE;
 
@@ -1237,13 +1228,10 @@ nsBindingManager::InheritsStyle(nsIContent* aContent, PRBool* aResult)
 {
   // Get our enclosing parent.
   *aResult = PR_TRUE;
-  nsCOMPtr<nsIContent> parent;
-  GetEnclosingScope(aContent, getter_AddRefs(parent));
+  nsCOMPtr<nsIContent> parent = GetEnclosingScope(aContent);
   if (parent) {
     // See if the parent is our parent.
-    nsCOMPtr<nsIContent> ourParent;
-    aContent->GetParent(*getter_AddRefs(ourParent));
-    if (ourParent == parent) {
+    if (aContent->GetParent() == parent) {
       // Yes. Check the binding and see if it wants to allow us
       // to inherit styles.
       nsCOMPtr<nsIXBLBinding> binding;
@@ -1261,26 +1249,14 @@ nsBindingManager::UseDocumentRules(nsIContent* aContent, PRBool* aResult)
   if (!aContent)
     return NS_OK;
 
-  nsCOMPtr<nsIContent> parent;
-  GetOutermostStyleScope(aContent, getter_AddRefs(parent));
-  *aResult = !parent;
+  *aResult = !GetOutermostStyleScope(aContent);
   return NS_OK;
 }
 
-void
-nsBindingManager::GetEnclosingScope(nsIContent* aContent,
-                                    nsIContent** aParent)
+nsIContent*
+nsBindingManager::GetOutermostStyleScope(nsIContent* aContent)
 {
-  // Look up the enclosing parent.
-  aContent->GetBindingParent(aParent);
-}
-
-void
-nsBindingManager::GetOutermostStyleScope(nsIContent* aContent,
-                                         nsIContent** aParent)
-{
-  nsCOMPtr<nsIContent> parent;
-  GetEnclosingScope(aContent, getter_AddRefs(parent));
+  nsIContent* parent = GetEnclosingScope(aContent);
   while (parent) {
     PRBool inheritsStyle = PR_TRUE;
     nsCOMPtr<nsIXBLBinding> binding;
@@ -1290,14 +1266,13 @@ nsBindingManager::GetOutermostStyleScope(nsIContent* aContent,
     }
     if (!inheritsStyle)
       break;
-    nsCOMPtr<nsIContent> child = parent;
-    GetEnclosingScope(child, getter_AddRefs(parent));
+    nsIContent* child = parent;
+    parent = GetEnclosingScope(child);
     if (parent == child)
       break; // The scrollbar case only is deliberately hacked to return itself
              // (see GetBindingParent in nsXULElement.cpp).
   }
-  *aParent = parent;
-  NS_IF_ADDREF(*aParent);
+  return parent;
 }
 
 void
@@ -1312,8 +1287,7 @@ nsBindingManager::WalkRules(nsISupportsArrayEnumFunc aFunc,
     binding->WalkRules(aFunc, aData);
   }
   if (aParent != aCurrContent) {
-    nsCOMPtr<nsIContent> par;
-    GetEnclosingScope(aCurrContent, getter_AddRefs(par));
+    nsCOMPtr<nsIContent> par = GetEnclosingScope(aCurrContent);
     if (par)
       WalkRules(aFunc, aData, aParent, par);
   }
@@ -1328,8 +1302,7 @@ nsBindingManager::WalkRules(nsIStyleSet* aStyleSet,
   if (!content)
     return NS_OK;
 
-  nsCOMPtr<nsIContent> parent;
-  GetOutermostStyleScope(content, getter_AddRefs(parent));
+  nsCOMPtr<nsIContent> parent = GetOutermostStyleScope(content);
 
   WalkRules(aFunc, aData, parent, content);
 
@@ -1339,9 +1312,8 @@ nsBindingManager::WalkRules(nsIStyleSet* aStyleSet,
   if (parent) {
     // We cut ourselves off, but we still need to walk the document's attribute sheet
     // so that inline style continues to work on anonymous content.
-    nsCOMPtr<nsIDocument> document;
-    content->GetDocument(*getter_AddRefs(document));
-    nsCOMPtr<nsIHTMLContentContainer> container(do_QueryInterface(document));
+    nsCOMPtr<nsIHTMLContentContainer> container(
+          do_QueryInterface(content->GetDocument()));
     if (container) {
       nsCOMPtr<nsIHTMLCSSStyleSheet> inlineSheet;
       container->GetInlineStyleSheet(getter_AddRefs(inlineSheet));  
@@ -1374,9 +1346,7 @@ nsBindingManager::GetNestedInsertionPoint(nsIContent* aParent, nsIContent* aChil
   *aResult = nsnull;
 
   // Check to see if the content is anonymous.
-  nsCOMPtr<nsIContent> bindingParent;
-  aChild->GetBindingParent(getter_AddRefs(bindingParent));
-  if (bindingParent == aParent)
+  if (aChild->GetBindingParent() == aParent)
     return NS_OK; // It is anonymous. Don't use the insertion point, since that's only
                   // for the explicit kids.
 
@@ -1415,8 +1385,7 @@ nsBindingManager::AttributeChanged(nsIDocument* aDocument,
                               nsIContent*  aContent,
                               PRInt32      aNameSpaceID,
                               nsIAtom*     aAttribute,
-                              PRInt32      aModType, 
-                              nsChangeHint aHint)
+                              PRInt32      aModType)
 {
   return NS_OK;
 }
@@ -1431,11 +1400,10 @@ nsBindingManager::ContentAppended(nsIDocument* aDocument,
     // It's anonymous.
     return NS_OK;
 
-  PRInt32 childCount;
-  nsCOMPtr<nsIContent> child;
-  aContainer->ChildCount(childCount);
-  aContainer->ChildAt(aNewIndexInContainer, *getter_AddRefs(child));
-  
+  PRInt32 childCount = aContainer->GetChildCount();
+
+  nsIContent *child = aContainer->GetChildAt(aNewIndexInContainer);
+
   nsCOMPtr<nsIContent> ins;
   GetNestedInsertionPoint(aContainer, child, getter_AddRefs(ins));
 
@@ -1458,7 +1426,7 @@ nsBindingManager::ContentAppended(nsIDocument* aDocument,
           // We're real. Jam all the kids in.
           // XXX Check the filters to find the correct points.
           for (PRInt32 j = aNewIndexInContainer; j < childCount; j++) {
-            aContainer->ChildAt(j, *getter_AddRefs(child));
+            child = aContainer->GetChildAt(j);
             point->AddChild(child);
             SetInsertionParent(child, ins);
           }
