@@ -87,20 +87,9 @@ public:
   nsXBLPrototypeHandler* mDestructor;  // Our class destructor.
 };
 
-static nsresult
+nsresult
 NS_NewXBLProtoImpl(nsXBLPrototypeBinding* aBinding, 
                    const PRUnichar* aClassName, 
-                   nsXBLProtoImpl** aResult) {
-  nsXBLProtoImpl* impl = new nsXBLProtoImpl();
-  if (!impl)
-    return NS_ERROR_OUT_OF_MEMORY;
-  if (aClassName)
-    impl->mClassName.AssignWithConversion(aClassName);
-  else
-    aBinding->GetBindingURI(impl->mClassName);
-  aBinding->SetImplementation(impl);
-  *aResult = impl;
-  return NS_OK;
-}
+                   nsXBLProtoImpl** aResult);
 
 #endif // nsXBLProtoImpl_h__
