@@ -223,11 +223,11 @@ static const PRInt32 kInsInitialSize = (NS_SIZE_IN_HEAP(sizeof(nsXBLInsertionPoi
 nsXBLPrototypeBinding::nsXBLPrototypeBinding(const nsACString& aID,
                                              nsIXBLDocumentInfo* aInfo,
                                              nsIContent* aElement)
-: mPrototypeHandler(nsnull),
-  mImplementation(nsnull),
+: mImplementation(nsnull),
   mBaseBinding(nsnull),
   mInheritStyle(PR_TRUE), 
   mHasBaseProto(PR_TRUE),
+  mKeyHandlersRegistered(PR_FALSE),
   mResources(nsnull),
   mAttributeTable(nsnull), 
   mInsertionPointTable(nsnull),
@@ -236,7 +236,7 @@ nsXBLPrototypeBinding::nsXBLPrototypeBinding(const nsACString& aID,
   
   mID = ToNewCString(aID);
 
-  mXBLDocInfoWeak = getter_AddRefs(NS_GetWeakReference(aInfo));
+  mXBLDocInfoWeak = do_GetWeakReference(aInfo);
   
   gRefCnt++;
   //  printf("REF COUNT UP: %d %s\n", gRefCnt, (const char*)mID);
@@ -269,7 +269,6 @@ nsXBLPrototypeBinding::~nsXBLPrototypeBinding(void)
   delete mInsertionPointTable;
   delete mInterfaceTable;
   delete mImplementation;
-  delete mPrototypeHandler;
   gRefCnt--;
   if (gRefCnt == 0) {
     delete kAttrPool;
@@ -503,23 +502,21 @@ nsXBLPrototypeBinding::AttributeChanged(nsIAtom* aAttribute,
       // set or unset on us.  We may also be a tag that is having
       // xbl:text set on us.
       nsCOMPtr<nsIAtom> tag;
-      realElement->GetTag(*getter_AddRefs(tag));
+      realElement->GetTag(getter_AddRefs(tag));
       if (dstAttr == nsXBLAtoms::xbltext || (tag == nsHTMLAtoms::html) && (dstAttr == nsHTMLAtoms::value)) {
         // Flush out all our kids.
-        PRInt32 childCount;
-        realElement->ChildCount(childCount);
-        for (PRInt32 i = 0; i < childCount; i++)
+        PRUint32 childCount = realElement->GetChildCount();
+        for (PRUint32 i = 0; i < childCount; i++)
           realElement->RemoveChildAt(0, aNotify);
-      
+
         if (!aRemoveFlag) {
           // Construct a new text node and insert it.
           nsAutoString value;
           aChangedElement->GetAttr(aNameSpaceID, aAttribute, value);
           if (!value.IsEmpty()) {
             nsCOMPtr<nsIDOMText> textNode;
-            nsCOMPtr<nsIDocument> doc;
-            aChangedElement->GetDocument(*getter_AddRefs(doc));
-            nsCOMPtr<nsIDOMDocument> domDoc(do_QueryInterface(doc));
+            nsCOMPtr<nsIDOMDocument> domDoc(
+                     do_QueryInterface(aChangedElement->GetDocument()));
             domDoc->CreateTextNode(value, getter_AddRefs(textNode));
             nsCOMPtr<nsIDOMNode> dummy;
             nsCOMPtr<nsIDOMElement> domElement(do_QueryInterface(realElement));
@@ -612,7 +609,7 @@ nsXBLPrototypeBinding::GetInsertionPoint(nsIContent* aBoundElement,
 {
   if (mInsertionPointTable) {
     nsCOMPtr<nsIAtom> tag;
-    aChild->GetTag(*getter_AddRefs(tag));
+    aChild->GetTag(getter_AddRefs(tag));
     nsISupportsKey key(tag);
     nsXBLInsertionPointEntry* entry = NS_STATIC_CAST(nsXBLInsertionPointEntry*, mInsertionPointTable->Get(&key));
     if (!entry) {
@@ -727,17 +724,15 @@ nsXBLPrototypeBinding::ImplementsInterface(REFNSIID aIID)
 already_AddRefed<nsIContent>
 nsXBLPrototypeBinding::GetImmediateChild(nsIAtom* aTag)
 {
-  PRInt32 childCount;
-  mBinding->ChildCount(childCount);
-  for (PRInt32 i = 0; i < childCount; i++) {
-    nsCOMPtr<nsIContent> child;
-    mBinding->ChildAt(i, *getter_AddRefs(child));
+  PRUint32 childCount = mBinding->GetChildCount();
+
+  for (PRUint32 i = 0; i < childCount; i++) {
+    nsIContent *child = mBinding->GetChildAt(i);
     nsCOMPtr<nsIAtom> tag;
-    child->GetTag(*getter_AddRefs(tag));
+    child->GetTag(getter_AddRefs(tag));
     if (aTag == tag) {
-      nsIContent* result = child;
-      NS_ADDREF(result);
-      return result;
+      NS_ADDREF(child);
+      return child;
     }
   }
 
@@ -771,17 +766,16 @@ nsXBLPrototypeBinding::LocateInstance(nsIContent* aBoundElement,
   if (aTemplChild == aTemplRoot || !aTemplChild)
     return nsnull;
 
-  nsCOMPtr<nsIContent> templParent;
+  nsCOMPtr<nsIContent> templParent = aTemplChild->GetParent();
   nsCOMPtr<nsIContent> copyParent;
   nsCOMPtr<nsIContent> childPoint;
-  aTemplChild->GetParent(*getter_AddRefs(templParent));
   
   if (aBoundElement) {
     nsCOMPtr<nsIAtom> tag;
-    templParent->GetTag(*getter_AddRefs(tag));
+    templParent->GetTag(getter_AddRefs(tag));
     if (tag == nsXBLAtoms::children) {
       childPoint = templParent;
-      childPoint->GetParent(*getter_AddRefs(templParent));
+      templParent = childPoint->GetParent();
     }
   }
 
@@ -798,12 +792,10 @@ nsXBLPrototypeBinding::LocateInstance(nsIContent* aBoundElement,
   if (childPoint && aBoundElement) {
     // First we have to locate this insertion point and use its index and its
     // count to detemine our precise position within the template.
-    nsCOMPtr<nsIDocument> doc;
-    aBoundElement->GetDocument(*getter_AddRefs(doc));
-    nsCOMPtr<nsIBindingManager> bm;
-    doc->GetBindingManager(getter_AddRefs(bm));
+    nsIDocument* doc = aBoundElement->GetDocument();
     nsCOMPtr<nsIXBLBinding> binding;
-    bm->GetBinding(aBoundElement, getter_AddRefs(binding));
+    doc->GetBindingManager()->GetBinding(aBoundElement,
+                                         getter_AddRefs(binding));
     
     nsCOMPtr<nsIXBLBinding> currBinding = binding;
     nsCOMPtr<nsIContent> anonContent;
@@ -833,12 +825,11 @@ nsXBLPrototypeBinding::LocateInstance(nsIContent* aBoundElement,
         defContent = currPoint->GetDefaultContent();
         if (defContent) {
           // Find out the index of the template element within the <children> elt.
-          PRInt32 index;
-          childPoint->IndexOf(aTemplChild, index);
+          PRInt32 index = childPoint->IndexOf(aTemplChild);
           
           // Now we just have to find the corresponding elt underneath the cloned
           // default content.
-          defContent->ChildAt(index, result); // addrefs
+          result = defContent->GetChildAt(index);
         } 
         break;
       }
@@ -846,11 +837,12 @@ nsXBLPrototypeBinding::LocateInstance(nsIContent* aBoundElement,
   }
   else if (copyParent)
   {
-    PRInt32 index;
-    templParent->IndexOf(aTemplChild, index);
-    copyParent->ChildAt(index, result); // Addref happens here.
+    PRInt32 index = templParent->IndexOf(aTemplChild);
+    result = copyParent->GetChildAt(index);
   }
 
+  NS_IF_ADDREF(result);
+
   return result;
 }
 
@@ -908,13 +900,12 @@ PRBool PR_CALLBACK SetAttrs(nsHashKey* aKey, void* aData, void* aClosure)
       if (realElement) {
         realElement->SetAttr(kNameSpaceID_None, dst, value, PR_FALSE);
         nsCOMPtr<nsIAtom> tag;
-        realElement->GetTag(*getter_AddRefs(tag));
+        realElement->GetTag(getter_AddRefs(tag));
         if (dst == nsXBLAtoms::xbltext ||
             (tag == nsHTMLAtoms::html) && (dst == nsHTMLAtoms::value) && !value.IsEmpty()) {
           nsCOMPtr<nsIDOMText> textNode;
-          nsCOMPtr<nsIDocument> doc;
-          changeData->mBoundElement->GetDocument(*getter_AddRefs(doc));
-          nsCOMPtr<nsIDOMDocument> domDoc(do_QueryInterface(doc));
+          nsCOMPtr<nsIDOMDocument> domDoc(
+                  do_QueryInterface(changeData->mBoundElement->GetDocument()));
           domDoc->CreateTextNode(value, getter_AddRefs(textNode));
           nsCOMPtr<nsIDOMNode> dummy;
           nsCOMPtr<nsIDOMElement> domElement(do_QueryInterface(realElement));
@@ -1058,12 +1049,9 @@ nsXBLPrototypeBinding::ConstructAttributeTable(nsIContent* aElement)
   }
 
   // Recur into our children.
-  PRInt32 childCount;
-  aElement->ChildCount(childCount);
-  for (PRInt32 i = 0; i < childCount; i++) {
-    nsCOMPtr<nsIContent> child;
-    aElement->ChildAt(i, *getter_AddRefs(child));
-    ConstructAttributeTable(child);
+  PRUint32 childCount = aElement->GetChildCount();
+  for (PRUint32 i = 0; i < childCount; i++) {
+    ConstructAttributeTable(aElement->GetChildAt(i));
   }
 }
 
@@ -1097,8 +1085,7 @@ nsXBLPrototypeBinding::ConstructInsertionTable(nsIContent* aContent)
     childrenElements->GetElementAt(i, getter_AddRefs(supp));
     nsCOMPtr<nsIContent> child(do_QueryInterface(supp));
     if (child) {
-      nsCOMPtr<nsIContent> parent; 
-      child->GetParent(*getter_AddRefs(parent));
+      nsCOMPtr<nsIContent> parent = child->GetParent(); 
 
       // Create an XBL insertion point entry.
       nsXBLInsertionPointEntry* xblIns = nsXBLInsertionPointEntry::Create(parent);
@@ -1142,8 +1129,7 @@ nsXBLPrototypeBinding::ConstructInsertionTable(nsIContent* aContent)
       // if we dynamically obtain our index each time, then the removals of previous
       // siblings will cause the index to adjust (and we won't have to take that into
       // account explicitly).
-      PRInt32 index;
-      parent->IndexOf(child, index);
+      PRInt32 index = parent->IndexOf(child);
       xblIns->SetInsertionIndex((PRUint32)index);
 
       // Now remove the <children> element from the template.  This ensures that the
@@ -1154,8 +1140,7 @@ nsXBLPrototypeBinding::ConstructInsertionTable(nsIContent* aContent)
       // See if the insertion point contains default content.  Default content must
       // be cached in our insertion point entry, since it will need to be cloned
       // in situations where no content ends up being placed at the insertion point.
-      PRInt32 defaultCount;
-      child->ChildCount(defaultCount);
+      PRUint32 defaultCount = child->GetChildCount();
       if (defaultCount > 0) {
         // Annotate the insertion point with our default content.
         xblIns->SetDefaultContent(child);
@@ -1245,13 +1230,13 @@ void
 nsXBLPrototypeBinding::GetNestedChildren(nsIAtom* aTag, nsIContent* aContent,
                                          nsISupportsArray** aList)
 {
-  PRInt32 childCount;
-  aContent->ChildCount(childCount);
-  for (PRInt32 i = 0; i < childCount; i++) {
-    nsCOMPtr<nsIContent> child;
-    aContent->ChildAt(i, *getter_AddRefs(child));
+  PRUint32 childCount = aContent->GetChildCount();
+
+  for (PRUint32 i = 0; i < childCount; i++) {
+    nsIContent *child = aContent->GetChildAt(i);
+
     nsCOMPtr<nsIAtom> tag;
-    child->GetTag(*getter_AddRefs(tag));
+    child->GetTag(getter_AddRefs(tag));
     if (aTag == tag) {
       if (!*aList)
         NS_NewISupportsArray(aList); // Addref happens here.
@@ -1272,3 +1257,41 @@ nsXBLPrototypeBinding::AddResourceListener(nsIContent* aBoundElement)
   mResources->AddResourceListener(aBoundElement);
   return NS_OK;
 }
+
+void
+nsXBLPrototypeBinding::CreateKeyHandlers()
+{
+  nsXBLPrototypeHandler* curr = mPrototypeHandler;
+  while (curr) {
+    nsCOMPtr<nsIAtom> eventAtom = curr->GetEventName();
+    if (eventAtom == nsXBLAtoms::keyup ||
+        eventAtom == nsXBLAtoms::keydown ||
+        eventAtom == nsXBLAtoms::keypress) {
+      PRUint8 phase = curr->GetPhase();
+      PRUint8 type = curr->GetType();
+
+      PRInt32 count = mKeyHandlers.Count();
+      PRInt32 i;
+      nsXBLKeyEventHandler* handler;
+      for (i = 0; i < count; ++i) {
+        handler = mKeyHandlers[i];
+        if (handler->Matches(eventAtom, phase, type))
+          break;
+      }
+
+      if (i == count) {
+        nsRefPtr<nsXBLKeyEventHandler> newHandler;
+        NS_NewXBLKeyEventHandler(eventAtom, phase, type,
+                                 getter_AddRefs(newHandler));
+        if (newHandler)
+          mKeyHandlers.AppendObject(newHandler);
+        handler = newHandler;
+      }
+
+      if (handler)
+        handler->AddProtoHandler(curr);
+    }
+
+    curr = curr->GetNextHandler();
+  }
+}
