@@ -137,8 +137,7 @@ public:
 
   void DocumentLoaded(nsIDocument* aBindingDoc)
   {
-    nsCOMPtr<nsIDocument> doc;
-    mBoundElement->GetDocument(getter_AddRefs(doc));
+    nsCOMPtr<nsIDocument> doc = mBoundElement->GetDocument();
     if (!doc)
       return;
 
@@ -151,11 +150,10 @@ public:
 
     // XXX Deal with layered bindings.
     // Now do a ContentInserted notification to cause the frames to get installed finally,
-    nsCOMPtr<nsIContent> parent;
-    mBoundElement->GetParent(getter_AddRefs(parent));
+    nsCOMPtr<nsIContent> parent = mBoundElement->GetParent();
     PRInt32 index = 0;
     if (parent)
-      parent->IndexOf(mBoundElement, index);
+      index = parent->IndexOf(mBoundElement);
         
     // If |mBoundElement| is (in addition to having binding |mBinding|)
     // also a descendant of another element with binding |mBinding|,
@@ -166,8 +164,7 @@ public:
     // has a primary frame and whether it's in the undisplayed map
     // before sending a ContentInserted notification, or bad things
     // will happen.
-    nsCOMPtr<nsIPresShell> shell;
-    doc->GetShellAt(0, getter_AddRefs(shell));
+    nsIPresShell *shell = doc->GetShellAt(0);
     if (shell) {
       nsIFrame* childFrame;
       shell->GetPrimaryFrameFor(mBoundElement, &childFrame);
@@ -277,7 +274,7 @@ nsXBLStreamListener::nsXBLStreamListener(nsXBLService* aXBLService,
   /* member initializers and constructor code */
   mXBLService = aXBLService;
   mInner = aInner;
-  mDocument = getter_AddRefs(NS_GetWeakReference(aDocument));
+  mDocument = do_GetWeakReference(aDocument);
   mBindingDocument = aBindingDocument;
 #ifdef MOZ_XUL
   gRefCnt++;
@@ -387,32 +384,26 @@ nsXBLStreamListener::Load(nsIDOMEvent* aEvent)
     // ready.
     if (count > 0) {
       nsXBLBindingRequest* req = (nsXBLBindingRequest*)mBindingRequests.ElementAt(0);
-      nsCOMPtr<nsIDocument> document;
-      req->mBoundElement->GetDocument(getter_AddRefs(document));
+      nsIDocument* document = req->mBoundElement->GetDocument();
       if (document)
         document->FlushPendingNotifications();
     }
 
     // Remove ourselves from the set of pending docs.
-    nsCOMPtr<nsIBindingManager> bindingManager;
-    doc->GetBindingManager(getter_AddRefs(bindingManager));
-    nsCOMPtr<nsIURI> uri;
-    mBindingDocument->GetDocumentURL(getter_AddRefs(uri));
+    nsIBindingManager *bindingManager = doc->GetBindingManager();
+    nsIURI *uri = mBindingDocument->GetDocumentURL();
     nsCAutoString str;
     uri->GetSpec(str);
     bindingManager->RemoveLoadingDocListener(str);
 
-    nsCOMPtr<nsIContent> root;
-    mBindingDocument->GetRootContent(getter_AddRefs(root));
-    if (!root) {
+    if (!mBindingDocument->GetRootContent()) {
       NS_ERROR("*** XBL doc with no root element! Something went horribly wrong! ***");
       return NS_ERROR_FAILURE;
     }
 
     // Put our doc info in the doc table.
     nsCOMPtr<nsIXBLDocumentInfo> info;
-    nsCOMPtr<nsIBindingManager> xblDocBindingManager;
-    mBindingDocument->GetBindingManager(getter_AddRefs(xblDocBindingManager));
+    nsIBindingManager *xblDocBindingManager = mBindingDocument->GetBindingManager();
     xblDocBindingManager->GetXBLDocumentInfo(str, getter_AddRefs(info));
     xblDocBindingManager->RemoveXBLDocumentInfo(info); // Break the self-imposed cycle.
     if (!info) {
@@ -444,8 +435,7 @@ nsXBLStreamListener::Load(nsIDOMEvent* aEvent)
     // don't construct or load their subdocs until they get a reflow.
     if (count > 0) {
       nsXBLBindingRequest* req = (nsXBLBindingRequest*)mBindingRequests.ElementAt(0);
-      nsCOMPtr<nsIDocument> document;
-      req->mBoundElement->GetDocument(getter_AddRefs(document));
+      nsIDocument* document = req->mBoundElement->GetDocument();
       if (document)
         document->FlushPendingNotifications();
     }
@@ -533,7 +523,7 @@ nsXBLService::~nsXBLService(void)
 // This function loads a particular XBL file and installs all of the bindings
 // onto the element.
 NS_IMETHODIMP
-nsXBLService::LoadBindings(nsIContent* aContent, const nsAString& aURL, PRBool aAugmentFlag,
+nsXBLService::LoadBindings(nsIContent* aContent, nsIURI* aURL, PRBool aAugmentFlag,
                            nsIXBLBinding** aBinding, PRBool* aResolveStyle) 
 { 
   *aBinding = nsnull;
@@ -541,15 +531,13 @@ nsXBLService::LoadBindings(nsIContent* aContent, const nsAString& aURL, PRBool a
 
   nsresult rv;
 
-  nsCOMPtr<nsIDocument> document;
-  aContent->GetDocument(getter_AddRefs(document));
+  nsCOMPtr<nsIDocument> document = aContent->GetDocument();
 
   // XXX document may be null if we're in the midst of paint suppression
   if (!document)
     return NS_OK;
 
-  nsCOMPtr<nsIBindingManager> bindingManager;
-  document->GetBindingManager(getter_AddRefs(bindingManager));
+  nsIBindingManager *bindingManager = document->GetBindingManager();
   
   nsCOMPtr<nsIXBLBinding> binding;
   bindingManager->GetBinding(aContent, getter_AddRefs(binding));
@@ -565,49 +553,49 @@ nsXBLService::LoadBindings(nsIContent* aContent, const nsAString& aURL, PRBool a
       }
       else {
         // See if the URIs match.
-        nsCAutoString uri;
-        styleBinding->GetBindingURI(uri);
-        if (uri.Equals(NS_ConvertUCS2toUTF8(aURL)))
+        nsCAutoString spec;
+        styleBinding->GetBindingURI(spec);
+        nsCOMPtr<nsIURI> uri;
+        // XXX This needs an origin charset.
+        NS_NewURI(getter_AddRefs(uri), spec.get());
+        PRBool equal;
+        if (NS_SUCCEEDED(uri->Equals(aURL, &equal)) && equal)
           return NS_OK;
-        else {
-          FlushStyleBindings(aContent);
-          binding = nsnull;
-        }
+        FlushStyleBindings(aContent);
+        binding = nsnull;
       }
     }
   }
 
   // Security check - remote pages can't load local bindings, except from chrome
-  nsCOMPtr<nsIURI> docURI;
-  rv = document->GetDocumentURL(getter_AddRefs(docURI));
-  NS_ENSURE_SUCCESS(rv, rv); //XXX can a document have no URI here?
+  nsIURI *docURI = document->GetDocumentURL();
   PRBool isChrome = PR_FALSE;
   rv = docURI->SchemeIs("chrome", &isChrome);
 
   if (NS_FAILED(rv) || !isChrome) {
-    nsCOMPtr<nsIURI> bindingURI;
-    rv = NS_NewURI(getter_AddRefs(bindingURI), aURL);
-    NS_ENSURE_SUCCESS(rv, rv);
-
     nsCOMPtr<nsIScriptSecurityManager> secMan(
       do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv));
     NS_ENSURE_SUCCESS(rv, rv);
 
-    rv = secMan->CheckLoadURI(docURI, bindingURI,
+    rv = secMan->CheckLoadURI(docURI, aURL,
                               nsIScriptSecurityManager::ALLOW_CHROME);
     if (NS_FAILED(rv))
       return rv;
   }
   nsCOMPtr<nsIXBLBinding> newBinding;
-  nsCAutoString url; url.AssignWithConversion(aURL);
-  if (NS_FAILED(rv = GetBinding(aContent, url, getter_AddRefs(newBinding)))) {
+  nsCAutoString spec;
+  aURL->GetSpec(spec);
+  if (NS_FAILED(rv = GetBinding(aContent, spec, getter_AddRefs(newBinding)))) {
     return rv;
   }
 
   if (!newBinding) {
-    nsCAutoString str( "Failed to locate XBL binding. XBL is now using id instead of name to reference bindings. Make sure you have switched over.  The invalid binding name is: ");
-    str.AppendWithConversion(aURL);
+#ifdef DEBUG
+    nsCAutoString spec;
+    aURL->GetSpec(spec);
+    nsCAutoString str(NS_LITERAL_CSTRING("Failed to locate XBL binding. XBL is now using id instead of name to reference bindings. Make sure you have switched over.  The invalid binding name is: ") + spec);
     NS_ERROR(str.get());
+#endif
     return NS_OK;
   }
 
@@ -664,15 +652,13 @@ nsXBLService::LoadBindings(nsIContent* aContent, const nsAString& aURL, PRBool a
 nsresult
 nsXBLService::FlushStyleBindings(nsIContent* aContent)
 {
-  nsCOMPtr<nsIDocument> document;
-  aContent->GetDocument(getter_AddRefs(document));
+  nsCOMPtr<nsIDocument> document = aContent->GetDocument();
 
   // XXX doc will be null if we're in the midst of paint suppression.
   if (! document)
     return NS_OK;
 
-  nsCOMPtr<nsIBindingManager> bindingManager;
-  document->GetBindingManager(getter_AddRefs(bindingManager));
+  nsIBindingManager *bindingManager = document->GetBindingManager();
   
   nsCOMPtr<nsIXBLBinding> binding;
   bindingManager->GetBinding(aContent, getter_AddRefs(binding));
@@ -683,8 +669,7 @@ nsXBLService::FlushStyleBindings(nsIContent* aContent)
 
     if (styleBinding) {
       // Clear out the script references.
-      nsCOMPtr<nsIDocument> document;
-      aContent->GetDocument(getter_AddRefs(document));
+      nsCOMPtr<nsIDocument> document = aContent->GetDocument();
       styleBinding->UnhookEventHandlers();
       styleBinding->ChangeDocument(document, nsnull);
     }
@@ -699,11 +684,9 @@ nsXBLService::FlushStyleBindings(nsIContent* aContent)
 NS_IMETHODIMP
 nsXBLService::ResolveTag(nsIContent* aContent, PRInt32* aNameSpaceID, nsIAtom** aResult)
 {
-  nsCOMPtr<nsIDocument> document;
-  aContent->GetDocument(getter_AddRefs(document));
+  nsIDocument* document = aContent->GetDocument();
   if (document) {
-    nsCOMPtr<nsIBindingManager> bindingManager;
-    document->GetBindingManager(getter_AddRefs(bindingManager));
+    nsIBindingManager *bindingManager = document->GetBindingManager();
   
     if (bindingManager)
       return bindingManager->ResolveTag(aContent, aNameSpaceID, aResult);
@@ -732,13 +715,9 @@ nsXBLService::GetXBLDocumentInfo(const nsCString& aURLStr, nsIContent* aBoundEle
 
   if (!*aResult) {
     // The second line of defense is the binding manager's document table.
-    nsCOMPtr<nsIDocument> boundDocument;
-    aBoundElement->GetDocument(getter_AddRefs(boundDocument));
-    if (boundDocument) {
-      nsCOMPtr<nsIBindingManager> bindingManager;
-      boundDocument->GetBindingManager(getter_AddRefs(bindingManager));
-      bindingManager->GetXBLDocumentInfo(aURLStr, aResult);
-    }
+    nsIDocument* boundDocument = aBoundElement->GetDocument();
+    if (boundDocument)
+      boundDocument->GetBindingManager()->GetXBLDocumentInfo(aURLStr, aResult);
   }
   return NS_OK;
 }
@@ -759,8 +738,7 @@ nsXBLService::AttachGlobalKeyHandler(nsIDOMEventReceiver* aReceiver)
   nsCOMPtr<nsIDOMEventReceiver> rec = aReceiver;
   nsCOMPtr<nsIContent> contentNode(do_QueryInterface(aReceiver));
   if (contentNode) {
-    nsCOMPtr<nsIDocument> doc;
-    contentNode->GetDocument(getter_AddRefs(doc));
+    nsCOMPtr<nsIDocument> doc = contentNode->GetDocument();
     if (doc)
       rec = do_QueryInterface(doc); // We're a XUL keyset. Attach to our document.
   }
@@ -897,8 +875,7 @@ NS_IMETHODIMP nsXBLService::GetBindingInternal(nsIContent* aBoundElement,
   uri.Right(ref, uri.Length() - (indx + 1));
   uri.Truncate(indx);
 
-  nsCOMPtr<nsIDocument> boundDocument;
-  aBoundElement->GetDocument(getter_AddRefs(boundDocument));
+  nsCOMPtr<nsIDocument> boundDocument = aBoundElement->GetDocument();
 
   nsCOMPtr<nsIXBLDocumentInfo> docInfo;
   LoadBindingDocumentInfo(aBoundElement, boundDocument, uri, ref, PR_FALSE, getter_AddRefs(docInfo));
@@ -1004,10 +981,8 @@ NS_IMETHODIMP nsXBLService::GetBindingInternal(nsIContent* aBoundElement,
         // Look up the prefix.
         // We have a base class binding. Load it right now.
         NS_ConvertUCS2toUTF8 urlCString(value);
-        nsCOMPtr<nsIURI> docURI;
-        doc->GetDocumentURL(getter_AddRefs(docURI));
         nsCAutoString urlStr;
-        docURI->Resolve(urlCString, urlStr);
+        doc->GetDocumentURL()->Resolve(urlCString, urlStr);
         if (NS_FAILED(GetBindingInternal(aBoundElement, urlStr, aPeekOnly, aIsReady, getter_AddRefs(baseBinding))))
           return NS_ERROR_FAILURE; // Binding not yet ready or an error occurred.
         if (!aPeekOnly) {
@@ -1057,10 +1032,10 @@ nsXBLService::LoadBindingDocumentInfo(nsIContent* aBoundElement, nsIDocument* aB
 
   if (!info) {
     // The second line of defense is the binding manager's document table.
-    nsCOMPtr<nsIBindingManager> bindingManager;
+    nsIBindingManager *bindingManager = nsnull;
 
     if (aBoundDocument) {
-      aBoundDocument->GetBindingManager(getter_AddRefs(bindingManager));
+      bindingManager = aBoundDocument->GetBindingManager();
       bindingManager->GetXBLDocumentInfo(aURLStr, getter_AddRefs(info));
     }
 
@@ -1110,8 +1085,7 @@ nsXBLService::LoadBindingDocumentInfo(nsIContent* aBoundElement, nsIDocument* aB
       FetchBindingDocument(aBoundElement, aBoundDocument, uri, aRef, aForceSyncLoad, getter_AddRefs(document));
    
       if (document) {
-        nsCOMPtr<nsIBindingManager> xblDocBindingManager;
-        document->GetBindingManager(getter_AddRefs(xblDocBindingManager));
+        nsIBindingManager *xblDocBindingManager = document->GetBindingManager();
         xblDocBindingManager->GetXBLDocumentInfo(aURLStr, getter_AddRefs(info));
         if (!info) {
           NS_ERROR("An XBL file is malformed.  Did you forget the XBL namespace on the bindings tag?");
@@ -1157,7 +1131,7 @@ nsXBLService::FetchBindingDocument(nsIContent* aBoundElement, nsIDocument* aBoun
   // Create an XML content sink and a parser. 
   nsCOMPtr<nsILoadGroup> loadGroup;
   if (aBoundDocument)
-    aBoundDocument->GetDocumentLoadGroup(getter_AddRefs(loadGroup));
+    loadGroup = aBoundDocument->GetDocumentLoadGroup();
 
   nsCOMPtr<nsIAtom> tagName;
   if (aBoundElement)
@@ -1202,9 +1176,12 @@ nsXBLService::FetchBindingDocument(nsIContent* aBoundElement, nsIDocument* aBoun
     rec->AddEventListener(NS_LITERAL_STRING("load"), (nsIDOMLoadListener*)xblListener, PR_FALSE);
 
     // Add ourselves to the list of loading docs.
-    nsCOMPtr<nsIBindingManager> bindingManager;
+    nsIBindingManager *bindingManager;
     if (aBoundDocument)
-      aBoundDocument->GetBindingManager(getter_AddRefs(bindingManager));
+      bindingManager = aBoundDocument->GetBindingManager();
+    else
+      bindingManager = nsnull;
+
     nsCAutoString uri;
     aURI->GetSpec(uri);
     if (bindingManager)
