@@ -76,9 +76,8 @@
 #include "nsICharsetAlias.h"
 #include "nsNetUtil.h"
 #include "nsDOMError.h"
-#include "nsScriptSecurityManager.h"
+#include "nsIScriptSecurityManager.h"
 #include "nsIPrincipal.h"
-#include "nsIAggregatePrincipal.h"
 #include "nsLayoutCID.h"
 #include "nsDOMAttribute.h"
 #include "nsGUIEvent.h"
@@ -236,17 +235,15 @@ nsXMLDocument::Init()
   return rv;
 }
 
-NS_IMETHODIMP 
+void
 nsXMLDocument::Reset(nsIChannel* aChannel, nsILoadGroup* aLoadGroup)
 {
-  nsresult result = nsDocument::Reset(aChannel, aLoadGroup);
-  if (NS_FAILED(result))
-    return result;
+  nsDocument::Reset(aChannel, aLoadGroup);
   nsCOMPtr<nsIURI> url;
   if (aChannel) {
-    result = aChannel->GetURI(getter_AddRefs(url));
+    nsresult result = aChannel->GetURI(getter_AddRefs(url));
     if (NS_FAILED(result))
-      return result;
+      return;
   }
 
   if (mAttrStyleSheet) {
@@ -256,11 +253,11 @@ nsXMLDocument::Reset(nsIChannel* aChannel, nsILoadGroup* aLoadGroup)
     mInlineStyleSheet->SetOwningDocument(nsnull);
   }
 
-  result = SetDefaultStylesheets(url);
+  SetDefaultStylesheets(url);
 
   mBaseTarget.Truncate();
 
-  return result;
+  mScriptContext = nsnull;
 }
 
 /////////////////////////////////////////////////////
@@ -330,21 +327,7 @@ nsXMLDocument::OnRedirect(nsIHttpChannel *aHttpChannel, nsIChannel *aNewChannel)
       return rv;
   }
 
-  nsCOMPtr<nsIPrincipal> newCodebase;
-  rv = secMan->GetCodebasePrincipal(newLocation,
-                                    getter_AddRefs(newCodebase));
-  if (NS_FAILED(rv))
-    return NS_ERROR_FAILURE;
-
-  nsCOMPtr<nsIAggregatePrincipal> agg = do_QueryInterface(mPrincipal, &rv);
-  NS_ASSERTION(NS_SUCCEEDED(rv), "Principal not an aggregate.");
-
-  if (NS_FAILED(rv))
-    return NS_ERROR_FAILURE;
-
-  rv = agg->SetCodebase(newCodebase);
-
-  return rv;
+  return secMan->GetCodebasePrincipal(newLocation, getter_AddRefs(mPrincipal));
 }
 
 NS_IMETHODIMP
@@ -400,7 +383,7 @@ nsXMLDocument::GetLoadGroup(nsILoadGroup **aLoadGroup)
       window->GetDocument(getter_AddRefs(domdoc));
       nsCOMPtr<nsIDocument> doc = do_QueryInterface(domdoc);
       if (doc) {
-        doc->GetDocumentLoadGroup(aLoadGroup);
+        *aLoadGroup = doc->GetDocumentLoadGroup().get(); // already_AddRefed
       }
     }
   }
@@ -419,54 +402,91 @@ nsXMLDocument::Load(const nsAString& aUrl, PRBool *aReturn)
   nsCOMPtr<nsIURI> uri;
   nsresult rv;
   
-  // Create a new URI
-  rv = NS_NewURI(getter_AddRefs(uri), aUrl, nsnull, mDocumentURL);
-  if (NS_FAILED(rv)) return rv;
-
-  // Get security manager, check to see if we're allowed to load this URI
-  nsCOMPtr<nsIScriptSecurityManager> secMan = 
-           do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
-  if (NS_FAILED(rv)) return rv;
-  rv = secMan->CheckConnect(nsnull, uri, "XMLDocument", "load");
-  if (NS_FAILED(rv)) {
-    // We need to return success here so that JS will get a proper exception
-    // thrown later. Native calls should always result in CheckConnect() succeeding,
-    // but in case JS calls C++ which calls this code the exception might be lost.
-    return NS_OK;
-  }
-
   // Partial Reset, need to restore principal for security reasons and
-  // event listener manager so that load listeners etc. will remain.
+  // event listener manager so that load listeners etc. will
+  // remain. This should be done before the security check is done to
+  // ensure that the document is reset even if the new document can't
+  // be loaded.
 
   nsCOMPtr<nsIPrincipal> principal(mPrincipal);
   nsCOMPtr<nsIEventListenerManager> elm(mListenerManager);
 
   Reset(nsnull, nsnull);
-  
+
   mPrincipal = principal;
   mListenerManager = elm;
-  
-  SetDocumentURL(uri);
-  SetBaseURL(uri);
 
-  // Store script context, if any, in case we encounter redirect
-  // (because we need it there)
+  nsCOMPtr<nsIScriptContext> callingContext;
+
   nsCOMPtr<nsIJSContextStack> stack =
     do_GetService("@mozilla.org/js/xpc/ContextStack;1");
   if (stack) {
     JSContext *cx;
     if (NS_SUCCEEDED(stack->Peek(&cx)) && cx) {
       nsContentUtils::GetDynamicScriptContext(cx,
-                                              getter_AddRefs(mScriptContext));
+                                              getter_AddRefs(callingContext));
     }
   }
 
+  nsIURI *baseURI = mDocumentURL;
+  nsCAutoString charset;
+
+  if (callingContext) {
+    nsCOMPtr<nsIScriptGlobalObject> sgo;
+    callingContext->GetGlobalObject(getter_AddRefs(sgo));
+
+    nsCOMPtr<nsIDOMWindow> window(do_QueryInterface(sgo));
+
+    if (window) {
+      nsCOMPtr<nsIDOMDocument> dom_doc;
+      window->GetDocument(getter_AddRefs(dom_doc));
+      nsCOMPtr<nsIDocument> doc(do_QueryInterface(dom_doc));
+
+      if (doc) {
+        baseURI = doc->GetBaseURL();
+        charset = doc->GetDocumentCharacterSet();
+      }
+    }
+  }
+
+  // Create a new URI
+  rv = NS_NewURI(getter_AddRefs(uri), aUrl, charset.get(), baseURI);
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
+  // Get security manager, check to see if we're allowed to load this URI
+  nsCOMPtr<nsIScriptSecurityManager> secMan = 
+           do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
+  rv = secMan->CheckConnect(nsnull, uri, "XMLDocument", "load");
+  if (NS_FAILED(rv)) {
+    // We need to return success here so that JS will get a proper
+    // exception thrown later. Native calls should always result in
+    // CheckConnect() succeeding, but in case JS calls C++ which calls
+    // this code the exception might be lost.
+    return NS_OK;
+  }
+
+  SetDocumentURL(uri);
+  SetBaseURL(uri);
+
+  // Store script context, if any, in case we encounter redirect
+  // (because we need it there)
+
+  mScriptContext = callingContext;
+
   // Find out if UniversalBrowserRead privileges are enabled - we will
   // need this in case of a redirect
   PRBool crossSiteAccessEnabled;
   rv = secMan->IsCapabilityEnabled("UniversalBrowserRead",
                                    &crossSiteAccessEnabled);
-  if (NS_FAILED(rv)) return rv;
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
 
   mCrossSiteAccessEnabled = crossSiteAccessEnabled;
 
@@ -481,7 +501,9 @@ nsXMLDocument::Load(const nsAString& aUrl, PRBool *aReturn)
   // which in turn keeps STOP button from becoming active  
   rv = NS_NewChannel(getter_AddRefs(channel), uri, nsnull, loadGroup, this, 
                      nsIRequest::LOAD_BACKGROUND);
-  if (NS_FAILED(rv)) return rv;
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
 
   // Set a principal for this document
   nsCOMPtr<nsISupports> channelOwner;
@@ -574,17 +596,14 @@ nsXMLDocument::StartDocumentLoad(const char* aCommand,
     // who puts the document on display to worry about enabling.
 
     // scripts
-    nsCOMPtr<nsIScriptLoader> loader;
-    nsresult rv = GetScriptLoader(getter_AddRefs(loader));
-    if (NS_FAILED(rv))
-      return rv;
+    nsIScriptLoader *loader = GetScriptLoader();
     if (loader) {
       loader->SetEnabled(PR_FALSE); // Do not load/process scripts when loading as data
     }
 
     // styles
     nsCOMPtr<nsICSSLoader> cssLoader;
-    rv = GetCSSLoader(*getter_AddRefs(cssLoader));
+    nsresult rv = GetCSSLoader(*getter_AddRefs(cssLoader));
     if (NS_FAILED(rv))
       return rv;
     if (cssLoader) {
@@ -661,7 +680,7 @@ nsXMLDocument::StartDocumentLoad(const char* aCommand,
   return NS_OK;
 }
 
-NS_IMETHODIMP 
+void
 nsXMLDocument::EndLoad()
 {
   mLoopingForSyncLoad = PR_FALSE;
@@ -685,7 +704,7 @@ nsXMLDocument::EndLoad()
 
     HandleDOMEvent(nsnull, &event, nsnull, NS_EVENT_FLAG_INIT, &status);
   }    
-  return nsDocument::EndLoad();  
+  nsDocument::EndLoad();  
 }
 
 NS_IMETHODIMP 
@@ -766,15 +785,13 @@ nsXMLDocument::InternalInsertStyleSheetAt(nsIStyleSheet* aSheet, PRInt32 aIndex)
   mStyleSheets.InsertObjectAt(aSheet, aIndex + mCatalogSheetCount + 1);
 }
 
-already_AddRefed<nsIStyleSheet>
-nsXMLDocument::InternalGetStyleSheetAt(PRInt32 aIndex)
+nsIStyleSheet*
+nsXMLDocument::InternalGetStyleSheetAt(PRInt32 aIndex) const
 {
   PRInt32 count = InternalGetNumberOfStyleSheets();
 
   if (aIndex >= 0 && aIndex < count) {
-    nsIStyleSheet* sheet = mStyleSheets[aIndex + mCatalogSheetCount + 1];
-    NS_ADDREF(sheet);
-    return sheet;
+    return mStyleSheets[aIndex + mCatalogSheetCount + 1];
   } else {
     NS_ERROR("Index out of range");
     return nsnull;
@@ -782,7 +799,7 @@ nsXMLDocument::InternalGetStyleSheetAt(PRInt32 aIndex)
 }
 
 PRInt32
-nsXMLDocument::InternalGetNumberOfStyleSheets()
+nsXMLDocument::InternalGetNumberOfStyleSheets() const
 {
   PRInt32 count = mStyleSheets.Count();
 
@@ -1024,13 +1041,10 @@ MatchElementId(nsIContent *aContent, const nsACString& aUTF8Id, const nsAString&
   }
   
   nsIContent *result = nsnull;
-  PRInt32 i, count;
+  PRUint32 i, count = aContent->GetChildCount();
 
-  aContent->ChildCount(count);
-  nsCOMPtr<nsIContent> child;
   for (i = 0; i < count && result == nsnull; i++) {
-    aContent->ChildAt(i, getter_AddRefs(child));
-    result = MatchElementId(child, aUTF8Id, aId);
+    result = MatchElementId(aContent->GetChildAt(i), aUTF8Id, aId);
   }  
 
   return result;
@@ -1089,18 +1103,16 @@ nsXMLDocument::SetDefaultStylesheets(nsIURI* aUrl)
   return rv;
 }
 
-NS_IMETHODIMP 
+void
 nsXMLDocument::SetBaseTarget(const nsAString &aBaseTarget)
 {
   mBaseTarget.Assign(aBaseTarget);
-  return NS_OK;
 }
 
-NS_IMETHODIMP 
-nsXMLDocument::GetBaseTarget(nsAString &aBaseTarget)
+void
+nsXMLDocument::GetBaseTarget(nsAString &aBaseTarget) const
 {
   aBaseTarget.Assign(mBaseTarget);
-  return NS_OK;
 }
 
 NS_IMETHODIMP
