@@ -48,10 +48,7 @@
 
 */
 
-#include "nsDOMCID.h"
 #include "nsIDOMNode.h"
-#include "nsIServiceManager.h"
-#include "nsISupportsArray.h"
 #include "nsRDFDOMNodeList.h"
 #include "nsContentUtils.h"
 
@@ -60,54 +57,11 @@
 
 
 nsRDFDOMNodeList::nsRDFDOMNodeList(void)
-    : //mInner(nsnull), Not being used?
-      mElements(nsnull)
 {
 }
 
 nsRDFDOMNodeList::~nsRDFDOMNodeList(void)
 {
-#ifdef DEBUG_REFS
-    --gInstanceCount;
-    fprintf(stdout, "%d - RDF: nsRDFDOMNodeList\n", gInstanceCount);
-#endif
-
-    NS_IF_RELEASE(mElements);
-    //delete mInner; Not being used?
-}
-
-nsresult
-nsRDFDOMNodeList::Create(nsRDFDOMNodeList** aResult)
-{
-    nsRDFDOMNodeList* list = new nsRDFDOMNodeList();
-    if (! list)
-        return NS_ERROR_OUT_OF_MEMORY;
-
-    nsresult rv;
-    if (NS_FAILED(rv = list->Init())) {
-        delete list;
-        return rv;
-    }
-
-    NS_ADDREF(list);
-    *aResult = list;
-    return NS_OK;
-}
-
-nsresult
-nsRDFDOMNodeList::CreateWithArray(nsISupportsArray* aArray,
-                                  nsRDFDOMNodeList** aResult)
-{
-    nsRDFDOMNodeList* list = new nsRDFDOMNodeList();
-    if (! list)
-        return NS_ERROR_OUT_OF_MEMORY;
-
-    list->mElements = aArray;
-    NS_IF_ADDREF(aArray);
-
-    NS_ADDREF(list);
-    *aResult = list;
-    return NS_OK;
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -137,10 +91,7 @@ nsRDFDOMNodeList::GetLength(PRUint32* aLength)
     if (! aLength)
         return NS_ERROR_NULL_POINTER;
 
-    PRUint32 cnt;
-    nsresult rv = mElements->Count(&cnt);
-    if (NS_FAILED(rv)) return rv;
-    *aLength = cnt;
+    *aLength = mElements.Count();
     return NS_OK;
 }
 
@@ -148,11 +99,13 @@ nsRDFDOMNodeList::GetLength(PRUint32* aLength)
 NS_IMETHODIMP
 nsRDFDOMNodeList::Item(PRUint32 aIndex, nsIDOMNode** aReturn)
 {
-    // It's ok to access past the end of the array here, if we do that
-    // we simply return nsnull, as per the DOM spec.
+    if (aIndex >= (PRUint32) mElements.Count()) {
+        // We simply return nsnull, as per the DOM spec.
+        *aReturn = nsnull;
+        return NS_OK;
+    }
 
-    // Cast is okay because we're in a closed system.
-    *aReturn = (nsIDOMNode*) mElements->ElementAt(aIndex);
+    NS_ADDREF(*aReturn = mElements[aIndex]);
     return NS_OK;
 }
 
@@ -160,19 +113,6 @@ nsRDFDOMNodeList::Item(PRUint32 aIndex, nsIDOMNode** aReturn)
 ////////////////////////////////////////////////////////////////////////
 // Implementation methods
 
-nsresult
-nsRDFDOMNodeList::Init(void)
-{
-    nsresult rv;
-    if (NS_FAILED(rv = NS_NewISupportsArray(&mElements))) {
-        NS_ERROR("unable to create elements array");
-        return rv;
-    }
-
-    return NS_OK;
-}
-
-
 NS_IMETHODIMP
 nsRDFDOMNodeList::AppendNode(nsIDOMNode* aNode)
 {
@@ -180,7 +120,7 @@ nsRDFDOMNodeList::AppendNode(nsIDOMNode* aNode)
     if (! aNode)
         return NS_ERROR_NULL_POINTER;
 
-    return mElements->AppendElement(aNode);
+    return mElements.AppendObject(aNode);
 }
 
 NS_IMETHODIMP
@@ -190,5 +130,5 @@ nsRDFDOMNodeList::RemoveNode(nsIDOMNode* aNode)
     if (! aNode)
         return NS_ERROR_NULL_POINTER;
 
-    return mElements->RemoveElement(aNode);
+    return mElements.RemoveObject(aNode);
 }
