@@ -14,7 +14,7 @@
  *
  * The Original Code is Mozilla Communicator client code.
  *
- * The Initial Developer of the Original Code is 
+ * The Initial Developer of the Original Code is
  * Netscape Communications Corporation.
  * Portions created by the Initial Developer are Copyright (C) 1998
  * the Initial Developer. All Rights Reserved.
@@ -91,7 +91,7 @@ class nsRDFDOMNodeList;
 class nsString;
 class nsXULAttributes;
 class nsVoidArray;
-class nsIWebShell;
+class nsIDocShell;
 
 class nsIObjectInputStream;
 class nsIObjectOutputStream;
@@ -195,7 +195,7 @@ public:
     Type                     mType;
 
     PRInt32                  mRefCnt;
-          
+
     virtual ~nsXULPrototypeNode() {}
     virtual nsresult Serialize(nsIObjectOutputStream* aStream,
                                nsIScriptContext* aContext,
@@ -206,11 +206,11 @@ public:
                                  nsISupportsArray* aNodeInfos) = 0;
 
     void AddRef() { ++mRefCnt; };
-    void Release() 
-    { 
-        --mRefCnt; 
-        if (mRefCnt == 0) 
-            delete this; 
+    void Release()
+    {
+        --mRefCnt;
+        if (mRefCnt == 0)
+            delete this;
     };
     virtual void ReleaseSubtree() { Release(); };
 
@@ -242,7 +242,7 @@ public:
         delete[] mChildren;
     }
 
-    virtual void ReleaseSubtree() 
+    virtual void ReleaseSubtree()
     {
       if (mChildren) {
         for (PRInt32 i = mNumChildren-1; i >= 0; i--) {
@@ -263,12 +263,12 @@ public:
                                  nsIURI* aDocumentURI,
                                  nsISupportsArray* aNodeInfos);
 
-    PRInt32                  mNumChildren;
+    PRUint32                 mNumChildren;
     nsXULPrototypeNode**     mChildren;           // [OWNER]
 
     nsCOMPtr<nsINodeInfo>    mNodeInfo;           // [OWNER]
 
-    PRInt32                  mNumAttributes;
+    PRUint32                 mNumAttributes;
     nsXULPrototypeAttribute* mAttributes;         // [OWNER]
 
     nsCOMPtr<nsIStyleRule>   mInlineStyleRule;    // [OWNER]
@@ -277,8 +277,8 @@ public:
     nsresult GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, nsAString& aValue);
 
 
-    static void ReleaseGlobals() 
-    { 
+    static void ReleaseGlobals()
+    {
         NS_IF_RELEASE(sCSSParser);
     }
 
@@ -305,12 +305,14 @@ public:
     virtual nsresult Serialize(nsIObjectOutputStream* aStream,
                                nsIScriptContext* aContext,
                                nsISupportsArray* aNodeInfos);
+    nsresult SerializeOutOfLine(nsIObjectOutputStream* aStream,
+                                nsIScriptContext* aContext);
     virtual nsresult Deserialize(nsIObjectInputStream* aStream,
                                  nsIScriptContext* aContext,
                                  nsIURI* aDocumentURI,
                                  nsISupportsArray* aNodeInfos);
-    virtual nsresult DeserializeOutOfLineScript(nsIObjectInputStream* aInput,
-                                                nsIScriptContext* aContext);
+    nsresult DeserializeOutOfLine(nsIObjectInputStream* aInput,
+                                  nsIScriptContext* aContext);
 
     nsresult Compile(const PRUnichar* aText, PRInt32 aTextLength,
                      nsIURI* aURI, PRUint16 aLineNo,
@@ -325,9 +327,9 @@ public:
     JSObject*                mJSObject;
     const char*              mLangVersion;
 
-    static void ReleaseGlobals() 
-    { 
-        NS_IF_RELEASE(sXULPrototypeCache); 
+    static void ReleaseGlobals()
+    {
+        NS_IF_RELEASE(sXULPrototypeCache);
     }
 
 protected:
@@ -402,38 +404,40 @@ public:
 
     // nsISupports
     NS_DECL_ISUPPORTS
-       
+
     // nsIContent (from nsIStyledContent)
-    NS_IMETHOD GetDocument(nsIDocument*& aResult) const;
+    NS_IMETHOD_(nsIDocument*) GetDocument() const;
     NS_IMETHOD SetDocument(nsIDocument* aDocument, PRBool aDeep, PRBool aCompileEventHandlers);
-    NS_IMETHOD GetParent(nsIContent*& aResult) const;
+    NS_IMETHOD_(nsIContent*) GetParent() const;
     NS_IMETHOD SetParent(nsIContent* aParent);
     NS_IMETHOD_(PRBool) IsNativeAnonymous() const;
     NS_IMETHOD_(void) SetNativeAnonymous(PRBool aAnonymous);
-    NS_IMETHOD CanContainChildren(PRBool& aResult) const;
-    NS_IMETHOD ChildCount(PRInt32& aResult) const;
-    NS_IMETHOD ChildAt(PRInt32 aIndex, nsIContent*& aResult) const;
-    NS_IMETHOD IndexOf(nsIContent* aPossibleChild, PRInt32& aResult) const;
-    NS_IMETHOD InsertChildAt(nsIContent* aKid, PRInt32 aIndex, PRBool aNotify,
+    NS_IMETHOD_(PRBool) CanContainChildren() const;
+    NS_IMETHOD_(PRUint32) GetChildCount() const;
+    NS_IMETHOD_(nsIContent *) GetChildAt(PRUint32 aIndex) const;
+    NS_IMETHOD_(PRInt32) IndexOf(nsIContent* aPossibleChild) const;
+    NS_IMETHOD InsertChildAt(nsIContent* aKid, PRUint32 aIndex, PRBool aNotify,
                              PRBool aDeepSetDocument);
-    NS_IMETHOD ReplaceChildAt(nsIContent* aKid, PRInt32 aIndex,
+    NS_IMETHOD ReplaceChildAt(nsIContent* aKid, PRUint32 aIndex,
                               PRBool aNotify, PRBool aDeepSetDocument);
     NS_IMETHOD AppendChildTo(nsIContent* aKid, PRBool aNotify,
                              PRBool aDeepSetDocument);
-    NS_IMETHOD RemoveChildAt(PRInt32 aIndex, PRBool aNotify);
-    NS_IMETHOD GetNameSpaceID(PRInt32& aNameSpeceID) const;
-    NS_IMETHOD GetTag(nsIAtom*& aResult) const;
-    NS_IMETHOD GetNodeInfo(nsINodeInfo*& aResult) const;
-    NS_IMETHOD NormalizeAttrString(const nsAString& aStr, nsINodeInfo*& aNodeInfo);
+    NS_IMETHOD RemoveChildAt(PRUint32 aIndex, PRBool aNotify);
+    NS_IMETHOD GetNameSpaceID(PRInt32* aNameSpeceID) const;
+    NS_IMETHOD GetTag(nsIAtom** aResult) const;
+    NS_IMETHOD_(nsINodeInfo *) GetNodeInfo() const;
+    NS_IMETHOD_(nsIAtom *) GetIDAttributeName() const;
+    NS_IMETHOD_(nsIAtom *) GetClassAttributeName() const;
+    NS_IMETHOD NormalizeAttrString(const nsAString& aStr, nsINodeInfo** aNodeInfo);
     NS_IMETHOD SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, const nsAString& aValue, PRBool aNotify);
     NS_IMETHOD SetAttr(nsINodeInfo *aNodeInfo, const nsAString& aValue, PRBool aNotify);
     NS_IMETHOD GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, nsAString& aResult) const;
-    NS_IMETHOD GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, nsIAtom*& aPrefix, nsAString& aResult) const;
+    NS_IMETHOD GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, nsIAtom** aPrefix, nsAString& aResult) const;
     NS_IMETHOD_(PRBool) HasAttr(PRInt32 aNameSpaceID, nsIAtom* aName) const;
     NS_IMETHOD UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, PRBool aNotify);
-    NS_IMETHOD GetAttrNameAt(PRInt32 aIndex, PRInt32& aNameSpaceID, 
-                             nsIAtom*& aName, nsIAtom*& aPrefix) const;
-    NS_IMETHOD GetAttrCount(PRInt32& aResult) const;
+    NS_IMETHOD GetAttrNameAt(PRUint32 aIndex, PRInt32* aNameSpaceID,
+                             nsIAtom** aName, nsIAtom** aPrefix) const;
+    NS_IMETHOD_(PRUint32) GetAttrCount() const;
 #ifdef DEBUG
     NS_IMETHOD List(FILE* out, PRInt32 aIndent) const;
     NS_IMETHOD DumpContent(FILE* out, PRInt32 aIndent,PRBool aDumpAll) const { return NS_OK; }
@@ -449,41 +453,42 @@ public:
     NS_IMETHOD SetContentID(PRUint32 aID);
 
     NS_IMETHOD RangeAdd(nsIDOMRange* aRange);
-    NS_IMETHOD RangeRemove(nsIDOMRange* aRange); 
-    NS_IMETHOD GetRangeList(nsVoidArray*& aResult) const;
+    NS_IMETHOD RangeRemove(nsIDOMRange* aRange);
+    NS_IMETHOD GetRangeList(nsVoidArray** aResult) const;
     NS_IMETHOD SetFocus(nsIPresContext* aPresContext);
     NS_IMETHOD RemoveFocus(nsIPresContext* aPresContext);
 
-    NS_IMETHOD GetBindingParent(nsIContent** aContent);
+    NS_IMETHOD_(nsIContent*) GetBindingParent() const;
     NS_IMETHOD SetBindingParent(nsIContent* aParent);
     NS_IMETHOD_(PRBool) IsContentOfType(PRUint32 aFlags);
+    NS_IMETHOD GetBaseURL(nsIURI** aURI) const;
     NS_IMETHOD GetListenerManager(nsIEventListenerManager** aResult);
 
     // nsIXMLContent
-    NS_IMETHOD MaybeTriggerAutoLink(nsIWebShell *aShell);
-    NS_IMETHOD GetXMLBaseURI(nsIURI **aURI);
+    NS_IMETHOD MaybeTriggerAutoLink(nsIDocShell *aShell);
 
     // nsIStyledContent
-    NS_IMETHOD GetID(nsIAtom*& aResult) const;
+    NS_IMETHOD GetID(nsIAtom** aResult) const;
     NS_IMETHOD GetClasses(nsVoidArray& aArray) const;
     NS_IMETHOD_(PRBool) HasClass(nsIAtom* aClass, PRBool aCaseSensitive) const;
 
     NS_IMETHOD WalkContentStyleRules(nsRuleWalker* aRuleWalker);
     NS_IMETHOD GetInlineStyleRule(nsIStyleRule** aStyleRule);
-    NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
-                                        nsChangeHint& aHint) const;
-
+    NS_IMETHOD GetAttributeChangeHint(const nsIAtom* aAttribute,
+                                      PRInt32 aModType,
+                                      nsChangeHint& aHint) const;
+    NS_IMETHOD_(PRBool) HasAttributeDependentStyle(const nsIAtom* aAttribute) const;
 
     // nsIXULContent
-    NS_IMETHOD PeekChildCount(PRInt32& aCount) const;
+    NS_IMETHOD_(PRUint32) PeekChildCount() const;
     NS_IMETHOD SetLazyState(LazyState aFlags);
     NS_IMETHOD ClearLazyState(LazyState aFlags);
     NS_IMETHOD GetLazyState(LazyState aFlag, PRBool& aValue);
     NS_IMETHOD AddScriptEventListener(nsIAtom* aName, const nsAString& aValue);
-    
+
     // nsIDOMNode (from nsIDOMElement)
     NS_DECL_NSIDOMNODE
-  
+
     // nsIDOMElement
     NS_DECL_NSIDOMELEMENT
 
@@ -495,6 +500,8 @@ public:
                                    void* aTarget,
                                    nsIAtom *aName,
                                    const nsAString& aBody,
+                                   const char* aURL,
+                                   PRUint32 aLineNo,
                                    void** aHandler);
     NS_IMETHOD GetCompiledEventHandler(nsIAtom *aName, void** aHandler);
 
@@ -641,7 +648,7 @@ protected:
     nsresult AddListenerFor(nsINodeInfo *aNodeInfo,
                             PRBool aCompileEventHandlers);
 
-    
+
     nsresult HideWindowChrome(PRBool aShouldHide);
 
 protected:
