@@ -124,9 +124,7 @@ void
 nsXULCommandDispatcher::EnsureFocusController()
 {
   if (!mFocusController) {
-    nsCOMPtr<nsIScriptGlobalObject> global;
-    mDocument->GetScriptGlobalObject(getter_AddRefs(global));
-    nsCOMPtr<nsPIDOMWindow> win(do_QueryInterface(global));
+    nsCOMPtr<nsPIDOMWindow> win(do_QueryInterface(mDocument->GetScriptGlobalObject()));
   
     // An inelegant way to retrieve this to be sure, but we are
     // guaranteed that the focus controller outlives us, so it
@@ -351,10 +349,7 @@ nsXULCommandDispatcher::UpdateCommands(const nsAString& aEventName)
     if (! content)
       return NS_ERROR_UNEXPECTED;
 
-    nsCOMPtr<nsIDocument> document;
-    rv = content->GetDocument(*getter_AddRefs(document));
-    NS_ASSERTION(NS_SUCCEEDED(rv), "unable to get document");
-    if (NS_FAILED(rv)) return rv;
+    nsCOMPtr<nsIDocument> document = content->GetDocument();
 
     NS_ASSERTION(document != nsnull, "element has no document");
     if (! document)
@@ -369,13 +364,10 @@ nsXULCommandDispatcher::UpdateCommands(const nsAString& aEventName)
             aeventnameC.get()));
 #endif
 
-    PRInt32 count = document->GetNumberOfShells();
-    for (PRInt32 i = 0; i < count; i++) {
-      nsCOMPtr<nsIPresShell> shell;
-      document->GetShellAt(i, getter_AddRefs(shell));
-      if (! shell)
-          continue;
-      
+    PRUint32 count = document->GetNumberOfShells();
+    for (PRUint32 i = 0; i < count; i++) {
+      nsIPresShell *shell = document->GetShellAt(i);
+
       // Retrieve the context in which our DOM event will fire.
       nsCOMPtr<nsIPresContext> context;
       rv = shell->GetPresContext(getter_AddRefs(context));
@@ -434,3 +426,17 @@ nsXULCommandDispatcher::GetControllerForCommand(const char *aCommand, nsIControl
   return mFocusController->GetControllerForCommand(aCommand, _retval);
 }
 
+NS_IMETHODIMP
+nsXULCommandDispatcher::GetSuppressFocusScroll(PRBool* aSuppressFocusScroll)
+{
+  EnsureFocusController();
+  return mFocusController->GetSuppressFocusScroll(aSuppressFocusScroll);
+}
+
+NS_IMETHODIMP
+nsXULCommandDispatcher::SetSuppressFocusScroll(PRBool aSuppressFocusScroll)
+{
+  EnsureFocusController();
+  return mFocusController->SetSuppressFocusScroll(aSuppressFocusScroll);
+}
+
