@@ -109,7 +109,6 @@
 #include "nsHTMLAtoms.h"
 
 static const char kNameSpaceSeparator = ':';
-static const char kNameSpaceDef[] = "xmlns";
 
 #ifdef PR_LOGGING
 static PRLogModuleInfo* gLog;
@@ -132,7 +131,7 @@ public:
 
     // nsIContentSink
     NS_IMETHOD WillBuildModel(void);
-    NS_IMETHOD DidBuildModel(PRInt32 aQualityLevel);
+    NS_IMETHOD DidBuildModel(void);
     NS_IMETHOD WillInterrupt(void);
     NS_IMETHOD WillResume(void);
     NS_IMETHOD SetParser(nsIParser* aParser);  
@@ -454,25 +453,8 @@ XULContentSinkImpl::WillBuildModel(void)
 }
 
 NS_IMETHODIMP 
-XULContentSinkImpl::DidBuildModel(PRInt32 aQualityLevel)
+XULContentSinkImpl::DidBuildModel(void)
 {
-#if FIXME
-    // XXX this is silly; who cares?
-    PRInt32 i, ns = mDocument->GetNumberOfShells();
-    for (i = 0; i < ns; i++) {
-        nsCOMPtr<nsIPresShell> shell;
-        mDocument->GetShellAt(i, getter_AddRefs(shell));
-        if (nsnull != shell) {
-            nsIViewManager* vm;
-            shell->GetViewManager(&vm);
-            if(vm) {
-                vm->SetQuality(nsContentQuality(aQualityLevel));
-            }
-            NS_RELEASE(vm);
-        }
-    }
-#endif
-
     nsCOMPtr<nsIDocument> doc = do_QueryReferent(mDocument);
     if (doc) {
         doc->EndLoad();
@@ -563,7 +545,7 @@ XULContentSinkImpl::ProcessStyleLink(nsIContent* aElement,
             return NS_ERROR_FAILURE; // doc went away!
 
         PRBool doneLoading;
-        rv = mCSSLoader->LoadStyleLink(aElement, url, aTitle, aMedia, kNameSpaceID_Unknown,
+        rv = mCSSLoader->LoadStyleLink(aElement, url, aTitle, aMedia,
                                        ((blockParser) ? mParser : nsnull),
                                        doneLoading, nsnull);
         if (NS_SUCCEEDED(rv) && blockParser && (! doneLoading)) {
@@ -579,7 +561,7 @@ XULContentSinkImpl::SetDocumentCharset(nsACString& aCharset)
 {
     nsCOMPtr<nsIDocument> doc = do_QueryReferent(mDocument);
     if (doc) {
-        return doc->SetDocumentCharacterSet(aCharset);
+        doc->SetDocumentCharacterSet(aCharset);
     }
   
     return NS_OK;
@@ -599,7 +581,7 @@ XULContentSinkImpl::Init(nsIDocument* aDocument, nsIXULPrototypeDocument* aProto
     
     nsresult rv;
 
-    mDocument    = getter_AddRefs(NS_GetWeakReference(aDocument));
+    mDocument    = do_GetWeakReference(aDocument);
     mPrototype   = aPrototype;
 
     rv = mPrototype->GetURI(getter_AddRefs(mDocumentURL));
@@ -785,12 +767,17 @@ NS_IMETHODIMP
 XULContentSinkImpl::HandleStartElement(const PRUnichar *aName, 
                                        const PRUnichar **aAtts,
                                        PRUint32 aAttsCount, 
-                                       PRUint32 aIndex, 
+                                       PRInt32 aIndex, 
                                        PRUint32 aLineNumber)
 { 
   // XXX Hopefully the parser will flag this before we get here. If
   // we're in the epilog, there should be no new elements
   NS_PRECONDITION(mState != eInEpilog, "tag in XUL doc epilog");
+  NS_PRECONDITION(aIndex >= -1, "Bogus aIndex");
+  NS_PRECONDITION(aAttsCount % 2 == 0, "incorrect aAttsCount");
+  // Adjust aAttsCount so it's the actual number of attributes
+  aAttsCount /= 2;
+  
   if (mState == eInEpilog)
       return NS_ERROR_UNEXPECTED;
 
@@ -830,6 +817,15 @@ XULContentSinkImpl::HandleStartElement(const PRUnichar *aName,
       break;
   }
 
+  // Set the ID attribute atom on the node info object for this node
+  if (aIndex != -1 && NS_SUCCEEDED(rv)) {
+    nsCOMPtr<nsIAtom> IDAttr = do_GetAtom(aAtts[aIndex]);
+
+    if (IDAttr) {
+      nodeInfo->SetIDAttributeAtom(IDAttr);
+    }
+  }
+
   return rv;
 }
 
@@ -957,11 +953,10 @@ NS_IMETHODIMP
 XULContentSinkImpl::HandleCharacterData(const PRUnichar *aData, 
                                         PRUint32 aLength)
 {
-  nsresult result = NS_OK;
-  if (aData) {
-    result = result = AddText(aData,aLength);
+  if (aData && mState != eInProlog && mState != eInEpilog) {
+    return AddText(aData, aLength);
   }
-  return result;
+  return NS_OK;
 }
 
 NS_IMETHODIMP 
@@ -1087,16 +1082,16 @@ XULContentSinkImpl::ReportError(const PRUnichar* aErrorText,
   NS_NAMED_LITERAL_STRING(name, "xmlns");
   NS_NAMED_LITERAL_STRING(value, "http://www.mozilla.org/newlayout/xml/parsererror.xml");
 
-  const PRUnichar* atts[] = {name.get(), value.get(), nsnull};;
+  const PRUnichar* atts[] = {name.get(), value.get(), nsnull};
     
-  rv = HandleStartElement(NS_LITERAL_STRING("parsererror").get(), atts, 1, -1, -1);
+  rv = HandleStartElement(NS_LITERAL_STRING("parsererror").get(), atts, 2, -1, 0);
   NS_ENSURE_SUCCESS(rv,rv);
 
   rv = HandleCharacterData(aErrorText, nsCRT::strlen(aErrorText));
   NS_ENSURE_SUCCESS(rv,rv);  
   
   const PRUnichar* noAtts[] = {0, 0};
-  rv = HandleStartElement(NS_LITERAL_STRING("sourcetext").get(), noAtts, 0, -1, -1);
+  rv = HandleStartElement(NS_LITERAL_STRING("sourcetext").get(), noAtts, 0, -1, 0);
   NS_ENSURE_SUCCESS(rv,rv);
   
   rv = HandleCharacterData(aSourceText, nsCRT::strlen(aSourceText));
@@ -1132,26 +1127,21 @@ XULContentSinkImpl::PushNameSpacesFrom(const PRUnichar** aAttributes)
     }
 
     static const NS_NAMED_LITERAL_STRING(kNameSpaceDef, "xmlns");
-    static const PRUint32 xmlns_len = 5; // kNameSpaceDef.Length();
-    NS_ASSERTION(kNameSpaceDef.Length() == xmlns_len,
-                 "xmlns_len incorrectly set!");
+    static const PRUint32 xmlns_len = kNameSpaceDef.Length();
 
     for (; *aAttributes; aAttributes += 2) {
         nsDependentString key(aAttributes[0]);
 
         // Look for "xmlns" at the start of the attribute name
 
-        PRUint32 key_len = key.Length();
-
-        if (key_len >= xmlns_len &&
-            Substring(key, 0, xmlns_len).Equals(kNameSpaceDef)) {
+        if (StringBeginsWith(key, kNameSpaceDef)) {
             nsCOMPtr<nsIAtom> prefixAtom;
 
-            // If key_len > xmlns_len we have a xmlns:foo type attribute,
+            // If key.Length() > xmlns_len we have a xmlns:foo type attribute,
             // extract the prefix. If not, we have a xmlns attribute in
             // which case there is no prefix.
 
-            if (key_len > xmlns_len) {
+            if (key.Length() > xmlns_len) {
                 nsDependentString::const_iterator start, end;
 
                 key.BeginReading(start);
@@ -1378,12 +1368,10 @@ XULContentSinkImpl::OpenScript(const PRUnichar** aAttributes,
   nsAutoString src;
   while (*aAttributes) {
       const nsDependentString key(aAttributes[0]);
-      if (key.Equals(NS_LITERAL_STRING("src"),
-                     nsCaseInsensitiveStringComparator())) {
+      if (key.Equals(NS_LITERAL_STRING("src"))) {
           src.Assign(aAttributes[1]);
       }
-      else if (key.Equals(NS_LITERAL_STRING("type"),
-                          nsCaseInsensitiveStringComparator())) {
+      else if (key.Equals(NS_LITERAL_STRING("type"))) {
           nsAutoString  type(aAttributes[1]);
           nsAutoString  mimeType;
           nsAutoString  params;
@@ -1409,8 +1397,7 @@ XULContentSinkImpl::OpenScript(const PRUnichar** aAttributes,
               jsVersionString = JS_VersionToString(jsVersion);
           }
       }
-      else if (key.Equals(NS_LITERAL_STRING("language"),
-                          nsCaseInsensitiveStringComparator())) {
+      else if (key.Equals(NS_LITERAL_STRING("language"))) {
         nsAutoString  lang(aAttributes[1]);
         isJavaScript = nsParserUtils::IsJavaScriptLanguage(lang, &jsVersionString);
       }
@@ -1453,8 +1440,7 @@ XULContentSinkImpl::OpenScript(const PRUnichar** aAttributes,
           // where it's already there.
           nsCOMPtr<nsIDocument> doc(do_QueryReferent(mDocument));
           if (doc) {
-              nsCOMPtr<nsIScriptGlobalObject> globalObject;
-              doc->GetScriptGlobalObject(getter_AddRefs(globalObject));
+              nsIScriptGlobalObject* globalObject = doc->GetScriptGlobalObject();
               if (globalObject) {
                   nsCOMPtr<nsIScriptContext> scriptContext;
                   globalObject->GetContext(getter_AddRefs(scriptContext));
