@@ -68,6 +68,7 @@
 #include "nsIScriptError.h"
 #include "nsIDOMScriptObjectFactory.h"
 #include "nsDOMCID.h"
+#include "nsArray.h"
 
 
 static NS_DEFINE_CID(kDOMScriptObjectFactoryCID,
@@ -146,7 +147,7 @@ public:
     NS_IMETHOD GetHeaderData(nsIAtom* aField, nsAString& aData) const;
     NS_IMETHOD SetHeaderData(nsIAtom* aField, const nsAString& aData);
 
-    NS_IMETHOD GetDocumentPrincipal(nsIPrincipal** aResult);
+    virtual nsIPrincipal* GetDocumentPrincipal();
     NS_IMETHOD SetDocumentPrincipal(nsIPrincipal* aPrincipal);
 
     NS_IMETHOD AwaitLoadDone(nsIXULDocument* aDocument, PRBool* aResult);
@@ -180,8 +181,19 @@ protected:
     friend NS_IMETHODIMP
     NS_NewXULPrototypeDocument(nsISupports* aOuter, REFNSIID aIID,
                                void** aResult);
+
+    nsresult NewXULPDGlobalObject(nsIScriptGlobalObject** aResult);
+
+    static nsIPrincipal* gSystemPrincipal;
+    static nsIScriptGlobalObject* gSystemGlobal;
+    static PRUint32 gRefCnt;
+
+    friend class nsXULPDGlobalObject;
 };
 
+nsIPrincipal* nsXULPrototypeDocument::gSystemPrincipal;
+nsIScriptGlobalObject* nsXULPrototypeDocument::gSystemGlobal;
+PRUint32 nsXULPrototypeDocument::gRefCnt;
 
 
 void PR_CALLBACK
@@ -229,6 +241,7 @@ nsXULPrototypeDocument::nsXULPrototypeDocument()
       mGlobalObject(nsnull),
       mLoaded(PR_FALSE)
 {
+    ++gRefCnt;
 }
 
 
@@ -261,6 +274,11 @@ nsXULPrototypeDocument::~nsXULPrototypeDocument()
     
     if (mRoot)
         mRoot->ReleaseSubtree();
+
+    if (--gRefCnt == 0) {
+        NS_IF_RELEASE(gSystemPrincipal);
+        NS_IF_RELEASE(gSystemGlobal);
+    }
 }
 
 NS_IMPL_ISUPPORTS3(nsXULPrototypeDocument,
@@ -293,6 +311,41 @@ NS_NewXULPrototypeDocument(nsISupports* aOuter, REFNSIID aIID, void** aResult)
     return rv;
 }
 
+// Helper method that shares a system global among all prototype documents
+// that have the system principal as their security principal.   Called by
+// nsXULPrototypeDocument::Read and nsXULPDGlobalObject::GetGlobalObject.
+// This method greatly reduces the number of nsXULPDGlobalObjects and their
+// nsIScriptContexts in apps that load many XUL documents via chrome: URLs.
+
+nsresult
+nsXULPrototypeDocument::NewXULPDGlobalObject(nsIScriptGlobalObject** aResult)
+{
+    nsIPrincipal *principal = GetDocumentPrincipal();
+    if (!principal)
+        return NS_ERROR_FAILURE;
+
+    // Now that GetDocumentPrincipal has succeeded, we can safely compare its
+    // result to gSystemPrincipal, in order to create gSystemGlobal if the two
+    // pointers are equal.  Thus, gSystemGlobal implies gSystemPrincipal.
+    nsCOMPtr<nsIScriptGlobalObject> global;
+    if (principal == gSystemPrincipal) {
+        if (!gSystemGlobal) {
+            gSystemGlobal = new nsXULPDGlobalObject();
+            if (! gSystemGlobal)
+                return NS_ERROR_OUT_OF_MEMORY;
+            NS_ADDREF(gSystemGlobal);
+        }
+        global = gSystemGlobal;
+    } else {
+        global = new nsXULPDGlobalObject();
+        if (! global)
+            return NS_ERROR_OUT_OF_MEMORY;
+        global->SetGlobalObjectOwner(this); // does not refcount
+    }
+    *aResult = global;
+    NS_ADDREF(*aResult);
+    return NS_OK;
+}
 
 //----------------------------------------------------------------------
 //
@@ -333,28 +386,21 @@ nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
     }
 
     // nsIPrincipal mDocumentPrincipal
-    nsCOMPtr<nsIScriptSecurityManager> securityManager = 
-             do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID);
-
-    if (! securityManager)
-        return NS_ERROR_FAILURE;
-
-    rv |= NS_ReadOptionalObject(aStream, PR_TRUE, getter_AddRefs(mDocumentPrincipal));
-    if (!mDocumentPrincipal) {
-        // XXX This should be handled by the security manager, see bug 160042
-        PRBool isChrome = PR_FALSE;
-        if (NS_SUCCEEDED(mURI->SchemeIs("chrome", &isChrome)) && isChrome)
-            rv |= securityManager->GetSystemPrincipal(getter_AddRefs(mDocumentPrincipal));
-        else
-            rv |= securityManager->GetCodebasePrincipal(mURI, getter_AddRefs(mDocumentPrincipal));
+    nsCOMPtr<nsIPrincipal> principal;
+    rv |= NS_ReadOptionalObject(aStream, PR_TRUE, getter_AddRefs(principal));
+    if (! principal) {
+        principal = GetDocumentPrincipal();
+        if (!principal)
+            rv |= NS_ERROR_FAILURE;
+    } else {
+        mNodeInfoManager->SetDocumentPrincipal(principal);
+        mDocumentPrincipal = principal;
     }
-    mNodeInfoManager->SetDocumentPrincipal(mDocumentPrincipal);
 
     // nsIScriptGlobalObject mGlobalObject
-    mGlobalObject = new nsXULPDGlobalObject();
+    NewXULPDGlobalObject(getter_AddRefs(mGlobalObject));
     if (! mGlobalObject)
         return NS_ERROR_OUT_OF_MEMORY;
-    rv |= mGlobalObject->SetGlobalObjectOwner(this); // does not refcount
 
     mRoot = new nsXULPrototypeElement();
     if (! mRoot)
@@ -366,9 +412,7 @@ nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
                  "no prototype script context!");
 
     // nsINodeInfo table
-    nsCOMPtr<nsISupportsArray> nodeInfos;
-    rv |= NS_NewISupportsArray(getter_AddRefs(nodeInfos));
-    NS_ENSURE_TRUE(nodeInfos, rv);
+    nsCOMArray<nsINodeInfo> nodeInfos;
 
     rv |= aStream->Read32(&referenceCount);
     nsAutoString namespaceURI, qualifiedName;
@@ -377,8 +421,9 @@ nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
         rv |= aStream->ReadString(qualifiedName);
 
         nsCOMPtr<nsINodeInfo> nodeInfo;
-        rv |= mNodeInfoManager->GetNodeInfo(qualifiedName, namespaceURI, *getter_AddRefs(nodeInfo));
-        rv |= nodeInfos->AppendElement(nodeInfo);
+        rv |= mNodeInfoManager->GetNodeInfo(qualifiedName, namespaceURI, getter_AddRefs(nodeInfo));
+        if (!nodeInfos.AppendObject(nodeInfo))
+            rv |= NS_ERROR_OUT_OF_MEMORY;
     }
 
     // Document contents
@@ -388,7 +433,7 @@ nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
     if ((nsXULPrototypeNode::Type)type != nsXULPrototypeNode::eType_Element)
         return NS_ERROR_FAILURE;
 
-    rv |= mRoot->Deserialize(aStream, scriptContext, mURI, nodeInfos);
+    rv |= mRoot->Deserialize(aStream, scriptContext, mURI, &nodeInfos);
     rv |= NotifyLoadDone();
 
     return rv;
@@ -433,15 +478,14 @@ nsXULPrototypeDocument::Write(nsIObjectOutputStream* aStream)
     rv |= NS_WriteOptionalObject(aStream, mDocumentPrincipal, PR_TRUE);
     
     // nsINodeInfo table
-    nsCOMPtr<nsISupportsArray> nodeInfos;
-    rv |= mNodeInfoManager->GetNodeInfoArray(getter_AddRefs(nodeInfos));
+    nsCOMArray<nsINodeInfo> nodeInfos;
+    rv |= mNodeInfoManager->GetNodeInfos(&nodeInfos);
     NS_ENSURE_SUCCESS(rv, rv);
-    PRUint32 nodeInfoCount;
-    nodeInfos->Count(&nodeInfoCount);
+    PRInt32 nodeInfoCount = nodeInfos.Count();
 
     rv |= aStream->Write32(nodeInfoCount);
-    for (i = 0; i < nodeInfoCount; ++i) {
-        nsCOMPtr<nsINodeInfo> nodeInfo = do_QueryElementAt(nodeInfos, i);
+    for (PRInt32 j = 0; j < nodeInfoCount; ++j) {
+        nsINodeInfo *nodeInfo = nodeInfos[j];
         NS_ENSURE_TRUE(nodeInfo, NS_ERROR_FAILURE);
 
         nsAutoString namespaceURI;
@@ -449,7 +493,7 @@ nsXULPrototypeDocument::Write(nsIObjectOutputStream* aStream)
         rv |= aStream->WriteWStringZ(namespaceURI.get());
 
         nsAutoString qualifiedName;
-        rv |= nodeInfo->GetQualifiedName(qualifiedName);
+        nodeInfo->GetQualifiedName(qualifiedName);
         rv |= aStream->WriteWStringZ(qualifiedName.get());
     }
 
@@ -461,7 +505,7 @@ nsXULPrototypeDocument::Write(nsIObjectOutputStream* aStream)
     rv |= globalObject->GetContext(getter_AddRefs(scriptContext));
     
     if (mRoot)
-        rv |= mRoot->Serialize(aStream, scriptContext, nodeInfos);
+        rv |= mRoot->Serialize(aStream, scriptContext, &nodeInfos);
  
     return rv;
 }
@@ -492,8 +536,7 @@ nsXULPrototypeDocument::SetURI(nsIURI* aURI)
     if (!mDocumentPrincipal) {
         // If the document doesn't have a principal yet we'll force the creation of one
         // so that mNodeInfoManager properly gets one.
-        nsCOMPtr<nsIPrincipal> principal;
-        GetDocumentPrincipal(getter_AddRefs(principal));
+        GetDocumentPrincipal();
     }
     return NS_OK;
 }
@@ -576,34 +619,37 @@ nsXULPrototypeDocument::SetHeaderData(nsIAtom* aField, const nsAString& aData)
 
 
 
-NS_IMETHODIMP
-nsXULPrototypeDocument::GetDocumentPrincipal(nsIPrincipal** aResult)
+nsIPrincipal*
+nsXULPrototypeDocument::GetDocumentPrincipal()
 {
     NS_PRECONDITION(mNodeInfoManager, "missing nodeInfoManager");
     if (!mDocumentPrincipal) {
-        nsresult rv;
-        nsCOMPtr<nsIScriptSecurityManager> securityManager = 
-                 do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
-
-        if (NS_FAILED(rv))
-            return NS_ERROR_FAILURE;
+        nsIScriptSecurityManager *securityManager =
+            nsContentUtils::GetSecurityManager();
+        nsresult rv = NS_OK;
 
         // XXX This should be handled by the security manager, see bug 160042
         PRBool isChrome = PR_FALSE;
-        if (NS_SUCCEEDED(mURI->SchemeIs("chrome", &isChrome)) && isChrome)
-            rv = securityManager->GetSystemPrincipal(getter_AddRefs(mDocumentPrincipal));
-        else
-            rv = securityManager->GetCodebasePrincipal(mURI, getter_AddRefs(mDocumentPrincipal));
+        if (NS_SUCCEEDED(mURI->SchemeIs("chrome", &isChrome)) && isChrome) {
+            if (gSystemPrincipal) {
+                mDocumentPrincipal = gSystemPrincipal;
+            } else {
+                rv = securityManager->
+                     GetSystemPrincipal(getter_AddRefs(mDocumentPrincipal));
+                NS_IF_ADDREF(gSystemPrincipal = mDocumentPrincipal);
+            }
+        } else {
+            rv = securityManager->
+                 GetCodebasePrincipal(mURI, getter_AddRefs(mDocumentPrincipal));
+        }
 
         if (NS_FAILED(rv))
-            return NS_ERROR_FAILURE;
+            return nsnull;
 
         mNodeInfoManager->SetDocumentPrincipal(mDocumentPrincipal);
     }
 
-    *aResult = mDocumentPrincipal;
-    NS_ADDREF(*aResult);
-    return NS_OK;
+    return mDocumentPrincipal;
 }
 
 
@@ -683,17 +729,12 @@ nsXULPrototypeDocument::NotifyLoadDone()
 NS_IMETHODIMP
 nsXULPrototypeDocument::GetScriptGlobalObject(nsIScriptGlobalObject** _result)
 {
-    if (!mGlobalObject) {
-        mGlobalObject = new nsXULPDGlobalObject();
-        if (!mGlobalObject) {
-            *_result = nsnull;
-            return NS_ERROR_OUT_OF_MEMORY;
-            }
-        mGlobalObject->SetGlobalObjectOwner(this); // does not refcount
-    }
+    nsresult rv = NS_OK;
+    if (!mGlobalObject)
+         rv = NewXULPDGlobalObject(getter_AddRefs(mGlobalObject));
     *_result = mGlobalObject;
-    NS_ADDREF(*_result);
-    return NS_OK;
+    NS_IF_ADDREF(*_result);
+    return rv;
 }
 
 NS_IMETHODIMP
@@ -900,11 +941,25 @@ NS_IMETHODIMP
 nsXULPDGlobalObject::GetPrincipal(nsIPrincipal** aPrincipal)
 {
     if (!mGlobalObjectOwner) {
+        // See nsXULPrototypeDocument::NewXULPDGlobalObject, the comment
+        // about gSystemGlobal implying gSystemPrincipal.
+        if (this == nsXULPrototypeDocument::gSystemGlobal) {
+            *aPrincipal = nsXULPrototypeDocument::gSystemPrincipal;
+            NS_ADDREF(*aPrincipal);
+            return NS_OK;
+        }
         *aPrincipal = nsnull;
         return NS_ERROR_FAILURE;
     }
     nsCOMPtr<nsIXULPrototypeDocument> protoDoc
       = do_QueryInterface(mGlobalObjectOwner);
-    return protoDoc->GetDocumentPrincipal(aPrincipal);
+
+    *aPrincipal = protoDoc->GetDocumentPrincipal();
+    if (!*aPrincipal) {
+        return NS_ERROR_FAILURE;
+    }
+
+    NS_ADDREF(*aPrincipal);
+    return NS_OK;
 }
 
