@@ -43,6 +43,7 @@
 
 #include "nsCOMPtr.h"
 #include "nsIContent.h"
+#include "nsINodeInfo.h"
 #include "nsIDocument.h"
 #include "nsIDOMElement.h"
 #include "nsIDOMXULCommandDispatcher.h"
@@ -154,29 +155,20 @@ nsXULContentUtils::FindChildByTag(nsIContent* aElement,
                                   nsIAtom* aTag,
                                   nsIContent** aResult)
 {
-    nsresult rv;
+    PRUint32 count = aElement->GetChildCount();
 
-    PRInt32 count;
-    if (NS_FAILED(rv = aElement->ChildCount(count)))
-        return rv;
-
-    for (PRInt32 i = 0; i < count; ++i) {
-        nsCOMPtr<nsIContent> kid;
-        if (NS_FAILED(rv = aElement->ChildAt(i, getter_AddRefs(kid))))
-            return rv; // XXX fatal
+    for (PRUint32 i = 0; i < count; ++i) {
+        nsIContent *kid = aElement->GetChildAt(i);
 
         PRInt32 nameSpaceID;
-        if (NS_FAILED(rv = kid->GetNameSpaceID(&nameSpaceID)))
-            return rv; // XXX fatal
+        kid->GetNameSpaceID(&nameSpaceID);
 
         if (nameSpaceID != aNameSpaceID)
             continue; // wrong namespace
 
-        nsCOMPtr<nsIAtom> kidTag;
-        if (NS_FAILED(rv = kid->GetTag(getter_AddRefs(kidTag))))
-            return rv; // XXX fatal
+        nsINodeInfo *ni = kid->GetNodeInfo();
 
-        if (kidTag.get() != aTag)
+        if (!ni || !ni->Equals(aTag))
             continue;
 
         *aResult = kid;
@@ -208,11 +200,8 @@ nsXULContentUtils::GetElementResource(nsIContent* aElement, nsIRDFResource** aRe
 
     // Since the element will store its ID attribute as a document-relative value,
     // we may need to qualify it first...
-    nsCOMPtr<nsIDocument> doc;
-    rv = aElement->GetDocument(getter_AddRefs(doc));
-    if (NS_FAILED(rv)) return rv;
-
-    NS_ASSERTION(doc != nsnull, "element is not in any document");
+    nsCOMPtr<nsIDocument> doc = aElement->GetDocument();
+    NS_ASSERTION(doc, "element is not in any document");
     if (! doc)
         return NS_ERROR_FAILURE;
 
@@ -240,12 +229,9 @@ nsXULContentUtils::GetElementRefResource(nsIContent* aElement, nsIRDFResource**
 
     if (rv == NS_CONTENT_ATTR_HAS_VALUE) {
         // We'll use rdf_MakeAbsolute() to translate this to a URL.
-        nsCOMPtr<nsIDocument> doc;
-        rv = aElement->GetDocument(getter_AddRefs(doc));
-        if (NS_FAILED(rv)) return rv;
+        nsCOMPtr<nsIDocument> doc = aElement->GetDocument();
 
-        nsCOMPtr<nsIURI> url;
-        doc->GetDocumentURL(getter_AddRefs(url));
+        nsIURI *url = doc->GetDocumentURL();
         NS_ASSERTION(url != nsnull, "element has no document");
         if (! url)
             return NS_ERROR_UNEXPECTED;
@@ -349,9 +335,7 @@ nsXULContentUtils::MakeElementURI(nsIDocument* aDocument, const nsAString& aElem
     else {
         nsresult rv;
 
-        nsCOMPtr<nsIURI> docURL;
-        rv = aDocument->GetBaseURL(getter_AddRefs(docURL));
-        if (NS_FAILED(rv)) return rv;
+        nsIURI *docURL = aDocument->GetBaseURL();
 
         // XXX Urgh. This is so broken; I'd really just like to use
         // NS_MakeAbsolueURI(). Unfortunatly, doing that breaks
@@ -408,12 +392,8 @@ nsXULContentUtils::MakeElementID(nsIDocument* aDocument, const nsAString& aURI,
     // DOM APIs.
     nsresult rv;
 
-    nsCOMPtr<nsIURI> docURL;
-    rv = aDocument->GetBaseURL(getter_AddRefs(docURL));
-    if (NS_FAILED(rv)) return rv;
-
     nsCAutoString spec;
-    docURL->GetSpec(spec);
+    aDocument->GetBaseURL()->GetSpec(spec);
 
     // XXX FIX ME to not do a copy
     nsAutoString str(aURI);
