@@ -289,13 +289,15 @@ NS_IMPL_NSIDOCUMENTOBSERVER_STATE_STUB(nsXULTemplateBuilder)
 NS_IMPL_NSIDOCUMENTOBSERVER_STYLE_STUB(nsXULTemplateBuilder)
 
 NS_IMETHODIMP
-nsXULTemplateBuilder::BeginUpdate(nsIDocument *aDocument)
+nsXULTemplateBuilder::BeginUpdate(nsIDocument *aDocument,
+                                  nsUpdateType aUpdateType)
 {
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsXULTemplateBuilder::EndUpdate(nsIDocument *aDocument)
+nsXULTemplateBuilder::EndUpdate(nsIDocument *aDocument,
+                                nsUpdateType aUpdateType)
 {
     return NS_OK;
 }
@@ -313,8 +315,7 @@ nsXULTemplateBuilder::AttributeChanged(nsIDocument *aDocument,
                                        nsIContent*  aContent,
                                        PRInt32      aNameSpaceID,
                                        nsIAtom*     aAttribute,
-                                       PRInt32      aModType, 
-                                       nsChangeHint aHint)
+                                       PRInt32      aModType)
 {
     // Check for a change to the 'ref' attribute on an atom, in which
     // case we may need to nuke and rebuild the entire content model
@@ -706,19 +707,18 @@ nsXULTemplateBuilder::LoadDataSources()
     if (allowneg == NS_LITERAL_STRING("false"))
 		mDB->SetAllowNegativeAssertions(PR_FALSE);
 
-    nsCOMPtr<nsIDocument> doc;
-    mRoot->GetDocument(getter_AddRefs(doc));
+    nsCOMPtr<nsIDocument> doc = mRoot->GetDocument();
     NS_ASSERTION(doc, "element has no document");
     if (! doc)
         return NS_ERROR_UNEXPECTED;
 
     // Grab the doc's principal...
-    nsCOMPtr<nsIPrincipal> docPrincipal;
-    rv = doc->GetPrincipal(getter_AddRefs(docPrincipal));
-    if (NS_FAILED(rv)) return rv;
+    nsIPrincipal *docPrincipal = doc->GetPrincipal();
+    if (!docPrincipal)
+        return NS_ERROR_FAILURE;
 
     PRBool isTrusted = PR_FALSE;
-    rv = IsSystemPrincipal(docPrincipal.get(), &isTrusted);
+    rv = IsSystemPrincipal(docPrincipal, &isTrusted);
     if (NS_FAILED(rv)) return rv;
 
     if (isTrusted) {
@@ -740,8 +740,7 @@ nsXULTemplateBuilder::LoadDataSources()
     //
     //     rdf:bookmarks rdf:history http://foo.bar.com/blah.cgi?baz=9
     //
-    nsCOMPtr<nsIURI> docurl;
-    doc->GetDocumentURL(getter_AddRefs(docurl));
+    nsIURI *docurl = doc->GetDocumentURL();
 
     nsAutoString datasources;
     mRoot->GetAttr(kNameSpaceID_None, nsXULAtoms::datasources, datasources);
@@ -849,14 +848,12 @@ nsXULTemplateBuilder::InitHTMLTemplateRoot()
     // 'database' and 'builder' properties onto aElement.
     nsresult rv;
 
-    nsCOMPtr<nsIDocument> doc;
-    mRoot->GetDocument(getter_AddRefs(doc));
-    NS_ASSERTION(doc != nsnull, "no document");
+    nsCOMPtr<nsIDocument> doc = mRoot->GetDocument();
+    NS_ASSERTION(doc, "no document");
     if (! doc)
         return NS_ERROR_UNEXPECTED;
 
-    nsCOMPtr<nsIScriptGlobalObject> global;
-    doc->GetScriptGlobalObject(getter_AddRefs(global));
+    nsIScriptGlobalObject *global = doc->GetScriptGlobalObject();
     if (! global)
         return NS_ERROR_UNEXPECTED;
 
@@ -1382,18 +1379,9 @@ nsXULTemplateBuilder::ComputeContainmentProperties()
 PRBool
 nsXULTemplateBuilder::IsTemplateElement(nsIContent* aContent)
 {
-    PRInt32 nameSpaceID;
-    aContent->GetNameSpaceID(&nameSpaceID);
+    nsINodeInfo *ni = aContent->GetNodeInfo();
 
-    if (nameSpaceID == kNameSpaceID_XUL) {
-        nsCOMPtr<nsIAtom> tag;
-        aContent->GetTag(getter_AddRefs(tag));
-
-        if (tag.get() == nsXULAtoms::Template)
-            return PR_TRUE;
-    }
-
-    return PR_FALSE;
+    return ni && ni->Equals(nsXULAtoms::Template, kNameSpaceID_XUL);
 }
 
 nsresult
@@ -1443,13 +1431,7 @@ nsXULTemplateBuilder::GetTemplateRoot(nsIContent** aResult)
     mRoot->GetAttr(kNameSpaceID_None, nsXULAtoms::templateAtom, templateID);
 
     if (!templateID.IsEmpty()) {
-        nsCOMPtr<nsIDocument> doc;
-        mRoot->GetDocument(getter_AddRefs(doc));
-        NS_ASSERTION(doc != nsnull, "root element has no document");
-        if (! doc)
-            return NS_ERROR_FAILURE;
-
-        nsCOMPtr<nsIDOMDocument> domDoc = do_QueryInterface(doc);
+        nsCOMPtr<nsIDOMDocument> domDoc = do_QueryInterface(mRoot->GetDocument());
         NS_ASSERTION(domDoc, "expected a XUL document");
         if (! domDoc)
             return NS_ERROR_FAILURE;
@@ -1465,15 +1447,13 @@ nsXULTemplateBuilder::GetTemplateRoot(nsIContent** aResult)
     {
         // If root node has no template attribute, then look for a child
         // node which is a template tag
-        PRInt32 count = 0;
-        mRoot->ChildCount(count);
+        PRUint32 count = mRoot->GetChildCount();
 
-        for (PRInt32 i = 0; i < count; ++i) {
-            nsCOMPtr<nsIContent> child;
-            mRoot->ChildAt(i, getter_AddRefs(child));
+        for (PRUint32 i = 0; i < count; ++i) {
+            nsIContent *child = mRoot->GetChildAt(i);
 
             if (IsTemplateElement(child)) {
-                NS_ADDREF(*aResult = child.get());
+                NS_ADDREF(*aResult = child);
                 return NS_OK;
             }
         }
@@ -1482,14 +1462,12 @@ nsXULTemplateBuilder::GetTemplateRoot(nsIContent** aResult)
 
     // If we couldn't find a real child, look through the anonymous
     // kids, too.
-    nsCOMPtr<nsIDocument> doc;
-    mRoot->GetDocument(getter_AddRefs(doc));
-    NS_ASSERTION(doc != nsnull, "root element has no document");
+    nsCOMPtr<nsIDocument> doc = mRoot->GetDocument();
+    NS_ASSERTION(doc, "root element has no document");
     if (! doc)
         return NS_ERROR_FAILURE;
 
-    nsCOMPtr<nsIBindingManager> bindingManager;
-    doc->GetBindingManager(getter_AddRefs(bindingManager));
+    nsIBindingManager *bindingManager = doc->GetBindingManager();
 
     if (bindingManager) {
         nsCOMPtr<nsIDOMNodeList> kids;
@@ -1552,27 +1530,15 @@ nsXULTemplateBuilder::CompileRules()
         mRules.PutSymbol(mMemberSymbol.get(), mMemberVar);
 
     // Compile the rules beneath the <template>
-    PRInt32 count = 0;
-    tmpl->ChildCount(count);
+    PRUint32 count = tmpl->GetChildCount();
 
-    PRInt32 nrules = 0;
+    PRUint32 nrules = 0;
 
-    for (PRInt32 i = 0; i < count; i++) {
-        nsCOMPtr<nsIContent> rule;
-        tmpl->ChildAt(i, getter_AddRefs(rule));
-        if (! rule)
-            break;
+    for (PRUint32 i = 0; i < count; i++) {
+        nsIContent *rule = tmpl->GetChildAt(i);
+        nsINodeInfo *ni = rule->GetNodeInfo();
 
-        PRInt32 nameSpaceID = kNameSpaceID_Unknown;
-        rule->GetNameSpaceID(&nameSpaceID);
-
-        if (nameSpaceID != kNameSpaceID_XUL)
-            continue;
-
-        nsCOMPtr<nsIAtom> tag;
-        rule->GetTag(getter_AddRefs(tag));
-
-        if (tag.get() == nsXULAtoms::rule) {
+        if (ni && ni->Equals(nsXULAtoms::rule, kNameSpaceID_XUL)) {
             ++nrules;
 
             // If the <rule> has a <conditions> element, then
@@ -1677,14 +1643,12 @@ nsXULTemplateBuilder::CompileExtendedRule(nsIContent* aRuleElement,
 
             // otherwise, append the children to the unvisited list: this
             // results in a breadth-first search.
-            PRInt32 count;
-            next->ChildCount(count);
+            PRUint32 count = next->GetChildCount();
 
-            for (PRInt32 i = 0; i < count; ++i) {
-                nsCOMPtr<nsIContent> child;
-                next->ChildAt(i, getter_AddRefs(child));
+            for (PRUint32 i = 0; i < count; ++i) {
+                nsIContent *child = next->GetChildAt(i);
 
-                unvisited.AppendElement(child.get());
+                unvisited.AppendElement(child);
             }
         }
     }
@@ -1754,18 +1718,14 @@ nsXULTemplateBuilder::CompileConditions(nsTemplateRule* aRule,
     // Compile an extended rule's conditions.
     nsresult rv;
 
-    PRInt32 count;
-    aConditions->ChildCount(count);
+    PRUint32 count = aConditions->GetChildCount();
 
-    for (PRInt32 i = 0; i < count; ++i) {
-        nsCOMPtr<nsIContent> condition;
-        aConditions->ChildAt(i, getter_AddRefs(condition));
-
-        nsCOMPtr<nsIAtom> tag;
-        condition->GetTag(getter_AddRefs(tag));
+    for (PRUint32 i = 0; i < count; ++i) {
+        nsIContent *condition = aConditions->GetChildAt(i);
 
         TestNode* testnode = nsnull;
-        rv = CompileCondition(tag, aRule, condition, aParentNode, &testnode);
+        rv = CompileCondition(condition->Tag(), aRule, condition,
+                              aParentNode, &testnode);
         if (NS_FAILED(rv)) return rv;
 
         // XXXwaterson proably wrong to just drill it straight down
@@ -1963,23 +1923,22 @@ nsXULTemplateBuilder::CompileBindings(nsTemplateRule* aRule, nsIContent* aBindin
     // Add an extended rule's bindings.
     nsresult rv;
 
-    PRInt32 count;
-    aBindings->ChildCount(count);
+    PRUint32 count = aBindings->GetChildCount();
 
-    for (PRInt32 i = 0; i < count; ++i) {
-        nsCOMPtr<nsIContent> binding;
-        aBindings->ChildAt(i, getter_AddRefs(binding));
+    for (PRUint32 i = 0; i < count; ++i) {
+        nsIContent *binding = aBindings->GetChildAt(i);
 
-        nsCOMPtr<nsIAtom> tag;
-        binding->GetTag(getter_AddRefs(tag));
+        nsINodeInfo *ni = binding->GetNodeInfo();
 
-        if (tag.get() == nsXULAtoms::binding) {
+        if (ni && ni->Equals(nsXULAtoms::binding, kNameSpaceID_XUL)) {
             rv = CompileBinding(aRule, binding);
         }
         else {
 #ifdef PR_LOGGING
             nsAutoString tagstr;
-            tag->ToString(tagstr);
+            if (ni) {
+                ni->GetQualifiedName(tagstr);
+            }
 
             nsCAutoString tagstrC;
             tagstrC.AssignWithConversion(tagstr);
@@ -2089,11 +2048,10 @@ nsXULTemplateBuilder::CompileSimpleRule(nsIContent* aRuleElement,
 
     PRBool hasContainerTest = PR_FALSE;
 
-    PRInt32 count;
-    aRuleElement->GetAttrCount(count);
+    PRUint32 count = aRuleElement->GetAttrCount();
 
     // Add constraints for the LHS
-    for (PRInt32 i = 0; i < count; ++i) {
+    for (PRUint32 i = 0; i < count; ++i) {
         PRInt32 attrNameSpaceID;
         nsCOMPtr<nsIAtom> attr, prefix;
         rv = aRuleElement->GetAttrNameAt(i, &attrNameSpaceID,
@@ -2269,14 +2227,13 @@ nsXULTemplateBuilder::AddSimpleRuleBindings(nsTemplateRule* aRule, nsIContent* a
     elements.AppendElement(aElement);
     while (elements.Count()) {
         // Pop the next element off the stack
-        PRInt32 i = elements.Count() - 1;
+        PRUint32 i = (PRUint32)(elements.Count() - 1);
         nsIContent* element = NS_STATIC_CAST(nsIContent*, elements[i]);
         elements.RemoveElementAt(i);
 
         // Iterate through its attributes, looking for substitutions
         // that we need to add as bindings.
-        PRInt32 count;
-        element->GetAttrCount(count);
+        PRUint32 count = element->GetAttrCount();
 
         for (i = 0; i < count; ++i) {
             PRInt32 nameSpaceID;
@@ -2294,12 +2251,10 @@ nsXULTemplateBuilder::AddSimpleRuleBindings(nsTemplateRule* aRule, nsIContent* a
         }
 
         // Push kids onto the stack, and search them next.
-        element->ChildCount(count);
+        count = element->GetChildCount();
 
-        while (--count >= 0) {
-            nsCOMPtr<nsIContent> child;
-            element->ChildAt(count, getter_AddRefs(child));
-            elements.AppendElement(child);
+        while (count-- > 0) {
+            elements.AppendElement(element->GetChildAt(count));
         }
     }
 
@@ -2313,7 +2268,7 @@ nsXULTemplateBuilder::AddBindingsFor(nsXULTemplateBuilder* aThis,
 {
     // We should *only* be recieving "rdf:"-style variables. Make
     // sure...
-    if (Substring(aVariable, PRUint32(0), PRUint32(4)) != NS_LITERAL_STRING("rdf:"))
+    if (!StringBeginsWith(aVariable, NS_LITERAL_STRING("rdf:")))
         return;
 
     nsTemplateRule* rule = NS_STATIC_CAST(nsTemplateRule*, aClosure);
