@@ -37,13 +37,6 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-#ifdef XP_OS2_VACPP
-// XXX every other file that pulls in _os2.h has no problem with HTMX there;
-// this one does; the problem may lie with the order of the headers below,
-// which is why this fix is here instead of in _os2.h
-typedef unsigned long HMTX;
-#endif
-
 #include "nsDocShell.h"
 #include "nsIWebShell.h"
 #include "nsWebShell.h"
@@ -73,8 +66,6 @@ typedef unsigned long HMTX;
 #include "nsVoidArray.h"
 #include "nsString.h"
 #include "nsReadableUtils.h"
-#include "nsWidgetsCID.h"
-#include "nsGfxCIID.h"
 #include "plevent.h"
 #include "prprf.h"
 #include "nsIPluginHost.h"
@@ -91,10 +82,10 @@ typedef unsigned long HMTX;
 #include "prthread.h"
 #include "nsXPIDLString.h"
 #include "nsDOMError.h"
-#include "nsLayoutCID.h"
 #include "nsIDOMRange.h"
 #include "nsIURIContentListener.h"
 #include "nsIDOMDocument.h"
+#include "nsIDOMHTMLAnchorElement.h"
 #include "nsIBaseWindow.h"
 #include "nsIDocShell.h"
 #include "nsIDocShellTreeItem.h"
@@ -158,14 +149,6 @@ static PRLogModuleInfo* gLogModule = PR_NewLogModule("webshell");
 
 //----------------------------------------------------------------------
 
-//----------------------------------------------------------------------
-
-// Class IID's
-static NS_DEFINE_IID(kEventQueueServiceCID,   NS_EVENTQUEUESERVICE_CID);
-static NS_DEFINE_CID(kCDOMRangeCID,           NS_RANGE_CID);
-
-//----------------------------------------------------------------------
-
 static PRBool
 IsOffline()
 {
@@ -562,7 +545,7 @@ nsWebShell::GetEventQueue(nsIEventQueue **aQueue)
   NS_ENSURE_ARG_POINTER(aQueue);
   *aQueue = 0;
 
-  nsCOMPtr<nsIEventQueueService> eventService(do_GetService(kEventQueueServiceCID));
+  nsCOMPtr<nsIEventQueueService> eventService(do_GetService(NS_EVENTQUEUESERVICE_CONTRACTID));
   if (eventService)
     eventService->GetThreadEventQueue(mThread, aQueue);
   return *aQueue ? NS_OK : NS_ERROR_FAILURE;
@@ -583,40 +566,32 @@ nsWebShell::OnLinkClickSync(nsIContent *aContent,
   // I promise this will be removed once we figure out a better way. 
   nsCAutoString scheme;
   aURI->GetScheme(scheme);
+
+  nsCAutoString spec;
+  aURI->GetSpec(spec);
                                           
   static const char kMailToURI[] = "mailto";                                    
   static const char kNewsURI[] = "news";                                        
   static const char kSnewsURI[] = "snews";                                      
   static const char kNntpURI[] = "nntp";                                        
   static const char kImapURI[] = "imap"; 
-  if (scheme.EqualsIgnoreCase(kMailToURI)) 
+  static const char kAddbookURI[] = "addbook";
+  static const char kPopURI[] = "pop";
+  static const char kMailboxURI[] = "mailbox";
+
+
+  if (scheme.EqualsIgnoreCase(kMailToURI) || scheme.EqualsIgnoreCase(kNewsURI) || scheme.EqualsIgnoreCase(kSnewsURI)   || 
+      scheme.EqualsIgnoreCase(kNntpURI)   || scheme.EqualsIgnoreCase(kImapURI) || scheme.EqualsIgnoreCase(kAddbookURI) ||
+      scheme.EqualsIgnoreCase(kPopURI)    || scheme.EqualsIgnoreCase(kMailboxURI)) 
   {
-    // the scheme is mailto, we can handle it
-  } 
-  else if (scheme.EqualsIgnoreCase(kNewsURI)) 
+    // we can handle all mail schemes
+  } else 
   {
-    // the scheme is news, we can handle it
-  } 
-  else if (scheme.EqualsIgnoreCase(kSnewsURI)) 
-  {
-     // the scheme is snews, we can handle it
-  } 
-  else if (scheme.EqualsIgnoreCase(kNntpURI)) 
-  {
-     // the scheme is nntp, we can handle it 
-  } else if (scheme.EqualsIgnoreCase(kImapURI)) 
-  {
-      // the scheme is imap, we can handle it
-  }
-  else 
-  {
-      // we don't handle this type, the the registered handler take it 
-      nsresult rv = NS_OK;
-      nsCOMPtr<nsIExternalProtocolService> extProtService = do_GetService(NS_EXTERNALPROTOCOLSERVICE_CONTRACTID, &rv);
-      NS_ENSURE_SUCCESS(rv,rv);
-      rv = extProtService->LoadUrl(aURI);
-      NS_ENSURE_SUCCESS(rv,rv);
-      return rv;                                                                  
+    // we don't handle this type, the the registered handler take it 
+    nsresult rv = NS_OK;
+    nsCOMPtr<nsIExternalProtocolService> extProtService = do_GetService(NS_EXTERNALPROTOCOLSERVICE_CONTRACTID, &rv);
+    NS_ENSURE_SUCCESS(rv,rv);
+    return extProtService->LoadUrl(aURI);                                                                 
   }                                                                               
 #endif
 
@@ -630,8 +605,7 @@ nsWebShell::OnLinkClickSync(nsIContent *aContent,
   aURI->SchemeIs("data", &isData);
 
   if (isJS || isData) {
-    nsCOMPtr<nsIDocument> sourceDoc;
-    aContent->GetDocument(getter_AddRefs(sourceDoc));
+    nsCOMPtr<nsIDocument> sourceDoc = aContent->GetDocument();
 
     if (!sourceDoc) {
       // The source is in a 'zombie' document, or not part of a
@@ -667,14 +641,20 @@ nsWebShell::OnLinkClickSync(nsIContent *aContent,
   nsCOMPtr<nsIDocument> refererDoc(do_QueryInterface(refererOwnerDoc));
   NS_ENSURE_TRUE(refererDoc, NS_ERROR_UNEXPECTED);
 
-  nsCOMPtr<nsIURI> referer;
-  refererDoc->GetDocumentURL(getter_AddRefs(referer));
+  nsIURI *referer = refererDoc->GetDocumentURL();
 
   // referer could be null here in some odd cases, but that's ok,
   // we'll just load the link w/o sending a referer in those cases.
 
   nsAutoString target(aTargetSpec);
 
+  // If this is an anchor element, grab its type property to use as a hint
+  nsAutoString typeHint;
+  nsCOMPtr<nsIDOMHTMLAnchorElement> anchor(do_QueryInterface(aContent));
+  if (anchor) {
+    anchor->GetType(typeHint);
+  }
+  
   // Initialize the DocShell / Request
   if (aDocShell) {
     *aDocShell = nsnull;
@@ -696,6 +676,7 @@ nsWebShell::OnLinkClickSync(nsIContent *aContent,
                             nsnull,             // No onwer
                             PR_TRUE,            // Inherit owner from document
                             target.get(),       // Window target
+                            NS_LossyConvertUCS2toASCII(typeHint).get(),
                             aPostDataStream,    // Post data stream
                             aHeadersDataStream, // Headers stream
                             LOAD_LINK,          // Load type
@@ -1071,6 +1052,7 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
                          nsnull,                            // Owner
                          PR_TRUE,                           // Inherit owner
                          nsnull,                            // No window target
+                         nsnull,                            // No type hint
                          inputStream,                       // Post data stream
                          nsnull,                            // No headers stream
                          LOAD_RELOAD_BYPASS_PROXY_AND_CACHE,// Load type
