@@ -154,7 +154,6 @@ protected:
             id == sParent_id       ||
             id == sScrollbars_id   ||
             id == sContent_id      ||
-            id == sSidebar_id      ||
             id == sMenubar_id      ||
             id == sToolbar_id      ||
             id == sLocationbar_id  ||
@@ -207,7 +206,6 @@ protected:
   static jsval sConstructor_id;
   static jsval s_content_id;
   static jsval sContent_id;
-  static jsval sSidebar_id;
   static jsval sMenubar_id;
   static jsval sToolbar_id;
   static jsval sLocationbar_id;
@@ -275,14 +273,14 @@ typedef nsDOMClassInfo nsDOMGenericSH;
 // all objects that should support things like
 // obj.onclick=function{...}
 
-class nsEventRecieverSH : public nsDOMGenericSH
+class nsEventReceiverSH : public nsDOMGenericSH
 {
 protected:
-  nsEventRecieverSH(nsDOMClassInfoData* aData) : nsDOMGenericSH(aData)
+  nsEventReceiverSH(nsDOMClassInfoData* aData) : nsDOMGenericSH(aData)
   {
   }
 
-  virtual ~nsEventRecieverSH()
+  virtual ~nsEventReceiverSH()
   {
   }
 
@@ -317,10 +315,10 @@ public:
 
 // Window scriptable helper
 
-class nsWindowSH : public nsEventRecieverSH
+class nsWindowSH : public nsEventReceiverSH
 {
 protected:
-  nsWindowSH(nsDOMClassInfoData* aData) : nsEventRecieverSH(aData)
+  nsWindowSH(nsDOMClassInfoData* aData) : nsEventReceiverSH(aData)
   {
   }
 
@@ -387,10 +385,10 @@ public:
 // DOM Node helper, this class deals with setting the parent for the
 // wrappers
 
-class nsNodeSH : public nsEventRecieverSH
+class nsNodeSH : public nsEventReceiverSH
 {
 protected:
-  nsNodeSH(nsDOMClassInfoData* aData) : nsEventRecieverSH(aData)
+  nsNodeSH(nsDOMClassInfoData* aData) : nsEventReceiverSH(aData)
   {
   }
 
@@ -436,12 +434,36 @@ public:
 };
 
 
-// NodeList scriptable helper
+// Generic array scriptable helper
 
-class nsArraySH : public nsDOMClassInfo
+class nsGenericArraySH : public nsDOMClassInfo
 {
 protected:
-  nsArraySH(nsDOMClassInfoData* aData) : nsDOMClassInfo(aData)
+  nsGenericArraySH(nsDOMClassInfoData* aData) : nsDOMClassInfo(aData)
+  {
+  }
+
+  virtual ~nsGenericArraySH()
+  {
+  }
+  
+public:
+  NS_IMETHOD Enumerate(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
+                       JSObject *obj, PRBool *_retval);
+  
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
+  {
+    return new nsGenericArraySH(aData);
+  }
+};
+
+
+// NodeList scriptable helper
+
+class nsArraySH : public nsGenericArraySH
+{
+protected:
+  nsArraySH(nsDOMClassInfoData* aData) : nsGenericArraySH(aData)
   {
   }
 
@@ -959,12 +981,12 @@ public:
 };
 
 
-// History helper
+// String array helper
 
-class nsStringArraySH : public nsDOMClassInfo
+class nsStringArraySH : public nsGenericArraySH
 {
 protected:
-  nsStringArraySH(nsDOMClassInfoData* aData) : nsDOMClassInfo(aData)
+  nsStringArraySH(nsDOMClassInfoData* aData) : nsGenericArraySH(aData)
   {
   }
 
@@ -1007,6 +1029,31 @@ public:
   }
 };
 
+// StringList scriptable helper
+
+class nsStringListSH : public nsStringArraySH
+{
+protected:
+  nsStringListSH(nsDOMClassInfoData* aData) : nsStringArraySH(aData)
+  {
+  }
+
+  virtual ~nsStringListSH()
+  {
+  }
+
+  virtual nsresult GetStringAt(nsISupports *aNative, PRInt32 aIndex,
+                               nsAString& aResult);
+
+public:
+  // Inherit GetProperty, Enumerate from nsStringArraySH
+  
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
+  {
+    return new nsStringListSH(aData);
+  }
+};
+
 
 // MediaList helper
 
