@@ -177,7 +177,7 @@ NS_ScriptErrorReporter(JSContext *cx,
       //send error event first, then proceed
       nsCOMPtr<nsIDocShell> docShell;
       globalObject->GetDocShell(getter_AddRefs(docShell));
-      if (docShell) {
+      if (docShell && !JSREPORT_IS_WARNING(report->flags)) {
         static PRInt32 errorDepth; // Recursion prevention
         errorDepth++;
 
@@ -309,7 +309,7 @@ ChangeCase(JSContext *cx, JSString *src, jsval *rval,
   nsAutoString result;
   changeCaseFnc(str, result);
 
-  JSString *ucstr = JS_NewUCStringCopyN(cx, result.get(), result.Length());
+  JSString *ucstr = JS_NewUCStringCopyN(cx, (jschar*)result.get(), result.Length());
   if (!ucstr) {
     return JS_FALSE;
   }
@@ -628,7 +628,7 @@ nsJSContext::EvaluateStringWithValue(const nsAString& aScript,
   JSPrincipals *jsprin;
   nsCOMPtr<nsIPrincipal> principal = aPrincipal;
   if (aPrincipal) {
-    aPrincipal->GetJSPrincipals(&jsprin);
+    aPrincipal->GetJsPrincipals(&jsprin);
   }
   else {
     nsCOMPtr<nsIScriptGlobalObject> global;
@@ -641,7 +641,7 @@ nsJSContext::EvaluateStringWithValue(const nsAString& aScript,
     rv = objPrincipal->GetPrincipal(getter_AddRefs(principal));
     if (NS_FAILED(rv))
       return NS_ERROR_FAILURE;
-    principal->GetJSPrincipals(&jsprin);
+    principal->GetJsPrincipals(&jsprin);
   }
   // From here on, we must JSPRINCIPALS_DROP(jsprin) before returning...
 
@@ -806,7 +806,7 @@ nsJSContext::EvaluateString(const nsAString& aScript,
   JSPrincipals *jsprin;
   nsCOMPtr<nsIPrincipal> principal = aPrincipal;
   if (aPrincipal) {
-    aPrincipal->GetJSPrincipals(&jsprin);
+    aPrincipal->GetJsPrincipals(&jsprin);
   }
   else {
     nsCOMPtr<nsIScriptGlobalObject> global;
@@ -819,7 +819,7 @@ nsJSContext::EvaluateString(const nsAString& aScript,
     rv = objPrincipal->GetPrincipal(getter_AddRefs(principal));
     if (NS_FAILED(rv))
       return NS_ERROR_FAILURE;
-    principal->GetJSPrincipals(&jsprin);
+    principal->GetJsPrincipals(&jsprin);
   }
   // From here on, we must JSPRINCIPALS_DROP(jsprin) before returning...
 
@@ -919,7 +919,7 @@ nsJSContext::CompileScript(const PRUnichar* aText,
     aScopeObject = ::JS_GetGlobalObject(mContext);
 
   JSPrincipals *jsprin;
-  aPrincipal->GetJSPrincipals(&jsprin);
+  aPrincipal->GetJsPrincipals(&jsprin);
   // From here on, we must JSPRINCIPALS_DROP(jsprin) before returning...
 
   PRBool ok = PR_FALSE;
@@ -1068,6 +1068,7 @@ AtomToEventHandlerName(nsIAtom *aName)
 NS_IMETHODIMP
 nsJSContext::CompileEventHandler(void *aTarget, nsIAtom *aName,
                                  const nsAString& aBody,
+                                 const char *aURL, PRUint32 aLineNo,
                                  PRBool aShared, void** aHandler)
 {
   if (!sSecurityManager) {
@@ -1092,7 +1093,7 @@ nsJSContext::CompileEventHandler(void *aTarget, nsIAtom *aName,
                                                        getter_AddRefs(prin));
     NS_ENSURE_SUCCESS(rv, rv);
 
-    prin->GetJSPrincipals(&jsprin);
+    prin->GetJsPrincipals(&jsprin);
     NS_ENSURE_TRUE(jsprin, NS_ERROR_NOT_AVAILABLE);
   }
 
@@ -1103,8 +1104,7 @@ nsJSContext::CompileEventHandler(void *aTarget, nsIAtom *aName,
                                           charName, 1, gEventArgv,
                                           (jschar*)(const PRUnichar*)PromiseFlatString(aBody).get(),
                                           aBody.Length(),
-                                          //XXXbe filename, lineno:
-                                          nsnull, 0);
+                                          aURL, aLineNo);
 
   if (jsprin) {
     JSPRINCIPALS_DROP(mContext, jsprin);
@@ -1146,7 +1146,7 @@ nsJSContext::CompileFunction(void* aTarget,
       nsCOMPtr<nsIPrincipal> prin;
       if (NS_FAILED(globalData->GetPrincipal(getter_AddRefs(prin))))
         return NS_ERROR_FAILURE;
-      prin->GetJSPrincipals(&jsprin);
+      prin->GetJsPrincipals(&jsprin);
     }
   }
 
@@ -1210,7 +1210,9 @@ nsJSContext::CallEventHandler(void *aTarget, void *aHandler, PRUint32 argc,
   // check if the event handler can be run on the object in question
   rv = securityManager->CheckFunctionAccess(mContext, aHandler, aTarget);
 
-  if (NS_SUCCEEDED(rv)) {
+  if (NS_FAILED(rv)) {
+    *aBoolResult = !aReverseReturnResult;
+  } else {
     jsval val;
     jsval funval = OBJECT_TO_JSVAL(aHandler);
     PRBool ok = ::JS_CallFunctionValue(mContext, (JSObject *)aTarget, funval,
