@@ -39,6 +39,8 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "nsGlobalWindow.h"
+#include "nsIScriptSecurityManager.h"
+#include "nsIScriptContext.h"
 #include "nsIWebShell.h"
 #include "nsIDocShell.h"
 #include "nsIDocShellLoadInfo.h"
@@ -56,7 +58,6 @@
 #include "nsEscape.h"
 #include "nsJSUtils.h"
 #include "nsIScriptSecurityManager.h"
-#include "nsICodebasePrincipal.h"
 #include "nsIDOMWindow.h"
 #include "nsIDOMDocument.h"
 #include "nsIDocument.h"
@@ -97,13 +98,8 @@ static nsresult GetDocumentCharacterSetForURI(const nsAString& aHref, nsACString
   nsCOMPtr<nsIDocument> doc(do_QueryInterface(domDoc));
   NS_ENSURE_TRUE(doc, NS_ERROR_FAILURE);
 
-  nsAutoString charset;
-  rv = doc->GetDocumentCharacterSet(charset);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  CopyUCS2toASCII(charset, aCharset);
-
-  return rv;
+  aCharset = doc->GetDocumentCharacterSet();
+  return NS_OK;
 }
 
 LocationImpl::LocationImpl(nsIDocShell *aDocShell)
@@ -575,10 +571,11 @@ LocationImpl::SetHrefWithBase(const nsAString& aHref,
 
       result = stack->Peek(&cx);
       if (cx) {
-        nsIScriptContext* scriptCX = (nsIScriptContext*)JS_GetContextPrivate(cx);
-       
-        if (scriptCX) {
-          scriptCX->GetProcessingScriptTag(&inScriptTag);
+        nsCOMPtr<nsIScriptContext> scriptContext;
+        nsJSUtils::GetDynamicScriptContext(cx, getter_AddRefs(scriptContext));
+
+        if (scriptContext) {
+          scriptContext->GetProcessingScriptTag(&inScriptTag);
         }  
       } //cx
     }  // stack
@@ -932,14 +929,11 @@ LocationImpl::GetSourceBaseURL(JSContext* cx, nsIURI** sourceURL)
   nsCOMPtr<nsIDocument> doc;
   nsresult rv = GetSourceDocument(cx, getter_AddRefs(doc));
   if (doc) {
-    rv = doc->GetBaseURL(*sourceURL);
-
-    if (!*sourceURL) {
-      doc->GetDocumentURL(sourceURL);
-    }
-  } else {
-    *sourceURL = nsnull;
+    NS_IF_ADDREF(*sourceURL = doc->GetBaseURL());
+    return NS_OK;
   }
+
+  *sourceURL = nsnull;
   return rv;
 }
 
@@ -949,7 +943,7 @@ LocationImpl::GetSourceURL(JSContext* cx, nsIURI** sourceURL)
   nsCOMPtr<nsIDocument> doc;
   nsresult rv = GetSourceDocument(cx, getter_AddRefs(doc));
   if (doc) {
-    doc->GetDocumentURL(sourceURL);
+    NS_IF_ADDREF(*sourceURL = doc->GetDocumentURL());
   } else {
     *sourceURL = nsnull;
   }
