@@ -25,17 +25,17 @@
 
 #include "nsIDOMWindow.h"
 #include "nsComposerCommandsUpdater.h"
-#include "nsIServiceManager.h"
+#include "nsComponentManagerUtils.h"
 #include "nsIDOMDocument.h"
 #include "nsISelection.h"
 #include "nsIScriptGlobalObject.h"
 
 #include "nsIInterfaceRequestorUtils.h"
+#include "nsString.h"
 
 #include "nsICommandManager.h"
 #include "nsPICommandUpdater.h"
 
-#include "nsIEditor.h"
 #include "nsIDocShell.h"
 #include "nsITransactionManager.h"
 
@@ -53,7 +53,7 @@ nsComposerCommandsUpdater::~nsComposerCommandsUpdater()
 }
 
 NS_IMPL_ISUPPORTS4(nsComposerCommandsUpdater, nsISelectionListener,
-                   nsIDocumentStateListener, nsITransactionListener, nsITimerCallback);
+                   nsIDocumentStateListener, nsITransactionListener, nsITimerCallback)
 
 #if 0
 #pragma mark -
