@@ -328,6 +328,27 @@ nsSetDocumentStateCommand::GetCommandStateParams(const char *aCommandName,
 
     return aParams->SetBooleanValue(STATE_ATTRIBUTE, modified);
   }
+  else if (!nsCRT::strcmp(aCommandName, "cmd_setDocumentReadOnly"))
+  {
+    NS_ENSURE_ARG_POINTER(aParams);
+
+    PRUint32 flags;
+    editor->GetFlags(&flags);
+    PRBool isReadOnly = flags & nsIPlaintextEditor::eEditorReadonlyMask;
+    return aParams->SetBooleanValue(STATE_ATTRIBUTE, isReadOnly);
+  }
+  else if (!nsCRT::strcmp(aCommandName, "cmd_setDocumentUseCSS"))
+  {
+    NS_ENSURE_ARG_POINTER(aParams);
+    nsCOMPtr<nsIHTMLEditor> htmleditor = do_QueryInterface(refCon);
+    if (!htmleditor)
+      return NS_ERROR_INVALID_ARG;
+
+    PRBool isCSS;
+    htmleditor->GetIsCSSEnabled(&isCSS);
+    return aParams->SetBooleanValue(STATE_ATTRIBUTE, isCSS);
+  }
+
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
@@ -445,9 +466,7 @@ nsDocumentStateCommand::GetCommandStateParams(const char *aCommandName,
       nsCOMPtr<nsIDocument> doc = do_QueryInterface(domDoc);
       if (!doc) return NS_ERROR_FAILURE;
 
-      nsCOMPtr<nsIURI> uri;
-      rv = doc->GetDocumentURL(getter_AddRefs(uri));
-      NS_ENSURE_SUCCESS(rv, rv);
+      nsIURI *uri = doc->GetDocumentURL();
       if (!uri) return NS_ERROR_FAILURE;
 
       return aParams->SetISupportsValue(STATE_DATA, (nsISupports*)uri);
