@@ -66,7 +66,6 @@ NS_IMETHODIMP nsHTMLEditor::AddDefaultProperty(nsIAtom *aProperty,
                                             const nsAString & aAttribute, 
                                             const nsAString & aValue)
 {
-  nsresult res = NS_OK;
   nsString outValue;
   PRInt32 index;
   nsString attr(aAttribute);
@@ -81,14 +80,13 @@ NS_IMETHODIMP nsHTMLEditor::AddDefaultProperty(nsIAtom *aProperty,
     PropItem *propItem = new PropItem(aProperty, attr, value);
     mDefaultStyles.AppendElement((void*)propItem);
   }
-  return res;
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsHTMLEditor::RemoveDefaultProperty(nsIAtom *aProperty, 
                                    const nsAString & aAttribute, 
                                    const nsAString & aValue)
 {
-  nsresult res = NS_OK;
   nsString outValue;
   PRInt32 index;
   nsString attr(aAttribute);
@@ -98,7 +96,7 @@ NS_IMETHODIMP nsHTMLEditor::RemoveDefaultProperty(nsIAtom *aProperty,
     if (item) delete item;
     mDefaultStyles.RemoveElementAt(index);
   }
-  return res;
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsHTMLEditor::RemoveAllDefaultProperties()
@@ -120,14 +118,12 @@ NS_IMETHODIMP nsHTMLEditor::SetCSSInlineProperty(nsIAtom *aProperty,
                             const nsAString & aAttribute, 
                             const nsAString & aValue)
 {
-  nsresult res = NS_OK;
   PRBool useCSS;
-
   GetIsCSSEnabled(&useCSS);
   if (useCSS) {
-    res = SetInlineProperty(aProperty, aAttribute, aValue);
+    return SetInlineProperty(aProperty, aAttribute, aValue);
   }
-  return res;
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsHTMLEditor::SetInlineProperty(nsIAtom *aProperty, 
@@ -138,9 +134,8 @@ NS_IMETHODIMP nsHTMLEditor::SetInlineProperty(nsIAtom *aProperty,
   if (!mRules) { return NS_ERROR_NOT_INITIALIZED; }
   ForceCompositionEnd();
 
-  nsresult res;
   nsCOMPtr<nsISelection>selection;
-  res = GetSelection(getter_AddRefs(selection));
+  nsresult res = GetSelection(getter_AddRefs(selection));
   if (NS_FAILED(res)) return res;
   if (!selection) return NS_ERROR_NULL_POINTER;
   nsCOMPtr<nsISelectionPrivate> selPriv(do_QueryInterface(selection));
@@ -305,22 +300,21 @@ nsHTMLEditor::SetInlinePropertyOnTextNode( nsIDOMCharacterData *aTextNode,
                                             const nsAString *aValue)
 {
   if (!aTextNode) return NS_ERROR_NULL_POINTER;
-  nsresult res = NS_OK;
   nsCOMPtr<nsIDOMNode> parent;
-  res = aTextNode->GetParentNode(getter_AddRefs(parent));
+  nsresult res = aTextNode->GetParentNode(getter_AddRefs(parent));
   if (NS_FAILED(res)) return res;
+
   nsAutoString tagString;
   aProperty->ToString(tagString);
   if (!CanContainTag(parent, tagString)) return NS_OK;
   
-  // dont need to do anything if no characters actually selected
+  // don't need to do anything if no characters actually selected
   if (aStartOffset == aEndOffset) return NS_OK;
   
-  nsCOMPtr<nsIDOMNode> tmp, node = do_QueryInterface(aTextNode);
+  nsCOMPtr<nsIDOMNode> node = do_QueryInterface(aTextNode);
   
-  // dont need to do anything if property already set on node
+  // don't need to do anything if property already set on node
   PRBool bHasProp;
-  nsCOMPtr<nsIDOMNode> styleNode;
   PRBool useCSS;
   GetIsCSSEnabled(&useCSS);
 
@@ -335,13 +329,18 @@ nsHTMLEditor::SetInlinePropertyOnTextNode( nsIDOMCharacterData *aTextNode,
                                                        COMPUTED_STYLE_TYPE);
   }
   else
+  {
+    nsCOMPtr<nsIDOMNode> styleNode;
     IsTextPropertySetByContent(node, aProperty, aAttribute, aValue, bHasProp, getter_AddRefs(styleNode));
+  }
+
   if (bHasProp) return NS_OK;
   
   // do we need to split the text node?
   PRUint32 textLen;
   aTextNode->GetLength(&textLen);
   
+  nsCOMPtr<nsIDOMNode> tmp;
   if ( (PRUint32)aEndOffset != textLen )
   {
     // we need to split off back of text node
@@ -438,14 +437,14 @@ nsHTMLEditor::SetInlinePropertyOnNode( nsIDOMNode *aNode,
         res = tmp->GetParentNode(getter_AddRefs(mergeParent));
         if (NS_FAILED(res)) return res;
         if (previousSibling &&
-            nsTextEditUtils::NodeIsType(previousSibling, NS_LITERAL_STRING("span")) &&
+            nsEditor::NodeIsType(previousSibling, nsEditProperty::span) &&
             NodesSameType(tmp, previousSibling))
         {
           res = JoinNodes(previousSibling, tmp, mergeParent);
           if (NS_FAILED(res)) return res;
         }
         if (nextSibling &&
-            nsTextEditUtils::NodeIsType(nextSibling, NS_LITERAL_STRING("span")) &&
+            nsEditor::NodeIsType(nextSibling, nsEditProperty::span) &&
             NodesSameType(tmp, nextSibling))
         {
           res = JoinNodes(tmp, nextSibling, mergeParent);
@@ -756,7 +755,7 @@ nsresult nsHTMLEditor::RemoveStyleInside(nsIDOMNode *aNode,
     }
   }  
   if ( aProperty == nsEditProperty::font &&    // or node is big or small and we are setting font size
-       (NodeIsType(aNode, nsEditProperty::big) || NodeIsType(aNode, nsEditProperty::small)) &&
+       (nsHTMLEditUtils::IsBig(aNode) || nsHTMLEditUtils::IsSmall(aNode)) &&
        aAttribute->Equals(NS_LITERAL_STRING("size"),nsCaseInsensitiveStringComparator()))       
   {
     res = RemoveContainer(aNode);  // if we are setting font size, remove any nested bigs and smalls
@@ -771,11 +770,12 @@ PRBool nsHTMLEditor::IsOnlyAttribute(nsIDOMNode *aNode,
   nsCOMPtr<nsIContent> content = do_QueryInterface(aNode);
   if (!content) return PR_FALSE;  // ooops
   
-  PRInt32 attrCount, i, nameSpaceID;
+  PRInt32 nameSpaceID;
   nsCOMPtr<nsIAtom> attrName, prefix;
-  content->GetAttrCount(attrCount);
+
+  PRUint32 i, attrCount = content->GetAttrCount();
   
-  for (i=0; i<attrCount; i++)
+  for (i = 0; i < attrCount; ++i)
   {
     content->GetAttrNameAt(i, &nameSpaceID, getter_AddRefs(attrName),
                            getter_AddRefs(prefix));
@@ -1134,9 +1134,7 @@ nsHTMLEditor::GetInlinePropertyBase(nsIAtom *aProperty,
       }
       else
       { // handle non-text leaf nodes here
-        PRBool canContainChildren;
-        content->CanContainChildren(canContainChildren);
-        if (canContainChildren)
+        if (content->CanContainChildren())
         {
           skipNode = PR_TRUE;
         }
@@ -1668,7 +1666,7 @@ nsHTMLEditor::RelativeFontChangeOnTextNode( PRInt32 aSizeChange,
   // look for siblings that are correct type of node
   nsCOMPtr<nsIDOMNode> sibling;
   GetPriorHTMLSibling(node, address_of(sibling));
-  if (sibling && NodeIsType(sibling, nodeType))
+  if (sibling && NodeIsType(sibling, (aSizeChange==1) ? nsEditProperty::big : nsEditProperty::small))
   {
     // previous sib is already right kind of inline node; slide this over into it
     res = MoveNode(node, sibling, -1);
@@ -1676,7 +1674,7 @@ nsHTMLEditor::RelativeFontChangeOnTextNode( PRInt32 aSizeChange,
   }
   sibling = nsnull;
   GetNextHTMLSibling(node, address_of(sibling));
-  if (sibling && NodeIsType(sibling, nodeType))
+  if (sibling && NodeIsType(sibling, (aSizeChange==1) ? nsEditProperty::big : nsEditProperty::small))
   {
     // following sib is already right kind of inline node; slide this over into it
     res = MoveNode(node, sibling, 0);
@@ -1794,7 +1792,7 @@ nsHTMLEditor::RelativeFontChangeOnNode( PRInt32 aSizeChange,
     // if we find one, move aNode into it.
     nsCOMPtr<nsIDOMNode> sibling;
     GetPriorHTMLSibling(aNode, address_of(sibling));
-    if (sibling && NodeIsType(sibling, NS_ConvertASCIItoUCS2(aSizeChange==1 ? "big" : "small")))
+    if (sibling && nsEditor::NodeIsType(sibling, (aSizeChange==1 ? nsEditProperty::big : nsEditProperty::small)))
     {
       // previous sib is already right kind of inline node; slide this over into it
       res = MoveNode(aNode, sibling, -1);
@@ -1802,7 +1800,7 @@ nsHTMLEditor::RelativeFontChangeOnNode( PRInt32 aSizeChange,
     }
     sibling = nsnull;
     GetNextHTMLSibling(aNode, address_of(sibling));
-    if (sibling && NodeIsType(sibling, NS_ConvertASCIItoUCS2(aSizeChange==1 ? "big" : "small")))
+    if (sibling && nsEditor::NodeIsType(sibling, (aSizeChange==1 ? nsEditProperty::big : nsEditProperty::small)))
     {
       // following sib is already right kind of inline node; slide this over into it
       res = MoveNode(aNode, sibling, 0);
@@ -1844,7 +1842,7 @@ nsHTMLEditor::GetFontFaceState(PRBool *aMixed, nsAString &outFace)
   if (!aMixed)
       return NS_ERROR_FAILURE;
   *aMixed = PR_TRUE;
-  outFace.SetLength(0);
+  outFace.Truncate();
 
   nsresult res;
   PRBool first, any, all;
@@ -1872,7 +1870,7 @@ nsHTMLEditor::GetFontFaceState(PRBool *aMixed, nsAString &outFace)
   if (!any)
   {
     // there was no font face attrs of any kind.  We are in normal font.
-    outFace.SetLength(0);
+    outFace.Truncate();
     *aMixed = PR_FALSE;
   }
   return res;
@@ -1884,7 +1882,7 @@ nsHTMLEditor::GetFontColorState(PRBool *aMixed, nsAString &aOutColor)
   if (!aMixed)
       return NS_ERROR_NULL_POINTER;
   *aMixed = PR_TRUE;
-  aOutColor.SetLength(0);
+  aOutColor.Truncate();
   
   nsresult res;
   NS_NAMED_LITERAL_STRING(colorStr, "color");
@@ -1902,7 +1900,7 @@ nsHTMLEditor::GetFontColorState(PRBool *aMixed, nsAString &aOutColor)
   if (!any)
   {
     // there was no font color attrs of any kind..
-    aOutColor.SetLength(0);
+    aOutColor.Truncate();
     *aMixed = PR_FALSE;
   }
   return res;
