@@ -233,7 +233,7 @@ END_COM_MAP()
 //
 NPError NPP_Initialize(void)
 {
-    NG_TRACE_METHOD(NPP_Initialize);
+    ATLTRACE(_T("NPP_Initialize()\n"));
     _Module.Lock();
     return NPERR_NO_ERROR;
 }
@@ -245,7 +245,7 @@ NPError NPP_Initialize(void)
 //
 void NPP_Shutdown(void)
 {
-    NG_TRACE_METHOD(NPP_Shutdown);
+    ATLTRACE(_T("NPP_Shutdown()\n"));
 #ifdef MOZ_ACTIVEX_PLUGIN_LIVECONNECT
     liveconnect_Shutdown();
 #endif
@@ -259,7 +259,7 @@ void NPP_Shutdown(void)
 //
 jref NPP_GetJavaClass(void)
 {
-    NG_TRACE_METHOD(NPP_GetJavaClass);
+    ATLTRACE(_T("NPP_GetJavaClass()\n"));
 #ifdef MOZ_ACTIVEX_PLUGIN_LIVECONNECT
     return liveconnect_GetJavaClass();
 #endif
@@ -404,7 +404,7 @@ MozAxAutoPushJSContext::MozAxAutoPushJSContext(JSContext *cx,
         if (NS_SUCCEEDED(mPushResult))
         {
             JSPrincipals* jsprinc;
-            principal->GetJSPrincipals(&jsprinc);
+            principal->GetJsPrincipals(&jsprinc);
 
             mFrame.script = JS_CompileScriptForPrincipals(cx, JS_GetGlobalObject(cx),
                                                           jsprinc, "", 0, "", 1);
@@ -855,8 +855,7 @@ NewControl(const char *pluginType,
                         nsCOMPtr<nsIDocument> doc(do_QueryInterface(DOMdocument));
                         if (doc)
                         {
-                            nsCOMPtr<nsIURI> baseURI;
-                            doc->GetBaseURL(*getter_AddRefs(baseURI));
+                            nsIURI *baseURI = doc->GetBaseURL();
                             if (baseURI)
                             {
                                 nsCAutoString newURL;
@@ -939,7 +938,7 @@ NPError NP_LOADDS NPP_New(NPMIMEType pluginType,
                 char* argv[],
                 NPSavedData* saved)
 {
-    NG_TRACE_METHOD(NPP_New);
+    ATLTRACE(_T("NPP_New()\n"));
 
     // trap duff args
     if (instance == NULL)
@@ -998,7 +997,7 @@ NPError NP_LOADDS NPP_New(NPMIMEType pluginType,
 NPError NP_LOADDS
 NPP_Destroy(NPP instance, NPSavedData** save)
 {
-    NG_TRACE_METHOD(NPP_Destroy);
+    ATLTRACE(_T("NPP_Destroy()\n"));
 
     PluginInstanceData *pData = (PluginInstanceData *) instance->pdata;
     if (pData == NULL)
@@ -1069,7 +1068,7 @@ NPP_Destroy(NPP instance, NPSavedData** save)
 NPError NP_LOADDS
 NPP_SetWindow(NPP instance, NPWindow* window)
 {
-    NG_TRACE_METHOD(NPP_SetWindow);
+    ATLTRACE(_T("NPP_SetWindow()\n"));
 
     // Reject silly parameters
     if (!window)
@@ -1140,7 +1139,7 @@ NPP_NewStream(NPP instance,
               NPBool seekable,
               uint16 *stype)
 {
-    NG_TRACE_METHOD(NPP_NewStream);
+    ATLTRACE(_T("NPP_NewStream()\n"));
 
     if(!instance)
     {
@@ -1162,7 +1161,7 @@ NPP_NewStream(NPP instance,
 void NP_LOADDS
 NPP_StreamAsFile(NPP instance, NPStream *stream, const char* fname)
 {
-    NG_TRACE_METHOD(NPP_StreamAsFile);
+    ATLTRACE(_T("NPP_StreamAsFile()\n"));
 
     if(fname == NULL || fname[0] == NULL)
     {
