@@ -59,19 +59,7 @@ nsUnknownContentTypeHandler::nsUnknownContentTypeHandler( ) {
 nsUnknownContentTypeHandler::~nsUnknownContentTypeHandler( ) { }
 
 
-NS_IMETHODIMP nsUnknownContentTypeHandler::ShowProgressDialog(nsIHelperAppLauncher *aLauncher, nsISupports *aContext ) {
-	nsresult rv = NS_OK;
-
-/* ATENTIE */ //printf("ShowProgressDialog!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n\n\n");
-
-	/* we need a dummy listener because the nsExternalAppHandler class verifies that a progress window has been displayed */
-	nsCOMPtr<nsIWebProgressListener> dummy = new nsWebProgressListener;
-	aLauncher->SetWebProgressListener( dummy );
-
-	return rv;
-	}
-
-NS_IMETHODIMP nsUnknownContentTypeHandler::Show( nsIHelperAppLauncher *aLauncher, nsISupports *aContext ) {
+NS_IMETHODIMP nsUnknownContentTypeHandler::Show( nsIHelperAppLauncher *aLauncher, nsISupports *aContext, PRBool aForced ) {
 	nsresult rv = NS_OK;
 /* ATENTIE */ //printf("Show!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n\n\n");
 
@@ -98,9 +86,9 @@ NS_IMETHODIMP nsUnknownContentTypeHandler::Show( nsIHelperAppLauncher *aLauncher
 	mimeInfo->GetMIMEType( &mimeType );
 
 	nsCOMPtr<nsIURI> aSourceUrl;
-	PRInt64 dummy;
 	nsCOMPtr<nsIFile> aFile;
-	aLauncher->GetDownloadInfo( getter_AddRefs(aSourceUrl), &dummy, getter_AddRefs(aFile) );
+	aLauncher->GetSource( getter_AddRefs(aSourceUrl) );
+	aLauncher->GetTargetFile( getter_AddRefs(aFile) );
 
 	char *url;
 	nsCAutoString specString;
@@ -127,7 +115,7 @@ NS_IMETHODIMP nsUnknownContentTypeHandler::Show( nsIHelperAppLauncher *aLauncher
 	}
 
 /* only Show() method is used - remove this code */
-NS_IMETHODIMP nsUnknownContentTypeHandler::PromptForSaveToFile( nsISupports * aWindowContext, const PRUnichar * aDefaultFile, const PRUnichar * aSuggestedFileExtension, nsILocalFile ** aNewFile ) {
+NS_IMETHODIMP nsUnknownContentTypeHandler::PromptForSaveToFile(nsIHelperAppLauncher * aLauncher, nsISupports * aWindowContext, const PRUnichar * aDefaultFile, const PRUnichar * aSuggestedFileExtension, nsILocalFile ** aNewFile ) {
 /* ATENTIE */ //printf("PromptForSaveToFile!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n\n\n");
 	return NS_OK;
 	}
