@@ -114,7 +114,7 @@ struct nsWatcherWindowEntry {
 
   nsWatcherWindowEntry(nsIDOMWindow *inWindow, nsIWebBrowserChrome *inChrome) {
 #ifdef USEWEAKREFS
-    mWindow = getter_AddRefs(NS_GetWeakReference(inWindow));
+    mWindow = do_GetWeakReference(inWindow);
 #else
     mWindow = inWindow;
 #endif
@@ -189,9 +189,9 @@ private:
   nsWatcherWindowEntry *mCurrentPosition;
 };
 
-NS_IMPL_ADDREF(nsWatcherWindowEnumerator);
-NS_IMPL_RELEASE(nsWatcherWindowEnumerator);
-NS_IMPL_QUERY_INTERFACE1(nsWatcherWindowEnumerator, nsISimpleEnumerator);
+NS_IMPL_ADDREF(nsWatcherWindowEnumerator)
+NS_IMPL_RELEASE(nsWatcherWindowEnumerator)
+NS_IMPL_QUERY_INTERFACE1(nsWatcherWindowEnumerator, nsISimpleEnumerator)
 
 nsWatcherWindowEnumerator::nsWatcherWindowEnumerator(nsWindowWatcher *inWatcher)
   : mWindowWatcher(inWatcher),
@@ -406,8 +406,8 @@ private:
  *********************** nsWindowWatcher ************************
  ****************************************************************/
 
-NS_IMPL_ADDREF(nsWindowWatcher);
-NS_IMPL_RELEASE(nsWindowWatcher);
+NS_IMPL_ADDREF(nsWindowWatcher)
+NS_IMPL_RELEASE(nsWindowWatcher)
 NS_IMPL_QUERY_INTERFACE2(nsWindowWatcher, nsIWindowWatcher, nsPIWindowWatcher)
 
 nsWindowWatcher::nsWindowWatcher() :
@@ -701,10 +701,7 @@ nsWindowWatcher::OpenWindowJS(nsIDOMWindow *aParent,
         newDocShell->GetContentViewer(getter_AddRefs(newContentViewer));
         nsCOMPtr<nsIMarkupDocumentViewer> newMarkupDocViewer(do_QueryInterface(newContentViewer));
         if (doc && newMarkupDocViewer) {
-          nsCAutoString charset;
-          rv = doc->GetDocumentCharacterSet(charset);
-          if (NS_SUCCEEDED(rv))
-            newMarkupDocViewer->SetDefaultCharacterSet(charset);
+          newMarkupDocViewer->SetDefaultCharacterSet(doc->GetDocumentCharacterSet());
         }
       }
     }
@@ -729,7 +726,7 @@ nsWindowWatcher::OpenWindowJS(nsIDOMWindow *aParent,
     JSObject *scriptObject = GetWindowScriptObject(aParent ? aParent : *_retval);
     nsWWJSUtils::nsGetStaticScriptContext(cx, scriptObject,
                                           getter_AddRefs(scriptCX));
-    if (!scriptCX ||
+    if (scriptCX &&
         NS_FAILED(scriptCX->GetSecurityManager(getter_AddRefs(secMan))))
       return NS_ERROR_FAILURE;
 
@@ -737,7 +734,7 @@ nsWindowWatcher::OpenWindowJS(nsIDOMWindow *aParent,
     newDocShell->CreateLoadInfo(getter_AddRefs(loadInfo));
     NS_ENSURE_TRUE(loadInfo, NS_ERROR_FAILURE);
 
-    if (NS_FAILED(rv) || !uriToLoadIsChrome) {
+    if (!uriToLoadIsChrome && secMan) {
       nsCOMPtr<nsIPrincipal> principal;
       if (NS_FAILED(secMan->GetSubjectPrincipal(getter_AddRefs(principal))))
         return NS_ERROR_FAILURE;
@@ -775,11 +772,8 @@ nsWindowWatcher::OpenWindowJS(nsIDOMWindow *aParent,
 
         nsCOMPtr<nsIDocument> doc(do_QueryInterface(document));
         if (doc) { 
-          nsCOMPtr<nsIURI> uri;
-          doc->GetDocumentURL(getter_AddRefs(uri));
-
           // Set the referrer
-          loadInfo->SetReferrer(uri);
+          loadInfo->SetReferrer(doc->GetDocumentURL());
         }
       }
     }
@@ -1133,7 +1127,7 @@ nsWindowWatcher::URIfromURL(const char *aURL,
 
   // failing that, use the given URL unmodified. It had better not be relative.
 
-  nsCOMPtr<nsIURI> baseURI;
+  nsIURI *baseURI = nsnull;
 
   // get baseWindow's document URI
   if (baseWindow) {
@@ -1143,7 +1137,7 @@ nsWindowWatcher::URIfromURL(const char *aURL,
       nsCOMPtr<nsIDocument> doc;
       doc = do_QueryInterface(domDoc);
       if (doc) {
-        doc->GetBaseURL(getter_AddRefs(baseURI));
+        baseURI = doc->GetBaseURL();
       }
     }
   }
