@@ -43,7 +43,6 @@
 #include "nsCOMPtr.h"
 #include "nsXPIDLString.h"
 #include "nsIURI.h"
-#include "nsNetCID.h"
 #include "nsIServiceManager.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIDOMWindow.h"
@@ -53,8 +52,6 @@
 #include "nsIObserverService.h"
 #include "nsString.h"
 
-static NS_DEFINE_CID(kIOServiceCID, NS_IOSERVICE_CID);
-
 // Possible behavior pref values
 #define IMAGE_ACCEPT 0
 #define IMAGE_NOFOREIGN 1
@@ -78,7 +75,7 @@ NS_IMPL_ISUPPORTS4(nsImgManager,
                    nsIImgManager, 
                    nsIContentPolicy,
                    nsIObserver,
-                   nsSupportsWeakReference);
+                   nsSupportsWeakReference)
 
 nsImgManager::nsImgManager()
 {
@@ -129,7 +126,8 @@ NS_IMETHODIMP nsImgManager::ShouldLoad(PRInt32 aContentType,
   nsresult rv = NS_OK;
 
   // we can't do anything w/ out these.
-  if (!aContentLoc || !aContext) return rv;
+  if (!aContentLoc || !aContext)
+    return rv;
 
   if (aContentType == nsIContentPolicy::IMAGE) {
     // we only want to check http, https, ftp
@@ -152,23 +150,26 @@ NS_IMETHODIMP nsImgManager::ShouldLoad(PRInt32 aContentType,
     if (!needToCheck)
       return NS_OK;
 
-    nsCOMPtr<nsIURI> baseURI;
     nsCOMPtr<nsIDocument> doc;
-    nsCOMPtr<nsINodeInfo> nodeinfo;
     nsCOMPtr<nsIContent> content = do_QueryInterface(aContext);
     NS_ASSERTION(content, "no content available");
     if (content) {
-      rv = content->GetDocument(getter_AddRefs(doc));
-      if (NS_FAILED(rv) || !doc) {
-        rv = content->GetNodeInfo(getter_AddRefs(nodeinfo));
-        if (NS_FAILED(rv) || !nodeinfo) return rv;
+      // XXXbz GetOwnerDocument
+      doc = content->GetDocument();
+      if (!doc) {
+        nsINodeInfo *nodeinfo = content->GetNodeInfo();
+        if (!nodeinfo)
+          return NS_OK;
 
-        rv = nodeinfo->GetDocument(getter_AddRefs(doc));
-        if (NS_FAILED(rv) || !doc) return rv;
+        doc = nodeinfo->GetDocument();
+        // XXX what should this code do if there is really no document?
+        if (!doc)
+          return NS_OK;
       }
 
-      rv = doc->GetBaseURL(getter_AddRefs(baseURI));
-      if (NS_FAILED(rv) || !baseURI) return rv;
+      nsIURI *baseURI = doc->GetBaseURL();
+      if (!baseURI)
+        return rv;
 
       nsCOMPtr<nsIDocShell> docshell;
       rv = GetRootDocShell(aWindow, getter_AddRefs(docshell));
@@ -187,7 +188,8 @@ NS_IMETHODIMP nsImgManager::ShouldLoad(PRInt32 aContentType,
       }
       
       rv =  TestPermission(aContentLoc, baseURI, aShouldLoad);
-      if (NS_FAILED(rv)) return rv;
+      if (NS_FAILED(rv))
+        return rv;
     }
   }
   return NS_OK;
@@ -217,6 +219,19 @@ nsImgManager::TestPermission(nsIURI *aCurrentURI,
     return NS_OK;
   }
 
+  // check the permission list first; if we find an entry, it overrides
+  // default prefs. this is new behavior, see bug 184059.
+  if (mPermissionManager) {
+    PRUint32 temp;
+    mPermissionManager->TestPermission(aCurrentURI, "image", &temp);
+
+    // if we found an entry, use it
+    if (temp != nsIPermissionManager::UNKNOWN_ACTION) {
+      *aPermission = (temp != nsIPermissionManager::DENY_ACTION);
+      return NS_OK;
+    }
+  }
+
   if (mBehaviorPref == IMAGE_DENY) {
     *aPermission = PR_FALSE;
     return NS_OK;
@@ -264,20 +279,9 @@ nsImgManager::TestPermission(nsIURI *aCurrentURI,
          firstHost.CharAt(firstHost.Length() - tail.Length() - 1) != '.') || 
         !tail.Equals(firstTail)) {
       *aPermission = PR_FALSE;
-      return NS_OK;
     }
   }
   
-  if (mPermissionManager) {
-    PRUint32 temp;
-    mPermissionManager->TestPermission(aCurrentURI, "image", &temp);
-    // Blacklist for now
-    *aPermission = (temp != nsIPermissionManager::DENY_ACTION);
-  } else {
-    // no permission manager, return ok
-    *aPermission = PR_TRUE;
-  }
-
   return NS_OK;
 }
 
