@@ -46,7 +46,6 @@
 
 ////////////////////////////////////////////////////////////////////////////////
 
-// XXX how do we choose this number?
 #define PL_ARENA_CONST_ALIGN_MASK 3
 #include "plarena.h"
 
@@ -155,7 +154,7 @@ nsPermissionEnumerator::Prefetch()
     if (entry) {
       // see if we've found it
       permission = entry->GetPermission(mTypeIndex);
-      if (permission != nsIPermissionManager::UNKNOWN_ACTION) {
+      if (permission != nsIPermissionManager::UNKNOWN_ACTION && mTypeArray[mTypeIndex]) {
         mNextPermission = new nsPermission(entry->GetHost(), 
                                            nsDependentCString(mTypeArray[mTypeIndex]),
                                            permission);
@@ -229,25 +228,17 @@ nsPermissionManager::Add(nsIURI     *aURI,
   nsresult rv;
 
   nsCAutoString hostPort;
-  aURI->GetHostPort(hostPort);
-  if (hostPort.IsEmpty()) {
-    // Nothing to add
-    return NS_OK;
-  }
+  rv = GetHostPort(aURI, hostPort);
+  // no host doesn't mean an error. just return the default
+  if (NS_FAILED(rv)) return NS_OK;
 
   PRInt32 typeIndex = GetTypeIndex(aType, PR_TRUE);
   if (typeIndex == -1 || aPermission >= NUMBER_OF_PERMISSIONS)
     return NS_ERROR_FAILURE;
 
-  rv = AddInternal(hostPort, typeIndex, aPermission);
+  rv = AddInternal(hostPort, typeIndex, aPermission, PR_TRUE);
   if (NS_FAILED(rv)) return rv;
 
-  // Notify permission manager dialog to update its display
-  //
-  // This used to be conditional, but now we use AddInternal 
-  // for cases when no notification is needed
-  NotifyObservers(hostPort);
-
   mChangedList = PR_TRUE;
   Write();
   return NS_OK;
@@ -257,8 +248,9 @@ nsPermissionManager::Add(nsIURI     *aURI,
 // bounds check aTypeIndex or aPermission. These are up to the caller.
 nsresult
 nsPermissionManager::AddInternal(const nsAFlatCString &aHost,
-                                 PRInt32 aTypeIndex,
-                                 PRUint32 aPermission)
+                                 PRInt32               aTypeIndex,
+                                 PRUint32              aPermission,
+                                 PRBool                aNotify)
 {
   if (!gHostArena) {
     gHostArena = new PLArenaPool;
@@ -267,7 +259,7 @@ nsPermissionManager::AddInternal(const nsAFlatCString &aHost,
     PL_INIT_ARENA_POOL(gHostArena, "PermissionHostArena", HOST_ARENA_SIZE);
   }
 
-  // When an entry already exists, AddEntry will return that, instead
+  // When an entry already exists, PutEntry will return that, instead
   // of adding a new one
   nsHostEntry *entry = mHostTable.PutEntry(aHost.get());
   if (!entry) return NS_ERROR_FAILURE;
@@ -275,8 +267,39 @@ nsPermissionManager::AddInternal(const nsAFlatCString &aHost,
   if (entry->PermissionsAreEmpty()) {
     ++mHostCount;
   }
+
+  PRUint32 oldPermission = entry->GetPermission(aTypeIndex);
   entry->SetPermission(aTypeIndex, aPermission);
 
+  // check whether we are deleting, adding, or changing a permission,
+  // so we can notify observers. this would be neater to do in Add(),
+  // but we need to do it here because we only know what type of notification
+  // to send (removal, addition, or change) after we've done the hash
+  // lookup.
+  if (aNotify) {
+    if (aPermission == nsIPermissionManager::UNKNOWN_ACTION) {
+      if (oldPermission != nsIPermissionManager::UNKNOWN_ACTION)
+        // deleting
+        NotifyObserversWithPermission(aHost,
+                                      mTypeArray[aTypeIndex],
+                                      oldPermission,
+                                      NS_LITERAL_STRING("deleted").get());
+    } else {
+      if (oldPermission == nsIPermissionManager::UNKNOWN_ACTION)
+        // adding
+        NotifyObserversWithPermission(aHost,
+                                      mTypeArray[aTypeIndex],
+                                      aPermission,
+                                      NS_LITERAL_STRING("added").get());
+      else
+        // changing
+        NotifyObserversWithPermission(aHost,
+                                      mTypeArray[aTypeIndex],
+                                      aPermission,
+                                      NS_LITERAL_STRING("changed").get());
+    }
+  }
+
   return NS_OK;
 }
 
@@ -292,6 +315,9 @@ nsPermissionManager::Remove(const nsACString &aHost,
 
   nsHostEntry* entry = mHostTable.GetEntry(PromiseFlatCString(aHost).get());
   if (entry) {
+    // cache the old permission before we delete it, to notify observers
+    PRUint32 oldPermission = entry->GetPermission(typeIndex);
+
     entry->SetPermission(typeIndex, nsIPermissionManager::UNKNOWN_ACTION);
 
     // If no more types are present, remove the entry
@@ -303,7 +329,11 @@ nsPermissionManager::Remove(const nsACString &aHost,
     Write();
 
     // Notify Observers
-    NotifyObservers(aHost);
+    if (oldPermission != nsIPermissionManager::UNKNOWN_ACTION)
+      NotifyObserversWithPermission(PromiseFlatCString(aHost),
+                                    aType,
+                                    oldPermission,
+                                    NS_LITERAL_STRING("deleted").get());
   }
   return NS_OK;
 }
@@ -312,6 +342,7 @@ NS_IMETHODIMP
 nsPermissionManager::RemoveAll()
 {
   RemoveAllFromMemory();
+  NotifyObservers(nsnull, NS_LITERAL_STRING("cleared").get());
   Write();
   return NS_OK;
 }
@@ -329,11 +360,9 @@ nsPermissionManager::TestPermission(nsIURI     *aURI,
   *aPermission = nsIPermissionManager::UNKNOWN_ACTION;
 
   nsCAutoString hostPort;
-  aURI->GetHostPort(hostPort);
-  // Don't error on no host. Just return UNKNOWN_ACTION as permission.
-  if (hostPort.IsEmpty()) {
-    return NS_OK;
-  }
+  nsresult rv = GetHostPort(aURI, hostPort);
+  // no host doesn't mean an error. just return the default
+  if (NS_FAILED(rv)) return NS_OK;
   
   PRInt32 typeIndex = GetTypeIndex(aType, PR_FALSE);
   // If type == -1, the type isn't known,
@@ -497,23 +526,34 @@ nsPermissionManager::GetTypeIndex(const char *aType,
   return firstEmpty;
 }
 
-
-// broadcast a notification that a permission has changed
-nsresult
-nsPermissionManager::NotifyObservers(const nsACString &aHost)
+// wrapper function for mangling (host,type,perm) triplet into an nsIPermission.
+void
+nsPermissionManager::NotifyObserversWithPermission(const nsACString &aHost,
+                                                   const char       *aType,
+                                                   PRUint32          aPermission,
+                                                   const PRUnichar  *aData)
 {
-  if (mObserverService) {
-    return mObserverService->NotifyObservers(NS_STATIC_CAST(nsIPermissionManager *, this),
-                                             kPermissionChangeNotification,
-                                             NS_ConvertUTF8toUCS2(aHost).get());
-  }
-  return NS_ERROR_FAILURE;
+  nsCOMPtr<nsIPermission> permission =
+    new nsPermission(aHost, nsDependentCString(aType), aPermission);
+  if (permission)
+    NotifyObservers(permission, aData);
 }
 
-// Note:
-// We don't do checkbox states here anymore.
-// When a consumer wants it back, that is up to the consumer, not this backend
-// For cookies, it is now done with a persist in the dialog xul file.
+// notify observers that the permission list changed. there are four possible
+// values for aData:
+// "deleted" means a permission was deleted. aPermission is the deleted permission.
+// "added"   means a permission was added. aPermission is the added permission.
+// "changed" means a permission was altered. aPermission is the new permission.
+// "cleared" means the entire permission list was cleared. aPermission is null.
+void
+nsPermissionManager::NotifyObservers(nsIPermission   *aPermission,
+                                     const PRUnichar *aData)
+{
+  if (mObserverService)
+    mObserverService->NotifyObservers(aPermission,
+                                      kPermissionChangeNotification,
+                                      aData);
+}
 
 static const char kTab = '\t';
 static const char kNew = '\n';
@@ -528,6 +568,11 @@ nsPermissionManager::Read()
   nsresult rv;
   
   if (!mPermissionsFile) {
+    // For backwards compatibility, add those types when not yet set.
+    mTypeArray[0] = PL_strdup("cookie");
+    mTypeArray[1] = PL_strdup("image");
+    mTypeArray[2] = PL_strdup("popup");
+
     return NS_ERROR_FAILURE;
   }
 
@@ -568,6 +613,17 @@ nsPermissionManager::Read()
       if (!PR_sscanf(buffer.get() + 1, "%u", &type) || type >= NUMBER_OF_TYPES) {
         continue;
       }
+
+      // Older versions of mozilla can't parse the permission string lines.
+      // They will put them back like '%0 0F' instead of '%0 cookie'
+      // Ignore those lines, and revert to the defaults later.
+      // XXX This means that when the user has additional types besides the
+      // default, those will be lost after using an old version with the
+      // new profile, and then going back to a new version.
+      if (!PL_strcmp(buffer.get() + stringIndex, "0F"))
+        continue;
+
+      NS_ASSERTION(GetTypeIndex(buffer.get() + stringIndex, PR_FALSE) == -1, "Corrupt cookperm.txt file");
       mTypeArray[type] = PL_strdup(buffer.get() + stringIndex);
 
       continue;
@@ -630,19 +686,21 @@ nsPermissionManager::Read()
     
       // Ignore @@@ as host. Old style checkbox status
       if (!permissionString.IsEmpty() && !host.Equals(NS_LITERAL_CSTRING("@@@@"))) {
-        rv = AddInternal(host, type, permission);
+        rv = AddInternal(host, type, permission, PR_FALSE);
         if (NS_FAILED(rv)) return rv;
       }
 
     }
   }
 
-  mChangedList = PR_FALSE;
+  // Old files, without the part that defines the types, assume 0 is cookie
+  // etc. So add those types, but make sure not to overwrite types from new
+  // style files.
+  GetTypeIndex("cookie", PR_TRUE);
+  GetTypeIndex("image", PR_TRUE);
+  GetTypeIndex("popup", PR_TRUE);
 
-  // For backwards compatibility, add those types when not yet set.
-  if (!mTypeArray[0]) mTypeArray[0] = PL_strdup("cookie");
-  if (!mTypeArray[1]) mTypeArray[1] = PL_strdup("image");
-  if (!mTypeArray[2]) mTypeArray[2] = PL_strdup("popup");
+  mChangedList = PR_FALSE;
 
   return NS_OK;
 }
@@ -763,3 +821,22 @@ nsPermissionManager::Write()
   return NS_OK;
 }
 
+nsresult
+nsPermissionManager::GetHostPort(nsIURI *aURI, nsACString &aResult)
+{
+  NS_ASSERTION(aURI, "could not get uri");
+
+  aURI->GetHostPort(aResult);
+
+  // If there is no host, use the scheme, and prepend "scheme:",
+  // to make sure it isn't a host or something.
+  if (aResult.IsEmpty()) {
+    aURI->GetScheme(aResult);
+    if (aResult.IsEmpty()) {
+      // still empty. Return error.
+      return NS_ERROR_FAILURE;
+    }
+    aResult = NS_LITERAL_CSTRING("scheme:") + aResult;
+  }
+  return NS_OK;
+}
