@@ -68,7 +68,7 @@ inDOMUtils::~inDOMUtils()
 {
 }
 
-NS_IMPL_ISUPPORTS1(inDOMUtils, inIDOMUtils);
+NS_IMPL_ISUPPORTS1(inDOMUtils, inIDOMUtils)
 
 ///////////////////////////////////////////////////////////////////////////////
 // inIDOMUtils
@@ -178,7 +178,8 @@ inDOMUtils::GetCSSStyleRules(nsIDOMElement *aElement,
     cssRule = do_QueryInterface(srule);
     if (cssRule) {
       cssRule->GetDOMRule(getter_AddRefs(domRule));
-      rules->InsertElementAt(domRule, 0);
+      if (domRule)
+        rules->InsertElementAt(domRule, 0);
     }
   }
 
@@ -218,8 +219,7 @@ inDOMUtils::GetBindingURLs(nsIDOMElement *aElement, nsISimpleEnumerator **_retva
   if (!doc1) return NS_OK;
   
   nsCOMPtr<nsIDocument> doc = do_QueryInterface(doc1);
-  nsCOMPtr<nsIBindingManager> bindingManager;
-  doc->GetBindingManager(getter_AddRefs(bindingManager));
+  nsIBindingManager *bindingManager = doc->GetBindingManager();
   if (!bindingManager) return NS_OK;
   
   nsCOMPtr<nsIContent> content = do_QueryInterface(aElement);
