@@ -145,7 +145,7 @@ inDOMView::~inDOMView()
 NS_IMPL_ISUPPORTS3(inDOMView,
                    inIDOMView,
                    nsITreeView,
-                   nsIDocumentObserver);
+                   nsIDocumentObserver)
 
 ////////////////////////////////////////////////////////////////////////
 // inIDOMView
@@ -316,11 +316,8 @@ inDOMView::GetCellProperties(PRInt32 row, const PRUnichar *colID, nsISupportsArr
   if (!node) return NS_ERROR_FAILURE;
 
   nsCOMPtr<nsIContent> content = do_QueryInterface(node->node);
-  if (content) {
-    nsCOMPtr<nsIContent> bparent;
-    content->GetBindingParent(getter_AddRefs(bparent));
-    if (bparent)
-      properties->AppendElement(kAnonymousAtom);
+  if (content && content->GetBindingParent()) {
+    properties->AppendElement(kAnonymousAtom);
   }
 
   PRUint16 nodeType;
@@ -417,7 +414,7 @@ inDOMView::GetCellText(PRInt32 row, const PRUnichar *colID, nsAString& _retval)
   } else if (col.Equals(NS_LITERAL_STRING("colNodeValue")))
     domNode->GetNodeValue(_retval);
   else {
-    if (Substring(col, 0, 4).Equals(NS_LITERAL_STRING("col@"))) {
+    if (StringBeginsWith(col, NS_LITERAL_STRING("col@"))) {
       nsCOMPtr<nsIDOMElement> el = do_QueryInterface(node->node);
       if (el) {
         nsAutoString attr;
@@ -638,7 +635,7 @@ NS_IMPL_NSIDOCUMENTOBSERVER_STYLE_STUB(inDOMView)
 
 NS_IMETHODIMP
 inDOMView::AttributeChanged(nsIDocument *aDocument, nsIContent* aContent, PRInt32 aNameSpaceID,
-                            nsIAtom* aAttribute, PRInt32 aModType, nsChangeHint aHint)
+                            nsIAtom* aAttribute, PRInt32 aModType)
 {
   if (!mTree) {
     return NS_ERROR_FAILURE;
@@ -754,8 +751,7 @@ inDOMView::ContentAppended(nsIDocument *aDocument,
     return NS_ERROR_FAILURE;
   }
 
-  nsCOMPtr<nsIContent> child;
-  aContainer->ChildAt(aNewIndexInContainer, getter_AddRefs(child));
+  nsIContent *child = aContainer->GetChildAt(aNewIndexInContainer);
 
   return ContentInserted(aDocument, aContainer, child,
                          aNewIndexInContainer);
