@@ -26,26 +26,63 @@
  * 1999-08-15 rginda@ndcico.com           v1.0
  */
 
-var DEBUG = true;
+var utils = new Object();
 
-var dumpln;
+var DEBUG = true;
 var dd;
 
-if (typeof document == "undefined") /* in xpcshell */
-    dumpln = print;
-else
-    if (typeof dump == "function")
-        dumpln = function (str) {dump (str + "\n");}
-    else if (jsenv.HAS_RHINO)
-        dumpln = function (str) {var out = java.lang.System.out;
-                                 out.println(str); out.flush(); }
-    else
-        dumpln = function () {} /* no suitable function */
-
-if (DEBUG)
-    dd = dumpln;
-else
+if (DEBUG) {
+    var _dd_pfx = "";
+    var _dd_singleIndent = "  ";
+    var _dd_indentLength = _dd_singleIndent.length;
+    var _dd_currentIndent = "";
+    var _dd_lastDumpWasOpen = false;
+    var _dd_timeStack = new Array();
+    var _dd_disableDepth = Number.MAX_VALUE;
+    var _dd_currentDepth = 0;
+    dd = function _dd(str) {
+             if (typeof str != "string") {
+                 dump(str + "\n");
+             } else if (str == "") {
+                 dump("<empty-string>\n");
+             } else if (str[str.length - 1] == "{") {
+                 ++_dd_currentDepth;
+                 if (_dd_currentDepth >= _dd_disableDepth)
+                     return;
+                 if (str.indexOf("OFF") == 0)
+                     _dd_disableDepth = _dd_currentDepth;
+                 _dd_timeStack.push (new Date());
+                 if (_dd_lastDumpWasOpen)
+                     dump("\n");
+                 dump (_dd_pfx + _dd_currentIndent + str);
+                 _dd_currentIndent += _dd_singleIndent;
+                 _dd_lastDumpWasOpen = true;
+             } else if (str[0] == "}") {
+                 if (--_dd_currentDepth >= _dd_disableDepth)
+                     return;
+                 _dd_disableDepth = Number.MAX_VALUE;
+                 var sufx = (new Date() - _dd_timeStack.pop()) / 1000 + " sec";
+                 _dd_currentIndent = 
+                     _dd_currentIndent.substr(0, _dd_currentIndent.length -
+                                              _dd_indentLength);
+                 if (_dd_lastDumpWasOpen)
+                     dump(str + " " + sufx + "\n");
+                 else
+                     dump(_dd_pfx + _dd_currentIndent + str + " " +
+                          sufx + "\n");
+                 _dd_lastDumpWasOpen = false;
+             } else {
+                 if (_dd_currentDepth >= _dd_disableDepth)
+                     return;
+                 if (_dd_lastDumpWasOpen)
+                     dump("\n");
+                 dump(_dd_pfx + _dd_currentIndent + str + "\n");
+                 _dd_lastDumpWasOpen = false;
+             }    
+         }
+} else {
     dd = function (){};
+}
 
 var jsenv = new Object();
 jsenv.HAS_SECURITYMANAGER = ((typeof netscape == "object") &&
@@ -140,7 +177,15 @@ function dumpObjectTree (o, recurse, compress, level)
                 break;
 
             case "object":
-                s += pfx + tee + i + " (object)\n";
+                s += pfx + tee + i + " (object)";
+                if (o[i] == null)
+                {
+                    s += " null\n";
+                    break;
+                }
+                
+                s += "\n";
+                
                 if (!compress)
                     s += pfx + "|\n";
                 if ((i != "parent") && (recurse))
@@ -176,6 +221,83 @@ function dumpObjectTree (o, recurse, compress, level)
     
 }
 
+function ecmaEscape(str)
+{
+    function replaceNonPrintables(ch)
+    {
+        rv = ch.charCodeAt().toString(16);
+        if (rv.length == 1)
+            rv = "0" + rv;
+        else if (rv.length == 3)
+            rv = "u0" + rv;
+        else if (rv.length == 4)
+            rv = "u" + rv;
+      
+        return "%" + rv;
+    };
+
+    // Replace any character that is not in the 69 character set
+    // [ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789@*_+-./]
+    // with an escape sequence.  Two digit sequences in the form %XX are used
+    // for characters whose codepoint is less than 255, %uXXXX for all others.
+    // See section B.2.1 of ECMA-262 rev3 for more information.
+    return str.replace(/[^A-Za-z0-9@*_+.\-\/]/g, replaceNonPrintables);
+}
+
+function ecmaUnescape(str)
+{
+    function replaceEscapes(seq)
+    {
+        var ary = seq.match(/([\da-f]{1,2})(.*)|u([\da-f]{1,4})/);
+        if (!ary)
+            return "<ERROR>";
+
+        if (ary[1])
+        {
+            // two digit escape, possibly with cruft after
+            rv = String.fromCharCode(parseInt(ary[1], 16)) + ary[2];
+        }
+        else
+        {
+            // four digits, no cruft
+            rv = String.fromCharCode(parseInt(ary[3], 16));
+        }
+
+        return rv;
+    };
+
+    // Replace the escape sequences %X, %XX, %uX, %uXX, %uXXX, and %uXXXX with
+    // the characters they represent, where X is a hexadecimal digit.
+    // See section B.2.2 of ECMA-262 rev3 for more information.
+    return str.replace(/%u?([\da-f]{1,4})/ig, replaceEscapes);
+}
+
+function replaceVars(str, vars)
+{
+    // replace "string $with a $variable", with
+    // "string " + vars["with"] + " with a " + vars["variable"]
+
+    function doReplace(symbol)
+    {
+        var name = symbol.substr(1);
+        if (name in vars)
+            return vars[name];
+        
+        return "$" + name;
+    };
+    
+    return str.replace(/(\$\w[\w\d\-]+)/g, doReplace);
+}
+    
+function formatException (ex)
+{
+    if (ex instanceof Error)
+        return getMsg (MSG_FMT_JSEXCEPTION, [ex.name, ex.message, ex.fileName, 
+                                             ex.lineNumber]);
+
+    return String(ex);
+}
+
 /*
  * Clones an existing object (Only the enumerable properties
  * of course.) use as a function..
@@ -185,7 +307,7 @@ function dumpObjectTree (o, recurse, compress, level)
  */
 function Clone (obj)
 {
-    robj = new Object();
+    var robj = new Object();
 
     for (var p in obj)
         robj[p] = obj[p];
@@ -194,6 +316,20 @@ function Clone (obj)
     
 }
 
+function Copy(source, dest, overwrite)
+{
+    if (!dest)
+        dest = new Object();
+    
+    for (var p in source)
+    {
+        if (overwrite || !(p in dest))
+            dest[p] = source[p];
+    }
+    
+    return dest;
+}
+
 /*
  * matches a real object against one or more pattern objects.
  * if you pass an array of pattern objects, |negate| controls wether to check
@@ -270,6 +406,16 @@ function matchEntry (partialName, list)
     
 }
 
+function encodeChar(ch)
+{
+   return "%" + ch.charCodeAt(0).toString(16);
+}
+
+function escapeFileName(fileName)
+{
+    return fileName.replace(/[^\w\d.,#-_]/g, encodeChar);
+}
+
 function getCommonPfx (list)
 {
     var pfx = list[0];
@@ -296,6 +442,23 @@ function getCommonPfx (list)
 
 }
 
+function openTopWin (url)
+{
+    return openDialog (getBrowserURL(), "_blank", "chrome,all,dialog=no", url);
+}
+    
+function getWindowByType (windowType)
+{
+    const MEDIATOR_CONTRACTID =
+        "@mozilla.org/appshell/window-mediator;1";
+    const nsIWindowMediator  = Components.interfaces.nsIWindowMediator;
+
+    var windowManager =
+        Components.classes[MEDIATOR_CONTRACTID].getService(nsIWindowMediator);
+
+    return windowManager.getMostRecentWindow(windowType);
+}
+
 function renameProperty (obj, oldname, newname)
 {
 
@@ -334,6 +497,22 @@ function newObject(contractID, iface)
     
 }
 
+function getContentWindow(frame)
+{
+    try
+    {
+        if (!frame || !("contentWindow" in frame))
+            return false;
+
+        return frame.contentWindow;
+    }
+    catch (ex)
+    {
+        // throws exception is contentWindow is gone
+        return null;
+    }
+}
+
 function getPriv (priv)
 {
     if (!jsenv.HAS_SECURITYMANAGER)
@@ -355,6 +534,16 @@ function getPriv (priv)
     
 }
 
+function len(o)
+{
+    var l = 0;
+    
+    for (var p in o)
+        ++l;
+    
+    return l;
+}
+
 function keys (o)
 {
     var rv = new Array();
@@ -389,13 +578,13 @@ function formatDateOffset (offset, format)
     {
         var ary = new Array();
         if (days > 0)
-            ary.push (getMsg("days", days));
+            ary.push (getMsg(MSG_DAYS, days));
         if (hours > 0)
-            ary.push (getMsg("hours", hours));
+            ary.push (getMsg(MSG_HOURS, hours));
         if (minutes > 0)
-            ary.push (getMsg("minutes", minutes));
+            ary.push (getMsg(MSG_MINUTES, minutes));
         if (seconds > 0 || offset == 0)
-            ary.push (getMsg("seconds", seconds));
+            ary.push (getMsg(MSG_SECONDS, seconds));
 
         format = ary.join(", ");
     }
@@ -410,6 +599,11 @@ function formatDateOffset (offset, format)
     return format;
 }
 
+function arrayHasElementAt(ary, i)
+{
+    return typeof ary[i] != "undefined";
+}
+
 function arrayContains (ary, elem)
 {
     return (arrayIndexOf (ary, elem) != -1);
@@ -426,38 +620,12 @@ function arrayIndexOf (ary, elem)
     
 function arrayInsertAt (ary, i, o)
 {
-
     ary.splice (i, 0, o);
-
-    /* doh, forgot about that 'splice' thing
-    if (ary.length < i)
-    {
-        this[i] = o;
-        return;
-    }
-
-    for (var j = ary.length; j > i; j--)
-        ary[j] = ary[j - 1];
-
-    ary[i] = o;
-    */
 }
 
 function arrayRemoveAt (ary, i)
 {
-
     ary.splice (i, 1);
-
-    /* doh, forgot about that 'splice' thing
-    if (ary.length < i)
-        return false;
-
-    for (var j = i; j < ary.length; j++)
-        ary[j] = ary[j + 1];
-
-    ary.length--;
-    */
-
 }
 
 /* length should be an even number >= 6 */
@@ -636,3 +804,98 @@ function map(func, ary) {
 
 }
 
+function getSpecialDirectory(name)
+{
+    if (!("directoryService" in utils))
+    {
+        const DS_CTR = "@mozilla.org/file/directory_service;1";
+        const nsIProperties = Components.interfaces.nsIProperties;
+
+        utils.directoryService =
+            Components.classes[DS_CTR].getService(nsIProperties);
+    }
+
+    return utils.directoryService.get(name, Components.interfaces.nsIFile);
+}
+
+function getFileFromURLSpec(url)
+{
+    const FILE_CTRID = "@mozilla.org/network/protocol;1?name=file";
+    const nsIFileProtocolHandler = Components.interfaces.nsIFileProtocolHandler;
+    
+    var handler = Components.classes[FILE_CTRID].createInstance();
+    handler = handler.QueryInterface(nsIFileProtocolHandler);
+    return handler.getFileFromURLSpec(url);
+}
+
+function getURLSpecFromFile (file)
+{
+    if (!file)
+        return null;
+
+    const IOS_CTRID = "@mozilla.org/network/io-service;1";
+    const LOCALFILE_CTRID = "@mozilla.org/file/local;1";
+
+    const nsIIOService = Components.interfaces.nsIIOService;
+    const nsILocalFile = Components.interfaces.nsILocalFile;
+    
+    if (typeof file == "string")
+    {
+        var fileObj =
+            Components.classes[LOCALFILE_CTRID].createInstance(nsILocalFile);
+        fileObj.initWithPath(file);
+        file = fileObj;
+    }
+    
+    var service = Components.classes[IOS_CTRID].getService(nsIIOService);
+    /* In sept 2002, bug 166792 moved this method to the nsIFileProtocolHandler
+     * interface, but we need to support older versions too. */
+    if ("getURLSpecFromFile" in service)
+        return service.getURLSpecFromFile(file);
+
+    var nsIFileProtocolHandler = Components.interfaces.nsIFileProtocolHandler;
+    var fileHandler = service.getProtocolHandler("file");
+    fileHandler = fileHandler.QueryInterface(nsIFileProtocolHandler);
+    return fileHandler.getURLSpecFromFile(file);
+}
+
+function alert(msg, parent, title)
+{
+    var PROMPT_CTRID = "@mozilla.org/embedcomp/prompt-service;1";
+    var nsIPromptService = Components.interfaces.nsIPromptService;
+    var ps = Components.classes[PROMPT_CTRID].createInstance(nsIPromptService);
+    if (!parent)
+        parent = window;
+    if (!title)
+        title = MSG_ALERT;
+    ps.alert (parent, title, msg);
+}
+
+function confirm(msg, parent, title)
+{
+    var PROMPT_CTRID = "@mozilla.org/embedcomp/prompt-service;1";
+    var nsIPromptService = Components.interfaces.nsIPromptService;
+    var ps = Components.classes[PROMPT_CTRID].createInstance(nsIPromptService);
+    if (!parent)
+        parent = window;
+    if (!title)
+        title = MSG_CONFIRM;
+    return ps.confirm (parent, title, msg);
+}
+
+function prompt(msg, initial, parent, title)
+{
+    var PROMPT_CTRID = "@mozilla.org/embedcomp/prompt-service;1";
+    var nsIPromptService = Components.interfaces.nsIPromptService;
+    var ps = Components.classes[PROMPT_CTRID].createInstance(nsIPromptService);
+    if (!parent)
+        parent = window;
+    if (!title)
+        title = MSG_PROMPT;
+    rv = { value: initial };
+
+    if (!ps.prompt (parent, title, msg, rv, null, {value: null}))
+        return null;
+
+    return rv.value
+}
