@@ -34,6 +34,7 @@
 #include "nsIDOMElement.h"
 #include "nsIDOMNamedNodeMap.h"
 #include "nsIDOMNode.h"
+#include "nsIDOM3Node.h"
 #include "nsIDOMNodeList.h"
 #include "txAtoms.h"
 #include "nsString.h"
@@ -343,9 +344,7 @@ PRInt32 Node::lookupNamespaceID(nsIAtom* aPrefix)
             return nsId;
         }
 
-        nsCOMPtr<nsIContent> temp(elem);
-        rv = temp->GetParent(*getter_AddRefs(elem));
-        NS_ENSURE_SUCCESS(rv, kNameSpaceID_Unknown);
+        elem = elem->GetParent();
     }
 
     if (aPrefix == txXMLAtoms::xmlns) {
@@ -458,17 +457,17 @@ Node::OrderInfo* Node::getOrderInfo()
             PRInt32 thisNS = getNamespaceID();
 
             // find this attribute in parents attrlist
-            PRInt32 count, i;
+            PRUint32 count, i;
             nsCOMPtr<nsIContent> owner = do_QueryInterface(parent->getNSObj());
-            owner->GetAttrCount(count);
+            count = owner->GetAttrCount();
             for (i = 0; i < count; ++i) {
                 nsCOMPtr<nsIAtom> attName;
                 nsCOMPtr<nsIAtom> attPrefix;
                 PRInt32 attNS;
                 owner->GetAttrNameAt(i,
-                                     attNS,
-                                     *getter_AddRefs(attName),
-                                     *getter_AddRefs(attPrefix));
+                                     &attNS,
+                                     getter_AddRefs(attName),
+                                     getter_AddRefs(attPrefix));
 
                 if (attName == thisName && attNS == thisNS) {
                     mOrderInfo->mOrder[lastElem] = i + kTxAttrIndexOffset;
@@ -487,8 +486,7 @@ Node::OrderInfo* Node::getOrderInfo()
             nsCOMPtr<nsIContent> parentCont(do_QueryInterface(parentObj));
             // Is parent an nsIContent
             if (parentCont) {
-                PRInt32 index;
-                parentCont->IndexOf(cont, index);
+                PRInt32 index = parentCont->IndexOf(cont);
                 mOrderInfo->mOrder[lastElem] = index + kTxChildIndexOffset;
                 return mOrderInfo;
             }
@@ -496,8 +494,7 @@ Node::OrderInfo* Node::getOrderInfo()
             nsCOMPtr<nsIDocument> parentDoc(do_QueryInterface(parentObj));
             // Is parent an nsIDocument
             if (parentDoc) {
-                PRInt32 index;
-                parentDoc->IndexOf(cont, index);
+                PRInt32 index = parentDoc->IndexOf(cont);
                 mOrderInfo->mOrder[lastElem] = index + kTxChildIndexOffset;
                 return mOrderInfo;
             }
