@@ -42,13 +42,13 @@
 #include "nsIAtom.h"
 #include "TxObject.h"
 #include "nsAutoPtr.h"
+#include "ExprResult.h"
 
 /*
   XPath class definitions.
   Much of this code was ported from XSL:P.
 */
 
-class ExprResult;
 class NodeSet;
 class txIParseContext;
 class txIMatchContext;
@@ -75,7 +75,8 @@ public:
      * for evaluation
      * @return the result of the evaluation
     **/
-    virtual ExprResult* evaluate(txIEvalContext* aContext) = 0;
+    virtual nsresult evaluate(txIEvalContext* aContext,
+                              txAExprResult** aResult) = 0;
 
     /**
      * Returns the String representation of this Expr.
@@ -90,7 +91,7 @@ public:
 }; //-- Expr
 
 #define TX_DECL_EVALUATE \
-    ExprResult* evaluate(txIEvalContext* aContext)
+    nsresult evaluate(txIEvalContext* aContext, txAExprResult** aResult)
 
 #define TX_DECL_EXPR \
     TX_DECL_EVALUATE; \
@@ -107,31 +108,36 @@ public:
 class FunctionCall : public Expr {
 
 public:
-
-    static const nsString INVALID_PARAM_COUNT;
-    static const nsString INVALID_PARAM_VALUE;
-
     virtual ~FunctionCall();
 
     /**
      * Virtual methods from Expr 
     **/
-    virtual ExprResult* evaluate(txIEvalContext* aContext) = 0;
     void toString(nsAString& aDest);
 
     /**
-     * Adds the given parameter to this FunctionCall's parameter list
-     * @param expr the Expr to add to this FunctionCall's parameter list
-    **/
+     * Adds the given parameter to this FunctionCall's parameter list.
+     * The ownership of the given Expr is passed over to the FunctionCall,
+     * even on failure.
+     * @param aExpr the Expr to add to this FunctionCall's parameter list
+     * @return nsresult indicating out of memory
+     */
     nsresult addParam(Expr* aExpr);
 
-    /*
+    /**
+     * Check if the number of parameters falls within a range.
+     *
+     * @param aParamCountMin minimum number of required parameters.
+     * @param aParamCountMax maximum number of parameters. If aParamCountMax
+     *                       is negative the maximum number is not checked.
+     * @return boolean representing whether the number of parameters falls
+     *         within the expected range or not.
+     *
      * XXX txIEvalContext should be txIParseContest, bug 143291
      */
-    virtual MBool requireParams(int aParamCountMin, txIEvalContext* aContext);
-    virtual MBool requireParams(int aParamCountMin,
-                                int aParamCountMax,
-                                txIEvalContext* aContext);
+    virtual PRBool requireParams(PRInt32 aParamCountMin,
+                                 PRInt32 aParamCountMax,
+                                 txIEvalContext* aContext);
 
 protected:
 
@@ -158,9 +164,10 @@ protected:
 
     /*
      * Evaluates the given Expression and converts its result to a NodeSet.
-     * If the result is not a NodeSet NULL is returned.
+     * If the result is not a NodeSet an error is returned.
      */
-    NodeSet* evaluateToNodeSet(Expr* aExpr, txIEvalContext* aContext);
+    nsresult evaluateToNodeSet(Expr* aExpr, txIEvalContext* aContext,
+                               NodeSet** aResult);
 
     /*
      * Returns the name of the function as an atom.
@@ -290,12 +297,15 @@ public:
     virtual ~PredicateList();
 
     /**
-     * Adds the given Expr to the list
-     * @param expr the Expr to add to the list
-    **/
-    void add(Expr* expr);
+     * Adds the given Expr to the list.
+     * The ownership of the given Expr is passed over the PredicateList,
+     * even on failure.
+     * @param aExpr the Expr to add to the list
+     * @return nsresult indicating out of memory
+     */
+    nsresult add(Expr* aExpr);
 
-    void evaluatePredicates(NodeSet* aNodes, txIMatchContext* aContext);
+    nsresult evaluatePredicates(NodeSet* aNodes, txIMatchContext* aContext);
 
     /**
      * returns true if this predicate list is empty
@@ -344,7 +354,12 @@ public:
      * @param nodeExpr the NodeExpr to use when matching Nodes
      * @param axisIdentifier the Axis Identifier in which to search for nodes
     **/
-    LocationStep(nsAutoPtr<txNodeTest> aNodeTest, LocationStepType aAxisIdentifier);
+    LocationStep(nsAutoPtr<txNodeTest>& aNodeTest,
+                 LocationStepType aAxisIdentifier)
+        : mNodeTest(aNodeTest),
+          mAxisIdentifier(aAxisIdentifier)
+    {
+    }
 
     TX_DECL_EXPR;
 
@@ -368,52 +383,31 @@ public:
     /**
      * Creates a new FilterExpr using the given Expr
      * @param expr the Expr to use for evaluation
-    **/
-    FilterExpr(Expr* aExpr);
-
-    /**
-     * Destructor, will delete all predicates and the given Expr
-    **/
-    virtual ~FilterExpr();
+     */
+    FilterExpr(nsAutoPtr<Expr>& aExpr)
+        : expr(aExpr)
+    {
+    }
 
     TX_DECL_EXPR;
 
 private:
-    Expr* expr;
+    nsAutoPtr<Expr> expr;
 
 }; //-- FilterExpr
 
 
-class NumberExpr : public Expr {
-
+class txLiteralExpr : public Expr {
 public:
-
-    NumberExpr(double dbl);
+    txLiteralExpr(double aDbl);
+    txLiteralExpr(const nsAString& aStr);
 
     TX_DECL_EXPR;
 
 private:
-
-    double _value;
+    nsRefPtr<txAExprResult> mValue;
 };
 
-/**
- * Represents a String expression
-**/
-class StringExpr : public Expr {
-
-public:
-
-    StringExpr(const nsAString& value);
-
-    TX_DECL_EXPR;
-
-private:
-
-    nsString value;
-}; //-- StringExpr
-
-
 /**
  * Represents an AdditiveExpr, a binary expression that
  * performs an additive operation between it's lvalue and rvalue:
@@ -428,15 +422,19 @@ public:
     //-- LF, changed from static const short to enum
     enum _AdditiveExprType { ADDITION = 1, SUBTRACTION };
 
-     AdditiveExpr(Expr* leftExpr, Expr* rightExpr, short op);
-     ~AdditiveExpr();
+     AdditiveExpr(nsAutoPtr<Expr>& aLeftExpr, nsAutoPtr<Expr>& aRightExpr,
+                  short aOp)
+         : op(aOp),
+           leftExpr(aLeftExpr),
+           rightExpr(aRightExpr)
+    {
+    }
 
     TX_DECL_EXPR;
 
 private:
     short op;
-    Expr* leftExpr;
-    Expr* rightExpr;
+    nsAutoPtr<Expr> leftExpr, rightExpr;
 }; //-- AdditiveExpr
 
 /**
@@ -446,13 +444,15 @@ class UnaryExpr : public Expr {
 
 public:
 
-     UnaryExpr(Expr* expr);
-     ~UnaryExpr();
+    UnaryExpr(nsAutoPtr<Expr>& aExpr)
+        : expr(aExpr)
+    {
+    }
 
     TX_DECL_EXPR;
 
 private:
-    Expr* expr;
+    nsAutoPtr<Expr> expr;
 }; //-- UnaryExpr
 
 /**
@@ -466,15 +466,19 @@ public:
     //-- BooleanExpr Types
     enum _BooleanExprType { AND = 1, OR };
 
-     BooleanExpr(Expr* leftExpr, Expr* rightExpr, short op);
-     ~BooleanExpr();
+     BooleanExpr(nsAutoPtr<Expr>& aLeftExpr, nsAutoPtr<Expr>& aRightExpr,
+                 short aOp)
+         : op(aOp),
+           leftExpr(aLeftExpr),
+           rightExpr(aRightExpr)
+    {
+    }
 
     TX_DECL_EXPR;
 
 private:
     short op;
-    Expr* leftExpr;
-    Expr* rightExpr;
+    nsAutoPtr<Expr> leftExpr, rightExpr;
 }; //-- BooleanExpr
 
 /**
@@ -493,15 +497,20 @@ public:
     //-- LF, changed from static const short to enum
     enum _MultiplicativeExprType { DIVIDE = 1, MULTIPLY, MODULUS };
 
-     MultiplicativeExpr(Expr* leftExpr, Expr* rightExpr, short op);
-     ~MultiplicativeExpr();
+     MultiplicativeExpr(nsAutoPtr<Expr>& aLeftExpr,
+                        nsAutoPtr<Expr>& aRightExpr,
+                        short aOp)
+         : op(aOp),
+           leftExpr(aLeftExpr),
+           rightExpr(aRightExpr)
+    {
+    }
 
     TX_DECL_EXPR;
 
 private:
     short op;
-    Expr* leftExpr;
-    Expr* rightExpr;
+    nsAutoPtr<Expr> leftExpr, rightExpr;
 }; //-- MultiplicativeExpr
 
 /**
@@ -526,12 +535,20 @@ public:
         GREATER_OR_EQUAL
     };
 
-    RelationalExpr(Expr* aLeftExpr, Expr* aRightExpr, RelationalExprType aOp);
+    RelationalExpr(nsAutoPtr<Expr>& aLeftExpr, nsAutoPtr<Expr>& aRightExpr,
+                   RelationalExprType aOp)
+        : mLeftExpr(aLeftExpr),
+          mRightExpr(aRightExpr),
+          mOp(aOp)
+    {
+    }
+
 
     TX_DECL_EXPR;
 
 private:
-    PRBool compareResults(ExprResult* aLeft, ExprResult* aRight);
+    PRBool compareResults(txIEvalContext* aContext, txAExprResult* aLeft,
+                          txAExprResult* aRight);
 
     nsAutoPtr<Expr> mLeftExpr;
     nsAutoPtr<Expr> mRightExpr;
@@ -581,15 +598,23 @@ public:
 
     /**
      * Adds the Expr to this PathExpr
-     * @param expr the Expr to add to this PathExpr
-    **/
-    void addExpr(Expr* expr, PathOperator pathOp);
+     * The ownership of the given Expr is passed over the PathExpr,
+     * even on failure.
+     * @param aExpr the Expr to add to this PathExpr
+     * @return nsresult indicating out of memory
+     */
+    nsresult addExpr(Expr* aExpr, PathOperator pathOp);
 
     TX_DECL_EXPR;
 
 private:
-    struct PathExprItem {
-        Expr* expr;
+    class PathExprItem {
+    public:
+        PathExprItem(Expr* aExpr, PathOperator aOp)
+            : expr(aExpr),
+              pathOp(aOp)
+        {}
+        nsAutoPtr<Expr> expr;
         PathOperator pathOp;
     };
 
@@ -599,9 +624,9 @@ private:
      * Selects from the descendants of the context node
      * all nodes that match the Expr
      */
-    void evalDescendants(Expr* aStep, Node* aNode,
-                         txIMatchContext* aContext,
-                         NodeSet* resNodes);
+    nsresult evalDescendants(Expr* aStep, Node* aNode,
+                             txIMatchContext* aContext,
+                             NodeSet* resNodes);
 
 }; //-- PathExpr
 
@@ -645,9 +670,12 @@ public:
 
     /**
      * Adds the PathExpr to this UnionExpr
-     * @param expr the Expr to add to this UnionExpr
-    **/
-    void addExpr(Expr* expr);
+     * The ownership of the given Expr is passed over the UnionExpr,
+     * even on failure.
+     * @param aExpr the Expr to add to this UnionExpr
+     * @return nsresult indicating out of memory
+     */
+    nsresult addExpr(Expr* aExpr);
 
     TX_DECL_EXPR;
 
@@ -657,7 +685,6 @@ private:
 
 }; //-- UnionExpr
 
-/* */
 #endif
 
 
