@@ -89,14 +89,19 @@ nsXPathEvaluator::CreateExpression(const nsAString & aExpression,
 
     nsCOMPtr<nsIDocument> doc = do_QueryReferent(mDocument);
     ParseContextImpl pContext(aResolver, !doc || doc->IsCaseSensitive());
-    Expr* expression = ExprParser::createExpr(PromiseFlatString(aExpression),
-                                              &pContext);
-    if (!expression)
+    nsAutoPtr<Expr> expression;
+    rv = txExprParser::createExpr(PromiseFlatString(aExpression), &pContext,
+                                  getter_Transfers(expression));
+    if (NS_FAILED(rv)) {
+        if (rv == NS_ERROR_DOM_NAMESPACE_ERR) {
+            return NS_ERROR_DOM_NAMESPACE_ERR;
+        }
+
         return NS_ERROR_DOM_INVALID_EXPRESSION_ERR;
+    }
 
     *aResult = new nsXPathExpression(expression, mRecycler);
     if (!*aResult) {
-        delete expression;
         return NS_ERROR_OUT_OF_MEMORY;
     }
 
@@ -124,8 +129,8 @@ nsXPathEvaluator::Evaluate(const nsAString & aExpression,
                            nsIDOMNode *aContextNode,
                            nsIDOMXPathNSResolver *aResolver,
                            PRUint16 aType,
-                           nsIDOMXPathResult *aInResult,
-                           nsIDOMXPathResult **aResult)
+                           nsISupports *aInResult,
+                           nsISupports **aResult)
 {
     // XXX Need to check document of aContextNode if created by
     //     QI'ing a document.
@@ -154,23 +159,28 @@ nsXPathEvaluator::SetDocument(nsIDOMDocument* aDocument)
 nsresult nsXPathEvaluator::ParseContextImpl::resolveNamespacePrefix
     (nsIAtom* aPrefix, PRInt32& aID)
 {
+    aID = kNameSpaceID_Unknown;
+
+    if (!mResolver) {
+        return NS_ERROR_DOM_NAMESPACE_ERR;
+    }
+
     nsAutoString prefix;
     if (aPrefix) {
         aPrefix->ToString(prefix);
     }
-    nsAutoString ns;
-    nsresult rv = NS_OK;
-    if (mResolver) {
-        mResolver->LookupNamespaceURI(prefix, ns);
-        NS_ENSURE_SUCCESS(rv, rv);
+
+    nsVoidableString ns;
+    nsresult rv = mResolver->LookupNamespaceURI(prefix, ns);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    if (DOMStringIsNull(ns)) {
+        return NS_ERROR_DOM_NAMESPACE_ERR;
     }
 
-    aID = kNameSpaceID_None;
     if (ns.IsEmpty()) {
-        return NS_OK;
-    }
-    if (!mResolver) {
-        aID = kNameSpaceID_Unknown;
+        aID = kNameSpaceID_None;
+
         return NS_OK;
     }
 
@@ -178,11 +188,12 @@ nsresult nsXPathEvaluator::ParseContextImpl::resolveNamespacePrefix
     return gTxNameSpaceManager->RegisterNameSpace(ns, aID);
 }
 
-nsresult nsXPathEvaluator::ParseContextImpl::resolveFunctionCall(nsIAtom* aName,
-                                                                 PRInt32 aID,
-                                                                 FunctionCall*& aFn)
+nsresult
+nsXPathEvaluator::ParseContextImpl::resolveFunctionCall(nsIAtom* aName,
+                                                        PRInt32 aID,
+                                                        FunctionCall*& aFn)
 {
-    return NS_ERROR_XPATH_PARSE_FAILURE;
+    return NS_ERROR_XPATH_UNKNOWN_FUNCTION;
 }
 
 PRBool nsXPathEvaluator::ParseContextImpl::caseInsensitiveNameTests()
@@ -190,9 +201,7 @@ PRBool nsXPathEvaluator::ParseContextImpl::caseInsensitiveNameTests()
     return !mIsCaseSensitive;
 }
 
-void nsXPathEvaluator::ParseContextImpl::receiveError(const nsAString& aMsg,
-                                                      nsresult aRes)
+void
+nsXPathEvaluator::ParseContextImpl::SetErrorOffset(PRUint32 aOffset)
 {
-    mLastError = aRes;
-    // forward aMsg to console service?
 }
