@@ -42,6 +42,8 @@
 
 #include "nsIDOMXPathExpression.h"
 #include "txIXPathContext.h"
+#include "txResultRecycler.h"
+#include "nsAutoPtr.h"
 
 class Expr;
 
@@ -51,7 +53,8 @@ class Expr;
 class nsXPathExpression : public nsIDOMXPathExpression
 {
 public:
-    nsXPathExpression(Expr* aExpression);
+    nsXPathExpression(nsAutoPtr<Expr>& aExpression,
+                      txResultRecycler* aRecycler);
     virtual ~nsXPathExpression();
 
     // nsISupports interface
@@ -61,13 +64,14 @@ public:
     NS_DECL_NSIDOMXPATHEXPRESSION
 
 private:
-    Expr* mExpression;
+    nsAutoPtr<Expr> mExpression;
+    nsRefPtr<txResultRecycler> mRecycler;
 
     class EvalContextImpl : public txIEvalContext
     {
     public:
-        EvalContextImpl(Node* aContextNode)
-            :mNode(aContextNode), mLastError(NS_OK)
+        EvalContextImpl(Node* aContextNode, txResultRecycler* aRecycler)
+            : mNode(aContextNode), mLastError(NS_OK), mRecycler(aRecycler)
         {
         }
 
@@ -84,6 +88,7 @@ private:
     private:
         Node* mNode;
         nsresult mLastError;
+        nsRefPtr<txResultRecycler> mRecycler;
     };
 };
 
