@@ -49,8 +49,6 @@
 
 #ifndef TX_EXE
 #include "nsIDOMDocument.h"
-#include "nsContentCID.h"
-static NS_DEFINE_CID(kXMLDocumentCID, NS_XMLDOCUMENT_CID);
 #endif
 
 const PRInt32 txExecutionState::kMaxRecursionDepth = 20000;
@@ -179,15 +177,6 @@ txExecutionState::init(Node* aNode,
     mResultHandler = handler;
     mOutputHandler->startDocument();
 
-    // Initiate first instruction
-    txStylesheet::ImportFrame* frame = 0;
-    txExpandedName nullName;
-    txInstruction* templ = mStylesheet->findTemplate(aNode, nullName,
-                                                     this, nsnull, &frame);
-    pushTemplateRule(frame, nullName, nsnull);
-    rv = runTemplate(templ);
-    NS_ENSURE_SUCCESS(rv, rv);
-
     // Set up loaded-documents-hash
     Document* sourceDoc;
     if (aNode->getNodeType() == Node::DOCUMENT_NODE) {
@@ -199,7 +188,7 @@ txExecutionState::init(Node* aNode,
     rv = mLoadedDocuments.init(sourceDoc);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    // Init members    
+    // Init members
     rv = mKeyHash.init();
     NS_ENSURE_SUCCESS(rv, rv);
     
@@ -214,7 +203,16 @@ txExecutionState::init(Node* aNode,
     mGlobalVarPlaceholderValue = new StringResult(NS_LITERAL_STRING("Error"), nsnull);
     NS_ENSURE_TRUE(mGlobalVarPlaceholderValue, NS_ERROR_OUT_OF_MEMORY);
 
-    return NS_OK;
+    // Initiate first instruction. This has to be done last since findTemplate
+    // might use us.
+    txStylesheet::ImportFrame* frame = 0;
+    txExpandedName nullName;
+    txInstruction* templ = mStylesheet->findTemplate(aNode, nullName,
+                                                     this, nsnull, &frame);
+    rv = pushTemplateRule(frame, nullName, nsnull);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    return runTemplate(templ);
 }
 
 nsresult
