@@ -40,6 +40,8 @@
 
 #include "nsIDocument.h"
 #include "nsIDocShell.h"
+#include "nsIScriptLoader.h"
+#include "nsIDOMDocument.h"
 #include "nsIDOMComment.h"
 #include "nsIDOMDocumentType.h"
 #include "nsIDOMDOMImplementation.h"
@@ -66,6 +68,8 @@
 #include "txStringUtils.h"
 #include "txURIUtils.h"
 #include "nsIHTMLDocument.h"
+#include "nsIStyleSheetLinkingElement.h"
+#include "nsIDocumentTransformer.h"
 
 extern nsINameSpaceManager* gTxNameSpaceManager;
 
@@ -86,6 +90,7 @@ txMozillaXMLOutput::txMozillaXMLOutput(const nsAString& aRootName,
                                        nsIDOMDocument* aResultDocument,
                                        nsITransformObserver* aObserver)
     : mBadChildLevel(0),
+      mTableState(NORMAL),
       mDontAddCurrent(PR_FALSE),
       mHaveTitleElement(PR_FALSE),
       mHaveBaseElement(PR_FALSE),
@@ -107,6 +112,7 @@ txMozillaXMLOutput::txMozillaXMLOutput(const nsAString& aRootName,
 txMozillaXMLOutput::txMozillaXMLOutput(txOutputFormat* aFormat,
                                        nsIDOMDocumentFragment* aFragment)
     : mBadChildLevel(0),
+      mTableState(NORMAL),
       mDontAddCurrent(PR_FALSE),
       mHaveTitleElement(PR_FALSE),
       mHaveBaseElement(PR_FALSE),
@@ -202,16 +208,14 @@ void txMozillaXMLOutput::endDocument()
 
     if (!mRefreshString.IsEmpty()) {
         nsCOMPtr<nsIDocument> doc = do_QueryInterface(mDocument);
-        nsCOMPtr<nsIScriptGlobalObject> sgo;
-        doc->GetScriptGlobalObject(getter_AddRefs(sgo));
+        nsIScriptGlobalObject *sgo = doc->GetScriptGlobalObject();
         if (sgo) {
             nsCOMPtr<nsIDocShell> docShell;
             sgo->GetDocShell(getter_AddRefs(docShell));
             nsCOMPtr<nsIRefreshURI> refURI = do_QueryInterface(docShell);
             if (refURI) {
-                nsCOMPtr<nsIURI> baseURI;
-                doc->GetBaseURL(*getter_AddRefs(baseURI));
-                refURI->SetupRefreshURIFromHeader(baseURI, mRefreshString);
+                refURI->SetupRefreshURIFromHeader(doc->GetBaseURL(),
+                                                  mRefreshString);
             }
         }
     }
@@ -233,10 +237,22 @@ void txMozillaXMLOutput::endElement(const nsAString& aName, const PRInt32 aNsID)
     }
     
 #ifdef DEBUG
-    nsAutoString nodeName;
-    mCurrentNode->GetNodeName(nodeName);
-    NS_ASSERTION(nodeName.Equals(aName, nsCaseInsensitiveStringComparator()),
-                 "Unbalanced startElement and endElement calls!");
+    if (mTableState != ADDED_TBODY) {
+        nsAutoString nodeName;
+        mCurrentNode->GetNodeName(nodeName);
+        NS_ASSERTION(nodeName.Equals(aName,
+                                     nsCaseInsensitiveStringComparator()),
+                     "Unbalanced startElement and endElement calls!");
+    }
+    else {
+        nsCOMPtr<nsIDOMNode> parent;
+        mCurrentNode->GetParentNode(getter_AddRefs(parent));
+        nsAutoString nodeName;
+        parent->GetNodeName(nodeName);
+        NS_ASSERTION(nodeName.Equals(aName,
+                                     nsCaseInsensitiveStringComparator()),
+                     "Unbalanced startElement and endElement calls!");
+    }
 #endif
 
     closePrevious(eCloseElement | eFlushText);
@@ -246,7 +262,7 @@ void txMozillaXMLOutput::endElement(const nsAString& aName, const PRInt32 aNsID)
         aNsID == kNameSpaceID_XHTML) {
         nsCOMPtr<nsIDOMElement> element = do_QueryInterface(mCurrentNode);
         NS_ASSERTION(element, "endElement'ing non-element");
-        endHTMLElement(element, aNsID == kNameSpaceID_XHTML);
+        endHTMLElement(element);
     }
 
     // Add the element to the tree if it wasn't added before and take one step
@@ -275,6 +291,9 @@ void txMozillaXMLOutput::endElement(const nsAString& aName, const PRInt32 aNsID)
         mCurrentNode->GetParentNode(getter_AddRefs(parent));
         mCurrentNode = parent;
     }
+
+    mTableState =
+        NS_STATIC_CAST(TableState, NS_PTR_TO_INT32(mTableStateStack.pop()));
 }
 
 void txMozillaXMLOutput::getOutputDocument(nsIDOMDocument** aDocument)
@@ -356,7 +375,11 @@ void txMozillaXMLOutput::startElement(const nsAString& aName,
         return;
     }
 
-    nsresult rv;
+    nsresult rv = mTableStateStack.push(NS_INT32_TO_PTR(mTableState));
+    if (NS_FAILED(rv)) {
+        return;
+    }
+    mTableState = NORMAL;
 
     nsCOMPtr<nsIDOMElement> element;
     mDontAddCurrent = PR_FALSE;
@@ -377,7 +400,7 @@ void txMozillaXMLOutput::startElement(const nsAString& aName,
             return;
         }
 
-        startHTMLElement(element);
+        startHTMLElement(element, PR_FALSE);
     }
     else {
         nsAutoString nsURI;
@@ -390,7 +413,7 @@ void txMozillaXMLOutput::startElement(const nsAString& aName,
         }
 
         if (aNsID == kNameSpaceID_XHTML)
-            startHTMLElement(element);
+            startHTMLElement(element, PR_TRUE);
     }
 
     if (mCreatingNewDocument) {
@@ -423,12 +446,10 @@ void txMozillaXMLOutput::closePrevious(PRInt8 aAction)
                                             getter_AddRefs(wrapper));
             NS_ASSERTION(NS_SUCCEEDED(rv), "Can't create wrapper element");
 
-            nsCOMPtr<nsIContent> childContent;
             nsCOMPtr<nsIDOMNode> child, resultNode;
-            PRInt32 childCount, i;
-            document->GetChildCount(childCount);
+            PRUint32 i, childCount = document->GetChildCount();
             for (i = 0; i < childCount; ++i) {
-                document->ChildAt(0, *getter_AddRefs(childContent));
+                nsCOMPtr<nsIContent> childContent = document->GetChildAt(0);
                 if (childContent == mRootContent) {
                     document->SetRootContent(nsnull);
                 }
@@ -487,23 +508,55 @@ void txMozillaXMLOutput::closePrevious(PRInt8 aAction)
     }
 }
 
-void txMozillaXMLOutput::startHTMLElement(nsIDOMElement* aElement)
+void txMozillaXMLOutput::startHTMLElement(nsIDOMElement* aElement, PRBool aXHTML)
 {
+    nsresult rv = NS_OK;
     nsCOMPtr<nsIAtom> atom;
     nsCOMPtr<nsIContent> content = do_QueryInterface(aElement);
-    content->GetTag(*getter_AddRefs(atom));
+    content->GetTag(getter_AddRefs(atom));
 
     mDontAddCurrent = (atom == txHTMLAtoms::script);
 
-    if (atom == txHTMLAtoms::head) {
+    if ((atom != txHTMLAtoms::tr || aXHTML) &&
+        NS_PTR_TO_INT32(mTableStateStack.peek()) == ADDED_TBODY) {
+        nsCOMPtr<nsIDOMNode> parent;
+        mCurrentNode->GetParentNode(getter_AddRefs(parent));
+        mCurrentNode.swap(parent);
+        mTableStateStack.pop();
+    }
+
+    if (atom == txHTMLAtoms::table && !aXHTML) {
+        mTableState = TABLE;
+    }
+    else if (atom == txHTMLAtoms::tr && !aXHTML &&
+             NS_PTR_TO_INT32(mTableStateStack.peek()) == TABLE) {
+        nsCOMPtr<nsIDOMElement> elem;
+        rv = createHTMLElement(NS_LITERAL_STRING("tbody"),
+                               getter_AddRefs(elem));
+        if (NS_FAILED(rv)) {
+            return;
+        }
+        nsCOMPtr<nsIDOMNode> dummy;
+        rv = mCurrentNode->AppendChild(elem, getter_AddRefs(dummy));
+        if (NS_FAILED(rv)) {
+            return;
+        }
+        rv = mTableStateStack.push(NS_INT32_TO_PTR(ADDED_TBODY));
+        if (NS_FAILED(rv)) {
+            return;
+        }
+        mCurrentNode = elem;
+    }
+    else if (atom == txHTMLAtoms::head &&
+             mOutputFormat.mMethod == eHTMLOutput) {
         // Insert META tag, according to spec, 16.2, like
         // <META http-equiv="Content-Type" content="text/html; charset=EUC-JP">
-        nsresult rv;
-        nsCOMPtr<nsIDOMNode> foo;
         nsCOMPtr<nsIDOMElement> meta;
-        rv = mDocument->CreateElement(NS_LITERAL_STRING("meta"),
-                                      getter_AddRefs(meta));
-        NS_ASSERTION(NS_SUCCEEDED(rv), "Can't create meta element");
+        rv = createHTMLElement(NS_LITERAL_STRING("meta"),
+                               getter_AddRefs(meta));
+        if (NS_FAILED(rv)) {
+            return;
+        }
         rv = meta->SetAttribute(NS_LITERAL_STRING("http-equiv"),
                                 NS_LITERAL_STRING("Content-Type"));
         NS_ASSERTION(NS_SUCCEEDED(rv), "Can't set http-equiv on meta");
@@ -514,7 +567,8 @@ void txMozillaXMLOutput::startHTMLElement(nsIDOMElement* aElement)
         rv = meta->SetAttribute(NS_LITERAL_STRING("content"),
                                 metacontent);
         NS_ASSERTION(NS_SUCCEEDED(rv), "Can't set content on meta");
-        rv = aElement->AppendChild(meta, getter_AddRefs(foo));
+        nsCOMPtr<nsIDOMNode> dummy;
+        rv = aElement->AppendChild(meta, getter_AddRefs(dummy));
         NS_ASSERTION(NS_SUCCEEDED(rv), "Can't append meta");
     }
     else if (mCreatingNewDocument) {
@@ -527,57 +581,29 @@ void txMozillaXMLOutput::startHTMLElement(nsIDOMElement* aElement)
     }
 }
 
-void txMozillaXMLOutput::endHTMLElement(nsIDOMElement* aElement,
-                                        PRBool aXHTML)
+void txMozillaXMLOutput::endHTMLElement(nsIDOMElement* aElement)
 {
     nsresult rv;
     nsCOMPtr<nsIContent> content = do_QueryInterface(aElement);
     NS_ASSERTION(content, "Can't QI to nsIContent");
 
     nsCOMPtr<nsIAtom> atom;
-    content->GetTag(*getter_AddRefs(atom));
+    content->GetTag(getter_AddRefs(atom));
 
-    // add <tbody> to tables if there is none (not in xhtml)
-    if (atom == txHTMLAtoms::table && !aXHTML) {
-        // Check if we have any table section.
-        nsCOMPtr<nsIDOMHTMLTableSectionElement> section;
-        nsCOMPtr<nsIContent> childContent;
-        PRInt32 count, i = 0;
+    if (mTableState == ADDED_TBODY) {
+        NS_ASSERTION(atom == txHTMLAtoms::tbody,
+                     "Element flagged as added tbody isn't a tbody");
+        nsCOMPtr<nsIDOMNode> parent;
+        mCurrentNode->GetParentNode(getter_AddRefs(parent));
+        mCurrentNode = parent;
+        mTableState = NS_STATIC_CAST(TableState,
+                                     NS_PTR_TO_INT32(mTableStateStack.pop()));
 
-        content->ChildCount(count);
-        while (!section && (i < count)) {
-            rv = content->ChildAt(i, *getter_AddRefs(childContent));
-            NS_ASSERTION(NS_SUCCEEDED(rv), "Something went wrong while getting a child");
-            section = do_QueryInterface(childContent);
-            ++i;
-        }
-
-        if (!section && (count > 0)) {
-            // If no section, wrap table's children in a tbody.
-            nsCOMPtr<nsIDOMElement> wrapper;
-
-            if (mDocumentIsHTML) {
-                rv = mDocument->CreateElement(NS_LITERAL_STRING("tbody"),
-                                              getter_AddRefs(wrapper));
-            }
-            else {
-                rv = mDocument->CreateElementNS(NS_LITERAL_STRING(kXHTMLNameSpaceURI),
-                                                NS_LITERAL_STRING("tbody"),
-                                                getter_AddRefs(wrapper));
-            }
-            NS_ASSERTION(NS_SUCCEEDED(rv), "Can't create tbody element");
-
-            if (wrapper) {
-                nsCOMPtr<nsIDOMNode> resultNode;
-
-                wrapChildren(mCurrentNode, wrapper);
-                rv = mCurrentNode->AppendChild(wrapper, getter_AddRefs(resultNode));
-                NS_ASSERTION(NS_SUCCEEDED(rv), "Can't append tbody element");
-            }
-        }
+        return;
     }
+
     // Load scripts
-    else if (mNotifier && atom == txHTMLAtoms::script) {
+    if (mNotifier && atom == txHTMLAtoms::script) {
         // Add this script element to the array of loading script elements.
         nsCOMPtr<nsIDOMHTMLScriptElement> scriptElement =
             do_QueryInterface(mCurrentNode);
@@ -630,7 +656,7 @@ void txMozillaXMLOutput::endHTMLElement(nsIDOMElement* aElement,
             return;
         
         TX_ToLowerCase(httpEquiv);
-        nsCOMPtr<nsIAtom> header = dont_AddRef(NS_NewAtom(httpEquiv));
+        nsCOMPtr<nsIAtom> header = do_GetAtom(httpEquiv);
         processHTTPEquiv(header, value);
     }
 
@@ -660,27 +686,6 @@ void txMozillaXMLOutput::processHTTPEquiv(nsIAtom* aHeader, const nsAString& aVa
         CopyUCS2toASCII(aValue, mRefreshString);
 }
 
-void txMozillaXMLOutput::wrapChildren(nsIDOMNode* aCurrentNode,
-                                      nsIDOMElement* aWrapper)
-{
-    nsCOMPtr<nsIDOMNodeList> children;
-    nsresult rv = aCurrentNode->GetChildNodes(getter_AddRefs(children));
-    if (NS_FAILED(rv)) {
-        NS_ASSERTION(0, "Can't get children!");
-        return;
-    }
-
-    nsCOMPtr<nsIDOMNode> child, resultNode;
-    PRUint32 count, i;
-    children->GetLength(&count);
-    for (i = 0; i < count; ++i) {
-        rv = children->Item(0, getter_AddRefs(child));
-        if (NS_SUCCEEDED(rv)) {
-            aWrapper->AppendChild(child, getter_AddRefs(resultNode));
-        }
-    }
-}
-
 nsresult
 txMozillaXMLOutput::createResultDocument(const nsAString& aName, PRInt32 aNsID,
                                          nsIDOMDocument* aSourceDocument,
@@ -732,8 +737,7 @@ txMozillaXMLOutput::createResultDocument(const nsAString& aName, PRInt32 aNsID,
     }
 
     // Set up script loader of the result document.
-    nsCOMPtr<nsIScriptLoader> loader;
-    doc->GetScriptLoader(getter_AddRefs(loader));
+    nsIScriptLoader *loader = doc->GetScriptLoader();
     if (loader) {
         if (mNotifier) {
             loader->AddObserver(mNotifier);
@@ -780,6 +784,18 @@ txMozillaXMLOutput::createResultDocument(const nsAString& aName, PRInt32 aNsID,
     return NS_OK;
 }
 
+nsresult
+txMozillaXMLOutput::createHTMLElement(const nsAString& aName,
+                                      nsIDOMElement** aResult)
+{
+    if (mDocumentIsHTML) {
+        return mDocument->CreateElement(aName, aResult);
+    }
+
+    return mDocument->CreateElementNS(NS_LITERAL_STRING(kXHTMLNameSpaceURI),
+                                      aName, aResult);
+}
+
 txTransformNotifier::txTransformNotifier()
     : mInTransform(PR_FALSE)
       
@@ -885,9 +901,8 @@ txTransformNotifier::SignalTransformEnd()
 
     // XXX Need a better way to determine transform success/failure
     if (mDocument) {
-        nsCOMPtr<nsIScriptLoader> loader;
         nsCOMPtr<nsIDocument> doc = do_QueryInterface(mDocument);
-        doc->GetScriptLoader(getter_AddRefs(loader));
+        nsIScriptLoader *loader = doc->GetScriptLoader();
         if (loader) {
             loader->RemoveObserver(this);
         }
