@@ -496,8 +496,7 @@ nsTypeAheadFind::UseInWindow(nsIDOMWindow *aDOMWin)
     return NS_OK;
   }
 
-  nsCOMPtr<nsIPresShell> presShell;
-  doc->GetShellAt(0, getter_AddRefs(presShell));
+  nsIPresShell *presShell = doc->GetShellAt(0);
 
   if (!presShell) {
     return NS_OK;
@@ -559,14 +558,15 @@ nsTypeAheadFind::HandleEvent(nsIDOMEvent* aEvent)
     event->GetOriginalTarget(getter_AddRefs(eventTarget));
     nsCOMPtr<nsIDocument> doc(do_QueryInterface(eventTarget));
     nsCOMPtr<nsIPresShell> focusedShell(do_QueryReferent(mFocusedWeakShell));
-    NS_ENSURE_TRUE(focusedShell && doc, NS_ERROR_FAILURE);
+    if (!focusedShell || !doc) {
+      return NS_ERROR_FAILURE;
+    }
 
-    PRInt32 numShells = doc->GetNumberOfShells();
-    nsCOMPtr<nsIPresShell> shellToBeDestroyed;
+    PRUint32 numShells = doc->GetNumberOfShells();
     PRBool cancelFind = PR_FALSE;
 
-    for (PRInt32 count = 0; count < numShells; count ++) {
-      doc->GetShellAt(count, getter_AddRefs(shellToBeDestroyed));
+    for (PRUint32 count = 0; count < numShells; count ++) {
+      nsIPresShell *shellToBeDestroyed = doc->GetShellAt(count);
       if (shellToBeDestroyed == focusedShell) {
         cancelFind = PR_TRUE;
         break;
@@ -815,7 +815,7 @@ nsTypeAheadFind::BackOneChar(PRBool *aIsBackspaceUsed)
   mDontTryExactMatch = PR_FALSE;
 
   // ---------- Get new find start ------------------
-  nsCOMPtr<nsIPresShell> presShell;
+  nsIPresShell *presShell = nsnull;
   if (!findBackwards) {
     // For backspace, start from where first char was found
     // unless in backspacing after repeating char mode
@@ -826,7 +826,7 @@ nsTypeAheadFind::BackOneChar(PRBool *aIsBackspaceUsed)
       startNode->GetOwnerDocument(getter_AddRefs(domDoc));
       nsCOMPtr<nsIDocument> doc(do_QueryInterface(domDoc));
       if (doc) {
-        doc->GetShellAt(0, getter_AddRefs(presShell));
+        presShell = doc->GetShellAt(0);
       }
     }
     if (!presShell) {
@@ -1342,9 +1342,9 @@ nsTypeAheadFind::FindItNow(nsIPresShell *aPresShell,
         }
 
         mFocusedWeakShell = do_GetWeakReference(presShell);
-        nsCOMPtr<nsIContent> docContent;
-        doc->GetRootContent(getter_AddRefs(docContent));
+        nsIContent *docContent = doc->GetRootContent();
         if (docContent) {
+          // XXXbryner Do we really want to focus the root content here?
           docContent->SetFocus(presContext);
         }
         // Get selection controller and selection for new frame/iframe
@@ -1493,7 +1493,7 @@ nsTypeAheadFind::GetSearchContainers(nsISupports *aContainer,
     rootContent = do_QueryInterface(bodyEl);
   }
   if (!rootContent) {
-    doc->GetRootContent(getter_AddRefs(rootContent));
+    rootContent = doc->GetRootContent();
   }
   nsCOMPtr<nsIDOMNode> rootNode(do_QueryInterface(rootContent));
 
@@ -1501,11 +1501,7 @@ nsTypeAheadFind::GetSearchContainers(nsISupports *aContainer,
     return NS_ERROR_FAILURE;
   }
 
-  PRInt32 childCount;
-
-  if (NS_FAILED(rootContent->ChildCount(childCount))) {
-    return NS_ERROR_FAILURE;
-  }
+  PRUint32 childCount = rootContent->GetChildCount();
 
   mSearchRange->SelectNodeContents(rootNode);
 
@@ -1579,9 +1575,6 @@ nsTypeAheadFind::RangeStartsInsideLink(nsIDOMRange *aRange,
   PRInt32 startOffset;
   aRange->GetStartOffset(&startOffset);
 
-  nsCOMPtr<nsIContent> childContent;
-  PRBool canContainChildren;
-
   startContent = do_QueryInterface(startNode);
   if (!startContent) {
     NS_NOTREACHED("startContent should never be null");
@@ -1589,9 +1582,8 @@ nsTypeAheadFind::RangeStartsInsideLink(nsIDOMRange *aRange,
   }
   origContent = startContent;
 
-  if (NS_SUCCEEDED(startContent->CanContainChildren(canContainChildren)) &&
-      canContainChildren) {
-    startContent->ChildAt(startOffset, *getter_AddRefs(childContent));
+  if (startContent->CanContainChildren()) {
+    nsIContent *childContent = startContent->GetChildAt(startOffset);
     if (childContent) {
       startContent = childContent;
     }
@@ -1649,10 +1641,9 @@ nsTypeAheadFind::RangeStartsInsideLink(nsIDOMRange *aRange,
     }
 
     // Get the parent
-    nsCOMPtr<nsIContent> parent, parentsFirstChild;
-    startContent->GetParent(*getter_AddRefs(parent));
+    nsCOMPtr<nsIContent> parent = startContent->GetParent();
     if (parent) {
-      parent->ChildAt(0, *getter_AddRefs(parentsFirstChild));
+      nsIContent *parentsFirstChild = parent->GetChildAt(0);
       nsCOMPtr<nsITextContent> textContent =
         do_QueryInterface(parentsFirstChild);
 
@@ -1661,7 +1652,7 @@ nsTypeAheadFind::RangeStartsInsideLink(nsIDOMRange *aRange,
         PRBool isOnlyWhitespace;
         textContent->IsOnlyWhitespace(&isOnlyWhitespace);
         if (isOnlyWhitespace)
-          parent->ChildAt(1, *getter_AddRefs(parentsFirstChild));
+          parentsFirstChild = parent->GetChildAt(1);
       }
 
       if (parentsFirstChild != startContent) {
@@ -1990,15 +1981,11 @@ nsTypeAheadFind::GetAutoStart(nsIDOMWindow *aDOMWin, PRBool *aIsAutoStartOn)
     }
   }
 
-  nsCOMPtr<nsIDocument> parentDoc;
-  doc->GetParentDocument(getter_AddRefs(parentDoc));
+  nsIDocument *parentDoc = doc->GetParentDocument();
   if (parentDoc) {
-    nsCOMPtr<nsIContent> browserElContent;
     // get content for <browser>
-    parentDoc->FindContentForSubDocument(doc,
-                                         getter_AddRefs(browserElContent));
     nsCOMPtr<nsIDOMElement> browserElement =
-      do_QueryInterface(browserElContent);
+      do_QueryInterface(parentDoc->FindContentForSubDocument(doc));
 
     if (browserElement) {
       nsAutoString tagName, autoFind, test;
@@ -2226,10 +2213,10 @@ void
 nsTypeAheadFind::RemoveDocListeners()
 {
   nsCOMPtr<nsIPresShell> presShell(do_QueryReferent(mFocusedWeakShell));
-  nsCOMPtr<nsIViewManager> vm;
+  nsIViewManager* vm = nsnull;
 
   if (presShell) {
-    presShell->GetViewManager(getter_AddRefs(vm));
+    vm = presShell->GetViewManager();
   }
 
   nsIScrollableView* scrollableView = nsnull;
@@ -2263,8 +2250,7 @@ nsTypeAheadFind::AttachDocListeners(nsIPresShell *aPresShell)
     return;
   }
 
-  nsCOMPtr<nsIViewManager> vm;
-  aPresShell->GetViewManager(getter_AddRefs(vm));
+  nsIViewManager* vm = aPresShell->GetViewManager();
   if (!vm) {
     return;
   }
@@ -2441,7 +2427,7 @@ nsTypeAheadFind::IsTargetContentOkay(nsIContent *aContent)
     // having a table of atoms just for it. Instead, we're paying for 1 extra 
     // string compare per keystroke, which isn't too bad.
     nsCOMPtr<nsIAtom> targetTagAtom;
-    aContent->GetTag(*getter_AddRefs(targetTagAtom));
+    aContent->GetTag(getter_AddRefs(targetTagAtom));
     nsAutoString targetTagString;
     targetTagAtom->ToString(targetTagString);
     if (targetTagString.Equals(NS_LITERAL_STRING("isindex"))) {
@@ -2488,21 +2474,18 @@ nsTypeAheadFind::GetTargetIfTypeAheadOkay(nsIDOMEvent *aEvent,
 
   // ---------- Is the keystroke in a new window? -------------------
 
-  nsCOMPtr<nsIDocument> doc;
-  if (NS_FAILED(targetContent->GetDocument(*getter_AddRefs(doc))) || !doc) {
+  nsCOMPtr<nsIDocument> doc = targetContent->GetDocument();
+  if (!doc) {
     return NS_OK;
   }
 
-  nsCOMPtr<nsIScriptGlobalObject> ourGlobal;
-  doc->GetScriptGlobalObject(getter_AddRefs(ourGlobal));
-  nsCOMPtr<nsIDOMWindow> domWin(do_QueryInterface(ourGlobal));
+  nsCOMPtr<nsIDOMWindow> domWin(do_QueryInterface(doc->GetScriptGlobalObject()));
   nsCOMPtr<nsIDOMWindow> topContentWin;
   GetStartWindow(domWin, getter_AddRefs(topContentWin));
 
   // ---------- Get presshell -----------
 
-  nsCOMPtr<nsIPresShell> presShell;
-  doc->GetShellAt(0, getter_AddRefs(presShell));
+  nsIPresShell *presShell = doc->GetShellAt(0);
   if (!presShell) {
     return NS_OK;
   }
@@ -2632,8 +2615,7 @@ nsTypeAheadFind::IsRangeVisible(nsIPresShell *aPresShell,
   // Set up the variables we need, return true if we can't get at them all
   const PRUint16 kMinPixels  = 12;
 
-  nsCOMPtr<nsIViewManager> viewManager;
-  aPresShell->GetViewManager(getter_AddRefs(viewManager));
+  nsIViewManager* viewManager = aPresShell->GetViewManager();
   if (!viewManager) {
     return PR_TRUE;
   }
@@ -2642,7 +2624,6 @@ nsTypeAheadFind::IsRangeVisible(nsIPresShell *aPresShell,
   // We don't use the more accurate AccGetBounds, because that is
   // more expensive and the STATE_OFFSCREEN flag that this is used
   // for only needs to be a rough indicator
-  nsRect relFrameRect;
   nsIView *containingView = nsnull;
   nsPoint frameOffset;
   float p2t;
@@ -2650,7 +2631,7 @@ nsTypeAheadFind::IsRangeVisible(nsIPresShell *aPresShell,
   nsRectVisibility rectVisibility = nsRectVisibility_kAboveViewport;
 
   if (!aGetTopVisibleLeaf) {
-    frame->GetRect(relFrameRect);
+    nsRect relFrameRect = frame->GetRect();
     frame->GetOffsetFromView(aPresContext, frameOffset, &containingView);
     if (!containingView) {
       // no view -- not visible
@@ -2695,7 +2676,7 @@ nsTypeAheadFind::IsRangeVisible(nsIPresShell *aPresShell,
       return PR_FALSE;
     }
 
-    frame->GetRect(relFrameRect);
+    nsRect relFrameRect = frame->GetRect();
     frame->GetOffsetFromView(aPresContext, frameOffset, &containingView);
     if (containingView) {
       relFrameRect.x = frameOffset.x;
@@ -2708,10 +2689,8 @@ nsTypeAheadFind::IsRangeVisible(nsIPresShell *aPresShell,
   }
 
   if (frame) {
-    nsCOMPtr<nsIContent> firstVisibleContent;
-    frame->GetContent(getter_AddRefs(firstVisibleContent));
     nsCOMPtr<nsIDOMNode> firstVisibleNode =
-      do_QueryInterface(firstVisibleContent);
+      do_QueryInterface(frame->GetContent());
 
     if (firstVisibleNode) {
       (*aFirstVisibleRange)->SelectNode(firstVisibleNode);
@@ -2872,6 +2851,10 @@ nsTypeAheadController::nsTypeAheadController(nsIFocusController *aFocusControlle
 {
 }
 
+nsTypeAheadController::~nsTypeAheadController()
+{
+}
+
 NS_IMETHODIMP
 nsTypeAheadController::IsCommandEnabled(const char *aCommand, PRBool *aResult)
 {
@@ -2936,8 +2919,6 @@ nsTypeAheadController::SupportsCommand(const char *aCommand, PRBool *aResult)
 NS_IMETHODIMP
 nsTypeAheadController::DoCommand(const char *aCommand)
 {
-  nsresult rv = NS_OK;
-
   PRBool isLinkSearch = PR_FALSE;
 
   if (nsCRT::strcmp(sLinkFindString, aCommand) == 0) {
@@ -3028,8 +3009,7 @@ nsTypeAheadController::EnsureContentWindow(nsIDOMWindowInternal *aFocusedWin,
         docShell->GetPresContext(getter_AddRefs(presContext));
         NS_ENSURE_TRUE(presContext, NS_ERROR_FAILURE);
 
-        nsCOMPtr<nsIContent> rootContent;
-        doc->GetRootContent(getter_AddRefs(rootContent));
+        nsIContent *rootContent = doc->GetRootContent();
         NS_ENSURE_TRUE(rootContent, NS_ERROR_FAILURE);
         rootContent->SetFocus(presContext);
       }
