@@ -44,7 +44,7 @@
 #include "nsIURI.h"
 #include "nsNetUtil.h"
 #include "nsIScriptSecurityManager.h"
-#include "nsICodebasePrincipal.h"
+#include "nsIPrincipal.h"
 #include "nsIVariant.h"
 #include "nsString.h"
 #include "nsSOAPUtils.h"
@@ -56,6 +56,7 @@
 #include "nsIDOMSerializer.h"
 #include "nsIWebScriptsAccessService.h"
 #include "nsMemory.h"
+#include "nsIDocument.h"
 
 nsHTTPSOAPTransport::nsHTTPSOAPTransport()
 {
@@ -89,6 +90,40 @@ nsresult DebugPrintDOM(nsIDOMNode * node)
 
 static NS_NAMED_LITERAL_STRING(kAnyURISchemaType, "anyURI");
 
+/**
+  * This method will replace the target document's 
+  * codebase principal with the subject codebase to
+  * override cross-domain checks. So use caution 
+  * because this might lead to a serious security breach
+  * if misused.
+  * @param aDocument - The target/response document.
+  */
+static 
+nsresult ChangePrincipal(nsIDOMDocument* aDocument)
+{
+  if (!aDocument)
+    return NS_OK;
+
+  nsresult rv;
+  nsCOMPtr<nsIScriptSecurityManager> secMgr = 
+    do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+  
+  nsCOMPtr<nsIDocument> targetDoc(do_QueryInterface(aDocument, &rv));
+  NS_ENSURE_SUCCESS(rv, rv);
+  
+  rv = secMgr->CheckSameOrigin(nsnull, targetDoc->GetDocumentURL());
+  // change the principal only if the script security 
+  // manager has denied access.
+  if (NS_FAILED(rv)) {
+    nsCOMPtr<nsIPrincipal> subjectPrincipal;
+    rv = secMgr->GetSubjectPrincipal(getter_AddRefs(subjectPrincipal));
+    if (NS_SUCCEEDED(rv))
+      targetDoc->SetPrincipal(subjectPrincipal);
+  }
+  return rv;
+}
+
 /**
  * Get and check the transport URI for accessibility.  In the future,
  * this might also attempt to automatically add a mustUnderstand
@@ -161,24 +196,20 @@ static nsresult GetTransportURI(nsISOAPCall * aCall, nsAString & aURI)
                               "SOAP_INVOKE_VERIFY_PRINCIPAL", 
                               "Source-verified message cannot be sent without principal.");
       }
-      nsCOMPtr<nsICodebasePrincipal> codebase = do_QueryInterface(principal,&rc);
-      if (NS_FAILED(rc)) 
-        return rc;
-  
-      if (!codebase) {
+
+      nsCOMPtr<nsIURI> uri;
+      principal->GetURI(getter_AddRefs(uri));
+      if (!uri) {
         return SOAP_EXCEPTION(NS_ERROR_FAILURE,
-                              "SOAP_INVOKE_VERIFY_CODEBASE", 
-                              "Source-verified message cannot be sent without codebase.");
+                              "SOAP_INVOKE_VERIFY_URI", 
+                              "Source-verified message cannot be sent without URI.");
       }
-  
-      char* str;
 
-      rc = codebase->GetSpec(&str);
+      nsCAutoString spec;
+      rc = uri->GetSpec(spec);
       if (NS_FAILED(rc)) 
         return rc;
-      CopyASCIItoUCS2(nsDependentCString(str), sourceURI);
-      nsMemory::Free(str);
-
+      CopyASCIItoUCS2(spec, sourceURI);
     }
 
 //  Adding a header to tell the server that it must understand and verify the source of the call
@@ -464,6 +495,7 @@ NS_IMETHODIMP
       rv = mRequest->GetResponseXML(getter_AddRefs(document));
       if (NS_SUCCEEDED(rv) && document) {
         rv = mResponse->SetMessage(document);
+        ChangePrincipal(document);
         DEBUG_DUMP_DOCUMENT("Asynchronous Response", document)
       } 
       else {
