@@ -50,7 +50,6 @@
 
 #include "nsIJSContextStack.h"
 #include "nsIScriptSecurityManager.h"
-#include "nsICodebasePrincipal.h"
 #include "nsIURI.h"
 
 nsDOMSerializer::nsDOMSerializer()
@@ -99,13 +98,11 @@ static nsresult SetUpEncoder(nsIDOMNode *aRoot, const char* aCharset, nsIDocumen
   if (NS_FAILED(rv))
     return rv;
 
-  nsAutoString charset;
+  nsCAutoString charset;
   if (aCharset) {
-    charset.AssignWithConversion(aCharset);
+    charset = aCharset;
   } else {
-    rv = document->GetDocumentCharacterSet(charset);
-    if (NS_FAILED(rv))
-      return rv;
+    charset = document->GetDocumentCharacterSet();
   }
   rv = encoder->SetCharset(charset);
   if (NS_FAILED(rv))
@@ -151,16 +148,12 @@ nsresult CheckSameOrigin(nsIDOMNode *aRoot)
     nsCOMPtr<nsIDocument> doc(do_QueryInterface(owner_doc));
 
     if (doc) {
-      nsCOMPtr<nsIPrincipal> principal;
       nsCOMPtr<nsIURI> root_uri;
 
-      doc->GetPrincipal(getter_AddRefs(principal));
+      nsIPrincipal *principal = doc->GetPrincipal();
 
-      nsCOMPtr<nsICodebasePrincipal> codebase_principal =
-        do_QueryInterface(principal);
-
-      if (codebase_principal) {
-        codebase_principal->GetURI(getter_AddRefs(root_uri));
+      if (principal) {
+        principal->GetURI(getter_AddRefs(root_uri));
       }
 
       if (root_uri) {
