@@ -47,7 +47,7 @@ static NS_DEFINE_CID( kXMLDocumentCID, NS_XMLDOCUMENT_CID );
 class nsMyListener : public nsIDOMLoadListener 
 {
 public:
-  nsMyListener() {NS_INIT_ISUPPORTS();}
+  nsMyListener() {}
   ~nsMyListener() {}
 
   void Start(const char *a);
@@ -272,10 +272,8 @@ int main (int argc, char* argv[])
     nsCRT::free(s);
     nsCOMPtr<nsIDocument> doc = do_QueryInterface(pDOMDocument);
     if (doc) {
-      nsCOMPtr<nsIURI> uri;
-      doc->GetDocumentURL(getter_AddRefs(uri));
       nsCAutoString spec;
-      uri->GetSpec(spec);
+      doc->GetDocumentURL()->GetSpec(spec);
       printf("Document URI=\"%s\"\n",spec.get());
     }
   }
