@@ -47,9 +47,8 @@
 #include <Polygon.h>
 #include <math.h>
 
-static NS_DEFINE_IID(kRenderingContextIID, NS_IRENDERING_CONTEXT_IID);
 
-NS_IMPL_THREADSAFE_ISUPPORTS1(nsRenderingContextBeOS, nsIRenderingContext)
+NS_IMPL_ISUPPORTS1(nsRenderingContextBeOS, nsIRenderingContext)
 
 static NS_DEFINE_CID(kRegionCID, NS_REGION_CID);
 
