@@ -20,7 +20,7 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
- *  Brian Ryner <bryner@netscape.com>  (Original Author)
+ *  Brian Ryner <bryner@brianryner.com>  (Original Author)
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -41,6 +41,8 @@
 #include "nsDrawingSurfaceGTK.h"
 #include "gtkdrawing.h"
 
+#include "nsIObserverService.h"
+#include "nsIServiceManager.h"
 #include "nsIFrame.h"
 #include "nsIPresShell.h"
 #include "nsIDocument.h"
@@ -57,7 +59,7 @@
 
 #include <gdk/gdkx.h>
 
-NS_IMPL_ISUPPORTS1(nsNativeThemeGTK, nsITheme)
+NS_IMPL_ISUPPORTS2(nsNativeThemeGTK, nsITheme, nsIObserver)
 
 static int gLastXError;
 
@@ -68,6 +70,11 @@ nsNativeThemeGTK::nsNativeThemeGTK()
     return;
   }
 
+  // We have to call moz_gtk_shutdown before the event loop stops running.
+  nsCOMPtr<nsIObserverService> obsServ =
+    do_GetService("@mozilla.org/observer-service;1");
+  obsServ->AddObserver(this, "quit-application", PR_FALSE);
+
   mDisabledAtom = do_GetAtom("disabled");
   mCheckedAtom = do_GetAtom("checked");
   mSelectedAtom = do_GetAtom("selected");
@@ -90,34 +97,45 @@ nsNativeThemeGTK::nsNativeThemeGTK()
 }
 
 nsNativeThemeGTK::~nsNativeThemeGTK() {
-  moz_gtk_shutdown();
 }
 
-static void GetPrimaryPresShell(nsIFrame* aFrame, nsIPresShell** aResult)
+NS_IMETHODIMP
+nsNativeThemeGTK::Observe(nsISupports *aSubject, const char *aTopic,
+                          const PRUnichar *aData)
 {
-  *aResult = nsnull;
+  if (!nsCRT::strcmp(aTopic, "quit-application")) {
+    moz_gtk_shutdown();
+  } else {
+    NS_NOTREACHED("unexpected topic");
+    return NS_ERROR_UNEXPECTED;
+  }
 
+  return NS_OK;
+}
+
+static nsIPresShell *
+GetPrimaryPresShell(nsIFrame* aFrame)
+{
   if (!aFrame)
-    return;
+    return nsnull;
+
+  nsIPresShell *shell = nsnull;
  
-  nsCOMPtr<nsIDocument> doc;
-  nsCOMPtr<nsIContent> content;
-  aFrame->GetContent(getter_AddRefs(content));
-  content->GetDocument(getter_AddRefs(doc));
-  if (doc)
-    doc->GetShellAt(0, aResult); // Addref happens here.
+  nsIDocument* doc = aFrame->GetContent()->GetDocument();
+  if (doc) {
+    shell = doc->GetShellAt(0);
+  }
+
+  return shell;
 }
 
 static void RefreshWidgetWindow(nsIFrame* aFrame)
 {
-  nsCOMPtr<nsIPresShell> shell;
-
-  GetPrimaryPresShell(aFrame, getter_AddRefs(shell));
+  nsIPresShell *shell = GetPrimaryPresShell(aFrame);
   if (!shell)
     return;
 
-  nsCOMPtr<nsIViewManager> vm;
-  shell->GetViewManager(getter_AddRefs(vm));
+  nsIViewManager* vm = shell->GetViewManager();
   if (!vm)
     return;
  
@@ -129,19 +147,16 @@ static PRInt32 GetContentState(nsIFrame* aFrame)
   if (!aFrame)
     return 0;
 
-  nsCOMPtr<nsIPresShell> shell;
-  GetPrimaryPresShell(aFrame, getter_AddRefs(shell));
+  nsIPresShell *shell = GetPrimaryPresShell(aFrame);
   if (!shell)
     return 0;
 
   nsCOMPtr<nsIPresContext> context;
   shell->GetPresContext(getter_AddRefs(context));
   nsCOMPtr<nsIEventStateManager> esm;
-  context->GetEventStateManager(getter_AddRefs(esm));
+  shell->GetPresContext()->GetEventStateManager(getter_AddRefs(esm));
   PRInt32 flags = 0;
-  nsCOMPtr<nsIContent> content;
-  aFrame->GetContent(getter_AddRefs(content));
-  esm->GetContentState(content, flags);
+  esm->GetContentState(aFrame->GetContent(), flags);
   return flags;
 }
 
@@ -149,10 +164,8 @@ static PRBool CheckBooleanAttr(nsIFrame* aFrame, nsIAtom* aAtom)
 {
   if (!aFrame)
     return PR_FALSE;
-  nsCOMPtr<nsIContent> content;
-  aFrame->GetContent(getter_AddRefs(content));
   nsAutoString attr;
-  nsresult res = content->GetAttr(kNameSpaceID_None, aAtom, attr);
+  nsresult res = aFrame->GetContent()->GetAttr(kNameSpaceID_None, aAtom, attr);
   if (res == NS_CONTENT_ATTR_NO_VALUE ||
       (res != NS_CONTENT_ATTR_NOT_THERE && attr.IsEmpty()))
     return PR_TRUE; // This handles the HTML case (an attr with no value is like a true val)
@@ -164,8 +177,7 @@ static PRInt32 CheckIntegerAttr(nsIFrame *aFrame, nsIAtom *aAtom)
   if (!aFrame)
     return 0;
 
-  nsCOMPtr<nsIContent> content;
-  aFrame->GetContent(getter_AddRefs(content));
+  nsIContent* content = aFrame->GetContent();
   if (!content)
     return 0;
 
@@ -206,7 +218,7 @@ nsNativeThemeGTK::GetGtkWidgetAndState(PRUint8 aWidgetType, nsIFrame* aFrame,
     } else {
       // for dropdown textfields, look at the parent frame (the textbox)
       if (aWidgetType == NS_THEME_DROPDOWN_TEXTFIELD)
-        aFrame->GetParent(&aFrame);
+        aFrame = aFrame->GetParent();
 
       PRInt32 eventState = GetContentState(aFrame);
 
@@ -223,9 +235,7 @@ nsNativeThemeGTK::GetGtkWidgetAndState(PRUint8 aWidgetType, nsIFrame* aFrame,
           aWidgetType == NS_THEME_SCROLLBAR_THUMB_HORIZONTAL) {
         // for scrollbars we need to go up two to go from the thumb to
         // the slider to the actual scrollbar object
-        nsIFrame *tmpFrame = aFrame;
-        tmpFrame->GetParent(&tmpFrame);
-        tmpFrame->GetParent(&tmpFrame);
+        nsIFrame *tmpFrame = aFrame->GetParent()->GetParent();
 
         aState->curpos = CheckIntegerAttr(tmpFrame, mCurPosAtom);
         aState->maxpos = CheckIntegerAttr(tmpFrame, mMaxPosAtom);
@@ -253,10 +263,9 @@ nsNativeThemeGTK::GetGtkWidgetAndState(PRUint8 aWidgetType, nsIFrame* aFrame,
       if (aFrame) {
         // For XUL checkboxes and radio buttons, the state of the parent
         // determines our state.
-        nsCOMPtr<nsIContent> content;
-        aFrame->GetContent(getter_AddRefs(content));
+        nsIContent* content = aFrame->GetContent();
         if (content->IsContentOfType(nsIContent::eXUL))
-          aFrame->GetParent(&aFrame);
+          aFrame = aFrame->GetParent();
         else {
           nsCOMPtr<nsIAtom> tag;
           content->GetTag(getter_AddRefs(tag));
@@ -341,10 +350,8 @@ nsNativeThemeGTK::GetGtkWidgetAndState(PRUint8 aWidgetType, nsIFrame* aFrame,
         else if (aWidgetType == NS_THEME_TAB_LEFT_EDGE)
           *aWidgetFlags |= MOZ_GTK_TAB_BEFORE_SELECTED;
 
-      nsCOMPtr<nsIContent> content;
-      aFrame->GetContent(getter_AddRefs(content));
-      if (content->HasAttr(kNameSpaceID_None, mFirstTabAtom))
-        *aWidgetFlags |= MOZ_GTK_TAB_FIRST;
+        if (aFrame->GetContent()->HasAttr(kNameSpaceID_None, mFirstTabAtom))
+          *aWidgetFlags |= MOZ_GTK_TAB_FIRST;
       }
 
       aGtkWidgetType = MOZ_GTK_TAB;
@@ -582,9 +589,7 @@ nsNativeThemeGTK::ThemeSupportsWidget(nsIPresContext* aPresContext,
   // Check for specific widgets to see if HTML has overridden the style.
   if (aFrame) {
     // For now don't support HTML.
-    nsCOMPtr<nsIContent> content;
-    aFrame->GetContent(getter_AddRefs(content));
-    if (content->IsContentOfType(nsIContent::eHTML))
+    if (aFrame->GetContent()->IsContentOfType(nsIContent::eHTML))
       return PR_FALSE;
   }
 
