@@ -151,7 +151,7 @@ nsresult nsDeviceContextOS2::Init( nsNativeDeviceContext aContext,
   long dpi;
   GFX (::DevQueryCaps(mPrintDC, CAPS_VERTICAL_FONT_RES, 1, &dpi), FALSE);
 
-  mPixelsToTwips = (float)NSIntPointsToTwips(72) / ((float)dpi);
+  mPixelsToTwips = ((float)NSIntPointsToTwips(72)) / ((float)dpi);
   mTwipsToPixels = 1.0 / mPixelsToTwips;
 
   GetTwipsToDevUnits(newscale);
@@ -347,61 +347,84 @@ nscolor GetSysColorInfo(int iSysColor)
   return NS_RGB( pRGB2->bRed, pRGB2->bGreen, pRGB2->bBlue);
 }
 
-/* Helper function to query font from INI file */
-
-void QueryFontFromINI(char* fontType, char* fontName, ULONG ulLength)
-{
-  ULONG ulMaxNameL = ulLength;
-
-  /* We had to switch to using PrfQueryProfileData because */
-  /* some users have binary font data in their INI files */
-  BOOL rc = PrfQueryProfileData(HINI_USER, "PM_SystemFonts", fontType,
-                                fontName, &ulMaxNameL);
-  /* If there was no entry in the INI, default to something */
-  if (rc == FALSE) {
-    /* Different values for DBCS vs. SBCS */
-    if (!IsDBCS()) {
-      /* WarpSans is only available on Warp 4 and above */
-      if (gIsWarp4)
-        strcpy(fontName, "9.WarpSans");
-      else
-        strcpy(fontName, "8.Helv");
-    } else {
-      /* WarpSans is only available on Warp 4 and above */
-      if (gIsWarp4)
-        strcpy(fontName, "9.WarpSans Combined");
-      else
-        strcpy(fontName, "10.Helv Combined");
-    }
-  } else {
-    /* null terminate fontname */
-    fontName[ulMaxNameL] = '\0';
-  }
-}
-
-
 nsresult GetSysFontInfo(nsSystemFontID aID, nsFont* aFont) 
 {
   char szFontNameSize[MAXNAMEL];
 
   switch (aID)
   {
+    case eSystemFont_Caption: 
+      if (!IsDBCS()) {
+        PrfQueryProfileString(HINI_USER, "PM_SystemFonts", "WindowText",
+                              gIsWarp4 ? "9.WarpSans" : "8.Helv",
+                              szFontNameSize, MAXNAMEL);
+      } else {
+        PrfQueryProfileString(HINI_USER, "PM_SystemFonts", "WindowText",
+                              gIsWarp4 ? "9.WarpSans Combined" : "8.Helv Combined",
+                              szFontNameSize, MAXNAMEL);
+      } /* endif */
+      break;
+
     case eSystemFont_Icon: 
-      QueryFontFromINI("IconText", szFontNameSize, MAXNAMEL);
+      if (!IsDBCS()) {
+        PrfQueryProfileString(HINI_USER, "PM_SystemFonts", "IconText",
+                              gIsWarp4 ? "9.WarpSans" : "8.Helv",
+                              szFontNameSize, MAXNAMEL);
+      } else {
+        PrfQueryProfileString(HINI_USER, "PM_SystemFonts", "IconText",
+                              gIsWarp4 ? "9.WarpSans Combined" : "8.Helv Combined",
+                              szFontNameSize, MAXNAMEL);
+      } /* endif */
       break;
 
     case eSystemFont_Menu: 
-      QueryFontFromINI("Menus", szFontNameSize, MAXNAMEL);
+      if (!IsDBCS()) {
+        PrfQueryProfileString(HINI_USER, "PM_SystemFonts", "Menus",
+                              gIsWarp4 ? "9.WarpSans Bold" : "10.Helv",
+                              szFontNameSize, MAXNAMEL);
+      } else {
+        PrfQueryProfileString(HINI_USER, "PM_SystemFonts", "Menus", 
+                              gIsWarp4 ? "9.WarpSans Combined" : "10.Helv Combined",
+                              szFontNameSize, MAXNAMEL);
+      } /* endif */
       break;
 
-    case eSystemFont_Caption: 
-
     case eSystemFont_MessageBox: 
+      if (!IsDBCS()) {
+        PrfQueryProfileString(HINI_USER, "PM_SystemFonts", "WindowText",
+                              gIsWarp4 ? "9.WarpSans" : "8.Helv",
+                              szFontNameSize, MAXNAMEL);
+      } else {
+        PrfQueryProfileString(HINI_USER, "PM_SystemFonts", "WindowText",
+                              gIsWarp4 ? "9.WarpSans Combined" : "8.Helv Combined",
+                              szFontNameSize, MAXNAMEL);
+      } /* endif */
+      break;
 
     case eSystemFont_SmallCaption: 
+      if (!IsDBCS()) {
+        PrfQueryProfileString(HINI_USER, "PM_SystemFonts", "WindowText",
+                              gIsWarp4 ? "9.WarpSans" : "8.Helv",
+                              szFontNameSize, MAXNAMEL);
+      } else {
+        PrfQueryProfileString(HINI_USER, "PM_SystemFonts", "WindowText", 
+                              gIsWarp4 ? "9.WarpSans Combined" : "8.Helv Combined",
+                              szFontNameSize, MAXNAMEL);
+      } /* endif */
+      break;
 
     case eSystemFont_StatusBar: 
     case eSystemFont_Tooltips: 
+      if (!IsDBCS()) {
+        PrfQueryProfileString(HINI_USER, "PM_SystemFonts", "WindowText",
+                              gIsWarp4 ? "9.WarpSans" : "8.Helv",
+                              szFontNameSize, MAXNAMEL);
+      } else {
+        PrfQueryProfileString(HINI_USER, "PM_SystemFonts", "WindowText",
+                              gIsWarp4 ? "9.WarpSans Combined" : "8.Helv Combined",
+                              szFontNameSize, MAXNAMEL);
+      } /* endif */
+      break;
 
     case eSystemFont_Widget:
 
@@ -415,7 +438,15 @@ nsresult GetSysFontInfo(nsSystemFontID aID, nsFont* aFont)
     case eSystemFont_PullDownMenu:
     case eSystemFont_List:
     case eSystemFont_Field:
-      QueryFontFromINI("WindowText", szFontNameSize, MAXNAMEL);
+      if (!IsDBCS()) {
+        PrfQueryProfileString(HINI_USER, "PM_SystemFonts", "WindowText",
+                              gIsWarp4 ? "9.WarpSans" : "8.Helv",
+                              szFontNameSize, MAXNAMEL);
+      } else {
+        PrfQueryProfileString(HINI_USER, "PM_SystemFonts", "WindowText",
+                              gIsWarp4 ? "9.WarpSans Combined" : "8.Helv Combined",
+                              szFontNameSize, MAXNAMEL);
+      } /* endif */
       break;
   } // switch 
 
