@@ -32,7 +32,6 @@
 #include "nsImageOS2.h"
 #include "nsRenderingContextOS2.h"
 #include "nsDeviceContextOS2.h"
-#include "imgScaler2.h"
 
 #define MAX_BUFFER_WIDTH        128
 #define MAX_BUFFER_HEIGHT       128
@@ -251,31 +250,6 @@ nsresult nsImageOS2::Draw( nsIRenderingContext &aContext,
                 aX, aY, aWidth, aHeight);
 }
 
-/** ---------------------------------------------------
- *  This is a helper routine to do the blending for the Draw method
- */
-void nsImageOS2 :: DrawComposited24(unsigned char *aBits,
-			     PRUint8 *aImageRGB, PRUint32 aStrideRGB,
-			     PRUint8 *aImageAlpha, PRUint32 aStrideAlpha,
-			     int aWidth, int aHeight)
-{
-  PRInt32 targetRowBytes = ((aWidth * 3) + 3) & ~3;
-
-  for (int y = 0; y < aHeight; y++) {
-    unsigned char *targetRow = aBits + y * targetRowBytes;
-    unsigned char *imageRow = aImageRGB + y * aStrideRGB;
-    unsigned char *alphaRow = aImageAlpha + y * aStrideAlpha;
-
-    for (int x = 0; x < aWidth;
-         x++, targetRow += 3, imageRow += 3, alphaRow++) {
-      unsigned alpha = *alphaRow;
-      MOZ_BLEND(targetRow[0], targetRow[0], imageRow[0], alpha);
-      MOZ_BLEND(targetRow[1], targetRow[1], imageRow[1], alpha);
-      MOZ_BLEND(targetRow[2], targetRow[2], imageRow[2], alpha);
-    }
-  }
-}
-
 NS_IMETHODIMP 
 nsImageOS2 :: Draw(nsIRenderingContext &aContext, nsDrawingSurface aSurface,
                   PRInt32 aSX, PRInt32 aSY, PRInt32 aSWidth, PRInt32 aSHeight,
@@ -422,7 +396,7 @@ nsImageOS2 :: Draw(nsIRenderingContext &aContext, nsDrawingSurface aSurface,
                POINTL aptlNew[ 4] = { { 0, 0 },              // TLL
                                       { bihMem.cx, bihMem.cy },                // TUR
                                       { aSX, mInfo->cy - (aSY + aSHeight) },   // SLL
-                                      { aSX + aSWidth+1, mInfo->cy - aSY+1 } };    // SUR
+                                      { aSX + aSWidth + 1, mInfo->cy - aSY + 1 } };    // SUR
    
                // Apply mask to target, clear pels we will fill in from the image
                MONOBITMAPINFO MaskBitmapInfo (mInfo);                                   
@@ -480,8 +454,8 @@ nsImageOS2 :: Draw(nsIRenderingContext &aContext, nsDrawingSurface aSurface,
           BITMAPINFOHEADER2 bihMem = { 0 };
 
           bihMem.cbFix = sizeof (BITMAPINFOHEADER2);
-          bihMem.cx = aDWidth;
-          bihMem.cy = aDHeight;
+          bihMem.cx = aSWidth;
+          bihMem.cy = aSHeight;
           bihMem.cPlanes = 1;
           LONG lBitCount = 0;
           GFX (::DevQueryCaps( hdcCompat, CAPS_COLOR_BITCOUNT, 1, &lBitCount), FALSE);
@@ -525,54 +499,51 @@ nsImageOS2 :: Draw(nsIRenderingContext &aContext, nsDrawingSurface aSurface,
 
               if( rc != GPI_ALTERROR )
               {
-		PRUint8 *imageRGB, *imageAlpha;
-		PRUint32 strideRGB, strideAlpha;
+                // Do manual alpha blending
+                PRUint8* pDest  = pRawBitData;
+                PRUint8* pSrc;
+                PRUint8* pAlpha;
 
-		/* Both scaled and unscaled images come through this code - save
-		   work if not scaling */
-		if ((aSWidth != aDWidth) || (aSHeight != aDHeight)) {
-		  /* Scale our image to match */
-		  imageRGB = (PRUint8 *)nsMemory::Alloc(3*aDWidth*aDHeight);
-		  imageAlpha = (PRUint8 *)nsMemory::Alloc(aDWidth*aDHeight);
-		      
-		  if (!imageRGB || !imageAlpha) {
-		    if (imageRGB)
-		      nsMemory::Free(imageRGB);
-		    if (imageAlpha)
-		      nsMemory::Free(imageAlpha);
-
-		    free(pRawBitData);
-		    GFX (::GpiSetBitmap (MemPS, NULLHANDLE), HBM_ERROR);
-		    GFX (::GpiDeleteBitmap (hMemBmp), FALSE);
-		    GFX (::GpiDestroyPS (MemPS), FALSE);
-		    ::DevCloseDC (MemDC);
-
-		    return NS_ERROR_FAILURE;
-		  }
-		      
-		  strideRGB = 3 * aDWidth;
-		  strideAlpha = aDWidth;
-		  RectStretch(aSWidth, aSHeight, aDWidth, aDHeight, 0, 0, aDWidth-1, aDHeight-1,
-			      mImageBits, mRowBytes, imageRGB, strideRGB, 24);
-		  RectStretch(aSWidth, aSHeight, aDWidth, aDHeight, 0, 0, aDWidth-1, aDHeight-1,
-			      mAlphaBits, mARowBytes, imageAlpha, strideAlpha, 8);
-		} else {
-                  PRUint32 srcy = mInfo->cy - (aSY + aSHeight);
-		  imageRGB = mImageBits + srcy * mRowBytes + aSX * 3;
-		  imageAlpha = mAlphaBits + srcy * mARowBytes + aSX;
-		  strideRGB = mRowBytes;
-		  strideAlpha = mARowBytes;
-		}
-
-		/* Do composite */
-		DrawComposited24(pRawBitData, imageRGB, strideRGB, imageAlpha, strideAlpha,
-				 aDWidth, aDHeight);
-		  
-		if ((aSWidth != aDWidth) || (aSHeight != aDHeight)) {
-		  /* Free scaled images */
-		  nsMemory::Free(imageRGB);
-		  nsMemory::Free(imageAlpha);
-		}
+                if (mInfo->cBitCount == 8)
+                {
+                  for (int y = mInfo->cy - aSY - aSHeight ; y < mInfo->cy - aSY ; y++)
+                  {
+                    pSrc   = mImageBits + (y * mRowBytes) + aSX;
+                    pAlpha = mAlphaBits + (y * mARowBytes) + aSX;
+                       
+                    for (int x = 0 ; x < bihDirect.cx ; x++)
+                    {
+                      pDest [0] = FAST_BLEND (mColorMap->Index [3 * (*pSrc) + 0], pDest [0], *pAlpha);
+                      pDest [1] = FAST_BLEND (mColorMap->Index [3 * (*pSrc) + 1], pDest [1], *pAlpha);
+                      pDest [2] = FAST_BLEND (mColorMap->Index [3 * (*pSrc) + 2], pDest [2], *pAlpha);
+                      pSrc++;
+                      pDest += 3;
+                      pAlpha++;
+                    }
+        
+                    pDest  = (PRUint8*) ((ULONG)(pDest + 3) & ~3);    // For next scanline align pointers on DWORD boundary
+                  }
+                }
+                else
+                {
+                  for (int y = mInfo->cy - aSY - aSHeight ; y < mInfo->cy - aSY ; y++)
+                  {
+                    pSrc   = mImageBits + (y * mRowBytes) + (aSX * 3);
+                    pAlpha = mAlphaBits + (y * mARowBytes) + aSX;
+                               
+                    for (int x = 0 ; x < bihDirect.cx ; x++)
+                    {
+                      pDest [0] = FAST_BLEND (pSrc [0], pDest [0], *pAlpha);
+                      pDest [1] = FAST_BLEND (pSrc [1], pDest [1], *pAlpha);
+                      pDest [2] = FAST_BLEND (pSrc [2], pDest [2], *pAlpha);
+                      pSrc  += 3;
+                      pDest += 3;
+                      pAlpha++;
+                    }
+        
+                    pDest  = (PRUint8*) ((ULONG)(pDest + 3) & ~3);    // For next scanline align pointers on DWORD boundary
+                  }
+                }
         
                 // Copy modified memory back to memory bitmap
                 GFX (::GpiSetBitmapBits (MemPS, 0, bihMem.cy, (PBYTE)pRawBitData, (PBITMAPINFO2)&bihDirect), GPI_ALTERROR);
@@ -630,7 +601,7 @@ nsImageOS2::UnlockImagePixels(PRBool aMaskPixels)
 
 void
 nsImageOS2::BuildTile (HPS hpsTile, PRUint8* pImageBits, PBITMAPINFO2 pBitmapInfo,
-                       nscoord aTileWidth, nscoord aTileHeight, float scale)
+                       nscoord aTileWidth, nscoord aTileHeight)
 {
    // If bitmap not fully loaded, then first fill area with background color.
    if (nsRect (0, 0, mInfo->cx, mInfo->cy) != mDecodedRect)
@@ -653,42 +624,32 @@ nsImageOS2::BuildTile (HPS hpsTile, PRUint8* pImageBits, PBITMAPINFO2 pBitmapInf
                        mDecodedRect.x, mDecodedRect.y,                         // SLL - in
                        mDecodedRect.XMost (), mDecodedRect.YMost () };         // SUR - ex
 
-   // Scale up
-   aptl[0].x *= scale;
-   aptl[0].y *= scale;
-   aptl[1].x = (mDecodedRect.XMost() * scale) - 1;
-   aptl[1].y = (mDecodedRect.YMost() * scale) - 1;
-   
    // Draw bitmap once into temporary PS
    GFX (::GpiDrawBits (hpsTile, (PBYTE)pImageBits, pBitmapInfo, 4, aptl, ROP_SRCCOPY, BBO_IGNORE), GPI_ERROR);
 
-   PRInt32 DestWidth  = mInfo->cx * scale;
-   PRInt32 DestHeight = mInfo->cy * scale;
+   PRInt32 DestWidth  = mInfo->cx;
+   PRInt32 DestHeight = mInfo->cy;
 
    // Copy bitmap horizontally, doubling each time
-   if (DestWidth > 0) {
-      while (DestWidth < aTileWidth)
-      {
-         POINTL aptlCopy [3] = { DestWidth, 0,                     // TLL - in
-                                 2 * DestWidth, DestHeight,        // TUR - ex
-                                 0, 0 };                           // SLL - in
+   while (DestWidth < aTileWidth)
+   {
+      POINTL aptlCopy [3] = { DestWidth, 0,                     // TLL - in
+                              2 * DestWidth, DestHeight,        // TUR - ex
+                              0, 0 };                           // SLL - in
 
-         GFX (::GpiBitBlt (hpsTile, hpsTile, 3, aptlCopy, ROP_SRCCOPY, 0L), GPI_ERROR);
-         DestWidth *= 2;
-      }
+      GFX (::GpiBitBlt (hpsTile, hpsTile, 3, aptlCopy, ROP_SRCCOPY, 0L), GPI_ERROR);
+      DestWidth *= 2;
    }
 
    // Copy bitmap vertically, doubling each time
-   if (DestHeight > 0) {
-      while (DestHeight < aTileHeight)
-      {
-         POINTL aptlCopy [4] = { 0, DestHeight,                    // TLL - in
-                                 DestWidth, 2 * DestHeight,        // TUR - ex
-                                 0, 0 };                           // SLL - in
+   while (DestHeight < aTileHeight)
+   {
+      POINTL aptlCopy [4] = { 0, DestHeight,                    // TLL - in
+                              DestWidth, 2 * DestHeight,        // TUR - ex
+                              0, 0 };                           // SLL - in
 
-         GFX (::GpiBitBlt (hpsTile, hpsTile, 3, aptlCopy, ROP_SRCCOPY, 0L), GPI_ERROR);
-         DestHeight *= 2;
-      }
+      GFX (::GpiBitBlt (hpsTile, hpsTile, 3, aptlCopy, ROP_SRCCOPY, 0L), GPI_ERROR);
+      DestHeight *= 2;
    }
 }
 
@@ -707,7 +668,7 @@ NS_IMETHODIMP nsImageOS2::DrawTile(nsIRenderingContext &aContext,
    PRBool didTile = PR_FALSE;
    PRInt32 ImageWidth = mInfo->cx;
    PRInt32 ImageHeight = mInfo->cy;
-
+   
    // Get the scale - if greater than 1 then do slow tile which
    nsIDeviceContext *theDeviceContext;
    float scale;
@@ -717,8 +678,6 @@ NS_IMETHODIMP nsImageOS2::DrawTile(nsIRenderingContext &aContext,
 
    nsRect ValidRect (0, 0, ImageWidth, ImageHeight);
    ValidRect.IntersectRect (ValidRect, mDecodedRect);
-   PRInt32 DestScaledWidth = PR_MAX(PRInt32(ValidRect.width * scale), 1);
-   PRInt32 DestScaledHeight = PR_MAX(PRInt32(ValidRect.height * scale), 1);
 
    nsRect DrawRect = aTileRect;
    DrawRect.MoveBy (-aSXOffset, -aSYOffset);
@@ -726,141 +685,180 @@ NS_IMETHODIMP nsImageOS2::DrawTile(nsIRenderingContext &aContext,
 
    // Don't bother tiling if we only have to draw the bitmap a couple of times
    // Can't tile with 8bit alpha masks because need access destination bitmap values
-   if ((ImageWidth < DrawRect.width / 2 || ImageHeight < DrawRect.height / 2) &&
-       (ImageWidth <= MAX_BUFFER_WIDTH) && (ImageHeight <= MAX_BUFFER_HEIGHT) &&
-        mAlphaDepth <= 1)
+   if ((scale > 1.0) ||
+       (ImageWidth > DrawRect.width / 2 && ImageHeight > DrawRect.height / 2) ||
+       (ImageWidth > MAX_BUFFER_WIDTH) || (ImageHeight > MAX_BUFFER_HEIGHT) ||
+        mAlphaDepth > 1)
+      return SlowTile (aContext, aSurface, aSXOffset, aSYOffset, aTileRect);   
+
+
+   nsDrawingSurfaceOS2 *surf = (nsDrawingSurfaceOS2*) aSurface;
+
+   // Find the compatible device context and create a memory one
+   HDC hdcCompat = GFX (::GpiQueryDevice (surf->GetPS ()), HDC_ERROR);
+
+   DEVOPENSTRUC dop = { 0, 0, 0, 0, 0 };
+   HDC MemDC = GFX (::DevOpenDC( (HAB)0, OD_MEMORY, "*", 5, (PDEVOPENDATA) &dop, hdcCompat), DEV_ERROR);
+
+   if( DEV_ERROR != MemDC)
    {
-      nsDrawingSurfaceOS2 *surf = (nsDrawingSurfaceOS2*) aSurface;
+      // create the PS
+      SIZEL sizel = { 0, 0 };
+      HPS MemPS = GFX (::GpiCreatePS (0, MemDC, &sizel, PU_PELS | GPIT_MICRO | GPIA_ASSOC), GPI_ERROR);
 
-      // Find the compatible device context and create a memory one
-      HDC hdcCompat = GFX (::GpiQueryDevice (surf->GetPS ()), HDC_ERROR);
-
-      DEVOPENSTRUC dop = { 0, 0, 0, 0, 0 };
-      HDC MemDC = GFX (::DevOpenDC( (HAB)0, OD_MEMORY, "*", 5, (PDEVOPENDATA) &dop, hdcCompat), DEV_ERROR);
-
-      if( DEV_ERROR != MemDC)
+      if( GPI_ERROR != MemPS)
       {
-         // create the PS
-         SIZEL sizel = { 0, 0 };
-         HPS MemPS = GFX (::GpiCreatePS (0, MemDC, &sizel, PU_PELS | GPIT_MICRO | GPIA_ASSOC), GPI_ERROR);
+         GFX (::GpiCreateLogColorTable (MemPS, 0, LCOLF_RGB, 0, 0, 0), FALSE);
 
-         if( GPI_ERROR != MemPS)
+         // now create a bitmap of the right size
+         BITMAPINFOHEADER2 hdr = { 0 };
+
+         hdr.cbFix = sizeof( BITMAPINFOHEADER2);
+         // Maximum size of tiled area (could do this better)
+         PRInt32 endWidth = ImageWidth;
+         while( endWidth < DrawRect.width)
+            endWidth *= 2;
+
+         PRInt32 endHeight = ImageHeight;
+         while( endHeight < DrawRect.height)
+            endHeight *= 2;
+
+         hdr.cx = endWidth;
+         hdr.cy = endHeight;
+         hdr.cPlanes = 1;
+
+         // find bitdepth
+         LONG lBitCount = 0;
+         GFX (::DevQueryCaps( hdcCompat, CAPS_COLOR_BITCOUNT, 1, &lBitCount), FALSE);
+         hdr.cBitCount = (USHORT) lBitCount;
+
+         RECTL  rcl;
+         surf->NS2PM_INEX (aTileRect, rcl);
+
+         POINTL aptlTile [3] = { rcl.xLeft, rcl.yBottom,                                 // TLL - in
+                                 rcl.xRight, rcl.yTop,                                   // TUR - ex
+                                 aSXOffset, endHeight - aTileRect.height - aSYOffset };  // SLL - in
+
+         HBITMAP hMemBmp = GFX (::GpiCreateBitmap (MemPS, &hdr, 0, 0, 0), GPI_ERROR);
+         if (hMemBmp != GPI_ERROR)
          {
-            GFX (::GpiCreateLogColorTable (MemPS, 0, LCOLF_RGB, 0, 0, 0), FALSE);
+            LONG ImageROP = ROP_SRCCOPY;
 
-            // now create a bitmap of the right size
-            BITMAPINFOHEADER2 hdr = { 0 };
+            GFX (::GpiSetBitmap (MemPS, hMemBmp), HBM_ERROR);
 
-            hdr.cbFix = sizeof( BITMAPINFOHEADER2);
-            // Maximum size of tiled area (could do this better)
-            PRInt32 endWidth = DestScaledWidth;
-            while( endWidth < DrawRect.width)
-               endWidth *= 2;
-
-            PRInt32 endHeight = DestScaledHeight;
-            while( endHeight < DrawRect.height)
-               endHeight *= 2;
-
-            hdr.cx = endWidth;
-            hdr.cy = endHeight;
-            hdr.cPlanes = 1;
-
-            // find bitdepth
-            LONG lBitCount = 0;
-            GFX (::DevQueryCaps( hdcCompat, CAPS_COLOR_BITCOUNT, 1, &lBitCount), FALSE);
-            hdr.cBitCount = (USHORT) lBitCount;
-
-            RECTL  rcl;
-            surf->NS2PM_INEX (aTileRect, rcl);
-
-            POINTL aptlTile [3] = { rcl.xLeft, rcl.yBottom,                                 // TLL - in
-                                    rcl.xRight, rcl.yTop,                                   // TUR - ex
-                                    aSXOffset, endHeight - aTileRect.height - aSYOffset };  // SLL - in
-            // For some reason offset does not work well with scaled output
-            if (scale > 1.0)
+            if (mAlphaDepth == 1)
             {
-               aptlTile[2].x = 0;
-               aptlTile[2].y = endHeight - aTileRect.height;
+               LONG BlackColor = GFX (::GpiQueryColorIndex (MemPS, 0, MK_RGB (0x00, 0x00, 0x00)), GPI_ALTERROR);    // CLR_BLACK;
+               LONG WhiteColor = GFX (::GpiQueryColorIndex (MemPS, 0, MK_RGB (0xFF, 0xFF, 0xFF)), GPI_ALTERROR);    // CLR_WHITE;
+
+               // WORKAROUND:
+               // PostScript drivers up to version 30.732 have problems with GpiQueryColorIndex.
+               // If we are in LCOL_RGB mode it doesn't return passed RGB color but returns error instead.
+
+               if (BlackColor == GPI_ALTERROR) BlackColor = MK_RGB (0x00, 0x00, 0x00);
+               if (WhiteColor == GPI_ALTERROR) WhiteColor = MK_RGB (0xFF, 0xFF, 0xFF);
+
+               // Set image foreground and background colors. These are used in transparent images for blitting 1-bit masks.
+               // To invert colors on ROP_SRCAND we map 1 to black and 0 to white
+               IMAGEBUNDLE ib;
+               ib.lColor     = BlackColor;        // map 1 in mask to 0x000000 (black) in destination
+               ib.lBackColor = WhiteColor;        // map 0 in mask to 0xFFFFFF (white) in destination
+               ib.usMixMode  = FM_OVERPAINT;
+               ib.usBackMixMode = BM_OVERPAINT;
+               GFX (::GpiSetAttrs (MemPS, PRIM_IMAGE, IBB_COLOR | IBB_BACK_COLOR | IBB_MIX_MODE | IBB_BACK_MIX_MODE, 0, (PBUNDLE)&ib), FALSE);
+
+
+               MONOBITMAPINFO MaskBitmapInfo (mInfo);
+               BuildTile (MemPS, mAlphaBits, MaskBitmapInfo, DrawRect.width, DrawRect.height);
+
+               // Apply mask to target, clear pels we will fill in from the image
+               GFX (::GpiBitBlt (surf->GetPS (), MemPS, 3, aptlTile, ROP_SRCAND, 0L), GPI_ERROR);
+
+               ImageROP = ROP_SRCPAINT;    // Original image must be combined with mask
             }
-            HBITMAP hMemBmp = GFX (::GpiCreateBitmap (MemPS, &hdr, 0, 0, 0), GPI_ERROR);
-            if (hMemBmp != GPI_ERROR)
-            {
-               LONG ImageROP = ROP_SRCCOPY;
 
-               GFX (::GpiSetBitmap (MemPS, hMemBmp), HBM_ERROR);
 
-               if (mAlphaDepth == 1)
-               {
-                  LONG BlackColor = GFX (::GpiQueryColorIndex (MemPS, 0, MK_RGB (0x00, 0x00, 0x00)), GPI_ALTERROR);    // CLR_BLACK;
-                  LONG WhiteColor = GFX (::GpiQueryColorIndex (MemPS, 0, MK_RGB (0xFF, 0xFF, 0xFF)), GPI_ALTERROR);    // CLR_WHITE;
+            BuildTile (MemPS, mImageBits, mInfo, DrawRect.width, DrawRect.height);
 
-                  // WORKAROUND:
-                  // PostScript drivers up to version 30.732 have problems with GpiQueryColorIndex.
-                  // If we are in LCOL_RGB mode it doesn't return passed RGB color but returns error instead.
+            GFX (::GpiBitBlt (surf->GetPS (), MemPS, 3, aptlTile, ImageROP, 0L), GPI_ERROR);
 
-                  if (BlackColor == GPI_ALTERROR) BlackColor = MK_RGB (0x00, 0x00, 0x00);
-                  if (WhiteColor == GPI_ALTERROR) WhiteColor = MK_RGB (0xFF, 0xFF, 0xFF);
+            didTile = PR_TRUE;
 
-                  // Set image foreground and background colors. These are used in transparent images for blitting 1-bit masks.
-                  // To invert colors on ROP_SRCAND we map 1 to black and 0 to white
-                  IMAGEBUNDLE ib;
-                  ib.lColor     = BlackColor;        // map 1 in mask to 0x000000 (black) in destination
-                  ib.lBackColor = WhiteColor;        // map 0 in mask to 0xFFFFFF (white) in destination
-                  ib.usMixMode  = FM_OVERPAINT;
-                  ib.usBackMixMode = BM_OVERPAINT;
-                  GFX (::GpiSetAttrs (MemPS, PRIM_IMAGE, IBB_COLOR | IBB_BACK_COLOR | IBB_MIX_MODE | IBB_BACK_MIX_MODE, 0, (PBUNDLE)&ib), FALSE);
-
-                  MONOBITMAPINFO MaskBitmapInfo (mInfo);
-                  BuildTile (MemPS, mAlphaBits, MaskBitmapInfo, DrawRect.width, DrawRect.height, scale);
-
-                  // Apply mask to target, clear pels we will fill in from the image
-                  GFX (::GpiBitBlt (surf->GetPS (), MemPS, 3, aptlTile, ROP_SRCAND, 0L), GPI_ERROR);
-
-                  ImageROP = ROP_SRCPAINT;    // Original image must be combined with mask
-               }
-
-               BuildTile (MemPS, mImageBits, mInfo, DrawRect.width, DrawRect.height, scale);
-
-               GFX (::GpiBitBlt (surf->GetPS (), MemPS, 3, aptlTile, ImageROP, 0L), GPI_ERROR);
-
-               didTile = PR_TRUE;
-
-               // Must deselect bitmap from PS before freeing bitmap and PS.
-               GFX (::GpiSetBitmap (MemPS, NULLHANDLE), HBM_ERROR);
-               GFX (::GpiDeleteBitmap (hMemBmp), FALSE);
-            }
-            GFX (::GpiDestroyPS (MemPS), FALSE);
+            // Must deselect bitmap from PS before freeing bitmap and PS.
+            GFX (::GpiSetBitmap (MemPS, NULLHANDLE), HBM_ERROR);
+            GFX (::GpiDeleteBitmap (hMemBmp), FALSE);
          }
-         GFX (::DevCloseDC (MemDC), DEV_ERROR);
+         GFX (::GpiDestroyPS (MemPS), FALSE);
       }
+      GFX (::DevCloseDC (MemDC), DEV_ERROR);
    }
 
    // If we failed to tile the bitmap, then use the old, slow, reliable way
    if( didTile == PR_FALSE)
-   {
-      // put the DestRect into absolute coordintes of the device
-      PRInt32 y0 = aTileRect.y - aSYOffset;
-      PRInt32 x0 = aTileRect.x - aSXOffset;
-      PRInt32 y1 = aTileRect.y + aTileRect.height;
-      PRInt32 x1 = aTileRect.x + aTileRect.width;
-
-      // this is the width and height of the image in pixels
-      // we need to map this to the pixel height of the device
-      nscoord ScaledTileWidth = PR_MAX(PRInt32(ImageWidth*scale), 1);
-      nscoord ScaledTileHeight = PR_MAX(PRInt32(ImageHeight*scale), 1);
-
-      for (PRInt32 y = y0; y < y1; y += ScaledTileHeight)
-      {
-        for (PRInt32 x = x0; x < x1;  x += ScaledTileWidth)
-        {
-          Draw(aContext, aSurface,
-               0, 0, PR_MIN(ValidRect.width, x1 - x), PR_MIN(ValidRect.height, y1 - y),
-               x, y, PR_MIN(DestScaledWidth, x1-x), PR_MIN(DestScaledHeight, y1 - y));
-        }
-      }
-   }
+      return SlowTile (aContext, aSurface, aSXOffset, aSYOffset, aTileRect);
 }
 
+nsresult nsImageOS2::SlowTile (nsIRenderingContext& aContext, nsDrawingSurface aSurface, 
+                               PRInt32 aSXOffset, PRInt32 aSYOffset, const nsRect &aTileRect)
+{
+  PRInt32 x0, y0, x1, y1;
+  nscoord ScaledTileWidth, ScaledTileHeight;
+  PRInt32 ImageWidth = mInfo->cx;
+  PRInt32 ImageHeight = mInfo->cy;
+  nsIDeviceContext *theDeviceContext;
+  float scale;
+  aContext.GetDeviceContext(theDeviceContext);
+  theDeviceContext->GetCanonicalPixelScale(scale);
+  PRInt32 DestScaledWidth = ImageWidth * scale;
+  PRInt32 DestScaledHeight = ImageHeight * scale;
+  
+  nsRect ValidRect (0, 0, ImageWidth, ImageHeight);
+  if (mDecodedRect.height < ImageHeight)
+  {
+    ValidRect.height = mDecodedRect.height - mDecodedRect.y;
+    DestScaledHeight = PR_MAX(PRInt32(ValidRect.height * scale), 1);
+  }
+  if (mDecodedRect.width < ImageWidth)
+  {
+    ValidRect.width = mDecodedRect.width - mDecodedRect.x;
+    DestScaledWidth = PR_MAX(PRInt32(ValidRect.width * scale), 1);
+  }
+  if (mDecodedRect.y > 0)
+  {
+    ValidRect.height -= mDecodedRect.height;
+    DestScaledHeight = PR_MAX(PRInt32(ValidRect.height * scale), 1);
+    ValidRect.y = mDecodedRect.height;
+  }  
+  if (mDecodedRect.x > 0)
+  {
+    ValidRect.width -= mDecodedRect.x;
+    DestScaledWidth = PR_MAX(PRInt32(ValidRect.width * scale), 1);
+    ValidRect.x = mDecodedRect.width;
+  }
+  
+  // put the DestRect into absolute coordintes of the device
+  y0 = aTileRect.y - aSYOffset;
+  x0 = aTileRect.x - aSXOffset;
+  y1 = aTileRect.y + aTileRect.height;
+  x1 = aTileRect.x + aTileRect.width;  
+
+  // this is the width and height of the image in pixels
+  // we need to map this to the pixel height of the device
+  ScaledTileWidth = PR_MAX(PRInt32(ImageWidth*scale), 1);
+  ScaledTileHeight = PR_MAX(PRInt32(ImageHeight*scale), 1);
+  
+  for (PRInt32 y = y0; y < y1; y += ScaledTileHeight)
+  {
+    for (PRInt32 x = x0; x < x1;  x += ScaledTileWidth)
+    {
+      Draw(aContext, aSurface,
+           0, 0, PR_MIN(ValidRect.width, x1 - x), PR_MIN(ValidRect.height, y1 - y),
+           x, y, PR_MIN(DestScaledWidth, x1-x), PR_MIN(DestScaledHeight, y1 - y));
+    }
+  }
+  
+  return NS_OK;
+}
 
 void nsImageOS2::NS2PM_ININ( const nsRect &in, RECTL &rcl)
 {
