@@ -52,13 +52,18 @@
 #endif
 #include "nsFontPSDebug.h"
 
+#include "nsArray.h"
+
 extern nsIAtom *gUsersLocale;
 #define NS_IS_BOLD(weight) ((weight) > 400 ? 1 : 0)
 
+#ifdef MOZ_ENABLE_FREETYPE2
 static nsFontPS* CreateFontPS(nsITrueTypeFontCatalogEntry*, const nsFont&,
                               nsFontMetricsPS*);
 
 static NS_DEFINE_CID(kFCSCID, NS_FONTCATALOGSERVICE_CID);
+#endif
+
 /** ---------------------------------------------------
  *  See documentation in nsFontMetricsPS.h
  *	@update 2/26/99 dwc
@@ -514,7 +519,7 @@ nsFontPSAFM::FindFont(const nsFont& aFont, nsFontMetricsPS* aFontMetrics)
 {
   nsAFMObject* afmInfo = new nsAFMObject();
   if (!afmInfo) return nsnull;
-  afmInfo->Init((PRInt32)(aFont.size / TWIPS_PER_POINT_FLOAT));
+  afmInfo->Init(aFont.size);
 
   PRInt16 fontIndex = afmInfo->CheckBasicFonts(aFont, PR_TRUE);
   if (fontIndex < 0) {
@@ -711,9 +716,16 @@ nsFontPSFreeType::CSSFontEnumCallback(const nsString& aFamily, PRBool aGeneric,
     if (!value.get())
       return PR_TRUE; // keep trying
     // strip down to just the family name
-    PRUint32 startFamily = value.FindChar('-') + 1;
-    PRUint32 endFamily = value.FindChar('-', startFamily);
-    familyname.Append(Substring(value, startFamily, endFamily - startFamily));
+    PRInt32 startFamily = value.FindChar('-') + 1;
+    if (startFamily < 0) // 1st '-' not found. Not FFRE but just familyName.
+      familyname = value;
+    else { 
+      PRInt32 endFamily = value.FindChar('-', startFamily);
+      if (endFamily < 0) // 2nd '-' not found
+        familyname.Append(Substring(value, startFamily));
+      else  // FFRE 
+        familyname.Append(Substring(value, startFamily, endFamily - startFamily));
+    }
     FIND_FONTPS_PRINTF(("generic font \"%s\" -> \"%s\"", name.get(),
                         familyname.get()));
   }
@@ -753,9 +765,16 @@ nsFontPSFreeType::AddUserPref(nsIAtom *aLang, const nsFont& aFont,
     return PR_FALSE;
 
   // strip down to just the family name
-  PRUint32 startFamily = value.FindChar('-') + 1;
-  PRUint32 endFamily = value.FindChar('-', startFamily);
-  fontName.Append(Substring(value, startFamily, endFamily - startFamily));
+  PRInt32 startFamily = value.FindChar('-') + 1;
+  if (startFamily < 0) // '-' not found. Not FFRE but just familyName 
+    fontName = value;
+  else {
+    PRInt32 endFamily = value.FindChar('-', startFamily);
+    if (endFamily < 0) // 2nd '-' not found
+      fontName.Append(Substring(value, startFamily));
+    else  // FFRE
+      fontName.Append(Substring(value, startFamily, endFamily - startFamily));
+  }
 
   AddFontEntries(fontName, fpi->lang, fpi->weight,
                  nsIFontCatalogService::kFCWidthAny, fpi->slant,
@@ -1034,7 +1053,7 @@ nsFontPSFreeType::AddFontEntries(nsACString& aFamilyName, nsACString& aLanguage,
   nsCOMPtr<nsIFontCatalogService> fcs(do_GetService(kFCSCID, &rv));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsISupportsArray> entryList = nsnull;
+  nsCOMPtr<nsIArray> entryList;
   rv = fcs->GetFontCatalogEntries(aFamilyName, aLanguage,
                                   aWeight, aWidth, aSlant, aSpacing,
                                   getter_AddRefs(entryList));
@@ -1043,16 +1062,13 @@ nsFontPSFreeType::AddFontEntries(nsACString& aFamilyName, nsACString& aLanguage,
   PRUint32 i, count = 0;
   NS_ENSURE_TRUE(entryList, NS_ERROR_FAILURE);
 
-  rv = entryList->Count(&count);
+  rv = entryList->GetLength(&count);
   NS_ENSURE_SUCCESS(rv, rv);
   ADD_ENTRY_FONTPS_PRINTF(("    count    = %d", count));
 
   for (i=0; i<count; i++) {
-    nsCOMPtr<nsISupports> item;
-    rv = entryList->GetElementAt(i, getter_AddRefs(item));
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    nsCOMPtr<nsITrueTypeFontCatalogEntry> entry(do_QueryInterface(item, &rv));
+    nsCOMPtr<nsITrueTypeFontCatalogEntry> entry = do_QueryElementAt(entryList,
+                                                                    i, &rv);
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsCAutoString fontname, stylename;
@@ -1607,7 +1623,7 @@ void nsFT2Type8Generator::GeneratePSFont(FILE* aFile)
     return;
  
   int wmode = 0;
-  if (mSubset.Length() > 0)
+  if (!mSubset.IsEmpty())
     FT2SubsetToType8(face, mSubset.get(), mSubset.Length(), wmode, aFile);
 }
 #endif //MOZ_ENABLE_FREETYPE2
