@@ -1,24 +1,40 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: NPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Netscape Public License
+ * Version 1.1 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/NPL/
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is Netscape
- * Communications Corporation.  Portions created by Netscape are
- * Copyright (C) 1998 Netscape Communications Corporation. All
- * Rights Reserved.
+ * The Initial Developer of the Original Code is 
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1998
+ * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): 
- */
+ * Contributor(s):
+ *
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the NPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the NPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 #ifdef NGLAYOUT_DDRAW
 #define INITGUID
@@ -26,7 +42,7 @@
 
 #include "nsDrawingSurfaceWin.h"
 #include "prmem.h"
-#include "nscrt.h"
+#include "nsCRT.h"
 
 //#define GFX_DEBUG
 
@@ -53,8 +69,6 @@ nsresult nsDrawingSurfaceWin::mDDrawResult = NS_OK;
 
 nsDrawingSurfaceWin :: nsDrawingSurfaceWin()
 {
-  NS_INIT_REFCNT();
-
   mDC = NULL;
   mOrigBitmap = nsnull;
   mSelectedBitmap = nsnull;
@@ -202,19 +216,40 @@ NS_IMETHODIMP nsDrawingSurfaceWin :: Lock(PRInt32 aX, PRInt32 aY,
       if (nsnull == mSelectedBitmap)
       {
         HBITMAP tbits = ::CreateCompatibleBitmap(mDC, 2, 2);
-        mLockedBitmap = (HBITMAP)::SelectObject(mDC, tbits);
+        if ( tbits != nsnull )
+        {
+          mLockedBitmap = (HBITMAP)::SelectObject(mDC, tbits);
 
-        ::GetObject(mLockedBitmap, sizeof(BITMAP), &mBitmap);
+          // if the SelectObject fails, or there was no bitmap returned from this DC
+          if ( mLockedBitmap != nsnull ) 
+          {
+            ::GetObject(mLockedBitmap, sizeof(BITMAP), &mBitmap);
 
-        mLockOffset = aY;
-        mLockHeight = min((PRInt32)aHeight, (mBitmap.bmHeight - aY));
+            mLockOffset = aY;
+            mLockHeight = min((PRInt32)aHeight, (mBitmap.bmHeight - aY));
 
-        mBitmapInfo = CreateBitmapInfo(mBitmap.bmWidth, mBitmap.bmHeight, mBitmap.bmBitsPixel, (void **)&mDIBits);
+            mBitmapInfo = CreateBitmapInfo(mBitmap.bmWidth, mBitmap.bmHeight, mBitmap.bmBitsPixel, (void **)&mDIBits);
 
-        if (!(aFlags & NS_LOCK_SURFACE_WRITE_ONLY))
-          ::GetDIBits(mDC, mLockedBitmap, mLockOffset, mLockHeight, mDIBits, mBitmapInfo, DIB_RGB_COLORS);
+            if ( mBitmapInfo != nsnull ) {
+              if (!(aFlags & NS_LOCK_SURFACE_WRITE_ONLY))
+                ::GetDIBits(mDC, mLockedBitmap, mLockOffset, mLockHeight, mDIBits, mBitmapInfo, DIB_RGB_COLORS);
 
-        mBitmap.bmBits = mDIBits;
+              mBitmap.bmBits = mDIBits;
+            } else {
+              ::DeleteObject(tbits);
+              return NS_ERROR_FAILURE;
+            }
+          } 
+          else 
+          {      
+            ::DeleteObject(tbits);
+            return NS_ERROR_FAILURE;
+          }
+        }
+        else
+        {
+          return NS_ERROR_FAILURE;
+        }
       }
       else
       {
@@ -611,12 +646,17 @@ BITMAPINFO * nsDrawingSurfaceWin :: CreateBitmapInfo(PRInt32 aWidth, PRInt32 aHe
 	    colortable = (PRUint8 *)rv + sizeof(BITMAPINFOHEADER);
 
       if ((aDepth == 16) || (aDepth == 32))
-        nsCRT::memcpy(colortable, masks, sizeof(DWORD) * allocsize);
+        memcpy(colortable, masks, sizeof(DWORD) * allocsize);
       else
-	      nsCRT::zero(colortable, sizeof(RGBQUAD) * palsize);
+        memset(colortable, 0, sizeof(RGBQUAD) * palsize);
 
-      if (nsnull != aBits)
+      if (nsnull != aBits) {
         *aBits = PR_Malloc(imagesize);
+        if ( nsnull == *aBits ) {
+          PR_Free(rv);
+          rv = nsnull;
+        }
+      }
     }
   }
 
