@@ -41,7 +41,6 @@
 #include "nsImageWin.h"
 #include "nsRenderingContextWin.h"
 #include "nsDeviceContextWin.h"
-#include "imgScaler.h"
 
 static nsresult BuildDIB(LPBITMAPINFOHEADER *aBHead, PRInt32 aWidth,
                          PRInt32 aHeight, PRInt32 aDepth, PRInt8 *aNumBitPix);
@@ -131,10 +130,6 @@ nsresult nsImageWin :: Init(PRInt32 aWidth, PRInt32 aHeight, PRInt32 aDepth,nsMa
     return NS_ERROR_UNEXPECTED;
   }
 
-  // limit images to 64k pixels on a side (~55 feet on a 100dpi monitor)
-  const PRInt32 k64KLimit = 0x0000FFFF;
-  if (aWidth > k64KLimit || aHeight > k64KLimit)
-      return NS_ERROR_FAILURE;
 
   if (mNumPaletteColors >= 0){
     // If we have a palette
@@ -531,8 +526,7 @@ nsImageWin::Draw(nsIRenderingContext &aContext, nsDrawingSurface aSurface,
         }
       } else if (8 == mAlphaDepth) {
         nsresult rv = DrawComposited(TheHDC, aDX, aDY, aDWidth, aDHeight,
-                                     aSX, srcy, aSWidth, aSHeight,
-                                     origDWidth, origDHeight);
+                                     aSX, srcy, aSWidth, aSHeight);
         if (NS_FAILED(rv)) {
           ((nsDrawingSurfaceWin *)aSurface)->ReleaseDC();
           return rv;
@@ -642,16 +636,14 @@ nsImageWin::Draw(nsIRenderingContext &aContext, nsDrawingSurface aSurface,
  *  This is a helper routine to do the blending for the DrawComposited method
  *  @update 1/04/02 dwc
  */
-void nsImageWin::DrawComposited24(unsigned char *aBits,
-                                  PRUint8 *aImageRGB, PRUint32 aStrideRGB,
-                                  PRUint8 *aImageAlpha, PRUint32 aStrideAlpha,
-                                  int aWidth, int aHeight)
+void nsImageWin::DrawComposited24(unsigned char *aBits, int aX, int aY, int aWidth, int aHeight)
 {
   PRInt32 targetRowBytes = ((aWidth * 3) + 3) & ~3;
   for (int y = 0; y < aHeight; y++) {
     unsigned char *targetRow = aBits + y * targetRowBytes;
-    unsigned char *imageRow = aImageRGB + y * aStrideRGB;
-    unsigned char *alphaRow = aImageAlpha + y * aStrideAlpha;
+    unsigned char *imageRow = mImageBits + (y + aY) * mRowBytes + 3 * aX;
+    unsigned char *alphaRow = mAlphaBits + (y + aY) * mARowBytes + aX;
+
 
     for (int x = 0; x < aWidth;
          x++, targetRow += 3, imageRow += 3, alphaRow++) {
@@ -668,27 +660,14 @@ void nsImageWin::DrawComposited24(unsigned char *aBits,
  *  Blend the image into a 24 bit buffer.. using an 8 bit alpha mask 
  *  @update 1/04/02 dwc
  */
-nsresult nsImageWin::DrawComposited(HDC TheHDC, int aDX, int aDY,
-                                    int aDWidth, int aDHeight,
-                                    int aSX, int aSY, int aSWidth, int aSHeight,
-                                    int aOrigDWidth, int aOrigDHeight)
+nsresult nsImageWin::DrawComposited(HDC TheHDC, int aDX, int aDY, int aDWidth, int aDHeight,
+                    int aSX, int aSY, int aSWidth, int aSHeight)
 {
   HDC memDC = ::CreateCompatibleDC(TheHDC);
   if (!memDC)
     return NS_ERROR_OUT_OF_MEMORY;
   unsigned char *screenBits;
-
-  PRBool scaling = PR_FALSE;
-  /* Both scaled and unscaled images come through this code */
-  if ((aDWidth != aSWidth) || (aDHeight != aSHeight)) {
-    scaling = PR_TRUE;
-    aDWidth = aOrigDWidth;
-    aDHeight = aOrigDHeight;
-    aSWidth = mBHead->biWidth;
-    aSHeight = mBHead->biHeight;
-  }
-
-  ALPHA24BITMAPINFO bmi(aDWidth, aDHeight);
+  ALPHA24BITMAPINFO bmi(aSWidth, aSHeight);
   HBITMAP tmpBitmap = ::CreateDIBSection(memDC, (LPBITMAPINFO)&bmi, DIB_RGB_COLORS,
                                          (LPVOID *)&screenBits, NULL, 0);
   if (!tmpBitmap) {
@@ -703,9 +682,10 @@ nsresult nsImageWin::DrawComposited(HDC TheHDC, int aDX, int aDY,
     return NS_ERROR_OUT_OF_MEMORY;
   }
 
+
   /* Copy from the HDC */
-  BOOL retval = ::BitBlt(memDC, 0, 0, aDWidth, aDHeight,
-                         TheHDC, aDX, aDY, SRCCOPY);
+  BOOL retval = ::StretchBlt(memDC, 0, 0, aSWidth, aSHeight,
+                             TheHDC, aDX, aDY, aDWidth, aDHeight, SRCCOPY);
   if (!retval) {
     /* select the old object again... */
     ::SelectObject(memDC, oldBitmap);
@@ -714,59 +694,13 @@ nsresult nsImageWin::DrawComposited(HDC TheHDC, int aDX, int aDY,
     return NS_ERROR_FAILURE;
   }
 
-  PRUint8 *imageRGB, *imageAlpha;
-  PRUint32 strideRGB, strideAlpha;
-
-  if (scaling) {
-    /* Scale our image to match */
-    imageRGB = (PRUint8 *)nsMemory::Alloc(3*aDWidth*aDHeight);
-    imageAlpha = (PRUint8 *)nsMemory::Alloc(aDWidth*aDHeight);
-
-    if (!imageRGB || !imageAlpha) {
-      if (imageRGB)
-        nsMemory::Free(imageRGB);
-      if (imageAlpha)
-        nsMemory::Free(imageAlpha);
-      ::SelectObject(memDC, oldBitmap);
-      ::DeleteObject(tmpBitmap);
-      ::DeleteDC(memDC);
-      return NS_ERROR_FAILURE;
-    }
-
-    strideRGB = 3 * aDWidth;
-    strideAlpha = aDWidth;
-    RectStretch(aSWidth, aSHeight, aDWidth, aDHeight,
-                0, 0, aDWidth-1, aDHeight-1,
-                mImageBits, mRowBytes, imageRGB, strideRGB, 24);
-    RectStretch(aSWidth, aSHeight, aDWidth, aDHeight,
-                0, 0, aDWidth-1, aDHeight-1,
-                mAlphaBits, mARowBytes, imageAlpha, strideAlpha, 8);
-  } else {
-    imageRGB = mImageBits + aSY * mRowBytes + aSX * 3;
-    imageAlpha = mAlphaBits + aSY * mARowBytes + aSX;
-    strideRGB = mRowBytes;
-    strideAlpha = mARowBytes;
-  }
-
   /* Do composite */
-  DrawComposited24(screenBits, imageRGB, strideRGB, imageAlpha, strideAlpha,
-                   aDWidth, aDHeight);
+  DrawComposited24(screenBits, aSX, aSY, aSWidth, aSHeight);
 
-  if (scaling) {
-    /* Free scaled images */
-    nsMemory::Free(imageRGB);
-    nsMemory::Free(imageAlpha);
-  }
 
   /* Copy back to the HDC */
-  if (scaling)
-    /* only copy back the valid portion of the image */
-    retval = ::BitBlt(TheHDC, aDX, aDY,
-                      aDWidth, (mDecodedY2*aDHeight + aSHeight - 1)/aSHeight,
-                      memDC, 0, 0, SRCCOPY);
-  else
-    retval = ::BitBlt(TheHDC, aDX, aDY, aDWidth, aDHeight,
-                      memDC, 0, 0, SRCCOPY);
+  retval = ::StretchBlt(TheHDC, aDX, aDY, aDWidth, aDHeight,
+                        memDC, 0, 0, aSWidth, aSHeight, SRCCOPY);
   if (!retval) {
     ::SelectObject(memDC, oldBitmap);
     ::DeleteObject(tmpBitmap);
@@ -801,9 +735,6 @@ NS_IMETHODIMP nsImageWin::DrawTile(nsIRenderingContext &aContext,
                                    PRInt32 aSXOffset, PRInt32 aSYOffset,
                                    const nsRect &aDestRect)
 {
-  NS_ASSERTION(aSXOffset < mBHead->biWidth && aSYOffset < mBHead->biHeight,
-               "Offset(s) are bigger than image.  Caller to DrawTile needs to do more thinking!");
-
   if (mDecodedX2 < mDecodedX1 || mDecodedY2 < mDecodedY1)
     return NS_OK;
 
@@ -814,7 +745,6 @@ NS_IMETHODIMP nsImageWin::DrawTile(nsIRenderingContext &aContext,
   PRInt32         x,y,width,height,canRaster;
   nsCOMPtr<nsIDeviceContext> theDeviceContext;
   HDC             theHDC;
-  nsRect          destRect,srcRect;
   nscoord         ScaledTileWidth,ScaledTileHeight;
 
   ((nsDrawingSurfaceWin *)aSurface)->GetTECHNOLOGY(&canRaster);
@@ -1841,8 +1771,6 @@ NS_IMETHODIMP nsImageWin::DrawToImage(nsIImage* aDstImage, nscoord aDX, nscoord
       {
         PRUint8 *dstAlpha;
         PRUint8 *alpha;
-        PRUint8 *dstGatherStart = nsnull;  // assign null to kill compiler warning
-        PRUint8 *srcGatherStart = nsnull;
         PRUint8 offset = aDX & 0x7; // x starts at 0
 
 
@@ -1945,8 +1873,7 @@ NS_IMETHODIMP nsImageWin::DrawToImage(nsIImage* aDstImage, nscoord aDX, nscoord
 
     if (8 == mAlphaDepth) {
       nsresult rv = DrawComposited(dstMemDC, aDX, aDY, aDWidth, aDHeight,
-                                   0, 0, mBHead->biWidth, mBHead->biHeight,
-                                   aDWidth, aDHeight);
+                                   0, 0, mBHead->biWidth, mBHead->biHeight);
       if (NS_FAILED(rv)) {
         ::SelectObject(dstMemDC, oldDstBits);
         ::DeleteDC(dstMemDC);
