@@ -203,8 +203,8 @@ static const nsCatalogData kCatalogTable[] = {
  {"-//W3C//DTD XHTML 1.0 Strict//EN",          "xhtml11.dtd", nsnull },
  {"-//W3C//DTD XHTML 1.0 Frameset//EN",        "xhtml11.dtd", nsnull },
  {"-//W3C//DTD XHTML Basic 1.0//EN",           "xhtml11.dtd", nsnull },
- {"-//W3C//DTD XHTML 1.1 plus MathML 2.0//EN", "mathml.dtd",  "resource:/res/mathml.css" },
- {"-//W3C//DTD XHTML 1.1 plus MathML 2.0 plus SVG 1.1//EN", "mathml.dtd", "resource:/res/mathml.css" },
+ {"-//W3C//DTD XHTML 1.1 plus MathML 2.0//EN", "mathml.dtd",  "resource://gre/res/mathml.css" },
+ {"-//W3C//DTD XHTML 1.1 plus MathML 2.0 plus SVG 1.1//EN", "mathml.dtd", "resource://gre/res/mathml.css" },
  {"-//W3C//DTD SVG 20001102//EN",              "svg.dtd",     nsnull },
  {"-//WAPFORUM//DTD XHTML Mobile 1.0//EN",     "xhtml11.dtd", nsnull },
  {nsnull, nsnull, nsnull}
@@ -269,7 +269,7 @@ IsLoadableDTD(const nsCatalogData* aCatalogData, nsCOMPtr<nsIURI>* aDTD)
   }
   
   nsCOMPtr<nsIFile> dtdPath;
-  NS_GetSpecialDirectory(NS_OS_CURRENT_PROCESS_DIR, 
+  NS_GetSpecialDirectory(NS_GRE_DIR, 
                          getter_AddRefs(dtdPath));
 
   if (!dtdPath)
@@ -306,7 +306,7 @@ IsLoadableDTD(const nsCatalogData* aCatalogData, nsCOMPtr<nsIURI>* aDTD)
 
 NS_IMPL_ISUPPORTS2(nsExpatDriver,
                    nsITokenizer,
-                   nsIDTD);
+                   nsIDTD)
 
 nsresult 
 NS_NewExpatDriver(nsIDTD** aResult) { 
@@ -346,9 +346,20 @@ nsExpatDriver::HandleStartElement(const PRUnichar *aValue,
 { 
   NS_ASSERTION(mSink, "content sink not found!");
 
+  // Calculate the total number of elements in aAtts.
+  // XML_GetSpecifiedAttributeCount will only give us the number of specified
+  // attrs (twice that number, actually), so we have to check for default attrs
+  // ourselves.
+  PRUint32 attrArrayLength;
+  for (attrArrayLength = XML_GetSpecifiedAttributeCount(mExpatParser);
+       aAtts[attrArrayLength];
+       attrArrayLength += 2) {
+    // Just looping till we find out what the length is
+  }
+  
   if (mSink){
     mSink->HandleStartElement(aValue, aAtts, 
-                              XML_GetSpecifiedAttributeCount(mExpatParser) / 2, 
+                              attrArrayLength,
                               XML_GetIdAttributeIndex(mExpatParser), 
                               XML_GetCurrentLineNumber(mExpatParser));
   }
@@ -1029,8 +1040,11 @@ nsExpatDriver::DidBuildModel(nsresult anErrorCode,
 {
   // Check for mSink is intentional. This would make sure
   // that DidBuildModel() is called only once on the sink.
-  nsresult result = (mSink)? aSink->DidBuildModel(0) : NS_OK;
-  NS_IF_RELEASE(mSink);
+  nsresult result = NS_OK;
+  if (mSink) {
+    result = aSink->DidBuildModel();
+    NS_RELEASE(mSink); // assigns null
+  }
   return result;
 }
 
@@ -1182,4 +1196,10 @@ nsExpatDriver::IntTagToStringTag(PRInt32 aIntTag) const
   return 0;
 }
 
+NS_IMETHODIMP_(nsIAtom *)
+nsExpatDriver::IntTagToAtom(PRInt32 aIntTag) const
+{
+  return 0;
+}
+
 /******************************************************************************/
