@@ -48,7 +48,6 @@
 #include "nsCRT.h" 
 #include "nsScanner.h"
 #include "plstr.h"
-#include "nsViewSourceHTML.h" 
 #include "nsIStringStream.h"
 #include "nsIChannel.h"
 #include "nsICachingChannel.h"
@@ -67,6 +66,10 @@
 #include "nsIServiceManager.h"
 //#define rickgdebug 
 
+#ifdef MOZ_VIEW_SOURCE
+#include "nsViewSourceHTML.h" 
+#endif
+
 #define NS_PARSER_FLAG_DTD_VERIFICATION       0x00000001
 #define NS_PARSER_FLAG_PARSER_ENABLED         0x00000002
 #define NS_PARSER_FLAG_OBSERVERS_ENABLED      0x00000004
@@ -79,9 +82,6 @@ static NS_DEFINE_IID(kISupportsIID, NS_ISUPPORTS_IID);
 static NS_DEFINE_CID(kCParserCID, NS_PARSER_CID); 
 static NS_DEFINE_IID(kIParserIID, NS_IPARSER_IID);
 
-static NS_DEFINE_IID(kExpatDriverCID, NS_EXPAT_DRIVER_CID);
-static NS_DEFINE_CID(kNavDTDCID, NS_CNAVDTD_CID);
-static NS_DEFINE_CID(kViewSourceDTDCID, NS_VIEWSOURCE_DTD_CID);
 static NS_DEFINE_CID(kEventQueueServiceCID, NS_EVENTQUEUESERVICE_CID);
 
 //-------------------------------------------------------------------
@@ -219,41 +219,31 @@ For more details @see bugzilla bug 76722
 
 struct nsParserContinueEvent : public PLEvent {
 
-  nsParserContinueEvent(nsIParser* aParser); 
-  ~nsParserContinueEvent() { }
+  nsParserContinueEvent(nsParser* aParser)
+  {
+    NS_ADDREF(aParser); 
+    PL_InitEvent(this, aParser, HandleEvent, DestroyEvent);  
+  }
 
-  void HandleEvent() {  
-    if (mParser) {
-      nsParser* parser = NS_STATIC_CAST(nsParser*, mParser);
-      parser->HandleParserContinueEvent();
-      NS_RELEASE(mParser);
-    }
-  };
- 
-  nsIParser* mParser; 
+  ~nsParserContinueEvent()
+  {
+    nsParser *parser = (nsParser*) owner;
+    NS_RELEASE(parser);
+  }
+
+  PR_STATIC_CALLBACK(void*) HandleEvent(PLEvent* aEvent)
+  {
+    nsParser *parser = (nsParser*) aEvent->owner;
+    parser->HandleParserContinueEvent();
+    return nsnull;
+  }
+
+  PR_STATIC_CALLBACK(void) DestroyEvent(PLEvent* aEvent)
+  {
+    delete (nsParserContinueEvent*) aEvent;
+  }
 };
 
-static void PR_CALLBACK HandlePLEvent(nsParserContinueEvent* aEvent)
-{
-  NS_ASSERTION(nsnull != aEvent,"Event is null");
-  aEvent->HandleEvent();
-}
-
-static void PR_CALLBACK DestroyPLEvent(nsParserContinueEvent* aEvent)
-{
-  NS_ASSERTION(nsnull != aEvent,"Event is null");
-  delete aEvent;
-}
-
-nsParserContinueEvent::nsParserContinueEvent(nsIParser* aParser)
-{
-  NS_ASSERTION(aParser, "null parameter");  
-  mParser = aParser; 
-  PL_InitEvent(this, aParser,
-               (PLHandleEventProc) ::HandlePLEvent,
-               (PLDestroyEventProc) ::DestroyPLEvent);  
-}
-
 //-------------- End ParseContinue Event Definition ------------------------
 
 
@@ -305,7 +295,7 @@ nsParser::nsParser() {
   }
 #endif
 
-  mCharset.Assign(NS_LITERAL_STRING("ISO-8859-1"));
+  mCharset.Assign(NS_LITERAL_CSTRING("ISO-8859-1"));
   mParserContext=0;
   mStreamStatus=0;
   mCharsetSource=kCharsetUninitialized;
@@ -417,11 +407,14 @@ nsresult
 nsParser::PostContinueEvent()
 {
   if (!(mFlags & NS_PARSER_FLAG_PENDING_CONTINUE_EVENT) && mEventQueue) {
-    nsParserContinueEvent* ev = new nsParserContinueEvent(NS_STATIC_CAST(nsIParser*, this));
-    NS_ENSURE_TRUE(ev,NS_ERROR_OUT_OF_MEMORY);
-    NS_ADDREF(this);
-    mEventQueue->PostEvent(ev);
-    mFlags |= NS_PARSER_FLAG_PENDING_CONTINUE_EVENT;
+    nsParserContinueEvent* ev = new nsParserContinueEvent(this);
+    NS_ENSURE_TRUE(ev, NS_ERROR_OUT_OF_MEMORY);
+    if (NS_FAILED(mEventQueue->PostEvent(ev))) {
+        NS_ERROR("failed to post parser continuation event");
+        PL_DestroyEvent(ev);
+    }
+    else
+        mFlags |= NS_PARSER_FLAG_PENDING_CONTINUE_EVENT;
   }
   return NS_OK;
 }
@@ -487,7 +480,7 @@ NS_IMETHODIMP_(void) nsParser::SetCommand(eParserCommands aParserCommand)
  *  @return	 nada
  */
 NS_IMETHODIMP_(void)
-nsParser::SetDocumentCharset(const nsAString& aCharset, PRInt32 aCharsetSource)
+nsParser::SetDocumentCharset(const nsACString& aCharset, PRInt32 aCharsetSource)
 {
   mCharset = aCharset;
   mCharsetSource = aCharsetSource; 
@@ -495,7 +488,7 @@ nsParser::SetDocumentCharset(const nsAString& aCharset, PRInt32 aCharsetSource)
      mParserContext->mScanner->SetDocumentCharset(aCharset, aCharsetSource);
 }
 
-void nsParser::SetSinkCharset(nsAString& aCharset)
+void nsParser::SetSinkCharset(nsACString& aCharset)
 {
   if (mSink) {
     mSink->SetDocumentCharset(aCharset);
@@ -1009,8 +1002,7 @@ static void DetermineHTMLParseMode(const nsString& aBuffer,
       if (!(resultFlags & PARSE_DTD_HAVE_INTERNAL_SUBSET) &&
           sysIDUCS2 == NS_LITERAL_STRING(
                "http://www.ibm.com/data/dtd/v11/ibmxhtml1-transitional.dtd")) {
-        aParseMode = eDTDMode_quirks;
-        aDocType = eHTML_Quirks;
+        aParseMode = eDTDMode_almost_standards;
       }
 
     } else {
@@ -1158,6 +1150,7 @@ FindSuitableDTD(CParserContext& aParserContext,
         sharedObjects->mDTDDeque.Push(theDTD);
         sharedObjects->mHasXMLDTD = PR_TRUE;
       }
+#ifdef MOZ_VIEW_SOURCE
       else if (!sharedObjects->mHasViewSourceDTD) {
         rv = NS_NewViewSourceHTML(&theDTD);  //do this so all non-html files can be viewed...
         NS_ENSURE_SUCCESS(rv, rv);
@@ -1165,6 +1158,7 @@ FindSuitableDTD(CParserContext& aParserContext,
         sharedObjects->mDTDDeque.Push(theDTD);
         sharedObjects->mHasViewSourceDTD = PR_TRUE;
       }
+#endif
     }
   }
 
@@ -1189,11 +1183,6 @@ nsParser::CancelParsingEvents()
     } 
 
     mFlags &= ~NS_PARSER_FLAG_PENDING_CONTINUE_EVENT;
-    /* Since we are taking this off of the queue, we need to do the NS_RELEASE
-     * that nsParserContinueEvent::HandleEvent would have done.
-     */
-    nsParser* me = this; 
-    NS_RELEASE(me);
   }
   return NS_OK;
 }
@@ -1282,17 +1271,22 @@ void nsParser::PushContext(CParserContext& aContext) {
  * @update	gess7/22/98
  * @return  prev. context
  */
-CParserContext* nsParser::PopContext() {
-  CParserContext* oldContext=mParserContext;
-  if(oldContext) {
-    mParserContext=oldContext->mPrevContext;
-    // If the old context was blocked, propagate the blocked state
-    // back to the new one. Also, propagate the stream listener state
-    // but don't override onStop state to guarantee the call to DidBuildModel().
+CParserContext* nsParser::PopContext() 
+{
+  CParserContext* oldContext = mParserContext;
+  if (oldContext) {
+    mParserContext = oldContext->mPrevContext;
     if (mParserContext) {
-      if(mParserContext->mStreamListenerState!=eOnStop) {
+      // If the old context was blocked, propagate the blocked state
+      // back to the new one. Also, propagate the stream listener state
+      // but don't override onStop state to guarantee the call to DidBuildModel().
+      if (mParserContext->mStreamListenerState != eOnStop) {
         mParserContext->mStreamListenerState = oldContext->mStreamListenerState;
       }
+      // Preserve tokenizer state so that information is not lost
+      // between document.write. This fixes bug 99467
+      if (mParserContext->mTokenizer)
+        mParserContext->mTokenizer->CopyState(oldContext->mTokenizer);
     }
   }
   return oldContext;
@@ -1568,7 +1562,7 @@ nsParser::Parse(const nsAString& aSourceBuffer,
 
   nsresult result=NS_OK;
 
-  if(aLastCall && (0==aSourceBuffer.Length())) {
+  if(aLastCall && aSourceBuffer.IsEmpty()) {
     // Nothing is being passed to the parser so return
     // immediately. mUnusedInput will get processed when
     // some data is actually passed in.
@@ -1584,7 +1578,7 @@ nsParser::Parse(const nsAString& aSourceBuffer,
   // till we're completely done. 
   nsCOMPtr<nsIParser> kungFuDeathGrip(this);
 
-  if(aSourceBuffer.Length() || mUnusedInput.Length()) { 
+  if(!aSourceBuffer.IsEmpty() || !mUnusedInput.IsEmpty()) {
     
     if (aVerifyEnabled) {
       mFlags |= NS_PARSER_FLAG_DTD_VERIFICATION;
@@ -1748,7 +1742,7 @@ nsresult nsParser::ResumeParse(PRBool allowIteration, PRBool aIsFinalChunk, PRBo
        
       while((result==NS_OK) && (theIterationIsOk)) {
         theFirstTime=PR_FALSE;
-        if(mUnusedInput.Length()>0) {
+        if(!mUnusedInput.IsEmpty()) {
           if(mParserContext->mScanner) {
             // -- Ref: Bug# 22485 --
             // Insert the unused input into the source buffer 
@@ -1994,7 +1988,7 @@ static inline PRBool IsSecondMarker(unsigned char aChar)
   }
 }
 
-static PRBool DetectByteOrderMark(const unsigned char* aBytes, PRInt32 aLen, nsString& oCharset, PRInt32& oCharsetSource) {
+static PRBool DetectByteOrderMark(const unsigned char* aBytes, PRInt32 aLen, nsCString& oCharset, PRInt32& oCharsetSource) {
  oCharsetSource= kCharsetFromAutoDetection;
  oCharset.Truncate();
  // See http://www.w3.org/TR/2000/REC-xml-20001006#sec-guessing
@@ -2009,26 +2003,26 @@ static PRBool DetectByteOrderMark(const unsigned char* aBytes, PRInt32 aLen, nsS
         // 00 00
         if((0xFE==aBytes[2]) && (0xFF==aBytes[3])) {
            // 00 00 FE FF UCS-4, big-endian machine (1234 order)
-           oCharset.AssignWithConversion(UCS4_BE);
+           oCharset.Assign(UCS4_BE);
         } else if((0x00==aBytes[2]) && (0x3C==aBytes[3])) {
            // 00 00 00 3C UCS-4, big-endian machine (1234 order)
-           oCharset.AssignWithConversion(UCS4_BE);
+           oCharset.Assign(UCS4_BE);
         } else if((0xFF==aBytes[2]) && (0xFE==aBytes[3])) {
            // 00 00 FF FE UCS-4, unusual octet order (2143)
-           oCharset.AssignWithConversion(UCS4_2143);
+           oCharset.Assign(UCS4_2143);
         } else if((0x3C==aBytes[2]) && (0x00==aBytes[3])) {
            // 00 00 3C 00 UCS-4, unusual octet order (2143)
-           oCharset.AssignWithConversion(UCS4_2143);
+           oCharset.Assign(UCS4_2143);
         } 
         oCharsetSource = kCharsetFromByteOrderMark;
      } else if((0x3C==aBytes[1]) && (0x00==aBytes[2])) {
         // 00 3C 00
         if(IsSecondMarker(aBytes[3])) {
            // 00 3C 00 SM UTF-16,  big-endian, no Byte Order Mark 
-           oCharset.AssignWithConversion(UTF16_BE); 
+           oCharset.Assign(UTF16_BE); 
         } else if((0x00==aBytes[3])) {
            // 00 3C 00 00 UCS-4, unusual octet order (3412)
-           oCharset.AssignWithConversion(UCS4_3412);
+           oCharset.Assign(UCS4_3412);
         } 
         oCharsetSource = kCharsetFromByteOrderMark;
      }
@@ -2038,10 +2032,10 @@ static PRBool DetectByteOrderMark(const unsigned char* aBytes, PRInt32 aLen, nsS
         // 3C 00 XX 00
         if(IsSecondMarker(aBytes[2])) {
            // 3C 00 SM 00 UTF-16,  little-endian, no Byte Order Mark 
-           oCharset.AssignWithConversion(UTF16_LE); 
+           oCharset.Assign(UTF16_LE); 
         } else if((0x00==aBytes[2])) {
            // 3C 00 00 00 UCS-4, little-endian machine (4321 order)
-           oCharset.AssignWithConversion(UCS4_LE); 
+           oCharset.Assign(UCS4_LE); 
         } 
         oCharsetSource = kCharsetFromByteOrderMark;
      // For html, meta tag detector is invoked before this so that we have 
@@ -2111,7 +2105,7 @@ static PRBool DetectByteOrderMark(const unsigned char* aBytes, PRInt32 aLen, nsS
                    // encoding value is invalid if it is UTF-16
                    if (count > 0 && 
                      (0 != PL_strcmp("UTF-16", (char*)(aBytes+encStart)))) {
-                     oCharset.AssignWithConversion((char*)(aBytes+encStart),count);
+                     oCharset.Assign((char*)(aBytes+encStart),count);
                      oCharsetSource = kCharsetFromMetaTag;
                    }
                    encodingFound = PR_TRUE;
@@ -2131,7 +2125,7 @@ static PRBool DetectByteOrderMark(const unsigned char* aBytes, PRInt32 aLen, nsS
      if((0xBB==aBytes[1]) && (0xBF==aBytes[2])) {
         // EF BB BF
         // Win2K UTF-8 BOM
-        oCharset.AssignWithConversion(UTF8); 
+        oCharset.Assign(UTF8); 
         oCharsetSource= kCharsetFromByteOrderMark;
      }
    break;
@@ -2139,10 +2133,10 @@ static PRBool DetectByteOrderMark(const unsigned char* aBytes, PRInt32 aLen, nsS
      if(0xFF==aBytes[1]) {
         if(0x00==aBytes[2] && 0x00==aBytes[3]) {
           // FE FF 00 00  UCS-4, unusual octet order (3412)
-          oCharset.AssignWithConversion(UCS4_3412);
+          oCharset.Assign(UCS4_3412);
         } else {
           // FE FF UTF-16, big-endian 
-          oCharset.AssignWithConversion(UTF16_BE); 
+          oCharset.Assign(UTF16_BE); 
         }
         oCharsetSource= kCharsetFromByteOrderMark;
      }
@@ -2151,11 +2145,11 @@ static PRBool DetectByteOrderMark(const unsigned char* aBytes, PRInt32 aLen, nsS
      if(0xFE==aBytes[1]) {
         if(0x00==aBytes[2] && 0x00==aBytes[3]) 
          // FF FE 00 00  UTF-32, little-endian
-           oCharset.AssignWithConversion(UCS4_LE); 
+           oCharset.Assign(UCS4_LE); 
         else
         // FF FE
         // UTF-16, little-endian 
-           oCharset.AssignWithConversion(UTF16_LE); 
+           oCharset.Assign(UTF16_LE); 
         oCharsetSource= kCharsetFromByteOrderMark;
      }
    break;
@@ -2164,7 +2158,7 @@ static PRBool DetectByteOrderMark(const unsigned char* aBytes, PRInt32 aLen, nsS
    // }
    // break;
  }  // switch
- return oCharset.Length() > 0;
+ return !oCharset.IsEmpty();
 }
 
 inline const char GetNextChar(nsACString::const_iterator& aStart,
@@ -2177,7 +2171,7 @@ inline const char GetNextChar(nsACString::const_iterator& aStart,
 PRBool 
 nsParser::DetectMetaTag(const char* aBytes, 
                         PRInt32 aLen, 
-                        nsString& aCharset, 
+                        nsCString& aCharset, 
                         PRInt32& aCharsetSource) 
 {
   aCharsetSource= kCharsetFromMetaTag;
@@ -2276,7 +2270,7 @@ nsParser::DetectMetaTag(const char* aBytes,
 
     // return true if we successfully got something for charset
     if (currPos != tokEnd) {
-      aCharset.Assign(NS_ConvertASCIItoUCS2(currPos.get(), tokEnd.get() - currPos.get()));
+      aCharset.Assign(currPos.get(), tokEnd.get() - currPos.get());
       return PR_TRUE;
     } 
     
@@ -2319,8 +2313,9 @@ ParserWriteFunc(nsIInputStream* in,
   }
 
   if(pws->mNeedCharsetCheck) { 
-    PRInt32 guessSource; 
-    nsAutoString guess, preferred; 
+    PRInt32 guessSource;
+    nsCAutoString guess;
+    nsCAutoString preferred; 
   
     pws->mNeedCharsetCheck = PR_FALSE; 
     if(pws->mParser->DetectMetaTag(buf, theNumRead,
@@ -2334,14 +2329,14 @@ ParserWriteFunc(nsIInputStream* in,
       // one of a designated set that we ignore.
       if (NS_SUCCEEDED(result) &&
           ((kCharsetFromByteOrderMark == guessSource) ||
-           (!preferred.Equals(NS_LITERAL_STRING("UTF-16")) &&
-            !preferred.Equals(NS_LITERAL_STRING("UTF-16BE")) &&
-            !preferred.Equals(NS_LITERAL_STRING("UTF-16LE")) &&
-            !preferred.Equals(NS_LITERAL_STRING("UTF-32BE")) &&
-            !preferred.Equals(NS_LITERAL_STRING("UTF-32LE"))))) {
-        guess.Assign(preferred);
+           (!preferred.Equals(NS_LITERAL_CSTRING("UTF-16")) &&
+            !preferred.Equals(NS_LITERAL_CSTRING("UTF-16BE")) &&
+            !preferred.Equals(NS_LITERAL_CSTRING("UTF-16LE")) &&
+            !preferred.Equals(NS_LITERAL_CSTRING("UTF-32BE")) &&
+            !preferred.Equals(NS_LITERAL_CSTRING("UTF-32LE"))))) {
+        guess = preferred;
         pws->mParser->SetDocumentCharset(guess, guessSource); 
-        pws->mParser->SetSinkCharset(guess);
+        pws->mParser->SetSinkCharset(preferred);
         nsCOMPtr<nsICachingChannel> channel(do_QueryInterface(pws->mRequest));
         if (channel) {
           nsCOMPtr<nsISupports> cacheToken;
@@ -2353,7 +2348,7 @@ ParserWriteFunc(nsIInputStream* in,
               nsresult rv =
 #endif
                 cacheDescriptor->SetMetaDataElement("charset",
-                                                    NS_ConvertUCS2toUTF8(guess).get());
+                                                    guess.get());
               NS_ASSERTION(NS_SUCCEEDED(rv),"cannot SetMetaDataElement");
             }
           }
