@@ -48,7 +48,6 @@
 #include "nsTextToSubURI.h"
 #include "nsCRT.h"
 
-static NS_DEFINE_CID(kITextToSubURIIID, NS_ITEXTTOSUBURI_IID);
 static NS_DEFINE_CID(kCharsetConverterManagerCID, NS_ICHARSETCONVERTERMANAGER_CID);
 
 nsTextToSubURI::nsTextToSubURI()
@@ -229,8 +228,10 @@ NS_IMETHODIMP  nsTextToSubURI::UnEscapeURIForUI(const nsACString & aCharset,
                                                 const nsACString &aURIFragment, 
                                                 nsAString &_retval)
 {
-  nsCAutoString unescapedSpec(aURIFragment);
-  NS_UnescapeURL(unescapedSpec);
+  nsCAutoString unescapedSpec;
+  // skip control octets (0x00 - 0x1f and 0x7f) when unescaping
+  NS_UnescapeURL(PromiseFlatCString(aURIFragment), 
+                 esc_SkipControl | esc_AlwaysCopy, unescapedSpec);
 
   return convertURItoUnicode(PromiseFlatCString(aCharset), unescapedSpec, PR_TRUE, _retval);
 }
