@@ -43,16 +43,18 @@
 #include "nsIComponentManager.h"
 #include "nsICategoryManager.h"
 #include "nsICharsetConverterManager.h"
-#include "nsICharsetConverterManager2.h"
+#include "nsEncoderDecoderUtils.h"
 #include "nsIUnicodeDecoder.h"
 #include "nsIUnicodeEncoder.h"
 #include "nsICharsetAlias.h"
 #include "nsIServiceManager.h"
 
 
+#include "nsUConvCID.h"
 #include "nsCharsetConverterManager.h"
 #include "nsCharsetAlias.h"
 #include "nsTextToSubURI.h"
+#include "nsUTF8ConverterService.h"
 #include "nsConverterInputStream.h"
 #include "nsPlatformCharset.h"
 
@@ -114,6 +116,8 @@
 #include "nsCP1257ToUnicode.h"
 #include "nsCP1258ToUnicode.h"
 #include "nsCP874ToUnicode.h"
+#include "nsISO885911ToUnicode.h"
+#include "nsTIS620ToUnicode.h"
 #include "nsCP866ToUnicode.h"
 #include "nsKOI8RToUnicode.h"
 #include "nsKOI8UToUnicode.h"
@@ -165,6 +169,8 @@
 #include "nsUnicodeToCP1257.h"
 #include "nsUnicodeToCP1258.h"
 #include "nsUnicodeToCP874.h"
+#include "nsUnicodeToISO885911.h"
+#include "nsUnicodeToTIS620.h"
 #include "nsUnicodeToCP866.h"
 #include "nsUnicodeToKOI8R.h"
 #include "nsUnicodeToKOI8U.h"
@@ -246,6 +252,7 @@
 #include "nsUCvTW2Dll.h"
 #include "nsEUCTWToUnicode.h"
 #include "nsUnicodeToEUCTW.h"
+#ifdef MOZ_EXTRA_X11CONVERTERS
 #include "nsUnicodeToCNS11643p1.h"
 #include "nsUnicodeToCNS11643p2.h"
 #include "nsUnicodeToCNS11643p3.h"
@@ -253,6 +260,7 @@
 #include "nsUnicodeToCNS11643p5.h"
 #include "nsUnicodeToCNS11643p6.h"
 #include "nsUnicodeToCNS11643p7.h"
+#endif
 
 // ucvtw
 #include "nsUCvTWCID.h"
@@ -297,7 +305,9 @@
 #include "nsUnicodeToCP936.h"
 #include "nsGB2312ToUnicodeV2.h"
 #include "nsUnicodeToGB2312V2.h"
+#ifdef MOZ_EXTRA_X11CONVERTERS
 #include "nsUnicodeToGB2312GL.h"
+#endif
 #include "nsISO2022CNToUnicode.h"
 #include "nsUnicodeToISO2022CN.h"
 #include "gbku.h"
@@ -340,7 +350,9 @@ NS_UCONV_REG_UNREG("windows-1255", NS_CP1255TOUNICODE_CID, NS_UNICODETOCP1255_CI
 NS_UCONV_REG_UNREG("windows-1256", NS_CP1256TOUNICODE_CID, NS_UNICODETOCP1256_CID)
 NS_UCONV_REG_UNREG("windows-1257", NS_CP1257TOUNICODE_CID, NS_UNICODETOCP1257_CID)
 NS_UCONV_REG_UNREG("windows-1258", NS_CP1258TOUNICODE_CID, NS_UNICODETOCP1258_CID)
-NS_UCONV_REG_UNREG("TIS-620", NS_CP874TOUNICODE_CID, NS_UNICODETOCP874_CID)
+NS_UCONV_REG_UNREG("TIS-620", NS_TIS620TOUNICODE_CID, NS_UNICODETOTIS620_CID)
+NS_UCONV_REG_UNREG("windows-874", NS_CP874TOUNICODE_CID, NS_UNICODETOCP874_CID)
+NS_UCONV_REG_UNREG("ISO-8859-11", NS_ISO885911TOUNICODE_CID, NS_UNICODETOISO885911_CID)
 NS_UCONV_REG_UNREG("IBM866", NS_CP866TOUNICODE_CID, NS_UNICODETOCP866_CID)
 NS_UCONV_REG_UNREG("KOI8-R", NS_KOI8RTOUNICODE_CID, NS_UNICODETOKOI8R_CID)
 NS_UCONV_REG_UNREG("KOI8-U", NS_KOI8UTOUNICODE_CID, NS_UNICODETOKOI8U_CID)
@@ -404,6 +416,7 @@ NS_UCONV_REG_UNREG_ENCODER("jis_0212-1990" , NS_UNICODETOJISX0212_CID)
 
     // ucvtw2
 NS_UCONV_REG_UNREG("x-euc-tw", NS_EUCTWTOUNICODE_CID, NS_UNICODETOEUCTW_CID)
+#ifdef MOZ_EXTRA_X11CONVERTERS
 NS_UCONV_REG_UNREG_ENCODER("x-cns-11643-1" , NS_UNICODETOCNS11643P1_CID)
 NS_UCONV_REG_UNREG_ENCODER("x-cns-11643-2" , NS_UNICODETOCNS11643P2_CID)
 NS_UCONV_REG_UNREG_ENCODER("x-cns-11643-3" , NS_UNICODETOCNS11643P3_CID)
@@ -411,6 +424,7 @@ NS_UCONV_REG_UNREG_ENCODER("x-cns-11643-4" , NS_UNICODETOCNS11643P4_CID)
 NS_UCONV_REG_UNREG_ENCODER("x-cns-11643-5" , NS_UNICODETOCNS11643P5_CID)
 NS_UCONV_REG_UNREG_ENCODER("x-cns-11643-6" , NS_UNICODETOCNS11643P6_CID)
 NS_UCONV_REG_UNREG_ENCODER("x-cns-11643-7" , NS_UNICODETOCNS11643P7_CID)
+#endif
 
     // ucvtw
 NS_UCONV_REG_UNREG("Big5", NS_BIG5TOUNICODE_CID, NS_UNICODETOBIG5_CID)
@@ -442,10 +456,14 @@ NS_UCONV_REG_UNREG("x-gbk", NS_GBKTOUNICODE_CID, NS_UNICODETOGBK_CID)
 NS_UCONV_REG_UNREG_ENCODER("x-gbk-noascii",  NS_UNICODETOGBKNOASCII_CID)
 #endif
 NS_UCONV_REG_UNREG("HZ-GB-2312", NS_HZTOUNICODE_CID, NS_UNICODETOHZ_CID)
+#ifdef MOZ_EXTRA_X11CONVERTERS
 NS_UCONV_REG_UNREG_ENCODER("gb_2312-80",  NS_UNICODETOGB2312GL_CID)
+#endif
 NS_UCONV_REG_UNREG("gb18030", NS_GB18030TOUNICODE_CID, NS_UNICODETOGB18030_CID)
+#ifdef MOZ_EXTRA_X11CONVERTERS
 NS_UCONV_REG_UNREG_ENCODER("gb18030.2000-0",  NS_UNICODETOGB18030Font0_CID)
 NS_UCONV_REG_UNREG_ENCODER("gb18030.2000-1",  NS_UNICODETOGB18030Font1_CID)
+#endif
 NS_UCONV_REG_UNREG_DECODER("ISO-2022-CN", NS_ISO2022CNTOUNICODE_CID)
   
 NS_CONVERTER_REGISTRY_END
@@ -515,19 +533,21 @@ NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToGBKNoAscii)
 #endif
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsHZToUnicode)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToHZ)
+#ifdef MOZ_EXTRA_X11CONVERTERS
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToGB2312GL)
+#endif
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsGB18030ToUnicode)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToGB18030)
+#ifdef MOZ_EXTRA_X11CONVERTERS
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToGB18030Font0)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsUnicodeToGB18030Font1)
+#endif
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsISO2022CNToUnicode)
 
 
 //----------------------------------------------------------------------------
 // Global functions and data [declaration]
 
-static NS_DEFINE_CID(kComponentManagerCID, NS_COMPONENTMANAGER_CID);
-
 #define DECODER_NAME_BASE "Unicode Decoder-"
 #define ENCODER_NAME_BASE "Unicode Encoder-"
 
@@ -677,6 +697,7 @@ nsConverterManagerDataRegister(nsIComponentManager* aCompMgr,
 
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsCharsetConverterManager)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsTextToSubURI)
+NS_GENERIC_FACTORY_CONSTRUCTOR(nsUTF8ConverterService)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsCharsetAlias2)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsConverterInputStream)
 NS_GENERIC_FACTORY_CONSTRUCTOR_INIT(nsPlatformCharset, Init)
@@ -726,6 +747,11 @@ static const nsModuleComponentInfo components[] =
     NS_UNICODEENCODEHELPER_CONTRACTID, 
     nsUnicodeEncodeHelperConstructor 
   },
+  { 
+    "Converter to/from UTF8 with charset", NS_UTF8CONVERTERSERVICE_CID,
+    NS_UTF8CONVERTERSERVICE_CONTRACTID, 
+    nsUTF8ConverterServiceConstructor
+  },
   { 
     "Unicode Encoder / Decoder for Script", NS_ISCRIPTABLEUNICODECONVERTER_CID,
     NS_ISCRIPTABLEUNICODECONVERTER_CONTRACTID, 
@@ -916,8 +942,18 @@ static const nsModuleComponentInfo components[] =
     nsCP1258ToUnicodeConstructor ,
   },
   { 
-    DECODER_NAME_BASE "TIS-620" , NS_CP874TOUNICODE_CID, 
+    DECODER_NAME_BASE "TIS-620" , NS_TIS620TOUNICODE_CID, 
     NS_UNICODEDECODER_CONTRACTID_BASE "TIS-620",
+    nsTIS620ToUnicodeConstructor ,
+  },
+  { 
+    DECODER_NAME_BASE "ISO-8859-11" , NS_ISO885911TOUNICODE_CID, 
+    NS_UNICODEDECODER_CONTRACTID_BASE "ISO-8859-11",
+    nsISO885911ToUnicodeConstructor ,
+  },
+  { 
+    DECODER_NAME_BASE "windows-874" , NS_CP874TOUNICODE_CID, 
+    NS_UNICODEDECODER_CONTRACTID_BASE "windows-874",
     nsCP874ToUnicodeConstructor ,
   },
   { 
@@ -1211,9 +1247,19 @@ static const nsModuleComponentInfo components[] =
     nsUnicodeToCP1258Constructor, 
   },
   { 
-    ENCODER_NAME_BASE "TIS-620" , NS_UNICODETOCP874_CID, 
+    ENCODER_NAME_BASE "TIS-620" , NS_UNICODETOTIS620_CID, 
     NS_UNICODEENCODER_CONTRACTID_BASE "TIS-620",
-    nsUnicodeToCP874Constructor, 
+    nsUnicodeToTIS620Constructor, 
+  },
+  { 
+    ENCODER_NAME_BASE "ISO-8859-11" , NS_UNICODETOISO885911_CID, 
+    NS_UNICODEENCODER_CONTRACTID_BASE "ISO-8859-11",
+    nsUnicodeToISO885911Constructor ,
+  },
+  { 
+    ENCODER_NAME_BASE "windows-874" , NS_UNICODETOCP874_CID, 
+    NS_UNICODEENCODER_CONTRACTID_BASE "windows-874",
+    nsUnicodeToCP874Constructor ,
   },
   { 
     ENCODER_NAME_BASE "IBM866" , NS_UNICODETOCP866_CID, 
@@ -1538,6 +1584,7 @@ static const nsModuleComponentInfo components[] =
     NS_UNICODEENCODER_CONTRACTID_BASE "x-euc-tw",
     nsUnicodeToEUCTWConstructor,
   },
+#ifdef MOZ_EXTRA_X11CONVERTERS
   { 
     ENCODER_NAME_BASE "x-cns-11643-1" , NS_UNICODETOCNS11643P1_CID, 
     NS_UNICODEENCODER_CONTRACTID_BASE "x-cns-11643-1",
@@ -1573,6 +1620,7 @@ static const nsModuleComponentInfo components[] =
     NS_UNICODEENCODER_CONTRACTID_BASE "x-cns-11643-7",
     nsUnicodeToCNS11643p7Constructor,
   },
+#endif
 
   // ucvtw
   { 
@@ -1714,16 +1762,19 @@ static const nsModuleComponentInfo components[] =
     NS_UNICODEENCODER_CONTRACTID_BASE "HZ-GB-2312",
     nsUnicodeToHZConstructor, 
   },  
+#ifdef MOZ_EXTRA_X11CONVERTERS
   { 
     ENCODER_NAME_BASE "gb_2312-80" , NS_UNICODETOGB2312GL_CID, 
     NS_UNICODEENCODER_CONTRACTID_BASE "gb_2312-80",
     nsUnicodeToGB2312GLConstructor, 
   },
+#endif
   { 
     DECODER_NAME_BASE "gb18030" , NS_GB18030TOUNICODE_CID, 
     NS_UNICODEDECODER_CONTRACTID_BASE "gb18030",
     nsGB18030ToUnicodeConstructor ,
   },  
+#ifdef MOZ_EXTRA_X11CONVERTERS
   { 
     ENCODER_NAME_BASE "gb18030.2000-0" , NS_UNICODETOGB18030Font0_CID, 
     NS_UNICODEENCODER_CONTRACTID_BASE "gb18030.2000-0",
@@ -1734,6 +1785,7 @@ static const nsModuleComponentInfo components[] =
     NS_UNICODEENCODER_CONTRACTID_BASE "gb18030.2000-1",
     nsUnicodeToGB18030Font1Constructor, 
   },  
+#endif
   { 
     ENCODER_NAME_BASE "gb18030" , NS_UNICODETOGB18030_CID, 
     NS_UNICODEENCODER_CONTRACTID_BASE "gb18030",
