@@ -59,7 +59,7 @@
 #include "ipcdPrivate.h"
 #include "ipcd.h"
 
-#ifdef XP_UNIX
+#if defined(XP_UNIX) || defined(XP_OS2)
 void
 IPC_Sleep(int seconds)
 {
@@ -70,7 +70,9 @@ IPC_Sleep(int seconds)
     }
     LOG(("\ndone sleeping\n"));
 }
+#endif
 
+#if defined(XP_UNIX)
 //-----------------------------------------------------------------------------
 // ipc directory and locking...
 //-----------------------------------------------------------------------------
@@ -187,7 +189,7 @@ static void ShutdownDaemonDir()
 ipcClient *ipcClients = NULL;
 int        ipcClientCount = 0;
 
-#ifdef XP_UNIX
+#if defined(XP_UNIX) || defined(XP_OS2)
 //
 // the first element of this array is always zero; this is done so that the
 // k'th element of ipcClientArray corresponds to the k'th element of
@@ -350,7 +352,7 @@ static void PollLoop(PRFileDesc *listenFD)
 PRStatus
 IPC_PlatformSendMsg(ipcClient  *client, ipcMessage *msg)
 {
-#ifdef XP_UNIX
+#if defined(XP_UNIX) || defined(XP_OS2)
     LOG(("IPC_PlatformSendMsg\n"));
 
     //
@@ -377,7 +379,7 @@ IPC_PlatformSendMsg(ipcClient  *client, ipcMessage *msg)
 
 int main(int argc, char **argv)
 {
-#ifdef XP_UNIX
+#if defined(XP_UNIX) || defined(XP_OS2)
     PRFileDesc *listenFD = NULL;
     PRNetAddr addr;
 
@@ -406,10 +408,12 @@ int main(int argc, char **argv)
     else
         PL_strncpyz(addr.local.path, argv[1], sizeof(addr.local.path));
 
+#ifndef XP_OS2
     if (!InitDaemonDir(addr.local.path)) {
         LOG(("InitDaemonDir failed\n"));
         goto end;
     }
+#endif
 
     listenFD = PR_OpenTCPSocket(PR_AF_LOCAL);
     if (!listenFD) {
@@ -436,11 +440,13 @@ end:
 
     //IPC_Sleep(5);
 
+#ifndef XP_OS2
     // it is critical that we release the lock before closing the socket,
     // otherwise, a client might launch another daemon that would be unable
     // to acquire the lock and would then leave the client without a daemon.
  
     ShutdownDaemonDir();
+#endif
 
     if (listenFD) {
         LOG(("closing socket\n"));
