@@ -59,12 +59,12 @@
 #include "nsIAppShell.h"
 #include "nsIJSContextStack.h"
 
-/* XXX
+/*
  * defining CAUTIOUS_SCRIPTHOOK makes jsds disable GC while calling out to the
- * script hook.  This is a hack to avoid some js engine problems that I havn't
- * properly tracked down.  I'm lame.
+ * script hook.  This was a hack to avoid some js engine problems that should
+ * be fixed now (see Mozilla bug 77636).
  */
-#define CAUTIOUS_SCRIPTHOOK
+#undef CAUTIOUS_SCRIPTHOOK
 
 #ifdef DEBUG_verbose
 #   define DEBUG_COUNT(name, count)                                             \
@@ -96,14 +96,15 @@
 }
 
 #define JSDS_MAJOR_VERSION 1
-#define JSDS_MINOR_VERSION 1
+#define JSDS_MINOR_VERSION 2
 
 #define NS_CATMAN_CTRID   "@mozilla.org/categorymanager;1"
 #define NS_JSRT_CTRID     "@mozilla.org/js/xpc/RuntimeService;1"
 
 #define AUTOREG_CATEGORY  "xpcom-autoregistration"
 #define APPSTART_CATEGORY "app-startup"
-#define JSD_STARTUP_ENTRY "JSDebugger Startup Observer"
+#define JSD_AUTOREG_ENTRY "JSDebugger Startup Observer"
+#define JSD_STARTUP_ENTRY "JSDebugger Startup Observer,service"
 
 JS_STATIC_DLL_CALLBACK (JSBool)
 jsds_GCCallbackProc (JSContext *cx, JSGCStatus status);
@@ -778,7 +779,7 @@ jsds_ScriptHookProc (JSDContext* jsdc, JSDScript* jsdscript, JSBool creating,
 
 /* Contexts */
 /*
-NS_IMPL_THREADSAFE_ISUPPORTS1(jsdContext, jsdIContext)
+NS_IMPL_THREADSAFE_ISUPPORTS1(jsdContext, jsdIContext); 
 
 NS_IMETHODIMP
 jsdContext::GetJSDContext(JSDContext **_rval)
@@ -2269,7 +2270,7 @@ jsdService::GetInitAtStartup (PRBool *_rval)
         nsresult autoreg_rv, appstart_rv;
         
         autoreg_rv = categoryManager->GetCategoryEntry(AUTOREG_CATEGORY, 
-                                                       JSD_STARTUP_ENTRY,
+                                                       JSD_AUTOREG_ENTRY,
                                                        getter_Copies(notused));
         appstart_rv = categoryManager->GetCategoryEntry(APPSTART_CATEGORY,
                                                         JSD_STARTUP_ENTRY,
@@ -2337,7 +2338,7 @@ jsdService::SetInitAtStartup (PRBool state)
 
     if (state) {
         rv = categoryManager->AddCategoryEntry(AUTOREG_CATEGORY,
-                                               JSD_STARTUP_ENTRY,
+                                               JSD_AUTOREG_ENTRY,
                                                jsdASObserverCtrID,
                                                PR_TRUE, PR_TRUE, nsnull);
         if (NS_FAILED(rv))
@@ -2351,7 +2352,7 @@ jsdService::SetInitAtStartup (PRBool state)
         mInitAtStartup = triYes;
     } else {
         rv = categoryManager->DeleteCategoryEntry(AUTOREG_CATEGORY,
-                                                  JSD_STARTUP_ENTRY, PR_TRUE);
+                                                  JSD_AUTOREG_ENTRY, PR_TRUE);
         if (NS_FAILED(rv))
             return rv;
         rv = categoryManager->DeleteCategoryEntry(APPSTART_CATEGORY,
@@ -2532,7 +2533,7 @@ jsdService::Off (void)
     mOn = PR_FALSE;
 
 #ifdef DEBUG
-    printf ("+++ JavaScript debuging hooks removed.\n");
+    printf ("+++ JavaScript debugging hooks removed.\n");
 #endif
 
     return NS_OK;
@@ -3234,9 +3235,7 @@ class jsdASObserver : public nsIObserver
     NS_DECL_ISUPPORTS
     NS_DECL_NSIOBSERVER
 
-    jsdASObserver ()
-    {
-    }    
+    jsdASObserver () {}    
 };
 
 NS_IMPL_THREADSAFE_ISUPPORTS1(jsdASObserver, nsIObserver)
@@ -3266,15 +3265,17 @@ jsdASObserver::Observe (nsISupports *aSubject, const char *aTopic,
         return rv;
 
     rv = jsds->OnForRuntime(rt);
+    if (NS_FAILED(rv))
+        return rv;
     
-    return rv;
+    return jsds->SetFlags(JSD_DISABLE_OBJECT_TRACE);
 }
 
 NS_GENERIC_FACTORY_CONSTRUCTOR(jsdASObserver)
 
 static const nsModuleComponentInfo components[] = {
-    {"JSDService", JSDSERVICE_CID,     jsdServiceCtrID, jsdServiceConstructor},
-    {"JSDASObserver",  JSDASO_CID,  jsdASObserverCtrID, jsdASObserverConstructor}
+    {"JSDService", JSDSERVICE_CID,    jsdServiceCtrID, jsdServiceConstructor},
+    {"JSDASObserver",  JSDASO_CID, jsdASObserverCtrID, jsdASObserverConstructor}
 };
 
 NS_IMPL_NSGETMODULE(JavaScript_Debugger, components)
