@@ -73,7 +73,14 @@ struct JSAtom {
 #define ATOM_TO_STRING(atom)      JSVAL_TO_STRING(ATOM_KEY(atom))
 #define ATOM_IS_BOOLEAN(atom)     JSVAL_IS_BOOLEAN(ATOM_KEY(atom))
 #define ATOM_TO_BOOLEAN(atom)     JSVAL_TO_BOOLEAN(ATOM_KEY(atom))
-#define ATOM_BYTES(atom)          JS_GetStringBytes(ATOM_TO_STRING(atom))
+
+/*
+ * Return a printable, lossless char[] representation of a string-type atom.
+ * The lifetime of the result extends at least until the next GC activation,
+ * longer if cx's string newborn root is not overwritten.
+ */
+extern JS_FRIEND_API(const char *)
+js_AtomToPrintableString(JSContext *cx, JSAtom *atom);
 
 #define ATOM_KEYWORD(atom)        ((struct keyword *)(atom)->entry.value)
 #define ATOM_SET_KEYWORD(atom,kw) ((atom)->entry.value = (kw))
@@ -173,6 +180,7 @@ struct JSAtomState {
     JSAtom              *inputAtom;
     JSAtom              *lengthAtom;
     JSAtom              *nameAtom;
+    JSAtom              *noSuchMethodAtom;
     JSAtom              *parentAtom;
     JSAtom              *protoAtom;
     JSAtom              *setAtom;
@@ -252,6 +260,7 @@ extern const char   js_index_str[];
 extern const char   js_input_str[];
 extern const char   js_length_str[];
 extern const char   js_name_str[];
+extern const char   js_noSuchMethod_str[];
 extern const char   js_parent_str[];
 extern const char   js_proto_str[];
 extern const char   js_setter_str[];
