@@ -6,7 +6,7 @@
  * the License at http://www.mozilla.org/NPL/
  *
  * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express oqr
+ * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
  *
@@ -70,11 +70,14 @@ js_NewContext(JSRuntime *rt, size_t stackChunkSize)
     memset(cx, 0, sizeof *cx);
 
     cx->runtime = rt;
+#if JS_STACK_GROWTH_DIRECTION > 0
+    cx->stackLimit = (jsuword)-1;
+#endif
 #ifdef JS_THREADSAFE
     js_InitContextForLocking(cx);
 #endif
 
-    JS_LOCK_RUNTIME(rt);
+    JS_LOCK_GC(rt);
     for (;;) {
         first = (rt->contextList.next == &rt->contextList);
         if (rt->state == JSRTS_UP) {
@@ -89,7 +92,7 @@ js_NewContext(JSRuntime *rt, size_t stackChunkSize)
         JS_WAIT_CONDVAR(rt->stateChange, JS_NO_TIMEOUT);
     }
     JS_APPEND_LINK(&cx->links, &rt->contextList);
-    JS_UNLOCK_RUNTIME(rt);
+    JS_UNLOCK_GC(rt);
 
     /*
      * First we do the infallible, every-time per-context initializations.
@@ -103,7 +106,7 @@ js_NewContext(JSRuntime *rt, size_t stackChunkSize)
     cx->jsop_ne = JSOP_NE;
     JS_InitArenaPool(&cx->stackPool, "stack", stackChunkSize, sizeof(jsval));
     JS_InitArenaPool(&cx->codePool, "code", 1024, sizeof(jsbytecode));
-    JS_InitArenaPool(&cx->notePool, "note", 256, sizeof(jssrcnote));
+    JS_InitArenaPool(&cx->notePool, "note", 1024, sizeof(jssrcnote));
     JS_InitArenaPool(&cx->tempPool, "temp", 1024, sizeof(jsdouble));
 
 #if JS_HAS_REGEXPS
@@ -139,10 +142,10 @@ js_NewContext(JSRuntime *rt, size_t stackChunkSize)
             return NULL;
         }
 
-        JS_LOCK_RUNTIME(rt);
+        JS_LOCK_GC(rt);
         rt->state = JSRTS_UP;
         JS_NOTIFY_ALL_CONDVAR(rt->stateChange);
-        JS_UNLOCK_RUNTIME(rt);
+        JS_UNLOCK_GC(rt);
     }
 
     return cx;
@@ -158,15 +161,30 @@ js_DestroyContext(JSContext *cx, JSGCMode gcmode)
     rt = cx->runtime;
 
     /* Remove cx from context list first. */
-    JS_LOCK_RUNTIME(rt);
+    JS_LOCK_GC(rt);
     JS_ASSERT(rt->state == JSRTS_UP || rt->state == JSRTS_LAUNCHING);
     JS_REMOVE_LINK(&cx->links);
     last = (rt->contextList.next == &rt->contextList);
     if (last)
         rt->state = JSRTS_LANDING;
-    JS_UNLOCK_RUNTIME(rt);
+    JS_UNLOCK_GC(rt);
 
     if (last) {
+#ifdef JS_THREADSAFE
+        /*
+         * If cx is not in a request already, begin one now so that we wait
+         * for any racing GC started on a not-last context to finish, before
+         * we plow ahead and unpin atoms.  Note that even though we begin a
+         * request here if necessary, we end all requests on cx below before
+         * forcing a final GC.  This lets any not-last context destruction
+         * racing in another thread try to force or maybe run the GC, but by
+         * that point, rt->state will not be JSRTS_UP, and that GC attempt
+         * will return early.
+         */
+        if (cx->requestDepth == 0)
+            JS_BeginRequest(cx);
+#endif
+
         /* Unpin all pinned atoms before final GC. */
         js_UnpinPinnedAtoms(&rt->atomState);
 
@@ -207,7 +225,7 @@ js_DestroyContext(JSContext *cx, JSGCMode gcmode)
 
     if (last) {
         /* Always force, so we wait for any racing GC to finish. */
-        js_ForceGC(cx);
+        js_ForceGC(cx, GC_LAST_CONTEXT);
 
         /* Iterate until no finalizer removes a GC root or lock. */
         while (rt->gcPoke)
@@ -218,13 +236,13 @@ js_DestroyContext(JSContext *cx, JSGCMode gcmode)
             js_FreeAtomState(cx, &rt->atomState);
 
         /* Take the runtime down, now that it has no contexts or atoms. */
-        JS_LOCK_RUNTIME(rt);
+        JS_LOCK_GC(rt);
         rt->state = JSRTS_DOWN;
         JS_NOTIFY_ALL_CONDVAR(rt->stateChange);
-        JS_UNLOCK_RUNTIME(rt);
+        JS_UNLOCK_GC(rt);
     } else {
         if (gcmode == JS_FORCE_GC)
-            js_ForceGC(cx);
+            js_ForceGC(cx, 0);
         else if (gcmode == JS_MAYBE_GC)
             JS_MaybeGC(cx);
     }
@@ -245,12 +263,18 @@ js_DestroyContext(JSContext *cx, JSGCMode gcmode)
         JS_free(cx, temp);
     }
 
+    /* Destroy the resolve recursion damper. */
+    if (cx->resolvingTable) {
+        JS_DHashTableDestroy(cx->resolvingTable);
+        cx->resolvingTable = NULL;
+    }
+
     /* Finally, free cx itself. */
     free(cx);
 }
 
 JSBool
-js_LiveContext(JSRuntime *rt, JSContext *cx)
+js_ValidContextPointer(JSRuntime *rt, JSContext *cx)
 {
     JSCList *cl;
 
@@ -263,18 +287,20 @@ js_LiveContext(JSRuntime *rt, JSContext *cx)
 }
 
 JSContext *
-js_ContextIterator(JSRuntime *rt, JSContext **iterp)
+js_ContextIterator(JSRuntime *rt, JSBool unlocked, JSContext **iterp)
 {
     JSContext *cx = *iterp;
 
-    JS_LOCK_RUNTIME(rt);
+    if (unlocked)
+        JS_LOCK_GC(rt);
     if (!cx)
         cx = (JSContext *)&rt->contextList;
     cx = (JSContext *)cx->links.next;
     if (&cx->links == &rt->contextList)
         cx = NULL;
     *iterp = cx;
-    JS_UNLOCK_RUNTIME(rt);
+    if (unlocked)
+        JS_UNLOCK_GC(rt);
     return cx;
 }
 
@@ -320,28 +346,31 @@ ReportError(JSContext *cx, const char *message, JSErrorReport *reportp)
  * type message, and then hope the process ends swiftly.
  */
 void
-js_ReportOutOfMemory(JSContext *cx, JSErrorCallback errorCallback)
+js_ReportOutOfMemory(JSContext *cx, JSErrorCallback callback)
 {
-    JSStackFrame *fp = cx->fp;
+    JSStackFrame *fp;
     JSErrorReport report;
     JSErrorReporter onError = cx->errorReporter;
+
     /* Get the message for this error, but we won't expand any arguments. */
-    const JSErrorFormatString *fmtData = (*errorCallback)(NULL, NULL, JSMSG_OUT_OF_MEMORY);
-    const char *msg = fmtData ? fmtData->format : "Out Of Memory";
+    const JSErrorFormatString *efs = callback(NULL, NULL, JSMSG_OUT_OF_MEMORY);
+    const char *msg = efs ? efs->format : "Out of memory";
 
+    /* Fill out the report, but don't do anything that requires allocation. */
     memset(&report, 0, sizeof (struct JSErrorReport));
-    /* Fill out the report, but don't do anything that requires an allocation. */
-    report.errorNumber = JSMSG_OUT_OF_MEMORY;
     report.flags = JSREPORT_ERROR;
+    report.errorNumber = JSMSG_OUT_OF_MEMORY;
 
-    /* Walk stack until we find a frame that is associated with
-       some script rather than a native frame. */
-    while (fp && (!fp->script || !fp->pc))
-        fp = fp->down;
-
-    if (fp) {
-        report.filename = fp->script->filename;
-        report.lineno = js_PCToLineNumber(fp->script, fp->pc);
+    /*
+     * Walk stack until we find a frame that is associated with some script
+     * rather than a native frame.
+     */
+    for (fp = cx->fp; fp; fp = fp->down) {
+        if (fp->script && fp->pc) {
+            report.filename = fp->script->filename;
+            report.lineno = js_PCToLineNumber(cx, fp->script, fp->pc);
+            break;
+        }
     }
 
     /*
@@ -357,44 +386,44 @@ js_ReportOutOfMemory(JSContext *cx, JSErrorCallback errorCallback)
     }
 
     if (onError)
-        (*onError)(cx, msg, &report);
+        onError(cx, msg, &report);
 }
 
 JSBool
 js_ReportErrorVA(JSContext *cx, uintN flags, const char *format, va_list ap)
 {
-    JSStackFrame *fp;
-    JSErrorReport report, *reportp;
     char *last;
+    JSStackFrame *fp;
+    JSErrorReport report;
     JSBool warning;
 
-    fp = cx->fp;
+    if ((flags & JSREPORT_STRICT) && !JS_HAS_STRICT_OPTION(cx))
+        return JS_TRUE;
 
-    /* Walk stack until we find a frame that is associated with
-       some script rather than a native frame. */
-    while (fp && (!fp->script || !fp->pc))
-        fp = fp->down;
-
-    reportp = &report;
-    memset(reportp, 0, sizeof (struct JSErrorReport));
-    report.flags = flags;
-    if (fp) {
-        report.filename = fp->script->filename;
-        report.lineno = js_PCToLineNumber(fp->script, fp->pc);
-        /* XXX should fetch line somehow */
-    }
     last = JS_vsmprintf(format, ap);
     if (!last)
         return JS_FALSE;
 
-    ReportError(cx, last, reportp);
-    free(last);
+    memset(&report, 0, sizeof (struct JSErrorReport));
+    report.flags = flags;
 
-    warning = JSREPORT_IS_WARNING(reportp->flags);
+    /* Find the top-most active script frame, for best line number blame. */
+    for (fp = cx->fp; fp; fp = fp->down) {
+        if (fp->script && fp->pc) {
+            report.filename = fp->script->filename;
+            report.lineno = js_PCToLineNumber(cx, fp->script, fp->pc);
+            break;
+        }
+    }
+
+    warning = JSREPORT_IS_WARNING(report.flags);
     if (warning && JS_HAS_WERROR_OPTION(cx)) {
-        reportp->flags &= ~JSREPORT_WARNING;
+        report.flags &= ~JSREPORT_WARNING;
         warning = JS_FALSE;
     }
+
+    ReportError(cx, last, &report);
+    free(last);
     return warning;
 }
 
@@ -415,7 +444,7 @@ js_ExpandErrorArguments(JSContext *cx, JSErrorCallback callback,
                         char **messagep, JSErrorReport *reportp,
                         JSBool *warningp, JSBool charArgs, va_list ap)
 {
-    const JSErrorFormatString *fmtData;
+    const JSErrorFormatString *efs;
     int i;
     int argCount;
 
@@ -427,11 +456,11 @@ js_ExpandErrorArguments(JSContext *cx, JSErrorCallback callback,
 
     *messagep = NULL;
     if (callback) {
-        fmtData = (*callback)(userRef, "Mountain View", errorNumber);
-        if (fmtData != NULL) {
+        efs = callback(userRef, NULL, errorNumber);
+        if (efs) {
             size_t totalArgsLength = 0;
             size_t argLengths[10]; /* only {0} thru {9} supported */
-            argCount = fmtData->argCount;
+            argCount = efs->argCount;
             JS_ASSERT(argCount <= 10);
             if (argCount > 0) {
                 /*
@@ -466,13 +495,13 @@ js_ExpandErrorArguments(JSContext *cx, JSErrorCallback callback,
              * for {X} in the format.
              */
             if (argCount > 0) {
-                if (fmtData->format) {
+                if (efs->format) {
                     const char *fmt;
                     const jschar *arg;
                     jschar *out;
                     int expandedArgs = 0;
                     size_t expandedLength
-                        = strlen(fmtData->format)
+                        = strlen(efs->format)
                             - (3 * argCount) /* exclude the {n} */
                             + totalArgsLength;
                     /*
@@ -483,9 +512,9 @@ js_ExpandErrorArguments(JSContext *cx, JSErrorCallback callback,
                         JS_malloc(cx, (expandedLength + 1) * sizeof(jschar));
                     if (!out)
                         goto error;
-                    fmt = fmtData->format;
+                    fmt = efs->format;
                     while (*fmt) {
-                        if (*fmt == '{') {      /* balance} */
+                        if (*fmt == '{') {
                             if (isdigit(fmt[1])) {
                                 int d = JS7_UNDEC(fmt[1]);
                                 JS_ASSERT(expandedArgs < argCount);
@@ -515,8 +544,8 @@ js_ExpandErrorArguments(JSContext *cx, JSErrorCallback callback,
                  * Zero arguments: the format string (if it exists) is the
                  * entire message.
                  */
-                if (fmtData->format) {
-                    *messagep = JS_strdup(cx, fmtData->format);
+                if (efs->format) {
+                    *messagep = JS_strdup(cx, efs->format);
                     if (!*messagep)
                         goto error;
                     reportp->ucmessage
@@ -568,42 +597,24 @@ js_ReportErrorNumberVA(JSContext *cx, uintN flags, JSErrorCallback callback,
     char *message;
     JSBool warning;
 
-    report.messageArgs = NULL;
-    report.ucmessage = NULL;
-    message = NULL;
+    if ((flags & JSREPORT_STRICT) && !JS_HAS_STRICT_OPTION(cx))
+        return JS_TRUE;
 
-    fp = cx->fp;
-    if (fp && fp->script && fp->pc) {
-        report.filename = fp->script->filename;
-        report.lineno = js_PCToLineNumber(fp->script, fp->pc);
-    } else {
-        /*  We can't find out where the error was from the current
-            frame so see if the next frame has a script/pc combo we
-            could use */
-        if (fp && fp->down && fp->down->script && fp->down->pc) {
-            report.filename = fp->down->script->filename;
-            report.lineno = js_PCToLineNumber(fp->down->script, fp->down->pc);
-        }
-        else {
-            report.filename = NULL;
-            report.lineno = 0;
-        }
-    }
-
-    /* XXX should fetch line somehow */
-    report.linebuf = NULL;
-    report.tokenptr = NULL;
+    memset(&report, 0, sizeof (struct JSErrorReport));
     report.flags = flags;
     report.errorNumber = errorNumber;
 
     /*
-     * XXX js_ExpandErrorArguments only sometimes fills these in, so we
-     * initialize them to clear garbage.
+     * If we can't find out where the error was based on the current frame,
+     * see if the next frame has a script/pc combo we can use.
      */
-    report.uclinebuf = NULL;
-    report.uctokenptr = NULL;
-    report.ucmessage = NULL;
-    report.messageArgs = NULL;
+    for (fp = cx->fp; fp; fp = fp->down) {
+        if (fp->script && fp->pc) {
+            report.filename = fp->script->filename;
+            report.lineno = js_PCToLineNumber(cx, fp->script, fp->pc);
+            break;
+        }
+    }
 
     if (!js_ExpandErrorArguments(cx, callback, userRef, errorNumber,
                                  &message, &report, &warning, charArgs, ap)) {
@@ -648,12 +659,13 @@ js_ReportErrorAgain(JSContext *cx, const char *message, JSErrorReport *reportp)
     if (onError) {
         JSDebugErrorHook hook = cx->runtime->debugErrorHook;
         if (hook &&
-            !hook(cx, cx->lastMessage, reportp, cx->runtime->debugErrorHookData)) {
+            !hook(cx, cx->lastMessage, reportp,
+                  cx->runtime->debugErrorHookData)) {
             onError = NULL;
         }
     }
     if (onError)
-        (*onError)(cx, cx->lastMessage, reportp);
+        onError(cx, cx->lastMessage, reportp);
 }
 
 void
