@@ -18,7 +18,7 @@
  * Copyright (C) 1998 Netscape Communications Corporation. All
  * Rights Reserved.
  *
- * Contributor(s): 
+ * Contributor(s):
  *
  * Alternatively, the contents of this file may be used under the
  * terms of the GNU Public License (the "GPL"), in which case the
@@ -48,6 +48,7 @@
 #include "jsemit.h"
 #include "jsfun.h"
 #include "jsinterp.h"
+#include "jslock.h"
 #include "jsnum.h"
 #include "jsopcode.h"
 #include "jsscript.h"
@@ -187,7 +188,7 @@ script_compile(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
 
     if (caller->script) {
         file = caller->script->filename;
-        line = js_PCToLineNumber(caller->script, caller->pc);
+        line = js_PCToLineNumber(cx, caller->script, caller->pc);
         principals = caller->script->principals;
     } else {
         file = NULL;
@@ -251,6 +252,7 @@ script_exec(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 }
 
 #if JS_HAS_XDR
+
 static JSBool
 XDRAtomListElement(JSXDRState *xdr, JSAtomListElement *ale)
 {
@@ -331,27 +333,37 @@ XDRAtomMap(JSXDRState *xdr, JSAtomMap *map)
 JSBool
 js_XDRScript(JSXDRState *xdr, JSScript **scriptp, JSBool *hasMagic)
 {
-    JSScript *script;
-    uint32 length, lineno, depth, magic, notelen, numtrys;
+    JSContext *cx;
+    JSScript *script, *newscript;
+    uint32 length, lineno, depth, magic, nsrcnotes, ntrynotes;
     uint32 prologLength, version;
+    JSBool filenameWasSaved;
+    jssrcnote *notes, *sn;
 
+    cx = xdr->cx;
     script = *scriptp;
-    numtrys = 0;
+    nsrcnotes = ntrynotes = 0;
+    filenameWasSaved = JS_FALSE;
+    notes = NULL;
 
     /*
      * Encode prologLength and version after script->length (_2 or greater),
      * but decode both new (>= _2) and old, prolog&version-free (_1) scripts.
-     * Version _3 supports principals serialization.
+     * Version _3 supports principals serialization.  Version _4 reorders the
+     * nsrcnotes and ntrynotes fields to come before everything except magic,
+     * length, prologLength, and version, so that srcnote and trynote storage
+     * can be allocated as part of the JSScript (along with bytecode storage).
      */
     if (xdr->mode == JSXDR_ENCODE)
         magic = JSXDR_MAGIC_SCRIPT_CURRENT;
     if (!JS_XDRUint32(xdr, &magic))
         return JS_FALSE;
-    if (magic != JSXDR_MAGIC_SCRIPT_3 &&
+    if (magic != JSXDR_MAGIC_SCRIPT_4 &&
+        magic != JSXDR_MAGIC_SCRIPT_3 &&
         magic != JSXDR_MAGIC_SCRIPT_2 &&
         magic != JSXDR_MAGIC_SCRIPT_1) {
         if (!hasMagic) {
-            JS_ReportErrorNumber(xdr->cx, js_GetErrorMessage, NULL,
+            JS_ReportErrorNumber(cx, js_GetErrorMessage, NULL,
                                  JSMSG_BAD_SCRIPT_MAGIC);
             return JS_FALSE;
         }
@@ -362,25 +374,25 @@ js_XDRScript(JSXDRState *xdr, JSScript **scriptp, JSBool *hasMagic)
         *hasMagic = JS_TRUE;
 
     if (xdr->mode == JSXDR_ENCODE) {
-        jssrcnote *sn = script->notes;
         length = script->length;
-        prologLength = script->main - script->code;
+        prologLength = PTRDIFF(script->main, script->code, jsbytecode);
         version = (int32)script->version;
         lineno = (uint32)script->lineno;
         depth = (uint32)script->depth;
 
+        /* Count the srcnotes, keeping notes pointing at the first one. */
+        notes = SCRIPT_NOTES(script);
+        for (sn = notes; !SN_IS_TERMINATOR(sn); sn = SN_NEXT(sn))
+            continue;
+        nsrcnotes = PTRDIFF(sn, notes, jssrcnote);
+        nsrcnotes++;            /* room for the terminator */
+
         /* Count the trynotes. */
         if (script->trynotes) {
-            for (; script->trynotes[numtrys].catchStart; numtrys++)
-                continue;
-            numtrys++;          /* room for the end marker */
+            while (script->trynotes[ntrynotes].catchStart)
+                ntrynotes++;
+            ntrynotes++;        /* room for the end marker */
         }
-
-        /* Count the src notes. */
-        while (!SN_IS_TERMINATOR(sn))
-            sn = SN_NEXT(sn);
-        notelen = sn - script->notes;
-        notelen++;              /* room for the terminator */
     }
 
     if (!JS_XDRUint32(xdr, &length))
@@ -390,58 +402,84 @@ js_XDRScript(JSXDRState *xdr, JSScript **scriptp, JSBool *hasMagic)
             return JS_FALSE;
         if (!JS_XDRUint32(xdr, &version))
             return JS_FALSE;
+
+        /* To fuse allocations, we need srcnote and trynote counts early. */
+        if (magic >= JSXDR_MAGIC_SCRIPT_4) {
+            if (!JS_XDRUint32(xdr, &nsrcnotes))
+                return JS_FALSE;
+            if (!JS_XDRUint32(xdr, &ntrynotes))
+                return JS_FALSE;
+        }
     }
 
     if (xdr->mode == JSXDR_DECODE) {
-        script = js_NewScript(xdr->cx, length);
+        script = js_NewScript(cx, length, nsrcnotes, ntrynotes);
         if (!script)
             return JS_FALSE;
         if (magic >= JSXDR_MAGIC_SCRIPT_2) {
             script->main += prologLength;
             script->version = (JSVersion) version;
+
+            /* If we know nsrcnotes, we allocated space for notes in script. */
+            if (magic >= JSXDR_MAGIC_SCRIPT_4)
+                notes = SCRIPT_NOTES(script);
         }
         *scriptp = script;
     }
 
-    /* Control hereafter must goto error on failure, to destroy script. */
-    if (!JS_XDRBytes(xdr, (char **)&script->code, length) ||
-        !XDRAtomMap(xdr, &script->atomMap) ||
-        !JS_XDRUint32(xdr, &notelen) ||
-        /* malloc on decode only */
-        (!(xdr->mode == JSXDR_ENCODE ||
-           (script->notes = (jssrcnote *)
-            JS_malloc(xdr->cx, notelen)) != NULL)) ||
-        !JS_XDRBytes(xdr, (char **)&script->notes, notelen) ||
-        !JS_XDRCStringOrNull(xdr, (char **)&script->filename) ||
-        !JS_XDRUint32(xdr, &lineno) ||
-        !JS_XDRUint32(xdr, &depth) ||
-        !JS_XDRUint32(xdr, &numtrys)) {
+    /*
+     * Control hereafter must goto error on failure, in order for the DECODE
+     * case to destroy script and conditionally free notes, which if non-null
+     * in the (DECODE and version < _4) case must point at a temporary vector
+     * allocated just below.
+     */
+    if (!JS_XDRBytes(xdr, (char *)script->code, length * sizeof(jsbytecode)) ||
+        !XDRAtomMap(xdr, &script->atomMap)) {
         goto error;
     }
 
+    if (magic < JSXDR_MAGIC_SCRIPT_4) {
+        if (!JS_XDRUint32(xdr, &nsrcnotes))
+            goto error;
+        if (xdr->mode == JSXDR_DECODE) {
+            notes = (jssrcnote *) JS_malloc(cx, nsrcnotes * sizeof(jssrcnote));
+            if (!notes)
+                goto error;
+        }
+    }
+
+    if (!JS_XDRBytes(xdr, (char *)notes, nsrcnotes * sizeof(jssrcnote)) ||
+        !JS_XDRCStringOrNull(xdr, (char **)&script->filename) ||
+        !JS_XDRUint32(xdr, &lineno) ||
+        !JS_XDRUint32(xdr, &depth) ||
+        (magic < JSXDR_MAGIC_SCRIPT_4 && !JS_XDRUint32(xdr, &ntrynotes))) {
+        goto error;
+    }
+
+    /* Script principals transcoding support comes with versions >= _3. */
     if (magic >= JSXDR_MAGIC_SCRIPT_3) {
         JSPrincipals *principals;
         uint32 encodeable;
 
         if (xdr->mode == JSXDR_ENCODE) {
             principals = script->principals;
-            encodeable = (xdr->cx->runtime->principalsTranscoder != NULL);
+            encodeable = (cx->runtime->principalsTranscoder != NULL);
             if (!JS_XDRUint32(xdr, &encodeable))
                 goto error;
             if (encodeable &&
-                !xdr->cx->runtime->principalsTranscoder(xdr, &principals)) {
+                !cx->runtime->principalsTranscoder(xdr, &principals)) {
                 goto error;
             }
         } else {
             if (!JS_XDRUint32(xdr, &encodeable))
                 goto error;
             if (encodeable) {
-                if (!xdr->cx->runtime->principalsTranscoder) {
-                    JS_ReportErrorNumber(xdr->cx, js_GetErrorMessage, NULL,
+                if (!cx->runtime->principalsTranscoder) {
+                    JS_ReportErrorNumber(cx, js_GetErrorMessage, NULL,
                                          JSMSG_CANT_DECODE_PRINCIPALS);
                     goto error;
                 }
-                if (!xdr->cx->runtime->principalsTranscoder(xdr, &principals))
+                if (!cx->runtime->principalsTranscoder(xdr, &principals))
                     goto error;
                 script->principals = principals;
             }
@@ -449,22 +487,57 @@ js_XDRScript(JSXDRState *xdr, JSScript **scriptp, JSBool *hasMagic)
     }
 
     if (xdr->mode == JSXDR_DECODE) {
+        const char *filename = script->filename;
+        if (filename) {
+            filename = js_SaveScriptFilename(cx, filename);
+            if (!filename)
+                goto error;
+            JS_free(cx, (void *) script->filename);
+            script->filename = filename;
+            filenameWasSaved = JS_TRUE;
+        }
         script->lineno = (uintN)lineno;
         script->depth = (uintN)depth;
-        if (numtrys) {
-            script->trynotes = (JSTryNote *)
-                JS_malloc(xdr->cx, sizeof(JSTryNote) * (numtrys + 1));
-            if (!script->trynotes)
+
+        if (magic < JSXDR_MAGIC_SCRIPT_4) {
+            /*
+             * Argh, we have to reallocate script, copy notes into the extra
+             * space after the bytecodes, and free the temporary notes vector.
+             * First, add enough slop to nsrcnotes so we can align the address
+             * after the srcnotes of the first trynote.
+             */
+            uint32 osrcnotes = nsrcnotes;
+
+            if (ntrynotes)
+                nsrcnotes += JSTRYNOTE_ALIGNMASK;
+            newscript = (JSScript *) JS_realloc(cx, script,
+                                                sizeof(JSScript) +
+                                                length * sizeof(jsbytecode) +
+                                                nsrcnotes * sizeof(jssrcnote) +
+                                                ntrynotes * sizeof(JSTryNote));
+            if (!newscript)
                 goto error;
-        } else {
-            script->trynotes = NULL;
+
+            *scriptp = script = newscript;
+            script->code = (jsbytecode *)(script + 1);
+            script->main = script->code + prologLength;
+            memcpy(script->code + length, notes, osrcnotes * sizeof(jssrcnote));
+            JS_free(cx, (void *) notes);
+            notes = NULL;
+            if (ntrynotes) {
+                script->trynotes = (JSTryNote *)
+                                   ((jsword)(SCRIPT_NOTES(script) + nsrcnotes) &
+                                    ~(jsword)JSTRYNOTE_ALIGNMASK);
+            }
         }
     }
-    for (; numtrys; numtrys--) {
-        JSTryNote *tn = &script->trynotes[numtrys - 1];
-        uint32 start = (ptrdiff_t) tn->start,
-               catchLength = (ptrdiff_t) tn->length,
-               catchStart = (ptrdiff_t) tn->catchStart;
+
+    while (ntrynotes) {
+        JSTryNote *tn = &script->trynotes[--ntrynotes];
+        uint32 start = (uint32) tn->start,
+               catchLength = (uint32) tn->length,
+               catchStart = (uint32) tn->catchStart;
+
         if (!JS_XDRUint32(xdr, &start) ||
             !JS_XDRUint32(xdr, &catchLength) ||
             !JS_XDRUint32(xdr, &catchStart)) {
@@ -478,17 +551,23 @@ js_XDRScript(JSXDRState *xdr, JSScript **scriptp, JSBool *hasMagic)
 
   error:
     if (xdr->mode == JSXDR_DECODE) {
-        js_DestroyScript(xdr->cx, script);
+        if (script->filename && !filenameWasSaved) {
+            JS_free(cx, (void *) script->filename);
+            script->filename = NULL;
+        }
+        if (notes && magic < JSXDR_MAGIC_SCRIPT_4)
+            JS_free(cx, (void *) notes);
+        js_DestroyScript(cx, script);
         *scriptp = NULL;
     }
     return JS_FALSE;
 }
-  
-#ifndef MOZILLA_CLIENT
+
+#if JS_HAS_XDR_FREEZE_THAW
 /*
  * These cannot be exposed to web content, and chrome does not need them, so
- * we take them out of the Mozilla client altogether.  Script.thaw is a hole
- * wide enough that you could hack your own native function and call it!
+ * we take them out of the Mozilla client altogether.  Fortunately, there is
+ * no way to serialize a native function (see fun_xdrObject in jsfun.c).
  */
 
 static JSBool
@@ -637,9 +716,9 @@ out:
     return ok;
 }
 
-static char js_thaw_str[] = "thaw";
+static const char js_thaw_str[] = "thaw";
 
-#endif /* !defined MOZILLA_CLIENT */
+#endif /* JS_HAS_XDR_FREEZE_THAW */
 #endif /* JS_HAS_XDR */
 
 static JSFunctionSpec script_methods[] = {
@@ -649,10 +728,10 @@ static JSFunctionSpec script_methods[] = {
     {js_toString_str,   script_toString,        0,0,0},
     {"compile",         script_compile,         2,0,0},
     {"exec",            script_exec,            1,0,0},
-#if JS_HAS_XDR && !defined MOZILLA_CLIENT
+#if JS_HAS_XDR_FREEZE_THAW
     {"freeze",          script_freeze,          0,0,0},
     {js_thaw_str,       script_thaw,            1,0,0},
-#endif /* JS_HAS_XDR && !defined MOZILLA_CLIENT */
+#endif /* JS_HAS_XDR_FREEZE_THAW */
     {0,0,0,0,0}
 };
 
@@ -712,7 +791,7 @@ Script(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
     return script_compile(cx, obj, argc, argv, rval);
 }
 
-#if JS_HAS_XDR && !defined MOZILLA_CLIENT
+#if JS_HAS_XDR_FREEZE_THAW
 
 static JSBool
 script_static_thaw(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
@@ -732,11 +811,11 @@ static JSFunctionSpec script_static_methods[] = {
     {0,0,0,0,0}
 };
 
-#else  /* !JS_HAS_XDR || defined MOZILLA_CLIENT */
+#else  /* !JS_HAS_XDR_FREEZE_THAW */
 
 #define script_static_methods   NULL
 
-#endif /* !JS_HAS_XDR || defined MOZILLA_CLIENT */
+#endif /* !JS_HAS_XDR_FREEZE_THAW */
 
 JSObject *
 js_InitScriptClass(JSContext *cx, JSObject *obj)
@@ -747,79 +826,245 @@ js_InitScriptClass(JSContext *cx, JSObject *obj)
 
 #endif /* JS_HAS_SCRIPT_OBJECT */
 
+/*
+ * Shared script filename management.
+ */
+static JSHashTable  *script_filename_table;
+#ifdef JS_THREADSAFE
+static JSLock       *script_filename_table_lock;
+#endif
+
+JS_STATIC_DLL_CALLBACK(int)
+js_compare_strings(const void *k1, const void *k2)
+{
+    return strcmp(k1, k2) == 0;
+}
+
+/* Shared with jsatom.c to save code space. */
+extern void * JS_DLL_CALLBACK
+js_alloc_table_space(void *priv, size_t size);
+
+extern void JS_DLL_CALLBACK
+js_free_table_space(void *priv, void *item);
+
+/* NB: This struct overlays JSHashEntry -- see jshash.h, do not reorganize. */
+typedef struct ScriptFilenameEntry {
+    JSHashEntry         *next;          /* hash chain linkage */
+    JSHashNumber        keyHash;        /* key hash function result */
+    const void          *key;           /* ptr to filename, below */
+    JSPackedBool        mark;           /* mark flag, for GC */
+    char                filename[3];    /* two or more bytes, NUL-terminated */
+} ScriptFilenameEntry;
+
+JS_STATIC_DLL_CALLBACK(JSHashEntry *)
+js_alloc_entry(void *priv, const void *key)
+{
+    size_t nbytes = offsetof(ScriptFilenameEntry, filename) + strlen(key) + 1;
+
+    return (JSHashEntry *) malloc(JS_MAX(nbytes, sizeof(JSHashEntry)));
+}
+
+JS_STATIC_DLL_CALLBACK(void)
+js_free_entry(void *priv, JSHashEntry *he, uintN flag)
+{
+    if (flag != HT_FREE_ENTRY)
+        return;
+    free(he);
+}
+
+static JSHashAllocOps table_alloc_ops = {
+    js_alloc_table_space,   js_free_table_space,
+    js_alloc_entry,         js_free_entry
+};
+
+JSBool
+js_InitScriptGlobals()
+{
+#ifdef JS_THREADSAFE
+    /* Must come through here once in primordial thread to init safely! */
+    if (!script_filename_table_lock) {
+        script_filename_table_lock = JS_NEW_LOCK();
+        if (!script_filename_table_lock)
+            return JS_FALSE;
+    }
+#endif
+    if (!script_filename_table) {
+        JS_ACQUIRE_LOCK(script_filename_table_lock);
+        if (!script_filename_table) {
+            script_filename_table =
+                JS_NewHashTable(16, JS_HashString, js_compare_strings, NULL,
+                                &table_alloc_ops, NULL);
+        }
+        JS_RELEASE_LOCK(script_filename_table_lock);
+        if (!script_filename_table)
+            return JS_FALSE;
+    }
+    return JS_TRUE;
+}
+
+void
+js_FreeScriptGlobals()
+{
+    if (script_filename_table) {
+        JS_HashTableDestroy(script_filename_table);
+        script_filename_table = NULL;
+    }
+#ifdef JS_THREADSAFE
+    if (script_filename_table_lock) {
+        JS_DESTROY_LOCK(script_filename_table_lock);
+        script_filename_table_lock = NULL;
+    }
+#endif
+}
+
+#ifdef DEBUG_brendan
+size_t sft_savings = 0;
+#endif
+
+const char *
+js_SaveScriptFilename(JSContext *cx, const char *filename)
+{
+    JSHashTable *table;
+    JSHashNumber hash;
+    JSHashEntry **hep;
+    ScriptFilenameEntry *sfe;
+
+    JS_ACQUIRE_LOCK(script_filename_table_lock);
+    table = script_filename_table;
+    hash = JS_HashString(filename);
+    hep = JS_HashTableRawLookup(table, hash, filename);
+    sfe = (ScriptFilenameEntry *) *hep;
+#ifdef DEBUG_brendan
+    if (sfe)
+        sft_savings += strlen(sfe->filename);
+#endif
+    if (!sfe) {
+        /* XXX this assumes NULL puns as JS_FALSE in the mark byte... */
+        sfe = (ScriptFilenameEntry *)
+              JS_HashTableRawAdd(table, hep, hash, filename, NULL);
+        if (sfe)
+            sfe->key = strcpy(sfe->filename, filename);
+        else
+            JS_ReportOutOfMemory(cx);
+    }
+    JS_RELEASE_LOCK(script_filename_table_lock);
+    return sfe ? sfe->filename : NULL;
+}
+
+void
+js_MarkScriptFilename(const char *filename)
+{
+    ScriptFilenameEntry *sfe;
+
+    /*
+     * Back up from filename by its offset within its hash table entry.
+     * The sfe->key member, redundant given sfe->filename but required by
+     * the old jshash.c code, here gives us a useful sanity check.  This
+     * assertion will very likely botch if someone tries to mark a string
+     * that wasn't allocated as an sfe->filename.
+     */
+    sfe = (ScriptFilenameEntry *)
+          (filename - offsetof(ScriptFilenameEntry, filename));
+    JS_ASSERT(sfe->key == sfe->filename);
+    sfe->mark = JS_TRUE;
+}
+
+JS_STATIC_DLL_CALLBACK(intN)
+js_script_filename_sweeper(JSHashEntry *he, intN i, void *arg)
+{
+    ScriptFilenameEntry *sfe = (ScriptFilenameEntry *) he;
+
+    if (!sfe->mark)
+        return HT_ENUMERATE_REMOVE;
+    sfe->mark = JS_FALSE;
+    return HT_ENUMERATE_NEXT;
+}
+
+void
+js_SweepScriptFilenames(JSRuntime *rt)
+{
+    JS_HashTableEnumerateEntries(script_filename_table,
+                                 js_script_filename_sweeper,
+                                 rt);
+#ifdef DEBUG_brendan
+    printf("script filename table savings so far: %u\n", sft_savings);
+#endif
+}
+
 JSScript *
-js_NewScript(JSContext *cx, uint32 length)
+js_NewScript(JSContext *cx, uint32 length, uint32 nsrcnotes, uint32 ntrynotes)
 {
     JSScript *script;
 
-    script = (JSScript *)
-        JS_malloc(cx, sizeof(JSScript) + length * sizeof(jsbytecode));
+    /* Round up source note count to align script->trynotes for its type. */
+    if (ntrynotes)
+        nsrcnotes += JSTRYNOTE_ALIGNMASK;
+    script = (JSScript *) JS_malloc(cx,
+                                    sizeof(JSScript) +
+                                    length * sizeof(jsbytecode) +
+                                    nsrcnotes * sizeof(jssrcnote) +
+                                    ntrynotes * sizeof(JSTryNote));
     if (!script)
         return NULL;
     memset(script, 0, sizeof(JSScript));
     script->code = script->main = (jsbytecode *)(script + 1);
     script->length = length;
     script->version = cx->version;
-    return script;
-}
-
-JSScript *
-js_NewScriptFromParams(JSContext *cx, jsbytecode *code, uint32 length,
-                       jsbytecode *prolog, uint32 prologLength,
-                       const char *filename, uintN lineno, uintN depth,
-                       jssrcnote *notes, JSTryNote *trynotes,
-                       JSPrincipals *principals)
-{
-    JSScript *script;
-
-    script = js_NewScript(cx, prologLength + length);
-    if (!script)
-        return NULL;
-    script->main += prologLength;
-    memcpy(script->code, prolog, prologLength * sizeof(jsbytecode));
-    memcpy(script->main, code, length * sizeof(jsbytecode));
-    if (filename) {
-        script->filename = JS_strdup(cx, filename);
-        if (!script->filename) {
-            js_DestroyScript(cx, script);
-            return NULL;
-        }
+    if (ntrynotes) {
+        script->trynotes = (JSTryNote *)
+                           ((jsword)(SCRIPT_NOTES(script) + nsrcnotes) &
+                            ~(jsword)JSTRYNOTE_ALIGNMASK);
     }
-    script->lineno = lineno;
-    script->depth = depth;
-    script->notes = notes;
-    script->trynotes = trynotes;
-    if (principals)
-        JSPRINCIPALS_HOLD(cx, principals);
-    script->principals = principals;
     return script;
 }
 
 JS_FRIEND_API(JSScript *)
 js_NewScriptFromCG(JSContext *cx, JSCodeGenerator *cg, JSFunction *fun)
 {
-    JSTryNote *trynotes;
-    jssrcnote *notes;
+    uint32 mainLength, prologLength, nsrcnotes, ntrynotes;
     JSScript *script;
+    const char *filename;
 
-    if (!js_FinishTakingTryNotes(cx, cg, &trynotes))
-        return NULL;
-    notes = js_FinishTakingSrcNotes(cx, cg);
-    script = js_NewScriptFromParams(cx, CG_BASE(cg), CG_OFFSET(cg),
-                                    CG_PROLOG_BASE(cg), CG_PROLOG_OFFSET(cg),
-                                    cg->filename, cg->firstLine,
-                                    cg->maxStackDepth, notes, trynotes,
-                                    cg->principals);
+    mainLength = CG_OFFSET(cg);
+    prologLength = CG_PROLOG_OFFSET(cg);
+    CG_COUNT_FINAL_SRCNOTES(cg, nsrcnotes);
+    CG_COUNT_FINAL_TRYNOTES(cg, ntrynotes);
+    script = js_NewScript(cx, prologLength + mainLength, nsrcnotes, ntrynotes);
     if (!script)
         return NULL;
-    if (!notes || !js_InitAtomMap(cx, &script->atomMap, &cg->atomList)) {
-        js_DestroyScript(cx, script);
-        return NULL;
+
+    /* Now that we have script, error control flow must go to label bad. */
+    script->main += prologLength;
+    memcpy(script->code, CG_PROLOG_BASE(cg), prologLength * sizeof(jsbytecode));
+    memcpy(script->main, CG_BASE(cg), mainLength * sizeof(jsbytecode));
+    if (!js_InitAtomMap(cx, &script->atomMap, &cg->atomList))
+        goto bad;
+
+    filename = cg->filename;
+    if (filename) {
+        script->filename = js_SaveScriptFilename(cx, filename);
+        if (!script->filename)
+            goto bad;
     }
+    script->lineno = cg->firstLine;
+    script->depth = cg->maxStackDepth;
+    if (cg->principals) {
+        script->principals = cg->principals;
+        JSPRINCIPALS_HOLD(cx, script->principals);
+    }
+
+    if (!js_FinishTakingSrcNotes(cx, cg, SCRIPT_NOTES(script)))
+        goto bad;
+    if (script->trynotes)
+        js_FinishTakingTryNotes(cx, cg, script->trynotes);
 
     /* Tell the debugger about this compiled script. */
     js_CallNewScriptHook(cx, script, fun);
     return script;
+
+bad:
+    js_DestroyScript(cx, script);
+    return NULL;
 }
 
 JS_FRIEND_API(void)
@@ -840,7 +1085,7 @@ js_CallNewScriptHook(JSContext *cx, JSScript *script, JSFunction *fun)
          * XXX and others within the engine.  See bug 40757 for discussion.
          */
         JSStackFrame dummy;
-        
+
         memset(&dummy, 0, sizeof dummy);
         dummy.down = cx->fp;
         dummy.script = script;
@@ -862,13 +1107,10 @@ js_DestroyScript(JSContext *cx, JSScript *script)
     rt = cx->runtime;
     hook = rt->destroyScriptHook;
     if (hook)
-        (*hook)(cx, script, rt->destroyScriptHookData);
+        hook(cx, script, rt->destroyScriptHookData);
 
     JS_ClearScriptTraps(cx, script);
     js_FreeAtomMap(cx, &script->atomMap);
-    JS_free(cx, (void *)script->filename);
-    JS_free(cx, script->notes);
-    JS_free(cx, script->trynotes);
     if (script->principals)
         JSPRINCIPALS_DROP(cx, script->principals);
     JS_free(cx, script);
@@ -886,6 +1128,9 @@ js_MarkScript(JSContext *cx, JSScript *script, void *arg)
     vector = map->vector;
     for (i = 0; i < length; i++)
         GC_MARK_ATOM(cx, vector[i], arg);
+
+    if (script->filename)
+        js_MarkScriptFilename(script->filename);
 }
 
 jssrcnote *
@@ -894,13 +1139,11 @@ js_GetSrcNote(JSScript *script, jsbytecode *pc)
     jssrcnote *sn;
     ptrdiff_t offset, target;
 
-    sn = script->notes;
-    if (!sn)
+    target = PTRDIFF(pc, script->code, jsbytecode);
+    if ((uint32)target >= script->length)
         return NULL;
-    target = PTRDIFF(pc, script->main, jsbytecode);
-    if ((uintN)target >= script->length)
-        return NULL;
-    for (offset = 0; !SN_IS_TERMINATOR(sn); sn = SN_NEXT(sn)) {
+    offset = 0;
+    for (sn = SCRIPT_NOTES(script); !SN_IS_TERMINATOR(sn); sn = SN_NEXT(sn)) {
         offset += SN_DELTA(sn);
         if (offset == target && SN_IS_GETTABLE(sn))
             return sn;
@@ -909,19 +1152,34 @@ js_GetSrcNote(JSScript *script, jsbytecode *pc)
 }
 
 uintN
-js_PCToLineNumber(JSScript *script, jsbytecode *pc)
+js_PCToLineNumber(JSContext *cx, JSScript *script, jsbytecode *pc)
 {
-    jssrcnote *sn;
-    ptrdiff_t offset, target;
+    JSAtom *atom;
+    JSFunction *fun;
     uintN lineno;
+    ptrdiff_t offset, target;
+    jssrcnote *sn;
     JSSrcNoteType type;
 
-    sn = script->notes;
-    if (!sn)
-        return 0;
-    target = PTRDIFF(pc, script->main, jsbytecode);
+    /*
+     * Special case: function definition needs no line number note because
+     * the function's script contains its starting line number.
+     */
+    if (*pc == JSOP_DEFFUN) {
+        atom = GET_ATOM(cx, script, pc);
+        fun = (JSFunction *) JS_GetPrivate(cx, ATOM_TO_OBJECT(atom));
+        return fun->script->lineno;
+    }
+
+    /*
+     * General case: walk through source notes accumulating their deltas,
+     * keeping track of line-number notes, until we pass the note for pc's
+     * offset within script->code.
+     */
     lineno = script->lineno;
-    for (offset = 0; !SN_IS_TERMINATOR(sn); sn = SN_NEXT(sn)) {
+    offset = 0;
+    target = PTRDIFF(pc, script->code, jsbytecode);
+    for (sn = SCRIPT_NOTES(script); !SN_IS_TERMINATOR(sn); sn = SN_NEXT(sn)) {
         offset += SN_DELTA(sn);
         type = (JSSrcNoteType) SN_TYPE(sn);
         if (type == SRC_SETLINE) {
@@ -940,16 +1198,14 @@ js_PCToLineNumber(JSScript *script, jsbytecode *pc)
 jsbytecode *
 js_LineNumberToPC(JSScript *script, uintN target)
 {
-    jssrcnote *sn;
-    uintN lineno;
     ptrdiff_t offset;
+    uintN lineno;
+    jssrcnote *sn;
     JSSrcNoteType type;
 
-    sn = script->notes;
-    if (!sn)
-        return NULL;
+    offset = 0;
     lineno = script->lineno;
-    for (offset = 0; !SN_IS_TERMINATOR(sn); sn = SN_NEXT(sn)) {
+    for (sn = SCRIPT_NOTES(script); !SN_IS_TERMINATOR(sn); sn = SN_NEXT(sn)) {
         if (lineno >= target)
             break;
         offset += SN_DELTA(sn);
@@ -960,21 +1216,18 @@ js_LineNumberToPC(JSScript *script, uintN target)
             lineno++;
         }
     }
-    return script->main + offset;
+    return script->code + offset;
 }
 
 uintN
 js_GetScriptLineExtent(JSScript *script)
 {
-    jssrcnote *sn;
     uintN lineno;
+    jssrcnote *sn;
     JSSrcNoteType type;
 
-    sn = script->notes;
-    if (!sn)
-        return 0;
     lineno = script->lineno;
-    for (; !SN_IS_TERMINATOR(sn); sn = SN_NEXT(sn)) {
+    for (sn = SCRIPT_NOTES(script); !SN_IS_TERMINATOR(sn); sn = SN_NEXT(sn)) {
         type = (JSSrcNoteType) SN_TYPE(sn);
         if (type == SRC_SETLINE) {
             lineno = (uintN) js_GetSrcNoteOffset(sn, 0);
