@@ -82,20 +82,20 @@ static const nsModuleComponentInfo components[] = {
   {nsnull, NS_JS_RUNTIME_SERVICE_CID,            XPC_RUNTIME_CONTRACTID,       nsIJSRuntimeServiceConstructor},
   {NS_SCRIPTERROR_CLASSNAME, NS_SCRIPTERROR_CID, NS_SCRIPTERROR_CONTRACTID,    nsScriptErrorConstructor      },
   {nsnull, SCRIPTABLE_INTERFACES_CID,            NS_SCRIPTABLE_INTERFACES_CONTRACTID,        nsXPCComponents_InterfacesConstructor },
-  {nsnull, XPCVARIANT_CID,                       XPCVARIANT_CONTRACTID,        nsnull, nsnull, nsnull, nsnull, NS_CI_INTERFACE_GETTER_NAME(XPCVariant), nsnull, &NS_CLASSINFO_NAME(XPCVariant)},
+  {nsnull, XPCVARIANT_CID,                       XPCVARIANT_CONTRACTID,        nsnull, nsnull, nsnull, nsnull, NS_CI_INTERFACE_GETTER_NAME(XPCVariant), nsnull, &NS_CLASSINFO_NAME(XPCVariant)}
 
 #ifdef MOZ_JSLOADER
   // jsloader stuff
-  { "JS component loader", MOZJSCOMPONENTLOADER_CID,
+ ,{ "JS component loader", MOZJSCOMPONENTLOADER_CID,
     mozJSComponentLoaderContractID, mozJSComponentLoaderConstructor,
     RegisterJSLoader, UnregisterJSLoader }
 #ifndef NO_SUBSCRIPT_LOADER
-  ,{ "JS subscript loader", MOZ_JSSUBSCRIPTLOADER_CID,
+ ,{ "JS subscript loader", MOZ_JSSUBSCRIPTLOADER_CID,
     mozJSSubScriptLoadContractID, mozJSSubScriptLoaderConstructor }
 #endif
 #endif
 #ifdef XPC_IDISPATCH_SUPPORT
-  ,{ nsnull, NS_IDISPATCH_SUPPORT_CID,            NS_IDISPATCH_SUPPORT_CONTRACTID,
+ ,{ nsnull, NS_IDISPATCH_SUPPORT_CID,            NS_IDISPATCH_SUPPORT_CONTRACTID,
     nsIDispatchSupportConstructor }
 #endif
 };
