@@ -44,6 +44,7 @@
 #include "nsIRequest.h"
 #include "nsCompatibility.h"
 #include "nsCOMPtr.h"
+#include "nsIPresShell.h"
 #ifdef IBMBIDI
 class nsBidiPresUtils;
 #endif // IBMBIDI
@@ -58,6 +59,7 @@ class nsIDocument;
 class nsIDeviceContext;
 class nsIFontMetrics;
 class nsIFrame;
+class nsIFrameManager;
 class nsIImage;
 class nsILinkHandler;
 class nsIPresShell;
@@ -83,12 +85,6 @@ class nsIRenderingContext;
 { 0x0a5d12e0, 0x944e, 0x11d1, \
   {0x93, 0x23, 0x00, 0x80, 0x5f, 0x8a, 0xdd, 0x32} }
 
-enum nsWidgetRendering {
-  eWidgetRendering_Native   = 1,
-  eWidgetRendering_Gfx      = 2,
-  eWidgetRendering_PartialGfx = 3
-};
-
 enum nsWidgetType {
   eWidgetType_Button  	= 1,
   eWidgetType_Checkbox	= 2,
@@ -137,6 +133,13 @@ public:
    * Get the PresentationShell that this context is bound to.
    */
   NS_IMETHOD GetShell(nsIPresShell** aResult) = 0;
+  nsIPresShell* GetPresShell() { return mShell; }
+
+  nsIDocument* GetDocument() { return GetPresShell()->GetDocument(); } 
+  nsIViewManager* GetViewManager() { return GetPresShell()->GetViewManager(); } 
+  nsIStyleSet* GetStyleSet() { return GetPresShell()->GetStyleSet(); } 
+  nsIFrameManager* GetFrameManager()
+    { return GetPresShell()->GetFrameManager(); } 
 
   /**
    * Access compatibility mode for this context
@@ -147,12 +150,6 @@ public:
   NS_IMETHOD GetCompatibilityMode(nsCompatibility* aModeResult) = 0;
   NS_IMETHOD SetCompatibilityMode(nsCompatibility aMode) = 0;
 
-  /**
-   * Access the widget rendering mode for this context
-   */
-  NS_IMETHOD GetWidgetRenderingMode(nsWidgetRendering* aModeResult) = 0;
-  NS_IMETHOD SetWidgetRenderingMode(nsWidgetRendering aMode) = 0;
-
   /**
    * Access the image animation mode for this context
    */
@@ -250,10 +247,9 @@ public:
  
    /**
     * Resolve a new style context for a content node and return the URL
-    * for its XBL binding, or the empty string if it has no binding
-    * specified in CSS.
+    * for its XBL binding, or null if it has no binding specified in CSS.
     */
-   NS_IMETHOD GetXBLBindingURL(nsIContent* aContent, nsAString& aResult) = 0;
+   NS_IMETHOD GetXBLBindingURL(nsIContent* aContent, nsIURI** aResult) = 0;
 
   /** 
    * For a given frame tree, get a new style context that is the equivalent
@@ -302,6 +298,7 @@ public:
   NS_IMETHOD GetDefaultColor(nscolor* aColor) = 0;
   NS_IMETHOD GetDefaultBackgroundColor(nscolor* aColor) = 0;
   NS_IMETHOD GetDefaultLinkColor(nscolor* aColor) = 0;
+  NS_IMETHOD GetDefaultActiveLinkColor(nscolor* aColor) = 0;
   NS_IMETHOD GetDefaultVisitedLinkColor(nscolor* aColor) = 0;
   NS_IMETHOD GetFocusBackgroundColor(nscolor* aColor) = 0;
   NS_IMETHOD GetFocusTextColor(nscolor* aColor) = 0; 
@@ -313,6 +310,7 @@ public:
   NS_IMETHOD SetDefaultColor(nscolor aColor) = 0;
   NS_IMETHOD SetDefaultBackgroundColor(nscolor aColor) = 0;
   NS_IMETHOD SetDefaultLinkColor(nscolor aColor) = 0;
+  NS_IMETHOD SetDefaultActiveLinkColor(nscolor aColor) = 0;
   NS_IMETHOD SetDefaultVisitedLinkColor(nscolor aColor) = 0;
 
   /**
@@ -322,7 +320,7 @@ public:
    * method will be invoked (via the ViewManager) so that the
    * appropriate damage repair is done.
    */
-  NS_IMETHOD LoadImage(const nsString& aURL,
+  NS_IMETHOD LoadImage(nsIURI* aURL,
                        nsIFrame* aTargetFrame,
                        imgIRequest **aRequest) = 0;
 
@@ -405,10 +403,12 @@ public:
    */
   NS_IMETHOD GetScaledPixelsToTwips(float* aScale) const = 0;
 
-  //be sure to Relase() after you are done with the Get()
   NS_IMETHOD GetDeviceContext(nsIDeviceContext** aResult) const = 0;
+  nsIDeviceContext* GetDeviceContext() { return mDeviceContext; }
 
   NS_IMETHOD GetEventStateManager(nsIEventStateManager** aManager) = 0;
+  nsIEventStateManager* GetEventStateManager();
+
   NS_IMETHOD GetLanguage(nsILanguageAtom** aLanguage) = 0;
 
   /**
@@ -515,7 +515,7 @@ public:
   /**
    * Get the document charset
    */
-  NS_IMETHOD GetBidiCharset(nsAString &aCharSet) const = 0;
+  NS_IMETHOD GetBidiCharset(nsACString &aCharSet) const = 0;
 
 
 #endif // IBMBIDI
@@ -558,6 +558,20 @@ public:
   NS_IMETHOD CountReflows(const char * aName, PRUint32 aType, nsIFrame * aFrame) = 0;
   NS_IMETHOD PaintCount(const char * aName, nsIRenderingContext* aRendingContext, nsIFrame * aFrame, PRUint32 aColor) = 0;
 #endif
+
+protected:
+  // IMPORTANT: The ownership implicit in the following member variables
+  // has been explicitly checked.  If you add any members to this class,
+  // please make the ownership explicit (pinkerton, scc).
+  
+  nsIPresShell*         mShell;         // [WEAK]
+  nsIDeviceContext*     mDeviceContext; // [STRONG] could be weak, but
+                                        // better safe than sorry.
+                                        // Cannot reintroduce cycles
+                                        // since there is no dependency
+                                        // from gfx back to layout.
+  nsIEventStateManager* mEventManager;  // [STRONG]
+
 };
 
 // Bit values for StartLoadImage's aImageStatus
