@@ -63,7 +63,6 @@ class nsStyleContext;
 class nsIFrameSelection;
 class nsIFrameManager;
 class nsILayoutHistoryState;
-class nsIArena;
 class nsIReflowCallback;
 class nsISupportsArray;
 class nsIDOMNode;
@@ -150,12 +149,19 @@ public:
   NS_IMETHOD AllocateStackMemory(size_t aSize, void** aResult) = 0;
   
   NS_IMETHOD GetDocument(nsIDocument** aResult) = 0;
+  nsIDocument* GetDocument() { return mDocument; }
 
   NS_IMETHOD GetPresContext(nsIPresContext** aResult) = 0;
+  nsIPresContext* GetPresContext() { return mPresContext; }
 
   NS_IMETHOD GetViewManager(nsIViewManager** aResult) = 0;
+  nsIViewManager* GetViewManager() { return mViewManager; }
 
   NS_IMETHOD GetStyleSet(nsIStyleSet** aResult) = 0;
+  nsIStyleSet* GetStyleSet() { return mStyleSet; }
+
+  NS_IMETHOD GetFrameManager(nsIFrameManager** aFrameManager) const = 0;
+  nsIFrameManager* GetFrameManager() { return mFrameManager; }
 
   NS_IMETHOD GetActiveAlternateStyleSheet(nsString& aSheetTitle) = 0;
 
@@ -166,7 +172,7 @@ public:
    * all style data for a given pres shell without necessarily reconstructing
    * all of the frames.
    */
-  NS_IMETHOD ReconstructStyleData(PRBool aRebuildRuleTree) = 0;
+  NS_IMETHOD ReconstructStyleData() = 0;
 
   /** Setup all style rules required to implement preferences
    * - used for background/text/link colors and link underlining
@@ -340,8 +346,7 @@ public:
    * The content object associated with aFrame should either be a IMG
    * element, an OBJECT element, or an APPLET element
    */
-  NS_IMETHOD CantRenderReplacedElement(nsIPresContext* aPresContext,
-                                       nsIFrame*       aFrame) = 0;
+  NS_IMETHOD CantRenderReplacedElement(nsIFrame* aFrame) = 0;
 
 
   /**
@@ -399,11 +404,6 @@ public:
    */
   NS_IMETHOD NotifyDestroyingFrame(nsIFrame* aFrame) = 0;
 
-  /**
-   * Returns the frame manager object
-   */
-  NS_IMETHOD GetFrameManager(nsIFrameManager** aFrameManager) const = 0;
-
   /**
    * Notify the Clipboard that we have something to copy.
    */
@@ -609,6 +609,18 @@ public:
   NS_IMETHOD BidiStyleChangeReflow(void) = 0;
 #endif
 
+protected:
+  // IMPORTANT: The ownership implicit in the following member variables
+  // has been explicitly checked.  If you add any members to this class,
+  // please make the ownership explicit (pinkerton, scc).
+
+  // these are the same Document and PresContext owned by the DocViewer.
+  // we must share ownership.
+  nsIDocument*              mDocument;      // [STRONG]
+  nsIPresContext*           mPresContext;   // [STRONG]
+  nsIStyleSet*              mStyleSet;      // [STRONG]
+  nsIViewManager*           mViewManager;   // [WEAK] docViewer owns it so I don't have to
+  nsIFrameManager*          mFrameManager;  // [STRONG]
 };
 
 /**
