@@ -79,8 +79,6 @@
 //------------------------------END OF IBM BIDI--------------------------------
 #endif //IBMBIDI
 
-static NS_DEFINE_CID(kLookAndFeelCID,  NS_LOOKANDFEEL_CID);
-
 //-----------------------------------------------------------------------------
 
 nsCaret::nsCaret()
@@ -113,7 +111,7 @@ NS_IMETHODIMP nsCaret::Init(nsIPresShell *inPresShell)
 {
   NS_ENSURE_ARG(inPresShell);
   
-  mPresShell = getter_AddRefs(NS_GetWeakReference(inPresShell));    // the presshell owns us, so no addref
+  mPresShell = do_GetWeakReference(inPresShell);    // the presshell owns us, so no addref
   NS_ASSERTION(mPresShell, "Hey, pres shell should support weak refs");
 
   // get nsILookAndFeel from the pres context, which has one cached.
@@ -151,7 +149,7 @@ NS_IMETHODIMP nsCaret::Init(nsIPresShell *inPresShell)
   nsCOMPtr<nsISelectionPrivate> privateSelection = do_QueryInterface(domSelection);
   if (privateSelection)
     privateSelection->AddSelectionListener(this);
-  mDomSelectionWeak = getter_AddRefs(NS_GetWeakReference(domSelection));
+  mDomSelectionWeak = do_GetWeakReference(domSelection);
   
   // set up the blink timer
   if (mVisible)
@@ -220,7 +218,7 @@ NS_IMETHODIMP nsCaret::GetCaretDOMSelection(nsISelection **aDOMSel)
 NS_IMETHODIMP nsCaret::SetCaretDOMSelection(nsISelection *aDOMSel)
 {
   NS_ENSURE_ARG_POINTER(aDOMSel);
-  mDomSelectionWeak = getter_AddRefs( NS_GetWeakReference(aDOMSel) );   // weak reference to pres shell
+  mDomSelectionWeak = do_GetWeakReference(aDOMSel);   // weak reference to pres shell
   return NS_OK;
 }
 
@@ -366,9 +364,6 @@ NS_IMETHODIMP nsCaret::GetCaretCoordinates(EViewCoordinates aRelativeToType, nsI
   nsPoint   framePos(0, 0);
   theFrame->GetPointFromOffset(presContext, rendContext, theFrameOffset, &framePos);
 
-  nsRect          frameRect;
-  theFrame->GetRect(frameRect);
-  
   // we don't need drawingView anymore so reuse that; reset viewOffset values for our purposes
   if (aRelativeToType == eClosestViewCoordinates)
   {
@@ -380,7 +375,7 @@ NS_IMETHODIMP nsCaret::GetCaretCoordinates(EViewCoordinates aRelativeToType, nsI
   viewOffset += framePos;
   outCoordinates->x = viewOffset.x;
   outCoordinates->y = viewOffset.y;
-  outCoordinates->height = frameRect.height;
+  outCoordinates->height = theFrame->GetSize().height;
   outCoordinates->width  = mCaretTwipsWidth;
   
   return NS_OK;
@@ -496,19 +491,18 @@ void nsCaret::KillTimer()
 //-----------------------------------------------------------------------------
 nsresult nsCaret::PrimeTimer()
 {
-  KillTimer();
-  
   // set up the blink timer
   if (!mReadOnly && mBlinkRate > 0)
   {
-    nsresult  err;
-    mBlinkTimer = do_CreateInstance("@mozilla.org/timer;1", &err);
-    
-    if (NS_FAILED(err))
-      return err;
-    
+    if (!mBlinkTimer) {
+      nsresult  err;
+      mBlinkTimer = do_CreateInstance("@mozilla.org/timer;1", &err);    
+      if (NS_FAILED(err))
+        return err;
+    }    
+
     mBlinkTimer->InitWithFuncCallback(CaretBlinkCallback, this, mBlinkRate,
-                                      nsITimer::TYPE_REPEATING_PRECISE);
+                                      nsITimer::TYPE_REPEATING_SLACK);
   }
 
   return NS_OK;
@@ -755,10 +749,7 @@ PRBool nsCaret::SetupDrawingFrameAndOffset(nsIDOMNode* aNode, PRInt32 aOffset, n
   // mark the frame, so we get notified on deletion.
   // frames are never unmarked, which means that we'll touch every frame we visit.
   // this is not ideal.
-  nsFrameState frameState;
-  theFrame->GetFrameState(&frameState);
-  frameState |= NS_FRAME_EXTERNAL_REFERENCE;
-  theFrame->SetFrameState(frameState);
+  theFrame->AddStateBits(NS_FRAME_EXTERNAL_REFERENCE);
 
   mLastCaretFrame = theFrame;
   mLastContentOffset = theFrameOffset;
@@ -807,8 +798,6 @@ void nsCaret::GetViewForRendering(nsIFrame *caretFrame, EViewCoordinates coordTy
 
   nsIView*    returnView = nsnull;    // views are not refcounted
   
-  nscoord   x, y;
-  
   // coorinates relative to the view we are going to use for drawing
   if (coordType == eRenderingViewCoordinates)
   {
@@ -818,23 +807,17 @@ void nsCaret::GetViewForRendering(nsIFrame *caretFrame, EViewCoordinates coordTy
     
     // walk up to the first view with a widget
     do {
-      theView->GetPosition(&x, &y);
-
       //is this a scrollable view?
       if (!scrollableView)
         theView->QueryInterface(NS_GET_IID(nsIScrollableView), (void **)&scrollableView);
 
-      PRBool hasWidget;
-      theView->HasWidget(&hasWidget);
-      if (hasWidget)
+      if (theView->HasWidget())
       {
         returnView = theView;
         break;
       }
-      drawViewOffset.x += x;
-      drawViewOffset.y += y;
-      
-      theView->GetParent(theView);
+      drawViewOffset += theView->GetPosition();
+      theView = theView->GetParent();
     } while (theView);
     
     viewOffset = withinViewOffset;
@@ -846,8 +829,7 @@ void nsCaret::GetViewForRendering(nsIFrame *caretFrame, EViewCoordinates coordTy
       scrollableView->GetClipView(&clipView);
       if (!clipView) return;      // should always have one
       
-      nsRect  bounds;
-      clipView->GetBounds(bounds);
+      nsRect  bounds = clipView->GetBounds();
       scrollableView->GetScrollPosition(bounds.x, bounds.y);
       
       bounds += drawViewOffset;   // offset to coords of returned view
@@ -855,7 +837,7 @@ void nsCaret::GetViewForRendering(nsIFrame *caretFrame, EViewCoordinates coordTy
     }
     else
     {
-      returnView->GetBounds(outClipRect);
+      outClipRect = returnView->GetBounds();
     }
 
     if (outRelativeView)
@@ -868,29 +850,18 @@ void nsCaret::GetViewForRendering(nsIFrame *caretFrame, EViewCoordinates coordTy
     viewOffset = withinViewOffset;
 
     do {
-      theView->GetPosition(&x, &y);
-
-      if (!returnView)
-      {
-        PRBool hasWidget;
-        theView->HasWidget(&hasWidget);
-        
-        if (hasWidget)
-          returnView = theView;
-      }
+      if (!returnView && theView->HasWidget())
+        returnView = theView;
       // is this right?
-      viewOffset.x += x;
-      viewOffset.y += y;
+      viewOffset += theView->GetPosition();
       
       if (outRelativeView && coordType == eTopLevelWindowCoordinates)
         *outRelativeView = theView;
 
-      theView->GetParent(theView);
+      theView = theView->GetParent();
     } while (theView);
-  
   }
   
-  
   *outRenderingView = returnView;
 }
 
@@ -982,8 +953,7 @@ void nsCaret::GetCaretRectAndInvert()
 {
   NS_ASSERTION(mLastCaretFrame != nsnull, "Should have a frame here");
   
-  nsRect    frameRect;
-  mLastCaretFrame->GetRect(frameRect);
+  nsRect    frameRect = mLastCaretFrame->GetRect();
   
   frameRect.x = 0;      // the origin is accounted for in GetViewForRendering()
   frameRect.y = 0;
@@ -993,7 +963,7 @@ void nsCaret::GetCaretRectAndInvert()
   nsIView   *drawingView;
   GetViewForRendering(mLastCaretFrame, eRenderingViewCoordinates, viewOffset, clipRect, &drawingView, nsnull);
   
-  if (drawingView == nsnull)
+  if (!drawingView)
     return;
   
   frameRect += viewOffset;
@@ -1058,7 +1028,6 @@ void nsCaret::GetCaretRectAndInvert()
   {
     nsPoint   framePos(0, 0);
     nsRect    caretRect = frameRect;
-    PRBool    aCanCacheFrameOffset = PR_FALSE;
     nsCOMPtr<nsISelection> domSelection = do_QueryReferent(mDomSelectionWeak);
     nsCOMPtr<nsISelectionPrivate> privateSelection = do_QueryInterface(domSelection);
 
