@@ -68,7 +68,6 @@
 #include "nsHTMLAtoms.h"
 #ifdef IBMBIDI
 #include "nsBidiUtils.h"
-//static NS_DEFINE_CID(kUBidiUtilCID, NS_UNICHARBIDIUTIL_CID);
 #endif
 
 // image copy stuff
@@ -145,8 +144,7 @@ nsresult nsCopySupport::HTMLCopy(nsISelection *aSel, nsIDocument *aDoc, PRInt16
   
   nsCOMPtr<nsIDocument> doc = do_QueryInterface(aDoc);
   if (doc) {
-    nsCOMPtr<nsIPresShell> shell;
-    doc->GetShellAt(0, getter_AddRefs(shell));
+    nsIPresShell *shell = doc->GetShellAt(0);
     if (shell) {
       nsCOMPtr<nsIPresContext> context;
       shell->GetPresContext(getter_AddRefs(context) );
@@ -291,10 +289,8 @@ nsresult nsCopySupport::DoHooks(nsIDocument *aDoc, nsITransferable *aTrans,
 
   *aDoPutOnClipboard = PR_TRUE;
 
-  nsCOMPtr<nsISupports> isupp;
-  nsresult rv = aDoc->GetContainer(getter_AddRefs(isupp));
-  nsCOMPtr<nsIDocShell> docShell = do_QueryInterface(isupp);
-  nsCOMPtr<nsIClipboardDragDropHookList> hookObj = do_GetInterface(docShell);
+  nsCOMPtr<nsISupports> container = aDoc->GetContainer();
+  nsCOMPtr<nsIClipboardDragDropHookList> hookObj = do_GetInterface(container);
   if (!hookObj) return NS_ERROR_FAILURE;
 
   nsCOMPtr<nsISimpleEnumerator> enumerator;
@@ -305,7 +301,9 @@ nsresult nsCopySupport::DoHooks(nsIDocument *aDoc, nsITransferable *aTrans,
   // nsIClipboardDragDropHooks.h
 
   nsCOMPtr<nsIClipboardDragDropHooks> override;
+  nsCOMPtr<nsISupports> isupp;
   PRBool hasMoreHooks = PR_FALSE;
+  nsresult rv = NS_OK;
   while (NS_SUCCEEDED(enumerator->HasMoreElements(&hasMoreHooks))
          && hasMoreHooks)
   {
@@ -314,7 +312,10 @@ nsresult nsCopySupport::DoHooks(nsIDocument *aDoc, nsITransferable *aTrans,
     override = do_QueryInterface(isupp);
     if (override)
     {
-      nsresult hookResult = override->OnCopyOrDrag(nsnull, aTrans, aDoPutOnClipboard);
+#ifdef DEBUG
+      nsresult hookResult =
+#endif
+      override->OnCopyOrDrag(nsnull, aTrans, aDoPutOnClipboard);
       NS_ASSERTION(NS_SUCCEEDED(hookResult), "OnCopyOrDrag hook failed");
       if (!*aDoPutOnClipboard)
         break;
@@ -353,21 +354,22 @@ nsresult nsCopySupport::IsPlainTextContext(nsISelection *aSel, nsIDocument *aDoc
     return NS_ERROR_NULL_POINTER;
   range->GetCommonAncestorContainer(getter_AddRefs(commonParent));
 
-  nsCOMPtr<nsIContent> tmp, selContent( do_QueryInterface(commonParent) );
-  while (selContent)
+  for (nsCOMPtr<nsIContent> selContent(do_QueryInterface(commonParent));
+       selContent;
+       selContent = selContent->GetParent())
   {
     // checking for selection inside a plaintext form widget
     nsCOMPtr<nsIAtom> atom;
     selContent->GetTag(getter_AddRefs(atom));
 
-    if (atom.get() == nsHTMLAtoms::input ||
-        atom.get() == nsHTMLAtoms::textarea)
+    if (atom == nsHTMLAtoms::input ||
+        atom == nsHTMLAtoms::textarea)
     {
       *aIsPlainTextContext = PR_TRUE;
       break;
     }
 
-    if (atom.get() == nsHTMLAtoms::body)
+    if (atom == nsHTMLAtoms::body)
     {
       // check for moz prewrap style on body.  If it's there we are 
       // in a plaintext editor.  This is pretty cheezy but I haven't 
@@ -381,8 +383,6 @@ nsresult nsCopySupport::IsPlainTextContext(nsISelection *aSel, nsIDocument *aDoc
         break;
       }
     }
-    selContent->GetParent(getter_AddRefs(tmp));
-    selContent = tmp;
   }
   
   // also consider ourselves in a text widget if we can't find an html document
