@@ -109,7 +109,7 @@ nsPresContext::PrefChangedCallback(const char* aPrefName, void* instance_data)
 
 #ifdef IBMBIDI
 PRBool
-IsVisualCharset(const nsAutoString& aCharset)
+IsVisualCharset(const nsCAutoString& aCharset)
 {
   if (aCharset.EqualsIgnoreCase("ibm864")             // Arabic//ahmed
       || aCharset.EqualsIgnoreCase("ibm862")          // Hebrew
@@ -184,8 +184,9 @@ nsPresContext::nsPresContext()
   // the minimum font-size is unconstrained by default
   mMinimumFontSize = 0;
 
-  mLinkColor = NS_RGB(0x33, 0x33, 0xFF);
-  mVisitedLinkColor = NS_RGB(0x66, 0x00, 0xCC);
+  mLinkColor = NS_RGB(0x00, 0x00, 0xEE);
+  mActiveLinkColor = NS_RGB(0xEE, 0x00, 0x00);
+  mVisitedLinkColor = NS_RGB(0x55, 0x1A, 0x8B);
   mUnderlineLinks = PR_TRUE;
 
   mUseFocusColors = PR_FALSE;
@@ -211,8 +212,10 @@ nsPresContext::~nsPresContext()
   NS_PRECONDITION(!mShell, "Presshell forgot to clear our mShell pointer");
   SetShell(nsnull);
 
-  if (mEventManager)
+  if (mEventManager) {
     mEventManager->SetPresContext(nsnull);   // unclear if this is needed, but can't hurt
+    NS_RELEASE(mEventManager);
+  }
 
   // Unregister preference callbacks
   if (mPrefs) {
@@ -220,6 +223,7 @@ nsPresContext::~nsPresContext()
     mPrefs->UnregisterCallback("browser.display.", nsPresContext::PrefChangedCallback, (void*)this);
     mPrefs->UnregisterCallback("browser.underline_anchors", nsPresContext::PrefChangedCallback, (void*)this);
     mPrefs->UnregisterCallback("browser.anchor_color", nsPresContext::PrefChangedCallback, (void*)this);
+    mPrefs->UnregisterCallback("browser.active_color", nsPresContext::PrefChangedCallback, (void*)this);
     mPrefs->UnregisterCallback("browser.visited_color", nsPresContext::PrefChangedCallback, (void*)this);
     mPrefs->UnregisterCallback("network.image.imageBehavior", nsPresContext::PrefChangedCallback, (void*)this);
     mPrefs->UnregisterCallback("image.animation_mode", nsPresContext::PrefChangedCallback, (void*)this);
@@ -232,6 +236,8 @@ nsPresContext::~nsPresContext()
     delete mBidiUtils;
   }
 #endif // IBMBIDI
+
+  NS_IF_RELEASE(mDeviceContext);
 }
 
 NS_IMPL_ISUPPORTS2(nsPresContext, nsIPresContext, nsIObserver)
@@ -305,7 +311,10 @@ nsPresContext::GetFontPreferences()
 
   // get font.minimum-size.[langGroup]
   PRInt32 size;
-  pref.Assign("font.minimum-size."); pref.Append(NS_ConvertUCS2toUTF8(langGroup));
+
+  pref.Assign("font.minimum-size.");
+  AppendUTF16toUTF8(langGroup, pref);
+
   rv = mPrefs->GetIntPref(pref.get(), &size);
   if (NS_SUCCEEDED(rv)) {
     if (unit == eUnit_px) {
@@ -320,7 +329,7 @@ nsPresContext::GetFontPreferences()
   nsCAutoString generic_dot_langGroup;
   for (PRInt32 eType = eDefaultFont_Variable; eType < eDefaultFont_COUNT; ++eType) {
     generic_dot_langGroup.Assign(kGenericFont[eType]);
-    generic_dot_langGroup.Append(NS_ConvertUCS2toUTF8(langGroup));
+    AppendUTF16toUTF8(langGroup, generic_dot_langGroup);
 
     nsFont* font;
     switch (eType) {
@@ -391,9 +400,10 @@ nsPresContext::GetFontPreferences()
     }
 
 #ifdef DEBUG_rbs
-    nsCAutoString family(NS_ConvertUCS2toUTF8(font->name));
     printf("%s Family-list:%s size:%d sizeAdjust:%.2f\n",
-            generic_dot_langGroup.get(), family.get(), font->size, font->sizeAdjust);
+            generic_dot_langGroup.get(),
+           NS_ConvertUCS2toUTF8(font->name).get(), font->size,
+           font->sizeAdjust);
 #endif
   }
 }
@@ -462,6 +472,9 @@ nsPresContext::GetUserPreferences()
   if (NS_SUCCEEDED(mPrefs->CopyCharPref("browser.anchor_color", getter_Copies(colorStr)))) {
     mLinkColor = MakeColorPref(colorStr);
   }
+  if (NS_SUCCEEDED(mPrefs->CopyCharPref("browser.active_color", getter_Copies(colorStr)))) {
+    mActiveLinkColor = MakeColorPref(colorStr);
+  }
   if (NS_SUCCEEDED(mPrefs->CopyCharPref("browser.visited_color", getter_Copies(colorStr)))) {
     mVisitedLinkColor = MakeColorPref(colorStr);
   }
@@ -576,7 +589,7 @@ nsPresContext::ClearStyleDataAndReflow()
     // Clear out all our style data.
     nsCOMPtr<nsIStyleSet> set;
     mShell->GetStyleSet(getter_AddRefs(set));
-    set->ClearStyleData(this, nsnull);
+    set->ClearStyleData(this);
 
     // Force a reflow of the root frame
     // XXX We really should only do a reflow if a preference that affects
@@ -618,7 +631,8 @@ nsPresContext::Init(nsIDeviceContext* aDeviceContext)
 {
   NS_ASSERTION(!(mInitialized == PR_TRUE), "attempt to reinit pres context");
 
-  mDeviceContext = dont_QueryInterface(aDeviceContext);
+  mDeviceContext = aDeviceContext;
+  NS_IF_ADDREF(mDeviceContext);
 
   mLangService = do_GetService(NS_LANGUAGEATOMSERVICE_CONTRACTID);
   mPrefs = do_GetService(NS_PREF_CONTRACTID);
@@ -628,6 +642,7 @@ nsPresContext::Init(nsIDeviceContext* aDeviceContext)
     mPrefs->RegisterCallback("browser.display.", nsPresContext::PrefChangedCallback, (void*)this);
     mPrefs->RegisterCallback("browser.underline_anchors", nsPresContext::PrefChangedCallback, (void*)this);
     mPrefs->RegisterCallback("browser.anchor_color", nsPresContext::PrefChangedCallback, (void*)this);
+    mPrefs->RegisterCallback("browser.active_color", nsPresContext::PrefChangedCallback, (void*)this);
     mPrefs->RegisterCallback("browser.visited_color", nsPresContext::PrefChangedCallback, (void*)this);
     mPrefs->RegisterCallback("network.image.imageBehavior", nsPresContext::PrefChangedCallback, (void*)this);
     mPrefs->RegisterCallback("image.animation_mode", nsPresContext::PrefChangedCallback, (void*)this);
@@ -668,7 +683,7 @@ nsPresContext::SetShell(nsIPresShell* aShell)
     if (NS_SUCCEEDED(mShell->GetDocument(getter_AddRefs(doc)))) {
       NS_ASSERTION(doc, "expect document here");
       if (doc) {
-        doc->GetBaseURL(*getter_AddRefs(mBaseURL));
+        mBaseURL = doc->GetBaseURL();
 
         if (mBaseURL) {
             PRBool isChrome = PR_FALSE;
@@ -683,10 +698,8 @@ nsPresContext::SetShell(nsIPresShell* aShell)
         }
 
         if (mLangService) {
-          nsAutoString charset;
           doc->AddCharSetObserver(this);
-          doc->GetDocumentCharacterSet(charset);
-          UpdateCharSet(charset.get());
+          UpdateCharSet(PromiseFlatCString(doc->GetDocumentCharacterSet()).get());
         }
       }
     }
@@ -704,10 +717,11 @@ nsPresContext::GetShell(nsIPresShell** aResult)
 }
 
 void
-nsPresContext::UpdateCharSet(const PRUnichar* aCharSet)
+nsPresContext::UpdateCharSet(const char* aCharSet)
 {
   if (mLangService) {
-    mLangService->LookupCharSet(aCharSet, getter_AddRefs(mLanguage));
+    mLangService->LookupCharSet(aCharSet,
+                                getter_AddRefs(mLanguage));
     GetFontPreferences();
     if (mLanguage) {
       nsCOMPtr<nsIAtom> langGroupAtom;
@@ -741,7 +755,7 @@ nsPresContext::Observe(nsISupports* aSubject,
                        const PRUnichar* aData)
 {
   if (!nsCRT::strcmp(aTopic, "charset")) {
-    UpdateCharSet(aData);
+    UpdateCharSet(NS_LossyConvertUCS2toASCII(aData).get());
     if (mDeviceContext) {
       mDeviceContext->FlushFontCache();
       ClearStyleDataAndReflow();
@@ -814,7 +828,7 @@ PR_STATIC_CALLBACK(PRBool) set_animation_mode(nsHashKey *aKey, void *aData, void
 //
 // Walks content and set the animation mode
 // this is a way to turn on/off image animations
-void nsPresContext::SetImgAnimations(nsCOMPtr<nsIContent>& aParent, PRUint16 aMode)
+void nsPresContext::SetImgAnimations(nsIContent *aParent, PRUint16 aMode)
 {
   nsCOMPtr<nsIImageLoadingContent> imgContent(do_QueryInterface(aParent));
   if (imgContent) {
@@ -824,14 +838,9 @@ void nsPresContext::SetImgAnimations(nsCOMPtr<nsIContent>& aParent, PRUint16 aMo
     SetImgAnimModeOnImgReq(imgReq, aMode);
   }
   
-  PRInt32 count;
-  aParent->ChildCount(count);
-  for (PRInt32 i=0;i<count;i++) {
-    nsCOMPtr<nsIContent> child;
-    aParent->ChildAt(i, *getter_AddRefs(child));
-    if (child) {
-      SetImgAnimations(child, aMode);
-    }
+  PRUint32 count = aParent->GetChildCount();
+  for (PRUint32 i = 0; i < count; ++i) {
+    SetImgAnimations(aParent->GetChildAt(i), aMode);
   }
 }
 
@@ -852,8 +861,7 @@ nsPresContext::SetImageAnimationMode(PRUint16 aMode)
   if (mShell != nsnull) {
     mShell->GetDocument(getter_AddRefs(doc));
     if (doc) {
-      nsCOMPtr<nsIContent> rootContent;
-      doc->GetRootContent(getter_AddRefs(rootContent));
+      nsIContent *rootContent = doc->GetRootContent();
       if (rootContent) {
         SetImgAnimations(rootContent, aMode);
       }
@@ -980,13 +988,14 @@ nsPresContext::ProbePseudoStyleContextFor(nsIContent* aParentContent,
 }
 
 NS_IMETHODIMP
-nsPresContext::GetXBLBindingURL(nsIContent* aContent, nsAString& aResult)
+nsPresContext::GetXBLBindingURL(nsIContent* aContent, nsIURI** aResult)
 {
   nsRefPtr<nsStyleContext> sc;
   sc = ResolveStyleContextFor(aContent, nsnull);
   NS_ENSURE_TRUE(sc, NS_ERROR_FAILURE);
 
-  aResult = sc->GetStyleDisplay()->mBinding;
+  *aResult = sc->GetStyleDisplay()->mBinding;
+  NS_IF_ADDREF(*aResult);
   return NS_OK;
 }
 
@@ -1002,7 +1011,7 @@ nsPresContext::ReParentStyleContext(nsIFrame* aFrame,
   nsCOMPtr<nsIFrameManager> manager;
   nsresult rv = mShell->GetFrameManager(getter_AddRefs(manager));
   if (NS_SUCCEEDED(rv) && manager) {
-    rv = manager->ReParentStyleContext(this, aFrame, aNewParentContext);
+    rv = manager->ReParentStyleContext(aFrame, aNewParentContext);
   }
   return rv;
 }
@@ -1154,6 +1163,14 @@ nsPresContext::GetDefaultLinkColor(nscolor* aColor)
   return NS_OK;
 }
 
+NS_IMETHODIMP 
+nsPresContext::GetDefaultActiveLinkColor(nscolor* aColor)
+{
+  NS_PRECONDITION(aColor, "null out param");
+  *aColor = mActiveLinkColor;
+  return NS_OK;
+}
+
 NS_IMETHODIMP 
 nsPresContext::GetDefaultVisitedLinkColor(nscolor* aColor)
 {
@@ -1226,6 +1243,13 @@ nsPresContext::SetDefaultLinkColor(nscolor aColor)
   return NS_OK;
 }
 
+NS_IMETHODIMP 
+nsPresContext::SetDefaultActiveLinkColor(nscolor aColor)
+{
+  mActiveLinkColor = aColor;
+  return NS_OK;
+}
+
 NS_IMETHODIMP 
 nsPresContext::SetDefaultVisitedLinkColor(nscolor aColor)
 {
@@ -1340,8 +1364,7 @@ nsPresContext::GetImageLoadFlags(nsLoadFlags& aLoadFlags)
   (void) mShell->GetDocument(getter_AddRefs(doc));
 
   if (doc) {
-    nsCOMPtr<nsILoadGroup> loadGroup;
-    (void) doc->GetDocumentLoadGroup(getter_AddRefs(loadGroup));
+    nsCOMPtr<nsILoadGroup> loadGroup = doc->GetDocumentLoadGroup();
 
     if (loadGroup) {
       loadGroup->GetLoadFlags(&aLoadFlags);
@@ -1352,7 +1375,7 @@ nsPresContext::GetImageLoadFlags(nsLoadFlags& aLoadFlags)
 }
 
 NS_IMETHODIMP
-nsPresContext::LoadImage(const nsString& aURL,
+nsPresContext::LoadImage(nsIURI* aURL,
                          nsIFrame* aTargetFrame,
                          imgIRequest **aRequest)
 {
@@ -1363,41 +1386,25 @@ nsPresContext::LoadImage(const nsString& aURL,
   nsVoidKey key(aTargetFrame);
   nsImageLoader *loader = NS_REINTERPRET_CAST(nsImageLoader*, mImageLoaders.Get(&key)); // addrefs
 
-  nsCOMPtr<nsIDocument> doc;
-  rv = mShell->GetDocument(getter_AddRefs(doc));
-  if (NS_FAILED(rv)) return rv;
-
-  nsCOMPtr<nsIURI> baseURI;
-  doc->GetBaseURL(*getter_AddRefs(baseURI));
-
-  nsCOMPtr<nsIURI> uri;
-  NS_NewURI(getter_AddRefs(uri), aURL, nsnull, baseURI);
-
   if (!loader) {
-    nsCOMPtr<nsIContent> content;
-    aTargetFrame->GetContent(getter_AddRefs(content));
+    nsIContent* content = aTargetFrame->GetContent();
 
     // Check with the content-policy things to make sure this load is permitted.
     nsresult rv;
     nsCOMPtr<nsIDOMElement> element(do_QueryInterface(content));
 
     if (content && element) {
-      nsCOMPtr<nsIDocument> document;
-      rv = content->GetDocument(*getter_AddRefs(document));
+      nsCOMPtr<nsIDocument> document = content->GetDocument();
 
       // If there is no document, skip the policy check
       // XXXldb This really means the document is being destroyed, so
       // perhaps we're better off skipping the load entirely.
       if (document) {
-        nsCOMPtr<nsIScriptGlobalObject> globalScript;
-        rv = document->GetScriptGlobalObject(getter_AddRefs(globalScript));
-
-        if (globalScript) {
-          nsCOMPtr<nsIDOMWindow> domWin(do_QueryInterface(globalScript));
-
+        nsCOMPtr<nsIDOMWindow> domWin(do_QueryInterface(document->GetScriptGlobalObject()));
+        if (domWin) {
           PRBool shouldLoad = PR_TRUE;
           rv = NS_CheckContentLoadPolicy(nsIContentPolicy::IMAGE,
-                                         uri, element, domWin, &shouldLoad);
+                                         aURL, element, domWin, &shouldLoad);
           if (NS_SUCCEEDED(rv) && !shouldLoad)
             return NS_ERROR_FAILURE;
         }
@@ -1423,13 +1430,10 @@ nsPresContext::LoadImage(const nsString& aURL,
   // Allow for a null target frame argument (for precached images)
   if (aTargetFrame) {
     // Mark frame as having loaded an image
-    nsFrameState state;
-    aTargetFrame->GetFrameState(&state);
-    state |= NS_FRAME_HAS_LOADED_IMAGES;
-    aTargetFrame->SetFrameState(state);
+    aTargetFrame->AddStateBits(NS_FRAME_HAS_LOADED_IMAGES);
   }
 
-  loader->Load(uri);
+  loader->Load(aURL);
 
   loader->GetRequest(aRequest);
 
@@ -1494,24 +1498,30 @@ nsPresContext::GetContainer(nsISupports** aResult)
   return CallQueryReferent(mContainer.get(), aResult);
 }
 
-NS_IMETHODIMP
-nsPresContext::GetEventStateManager(nsIEventStateManager** aManager)
+nsIEventStateManager*
+nsIPresContext::GetEventStateManager()
 {
-  NS_PRECONDITION(aManager, "null ptr");
-
   if (!mEventManager) {
-    nsresult rv;
-    mEventManager = do_CreateInstance(kEventStateManagerCID,&rv);
+    nsresult rv = CallCreateInstance(kEventStateManagerCID, &mEventManager);
     if (NS_FAILED(rv)) {
-      return rv;
+      return nsnull;
     }
 
     //Not refcnted, set null in destructor
     mEventManager->SetPresContext(this);
   }
+  return mEventManager;
+}
 
-  *aManager = mEventManager;
-  NS_IF_ADDREF(*aManager);
+NS_IMETHODIMP
+nsPresContext::GetEventStateManager(nsIEventStateManager** aManager)
+{
+  NS_PRECONDITION(aManager, "null ptr");
+
+  *aManager = GetEventStateManager();
+  if (!*aManager)
+    return NS_ERROR_OUT_OF_MEMORY;
+  NS_ADDREF(*aManager);
   return NS_OK;
 }
 
@@ -1546,7 +1556,7 @@ nsPresContext::GetBidiEnabled(PRBool* aBidiEnabled) const
     mShell->GetDocument(getter_AddRefs(doc) );
     NS_ASSERTION(doc, "PresShell has no document in nsPresContext::GetBidiEnabled");
     if (doc) {
-      doc->GetBidiEnabled(aBidiEnabled);
+      *aBidiEnabled = doc->GetBidiEnabled();
     }
   }
   return NS_OK;
@@ -1638,7 +1648,7 @@ nsPresContext::GetIsBidiSystem(PRBool& aResult) const
 }
 
 NS_IMETHODIMP
-nsPresContext::GetBidiCharset(nsAString &aCharSet) const
+nsPresContext::GetBidiCharset(nsACString &aCharSet) const
 {
   aCharSet = mCharset;
   return NS_OK;
@@ -1706,7 +1716,7 @@ nsPresContext::ThemeChanged()
   if (!mShell)
     return NS_OK;
 
-  return mShell->ReconstructStyleData(PR_FALSE);
+  return mShell->ReconstructStyleData();
 }
 
 NS_IMETHODIMP
