@@ -55,7 +55,7 @@ class nsIntervalSet;
  * Child list name indices
  * @see #GetAdditionalChildListName()
  */
-#define NS_BLOCK_FRAME_FLOATER_LIST_INDEX   0
+#define NS_BLOCK_FRAME_FLOAT_LIST_INDEX   0
 #define NS_BLOCK_FRAME_BULLET_LIST_INDEX    1
 #define NS_BLOCK_FRAME_OVERFLOW_LIST_INDEX  2
 #define NS_BLOCK_FRAME_ABSOLUTE_LIST_INDEX  3
@@ -163,8 +163,7 @@ public:
                               nsIContent*     aChild,
                               PRInt32         aNameSpaceID,
                               nsIAtom*        aAttribute,
-                              PRInt32         aModType, 
-                              PRInt32         aHint);
+                              PRInt32         aModType);
 
 #ifdef DO_SELECTION
   NS_IMETHOD  HandleEvent(nsIPresContext* aPresContext,
@@ -196,20 +195,15 @@ public:
 
   static nsresult GetCurrentLine(nsBlockReflowState *aState, nsLineBox **aOutCurrentLine);
 
-  static void CombineRects(const nsRect& r1, nsRect& r2);
-
   inline nscoord GetAscent() { return mAscent; }
 
   // Create a contination for aPlaceholder and its out of flow frame and
-  // add it to the list of overflow floaters
+  // add it to the list of overflow floats
   nsresult SplitPlaceholder(nsIPresContext& aPresContext, nsIFrame& aPlaceholder);
 
   void UndoSplitPlaceholders(nsBlockReflowState& aState,
                              nsIFrame*           aLastPlaceholder);
   
-#ifdef MOZ_ACCESSIBILITY_ATK
-  NS_IMETHOD GetAccessible(nsIAccessible** aAccessible);
-#endif
 protected:
   nsBlockFrame();
   virtual ~nsBlockFrame();
@@ -257,7 +251,7 @@ protected:
   }
 
   /** move the frames contained by aLine by aDY
-    * if aLine is a block, it's child floaters are added to the state manager
+    * if aLine is a block, it's child floats are added to the state manager
     */
   void SlideLine(nsBlockReflowState& aState,
                  nsLineBox* aLine, nscoord aDY);
@@ -299,7 +293,7 @@ protected:
   nsresult DoRemoveFrame(nsIPresContext* aPresContext,
                          nsIFrame* aDeletedFrame);
 
-  // Remove a floater, abs, rel positioned frame from the appropriate block's list
+  // Remove a float, abs, rel positioned frame from the appropriate block's list
   static void DoRemoveOutOfFlowFrame(nsIPresContext* aPresContext,
                                      nsIFrame*       aFrame);
 
@@ -430,12 +424,12 @@ protected:
                              nsIFrame* aFrame,
                              PRUint8* aLineReflowStatus);
 
-  // An incomplete aReflowStatus indicates the floater should be split
+  // An incomplete aReflowStatus indicates the float should be split
   // but only if the available height is constrained.
-  nsresult ReflowFloater(nsBlockReflowState& aState,
-                         nsPlaceholderFrame* aPlaceholder,
-                         nsFloaterCache*     aFloaterCache,
-                         nsReflowStatus&     aReflowStatus);
+  nsresult ReflowFloat(nsBlockReflowState& aState,
+                       nsPlaceholderFrame* aPlaceholder,
+                       nsFloatCache*       aFloatCache,
+                       nsReflowStatus&     aReflowStatus);
 
   //----------------------------------------
   // Methods for pushing/pulling lines/frames
@@ -481,15 +475,15 @@ protected:
                              nsFramePaintLayer    aWhichLayer,
                              PRUint32             aFlags = 0);
 
-  void PaintFloaters(nsIPresContext* aPresContext,
-                     nsIRenderingContext& aRenderingContext,
-                     const nsRect& aDirtyRect);
+  void PaintFloats(nsIPresContext* aPresContext,
+                   nsIRenderingContext& aRenderingContext,
+                   const nsRect& aDirtyRect);
 
-  void PropagateFloaterDamage(nsBlockReflowState& aState,
-                              nsLineBox* aLine,
-                              nscoord aDeltaY);
+  void PropagateFloatDamage(nsBlockReflowState& aState,
+                            nsLineBox* aLine,
+                            nscoord aDeltaY);
 
-  void BuildFloaterList();
+  void BuildFloatList();
 
   //----------------------------------------
   // List handling kludge
@@ -538,8 +532,8 @@ protected:
 
   nsLineList mLines;
 
-  // List of all floaters in this block
-  nsFrameList mFloaters;
+  // List of all floats in this block
+  nsFrameList mFloats;
 
   // XXX_fix_me: subclass one more time!
   // For list-item frames, this is the bullet frame.
