@@ -583,7 +583,6 @@ FrameManager::GetPrimaryFrameFor(nsIContent* aContent, nsIFrame** aResult)
     return NS_ERROR_FAILURE;
   }
 
-  nsresult rv;
   if (mPrimaryFrameMap.ops) {
     PrimaryFrameMapEntry *entry = NS_STATIC_CAST(PrimaryFrameMapEntry*,
         PL_DHashTableOperate(&mPrimaryFrameMap, aContent, PL_DHASH_LOOKUP));
@@ -620,14 +619,13 @@ FrameManager::GetPrimaryFrameFor(nsIContent* aContent, nsIFrame** aResult)
       nsIContent* parent = aContent->GetParent();
       if (parent)
       {
-        PRInt32 index;
-        rv = parent->IndexOf(aContent, index);
-        if (NS_SUCCEEDED(rv) && index > 0)  // no use looking if it's the first child
+        PRInt32 index = parent->IndexOf(aContent);
+        if (index > 0)  // no use looking if it's the first child
         {
-          nsCOMPtr<nsIContent> prevSibling;
+          nsIContent *prevSibling;
           nsCOMPtr<nsIAtom> tag;
           do {
-            parent->ChildAt(--index, getter_AddRefs(prevSibling));
+            prevSibling = parent->GetChildAt(--index);
             prevSibling->GetTag(getter_AddRefs(tag));
           } while (index &&
                    (tag == nsLayoutAtoms::textTagName ||
@@ -635,7 +633,7 @@ FrameManager::GetPrimaryFrameFor(nsIContent* aContent, nsIFrame** aResult)
                     tag == nsLayoutAtoms::processingInstructionTagName));
           if (prevSibling) {
             entry = NS_STATIC_CAST(PrimaryFrameMapEntry*,
-                PL_DHashTableOperate(&mPrimaryFrameMap, prevSibling.get(),
+                PL_DHashTableOperate(&mPrimaryFrameMap, prevSibling,
                                      PL_DHASH_LOOKUP));
             if (PL_DHASH_ENTRY_IS_BUSY(entry))
               hint.mPrimaryFrameForPrevSibling = entry->frame;
@@ -1198,8 +1196,7 @@ nsresult CantRenderReplacedElementEvent::AddLoadGroupRequest(nsIPresShell* aPres
   if (NS_FAILED(rv)) return rv;
   if (!mDummyLayoutRequest) return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsILoadGroup> loadGroup;
-  doc->GetDocumentLoadGroup(getter_AddRefs(loadGroup));
+  nsCOMPtr<nsILoadGroup> loadGroup = doc->GetDocumentLoadGroup();
   if (!loadGroup) return NS_ERROR_FAILURE;
   
   rv = mDummyLayoutRequest->SetLoadGroup(loadGroup);
@@ -1226,8 +1223,7 @@ nsresult CantRenderReplacedElementEvent::RemoveLoadGroupRequest()
     presShell->GetDocument(getter_AddRefs(doc));
     if (!doc) return NS_ERROR_FAILURE;;
 
-    nsCOMPtr<nsILoadGroup> loadGroup;
-    doc->GetDocumentLoadGroup(getter_AddRefs(loadGroup));
+    nsCOMPtr<nsILoadGroup> loadGroup = doc->GetDocumentLoadGroup();
     if (!loadGroup) return NS_ERROR_FAILURE;
 
     rv = loadGroup->RemoveRequest(request, nsnull, NS_OK);
@@ -1264,12 +1260,17 @@ FrameManager::CantRenderReplacedElement(nsIFrame*       aFrame)
       // Create a new event
       ev = new CantRenderReplacedElementEvent(this, aFrame, mPresShell);
 
-      // Add the event to our linked list of posted events
-      ev->mNext = mPostedEvents;
-      mPostedEvents = ev;
-
       // Post the event
-      eventQueue->PostEvent(ev);
+      rv = eventQueue->PostEvent(ev);
+      if (NS_FAILED(rv)) {
+        NS_ERROR("failed to post event");
+        PL_DestroyEvent(ev);
+      }
+      else {
+        // Add the event to our linked list of posted events
+        ev->mNext = mPostedEvents;
+        mPostedEvents = ev;
+      }
     }
   }
 
@@ -1294,7 +1295,7 @@ DumpContext(nsIFrame* aFrame, nsStyleContext* aContext)
   if (aContext) {
     fprintf(stdout, " style: %p ", NS_STATIC_CAST(void*, aContext));
 
-    nsCOMPtr<nsIAtom> pseudoTag = aContext->GetPseudoType();
+    nsIAtom* pseudoTag = aContext->GetPseudoType();
     if (pseudoTag) {
       nsAutoString  buffer;
       pseudoTag->ToString(buffer);
@@ -1566,12 +1567,11 @@ HasAttributeContent(nsStyleContext* aStyleContext,
     const nsStyleContent* content = aStyleContext->GetStyleContent();
     PRUint32 count = content->ContentCount();
     while ((0 < count) && (! result)) {
-      nsStyleContentType  contentType;
-      nsAutoString        contentString;
-      content->GetContentAt(--count, contentType, contentString);
-      if (eStyleContentType_Attr == contentType) {
+      const nsStyleContentData &data = content->ContentAt(--count);
+      if (eStyleContentType_Attr == data.mType) {
         nsIAtom* attrName = nsnull;
         PRInt32 attrNameSpace = kNameSpaceID_None;
+        nsAutoString contentString(data.mContent.mString);
         PRInt32 barIndex = contentString.FindChar('|'); // CSS namespace delimiter
         if (-1 != barIndex) {
           nsAutoString  nameSpaceVal;
@@ -1634,7 +1634,7 @@ FrameManager::ReResolveStyleContext(nsIPresContext* aPresContext,
 
   if (oldContext) {
     oldContext->AddRef();
-    nsCOMPtr<nsIAtom> pseudoTag = oldContext->GetPseudoType();
+    nsIAtom* pseudoTag = oldContext->GetPseudoType();
     nsIContent* localContent = aFrame->GetContent();
     nsIContent* content = localContent ? localContent : aParentContent;
 
@@ -1733,8 +1733,12 @@ FrameManager::ReResolveStyleContext(nsIPresContext* aPresContext,
         const nsStyleBackground* oldColor = oldContext->GetStyleBackground();
         const nsStyleBackground* newColor = newContext->GetStyleBackground();
 
-        if (!oldColor->mBackgroundImage.IsEmpty() &&
-            oldColor->mBackgroundImage != newColor->mBackgroundImage) {
+        PRBool equal;
+        if (oldColor->mBackgroundImage &&
+            (!newColor->mBackgroundImage ||
+             NS_FAILED(oldColor->mBackgroundImage->Equals(
+                                        newColor->mBackgroundImage, &equal)) ||
+             !equal)) {
           // stop the image loading for the frame, the image has changed
           aPresContext->StopImagesFor(aFrame);
         }
