@@ -135,6 +135,7 @@ public:
   nsIBox* mHScrollbarBox;
   nsIBox* mVScrollbarBox;
   nsIBox* mScrollAreaBox;
+  nsIBox* mScrollCornerBox;
   nscoord mOnePixel;
   nsGfxScrollFrame* mOuter;
   nsIScrollableView* mScrollableView;
@@ -351,12 +352,8 @@ nsGfxScrollFrame::CreateAnonymousContent(nsIPresContext* aPresContext,
   if (isPaginated) {
     // allow scrollbars if this is the child of the viewport, because
     // we must be the scrollbars for the print preview window
-    nsIFrame* parent;
-    GetParent(&parent);
-    nsCOMPtr<nsIAtom> parentType;
-    if (parent)
-      parent->GetFrameType(getter_AddRefs(parentType));
-    if (parentType != nsLayoutAtoms::viewportFrame) {
+    nsIFrame* parent = GetParent();
+    if (!parent || parent->GetType() != nsLayoutAtoms::viewportFrame) {
       SetScrollbarVisibility(aPresContext, PR_FALSE, PR_FALSE);
       return NS_OK;
     }
@@ -372,10 +369,7 @@ nsGfxScrollFrame::CreateAnonymousContent(nsIPresContext* aPresContext,
   nsCOMPtr<nsITextControlFrame> textFrame(do_QueryInterface(mParent));
   if (textFrame) {
     // Make sure we are not a text area.
-    nsCOMPtr<nsIContent> content;
-
-    mParent->GetContent(getter_AddRefs(content));
-    nsCOMPtr<nsIDOMHTMLTextAreaElement> textAreaElement(do_QueryInterface(content));
+    nsCOMPtr<nsIDOMHTMLTextAreaElement> textAreaElement(do_QueryInterface(mParent->GetContent()));
     if (!textAreaElement) {
       SetScrollbarVisibility(aPresContext, PR_FALSE, PR_FALSE);
       return NS_OK;
@@ -389,20 +383,19 @@ nsGfxScrollFrame::CreateAnonymousContent(nsIPresContext* aPresContext,
   if (!elementFactory)
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsINodeInfoManager> nodeInfoManager;
+  nsINodeInfoManager *nodeInfoManager = nsnull;
   if (document)
-    document->GetNodeInfoManager(*getter_AddRefs(nodeInfoManager));
+    nodeInfoManager = document->GetNodeInfoManager();
   NS_ENSURE_TRUE(nodeInfoManager, NS_ERROR_FAILURE);
 
   nsCOMPtr<nsINodeInfo> nodeInfo;
-  nodeInfoManager->GetNodeInfo(NS_LITERAL_STRING("scrollbar"),
-                               NS_LITERAL_STRING(""),
-                               NS_LITERAL_STRING("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul"),
-                               *getter_AddRefs(nodeInfo));
+  nodeInfoManager->GetNodeInfo(NS_LITERAL_CSTRING("scrollbar"), nsnull,
+                               kNameSpaceID_XUL, getter_AddRefs(nodeInfo));
 
   ScrollbarStyles styles = GetScrollbarStyles();
-  if (styles.mHorizontal == NS_STYLE_OVERFLOW_AUTO
-      || styles.mHorizontal == NS_STYLE_OVERFLOW_SCROLL) {
+  PRBool canHaveHorizontal = styles.mHorizontal == NS_STYLE_OVERFLOW_AUTO
+    || styles.mHorizontal == NS_STYLE_OVERFLOW_SCROLL;
+  if (canHaveHorizontal) {
     nsCOMPtr<nsIContent> content;
     elementFactory->CreateInstanceByTag(nodeInfo, getter_AddRefs(content));
     content->SetAttr(kNameSpaceID_None, nsXULAtoms::orient,
@@ -412,8 +405,9 @@ nsGfxScrollFrame::CreateAnonymousContent(nsIPresContext* aPresContext,
     aAnonymousChildren.AppendElement(content);
   }
 
-  if (styles.mVertical == NS_STYLE_OVERFLOW_AUTO
-      || styles.mVertical == NS_STYLE_OVERFLOW_SCROLL) {
+  PRBool canHaveVertical = styles.mVertical == NS_STYLE_OVERFLOW_AUTO
+    || styles.mVertical == NS_STYLE_OVERFLOW_SCROLL;
+  if (canHaveVertical) {
     nsCOMPtr<nsIContent> content;
     elementFactory->CreateInstanceByTag(nodeInfo, getter_AddRefs(content));
     content->SetAttr(kNameSpaceID_None, nsXULAtoms::orient,
@@ -423,6 +417,14 @@ nsGfxScrollFrame::CreateAnonymousContent(nsIPresContext* aPresContext,
     aAnonymousChildren.AppendElement(content);
   }
 
+  if (canHaveHorizontal && canHaveVertical) {
+    nodeInfoManager->GetNodeInfo(NS_LITERAL_CSTRING("scrollcorner"), nsnull,
+                                 kNameSpaceID_XUL, getter_AddRefs(nodeInfo));
+    nsCOMPtr<nsIContent> content;
+    elementFactory->CreateInstanceByTag(nodeInfo, getter_AddRefs(content));
+    aAnonymousChildren.AppendElement(content);
+  }
+
   return NS_OK;
 }
 
@@ -445,9 +447,8 @@ nsGfxScrollFrame::Init(nsIPresContext*  aPresContext,
                     nsIFrame*        aPrevInFlow)
 {
   mPresContext = aPresContext;
-  nsresult  rv = nsBoxFrame::Init(aPresContext, aContent,
-                                            aParent, aStyleContext,
-                                            aPrevInFlow);
+  nsresult  rv = nsBoxFrame::Init(aPresContext, aContent, aParent, aStyleContext,
+                                  aPrevInFlow);
   return rv;
 }
 
@@ -456,6 +457,7 @@ void nsGfxScrollFrame::ReloadChildFrames(nsIPresContext* aPresContext)
   mInner->mScrollAreaBox = nsnull;
   mInner->mHScrollbarBox = nsnull;
   mInner->mVScrollbarBox = nsnull;
+  mInner->mScrollCornerBox = nsnull;
 
   nsIFrame* frame = nsnull;
   FirstChild(aPresContext, nsnull, &frame);
@@ -465,15 +467,12 @@ void nsGfxScrollFrame::ReloadChildFrames(nsIPresContext* aPresContext)
     nsIBox* box = nsnull;
     frame->QueryInterface(NS_GET_IID(nsIBox), (void**)&box);
     if (box) {
-      nsCOMPtr<nsIAtom> frameType;
-      frame->GetFrameType(getter_AddRefs(frameType));
-      if (frameType == nsLayoutAtoms::scrollFrame) {
+      if (frame->GetType() == nsLayoutAtoms::scrollFrame) {
         NS_ASSERTION(!mInner->mScrollAreaBox, "Found multiple scroll areas?");
         mInner->mScrollAreaBox = box;
         understood = PR_TRUE;
       } else {
-        nsCOMPtr<nsIContent> content;
-        frame->GetContent(getter_AddRefs(content));
+        nsIContent* content = frame->GetContent();
         if (content) {
           nsAutoString value;
           if (NS_CONTENT_ATTR_HAS_VALUE == content->GetAttr(kNameSpaceID_None,
@@ -487,6 +486,11 @@ void nsGfxScrollFrame::ReloadChildFrames(nsIPresContext* aPresContext)
               mInner->mVScrollbarBox = box;
             }
             understood = PR_TRUE;
+          } else {
+            // probably a scrollcorner
+            NS_ASSERTION(!mInner->mScrollCornerBox, "Found multiple scrollcorners");
+            mInner->mScrollCornerBox = box;
+            understood = PR_TRUE;
           }
         }
       }
@@ -494,7 +498,7 @@ void nsGfxScrollFrame::ReloadChildFrames(nsIPresContext* aPresContext)
 
     NS_ASSERTION(understood, "What is this frame doing here?");
 
-    frame->GetNextSibling(&frame);
+    frame = frame->GetNextSibling();
   }
 }
   
@@ -615,26 +619,17 @@ nsGfxScrollFrame::GetContentAndOffsetsFromPoint(nsIPresContext* aCX,
 
   nsIFrame* frame = nsnull;
   mInner->mScrollAreaBox->GetFrame(&frame);
-  nsIView *view;
   nsPoint point(aPoint);
-  nsPoint currentPoint;
   //we need to translate the coordinates to the inner
-  nsresult result = GetClosestViewForFrame(aCX, this, &view);
-  if (NS_FAILED(result))
-    return result;
+  nsIView *view = GetClosestView();
   if (!view)
     return NS_ERROR_FAILURE;
 
-  nsIView *innerView;
-  result = GetClosestViewForFrame(aCX, frame, &innerView);
-  if (NS_FAILED(result))
-    return result;
+  nsIView *innerView = GetClosestView();
   while (view != innerView && innerView)
   {
-    innerView->GetPosition(&currentPoint.x, &currentPoint.y);
-    point.x -= currentPoint.x;
-    point.y -= currentPoint.y;
-    innerView->GetParent(innerView);
+    point -= innerView->GetPosition();
+    innerView = innerView->GetParent();
   }
 
   return frame->GetContentAndOffsetsFromPoint(aCX, point, aNewContent, aContentOffset, aContentOffsetEnd, aBeginFrameContent);
@@ -646,13 +641,10 @@ nsGfxScrollFrame::GetSkipSides() const
   return 0;
 }
 
-NS_IMETHODIMP
-nsGfxScrollFrame::GetFrameType(nsIAtom** aType) const
+nsIAtom*
+nsGfxScrollFrame::GetType() const
 {
-  NS_PRECONDITION(nsnull != aType, "null OUT parameter pointer");
-  *aType = nsLayoutAtoms::scrollFrame; 
-  NS_ADDREF(*aType);
-  return NS_OK;
+  return nsLayoutAtoms::scrollFrame; 
 }
 
 NS_IMETHODIMP
@@ -860,6 +852,17 @@ nsGfxScrollFrame::Reflow(nsIPresContext*      aPresContext,
   return rv;
 }
 
+nsGfxScrollFrame*
+nsGfxScrollFrame::GetScrollFrameForPort(nsIFrame* aPort) {
+  NS_ASSERTION(aPort, "no port");
+  nsIFrame* parent = aPort->GetParent();
+  if (!parent)
+    return nsnull;
+  nsIScrollableFrame* scFrame =
+    (nsCOMPtr<nsIScrollableFrame>(do_QueryInterface(parent)));
+  return NS_STATIC_CAST(nsGfxScrollFrame*, scFrame);
+}
+
 NS_IMETHODIMP_(nsrefcnt) 
 nsGfxScrollFrame::AddRef(void)
 {
@@ -893,12 +896,14 @@ NS_INTERFACE_MAP_END_INHERITING(nsBoxFrame)
 
 //-------------------- Inner ----------------------
 
-nsGfxScrollFrameInner::nsGfxScrollFrameInner(nsGfxScrollFrame* aOuter):mHScrollbarBox(nsnull),
-                                               mVScrollbarBox(nsnull),
-                                               mScrollAreaBox(nsnull),
-                                               mOnePixel(20),
-                                               mHasVerticalScrollbar(PR_FALSE), 
-                                               mHasHorizontalScrollbar(PR_FALSE)
+nsGfxScrollFrameInner::nsGfxScrollFrameInner(nsGfxScrollFrame* aOuter)
+  : mHScrollbarBox(nsnull),
+    mVScrollbarBox(nsnull),
+    mScrollAreaBox(nsnull),
+    mScrollCornerBox(nsnull),
+    mOnePixel(20),
+    mHasVerticalScrollbar(PR_FALSE), 
+    mHasHorizontalScrollbar(PR_FALSE)
 {
    mOuter = aOuter;
    mMaxElementWidth = 0;
@@ -993,15 +998,10 @@ nsGfxScrollFrameInner::CurPosAttributeChanged(nsIPresContext* aPresContext,
      if (mVScrollbarBox)
        mVScrollbarBox->GetFrame(&vframe);
 
-     nsCOMPtr<nsIContent> vcontent;
-     nsCOMPtr<nsIContent> hcontent;
+     nsIContent* vcontent = vframe ? vframe->GetContent() : nsnull;
+     nsIContent* hcontent = hframe ? hframe->GetContent() : nsnull;
 
-     if (hframe)
-       hframe->GetContent(getter_AddRefs(hcontent));
-     if (vframe)
-       vframe->GetContent(getter_AddRefs(vcontent));
-
-     if (hcontent.get() == aContent || vcontent.get() == aContent)
+     if (hcontent == aContent || vcontent == aContent)
      {
         nscoord x = 0;
         nscoord y = 0;
@@ -1065,7 +1065,7 @@ nsGfxScrollFrameInner::CurPosAttributeChanged(nsIPresContext* aPresContext,
             // likewise for vcontent and vframe. Thus targetFrame will always
             // be non-null in here.
             nsIFrame* targetFrame =
-              hcontent.get() == aContent ? hframe : vframe;
+              hcontent == aContent ? hframe : vframe;
             presShell->HandleEventWithTarget(&event, targetFrame,
                                              aContent,
                                              NS_EVENT_FLAG_INIT, &status);
@@ -1079,15 +1079,18 @@ nsGfxScrollFrameInner::CurPosAttributeChanged(nsIPresContext* aPresContext,
 nsIScrollableView*
 nsGfxScrollFrameInner::GetScrollableView(nsIPresContext* aPresContext)
 {
-  nsIView*           view;
   nsIFrame* frame = nsnull;
   mScrollAreaBox->GetFrame(&frame);
-  frame->GetView(aPresContext, &view);
+  nsIView* view = frame->GetView();
   if (!view) return nsnull;
 
   nsIScrollableView* scrollingView;
-  nsresult result = view->QueryInterface(NS_GET_IID(nsIScrollableView), (void**)&scrollingView);
-  NS_ASSERTION(NS_SUCCEEDED(result), "assertion gfx scrollframe does not contain a scrollframe");          
+#ifdef DEBUG
+  nsresult result =
+#endif
+  CallQueryInterface(view, &scrollingView);
+  NS_ASSERTION(NS_SUCCEEDED(result),
+               "assertion gfx scrollframe does not contain a scrollframe");
   return scrollingView;
 }
 
@@ -1574,6 +1577,34 @@ nsGfxScrollFrameInner::Layout(nsBoxLayoutState& aState)
      needsLayout = PR_FALSE;
   }
 
+  // place the scrollcorner
+  if (mScrollCornerBox) {
+    nsRect r(0, 0, 0, 0);
+    if (clientRect.x != scrollAreaRect.x) {
+      // scrollbar (if any) on left
+      r.x = clientRect.x;
+      r.width = scrollAreaRect.x - clientRect.x;
+      NS_ASSERTION(r.width >= 0, "Scroll area should be inside client rect");
+    } else {
+      // scrollbar (if any) on right
+      r.x = scrollAreaRect.XMost();
+      r.width = clientRect.XMost() - scrollAreaRect.XMost();
+      NS_ASSERTION(r.width >= 0, "Scroll area should be inside client rect");
+    }
+    if (clientRect.y != scrollAreaRect.y) {
+      // scrollbar (if any) on top
+      r.y = clientRect.y;
+      r.height = scrollAreaRect.y - clientRect.y;
+      NS_ASSERTION(r.height >= 0, "Scroll area should be inside client rect");
+    } else {
+      // scrollbar (if any) on bottom
+      r.y = scrollAreaRect.YMost();
+      r.height = clientRect.YMost() - scrollAreaRect.YMost();
+      NS_ASSERTION(r.height >= 0, "Scroll area should be inside client rect");
+    }
+    LayoutBox(aState, mScrollCornerBox, r); 
+  }
+
   // may need to update fixed position children of the viewport,
   // if the client area changed size because of some dirty reflow
   // (if the reflow is initial or resize, the fixed children will
@@ -1581,12 +1612,9 @@ nsGfxScrollFrameInner::Layout(nsBoxLayoutState& aState)
   if ((oldScrollAreaBounds.width != scrollAreaRect.width
       || oldScrollAreaBounds.height != scrollAreaRect.height)
       && nsBoxLayoutState::Dirty == aState.GetLayoutReason()) {
-    nsIFrame* parentFrame;
-    mOuter->GetParent(&parentFrame);
+    nsIFrame* parentFrame = mOuter->GetParent();
     if (parentFrame) {
-      nsCOMPtr<nsIAtom> parentFrameType;
-      parentFrame->GetFrameType(getter_AddRefs(parentFrameType));
-      if (parentFrameType.get() == nsLayoutAtoms::viewportFrame) {
+      if (parentFrame->GetType() == nsLayoutAtoms::viewportFrame) {
         // Usually there are no fixed children, so don't do anything unless there's
         // at least one fixed child
         nsIFrame* child;
@@ -1634,11 +1662,9 @@ nsGfxScrollFrameInner::SetAttribute(nsIBox* aBox, nsIAtom* aAtom, nscoord aSize,
   {
       nsIFrame* frame = nsnull;
       aBox->GetFrame(&frame);
-      nsCOMPtr<nsIContent> content;
-      frame->GetContent(getter_AddRefs(content));
       nsAutoString newValue;
       newValue.AppendInt(aSize);
-      content->SetAttr(kNameSpaceID_None, aAtom, newValue, aReflow);
+      frame->GetContent()->SetAttr(kNameSpaceID_None, aAtom, newValue, aReflow);
       return PR_TRUE;
   }
 
@@ -1655,21 +1681,16 @@ nsGfxScrollFrameInner::GetScrolledSize(nsIPresContext* aPresContext,
 {
 
   // our scrolled size is the size of our scrolled view.
-  nsSize size;
   nsIBox* child = nsnull;
   mScrollAreaBox->GetChildBox(&child);
   nsIFrame* frame;
   child->GetFrame(&frame);
-  nsIView* view;
-  frame->GetView(aPresContext, &view);
+  nsIView* view = frame->GetView();
   NS_ASSERTION(view,"Scrolled frame must have a view!!!");
   
-  nsRect rect(0,0,0,0);
-  view->GetBounds(rect);
-
-  size.width = rect.width;
-  size.height = rect.height;
-
+  nsRect rect = view->GetBounds();
+  nsSize size(rect.width, rect.height);
+ 
   nsBox::AddMargin(child, size);
   nsBox::AddBorderAndPadding(mScrollAreaBox, size);
   nsBox::AddInset(mScrollAreaBox, size);
@@ -1689,8 +1710,7 @@ nsGfxScrollFrameInner::SetScrollbarVisibility(nsIBox* aScrollbar, PRBool aVisibl
   nsIFrame* frame = nsnull;
   aScrollbar->GetFrame(&frame);
 
-  nsCOMPtr<nsIContent> content;
-  frame->GetContent(getter_AddRefs(content));
+  nsIContent* content = frame->GetContent();
   
   PRBool old = PR_TRUE;
 
@@ -1729,8 +1749,7 @@ nsGfxScrollFrameInner::GetIntegerAttribute(nsIBox* aBox, nsIAtom* atom, PRInt32
     nsIFrame* frame = nsnull;
     aBox->GetFrame(&frame);
 
-    nsCOMPtr<nsIContent> content;
-    frame->GetContent(getter_AddRefs(content));
+    nsIContent* content = frame->GetContent();
 
     nsAutoString value;
     if (NS_CONTENT_ATTR_HAS_VALUE == content->GetAttr(kNameSpaceID_None, atom, value))
@@ -1747,6 +1766,7 @@ nsGfxScrollFrameInner::GetIntegerAttribute(nsIBox* aBox, nsIAtom* atom, PRInt32
 nsresult 
 nsGfxScrollFrame::GetContentOf(nsIContent** aContent)
 {
-    GetContent(aContent);
-    return NS_OK;
+  *aContent = GetContent();
+  NS_IF_ADDREF(*aContent);
+  return NS_OK;
 }
