@@ -493,9 +493,8 @@ nsImageFrame::HandleLoadError(nsresult aStatus, nsIPresShell* aPresShell)
     else {
       // We are in quirks mode, so we can just check the tag name; no need to
       // check the namespace.
-      nsCOMPtr<nsINodeInfo> nodeInfo;
-      mContent->GetNodeInfo(getter_AddRefs(nodeInfo));
-      
+      nsINodeInfo *nodeInfo = mContent->GetNodeInfo();
+
       if (!mContent->HasAttr(kNameSpaceID_None, nsHTMLAtoms::alt) &&
           nodeInfo &&
           !nodeInfo->Equals(nsHTMLAtoms::object)) {
@@ -1431,8 +1430,7 @@ nsImageFrame::Paint(nsIPresContext*      aPresContext,
           nsCOMPtr<nsIContent> parentContent = mContent->GetParent();
           if (parentContent)
           {
-            PRInt32 thisOffset;
-            parentContent->IndexOf(mContent, thisOffset);
+            PRInt32 thisOffset = parentContent->IndexOf(mContent);
             nsCOMPtr<nsIDOMNode> parentNode = do_QueryInterface(parentContent);
             nsCOMPtr<nsIDOMNode> rangeNode;
             PRInt32 rangeOffset;
@@ -1507,7 +1505,6 @@ nsImageFrame::TriggerLink(nsIPresContext* aPresContext,
   aPresContext->GetLinkHandler(getter_AddRefs(handler));
   if (nsnull != handler) {
     if (aClick) {
-      nsresult proceed = NS_OK;
       // Check that this page is allowed to load this URI.
       // Almost a copy of the similarly named method in nsGenericElement
       nsresult rv;
@@ -1521,17 +1518,18 @@ nsImageFrame::TriggerLink(nsIPresContext* aPresContext,
       if (NS_SUCCEEDED(rv) && ps) 
         rv = ps->GetDocument(getter_AddRefs(doc));
       
-      nsCOMPtr<nsIURI> baseURI;
-      if (NS_SUCCEEDED(rv) && doc) 
-        doc->GetDocumentURL(getter_AddRefs(baseURI));
-      
-      if (NS_SUCCEEDED(rv)) 
-        proceed = securityManager->CheckLoadURI(baseURI, aURI, nsIScriptSecurityManager::STANDARD);
+      if (NS_SUCCEEDED(rv)) {
+        nsIURI *baseURI = doc ? doc->GetDocumentURL() : nsnull;
 
-      // Only pass off the click event if the script security manager
-      // says it's ok.
-      if (NS_SUCCEEDED(proceed))
-        handler->OnLinkClick(mContent, eLinkVerb_Replace, aURI, aTargetSpec.get());
+        rv = securityManager->CheckLoadURI(baseURI, aURI,
+                                           nsIScriptSecurityManager::STANDARD);
+
+        // Only pass off the click event if the script security manager
+        // says it's ok.
+        if (NS_SUCCEEDED(rv))
+          handler->OnLinkClick(mContent, eLinkVerb_Replace, aURI,
+                               aTargetSpec.get());
+      }
     }
     else {
       handler->OnOverLink(mContent, aURI, aTargetSpec.get());
@@ -1693,17 +1691,15 @@ nsImageFrame::HandleEvent(nsIPresContext* aPresContext,
             // element to provide the basis for the destination url.
             nsAutoString src;
             if (GetAnchorHREFAndTarget(src, target)) {
-              nsCOMPtr<nsINodeInfo> nodeInfo;
-              mContent->GetNodeInfo(getter_AddRefs(nodeInfo));
+              nsINodeInfo *nodeInfo = mContent->GetNodeInfo();
               NS_ASSERTION(nodeInfo, "Image content without a nodeinfo?");
               nsIDocument* doc = nodeInfo->GetDocument();
               nsCAutoString charset;
               if (doc) {
-                doc->GetDocumentCharacterSet(charset);
+                charset = doc->GetDocumentCharacterSet();
               } 
               nsCOMPtr<nsIURI> uri;
-              nsresult rv = NS_NewURI(getter_AddRefs(uri), src,
-                                      charset.get(),
+              nsresult rv = NS_NewURI(getter_AddRefs(uri), src, charset.get(),
                                       baseURL);
               NS_ENSURE_SUCCESS(rv, rv);
             
@@ -1888,7 +1884,7 @@ nsImageFrame::GetDocumentCharacterSet(nsACString& aCharset) const
   if (mContent) {
     NS_ASSERTION(mContent->GetDocument(),
                  "Frame still alive after content removed from document!");
-    mContent->GetDocument()->GetDocumentCharacterSet(aCharset);
+    aCharset = mContent->GetDocument()->GetDocumentCharacterSet();
   }
 }
 
@@ -1926,13 +1922,13 @@ nsImageFrame::GetLoadGroup(nsIPresContext *aPresContext, nsILoadGroup **aLoadGro
   if (!doc)
     return;
 
-  doc->GetDocumentLoadGroup(aLoadGroup);
+  *aLoadGroup = doc->GetDocumentLoadGroup().get();  // already_AddRefed
 }
 
 nsresult nsImageFrame::LoadIcons(nsIPresContext *aPresContext)
 {
-  NS_NAMED_LITERAL_STRING(loadingSrc,"resource:/res/loading-image.gif"); 
-  NS_NAMED_LITERAL_STRING(brokenSrc,"resource:/res/broken-image.gif");
+  NS_NAMED_LITERAL_STRING(loadingSrc,"resource://gre/res/loading-image.gif"); 
+  NS_NAMED_LITERAL_STRING(brokenSrc,"resource://gre/res/broken-image.gif");
 
   PRBool doLoad = PR_FALSE;  // only load icons once...
 
