@@ -448,16 +448,14 @@ void RectArea::ParseCoords(const nsAString& aSpec)
     if (NS_FAILED(rv))
       return;
 
-    nsCOMPtr<nsINodeInfo> nodeInfo;
-    mArea->GetNodeInfo(getter_AddRefs(nodeInfo));
+    // XXX GetOwnerDocument
+    nsINodeInfo *nodeInfo = mArea->GetNodeInfo();
     NS_ASSERTION(nodeInfo, "Element with no nodeinfo");
-    
-    nsCOMPtr<nsIDocument> doc;
-    nodeInfo->GetDocument(getter_AddRefs(doc));
+
+    nsIDocument* doc = nodeInfo->GetDocument();
     nsCAutoString urlSpec;
     if (doc) {
-      nsCOMPtr<nsIURI> uri;
-      doc->GetDocumentURL(getter_AddRefs(uri));
+      nsIURI *uri = doc->GetDocumentURL();
       if (uri) {
         uri->GetSpec(urlSpec);
       }
@@ -737,10 +735,12 @@ void CircleArea::GetRect(nsIPresContext* aCX, nsRect& aRect)
 //----------------------------------------------------------------------
 
 
-nsImageMap::nsImageMap()
+nsImageMap::nsImageMap() :
+  mPresShell(nsnull),
+  mImageFrame(nsnull),
+  mDocument(nsnull),
+  mContainsBlockContents(PR_FALSE)
 {
-  mDocument = nsnull;
-  mContainsBlockContents = PR_FALSE;
 }
 
 nsImageMap::~nsImageMap()
@@ -760,8 +760,8 @@ nsImageMap::~nsImageMap()
   }
 
   FreeAreas();
-  if (nsnull != mDocument) {
-    mDocument->RemoveObserver(NS_STATIC_CAST(nsIDocumentObserver*, this));
+  if (mDocument) {
+    mDocument->RemoveObserver(this);
   }
 }
 
@@ -769,7 +769,7 @@ NS_IMPL_ISUPPORTS4(nsImageMap,
                    nsIDocumentObserver,
                    nsIDOMFocusListener,
                    nsIDOMEventListener,
-                   nsIImageMap);
+                   nsIImageMap)
 
 NS_IMETHODIMP
 nsImageMap::GetBoundsForAreaContent(nsIContent *aContent, 
@@ -816,12 +816,9 @@ nsImageMap::Init(nsIPresShell* aPresShell, nsIFrame* aImageFrame, nsIDOMHTMLMapE
   nsresult rv;
   mMap = do_QueryInterface(aMap, &rv);
   NS_ASSERTION(mMap, "aMap is not an nsIHTMLContent!");
-  rv = mMap->GetDocument(&mDocument);
-  if (NS_SUCCEEDED(rv) && mDocument) {
-    mDocument->AddObserver(NS_STATIC_CAST(nsIDocumentObserver*, this));
-    // mDocument is a weak reference, so release the reference we got
-    nsIDocument *temp = mDocument;
-    NS_RELEASE(temp);
+  mDocument = mMap->GetDocument();
+  if (mDocument) {
+    mDocument->AddObserver(this);
   }
 
   // "Compile" the areas in the map into faster access versions
@@ -834,20 +831,18 @@ nsresult
 nsImageMap::UpdateAreasForBlock(nsIContent* aParent, PRBool* aFoundAnchor)
 {
   nsresult rv = NS_OK;
-  PRInt32 i, n;
-  aParent->ChildCount(n);
+  PRUint32 i, n = aParent->GetChildCount();
+
   for (i = 0; (i < n) && NS_SUCCEEDED(rv); i++) {
-    nsCOMPtr<nsIContent> child;
-    rv = aParent->ChildAt(i, getter_AddRefs(child));
-    if (NS_SUCCEEDED(rv)) {
-      nsCOMPtr<nsIDOMHTMLAnchorElement> area = do_QueryInterface(child, &rv);
-      if (NS_SUCCEEDED(rv)) {
-        *aFoundAnchor = PR_TRUE;
-        rv = AddArea(child);
-      }
-      else {
-        rv = UpdateAreasForBlock(child, aFoundAnchor);
-      }
+    nsIContent *child = aParent->GetChildAt(i);
+
+    nsCOMPtr<nsIDOMHTMLAnchorElement> area = do_QueryInterface(child);
+    if (area) {
+      *aFoundAnchor = PR_TRUE;
+      rv = AddArea(child);
+    }
+    else {
+      rv = UpdateAreasForBlock(child, aFoundAnchor);
     }
   }
   
@@ -860,17 +855,14 @@ nsImageMap::UpdateAreas()
   // Get rid of old area data
   FreeAreas();
 
-  PRInt32 i, n;
+  PRUint32 i, n = mMap->GetChildCount();
   PRBool containsBlock = PR_FALSE, containsArea = PR_FALSE;
 
-  mMap->ChildCount(n);
   for (i = 0; i < n; i++) {
-    nsCOMPtr<nsIContent> child;
-    mMap->ChildAt(i, getter_AddRefs(child));
+    nsIContent *child = mMap->GetChildAt(i);
 
     // Only look at elements and not text, comments, etc.
-    nsCOMPtr<nsIDOMHTMLElement> element = do_QueryInterface(child);
-    if (! element)
+    if (!child->IsContentOfType(nsIContent::eHTML))
       continue;
 
     // First check if this map element contains an AREA element.
@@ -1027,8 +1019,7 @@ PRBool
 nsImageMap::IsAncestorOf(nsIContent* aContent,
                          nsIContent* aAncestorContent)
 {
-  nsCOMPtr<nsIContent> parent;
-  aContent->GetParent(getter_AddRefs(parent));
+  nsCOMPtr<nsIContent> parent = aContent->GetParent();
   if (parent) {
     return parent == aAncestorContent ||
            IsAncestorOf(parent, aAncestorContent);
@@ -1044,9 +1035,8 @@ nsImageMap::ContentChanged(nsIDocument *aDocument,
 {
   // If the parent of the changing content node is our map then update
   // the map.
-  nsCOMPtr<nsIContent> parent;
-  nsresult rv = aContent->GetParent(getter_AddRefs(parent));
-  if (NS_SUCCEEDED(rv) && (nsnull != parent)) {
+  nsIContent* parent = aContent->GetParent();
+  if (parent) {
     if ((parent == mMap) || 
         (mContainsBlockContents && IsAncestorOf(parent, mMap))) {
       UpdateAreas();
@@ -1060,13 +1050,11 @@ nsImageMap::AttributeChanged(nsIDocument *aDocument,
                              nsIContent*  aContent,
                              PRInt32      aNameSpaceID,
                              nsIAtom*     aAttribute,
-                             PRInt32      aModType, 
-                             nsChangeHint aHint)
+                             PRInt32      aModType)
 {
   // If the parent of the changing content node is our map then update
   // the map.
-  nsCOMPtr<nsIContent> parent;
-  aContent->GetParent(getter_AddRefs(parent));
+  nsIContent* parent = aContent->GetParent();
   if ((parent == mMap) || 
       (mContainsBlockContents && IsAncestorOf(parent, mMap))) {
     UpdateAreas();
@@ -1155,11 +1143,10 @@ nsImageMap::ChangeFocus(nsIDOMEvent* aEvent, PRBool aFocus) {
             //Set or Remove internal focus
             area->HasFocus(aFocus);
             //Now invalidate the rect
-            nsCOMPtr<nsIDocument> doc;
+            nsCOMPtr<nsIDocument> doc = targetContent->GetDocument();
             //This check is necessary to see if we're still attached to the doc
-            if (NS_SUCCEEDED(targetContent->GetDocument(getter_AddRefs(doc))) && doc) {
-              nsCOMPtr<nsIPresShell> presShell;
-              doc->GetShellAt(0, getter_AddRefs(presShell));
+            if (doc) {
+              nsIPresShell *presShell = doc->GetShellAt(0);
               if (presShell) {
                 nsIFrame* imgFrame;
                 if (NS_SUCCEEDED(presShell->GetPrimaryFrameFor(targetContent, &imgFrame)) && imgFrame) {
