@@ -107,13 +107,10 @@ nsInlineFrame::GetFrameName(nsAString& aResult) const
 }
 #endif
 
-NS_IMETHODIMP
-nsInlineFrame::GetFrameType(nsIAtom** aType) const
+nsIAtom*
+nsInlineFrame::GetType() const
 {
-  NS_PRECONDITION(nsnull != aType, "null OUT parameter pointer");
-  *aType = nsLayoutAtoms::inlineFrame;
-  NS_ADDREF(*aType);
-  return NS_OK;
+  return nsLayoutAtoms::inlineFrame;
 }
 
 inline PRBool
@@ -139,16 +136,16 @@ IsMarginZero(nsStyleUnit aUnit, nsStyleCoord &aCoord)
             (aUnit == eStyleUnit_Percent && aCoord.GetPercentValue() == 0.0));
 }
 
-NS_IMETHODIMP
-nsInlineFrame::IsEmpty(nsCompatibility aCompatMode, PRBool aIsPre,
-                       PRBool* aResult)
+/* virtual */ PRBool
+nsInlineFrame::IsEmpty()
 {
 #if 0
   // I used to think inline frames worked this way, but it seems they
   // don't.  At least not in our codebase.
-  if (aCompatMode == eCompatibility_FullStandards) {
-    *aResult = PR_FALSE;
-    return NS_OK;
+  nsCompatibility compatMode;
+  GetPresContext()->GetCompatibilityMode(&compatMode);
+  if (compatMode == eCompatibility_FullStandards) {
+    return PR_FALSE;
   }
 #endif
   const nsStyleMargin* margin = GetStyleMargin();
@@ -172,17 +169,14 @@ nsInlineFrame::IsEmpty(nsCompatibility aCompatMode, PRBool aIsPre,
                     margin->mMargin.GetRight(coord)) ||
       !IsMarginZero(margin->mMargin.GetLeftUnit(),
                     margin->mMargin.GetLeft(coord))) {
-    *aResult = PR_FALSE;
-    return NS_OK;
+    return PR_FALSE;
   }
 
-  *aResult = PR_TRUE;
-  for (nsIFrame *kid = mFrames.FirstChild(); kid; kid->GetNextSibling(&kid)) {
-    kid->IsEmpty(aCompatMode, aIsPre, aResult);
-    if (! *aResult)
-      break;
+  for (nsIFrame *kid = mFrames.FirstChild(); kid; kid = kid->GetNextSibling()) {
+    if (!kid->IsEmpty())
+      return PR_FALSE;
   }
-  return NS_OK;
+  return PR_TRUE;
 }
 
 NS_IMETHODIMP
@@ -244,27 +238,20 @@ nsInlineFrame::RemoveFrame(nsIPresContext* aPresContext,
 
   if (aOldFrame) {
     // Loop and destroy the frame and all of its continuations.
-    PRBool generateReflowCommand = PR_FALSE;
-
     // If the frame we are removing is a brFrame, we need a reflow so
     // the line the brFrame was on can attempt to pull up any frames
     // that can fit from lines below it.
-    nsCOMPtr<nsIAtom> frameType;
-    aOldFrame->GetFrameType(getter_AddRefs(frameType));
-    if (frameType == nsLayoutAtoms::brFrame)
-      generateReflowCommand = PR_TRUE;
+    PRBool generateReflowCommand =
+      aOldFrame->GetType() == nsLayoutAtoms::brFrame;
 
-    nsIFrame* oldFrameParent;
-    aOldFrame->GetParent(&oldFrameParent);
-    nsInlineFrame* parent = (nsInlineFrame*) oldFrameParent;
-    while (nsnull != aOldFrame) {
+    nsInlineFrame* parent = NS_STATIC_CAST(nsInlineFrame*, aOldFrame->GetParent());
+    while (aOldFrame) {
 #ifdef IBMBIDI
       if (nsLayoutAtoms::nextBidi != aListName) {
 #endif
       // If the frame being removed has zero size then don't bother
       // generating a reflow command, otherwise make sure we do.
-      nsRect bbox;
-      aOldFrame->GetRect(bbox);
+      nsRect bbox = aOldFrame->GetRect();
       if (bbox.width || bbox.height) {
         generateReflowCommand = PR_TRUE;
       }
@@ -279,8 +266,8 @@ nsInlineFrame::RemoveFrame(nsIPresContext* aPresContext,
       aOldFrame->GetNextInFlow(&oldFrameNextInFlow);
       parent->mFrames.DestroyFrame(aPresContext, aOldFrame);
       aOldFrame = oldFrameNextInFlow;
-      if (nsnull != aOldFrame) {
-        aOldFrame->GetParent((nsIFrame**) &parent);
+      if (aOldFrame) {
+        parent = NS_STATIC_CAST(nsInlineFrame*, aOldFrame->GetParent());
       }
     }
 
@@ -416,9 +403,7 @@ nsInlineFrame::Reflow(nsIPresContext*          aPresContext,
       // our prev-in-flow's overflow list, it's possible that we have not set
       // the parent pointer for these frames. Check the first frame to see, and
       // if we haven't set the parent pointer then set it now
-      nsIFrame* parent;
-      overflowFrames->GetParent(&parent);
-      mFrames.AppendFrames(parent == this ? nsnull : this, overflowFrames);
+      mFrames.AppendFrames(overflowFrames->GetParent() == this ? nsnull : this, overflowFrames);
     }
   }
 
@@ -553,7 +538,7 @@ nsInlineFrame::ReflowFrames(nsIPresContext* aPresContext,
       break;
     }
     irs.mPrevFrame = frame;
-    frame->GetNextSibling(&frame);
+    frame = frame->GetNextSibling();
   }
 
   // Attempt to pull frames from our next-in-flow until we can't
@@ -686,9 +671,7 @@ nsInlineFrame::ReflowFrames(nsIPresContext* aPresContext,
 static 
 void SetContainsPercentAwareChild(nsIFrame *aFrame)
 {
-  nsFrameState myFrameState;
-  aFrame->GetFrameState(&myFrameState);
-  aFrame->SetFrameState(myFrameState | NS_INLINE_FRAME_CONTAINS_PERCENT_AWARE_CHILD);
+  aFrame->AddStateBits(NS_INLINE_FRAME_CONTAINS_PERCENT_AWARE_CHILD);
 }
 
 static
@@ -696,9 +679,7 @@ void MarkPercentAwareFrame(nsIPresContext *aPresContext,
                            nsInlineFrame  *aInline,
                            nsIFrame       *aFrame)
 {
-  nsFrameState childFrameState;
-  aFrame->GetFrameState(&childFrameState);
-  if (childFrameState & NS_FRAME_REPLACED_ELEMENT) 
+  if (aFrame->GetStateBits() & NS_FRAME_REPLACED_ELEMENT) 
   { // aFrame is a replaced element, check it's style
     if (nsLineLayout::IsPercentageAwareReplacedElement(aPresContext, aFrame)) {
       SetContainsPercentAwareChild(aInline);
@@ -710,7 +691,7 @@ void MarkPercentAwareFrame(nsIPresContext *aPresContext,
     aFrame->FirstChild(aPresContext, nsnull, &child);
     if (child)
     { // aFrame is an inline container frame, check my frame state
-      if (childFrameState & NS_INLINE_FRAME_CONTAINS_PERCENT_AWARE_CHILD) {
+      if (aFrame->GetStateBits() & NS_INLINE_FRAME_CONTAINS_PERCENT_AWARE_CHILD) {
         SetContainsPercentAwareChild(aInline); // if a child container is effected, so am I
       }
     }
@@ -760,11 +741,8 @@ nsInlineFrame::ReflowInlineFrame(nsIPresContext* aPresContext,
         // remaining child frames in our child list with the wrong parent
         // frame pointer...
         if (irs.mSetParentPointer) {
-          nsIFrame* f;
-          aFrame->GetNextSibling(&f);
-          while (f) {
+          for (nsIFrame* f = aFrame->GetNextSibling(); f; f = f->GetNextSibling()) {
             f->SetParent(this);
-            f->GetNextSibling(&f);
           }
         }
       }
@@ -778,9 +756,8 @@ nsInlineFrame::ReflowInlineFrame(nsIPresContext* aPresContext,
           return rv;
         }
       }
-      nsIFrame* nextFrame;
-      aFrame->GetNextSibling(&nextFrame);
-      if (nsnull != nextFrame) {
+      nsIFrame* nextFrame = aFrame->GetNextSibling();
+      if (nextFrame) {
         aStatus |= NS_FRAME_NOT_COMPLETE;
         PushFrames(aPresContext, nextFrame, aFrame);
       }
@@ -799,9 +776,7 @@ nsInlineFrame::ReflowInlineFrame(nsIPresContext* aPresContext,
     }
   }
   else if (NS_FRAME_IS_NOT_COMPLETE(aStatus)) {
-    nsCOMPtr<nsIAtom> frameType;
-    aFrame->GetFrameType(getter_AddRefs(frameType));
-    if (nsLayoutAtoms::placeholderFrame == frameType) {
+    if (nsLayoutAtoms::placeholderFrame == aFrame->GetType()) {
       nsBlockReflowState* blockRS = lineLayout->mBlockRS;
       blockRS->mBlock->SplitPlaceholder(*aPresContext, *aFrame);
     }
@@ -812,9 +787,8 @@ nsInlineFrame::ReflowInlineFrame(nsIPresContext* aPresContext,
         return rv;
       }
       if (!reflowingFirstLetter) {
-        nsIFrame* nextFrame;
-        aFrame->GetNextSibling(&nextFrame);
-        if (nsnull != nextFrame) {
+        nsIFrame* nextFrame = aFrame->GetNextSibling();
+        if (nextFrame) {
           PushFrames(aPresContext, nextFrame, aFrame);
         }
       }
@@ -854,11 +828,7 @@ nsInlineFrame::PushFrames(nsIPresContext* aPresContext,
 {
   NS_PRECONDITION(nsnull != aFromChild, "null pointer");
   NS_PRECONDITION(nsnull != aPrevSibling, "pushing first child");
-#ifdef DEBUG
-  nsIFrame* prevNextSibling;
-  aPrevSibling->GetNextSibling(&prevNextSibling);
-  NS_PRECONDITION(prevNextSibling == aFromChild, "bad prev sibling");
-#endif
+  NS_PRECONDITION(aPrevSibling->GetNextSibling() == aFromChild, "bad prev sibling");
 
 #ifdef NOISY_PUSHING
       printf("%p pushing aFromChild %p, disconnecting from prev sib %p\n", 
@@ -912,10 +882,10 @@ NS_IMETHODIMP nsInlineFrame::GetAccessible(nsIAccessible** aAccessible)
   // Broken image accessibles are created here, because layout
   // replaces the image or image control frame with an inline frame
   *aAccessible = nsnull;
-  nsCOMPtr<nsIAtom> tagAtom;
-  mContent->GetTag(*getter_AddRefs(tagAtom));
-  if (tagAtom == nsHTMLAtoms::img || tagAtom == nsHTMLAtoms::input || 
-     tagAtom == nsHTMLAtoms::label || tagAtom == nsHTMLAtoms::hr) {
+  nsIAtom *tagAtom = mContent->Tag();
+  if ((tagAtom == nsHTMLAtoms::img || tagAtom == nsHTMLAtoms::input || 
+       tagAtom == nsHTMLAtoms::label || tagAtom == nsHTMLAtoms::hr) &&
+      mContent->IsContentOfType(nsIContent::eHTML)) {
     // Only get accessibility service if we're going to use it
     nsCOMPtr<nsIAccessibilityService> accService(do_GetService("@mozilla.org/accessibilityService;1"));
     if (!accService)
@@ -943,10 +913,9 @@ ReParentChildListStyle(nsIPresContext* aPresContext,
                        nsStyleContext* aParentStyleContext,
                        nsFrameList& aFrameList)
 {
-  nsIFrame* kid = aFrameList.FirstChild();
-  while (nsnull != kid) {
+  for (nsIFrame* kid = aFrameList.FirstChild(); kid;
+       kid = kid->GetNextSibling()) {
     aPresContext->ReParentStyleContext(kid, aParentStyleContext);
-    kid->GetNextSibling(&kid);
   }
 }
 
@@ -977,13 +946,10 @@ nsFirstLineFrame::GetFrameName(nsAString& aResult) const
 }
 #endif
 
-NS_IMETHODIMP
-nsFirstLineFrame::GetFrameType(nsIAtom** aType) const
+nsIAtom*
+nsFirstLineFrame::GetType() const
 {
-  NS_PRECONDITION(nsnull != aType, "null OUT parameter pointer");
-  *aType = nsLayoutAtoms::lineFrame;
-  NS_ADDREF(*aType);
-  return NS_OK;
+  return nsLayoutAtoms::lineFrame;
 }
 
 void
@@ -1027,8 +993,8 @@ nsFirstLineFrame::Reflow(nsIPresContext* aPresContext,
     if (prevOverflowFrames) {
       nsFrameList frames(prevOverflowFrames);
       
-      ReParentChildListStyle(aPresContext, mStyleContext, frames);
       mFrames.InsertFrames(this, nsnull, prevOverflowFrames);
+      ReParentChildListStyle(aPresContext, mStyleContext, frames);
     }
   }
 
@@ -1038,8 +1004,8 @@ nsFirstLineFrame::Reflow(nsIPresContext* aPresContext,
     NS_ASSERTION(mFrames.NotEmpty(), "overflow list w/o frames");
     nsFrameList frames(overflowFrames);
 
-    ReParentChildListStyle(aPresContext, mStyleContext, frames);
     mFrames.AppendFrames(nsnull, overflowFrames);
+    ReParentChildListStyle(aPresContext, mStyleContext, frames);
   }
 
   // Set our own reflow state (additional state above and beyond
@@ -1085,9 +1051,7 @@ nsFirstLineFrame::Reflow(nsIPresContext* aPresContext,
     if (mStyleContext == first->mStyleContext) {
       // Fixup our style context and our children. First get the
       // proper parent context.
-      nsIFrame* parentFrame;
-      first->GetParent(&parentFrame);
-      nsStyleContext* parentContext = parentFrame->GetStyleContext();
+      nsStyleContext* parentContext = first->GetParent()->GetStyleContext();
       if (parentContext) {
         // Create a new style context that is a child of the parent
         // style context thus removing the :first-line style. This way
@@ -1253,13 +1217,10 @@ nsPositionedInlineFrame::FirstChild(nsIPresContext* aPresContext,
   return nsInlineFrame::FirstChild(aPresContext, aListName, aFirstChild);
 }
 
-NS_IMETHODIMP
-nsPositionedInlineFrame::GetFrameType(nsIAtom** aType) const
+nsIAtom*
+nsPositionedInlineFrame::GetType() const
 {
-  NS_PRECONDITION(nsnull != aType, "null OUT parameter pointer");
-  *aType = nsLayoutAtoms::positionedInlineFrame;
-  NS_ADDREF(*aType);
-  return NS_OK;
+  return nsLayoutAtoms::positionedInlineFrame;
 }
 
 NS_IMETHODIMP
