@@ -43,6 +43,7 @@
 #include "nsIPresShell.h"
 #include "nsIPluginHost.h"
 #include "nsplugin.h"
+#include "nsIWidget.h"
 #include "nsIObjectFrame.h"
 
 #ifdef ACCESSIBILITY
@@ -83,8 +84,10 @@ public:
                           nsGUIEvent*     aEvent,
                           nsEventStatus*  aEventStatus);
 
-  NS_IMETHOD Scrolled(nsIView *aView);
   NS_IMETHOD GetFrameType(nsIAtom** aType) const;
+  
+  virtual PRBool SupportsVisibilityHidden() { return PR_FALSE; }
+
 #ifdef DEBUG
   NS_IMETHOD GetFrameName(nsAString& aResult) const;
 #endif
@@ -129,7 +132,6 @@ public:
 
   void FixUpURLS(const nsString &name, nsString &value);
 
-  nsIPresContext *mPresContext;  // weak ref
 protected:
   // nsISupports
   NS_IMETHOD_(nsrefcnt) AddRef(void);
@@ -139,10 +141,11 @@ protected:
 
   virtual PRIntn GetSkipSides() const;
 
-  virtual void GetDesiredSize(nsIPresContext* aPresContext,
-                              const nsHTMLReflowState& aReflowState,
-                              nsHTMLReflowMetrics& aDesiredSize);
-
+  // NOTE:  This frame class does not inherit from |nsLeafFrame|, so
+  // this is not a virtual method implementation.
+  void GetDesiredSize(nsIPresContext* aPresContext,
+                      const nsHTMLReflowState& aReflowState,
+                      nsHTMLReflowMetrics& aDesiredSize);
 
   nsresult SetFullURL(nsIURI* aURL);
 
@@ -168,8 +171,6 @@ protected:
                        nsReflowStatus&          aStatus,
                        nsIFrame*                child);
  
-  nsresult GetBaseURL(nsIURI* &aURL);
-
   // check attributes and optionally CSS to see if we should display anything
   PRBool IsHidden(PRBool aCheckVisibilityStyle = PR_TRUE) const;
 
@@ -177,11 +178,14 @@ protected:
 
   nsresult GetWindowOriginInPixels(nsIPresContext * aPresContext, PRBool aWindoless, nsPoint* aOrigin);
 
+  friend class nsPluginInstanceOwner;
+  
+  nsIPresContext *mPresContext;  // weak ref
 private:
   nsPluginInstanceOwner *mInstanceOwner;
-  nsIURI                *mFullURL;
+  nsCOMPtr<nsIURI>      mFullURL;
   nsIFrame              *mFirstChild;
-  nsIWidget             *mWidget;
+  nsCOMPtr<nsIWidget>   mWidget;
   nsRect                mWindowlessRect;
 };
 
