@@ -388,9 +388,8 @@ nsComboboxControlFrame::Init(nsIPresContext*  aPresContext,
   // Start - Temporary fix for Bug 36558
   //-------------------------------
   mGoodToGo = PR_FALSE;
-  nsCOMPtr<nsIDocument> document;
-  nsresult rv = aContent->GetDocument(getter_AddRefs(document));
-  if (NS_SUCCEEDED(rv) && document) {
+  nsIDocument* document = aContent->GetDocument();
+  if (document) {
 #ifdef MOZ_XUL
     nsCOMPtr<nsIXULDocument> xulDoc(do_QueryInterface(document));
     mGoodToGo = xulDoc?PR_FALSE:PR_TRUE;
@@ -431,7 +430,7 @@ nsComboboxControlFrame::InitializeControl(nsIPresContext* aPresContext)
 
 //--------------------------------------------------------------
 NS_IMETHODIMP_(PRInt32)
-nsComboboxControlFrame::GetType() const
+nsComboboxControlFrame::GetFormControlType() const
 {
   return NS_FORM_SELECT;
 }
@@ -630,7 +629,7 @@ nsComboboxControlFrame::ShowList(nsIPresContext* aPresContext, PRBool aShowList)
   presShell->FlushPendingNotifications(PR_FALSE);
 
   if (widget)
-    widget->CaptureRollupEvents((nsIRollupListener *)this, mDroppedDown, PR_TRUE);
+    widget->CaptureRollupEvents((nsIRollupListener *)this, mDroppedDown, aShowList);
 
 }
 
@@ -947,7 +946,7 @@ nsComboboxControlFrame::ReflowCombobox(nsIPresContext *         aPresContext,
     aDropDownBtn->SetRect(buttonRect);
     SetChildFrameSize(aDropDownBtn, aBtnWidth, aDesiredSize.height);
     aDesiredSize.width = 0;
-    aDesiredSize.height = dispHeight;
+    aDesiredSize.height = dispHeight + aBorderPadding.top + aBorderPadding.bottom;
     // XXX What about ascent and descent?
     return;
   }
@@ -2124,13 +2123,11 @@ nsComboboxControlFrame::CreateAnonymousContent(nsIPresContext* aPresContext,
     mDisplayContent = do_QueryInterface(labelContent);
     mDisplayContent->SetText(NS_LITERAL_STRING("X"), PR_TRUE);
 
-    nsCOMPtr<nsIDocument> doc;
-    mContent->GetDocument(getter_AddRefs(doc));
+    nsCOMPtr<nsIDocument> doc = mContent->GetDocument();
     // mContent->AppendChildTo(labelContent, PR_FALSE, PR_FALSE);
 
-    nsCOMPtr<nsINodeInfoManager> nimgr;
-    result = doc->GetNodeInfoManager(getter_AddRefs(nimgr));
-    NS_ENSURE_SUCCESS(result, result);
+    nsINodeInfoManager *nimgr = doc->GetNodeInfoManager();
+    NS_ENSURE_TRUE(nimgr, NS_ERROR_FAILURE);
 
     nsCOMPtr<nsINodeInfo> nodeInfo;
     nimgr->GetNodeInfo(nsHTMLAtoms::input, nsnull, kNameSpaceID_None,
@@ -2304,7 +2301,7 @@ nsComboboxControlFrame::SetInitialChildList(nsIPresContext* aPresContext,
       nsIFormControlFrame* fcFrame = nsnull;
       CallQueryInterface(child, &fcFrame);
       if (fcFrame) {
-        if (fcFrame->GetType() == NS_FORM_INPUT_BUTTON) {
+        if (fcFrame->GetFormControlType() == NS_FORM_INPUT_BUTTON) {
           mButtonFrame = child;
         }
       } else {
@@ -2405,7 +2402,7 @@ nsComboboxControlFrame::Paint(nsIPresContext*     aPresContext,
     nsAreaFrame::Paint(aPresContext, aRenderingContext, aDirtyRect,
                        NS_FRAME_PAINT_LAYER_BACKGROUND);
     nsAreaFrame::Paint(aPresContext, aRenderingContext, aDirtyRect,
-                       NS_FRAME_PAINT_LAYER_FLOATERS);
+                       NS_FRAME_PAINT_LAYER_FLOATS);
     nsAreaFrame::Paint(aPresContext, aRenderingContext, aDirtyRect,
                        NS_FRAME_PAINT_LAYER_FOREGROUND);
 
