@@ -61,6 +61,7 @@
 #include "nsIAccessibilityService.h"
 #endif
 #include "nsIServiceManager.h"
+#include "nsSpaceManager.h"
 
 class nsLegendFrame;
 
@@ -141,10 +142,7 @@ NS_NewFieldSetFrame(nsIPresShell* aPresShell, nsIFrame** aNewFrame, PRUint32 aSt
   }
 
   // set the state flags (if any are provided)
-  nsFrameState state;
-  it->GetFrameState( &state );
-  state |= aStateFlags;
-  it->SetFrameState( state );
+  it->AddStateBits(aStateFlags);
   
   *aNewFrame = it;
   return NS_OK;
@@ -176,7 +174,7 @@ nsFieldSetFrame::SetInitialChildList(nsIPresContext* aPresContext,
  
   // get the content and legend frames.
   mContentFrame = aChildList;
-  mContentFrame->GetNextSibling(&mLegendFrame);
+  mLegendFrame = mContentFrame->GetNextSibling();
 
   // Queue up the frames for the content frame
   return nsHTMLContainerFrame::SetInitialChildList(aPresContext, nsnull, aChildList);
@@ -196,10 +194,8 @@ nsFieldSetFrame::Paint(nsIPresContext*      aPresContext,
     if (NS_SUCCEEDED(IsVisibleForPainting(aPresContext, aRenderingContext, PR_TRUE, &isVisible)) && 
                      isVisible && mRect.width && mRect.height) {
       PRIntn skipSides = GetSkipSides();
-      const nsStyleBorder* borderStyle = 
-        (const nsStyleBorder*)mStyleContext->GetStyleData(eStyleStruct_Border);
-      const nsStylePadding* paddingStyle = 
-        (const nsStylePadding*)mStyleContext->GetStyleData(eStyleStruct_Padding);
+      const nsStyleBorder* borderStyle = GetStyleBorder();
+      const nsStylePadding* paddingStyle = GetStylePadding();
        
         nsMargin border;
         if (!borderStyle->GetBorder(border)) {
@@ -278,9 +274,7 @@ nsFieldSetFrame::Paint(nsIPresContext*      aPresContext,
 
 #ifdef DEBUG
   if ((NS_FRAME_PAINT_LAYER_DEBUG == aWhichLayer) && GetShowFrameBorders()) {
-    nsIView* view;
-    GetView(aPresContext, &view);
-    if (nsnull != view) {
+    if (HasView()) {
       aRenderingContext.SetColor(NS_RGB(0,0,255));
     }
     else {
@@ -304,7 +298,17 @@ nsFieldSetFrame::Reflow(nsIPresContext*          aPresContext,
   DISPLAY_REFLOW(aPresContext, this, aReflowState, aDesiredSize, aStatus);
 
   // Initialize OUT parameter
-   aStatus = NS_FRAME_COMPLETE;
+  aStatus = NS_FRAME_COMPLETE;
+
+  // Should we create a space manager?
+  nsAutoSpaceManager autoSpaceManager(NS_CONST_CAST(nsHTMLReflowState &, aReflowState));
+
+  // XXXldb If we start storing the space manager in the frame rather
+  // than keeping it around only during reflow then we should create it
+  // only when there are actually floats to manage.  Otherwise things
+  // like tables will gain significant bloat.
+  if (NS_BLOCK_SPACE_MGR & mState)
+    autoSpaceManager.CreateSpaceManagerFor(aPresContext, this);
 
   if (aDesiredSize.mComputeMEW) {
     aDesiredSize.mMaxElementWidth = 0;
@@ -364,17 +368,17 @@ nsFieldSetFrame::Reflow(nsIPresContext*          aPresContext,
     // if dirty then check dirty flags
     if (reason == eReflowReason_Dirty) 
     {
-        if (reflowContent) {
-              nsFrameState state;
-              mContentFrame->GetFrameState(&state);
-              reflowContent = (state & NS_FRAME_IS_DIRTY) || (state & NS_FRAME_HAS_DIRTY_CHILDREN);
-        }
+      if (reflowContent) {
+        reflowContent =
+          (mContentFrame->GetStateBits()
+           & (NS_FRAME_IS_DIRTY | NS_FRAME_HAS_DIRTY_CHILDREN)) != 0;
+      }
 
-        if (reflowLegend) {
-              nsFrameState state;
-              mLegendFrame->GetFrameState(&state);
-              reflowLegend = (state & NS_FRAME_IS_DIRTY) || (state & NS_FRAME_HAS_DIRTY_CHILDREN);
-        }
+      if (reflowLegend) {
+        reflowLegend =
+          (mLegendFrame->GetStateBits()
+           & (NS_FRAME_IS_DIRTY | NS_FRAME_HAS_DIRTY_CHILDREN)) != 0;
+      }
     }
 
     // availSize could have unconstrained values, don't perform any addition on them
@@ -390,9 +394,7 @@ nsFieldSetFrame::Reflow(nsIPresContext*          aPresContext,
     nsMargin legendMargin(0,0,0,0);
     // reflow the legend only if needed
     if (mLegendFrame) {
-        const nsStyleMargin* marginStyle;
-        mLegendFrame->GetStyleData(eStyleStruct_Margin,
-                              (const nsStyleStruct*&) marginStyle);
+        const nsStyleMargin* marginStyle = mLegendFrame->GetStyleMargin();
         marginStyle->GetMargin(legendMargin);
 
         if (reflowLegend) {
@@ -510,13 +512,10 @@ nsFieldSetFrame::Reflow(nsIPresContext*          aPresContext,
               aDesiredSize.mMaxElementWidth = kidDesiredSize.mMaxElementWidth;
             }
 
-            nsFrameState  kidState;
-            mContentFrame->GetFrameState(&kidState);
-
            // printf("width: %d, height: %d\n", desiredSize.mCombinedArea.width, desiredSize.mCombinedArea.height);
 
             /*
-            if (kidState & NS_FRAME_OUTSIDE_CHILDREN) {
+            if (mContentFrame->GetStateBits() & NS_FRAME_OUTSIDE_CHILDREN) {
                  mState |= NS_FRAME_OUTSIDE_CHILDREN;
                  aDesiredSize.mOverflowArea.width += borderPadding.left + borderPadding.right;
                  aDesiredSize.mOverflowArea.height += borderPadding.top + borderPadding.bottom + mLegendSpace;
@@ -527,10 +526,8 @@ nsFieldSetFrame::Reflow(nsIPresContext*          aPresContext,
 
         } else {
             // if we don't need to reflow just get the old size
-            mContentFrame->GetRect(contentRect);
-            const nsStyleMargin* marginStyle;
-            mContentFrame->GetStyleData(eStyleStruct_Margin,
-                                  (const nsStyleStruct*&) marginStyle);
+            contentRect = mContentFrame->GetRect();
+            const nsStyleMargin* marginStyle = mContentFrame->GetStyleMargin();
 
             nsMargin m(0,0,0,0);
             marginStyle->GetMargin(m);
@@ -567,12 +564,11 @@ nsFieldSetFrame::Reflow(nsIPresContext*          aPresContext,
       nsRect actualLegendRect(mLegendRect);
       actualLegendRect.Deflate(legendMargin);
 
-      nsPoint curOrigin;
-      mLegendFrame->GetOrigin(curOrigin);
+      nsPoint curOrigin = mLegendFrame->GetPosition();
 
       // only if the origin changed
       if ((curOrigin.x != mLegendRect.x) || (curOrigin.y != mLegendRect.y)) {
-          mLegendFrame->MoveTo(aPresContext,  actualLegendRect.x , actualLegendRect.y);
+          mLegendFrame->SetPosition(nsPoint(actualLegendRect.x , actualLegendRect.y));
           nsContainerFrame::PositionFrameView(aPresContext, mLegendFrame);
 
           // We need to recursively process the legend frame's
@@ -605,6 +601,12 @@ nsFieldSetFrame::Reflow(nsIPresContext*          aPresContext,
         // add in padding.
         aDesiredSize.mMaxElementWidth += borderPadding.left + borderPadding.right;
     }
+    aDesiredSize.mOverflowArea = nsRect(0, 0, aDesiredSize.width, aDesiredSize.height);
+    if (mLegendFrame)
+      ConsiderChildOverflow(aPresContext, aDesiredSize.mOverflowArea, mLegendFrame);
+    if (mContentFrame)
+      ConsiderChildOverflow(aPresContext, aDesiredSize.mOverflowArea, mContentFrame);
+    StoreOverflow(aPresContext, aDesiredSize);
 #ifdef NOISY_REFLOW
     printf("FIELDSET:  w=%d, maxWidth=%d, MEW=%d\n",
            aDesiredSize.width, aDesiredSize.mMaximumWidth, 
@@ -658,16 +660,10 @@ nsFieldSetFrame::RemoveFrame(nsIPresContext* aPresContext,
   // XXX XXX
   // XXX temporary fix for bug 70648
   if (aOldFrame == mLegendFrame) {   
-    nsIFrame* sibling;
-    mContentFrame->GetNextSibling(&sibling);
+    nsIFrame* sibling = mContentFrame->GetNextSibling();
     NS_ASSERTION(sibling == mLegendFrame, "legendFrame is not next sibling");
-#ifdef DEBUG
-    nsIFrame* legendParent;
-    mLegendFrame->GetParent(&legendParent);
-    NS_ASSERTION(legendParent == this, "Legend Parent has wrong parent");
-#endif
-    nsIFrame* legendSibling;
-    sibling->GetNextSibling(&legendSibling);
+    NS_ASSERTION(mLegendFrame->GetParent() == this, "Legend Parent has wrong parent");
+    nsIFrame* legendSibling = sibling->GetNextSibling();
     // replace the legend, which is the next sibling, with any siblings of the legend (XXX always null?)
     mContentFrame->SetNextSibling(legendSibling);
     // OK, the legend is now removed from the sibling list, but who has ownership of it?
