@@ -231,7 +231,7 @@ nsGfxCheckboxControlFrame::Paint(nsIPresContext*   aPresContext,
     if (!mCheckButtonFaceStyle && GetCheckboxState()) {
       const nsStyleBackground* myColor = mCheckButtonFaceStyle->GetStyleBackground();
 
-      if (myColor->mBackgroundImage.Length() > 0) {
+      if (myColor->mBackgroundImage) {
         const nsStyleBorder* myBorder = mCheckButtonFaceStyle->GetStyleBorder();
         const nsStylePadding* myPadding = mCheckButtonFaceStyle->GetStylePadding();
         const nsStylePosition* myPosition = mCheckButtonFaceStyle->GetStylePosition();
