@@ -186,8 +186,7 @@ NS_IMETHODIMP nsHTMLButtonControlFrame::GetAccessible(nsIAccessible** aAccessibl
   nsCOMPtr<nsIAccessibilityService> accService = do_GetService("@mozilla.org/accessibilityService;1");
 
   if (accService) {
-    nsCOMPtr<nsIContent> content;
-    GetContent(getter_AddRefs(content));
+    nsIContent* content = GetContent();
     nsCOMPtr<nsIDOMHTMLButtonElement> buttonElement(do_QueryInterface(content));
     if (buttonElement) //If turned XBL-base form control off, the frame contains HTML 4 button
       return accService->CreateHTML4ButtonAccessible(NS_STATIC_CAST(nsIFrame*, this), aAccessible);
@@ -202,7 +201,7 @@ NS_IMETHODIMP nsHTMLButtonControlFrame::GetAccessible(nsIAccessible** aAccessibl
 
 
 NS_IMETHODIMP_(PRInt32)
-nsHTMLButtonControlFrame::GetType() const
+nsHTMLButtonControlFrame::GetFormControlType() const
 {
   return nsFormControlHelper::GetType(mContent);
 }
@@ -219,6 +218,22 @@ nsHTMLButtonControlFrame::GetValue(nsAString* aResult)
   return nsFormControlHelper::GetValueAttr(mContent, aResult);
 }
 
+void
+nsHTMLButtonControlFrame::ReParentFrameList(nsIFrameManager* aFrameManager,
+                                            nsIFrame* aFrameList)
+{
+  // get the new parent context from the first child: that is the
+  // frame that the subsequent children will be made children of
+  nsStyleContext* newParentContext = mFrames.FirstChild()->GetStyleContext();
+
+  // Set the parent for each of the child frames
+  for (nsIFrame* frame = aFrameList; frame; frame = frame->GetNextSibling()) {
+    frame->SetParent(mFrames.FirstChild());
+    // now reparent the contexts for the reparented frame too
+    aFrameManager->ReParentStyleContext(frame, newParentContext);
+  }
+}
+
 PRBool
 nsHTMLButtonControlFrame::IsReset(PRInt32 type)
 {
@@ -272,11 +287,9 @@ nsHTMLButtonControlFrame::GetTranslatedRect(nsIPresContext* aPresContext, nsRect
   nsIView* view;
   nsPoint viewOffset(0,0);
   GetOffsetFromView(aPresContext, viewOffset, &view);
-  while (nsnull != view) {
-    nsPoint tempOffset;
-    view->GetPosition(&tempOffset.x, &tempOffset.y);
-    viewOffset += tempOffset;
-    view->GetParent(view);
+  while (view) {
+    viewOffset += view->GetPosition();
+    view = view->GetParent();
   }
   aRect = nsRect(viewOffset.x, viewOffset.y, mRect.width, mRect.height);
 }
@@ -320,31 +333,9 @@ nsHTMLButtonControlFrame::SetInitialChildList(nsIPresContext* aPresContext,
                                               nsIAtom*        aListName,
                                               nsIFrame*       aChildList)
 {
-  // get the frame manager and the style context of the new parent frame
-  // this is used whent he children are reparented below
   // NOTE: the whole reparenting should not need to happen: see bugzilla bug 51767
-  //
-  nsCOMPtr<nsIPresShell> shell;
-  nsCOMPtr<nsIFrameManager> frameManager;
-  nsStyleContext* newParentContext;
-  aPresContext->GetShell(getter_AddRefs(shell));
-  if (shell) {
-    shell->GetFrameManager(getter_AddRefs(frameManager));
-  }
-  // get the new parent context from the first child: that is the frame that the
-  // subsequent children will be made children of
-  newParentContext = mFrames.FirstChild()->GetStyleContext();
-
-  // Set the parent for each of the child frames
-  for (nsIFrame* frame = aChildList; nsnull != frame; frame->GetNextSibling(&frame)) {
-    frame->SetParent(mFrames.FirstChild());
-    // now reparent the contexts for the reparented frame too
-    if (frameManager) {
-      frameManager->ReParentStyleContext(aPresContext,frame,newParentContext);
-    }
-  }
-
-  // Queue up the frames for the inline frame
+  ReParentFrameList(aPresContext->GetFrameManager(), aChildList);
+  
   return mFrames.FirstChild()->SetInitialChildList(aPresContext, nsnull, aChildList);
 }
 
@@ -366,7 +357,7 @@ nsHTMLButtonControlFrame::Paint(nsIPresContext*      aPresContext,
 
 #if 0 // old way
   PaintChildren(aPresContext, aRenderingContext, aDirtyRect, NS_FRAME_PAINT_LAYER_BACKGROUND);
-  PaintChildren(aPresContext, aRenderingContext, aDirtyRect, NS_FRAME_PAINT_LAYER_FLOATERS);
+  PaintChildren(aPresContext, aRenderingContext, aDirtyRect, NS_FRAME_PAINT_LAYER_FLOATS);
   PaintChildren(aPresContext, aRenderingContext, aDirtyRect, NS_FRAME_PAINT_LAYER_FOREGROUND);
 
 #else // temporary
@@ -387,7 +378,7 @@ nsHTMLButtonControlFrame::Paint(nsIPresContext*      aPresContext,
   aRenderingContext.SetClipRect(rect, nsClipCombine_kIntersect, clipEmpty);
 
   PaintChildren(aPresContext, aRenderingContext, aDirtyRect, NS_FRAME_PAINT_LAYER_BACKGROUND);
-  PaintChildren(aPresContext, aRenderingContext, aDirtyRect, NS_FRAME_PAINT_LAYER_FLOATERS);
+  PaintChildren(aPresContext, aRenderingContext, aDirtyRect, NS_FRAME_PAINT_LAYER_FLOATS);
   PaintChildren(aPresContext, aRenderingContext, aDirtyRect, NS_FRAME_PAINT_LAYER_FOREGROUND);
 
   aRenderingContext.PopState(clipEmpty);
@@ -440,19 +431,12 @@ nsHTMLButtonControlFrame::Reflow(nsIPresContext* aPresContext,
 #if 0
   if (!mDidInit) {
     // create our view, we need a view to grab the mouse 
-    nsIView* view;
-    GetView(&view);
+    nsIView* view = GetView();
     if (!view) {
       nsresult result = nsComponentManager::CreateInstance(kViewCID, nsnull, NS_GET_IID(nsIView), (void **)&view);
-      nsCOMPtr<nsIPresShell> presShell;
-      aPresContext->GetShell(getter_AddRefs(presShell));
-      nsCOMPtr<nsIViewManager> viewMan;
-      presShell->GetViewManager(getter_AddRefs(viewMan));
-
-      nsIFrame* parWithView;
-      nsIView *parView;
-      GetParentWithView(&parWithView);
-      parWithView->GetView(&parView);
+      nsIViewManager* viewMan = aPresContext->GetViewManager();
+      nsIFrame* parWithView = GetAncestorWithView();
+      nsIView* parView = parWithView->GetView();
       // the view's size is not know yet, but its size will be kept in synch with our frame.
       nsRect boundBox(0, 0, 500, 500); 
       result = view->Init(viewMan, boundBox, parView, nsnull);
@@ -460,8 +444,7 @@ nsHTMLButtonControlFrame::Reflow(nsIPresContext* aPresContext,
       SetView(view);
 
       // set the opacity
-      viewMan->SetViewOpacity(view, GetStyleColor()->mOpacity);
-
+      viewMan->SetViewOpacity(view, GetStyleDisplay()->mOpacity);
     }
     mDidInit = PR_TRUE;
   }
@@ -652,12 +635,9 @@ nsHTMLButtonControlFrame::GetFont(nsIPresContext* aPresContext,
 NS_IMETHODIMP
 nsHTMLButtonControlFrame::GetFormContent(nsIContent*& aContent) const
 {
-  nsIContent* content;
-  nsresult    rv;
-
-  rv = GetContent(&content);
-  aContent = content;
-  return rv;
+  aContent = GetContent();
+  NS_IF_ADDREF(aContent);
+  return NS_OK;
 }
 
 nscoord 
@@ -736,10 +716,53 @@ nsHTMLButtonControlFrame::AppendFrames(nsIPresContext* aPresContext,
                                        nsIAtom*        aListName,
                                        nsIFrame*       aFrameList)
 {
+  ReParentFrameList(aPresContext->GetFrameManager(), aFrameList);
   return mFrames.FirstChild()->AppendFrames(aPresContext,
-                                     aPresShell,
-                                     aListName,
-                                     aFrameList);
+                                            aPresShell,
+                                            aListName,
+                                            aFrameList);
+}
+
+NS_IMETHODIMP
+nsHTMLButtonControlFrame::InsertFrames(nsIPresContext* aPresContext,
+                                       nsIPresShell&   aPresShell,
+                                       nsIAtom*        aListName,
+                                       nsIFrame*       aPrevFrame,
+                                       nsIFrame*       aFrameList)
+{
+  ReParentFrameList(aPresContext->GetFrameManager(), aFrameList);
+  return mFrames.FirstChild()->InsertFrames(aPresContext,
+                                            aPresShell,
+                                            aListName,
+                                            aPrevFrame,
+                                            aFrameList);
+}
+
+NS_IMETHODIMP
+nsHTMLButtonControlFrame::RemoveFrame(nsIPresContext* aPresContext,
+                                      nsIPresShell&   aPresShell,
+                                      nsIAtom*        aListName,
+                                      nsIFrame*       aOldFrame)
+{
+  return mFrames.FirstChild()->RemoveFrame(aPresContext,
+                                           aPresShell,
+                                           aListName,
+                                           aOldFrame);
+}
+
+NS_IMETHODIMP
+nsHTMLButtonControlFrame::ReplaceFrame(nsIPresContext* aPresContext,
+                                       nsIPresShell&   aPresShell,
+                                       nsIAtom*        aListName,
+                                       nsIFrame*       aOldFrame,
+                                       nsIFrame*       aNewFrame)
+{
+  ReParentFrameList(aPresContext->GetFrameManager(), aNewFrame);
+  return mFrames.FirstChild()->ReplaceFrame(aPresContext,
+                                            aPresShell,
+                                            aListName,
+                                            aOldFrame,
+                                            aNewFrame);
 }
 
 NS_IMETHODIMP
