@@ -122,6 +122,10 @@ public:
   // from nsIAnonymousContentCreator
   NS_IMETHOD CreateAnonymousContent(nsIPresContext* aPresContext,
                                     nsISupportsArray& aChildList);
+  NS_IMETHOD CreateFrameFor(nsIPresContext*   aPresContext,
+                               nsIContent *      aContent,
+                               nsIFrame**        aFrame);
+  virtual void PostCreateFrames();
 
   // Utility methods to set current widget state
   void SetValue(const nsAString& aValue);
@@ -179,8 +183,7 @@ public:
                               nsIContent*     aChild,
                               PRInt32         aNameSpaceID,
                               nsIAtom*        aAttribute,
-                              PRInt32         aModType, 
-                              PRInt32         aHint);
+                              PRInt32         aModType);
 
   NS_IMETHOD GetText(nsString* aText);
 
@@ -280,10 +283,6 @@ protected:
                                  nsSize&               aDesiredSize,
                                  nsSize&               aMinSize);
 
-  NS_IMETHOD CreateFrameFor(nsIPresContext*   aPresContext,
-                               nsIContent *      aContent,
-                               nsIFrame**        aFrame);
-
   PRInt32 GetWidthInCharacters() const;
 
   // nsIScrollableViewProvider
