@@ -212,8 +212,7 @@ nsTableCellFrame::AttributeChanged(nsIPresContext* aPresContext,
                                    nsIContent*     aChild,
                                    PRInt32         aNameSpaceID,
                                    nsIAtom*        aAttribute,
-                                   PRInt32         aModType, 
-                                   PRInt32         aHint)
+                                   PRInt32         aModType)
 {
   // let the table frame decide what to do
   nsTableFrame* tableFrame = nsnull; 
@@ -346,7 +345,7 @@ nsTableCellFrame::DecorateForSelection(nsIPresContext* aPresContext,
         result = frameSelection->GetTableCellSelection(&tableCellSelectionMode);
         if (NS_SUCCEEDED(result) && tableCellSelectionMode) {
           nscolor       bordercolor;
-          if(displaySelection == nsISelectionController::SELECTION_DISABLED) {
+          if (displaySelection == nsISelectionController::SELECTION_DISABLED) {
             bordercolor = NS_RGB(176,176,176);// disabled color
           }
           else {
@@ -470,29 +469,31 @@ nsTableCellFrame::Paint(nsIPresContext*      aPresContext,
     PRBool clipState;
     nsPoint offset;
     GetCollapseOffset(aPresContext, offset);
+    PRBool pushed = PR_FALSE;
     if ((0 != offset.x) || (0 != offset.y)) {
       aRenderingContext.PushState();
+      pushed = PR_TRUE;
       aRenderingContext.Translate(offset.x, offset.y);
       aRenderingContext.SetClipRect(nsRect(-offset.x, -offset.y, mRect.width, mRect.height),
                                     nsClipCombine_kIntersect, clipState);
     }
     else {
-      if ((NS_STYLE_OVERFLOW_HIDDEN == disp->mOverflow) || HasPctOverHeight()) {
+      if (NS_STYLE_OVERFLOW_HIDDEN == disp->mOverflow ||
+          // XXXldb SCROLLBARS_NONE should really create a scrollframe,
+          // but test here since it doesn't.
+          NS_STYLE_OVERFLOW_SCROLLBARS_NONE == disp->mOverflow ||
+          HasPctOverHeight()) {
         aRenderingContext.PushState();
+        pushed = PR_TRUE;
         SetOverflowClipRect(aRenderingContext);
       }    
     }
 
     PaintChildren(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer, aFlags);
 
-    if ((0 != offset.x) || (0 != offset.y)) {
+    if (pushed) {
       aRenderingContext.PopState(clipState);
     }
-    else { 
-      if ((NS_STYLE_OVERFLOW_HIDDEN == disp->mOverflow) || HasPctOverHeight()) {
-        aRenderingContext.PopState(clipState);         
-      }
-    }
   } 
   
   DO_GLOBAL_REFLOW_COUNT_DSP_J("nsTableCellFrame", &aRenderingContext, 0);
@@ -643,6 +644,12 @@ void nsTableCellFrame::VerticallyAlignChild(nsIPresContext*          aPresContex
       kidYTop = nsTableFrame::RoundToPixel(kidYTop, p2t, eAlwaysRoundDown);
   }
   firstKid->SetPosition(nsPoint(kidRect.x, kidYTop));
+  nsHTMLReflowMetrics desiredSize(PR_FALSE);
+  desiredSize.width = mRect.width;
+  desiredSize.height = mRect.height;
+  desiredSize.mOverflowArea = nsRect(0, 0, mRect.width, mRect.height);
+  ConsiderChildOverflow(aPresContext, desiredSize.mOverflowArea, firstKid);
+  StoreOverflow(aPresContext, desiredSize);
   if (kidYTop != kidRect.y) {
     // Make sure any child views are correctly positioned. We know the inner table
     // cell won't have a view
@@ -1019,6 +1026,8 @@ NS_METHOD nsTableCellFrame::Reflow(nsIPresContext*          aPresContext,
 
   aDesiredSize.ascent  += kidSize.ascent;
   aDesiredSize.descent += kidSize.descent;
+  
+  // the overflow area will be computed when the child will be vertically aligned
 
   if (aDesiredSize.mComputeMEW) {
     aDesiredSize.mMaxElementWidth =
@@ -1097,108 +1106,6 @@ PRBool nsTableCellFrame::ConvertToPixelValue(nsHTMLValue& aValue, PRInt32 aDefau
   return PR_TRUE;
 }
 
-void nsTableCellFrame::MapBorderPadding(nsIPresContext* aPresContext)
-{
-  // Check to see if the table has cell padding or defined for the table. If true, 
-  // then this setting overrides any specific border or padding information in the 
-  // cell. If these attributes are not defined, the the cells attributes are used
-
-  nsTableFrame* tableFrame;
-  nsTableFrame::GetTableFrame(this, tableFrame);
-  NS_ASSERTION(tableFrame,"Table must not be null");
-  if (!tableFrame)
-    return;
-
-  // get the table frame style context, and from it get cellpadding, cellspacing, and border info
-  const nsStyleTable* tableStyle = tableFrame->GetStyleTable();
-
-  MapVAlignAttribute(aPresContext, tableFrame);
-  MapHAlignAttribute(aPresContext, tableFrame);
-  
-}
-
-/* XXX: this code will not work properly until the style and layout code has been updated 
- * as outlined in Bugzilla bug report 1802 and 915 */
-void nsTableCellFrame::MapVAlignAttribute(nsIPresContext* aPresContext, nsTableFrame *aTableFrame)
-{
-#if 0
-  const nsStyleTextReset* textStyle = GetStyleTextReset();
-  // check if valign is set on the cell
-  // this condition will also be true if we inherited valign from the row or rowgroup
-  if (textStyle->mVerticalAlign.GetUnit() == eStyleUnit_Enumerated) {
-    return; // valign is already set on this cell
-  }
-
-  // check if valign is set on the cell's COL (or COLGROUP by inheritance)
-  PRInt32 colIndex;
-  GetColIndex(colIndex);
-  nsTableColFrame* colFrame = aTableFrame->GetColFrame(colIndex);
-  if (colFrame) {
-    const nsStyleTextReset* colTextStyle = colFrame->GetStyleTextReset();
-    if (colTextStyle->mVerticalAlign.GetUnit() == eStyleUnit_Enumerated) {
-      nsStyleTextReset* cellTextStyle = (nsStyleTextReset*)mStyleContext->GetMutableStyleData(eStyleStruct_TextReset);
-      cellTextStyle->mVerticalAlign.SetIntValue(colTextStyle->mVerticalAlign.GetIntValue(), eStyleUnit_Enumerated);
-      return; // valign set from COL info
-    }
-  }
-
-  // otherwise, set the vertical align attribute to the HTML default
-  nsStyleTextReset* cellTextStyle = (nsStyleTextReset*)mStyleContext->GetMutableStyleData(eStyleStruct_TextReset);
-  cellTextStyle->mVerticalAlign.SetIntValue(NS_STYLE_VERTICAL_ALIGN_MIDDLE, eStyleUnit_Enumerated);
-#endif
-}
-
-/* XXX: this code will not work properly until the style and layout code has been updated 
- * as outlined in Bugzilla bug report 1802 and 915.
- * In particular, mTextAlign has to be an nsStyleCoord, not just an int */
-void nsTableCellFrame::MapHAlignAttribute(nsIPresContext* aPresContext, 
-                                          nsTableFrame*   aTableFrame)
-{
-#if 0
-  const nsStyleText* textStyle = GetStyleText();
-  // check if halign is set on the cell
-  // cells do not inherited halign from the row or rowgroup
-  if (NS_STYLE_TEXT_ALIGN_DEFAULT != textStyle->mTextAlign) {
-    return; // text align is already set on this cell
-  }
-
-  // check if halign is set on the cell's ROW (or ROWGROUP by inheritance)
-  nsIFrame* rowFrame = GetParent();
-  const nsStyleText* rowTextStyle = rowFrame->GetStyleText();
-  if (NS_STYLE_TEXT_ALIGN_DEFAULT != rowTextStyle->mTextAlign) {
-    nsStyleText* cellTextStyle = (nsStyleText*)mStyleContext->GetMutableStyleData(eStyleStruct_Text);
-    cellTextStyle->mTextAlign = rowTextStyle->mTextAlign;
-    return; // halign set from ROW info
-  }
-
-  // check if halign is set on the cell's COL (or COLGROUP by inheritance)
-  PRInt32 colIndex;
-  GetColIndex(colIndex);
-  nsTableColFrame* colFrame = aTableFrame->GetColFrame(colIndex);
-  if (colFrame) {
-    const nsStyleText* colTextStyle = colFrame->GetStyleText();
-    if (NS_STYLE_TEXT_ALIGN_DEFAULT != colTextStyle->mTextAlign) {
-      nsStyleText* cellTextStyle = (nsStyleText*)mStyleContext->GetMutableStyleData(eStyleStruct_Text);
-      cellTextStyle->mTextAlign = colTextStyle->mTextAlign;
-      return; // halign set from COL info
-    }
-  }
-
-  // otherwise, set the text align to the HTML default (center for TH, left for TD)
-  nsStyleText* cellTextStyle = (nsStyleText*)mStyleContext->GetMutableStyleData(eStyleStruct_Text);
-  nsIAtom* tag;
-  if (mContent) {
-    mContent->GetTag(tag);
-    if (tag) {
-      cellTextStyle->mTextAlign = (nsHTMLAtoms::th == tag)
-                                  ? NS_STYLE_TEXT_ALIGN_CENTER 
-                                  : NS_STYLE_TEXT_ALIGN_LEFT;
-    }
-    NS_IF_RELEASE(tag);
-  }
-#endif
-}  
-
 /* ----- global methods ----- */
 
 NS_IMPL_ADDREF_INHERITED(nsTableCellFrame, nsHTMLContainerFrame)
