@@ -55,12 +55,12 @@
 #define COL_GROUP_TYPE_OFFSET        30
 
 nsTableColGroupType 
-nsTableColGroupFrame::GetType() const 
+nsTableColGroupFrame::GetColType() const 
 {
   return (nsTableColGroupType)((mState & COL_GROUP_TYPE_BITS) >> COL_GROUP_TYPE_OFFSET);
 }
 
-void nsTableColGroupFrame::SetType(nsTableColGroupType aType) 
+void nsTableColGroupFrame::SetColType(nsTableColGroupType aType) 
 {
   PRUint32 type = aType - eColGroupContent;
   mState |= (type << COL_GROUP_TYPE_OFFSET);
@@ -74,9 +74,7 @@ void nsTableColGroupFrame::ResetColIndices(nsIPresContext* aPresContext,
   nsTableColGroupFrame* colGroupFrame = (nsTableColGroupFrame*)aFirstColGroup;
   PRInt32 colIndex = aFirstColIndex;
   while (colGroupFrame) {
-    nsCOMPtr<nsIAtom> cgType;
-    colGroupFrame->GetFrameType(getter_AddRefs(cgType));
-    if (nsLayoutAtoms::tableColGroupFrame == cgType.get()) {
+    if (nsLayoutAtoms::tableColGroupFrame == colGroupFrame->GetType()) {
       // reset the starting col index for the first cg only if
       // aFirstColIndex is smaller than the existing starting col index
       if ((colIndex != aFirstColIndex) ||
@@ -88,16 +86,15 @@ void nsTableColGroupFrame::ResetColIndices(nsIPresContext* aPresContext,
         colGroupFrame->FirstChild(aPresContext, nsnull, &colFrame);
       }
       while (colFrame) {
-        nsCOMPtr<nsIAtom> colType;
-        colFrame->GetFrameType(getter_AddRefs(colType));
-        if (nsLayoutAtoms::tableColFrame == colType.get()) {
+        if (nsLayoutAtoms::tableColFrame == colFrame->GetType()) {
           ((nsTableColFrame*)colFrame)->SetColIndex(colIndex);
           colIndex++;
         }
-        colFrame->GetNextSibling(&colFrame);
+        colFrame = colFrame->GetNextSibling();
       }
     }
-    colGroupFrame->GetNextSibling((nsIFrame**)&colGroupFrame);
+    colGroupFrame = NS_STATIC_CAST(nsTableColGroupFrame*,
+                                   colGroupFrame->GetNextSibling());
   }
 }
 
@@ -119,9 +116,7 @@ nsTableColGroupFrame::AddColsToTable(nsIPresContext&  aPresContext,
   nsIFrame* kidFrame = aFirstFrame;
   PRBool foundLastFrame = PR_FALSE;
   while (kidFrame) {
-    nsIAtom* kidType;
-    kidFrame->GetFrameType(&kidType);
-    if (nsLayoutAtoms::tableColFrame == kidType) {
+    if (nsLayoutAtoms::tableColFrame == kidFrame->GetType()) {
       ((nsTableColFrame*)kidFrame)->SetColIndex(colIndex);
       if (!foundLastFrame) {
         mColCount++;
@@ -129,19 +124,14 @@ nsTableColGroupFrame::AddColsToTable(nsIPresContext&  aPresContext,
       }
       colIndex++;
     }
-    NS_IF_RELEASE(kidType);
     if (kidFrame == aLastFrame) {
       foundLastFrame = PR_TRUE;
     }
-    kidFrame->GetNextSibling(&kidFrame); 
+    kidFrame = kidFrame->GetNextSibling(); 
   }
 
-  if (aResetSubsequentColIndices) {
-    nsIFrame* nextSibling;
-    GetNextSibling(&nextSibling);
-    if (nextSibling) {
-      ResetColIndices(&aPresContext, nextSibling, colIndex);
-    }
+  if (aResetSubsequentColIndices && GetNextSibling()) {
+    ResetColIndices(&aPresContext, GetNextSibling(), colIndex);
   }
 
   return rv;
@@ -154,15 +144,15 @@ nsTableColGroupFrame::FindParentForAppendedCol(nsTableFrame*  aTableFrame,
 {
   nsVoidArray& cols = aTableFrame->GetColCache();
   PRInt32 numCols = cols.Count();
-  nsIFrame* lastColGroup;
   if (numCols == 0) return nsnull; // no columns so no colgroups
   nsIFrame* lastCol = (nsIFrame*)cols.ElementAt(numCols - 1);
   NS_ASSERTION(lastCol,"null entry in column array");
-  lastCol->GetParent(&lastColGroup);
+  nsIFrame* lastColGroup = lastCol->GetParent();
   if (!lastColGroup) return nsnull; // shouldn't happen
  
-  nsTableColGroupFrame* relevantColGroup = (nsTableColGroupFrame *)lastColGroup;
-  nsTableColGroupType relevantColGroupType = relevantColGroup->GetType();
+  nsTableColGroupFrame* relevantColGroup =
+    NS_STATIC_CAST(nsTableColGroupFrame *, lastColGroup);
+  nsTableColGroupType relevantColGroupType = relevantColGroup->GetColType();
   if (eColGroupAnonymousCell == relevantColGroupType) {
     if (eColAnonymousCell == aColType) {
       return relevantColGroup;
@@ -171,9 +161,9 @@ nsTableColGroupFrame::FindParentForAppendedCol(nsTableFrame*  aTableFrame,
       // find the next to last col group
       for (PRInt32 colX = numCols - 2; colX >= 0; colX--) {
         nsTableColFrame* colFrame = (nsTableColFrame*)cols.ElementAt(colX);
-        nsTableColGroupFrame* colGroupFrame;
-        colFrame->GetParent((nsIFrame**)&colGroupFrame);
-        nsTableColGroupType cgType = colGroupFrame->GetType();
+        nsTableColGroupFrame* colGroupFrame =
+          NS_STATIC_CAST(nsTableColGroupFrame*, colFrame->GetParent());
+        nsTableColGroupType cgType = colGroupFrame->GetColType();
         if (cgType != relevantColGroupType) {
           relevantColGroup = colGroupFrame;
           relevantColGroupType = cgType;
@@ -205,8 +195,7 @@ nsTableColGroupFrame::GetLastRealColGroup(nsTableFrame* aTableFrame,
   nsIFrame* nextToLastColGroup = nsnull;
   nsIFrame* lastColGroup       = colGroups.FirstChild();
   while(lastColGroup) {
-    nsIFrame* next;
-    lastColGroup->GetNextSibling(&next);
+    nsIFrame* next = lastColGroup->GetNextSibling();
     if (next) {
       nextToLastColGroup = lastColGroup;
       lastColGroup = next;
@@ -218,7 +207,8 @@ nsTableColGroupFrame::GetLastRealColGroup(nsTableFrame* aTableFrame,
 
   if (!lastColGroup) return PR_TRUE; // there are no col group frames
  
-  nsTableColGroupType lastColGroupType = ((nsTableColGroupFrame *)lastColGroup)->GetType();
+  nsTableColGroupType lastColGroupType =
+    ((nsTableColGroupFrame *)lastColGroup)->GetColType();
   if (eColGroupAnonymousCell == lastColGroupType) {
     *aLastColGroup = nextToLastColGroup;
     return PR_FALSE;
@@ -313,7 +303,7 @@ nsTableColGroupFrame::RemoveChild(nsIPresContext&  aPresContext,
   nsIFrame* nextChild = nsnull;
   if (aResetColIndices) {
     colIndex = aChild.GetColIndex();
-    aChild.GetNextSibling(&nextChild);
+    nextChild = aChild.GetNextSibling();
   }
   if (mFrames.DestroyFrame(&aPresContext, (nsIFrame*)&aChild)) {
     mColCount--;
@@ -346,20 +336,16 @@ nsTableColGroupFrame::RemoveChildrenAtEnd(nsIPresContext& aPresContext,
   PRInt32 offsetX = 0;
   nsIFrame* kidFrame = mFrames.FirstChild();
   while(kidFrame) {
-    nsIAtom* kidType;
-    kidFrame->GetFrameType(&kidType);
-    if (nsLayoutAtoms::tableColFrame == kidType) {
+    if (nsLayoutAtoms::tableColFrame == kidFrame->GetType()) {
       offsetX++;
       if (offsetX > offsetOfFirstRemoval) {
         nsIFrame* byebye = kidFrame;
-        kidFrame->GetNextSibling(&kidFrame);
+        kidFrame = kidFrame->GetNextSibling();
         mFrames.DestroyFrame(&aPresContext, byebye);
-        NS_RELEASE(kidType);
         continue;
       }
     }
-    NS_IF_RELEASE(kidType);
-    kidFrame->GetNextSibling(&kidFrame);
+    kidFrame = kidFrame->GetNextSibling();
   }
 }
 
@@ -372,9 +358,7 @@ nsTableColGroupFrame::RemoveFrame(nsIPresContext* aPresContext,
 {
   if (!aOldFrame) return NS_OK;
 
-  nsIAtom* frameType = nsnull;
-  aOldFrame->GetFrameType(&frameType);
-  if (nsLayoutAtoms::tableColFrame == frameType) {
+  if (nsLayoutAtoms::tableColFrame == aOldFrame->GetType()) {
     nsTableColFrame* colFrame = (nsTableColFrame*)aOldFrame;
     PRInt32 colIndex = colFrame->GetColIndex();
     RemoveChild(*aPresContext, *colFrame, PR_TRUE);
@@ -393,7 +377,6 @@ nsTableColGroupFrame::RemoveFrame(nsIPresContext* aPresContext,
   else {
     mFrames.DestroyFrame(aPresContext, aOldFrame);
   }
-  NS_IF_RELEASE(frameType);
 
   return NS_OK;
 }
@@ -456,8 +439,8 @@ NS_METHOD nsTableColGroupFrame::Reflow(nsIPresContext*          aPresContext,
     rv = IncrementalReflow(aPresContext, aDesiredSize, aReflowState, aStatus);
   }
 
-  for (kidFrame = mFrames.FirstChild(); nsnull != kidFrame;
-       kidFrame->GetNextSibling(&kidFrame)) {
+  for (kidFrame = mFrames.FirstChild(); kidFrame;
+       kidFrame = kidFrame->GetNextSibling()) {
     // Give the child frame a chance to reflow, even though we know it'll have 0 size
     nsHTMLReflowMetrics kidSize(nsnull);
     // XXX Use a valid reason...
@@ -598,7 +581,7 @@ nsTableColFrame * nsTableColGroupFrame::GetNextColumn(nsIFrame *aChildFrame)
       result = (nsTableColFrame *)childFrame;
       break;
     }
-    childFrame->GetNextSibling(&childFrame);
+    childFrame = childFrame->GetNextSibling();
   }
   return result;
 }
@@ -619,7 +602,7 @@ nsTableColFrame * nsTableColGroupFrame::GetColumnAt (PRInt32 aColIndex)
         result = col;
       }
     }
-    childFrame->GetNextSibling(&childFrame);
+    childFrame = childFrame->GetNextSibling();
   }
 
   return result;
@@ -628,14 +611,13 @@ nsTableColFrame * nsTableColGroupFrame::GetColumnAt (PRInt32 aColIndex)
 PRInt32 nsTableColGroupFrame::GetSpan()
 {
   PRInt32 span = 1;
-  nsCOMPtr<nsIContent> iContent;
-  nsresult rv = GetContent(getter_AddRefs(iContent));
-  if (NS_FAILED(rv) || !iContent) return rv;
+  nsIContent* iContent = GetContent();
+  if (!iContent) return NS_OK;
 
   // col group element derives from col element
   nsIDOMHTMLTableColElement* cgContent = nsnull;
-  rv = iContent->QueryInterface(NS_GET_IID(nsIDOMHTMLTableColElement), 
-                               (void **)&cgContent);  
+  nsresult rv = iContent->QueryInterface(NS_GET_IID(nsIDOMHTMLTableColElement), 
+                                         (void **)&cgContent);
   if (cgContent && NS_SUCCEEDED(rv)) { 
     cgContent->GetSpan(&span);
     // XXX why does this work!!
@@ -671,21 +653,17 @@ nsTableColGroupFrame::GetColGroupFrameContaining(nsIPresContext*  aPresContext,
 {
   nsIFrame* childFrame = aColGroupList.FirstChild();
   while (childFrame) { 
-    nsIAtom* frameType = nsnull;
-    childFrame->GetFrameType(&frameType);
-    if (nsLayoutAtoms::tableColGroupFrame == frameType) {
+    if (nsLayoutAtoms::tableColGroupFrame == childFrame->GetType()) {
       nsTableColFrame* colFrame = nsnull;
       childFrame->FirstChild(aPresContext, nsnull, (nsIFrame **)&colFrame);
       while (colFrame) {
         if (colFrame == &aColFrame) {
-          NS_RELEASE(frameType);
           return (nsTableColGroupFrame *)childFrame;
         }
-        colFrame->GetNextSibling((nsIFrame **)&colFrame);
+        colFrame = NS_STATIC_CAST(nsTableColFrame*, colFrame->GetNextSibling());
       }
     }
-    NS_IF_RELEASE(frameType);
-    childFrame->GetNextSibling(&childFrame);
+    childFrame = childFrame->GetNextSibling();
   }
   return nsnull;
 }
@@ -731,13 +709,10 @@ nsTableColGroupFrame::Init(nsIPresContext*  aPresContext,
   return rv;
 }
 
-NS_IMETHODIMP
-nsTableColGroupFrame::GetFrameType(nsIAtom** aType) const
+nsIAtom*
+nsTableColGroupFrame::GetType() const
 {
-  NS_PRECONDITION(nsnull != aType, "null OUT parameter pointer");
-  *aType = nsLayoutAtoms::tableColGroupFrame; 
-  NS_ADDREF(*aType);
-  return NS_OK;
+  return nsLayoutAtoms::tableColGroupFrame;
 }
 
 #ifdef DEBUG
