@@ -213,27 +213,6 @@ nsTableRowFrame::Init(nsIPresContext*  aPresContext,
 }
 
 
-NS_IMETHODIMP
-nsTableRowFrame::SetInitialChildList(nsIPresContext* aPresContext,
-                                     nsIAtom*        aListName,
-                                     nsIFrame*       aChildList)
-{
-  nsresult rv = nsHTMLContainerFrame::SetInitialChildList(aPresContext, aListName, aChildList);
-  // see if the row has a cell with a row span > 1
-  for (nsIFrame* kidFrame = mFrames.FirstChild(); 
-       kidFrame && !(mState & NS_FRAME_OUTSIDE_CHILDREN); 
-       kidFrame->GetNextSibling(&kidFrame)) {
-    nsCOMPtr<nsIAtom> frameType;
-    kidFrame->GetFrameType(getter_AddRefs(frameType));
-    if (IS_TABLE_CELL(frameType.get())) {
-      if (((nsTableCellFrame*)kidFrame)->GetRowSpan() > 1) {
-        mState |= NS_FRAME_OUTSIDE_CHILDREN;
-      }
-    }
-  }
-  return rv;
-}
-
 NS_IMETHODIMP
 nsTableRowFrame::AppendFrames(nsIPresContext* aPresContext,
                               nsIPresShell&   aPresShell,
@@ -246,7 +225,8 @@ nsTableRowFrame::AppendFrames(nsIPresContext* aPresContext,
   // Add the new cell frames to the table
   nsTableFrame *tableFrame = nsnull;
   nsTableFrame::GetTableFrame(this, tableFrame);
-  for (nsIFrame* childFrame = aFrameList; childFrame; childFrame->GetNextSibling(&childFrame)) {
+  for (nsIFrame* childFrame = aFrameList; childFrame;
+       childFrame = childFrame->GetNextSibling()) {
     nsCOMPtr<nsIAtom> frameType;
     childFrame->GetFrameType(getter_AddRefs(frameType));
     if (IS_TABLE_CELL(frameType.get())) {
@@ -254,9 +234,6 @@ nsTableRowFrame::AppendFrames(nsIPresContext* aPresContext,
       tableFrame->AppendCell(*aPresContext, (nsTableCellFrame&)*childFrame, GetRowIndex());
       // XXX this could be optimized with some effort
       tableFrame->SetNeedStrategyInit(PR_TRUE);
-      if (!(mState & NS_FRAME_OUTSIDE_CHILDREN) && (((nsTableCellFrame*)childFrame)->GetRowSpan() > 1)) {
-        mState |= NS_FRAME_OUTSIDE_CHILDREN;
-      }
     }
   }
 
@@ -283,16 +260,14 @@ nsTableRowFrame::InsertFrames(nsIPresContext* aPresContext,
   nsIAtom* cellFrameType = (tableFrame->IsBorderCollapse()) ? nsLayoutAtoms::bcTableCellFrame : nsLayoutAtoms::tableCellFrame;
   nsTableCellFrame* prevCellFrame = (nsTableCellFrame *)nsTableFrame::GetFrameAtOrBefore(aPresContext, this, aPrevFrame, cellFrameType);
   nsVoidArray cellChildren;
-  for (nsIFrame* childFrame = aFrameList; childFrame; childFrame->GetNextSibling(&childFrame)) {
+  for (nsIFrame* childFrame = aFrameList; childFrame;
+       childFrame = childFrame->GetNextSibling()) {
     nsCOMPtr<nsIAtom> frameType;
     childFrame->GetFrameType(getter_AddRefs(frameType));
     if (IS_TABLE_CELL(frameType.get())) {
       cellChildren.AppendElement(childFrame);
       // XXX this could be optimized with some effort
       tableFrame->SetNeedStrategyInit(PR_TRUE);
-      if (!(mState & NS_FRAME_OUTSIDE_CHILDREN) && (((nsTableCellFrame*)childFrame)->GetRowSpan() > 1)) {
-        mState |= NS_FRAME_OUTSIDE_CHILDREN;
-      }
     }
   }
   // insert the cells into the cell map
@@ -356,20 +331,16 @@ GetHeightOfRowsSpannedBelowFirst(nsTableCellFrame& aTableCellFrame,
   nscoord cellSpacingY = aTableFrame.GetCellSpacingY();
   PRInt32 rowSpan = aTableFrame.GetEffectiveRowSpan(aTableCellFrame);
   // add in height of rows spanned beyond the 1st one
-  nsIFrame* nextRow = nsnull;
-  aTableCellFrame.GetParent((nsIFrame **)&nextRow);
-  nextRow->GetNextSibling(&nextRow);
+  nsIFrame* nextRow = aTableCellFrame.GetParent()->GetNextSibling();
   for (PRInt32 rowX = 1; ((rowX < rowSpan) && nextRow);) {
     nsCOMPtr<nsIAtom> frameType;
     nextRow->GetFrameType(getter_AddRefs(frameType));
     if (nsLayoutAtoms::tableRowFrame == frameType.get()) {
-      nsRect rect;
-      nextRow->GetRect(rect);
-      height += rect.height;
+      height += nextRow->GetSize().height;
       rowX++;
     }
     height += cellSpacingY;
-    nextRow->GetNextSibling(&nextRow);
+    nextRow = nextRow->GetNextSibling();
   }
   return height;
 }
@@ -384,7 +355,7 @@ nsTableRowFrame::GetFirstCell()
     if (IS_TABLE_CELL(frameType.get())) {
       return (nsTableCellFrame*)childFrame;
     }
-    childFrame->GetNextSibling(&childFrame);
+    childFrame = childFrame->GetNextSibling();
   }
   return nsnull;
 }
@@ -400,9 +371,15 @@ nsTableRowFrame::DidResize(nsIPresContext*          aPresContext,
   nsTableFrame* tableFrame;
   nsTableFrame::GetTableFrame(this, tableFrame);
   if (!tableFrame) return;
-
+  
   nsTableIterator iter(aPresContext, *this, eTableDIR);
   nsIFrame* childFrame = iter.First();
+  
+  nsHTMLReflowMetrics desiredSize(PR_FALSE);
+  desiredSize.width = mRect.width;
+  desiredSize.height = mRect.height;
+  desiredSize.mOverflowArea = nsRect(0, 0, desiredSize.width,
+                                      desiredSize.height);
 
   while (childFrame) {
     nsCOMPtr<nsIAtom> frameType;
@@ -412,14 +389,12 @@ nsTableRowFrame::DidResize(nsIPresContext*          aPresContext,
       nscoord cellHeight = mRect.height + GetHeightOfRowsSpannedBelowFirst(*cellFrame, *tableFrame);
 
       // resize the cell's height
-      nsSize  cellFrameSize;
-      cellFrame->GetSize(cellFrameSize);
       //if (cellFrameSize.height!=cellHeight)
       {
         // XXX If the cell frame has a view, then we need to resize
         // it as well. We would like to only do that if the cell's size
         // is changing. Why is the 'if' stmt above commented out?
-        cellFrame->SizeTo(aPresContext, cellFrameSize.width, cellHeight);
+        cellFrame->SetSize(nsSize(cellFrame->GetSize().width, cellHeight));
         // realign cell content based on the new height
         /*nsHTMLReflowMetrics desiredSize(nsnull);
         nsHTMLReflowState kidReflowState(aPresContext, aReflowState,
@@ -433,12 +408,13 @@ nsTableRowFrame::DidResize(nsIPresContext*          aPresContext,
         //ReflowChild(cellFrame, aPresContext, desiredSize, kidReflowState, status);
 
         cellFrame->VerticallyAlignChild(aPresContext, aReflowState, mMaxCellAscent);
+        ConsiderChildOverflow(aPresContext, desiredSize.mOverflowArea, cellFrame);
       }
     }
     // Get the next child
     childFrame = iter.Next();
   }
-
+  StoreOverflow(aPresContext, desiredSize);
   // Let our base class do the usual work
 }
 
@@ -540,7 +516,8 @@ nsTableRowFrame::CalcHeight(const nsHTMLReflowState& aReflowState)
     SetPctHeight(position->mHeight.GetPercentValue());
   }
 
-  for (nsIFrame* kidFrame = mFrames.FirstChild(); kidFrame; kidFrame->GetNextSibling(&kidFrame)) {
+  for (nsIFrame* kidFrame = mFrames.FirstChild(); kidFrame;
+       kidFrame = kidFrame->GetNextSibling()) {
     nsCOMPtr<nsIAtom> frameType;
     kidFrame->GetFrameType(getter_AddRefs(frameType));
     if (IS_TABLE_CELL(frameType.get())) {
@@ -566,9 +543,8 @@ PRBool IsFirstRow(nsIPresContext*  aPresContext,
 {
   nsIFrame* firstRowGroup = nsnull;
   aTable.FirstChild(aPresContext, nsnull, &firstRowGroup);
-  nsIFrame* rowGroupFrame = nsnull;
-  nsresult rv = aRow.GetParent(&rowGroupFrame);
-  if (NS_SUCCEEDED(rv) && (rowGroupFrame == firstRowGroup)) {
+  nsIFrame* rowGroupFrame = aRow.GetParent();
+  if (rowGroupFrame == firstRowGroup) {
     nsIFrame* firstRow;
     rowGroupFrame->FirstChild(aPresContext, nsnull, &firstRow);
     return (&aRow == firstRow);
@@ -624,48 +600,6 @@ nsTableRowFrame::GetSkipSides() const
   return skip;
 }
 
-/** overloaded method from nsContainerFrame.  The difference is that 
-  * we don't want to clip our children, so a cell can do a rowspan
-  */
-void nsTableRowFrame::PaintChildren(nsIPresContext*      aPresContext,
-                                    nsIRenderingContext& aRenderingContext,
-                                    const nsRect&        aDirtyRect,
-                                    nsFramePaintLayer    aWhichLayer,
-                                    PRUint32             aFlags)
-{
-  nsIFrame* kid = mFrames.FirstChild();
-  while (nsnull != kid) {
-    nsIView *pView;
-     
-    kid->GetView(aPresContext, &pView);
-    if (nsnull == pView) {
-      nsRect kidRect;
-      kid->GetRect(kidRect);
-      nsRect damageArea;
-      PRBool overlap = damageArea.IntersectRect(aDirtyRect, kidRect);
-      if (overlap) {
-        PRBool clipState;
-        // Translate damage area into kid's coordinate system
-        nsRect kidDamageArea(damageArea.x - kidRect.x,
-                             damageArea.y - kidRect.y,
-                             damageArea.width, damageArea.height);
-        aRenderingContext.PushState();
-        aRenderingContext.Translate(kidRect.x, kidRect.y);
-        kid->Paint(aPresContext, aRenderingContext, kidDamageArea,
-                   aWhichLayer, aFlags);
-#ifdef DEBUG
-        if ((NS_FRAME_PAINT_LAYER_DEBUG == aWhichLayer) &&
-            GetShowFrameBorders()) {
-          aRenderingContext.SetColor(NS_RGB(255,0,0));
-          aRenderingContext.DrawRect(0, 0, kidRect.width, kidRect.height);
-        }
-#endif
-        aRenderingContext.PopState(clipState);
-      }
-    }
-    kid->GetNextSibling(&kid);
-  }
-}
 
 /* we overload this here because rows have children that can span outside of themselves.
  * so the default "get the child rect, see if it contains the event point" action isn't
@@ -691,8 +625,6 @@ nsTableRowFrame::GetFrameForPoint(nsIPresContext*        aPresContext,
   nsIFrame *kid, *hit;
   nsPoint tmp;
 
-  PRBool inThisFrame = mRect.Contains(aPoint);
-
   FirstChild(aPresContext, nsnull, &kid);
   *aFrame = nsnull;
   tmp.MoveTo(aPoint.x - mRect.x, aPoint.y - mRect.y);
@@ -702,7 +634,7 @@ nsTableRowFrame::GetFrameForPoint(nsIPresContext*        aPresContext,
     if (NS_SUCCEEDED(rv) && hit) {
       *aFrame = hit;
     }
-    kid->GetNextSibling(&kid);
+    kid = kid->GetNextSibling();
   }
 
   if (*aFrame) {
@@ -793,11 +725,10 @@ CalcAvailWidth(nsTableFrame&     aTableFrame,
   } 
   aCellAvailWidth = aColAvailWidth;
 
-  nsFrameState  frameState;
-  aCellFrame.GetFrameState(&frameState);
   // for a cell with a colspan > 1, use its fix width (if set) as the avail width 
   // if this is its initial reflow
-  if ((frameState & NS_FRAME_FIRST_REFLOW) && (aTableFrame.GetEffectiveColSpan(aCellFrame) > 1)) {
+  if ((aCellFrame.GetStateBits() & NS_FRAME_FIRST_REFLOW)
+      && (aTableFrame.GetEffectiveColSpan(aCellFrame) > 1)) {
     // see if the cell has a style width specified
     const nsStylePosition* cellPosition = aCellFrame.GetStylePosition();
     if (eStyleUnit_Coord == cellPosition->mWidth.GetUnit()) {
@@ -865,13 +796,13 @@ nscoord CalcHeightFromUnpaginatedHeight(nsIPresContext*  aPresContext,
                                         nsTableRowFrame& aRow)
 {
   nscoord height = 0;
-  nsTableRowFrame* firstInFlow = (nsTableRowFrame*)aRow.GetFirstInFlow(); if (!firstInFlow) ABORT1(0);
+  nsTableRowFrame* firstInFlow = (nsTableRowFrame*)aRow.GetFirstInFlow();
+  if (!firstInFlow) ABORT1(0);
   if (firstInFlow->HasUnpaginatedHeight()) {
     height = firstInFlow->GetUnpaginatedHeight(aPresContext);
-    for (nsIFrame* prevInFlow = aRow.GetPrevInFlow(); prevInFlow; prevInFlow->GetPrevInFlow(&prevInFlow)) {
-      nsRect rect;
-      prevInFlow->GetRect(rect);
-      height -= rect.height;
+    for (nsIFrame* prevInFlow = aRow.GetPrevInFlow(); prevInFlow;
+         prevInFlow->GetPrevInFlow(&prevInFlow)) {
+      height -= prevInFlow->GetSize().height;
     }
   }
   return PR_MAX(height, 0);
@@ -933,16 +864,12 @@ nsTableRowFrame::ReflowChildren(nsIPresContext*          aPresContext,
   // Reflow each of our existing cell frames
   nsIFrame* kidFrame = iter.First();
   while (kidFrame) {
-    // Get the frame state bits
-    nsFrameState  frameState;
-    kidFrame->GetFrameState(&frameState);
-    
     nsCOMPtr<nsIAtom> frameType;
     kidFrame->GetFrameType(getter_AddRefs(frameType));
 
     // See if we should only reflow the dirty child frames
     PRBool doReflowChild = PR_TRUE;
-    if (aDirtyOnly && ((frameState & NS_FRAME_IS_DIRTY) == 0)) {
+    if (aDirtyOnly && ((kidFrame->GetStateBits() & NS_FRAME_IS_DIRTY) == 0)) {
       doReflowChild = PR_FALSE;
     }
     else if ((NS_UNCONSTRAINEDSIZE != aReflowState.availableHeight) && IS_TABLE_CELL(frameType)) {
@@ -984,7 +911,7 @@ nsTableRowFrame::ReflowChildren(nsIPresContext*          aPresContext,
 
         // remember the rightmost (ltr) or leftmost (rtl) column this cell spans into
         prevColIndex = (iter.IsLeftToRight()) ? cellColIndex + (cellColSpan - 1) : cellColIndex;
-        nsHTMLReflowMetrics desiredSize(nsnull);
+        nsHTMLReflowMetrics desiredSize(PR_FALSE);
   
         // If the avail width is not the same as last time we reflowed the cell or
         // the cell wants to be bigger than what was available last time or
@@ -1008,7 +935,7 @@ nsTableRowFrame::ReflowChildren(nsIPresContext*          aPresContext,
           nsReflowReason reason = eReflowReason_Resize;
           PRBool cellToWatch = PR_FALSE;
           // If it's a dirty frame, then check whether it's the initial reflow
-          if (frameState & NS_FRAME_FIRST_REFLOW) {
+          if (kidFrame->GetStateBits() & NS_FRAME_FIRST_REFLOW) {
             reason = eReflowReason_Initial;
             cellToWatch = PR_TRUE;
           }
@@ -1076,7 +1003,7 @@ nsTableRowFrame::ReflowChildren(nsIPresContext*          aPresContext,
           nsSize priorSize = cellFrame->GetDesiredSize();
           desiredSize.width = priorSize.width;
           desiredSize.height = priorSize.height;
-
+          
           // if we are in a floated table, our position is not yet established, so we cannot reposition our views
           // the containing glock will do this for us after positioning the table
           if (!aTableFrame.GetStyleDisplay()->IsFloating()) {
@@ -1114,13 +1041,14 @@ nsTableRowFrame::ReflowChildren(nsIPresContext*          aPresContext,
         }
 
         FinishReflowChild(kidFrame, aPresContext, nsnull, desiredSize, x, 0, 0);
+        
         x += desiredSize.width;  
       }
       else {// it's an unknown frame type, give it a generic reflow and ignore the results
         nsTableCellReflowState kidReflowState(aPresContext, aReflowState,
                                               kidFrame, nsSize(0,0), eReflowReason_Resize);
         InitChildReflowState(*aPresContext, nsSize(0,0), PR_FALSE, p2t, kidReflowState);
-        nsHTMLReflowMetrics desiredSize(nsnull);
+        nsHTMLReflowMetrics desiredSize(PR_FALSE);
         nsReflowStatus  status;
         ReflowChild(kidFrame, aPresContext, desiredSize, kidReflowState, 0, 0, 0, status);
         kidFrame->DidReflow(aPresContext, nsnull, NS_FRAME_REFLOW_FINISHED);
@@ -1128,11 +1056,9 @@ nsTableRowFrame::ReflowChildren(nsIPresContext*          aPresContext,
     }
     else if (IS_TABLE_CELL(frameType.get())) {
       // we need to account for the cell's width even if it isn't reflowed
-      nsRect rect;
-      kidFrame->GetRect(rect);
-      x += rect.width;
+      x += kidFrame->GetSize().width;
     }
-
+    ConsiderChildOverflow(aPresContext, aDesiredSize.mOverflowArea, kidFrame);
     kidFrame = iter.Next(); // Get the next child
     // if this was the last child, and it had a colspan>1, add in the cellSpacing for the colspan
     // if the last kid wasn't a colspan, then we still have the colspan of the last real cell
@@ -1171,7 +1097,9 @@ nsTableRowFrame::ReflowChildren(nsIPresContext*          aPresContext,
       aDesiredSize.height = PR_MIN(aDesiredSize.height, aReflowState.availableHeight);
     }
   }
-
+  nsRect rowRect(0, 0, aDesiredSize.width, aDesiredSize.height);
+  aDesiredSize.mOverflowArea.UnionRect(aDesiredSize.mOverflowArea, rowRect);
+  StoreOverflow(aPresContext, aDesiredSize);
   return rv;
 }
 
@@ -1262,8 +1190,7 @@ nsTableRowFrame::IR_TargetIsChild(nsIPresContext*          aPresContext,
   if (NS_STYLE_DISPLAY_TABLE_CELL == childDisplay->mDisplay) {
     nsTableCellFrame* cellFrame = (nsTableCellFrame*)aNextFrame;
     // Get the x coord of the cell
-    nsPoint cellOrigin;
-    cellFrame->GetOrigin(cellOrigin);
+    nsPoint cellOrigin = cellFrame->GetPosition();
 
     // At this point, we know the column widths. Compute the cell available width
     PRInt32 cellColIndex;
@@ -1393,9 +1320,10 @@ nsTableRowFrame::IR_TargetIsChild(nsIPresContext*          aPresContext,
     if (!aDesiredSize.mNothingChanged) {
       if (aDesiredSize.height == mRect.height) { // our height didn't change
         cellFrame->VerticallyAlignChild(aPresContext, aReflowState, mMaxCellAscent);
-        nsRect dirtyRect;
-        cellFrame->GetRect(dirtyRect);
+        nsRect dirtyRect = cellFrame->GetRect();
         dirtyRect.height = mRect.height;
+        ConsiderChildOverflow(aPresContext, aDesiredSize.mOverflowArea, cellFrame);
+        dirtyRect.UnionRect(dirtyRect, aDesiredSize.mOverflowArea);
         Invalidate(aPresContext, dirtyRect);
       }
     }
@@ -1410,7 +1338,8 @@ nsTableRowFrame::IR_TargetIsChild(nsIPresContext*          aPresContext,
   // can't just return the status of the cell frame we just reflowed...
   aStatus = NS_FRAME_COMPLETE;
   if (mNextInFlow) {
-    for (nsIFrame* cell = mFrames.FirstChild(); cell; cell->GetNextSibling(&cell)) {
+    for (nsIFrame* cell = mFrames.FirstChild(); cell;
+         cell = cell->GetNextSibling()) {
       nsIFrame* contFrame;
   
       cell->GetNextInFlow(&contFrame);
@@ -1500,15 +1429,13 @@ nsTableRowFrame::Contains(nsIPresContext* aPresContext,
     nsIFrame* kid;
     FirstChild(aPresContext, nsnull, &kid);
     while (nsnull != kid) {
-      nsRect kidRect;
-      kid->GetRect(kidRect);
       nsPoint point(aPoint);
       point.MoveBy(-mRect.x, -mRect.y); // offset the point to check by the row container
-      if (kidRect.Contains(point)) {
+      if (kid->GetRect().Contains(point)) {
         result = PR_TRUE;
         break;
       }
-      kid->GetNextSibling(&kid);
+      kid = kid->GetNextSibling();
     }
   }
   return result;
@@ -1530,8 +1457,7 @@ nsTableRowFrame::ReflowCellFrame(nsIPresContext*          aPresContext,
   nsTableFrame::GetTableFrame(this, tableFrame); if (!tableFrame) ABORT1(0);
 
   // Reflow the cell frame with the specified height. Use the existing width
-  nsSize  cellSize;
-  aCellFrame->GetSize(cellSize);
+  nsSize cellSize = aCellFrame->GetSize();
   
   nsSize  availSize(cellSize.width, aAvailableHeight);
   PRBool borderCollapse = ((nsTableFrame*)tableFrame->GetFirstInFlow())->IsBorderCollapse();
@@ -1540,14 +1466,14 @@ nsTableRowFrame::ReflowCellFrame(nsIPresContext*          aPresContext,
                                          eReflowReason_Resize);
   InitChildReflowState(*aPresContext, availSize, borderCollapse, p2t, cellReflowState);
 
-  nsHTMLReflowMetrics desiredSize(nsnull);
+  nsHTMLReflowMetrics desiredSize(PR_FALSE);
 
   ReflowChild(aCellFrame, aPresContext, desiredSize, cellReflowState,
               0, 0, NS_FRAME_NO_MOVE_FRAME, aStatus);
   PRBool fullyComplete = NS_FRAME_IS_COMPLETE(aStatus) && !NS_FRAME_IS_TRUNCATED(aStatus);
 
-  aCellFrame->SizeTo(aPresContext, cellSize.width, 
-                     (fullyComplete) ? aAvailableHeight : desiredSize.height);
+  aCellFrame->SetSize(
+    nsSize(cellSize.width, fullyComplete ? aAvailableHeight : desiredSize.height));
 
   // XXX What happens if this cell has 'vertical-align: baseline' ?
   // XXX Why is it assumed that the cell's ascent hasn't changed ?
@@ -1577,7 +1503,8 @@ nsTableRowFrame::InsertCellFrame(nsTableCellFrame* aFrame,
 {
   // Find the cell frame where col index < aColIndex
   nsTableCellFrame* priorCell = nsnull;
-  for (nsIFrame* child = mFrames.FirstChild(); child; child->GetNextSibling(&child)) {
+  for (nsIFrame* child = mFrames.FirstChild(); child;
+       child = child->GetNextSibling()) {
     nsCOMPtr<nsIAtom> frameType;
     child->GetFrameType(getter_AddRefs(frameType));
     if (!IS_TABLE_CELL(frameType.get())) {
@@ -1612,15 +1539,14 @@ nsTableRowFrame::GetFrameType(nsIAtom** aType) const
 nsTableRowFrame*  
 nsTableRowFrame::GetNextRow() const
 {
-  nsIFrame* childFrame;
-  GetNextSibling(&childFrame);
+  nsIFrame* childFrame = GetNextSibling();
   while (childFrame) {
     nsCOMPtr<nsIAtom> frameType;
     childFrame->GetFrameType(getter_AddRefs(frameType));
     if (nsLayoutAtoms::tableRowFrame == frameType.get()) {
       return (nsTableRowFrame*)childFrame;
     }
-    childFrame->GetNextSibling(&childFrame);
+    childFrame = childFrame->GetNextSibling();
   }
   return nsnull;
 }
