@@ -377,7 +377,7 @@ nsTableRowGroupFrame::ReflowChildren(nsIPresContext*        aPresContext,
       nsSize kidAvailSize(aReflowState.availSize);
       if (0 >= kidAvailSize.height)
         kidAvailSize.height = 1;      // XXX: HaCk - we don't handle negative heights yet
-      nsHTMLReflowMetrics desiredSize(nsnull);
+      nsHTMLReflowMetrics desiredSize(PR_FALSE);
       desiredSize.width = desiredSize.height = desiredSize.ascent = desiredSize.descent = 0;
   
       // Reflow the child into the available space, giving it as much height as
@@ -442,6 +442,7 @@ nsTableRowGroupFrame::ReflowChildren(nsIPresContext*        aPresContext,
       // Adjust the running y-offset so we know where the next row should be placed
       aReflowState.y += kidFrame->GetSize().height + cellSpacingY;
     }
+    ConsiderChildOverflow(aPresContext, aDesiredSize.mOverflowArea, kidFrame);
   }
 
   // adjust the rows after the ones that were reflowed
@@ -503,6 +504,7 @@ UpdateHeights(RowInfo& aRowInfo,
 void 
 nsTableRowGroupFrame::DidResizeRows(nsIPresContext&          aPresContext,
                                     const nsHTMLReflowState& aReflowState,
+                                    nsHTMLReflowMetrics&     aDesiredSize,
                                     nsTableRowFrame*         aStartRowFrameIn)
 {
   // update the cells spanning rows with their new heights
@@ -510,8 +512,13 @@ nsTableRowGroupFrame::DidResizeRows(nsIPresContext&          aPresContext,
   PRInt32 rowIndex;
   nsTableRowFrame* rowFrame;
   nsTableRowFrame* startRowFrame = (aStartRowFrameIn) ? aStartRowFrameIn: GetFirstRow();
+  if (!aStartRowFrameIn || startRowFrame == GetFirstRow()) {
+    // Reset the overflow area
+    aDesiredSize.mOverflowArea = nsRect(0, 0, 0, 0);
+  }
   for (rowFrame = startRowFrame, rowIndex = 0; rowFrame; rowFrame = rowFrame->GetNextRow(), rowIndex++) {
     rowFrame->DidResize(&aPresContext, aReflowState);
+    ConsiderChildOverflow(&aPresContext, aDesiredSize.mOverflowArea, rowFrame);
   }
 }
 
@@ -842,7 +849,7 @@ nsTableRowGroupFrame::CalculateRowHeights(nsIPresContext*          aPresContext,
     CacheRowHeightsForPrinting(aPresContext, GetFirstRow());
   }
 
-  DidResizeRows(*aPresContext, aReflowState, startRowFrame);
+  DidResizeRows(*aPresContext, aReflowState, aDesiredSize, startRowFrame);
 
   aDesiredSize.height = rowGroupHeight; // Adjust our desired size
   delete [] rowInfo; // cleanup
@@ -1068,7 +1075,7 @@ nsTableRowGroupFrame::SplitRowGroup(nsIPresContext*          aPresContext,
                                          eReflowReason_Resize);
         InitChildReflowState(*aPresContext, borderCollapse, p2t, rowReflowState);
         rowReflowState.mFlags.mIsTopOfPage = isTopOfPage; // set top of page
-        nsHTMLReflowMetrics rowMetrics(nsnull);
+        nsHTMLReflowMetrics rowMetrics(PR_FALSE);
 
         // Reflow the cell with the constrained height. A cell with rowspan >1 will get this
         // reflow later during SplitSpanningCells.
@@ -1287,7 +1294,7 @@ nsTableRowGroupFrame::Reflow(nsIPresContext*          aPresContext,
     // but we need to correctly calculate the row group height and we can't if there
     // are row spans unless we do this step
     if (aReflowState.mFlags.mSpecialHeightReflow) {
-      DidResizeRows(*aPresContext, aReflowState);
+      DidResizeRows(*aPresContext, aReflowState, aDesiredSize);
       if (isPaginated) {
         CacheRowHeightsForPrinting(aPresContext, GetFirstRow());
       }
@@ -1325,6 +1332,9 @@ nsTableRowGroupFrame::Reflow(nsIPresContext*          aPresContext,
     aDesiredSize.height = GetHeightOfRows(aPresContext);
   }
 
+  aDesiredSize.mOverflowArea.UnionRect(aDesiredSize.mOverflowArea, nsRect(0, 0, aDesiredSize.width,
+	                                                                      aDesiredSize.height)); 
+  StoreOverflow(aPresContext, aDesiredSize);
 #if defined DEBUG_TABLE_REFLOW_TIMING
   nsTableFrame::DebugReflow(this, (nsHTMLReflowState&)aReflowState, &aDesiredSize, aStatus);
 #endif
@@ -1711,6 +1721,8 @@ nsTableRowGroupFrame::IR_TargetIsChild(nsIPresContext*        aPresContext,
       } else {
         // Inform the row of its new height.
         ((nsTableRowFrame*)aNextFrame)->DidResize(aPresContext, aReflowState.reflowState);
+        // the overflow area may have changed inflate the overflow area
+        ConsiderChildOverflow(aPresContext, aDesiredSize.mOverflowArea, aNextFrame);
         if (aReflowState.tableFrame->IsAutoHeight()) {
           // Because other cells in the row may need to be be aligned differently,
           // repaint the entire row
@@ -1853,7 +1865,7 @@ nsTableRowGroupFrame::GetBCBorderWidth(float     aPixelsToTwips,
   nsTableRowFrame* firstRowFrame = nsnull;
   nsTableRowFrame* lastRowFrame = nsnull;
   for (nsTableRowFrame* rowFrame = GetFirstRow(); rowFrame; rowFrame = rowFrame->GetNextRow()) {
-    if(!firstRowFrame) {
+    if (!firstRowFrame) {
       firstRowFrame = rowFrame;
     }
     lastRowFrame = rowFrame;
@@ -1899,21 +1911,21 @@ nsTableRowGroupFrame::GetLine(PRInt32    aLineNumber,
     return NS_ERROR_FAILURE;
 
   nsTableCellMap* cellMap = parentFrame->GetCellMap();
-  if(!cellMap)
+  if (!cellMap)
      return NS_ERROR_FAILURE;
 
-  if(aLineNumber >= cellMap->GetRowCount())
+  if (aLineNumber >= cellMap->GetRowCount())
     return NS_ERROR_INVALID_ARG;
   
   *aLineFlags = 0;/// should we fill these in later?
   // not gonna touch aLineBounds right now
 
   CellData* firstCellData = cellMap->GetDataAt(aLineNumber, 0);
-  if(!firstCellData) 
+  if (!firstCellData)
     return NS_ERROR_FAILURE;
 
   *aFirstFrameOnLine = (nsIFrame*)firstCellData->GetCellFrame();
-  if(!(*aFirstFrameOnLine))
+  if (!(*aFirstFrameOnLine))
   {
     while((aLineNumber > 0)&&(!(*aFirstFrameOnLine)))
     {
@@ -1984,7 +1996,7 @@ nsTableRowGroupFrame::FindFrameAt(PRInt32    aLineNumber,
   nsTableFrame* parentFrame = nsnull;
   nsTableFrame::GetTableFrame(this, parentFrame);
   nsTableCellMap* cellMap = parentFrame->GetCellMap();
-  if(!cellMap)
+  if (!cellMap)
      return NS_ERROR_FAILURE;
 
   colCount = cellMap->GetColCount();
@@ -2002,7 +2014,7 @@ nsTableRowGroupFrame::FindFrameAt(PRInt32    aLineNumber,
       continue;
     tempFrame = (nsIFrame*)cellData->GetCellFrame();
 
-    if(!tempFrame)
+    if (!tempFrame)
       continue;
     
     nsRect tempRect = tempFrame->GetRect();//offsetting x to be in row coordinates
@@ -2016,13 +2028,13 @@ nsTableRowGroupFrame::FindFrameAt(PRInt32    aLineNumber,
       gotParentRect = PR_TRUE;
     }
 
-    if(i==0 &&(aX <= 0))//short circuit for negative x coords
+    if (i==0 &&(aX <= 0))//short circuit for negative x coords
     {
       *aXIsBeforeFirstFrame = PR_TRUE;
       *aFrameFound = tempFrame;
       return NS_OK;
     }
-    if(aX < tempRect.x)
+    if (aX < tempRect.x)
     {
       return NS_ERROR_FAILURE;
     }
@@ -2035,7 +2047,7 @@ nsTableRowGroupFrame::FindFrameAt(PRInt32    aLineNumber,
   //x coord not found in frame, return last frame
   *aXIsAfterLastFrame = PR_TRUE;
   *aFrameFound = tempFrame;
-  if(!(*aFrameFound))
+  if (!(*aFrameFound))
     return NS_ERROR_FAILURE;
   return NS_OK;
 }
@@ -2048,13 +2060,13 @@ nsTableRowGroupFrame::GetNextSiblingOnLine(nsIFrame*& aFrame,
 
   nsITableCellLayout* cellFrame;
   nsresult result = CallQueryInterface(aFrame, &cellFrame);
-  if(NS_FAILED(result)) 
+  if (NS_FAILED(result))
     return result;
 
   nsTableFrame* parentFrame = nsnull;
   result = nsTableFrame::GetTableFrame(this, parentFrame);
   nsTableCellMap* cellMap = parentFrame->GetCellMap();
-  if(!cellMap)
+  if (!cellMap)
      return NS_ERROR_FAILURE;
 
 
@@ -2064,10 +2076,10 @@ nsTableRowGroupFrame::GetNextSiblingOnLine(nsIFrame*& aFrame,
 
   CellData* cellData = cellMap->GetDataAt(aLineNumber, colIndex + 1);
   
-  if(!cellData)// if this isnt a valid cell, drop down and check the next line
+  if (!cellData)// if this isnt a valid cell, drop down and check the next line
   {
     cellData = cellMap->GetDataAt(aLineNumber + 1, 0);
-    if(!cellData)
+    if (!cellData)
     {
       //*aFrame = nsnull;
       return NS_ERROR_FAILURE;
@@ -2075,19 +2087,19 @@ nsTableRowGroupFrame::GetNextSiblingOnLine(nsIFrame*& aFrame,
   }
 
   aFrame = (nsIFrame*)cellData->GetCellFrame();
-  if(!aFrame)
+  if (!aFrame)
   {
     //PRInt32 numCellsInRow = cellMap->GetNumCellsOriginatingInRow(aLineNumber) - 1;
     PRInt32 tempCol = colIndex + 1;
     PRInt32 tempRow = aLineNumber;
-    while((tempCol > 0) && (!aFrame))
+    while ((tempCol > 0) && (!aFrame))
     {
       tempCol--;
       cellData = cellMap->GetDataAt(aLineNumber, tempCol);
       aFrame = (nsIFrame*)cellData->GetCellFrame();
-      if(!aFrame && (tempCol==0))
+      if (!aFrame && (tempCol==0))
       {
-        while((tempRow > 0) && (!aFrame))
+        while ((tempRow > 0) && (!aFrame))
         {
           tempRow--;
           cellData = cellMap->GetDataAt(tempRow, 0);
