@@ -193,8 +193,7 @@ public:
                    nsIContent*     aChild,
                    PRInt32         aNameSpaceID,
                    nsIAtom*        aAttribute,
-                   PRInt32         aModType, 
-                   PRInt32         aHint);
+                   PRInt32         aModType);
 
   // --------------------------------------------------------------------------
   // Additional methods 
