@@ -22,6 +22,7 @@
 #include "nsINameSpaceManager.h"
 #include "nsMathMLFrame.h"
 #include "nsMathMLChar.h"
+#include "nsCSSAnonBoxes.h"
 
 // used to map attributes into CSS rules
 #include "nsIDocument.h"
@@ -112,11 +113,11 @@ nsMathMLFrame::ResolveMathMLCharStyle(nsIPresContext*  aPresContext,
                                       nsMathMLChar*    aMathMLChar,
                                       PRBool           aIsMutableChar)
 {
-  nsIAtom* fontAtom = (aIsMutableChar) ?
-    nsMathMLAtoms::fontstyle_stretchy :
-    nsMathMLAtoms::fontstyle_anonymous; // savings
+  nsIAtom* pseudoStyle = (aIsMutableChar) ?
+    nsCSSAnonBoxes::mozMathStretchy :
+    nsCSSAnonBoxes::mozMathAnonymous; // savings
   nsRefPtr<nsStyleContext> newStyleContext;
-  newStyleContext = aPresContext->ResolvePseudoStyleContextFor(aContent, fontAtom, 
+  newStyleContext = aPresContext->ResolvePseudoStyleContextFor(aContent, pseudoStyle,
 							       aParentStyleContext);
   if (newStyleContext)
     aMathMLChar->SetStyleContext(newStyleContext);
@@ -533,9 +534,14 @@ GetMathMLAttributeStyleSheet(nsIPresContext* aPresContext,
   nsCOMPtr<nsICSSStyleSheet> cssSheet(do_CreateInstance(kCSSStyleSheetCID));
   if (!cssSheet)
     return;
-  cssSheet->Init(uri);
+  cssSheet->SetURL(uri);
+  nsCOMPtr<nsIDOMCSSStyleSheet> domSheet(do_QueryInterface(cssSheet));
+  if (domSheet) {
+    PRUint32 index;
+    domSheet->InsertRule(NS_LITERAL_STRING("@namespace url(http://www.w3.org/1998/Math/MathML);"),
+                                           0, &index);
+  }
   cssSheet->SetTitle(NS_ConvertASCIItoUCS2(kTitle));
-  cssSheet->SetDefaultNameSpaceID(kNameSpaceID_MathML);
   nsCOMPtr<nsIStyleSheet> sheet(do_QueryInterface(cssSheet));
 
   // all done, no further activity from the net involved, so we better do this
@@ -553,9 +559,9 @@ nsMathMLFrame::MapAttributesIntoCSS(nsIPresContext* aPresContext,
 {
   // normal case, quick return if there are no attributes
   NS_ASSERTION(aContent, "null arg");
-  PRInt32 attrCount = 0;
+  PRUint32 attrCount = 0;
   if (aContent)
-    aContent->GetAttrCount(attrCount);
+    attrCount = aContent->GetAttrCount();
   if (!attrCount)
     return 0;
 
@@ -581,7 +587,7 @@ nsMathMLFrame::MapAttributesIntoCSS(nsIPresContext* aPresContext,
   nsCOMPtr<nsIAtom> prefix;
   nsCOMPtr<nsIAtom> attrAtom;
   PRInt32 ruleCount = 0;
-  for (PRInt32 i = 0; i < attrCount; ++i) {
+  for (PRUint32 i = 0; i < attrCount; ++i) {
     aContent->GetAttrNameAt(i, &nameSpaceID,
                             getter_AddRefs(attrAtom),
                             getter_AddRefs(prefix));
@@ -625,7 +631,7 @@ nsMathMLFrame::MapAttributesIntoCSS(nsIPresContext* aPresContext,
     if (!sheet) {
       // first time... we do this to defer the lookup up to the
       // point where we encounter attributes that actually matter
-      aContent->GetDocument(getter_AddRefs(doc));
+      doc = aContent->GetDocument();
       if (!doc) 
         return 0;
       GetMathMLAttributeStyleSheet(aPresContext, getter_AddRefs(sheet));
@@ -666,7 +672,7 @@ nsMathMLFrame::MapAttributesIntoCSS(nsIPresContext* aPresContext,
       // insert the rule (note: when the sheet already has @namespace and
       // friends, insert after them, e.g., at the end, otherwise it won't work)
       // For MathML 2, insert at the end to give it precedence
-      PRInt32 pos = (map->compatibility == kMathMLversion2) ? count : 0;
+      PRInt32 pos = (map->compatibility == kMathMLversion2) ? count : 1;
       PRUint32 index;
       domSheet->InsertRule(cssRule, pos, &index);
       ++ruleCount;
