@@ -82,10 +82,8 @@ nsMathMLmmultiscriptsFrame::TransmitAutomaticData(nsIPresContext* aPresContext)
   nsAutoVoidArray subScriptFrames;
   nsIFrame* childFrame = mFrames.FirstChild();
   while (childFrame) {
-    nsCOMPtr<nsIContent> childContent;
     nsCOMPtr<nsIAtom> childTag;
-    childFrame->GetContent(getter_AddRefs(childContent));
-    childContent->GetTag(*getter_AddRefs(childTag));
+    childFrame->GetContent()->GetTag(getter_AddRefs(childTag));
     if (childTag.get() == nsMathMLAtoms::mprescripts_) {
       // mprescripts frame
     }
@@ -104,7 +102,7 @@ nsMathMLmmultiscriptsFrame::TransmitAutomaticData(nsIPresContext* aPresContext)
       isSubScript = !isSubScript;
     }
     count++;
-    childFrame->GetNextSibling(&childFrame);
+    childFrame = childFrame->GetNextSibling();
   }
   for (PRInt32 i = subScriptFrames.Count() - 1; i >= 0; i--) {
     childFrame = (nsIFrame*)subScriptFrames[i];
@@ -260,10 +258,8 @@ nsMathMLmmultiscriptsFrame::Place(nsIPresContext*      aPresContext,
 
   nsIFrame* childFrame = mFrames.FirstChild();
   while (childFrame) {
-    nsCOMPtr<nsIContent> childContent;
     nsCOMPtr<nsIAtom> childTag;
-    childFrame->GetContent(getter_AddRefs(childContent));
-    childContent->GetTag(*getter_AddRefs(childTag));
+    childFrame->GetContent()->GetTag(getter_AddRefs(childTag));
 
     if (childTag.get() == nsMathMLAtoms::mprescripts_) {
       if (mprescriptsFrame) {
@@ -373,7 +369,7 @@ nsMathMLmmultiscriptsFrame::Place(nsIPresContext*      aPresContext,
       isSubScript = !isSubScript;
     }
     count++;
-    childFrame->GetNextSibling(&childFrame);
+    childFrame = childFrame->GetNextSibling();
   }
   // note: width=0 if all sup-sub pairs match correctly
   if ((0 != width) || !baseFrame || !subScriptFrame || !supScriptFrame) {
@@ -455,7 +451,7 @@ nsMathMLmmultiscriptsFrame::Place(nsIPresContext*      aPresContext,
           dx += mScriptSpace + width;
         }
       }
-      childFrame->GetNextSibling(&childFrame);
+      childFrame = childFrame->GetNextSibling();
     } while (mprescriptsFrame != childFrame);
   }
 
