@@ -141,8 +141,8 @@ nsMathMLmsupFrame::PlaceSuperScript(nsIPresContext*      aPresContext,
   nsIFrame* supScriptFrame = nsnull;
   aFrame->FirstChild(aPresContext, nsnull, &baseFrame);
   if (baseFrame)
-    baseFrame->GetNextSibling(&supScriptFrame);
-  if (!baseFrame || !supScriptFrame || HasNextSibling(supScriptFrame)) {
+    supScriptFrame = baseFrame->GetNextSibling();
+  if (!baseFrame || !supScriptFrame || supScriptFrame->GetNextSibling()) {
     // report an error, encourage people to get their markups in order
     NS_WARNING("invalid markup");
     return NS_STATIC_CAST(nsMathMLContainerFrame*, 
