@@ -84,8 +84,7 @@ public:
                                nsIContent*     aChild,
                                PRInt32         aNameSpaceID,
                                nsIAtom*        aAttribute,
-                               PRInt32         aModType,
-                               PRInt32         aHint);
+                               PRInt32         aModType);
 
   NS_IMETHOD DidSetStyleContext(nsIPresContext* aPresContext);
   
