@@ -269,10 +269,7 @@ nsBoxFrame::Init(nsIPresContext*  aPresContext,
     PRBool needsWidget = PR_FALSE;
     parent->ChildrenMustHaveWidgets(needsWidget);
     if (needsWidget) {
-
-        if (!HasView()) {
-           nsHTMLContainerFrame::CreateViewForFrame(aPresContext,this,mStyleContext,nsnull,PR_TRUE); 
-        }
+        nsHTMLContainerFrame::CreateViewForFrame(this, nsnull, PR_TRUE);
 
         nsIView* view = GetView();
         if (!view->HasWidget())
@@ -761,8 +758,7 @@ nsBoxFrame::IsInitialReflowForPrintPreview(nsBoxLayoutState& aState,
       nsCOMPtr<nsIDocument> doc;
       presShell->GetDocument(getter_AddRefs(doc));
       if (!doc) return PR_FALSE;
-      nsCOMPtr<nsIURI> uri;
-      doc->GetDocumentURL(getter_AddRefs(uri));
+      nsIURI *uri = doc->GetDocumentURL();
       if (!uri) return PR_FALSE;
       uri->SchemeIs("chrome", &aIsChrome);
       return PR_TRUE;
@@ -826,13 +822,6 @@ nsBoxFrame::Reflow(nsIPresContext*          aPresContext,
   m = aReflowState.mComputedBorderPadding;
   // GetBorderAndPadding(m);
 
-  // this happens sometimes. So lets handle it gracefully.
-  if (aReflowState.mComputedHeight == 0) {
-    nsSize minSize(0,0);
-    GetMinSize(state, minSize);
-    computedSize.height = minSize.height - m.top - m.bottom;
-  }
-
   nsSize prefSize(0,0);
 
   // if we are told to layout intrinic then get our preferred size.
@@ -2484,8 +2473,7 @@ nsBoxFrame::CreateViewForFrame(nsIPresContext*  aPresContext,
     
     // See if the frame is a scrolled frame
     if (!aForce) {
-      nsCOMPtr<nsIAtom> pseudoTag = aStyleContext->GetPseudoType();
-      if (pseudoTag == nsCSSAnonBoxes::scrolledContent) {
+      if (aStyleContext->GetPseudoType() == nsCSSAnonBoxes::scrolledContent) {
         NS_FRAME_LOG(NS_FRAME_TRACE_CALLS,
           ("nsBoxFrame::CreateViewForFrame: scrolled frame=%p", aFrame));
         aForce = PR_TRUE;
@@ -2552,13 +2540,8 @@ nsBoxFrame::CreateViewForFrame(nsIPresContext*  aPresContext,
             // object's child elements, we can't tell if it's a leaf by looking
             // at whether the frame has any child frames
             nsIContent* content = aFrame->GetContent();
-            PRBool canContainChildren = PR_FALSE;
 
-            if (content) {
-              content->CanContainChildren(canContainChildren);
-            }
-
-            if (canContainChildren) {
+            if (content && content->CanContainChildren()) {
               // The view needs to be visible, but marked as having transparent
               // content
               viewHasTransparentContent = PR_TRUE;
