@@ -118,9 +118,7 @@ HandleImagePLEvent(nsIContent *aContent, PRUint32 aMessage, PRUint32 aFlags)
     return;
   }
 
-  nsCOMPtr<nsIPresShell> pres_shell;
-  doc->GetShellAt(0, getter_AddRefs(pres_shell));
-
+  nsIPresShell *pres_shell = doc->GetShellAt(0);
   if (!pres_shell) {
     return;
   }
@@ -262,10 +260,9 @@ nsImageBoxFrame::AttributeChanged(nsIPresContext* aPresContext,
                                nsIContent* aChild,
                                PRInt32 aNameSpaceID,
                                nsIAtom* aAttribute,
-                               PRInt32 aModType, 
-                               PRInt32 aHint)
+                               PRInt32 aModType)
 {
-  nsresult rv = nsLeafBoxFrame::AttributeChanged(aPresContext, aChild, aNameSpaceID, aAttribute, aModType, aHint);
+  nsresult rv = nsLeafBoxFrame::AttributeChanged(aPresContext, aChild, aNameSpaceID, aAttribute, aModType);
 
   PRBool aResize;
   PRBool aRedraw;
@@ -360,12 +357,17 @@ void
 nsImageBoxFrame::GetImageSource()
 {
   // get the new image src
-  mContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::src, mSrc);
-
-  // if the new image is empty
-  if (mSrc.IsEmpty()) {
-    mUseSrcAttr = PR_FALSE;
-
+  nsAutoString src;
+  mContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::src, src);
+  mUseSrcAttr = !src.IsEmpty();
+  if (mUseSrcAttr) {
+    nsCOMPtr<nsIURI> baseURI;
+    if (mContent) {
+      mContent->GetBaseURL(getter_AddRefs(baseURI));
+    }
+    // XXX origin charset needed
+    NS_NewURI(getter_AddRefs(mURI), src, nsnull, baseURI);
+  } else {
     // Only get the list-style-image if we aren't being drawn
     // by a native theme.
     const nsStyleDisplay* disp = GetStyleDisplay();
@@ -374,14 +376,8 @@ nsImageBoxFrame::GetImageSource()
       return;
 
     // get the list-style-image
-    const nsStyleList* myList = GetStyleList();
-  
-    if (myList->mListStyleImage.Length() > 0) {
-      mSrc = myList->mListStyleImage;
-    }
+    mURI = GetStyleList()->mListStyleImage;
   }
-  else
-    mUseSrcAttr = PR_TRUE;
 }
 
 void
@@ -417,7 +413,7 @@ nsImageBoxFrame::UpdateImage(nsIPresContext*  aPresContext, PRBool& aResize)
   aResize = PR_FALSE;
 
   // get the new image src
-  if (mSrc.IsEmpty()) {
+  if (!mURI) {
     mSizeFrozen = PR_TRUE;
     mHasImage = PR_FALSE;
     aResize = PR_TRUE;
@@ -430,21 +426,16 @@ nsImageBoxFrame::UpdateImage(nsIPresContext*  aPresContext, PRBool& aResize)
     return;
   }
 
-  nsCOMPtr<nsIURI> baseURI;
-  GetBaseURI(getter_AddRefs(baseURI));
-  nsCOMPtr<nsIURI> srcURI;
-  NS_NewURI(getter_AddRefs(srcURI), mSrc, nsnull, baseURI);
-
+  nsresult rv;
   if (mImageRequest) {
     nsCOMPtr<nsIURI> requestURI;
-    nsresult rv = mImageRequest->GetURI(getter_AddRefs(requestURI));
+    rv = mImageRequest->GetURI(getter_AddRefs(requestURI));
     NS_ASSERTION(NS_SUCCEEDED(rv) && requestURI,"no request URI");
     if (NS_FAILED(rv) || !requestURI) return;
 
     PRBool eq;
-    requestURI->Equals(srcURI, &eq);
     // if the source uri and the current one are the same, return
-    if (eq)
+    if (NS_SUCCEEDED(requestURI->Equals(mURI, &eq)) && eq)
       return;
   }
 
@@ -457,7 +448,6 @@ nsImageBoxFrame::UpdateImage(nsIPresContext*  aPresContext, PRBool& aResize)
     mImageRequest = nsnull;
   }
 
-  nsresult rv;
   nsCOMPtr<imgILoader> il(do_GetService("@mozilla.org/image/loader;1", &rv));
   if (NS_FAILED(rv)) return;
 
@@ -465,17 +455,17 @@ nsImageBoxFrame::UpdateImage(nsIPresContext*  aPresContext, PRBool& aResize)
   GetLoadGroup(aPresContext, getter_AddRefs(loadGroup));
 
   // Get the document URI for the referrer...
-  nsCOMPtr<nsIURI> documentURI;
+  nsIURI *documentURI = nsnull;
   nsCOMPtr<nsIDocument> doc;
   if (mContent) {
-    (void) mContent->GetDocument(*getter_AddRefs(doc));
+    doc = mContent->GetDocument();
     if (doc) {
-      doc->GetDocumentURL(getter_AddRefs(documentURI));
+      documentURI = doc->GetDocumentURL();
     }
   }
 
   // XXX: initialDocumentURI is NULL!
-  il->LoadImage(srcURI, nsnull, documentURI, loadGroup, mListener, doc,
+  il->LoadImage(mURI, nsnull, documentURI, loadGroup, mListener, doc,
                 mLoadFlags, nsnull, nsnull, getter_AddRefs(mImageRequest));
 
   aResize = PR_TRUE;
@@ -581,11 +571,13 @@ nsImageBoxFrame::DidSetStyleContext( nsIPresContext* aPresContext )
     return NS_OK;
 
   // If list-style-image changes, we have a new image.
-  nsAutoString newSrc;
-  if (myList->mListStyleImage.Equals(mSrc))
+  nsIURI *newURI = myList->mListStyleImage;
+  PRBool equal;
+  if (newURI == mURI ||   // handles null==null
+      (newURI && mURI && NS_SUCCEEDED(newURI->Equals(mURI, &equal)) && equal))
     return NS_OK;
 
-  mSrc = myList->mListStyleImage;
+  mURI = newURI;
 
   PRBool aResize;
   UpdateImage(aPresContext, aResize);
@@ -703,26 +695,6 @@ nsImageBoxFrame::GetFrameName(nsAString& aResult) const
 #endif
 
 
-void
-nsImageBoxFrame::GetBaseURI(nsIURI **uri)
-{
-  nsresult rv;
-  nsCOMPtr<nsIURI> baseURI;
-  nsCOMPtr<nsIHTMLContent> htmlContent(do_QueryInterface(mContent, &rv));
-  if (NS_SUCCEEDED(rv)) {
-    htmlContent->GetBaseURL(*getter_AddRefs(baseURI));
-  }
-  else {
-    nsCOMPtr<nsIDocument> doc;
-    mContent->GetDocument(*getter_AddRefs(doc));
-    if (doc) {
-      doc->GetBaseURL(*getter_AddRefs(baseURI));
-    }
-  }
-  *uri = baseURI;
-  NS_IF_ADDREF(*uri);
-}
-
 void
 nsImageBoxFrame::GetLoadGroup(nsIPresContext *aPresContext, nsILoadGroup **aLoadGroup)
 {
@@ -737,7 +709,7 @@ nsImageBoxFrame::GetLoadGroup(nsIPresContext *aPresContext, nsILoadGroup **aLoad
   if (!doc)
     return;
 
-  doc->GetDocumentLoadGroup(aLoadGroup);
+  *aLoadGroup = doc->GetDocumentLoadGroup().get(); // already_AddRefed
 }
 
 
