@@ -595,8 +595,7 @@ nsMenuPopupFrame::AdjustClientXYForNestedDocuments ( nsIDOMXULDocument* inPopupD
   if ( targetAsContent ) {
     nsCOMPtr<nsIDocument> targetDocument = targetAsContent->GetDocument();
     if (targetDocument) {
-      nsCOMPtr<nsIPresShell> shell;
-      targetDocument->GetShellAt(0, getter_AddRefs(shell));
+      nsIPresShell *shell = targetDocument->GetShellAt(0);
       if ( shell ) {
         // We might be inside a popup widget. If so, we need to use that widget and
         // not the root view's widget.
@@ -929,8 +928,6 @@ nsMenuPopupFrame::SyncViewWithFrame(nsIPresContext* aPresContext,
   aPresContext->GetShell(getter_AddRefs(presShell));
   nsCOMPtr<nsIDocument> document;
   presShell->GetDocument(getter_AddRefs(document));
-  nsCOMPtr<nsIScriptGlobalObject> scriptGlobalObject;
-  document->GetScriptGlobalObject(getter_AddRefs(scriptGlobalObject));
   
   nsCOMPtr<nsIAtom> tag;
   mContent->GetTag(getter_AddRefs(tag));
@@ -987,7 +984,7 @@ nsMenuPopupFrame::SyncViewWithFrame(nsIPresContext* aPresContext,
   // Compute info about the screen dimensions. Because of multiple monitor systems,
   // the left or top sides of the screen may be in negative space (main monitor is on the
   // right, etc). We need to be sure to do the right thing.
-  nsCOMPtr<nsIDOMWindowInternal> window(do_QueryInterface(scriptGlobalObject));
+  nsCOMPtr<nsIDOMWindowInternal> window(do_QueryInterface(document->GetScriptGlobalObject()));
   nsCOMPtr<nsIDOMScreen> screen;
   window->GetScreen(getter_AddRefs(screen));
   PRInt32 screenWidth = 0, screenHeight = 0;
