@@ -156,8 +156,7 @@ nsScrollbarButtonFrame::MouseClicked()
        return;
 
    // get the scrollbars content node
-   nsCOMPtr<nsIContent> content;
-   scrollbar->GetContent(getter_AddRefs(content));
+   nsIContent* content = scrollbar->GetContent();
 
    // get the current pos
    PRInt32 curpos = nsSliderFrame::GetCurrentPosition(content);
@@ -221,8 +220,7 @@ nsScrollbarButtonFrame::GetChildWithTag(nsIPresContext* aPresContext,
   while (nsnull != childFrame) 
   {    
     // get the content node
-    nsCOMPtr<nsIContent> child;  
-    childFrame->GetContent(getter_AddRefs(child));
+    nsIContent* child = childFrame->GetContent();
 
     if (child) {
       // see if it is the child
@@ -240,8 +238,7 @@ nsScrollbarButtonFrame::GetChildWithTag(nsIPresContext* aPresContext,
      if (result != nsnull) 
        return NS_OK;
 
-    nsresult rv = childFrame->GetNextSibling(&childFrame);
-    NS_ASSERTION(rv == NS_OK,"failed to get next child");
+    childFrame = childFrame->GetNextSibling();
   }
 
   result = nsnull;
@@ -251,14 +248,13 @@ nsScrollbarButtonFrame::GetChildWithTag(nsIPresContext* aPresContext,
 nsresult
 nsScrollbarButtonFrame::GetParentWithTag(nsIAtom* toFind, nsIFrame* start, nsIFrame*& result)
 {
-   while(nsnull != start)
+   while (start)
    {
-      start->GetParent(&start);
+      start = start->GetParent();
 
       if (start) {
         // get the content node
-        nsCOMPtr<nsIContent> child;  
-        start->GetContent(getter_AddRefs(child));
+        nsIContent* child = start->GetContent();
 
         nsCOMPtr<nsIAtom> atom;
         if (child && child->GetTag(getter_AddRefs(atom)) == NS_OK && atom.get() == toFind) {
