@@ -86,9 +86,9 @@ static already_AddRefed<nsIContent>
 GetContentOfBox(nsIBox *aBox)
 {
   nsIFrame *frame;
-  nsIContent *content;
   aBox->GetFrame(&frame);
-  frame->GetContent(&content);
+  nsIContent* content = frame->GetContent();
+  NS_IF_ADDREF(content);
   return content;
 }
 
@@ -137,12 +137,8 @@ nsSliderFrame::Init(nsIPresContext*  aPresContext,
   }
 
   CreateViewForFrame(aPresContext,this,aContext,PR_TRUE);
-  nsIView* view = GetView(aPresContext);
-  nsCOMPtr<nsIViewManager> vm;
-  view->GetViewManager(*getter_AddRefs(vm));
-  vm->SetViewContentTransparency(view, PR_TRUE);
-  // XXX Hack
-  mPresContext = aPresContext;
+  nsIView* view = GetView();
+  view->GetViewManager()->SetViewContentTransparency(view, PR_TRUE);
   return rv;
 }
 
@@ -237,11 +233,11 @@ nsSliderFrame::AttributeChanged(nsIPresContext* aPresContext,
                                nsIContent* aChild,
                                PRInt32 aNameSpaceID,
                                nsIAtom* aAttribute,
-                               PRInt32 aModType, 
-                               PRInt32 aHint)
+                               PRInt32 aModType)
 {
   nsresult rv = nsBoxFrame::AttributeChanged(aPresContext, aChild,
-                                              aNameSpaceID, aAttribute, aModType, aHint);
+                                             aNameSpaceID, aAttribute,
+                                             aModType);
   // if the current position changes
   if (aAttribute == nsXULAtoms::curpos) {
      rv = CurrentPositionChanged(aPresContext);
@@ -279,10 +275,9 @@ nsSliderFrame::AttributeChanged(nsIPresContext* aPresContext,
       }
   }
 
-  if ((aHint != NS_STYLE_HINT_REFLOW) &&
-             (aAttribute == nsXULAtoms::maxpos ||
-             aAttribute == nsXULAtoms::pageincrement ||
-             aAttribute == nsXULAtoms::increment)) {
+  if (aAttribute == nsXULAtoms::maxpos ||
+      aAttribute == nsXULAtoms::pageincrement ||
+      aAttribute == nsXULAtoms::increment) {
       nsCOMPtr<nsIPresShell> shell;
       aPresContext->GetShell(getter_AddRefs(shell));
 
@@ -466,7 +461,7 @@ nsSliderFrame::HandleEvent(nsIPresContext* aPresContext,
   scrollbar = GetContentOfBox(scrollbarBox);
   PRBool isHorizontal = IsHorizontal();
 
-  if (isDraggingThumb(aPresContext))
+  if (isDraggingThumb())
   {
       // we want to draw immediately if the user doing it directly with the
       // mouse that makes redrawing much faster.
@@ -498,7 +493,7 @@ nsSliderFrame::HandleEvent(nsIPresContext* aPresContext,
        {
          // if we hit a scrollable view make sure we take into account
          // how much we are scrolled.
-         nsIView* view = parent->GetView(aPresContext);
+         nsIView* view = parent->GetView();
          if (view) {
            nsIScrollableView* scrollingView;
            nsresult result = CallQueryInterface(view, &scrollingView);
@@ -510,22 +505,19 @@ nsSliderFrame::HandleEvent(nsIPresContext* aPresContext,
            }
          }
 
-         nsRect r;
-         parent->GetRect(r);
+         nsRect r = parent->GetRect();
          isHorizontal ? start -= r.x : start -= r.y;
 
          if (view) {
-           nsCOMPtr<nsIWidget> widget;
-           view->GetWidget(*getter_AddRefs(widget));
-           if (widget) {
+           if (view->HasWidget()) {
              nsWindowType windowType;
-             widget->GetWindowType(windowType);
+             view->GetWidget()->GetWindowType(windowType);
              if (windowType == eWindowType_popup)
                break;
            }
          }
 
-         parent->GetParent(&parent);
+         parent = parent->GetParent();
        }
 
 #else // Kin's new offset calculation code.
@@ -536,13 +528,12 @@ nsSliderFrame::HandleEvent(nsIPresContext* aPresContext,
        // in the coordinate system for the frame's contained view so that it
        // matches aEvent->point's coordinate system.
 
-       nsIView *view = this->GetClosestView(aPresContext);
+       nsIView *view = GetClosestView();
 
        nsCOMPtr<nsIWidget> rootWidget;
 
        if (view) {
-         nsCOMPtr<nsIViewManager> vm;
-         view->GetViewManager(*getter_AddRefs(vm));
+         nsIViewManager* vm = view->GetViewManager();
          if (vm)
            vm->GetWidget(getter_AddRefs(rootWidget));
        }
@@ -556,8 +547,7 @@ nsSliderFrame::HandleEvent(nsIPresContext* aPresContext,
          // view's coordinate system.
 
          while (view) {
-           nsCOMPtr<nsIWidget> widget;
-           view->GetWidget(*getter_AddRefs(widget));
+           nsIWidget* widget = view->GetWidget();
 
            if (widget) {
              nsWindowType windowType;
@@ -567,11 +557,10 @@ nsSliderFrame::HandleEvent(nsIPresContext* aPresContext,
                break;
            }
 
-           nscoord vx=0, vy=0;
-           view->GetPosition(&vx, &vy);
-           isHorizontal ? start -= vx : start -= vy;
+           nsPoint p = view->GetPosition();
+           start -= (isHorizontal ? p.x : p.y);
 
-           view->GetParent(view);
+           view = view->GetParent();
          }
        }
 
@@ -585,8 +574,7 @@ nsSliderFrame::HandleEvent(nsIPresContext* aPresContext,
        pos -= start;
        PRBool isMouseOutsideThumb = PR_FALSE;
        if (mSnapMultiplier) {
-         nsSize thumbSize;
-         thumbFrame->GetSize(thumbSize);
+         nsSize thumbSize = thumbFrame->GetSize();
          if (isHorizontal) {
            // horizontal scrollbar - check if mouse is above or below thumb
            if (aEvent->point.y < (-mSnapMultiplier * thumbSize.height) || 
@@ -628,7 +616,7 @@ nsSliderFrame::HandleEvent(nsIPresContext* aPresContext,
        // stop capturing
       //printf("stop capturing\n");
       AddListener();
-      DragThumb(aPresContext, PR_FALSE);
+      DragThumb(PR_FALSE);
       mRedrawImmediate = PR_FALSE;//we MUST call nsFrame HandleEvent for mouse ups to maintain the selection state and capture state.
       return nsFrame::HandleEvent(aPresContext, aEvent, aEventStatus);
     }
@@ -640,7 +628,8 @@ nsSliderFrame::HandleEvent(nsIPresContext* aPresContext,
     //return nsFrame::HandleEvent(aPresContext, aEvent, aEventStatus);
     return NS_OK;
   }
-  else if (mMiddlePref && aEvent->message == NS_MOUSE_MIDDLE_BUTTON_DOWN) {
+  else if ((aEvent->message == NS_MOUSE_LEFT_BUTTON_DOWN && ((nsMouseEvent *)aEvent)->isShift)
+      || (mMiddlePref && aEvent->message == NS_MOUSE_MIDDLE_BUTTON_DOWN)) {
     // convert coord from twips to pixels
     nscoord pos = isHorizontal ? aEvent->point.x : aEvent->point.y;
     float p2t;
@@ -650,8 +639,7 @@ nsSliderFrame::HandleEvent(nsIPresContext* aPresContext,
 
    // adjust so that the middle of the thumb is placed under the click
     nsIFrame* thumbFrame = mFrames.FirstChild();
-    nsSize thumbSize;
-    thumbFrame->GetSize(thumbSize);
+    nsSize thumbSize = thumbFrame->GetSize();
     nscoord thumbLength = isHorizontal ? thumbSize.width : thumbSize.height;
     thumbLength /= onePixel;
     pospx -= (thumbLength/2);
@@ -670,7 +658,7 @@ nsSliderFrame::HandleEvent(nsIPresContext* aPresContext,
     {
       // if we hit a scrollable view make sure we take into account
       // how much we are scrolled.
-      nsIView* view = parent->GetView(aPresContext);
+      nsIView* view = parent->GetView();
       if (view) {
         nsIScrollableView* scrollingView;
         nsresult result = CallQueryInterface(view, &scrollingView);
@@ -682,21 +670,17 @@ nsSliderFrame::HandleEvent(nsIPresContext* aPresContext,
         }
       }
 
-      nsRect r;
-      parent->GetRect(r);
-      isHorizontal ? pos += r.x : pos += r.y;
-      parent->GetParent(&parent);
+      nsPoint p = parent->GetPosition();
+      pos += isHorizontal ? p.x : p.y;
+      parent = parent->GetParent();
     }
 
-    RemoveListener();
-    DragThumb(mPresContext, PR_TRUE);
-    nsRect thumbRect;
-    thumbFrame->GetRect(thumbRect);
+    DragThumb(PR_TRUE);
 
     if (isHorizontal)
-      mThumbStart = thumbRect.x;
+      mThumbStart = thumbFrame->GetPosition().x;
     else
-      mThumbStart = thumbRect.y;
+      mThumbStart = thumbFrame->GetPosition().y;
 
     mDragStartPx =pos/onePixel;
   }
@@ -788,8 +772,7 @@ nsSliderFrame::CurrentPositionChanged(nsIPresContext* aPresContext)
     if (!thumbFrame)
       return NS_OK; // The thumb may stream in asynchronously via XBL.
 
-    nsRect thumbRect;
-    thumbFrame->GetRect(thumbRect);
+    nsRect thumbRect = thumbFrame->GetRect();
 
     nsRect clientRect;
     GetClientRect(clientRect);
@@ -803,7 +786,7 @@ nsSliderFrame::CurrentPositionChanged(nsIPresContext* aPresContext)
        newThumbRect.y = clientRect.y + nscoord(float(curpospx)*mRatio);
 
     // set the rect
-    thumbFrame->SetRect(aPresContext, newThumbRect);
+    thumbFrame->SetRect(newThumbRect);
 
     // figure out the union of the rect so we know what to redraw
     nsRect changeRect;
@@ -859,7 +842,7 @@ nsSliderFrame::SetCurrentPosition(nsIContent* scrollbar, nsIFrame* aThumbFrame,
     if (mediator) {
       mediator->PositionChanged(GetCurrentPosition(scrollbar), newpos);
       UpdateAttribute(scrollbar, newpos, PR_FALSE, aIsSmooth);
-      CurrentPositionChanged(mPresContext);
+      CurrentPositionChanged(GetPresContext());
       return;
     }
   }
@@ -877,27 +860,20 @@ NS_IMETHODIMP  nsSliderFrame::GetFrameForPoint(nsIPresContext* aPresContext,
                                              nsFramePaintLayer aWhichLayer,
                                              nsIFrame**     aFrame)
 {
-  if ((aWhichLayer != NS_FRAME_PAINT_LAYER_FOREGROUND))
-    return NS_ERROR_FAILURE;
-
   // This is EVIL, we shouldn't be messing with GetFrameForPoint just to get
   // thumb mouse drag events to arrive at the slider!
-  if (isDraggingThumb(aPresContext))
+  if (isDraggingThumb())
   {
     // XXX I assume it's better not to test for visibility here.
     *aFrame = this;
     return NS_OK;
   }
 
-  if (!mRect.Contains(aPoint))
-    return NS_ERROR_FAILURE;
-
-
   if (NS_SUCCEEDED(nsBoxFrame::GetFrameForPoint(aPresContext, aPoint, aWhichLayer, aFrame)))
     return NS_OK;
 
   // always return us (if visible)
-  if (GetStyleVisibility()->IsVisible()) {
+  if (mRect.Contains(aPoint) && GetStyleVisibility()->IsVisible()) {
     *aFrame = this;
     return NS_OK;
   }
@@ -922,19 +898,21 @@ nsSliderFrame::SetInitialChildList(nsIPresContext* aPresContext,
 nsresult
 nsSliderMediator::MouseDown(nsIDOMEvent* aMouseEvent)
 {
-  if (mSlider)
+  // Only process the event if the thumb is not being dragged.
+  if (mSlider && !mSlider->isDraggingThumb())
     return mSlider->MouseDown(aMouseEvent);
-  else
-    return NS_OK;
+
+  return NS_OK;
 }
 
 nsresult
 nsSliderMediator::MouseUp(nsIDOMEvent* aMouseEvent)
 {
-  if (mSlider)
+  // Only process the event if the thumb is not being dragged.
+  if (mSlider && !mSlider->isDraggingThumb())
     return mSlider->MouseUp(aMouseEvent);
-  else
-    return NS_OK;
+
+  return NS_OK;
 }
 
 nsresult
@@ -947,14 +925,18 @@ nsSliderFrame::MouseDown(nsIDOMEvent* aMouseEvent)
   nsCOMPtr<nsIDOMMouseEvent> mouseEvent(do_QueryInterface(aMouseEvent));
 
   PRUint16 button = 0;
+  PRBool scrollToClick = PR_FALSE;
+  mouseEvent->GetShiftKey(&scrollToClick);
   mouseEvent->GetButton(&button);
-  if((mMiddlePref && button != 0 && button != 1) ||
-     (!mMiddlePref && button != 0))
+  if (button != 0) {
+    if (button != 1 || !mMiddlePref)
       return NS_OK;
+    scrollToClick = PR_TRUE;
+  }
 
-  // If middle button, first place the middle of the slider thumb
-  // under the click
-  if (button == 1) {
+  // If shift click or middle button, first
+  // place the middle of the slider thumb under the click
+  if (scrollToClick) {
 
     nscoord pos;
     nscoord pospx;
@@ -962,8 +944,7 @@ nsSliderFrame::MouseDown(nsIDOMEvent* aMouseEvent)
     // mouseEvent has click coordinates in pixels, convert to twips first
     isHorizontal ? mouseEvent->GetClientX(&pospx) : mouseEvent->GetClientY(&pospx);
     float p2t;
-    // XXX hack
-    mPresContext->GetScaledPixelsToTwips(&p2t);
+    GetPresContext()->GetScaledPixelsToTwips(&p2t);
     nscoord onePixel = NSIntPixelsToTwips(1, p2t);
     pos = pospx * onePixel;
 
@@ -973,7 +954,7 @@ nsSliderFrame::MouseDown(nsIDOMEvent* aMouseEvent)
        // if we hit a scrollable view make sure we take into account
        // how much we are scrolled.
        // XXX hack
-       nsIView* view = parent->GetView(mPresContext);
+       nsIView* view = parent->GetView();
        if (view) {
          nsIScrollableView* scrollingView;
          nsresult result = CallQueryInterface(view, &scrollingView);
@@ -985,10 +966,8 @@ nsSliderFrame::MouseDown(nsIDOMEvent* aMouseEvent)
          }
        }
 
-      nsRect r;
-      parent->GetRect(r);
-      isHorizontal ? pos -= r.x : pos -= r.y;
-      parent->GetParent(&parent);
+      pos -= (isHorizontal ? parent->GetPosition().x : parent->GetPosition().y);
+      parent = parent->GetParent();
     }
 
     // now convert back into pixels
@@ -996,10 +975,7 @@ nsSliderFrame::MouseDown(nsIDOMEvent* aMouseEvent)
 
     // adjust so that the middle of the thumb is placed under the click
     nsIFrame* thumbFrame = mFrames.FirstChild();
-    nsRect thumbRect;
-    thumbFrame->GetRect(thumbRect);
-    nsSize thumbSize;
-    thumbFrame->GetSize(thumbSize);
+    nsSize thumbSize = thumbFrame->GetSize();
     nscoord thumbLength = isHorizontal ? thumbSize.width : thumbSize.height;
     thumbLength /= onePixel;
     pospx -= (thumbLength/2);
@@ -1015,8 +991,7 @@ nsSliderFrame::MouseDown(nsIDOMEvent* aMouseEvent)
     SetCurrentPosition(scrollbar, thumbFrame, pospx, PR_FALSE);
   }
 
-  RemoveListener();
-  DragThumb(mPresContext, PR_TRUE);
+  DragThumb(PR_TRUE);
   PRInt32 c = 0;
   if (isHorizontal)
      mouseEvent->GetClientX(&c);
@@ -1025,13 +1000,11 @@ nsSliderFrame::MouseDown(nsIDOMEvent* aMouseEvent)
 
   mDragStartPx = c;
   nsIFrame* thumbFrame = mFrames.FirstChild();
-  nsRect thumbRect;
-  thumbFrame->GetRect(thumbRect);
 
   if (isHorizontal)
-     mThumbStart = thumbRect.x;
+     mThumbStart = thumbFrame->GetPosition().x;
   else
-     mThumbStart = thumbRect.y;
+     mThumbStart = thumbFrame->GetPosition().y;
 
   //printf("Pressed mDragStartPx=%d\n",mDragStartPx);
 
@@ -1046,18 +1019,18 @@ nsSliderFrame::MouseUp(nsIDOMEvent* aMouseEvent)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsSliderFrame :: DragThumb(nsIPresContext* aPresContext, PRBool aGrabMouseEvents)
+void
+nsSliderFrame::DragThumb(PRBool aGrabMouseEvents)
 {
     // get its view
-  nsIView* view = GetView(aPresContext);
-  nsCOMPtr<nsIViewManager> viewMan;
-  PRBool result;
+  nsIView* view = GetView();
 
   if (view) {
-    view->GetViewManager(*getter_AddRefs(viewMan));
+    nsIViewManager* viewMan = view->GetViewManager();
 
     if (viewMan) {
+      PRBool result;
+
       if (aGrabMouseEvents) {
         viewMan->GrabMouseEvents(view,result);
       } else {
@@ -1065,19 +1038,16 @@ nsSliderFrame :: DragThumb(nsIPresContext* aPresContext, PRBool aGrabMouseEvents
       }
     }
   }
-
-  return NS_OK;
 }
 
 PRBool
-nsSliderFrame :: isDraggingThumb(nsIPresContext* aPresContext)
+nsSliderFrame::isDraggingThumb()
 {
     // get its view
-  nsIView* view = GetView(aPresContext);
-  nsCOMPtr<nsIViewManager> viewMan;
+  nsIView* view = GetView();
 
   if (view) {
-    view->GetViewManager(*getter_AddRefs(viewMan));
+    nsIViewManager* viewMan = view->GetViewManager();
 
     if (viewMan) {
         nsIView* grabbingView;
@@ -1100,12 +1070,10 @@ nsSliderFrame::AddListener()
 
   nsIFrame* thumbFrame = mFrames.FirstChild();
   if (thumbFrame) {
-    nsCOMPtr<nsIContent> content;
-    thumbFrame->GetContent(getter_AddRefs(content));
+    nsCOMPtr<nsIDOMEventReceiver>
+      receiver(do_QueryInterface(thumbFrame->GetContent()));
 
-    nsCOMPtr<nsIDOMEventReceiver> reciever(do_QueryInterface(content));
-
-    reciever->AddEventListenerByIID(mMediator, NS_GET_IID(nsIDOMMouseListener));
+    receiver->AddEventListenerByIID(mMediator, NS_GET_IID(nsIDOMMouseListener));
   }
 }
 
@@ -1118,12 +1086,10 @@ nsSliderFrame::RemoveListener()
   if (!thumbFrame)
     return;
 
-  nsCOMPtr<nsIContent> content;
-  thumbFrame->GetContent(getter_AddRefs(content));
+  nsCOMPtr<nsIDOMEventReceiver>
+    receiver(do_QueryInterface(thumbFrame->GetContent()));
 
-  nsCOMPtr<nsIDOMEventReceiver> reciever(do_QueryInterface(content));
-
-  reciever->RemoveEventListenerByIID(mMediator, NS_GET_IID(nsIDOMMouseListener));
+  receiver->RemoveEventListenerByIID(mMediator, NS_GET_IID(nsIDOMMouseListener));
 }
 
 NS_IMETHODIMP
@@ -1131,12 +1097,14 @@ nsSliderFrame::HandlePress(nsIPresContext* aPresContext,
                            nsGUIEvent*     aEvent,
                            nsEventStatus*  aEventStatus)
 {
+  if (((nsMouseEvent *)aEvent)->isShift)
+    return NS_OK;
+
   nsIFrame* thumbFrame = mFrames.FirstChild();
   if (!thumbFrame) // display:none?
     return NS_OK;
   
-  nsRect thumbRect;
-  thumbFrame->GetRect(thumbRect);
+  nsRect thumbRect = thumbFrame->GetRect();
   
   nscoord change = 1;
   if (IsHorizontal() ? aEvent->point.x < thumbRect.x 
@@ -1206,10 +1174,8 @@ nsSliderFrame::EnsureOrient()
 
   nsIFrame* frame = nsnull;
   scrollbarBox->GetFrame(&frame);
-  nsFrameState state;
-  frame->GetFrameState(&state);
 
-  PRBool isHorizontal = state & NS_STATE_IS_HORIZONTAL;
+  PRBool isHorizontal = frame->GetStateBits() & NS_STATE_IS_HORIZONTAL;
   if (isHorizontal)
       mState |= NS_STATE_IS_HORIZONTAL;
   else
@@ -1236,8 +1202,7 @@ NS_IMETHODIMP_(void) nsSliderFrame::Notify(nsITimer *timer)
     PRBool stop = PR_FALSE;
 
     nsIFrame* thumbFrame = mFrames.FirstChild();
-    nsRect thumbRect;
-    thumbFrame->GetRect(thumbRect);
+    nsRect thumbRect = thumbFrame->GetRect();
 
     PRBool isHorizontal = IsHorizontal();
 
@@ -1275,6 +1240,6 @@ NS_INTERFACE_MAP_BEGIN(nsSliderMediator)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsITimerCallback)
 NS_INTERFACE_MAP_END
 
-NS_IMPL_ADDREF(nsSliderMediator);
-NS_IMPL_RELEASE(nsSliderMediator);
+NS_IMPL_ADDREF(nsSliderMediator)
+NS_IMPL_RELEASE(nsSliderMediator)
 
