@@ -48,6 +48,7 @@
 #include "nsHTMLAtoms.h"
 #include "nsISupportsArray.h"
 #include "nsIDOMElement.h"
+#include "nsIDOMXULElement.h"
 #include "nsIDOMDocument.h"
 #include "nsIXMLContent.h"
 #include "nsIPresContext.h"
@@ -65,7 +66,6 @@
 #include "nsFrameNavigator.h"
 #include "nsHTMLParts.h"
 #include "nsILookAndFeel.h"
-#include "nsIPresShell.h"
 #include "nsStyleContext.h"
 #include "nsWidgetsCID.h"
 #include "nsBoxLayoutState.h"
@@ -183,11 +183,9 @@ NS_IMPL_ISUPPORTS2(nsSplitterFrameInner, nsIDOMMouseListener, nsIDOMMouseMotionL
 nsSplitterFrameInner::ResizeType
 nsSplitterFrameInner::GetResizeBefore()
 {
-    nsCOMPtr<nsIContent> content;  
-    mOuter->GetContent(getter_AddRefs(content));
-
     nsString value;
-    content->GetAttr(kNameSpaceID_None, nsXULAtoms::resizebefore, value);
+    mOuter->GetContent()->GetAttr(kNameSpaceID_None,
+                                  nsXULAtoms::resizebefore, value);
     if (value.EqualsIgnoreCase("farthest"))
       return Farthest;
     else
@@ -203,11 +201,9 @@ nsSplitterFrameInner::~nsSplitterFrameInner()
 nsSplitterFrameInner::ResizeType
 nsSplitterFrameInner::GetResizeAfter()
 {
-    nsCOMPtr<nsIContent> content;  
-    mOuter->GetContent(getter_AddRefs(content));
-
     nsString value;
-    content->GetAttr(kNameSpaceID_None, nsXULAtoms::resizeafter, value);
+    mOuter->GetContent()->GetAttr(kNameSpaceID_None,
+                                  nsXULAtoms::resizeafter, value);
     if (value.EqualsIgnoreCase("farthest"))
       return Farthest;
     else if (value.EqualsIgnoreCase("grow"))
@@ -219,11 +215,9 @@ nsSplitterFrameInner::GetResizeAfter()
 nsSplitterFrameInner::State
 nsSplitterFrameInner::GetState()
 {
-    nsCOMPtr<nsIContent> content;  
-    mOuter->GetContent(getter_AddRefs(content));
-
     nsString value;
-    content->GetAttr(kNameSpaceID_None, nsXULAtoms::state, value);
+    mOuter->GetContent()->GetAttr(kNameSpaceID_None,
+                                  nsXULAtoms::state, value);
     if (value.EqualsIgnoreCase("dragging"))
       return Dragging;
     else if (value.EqualsIgnoreCase("collapsed"))
@@ -297,14 +291,14 @@ nsSplitterFrame::GetCursor(nsIPresContext* aPresContext,
 
 NS_IMETHODIMP
 nsSplitterFrame::AttributeChanged(nsIPresContext* aPresContext,
-                               nsIContent* aChild,
-                               PRInt32 aNameSpaceID,
-                               nsIAtom* aAttribute,
-                               PRInt32 aModType, 
-                               PRInt32 aHint)
+                                  nsIContent* aChild,
+                                  PRInt32 aNameSpaceID,
+                                  nsIAtom* aAttribute,
+                                  PRInt32 aModType)
 {
   nsresult rv = nsBoxFrame::AttributeChanged(aPresContext, aChild,
-                                              aNameSpaceID, aAttribute, aModType, aHint);
+                                              aNameSpaceID, aAttribute,
+                                              aModType);
   // if the alignment changed. Let the grippy know
   if (aAttribute == nsHTMLAtoms::align) {
      // tell the slider its attribute changed so it can 
@@ -312,7 +306,7 @@ nsSplitterFrame::AttributeChanged(nsIPresContext* aPresContext,
      nsIFrame* grippy = nsnull;
      nsScrollbarButtonFrame::GetChildWithTag(aPresContext, nsXULAtoms::grippy, this, grippy);
      if (grippy)
-        grippy->AttributeChanged(aPresContext, aChild, aNameSpaceID, aAttribute, aModType, aHint);
+        grippy->AttributeChanged(aPresContext, aChild, aNameSpaceID, aAttribute, aModType);
   } else if (aAttribute == nsXULAtoms::state) {
         mInner->UpdateState();
   }
@@ -367,11 +361,10 @@ nsSplitterFrame::Init(nsIPresContext*  aPresContext,
   // XXX Hack because we need the pres context in some of the event handling functions...
   mPresContext = aPresContext; 
 
-  nsHTMLContainerFrame::CreateViewForFrame(aPresContext,this,aContext,nsnull,PR_TRUE);
-  nsIView* view = GetView(aPresContext);
+  nsHTMLContainerFrame::CreateViewForFrame(this, nsnull, PR_TRUE);
+  nsIView* view = GetView();
 
-  nsCOMPtr<nsIViewManager> viewManager;
-  view->GetViewManager(*getter_AddRefs(viewManager));
+  nsIViewManager* viewManager = view->GetViewManager();
   viewManager->SetViewContentTransparency(view, PR_TRUE);
   viewManager->SetViewZIndex(view, PR_FALSE, kMaxZ);
 
@@ -395,10 +388,7 @@ nsSplitterFrame::DoLayout(nsBoxLayoutState& aState)
   nsIFrame* frame;
   GetFrame(&frame);
 
-  nsFrameState childState;
-  frame->GetFrameState(&childState);
-
-  if (childState & NS_FRAME_FIRST_REFLOW) 
+  if (frame->GetStateBits() & NS_FRAME_FIRST_REFLOW) 
   {
     GetParentBox(&mInner->mParentBox);
     mInner->UpdateState();
@@ -517,6 +507,12 @@ nsSplitterFrameInner::MouseUp(nsIPresContext* aPresContext, nsGUIEvent* aEvent)
 
           mPressed = PR_FALSE;
 
+          // if we dragged then fire a command event.
+          if (mDidDrag) {
+              nsCOMPtr<nsIDOMXULElement> element = do_QueryInterface(mOuter->GetContent());
+              element->DoCommand();
+          }
+
           //printf("MouseUp\n");
 
       }
@@ -550,7 +546,7 @@ nsSplitterFrameInner::MouseDrag(nsIPresContext* aPresContext, nsGUIEvent* aEvent
            {
               // if we hit a scrollable view make sure we take into account
               // how much we are scrolled.
-              nsIView* view = parent->GetView(aPresContext);
+              nsIView* view = parent->GetView();
               if (view) {
                 nsIScrollableView* scrollingView;
                 nsresult result = CallQueryInterface(view, &scrollingView);
@@ -562,10 +558,9 @@ nsSplitterFrameInner::MouseDrag(nsIPresContext* aPresContext, nsGUIEvent* aEvent
                 }
               }
        
-             nsRect r;
-             parent->GetRect(r);
+             nsRect r = parent->GetRect();
              isHorizontal ? start -= r.x : start -= r.y;
-             parent->GetParent(&parent);
+             parent = parent->GetParent();
            }
 
            // take our current position and substract the start location
@@ -670,25 +665,22 @@ nsSplitterFrameInner::AddListener(nsIPresContext* aPresContext)
 {
   nsIFrame* thumbFrame = nsnull;
   mOuter->FirstChild(aPresContext, nsnull,&thumbFrame);
-  nsCOMPtr<nsIContent> content;
-  mOuter->GetContent(getter_AddRefs(content));
 
-  nsCOMPtr<nsIDOMEventReceiver> reciever(do_QueryInterface(content));
+  nsCOMPtr<nsIDOMEventReceiver>
+    receiver(do_QueryInterface(mOuter->GetContent()));
 
-  reciever->AddEventListenerByIID(NS_STATIC_CAST(nsIDOMMouseListener*,this), NS_GET_IID(nsIDOMMouseListener));
-  reciever->AddEventListenerByIID(NS_STATIC_CAST(nsIDOMMouseMotionListener*,this), NS_GET_IID(nsIDOMMouseMotionListener));
+  receiver->AddEventListenerByIID(NS_STATIC_CAST(nsIDOMMouseListener*,this), NS_GET_IID(nsIDOMMouseListener));
+  receiver->AddEventListenerByIID(NS_STATIC_CAST(nsIDOMMouseMotionListener*,this), NS_GET_IID(nsIDOMMouseMotionListener));
 }
 
 void
 nsSplitterFrameInner::RemoveListener()
 {
-  nsCOMPtr<nsIContent> content;
-  mOuter->GetContent(getter_AddRefs(content));
+  nsCOMPtr<nsIDOMEventReceiver>
+    receiver(do_QueryInterface(mOuter->GetContent()));
 
-  nsCOMPtr<nsIDOMEventReceiver> reciever(do_QueryInterface(content));
-
-  reciever->RemoveEventListenerByIID(NS_STATIC_CAST(nsIDOMMouseListener*,this),NS_GET_IID(nsIDOMMouseListener));
-  reciever->RemoveEventListenerByIID(NS_STATIC_CAST(nsIDOMMouseMotionListener*,this),NS_GET_IID(nsIDOMMouseMotionListener));
+  receiver->RemoveEventListenerByIID(NS_STATIC_CAST(nsIDOMMouseListener*,this),NS_GET_IID(nsIDOMMouseListener));
+  receiver->RemoveEventListenerByIID(NS_STATIC_CAST(nsIDOMMouseMotionListener*,this),NS_GET_IID(nsIDOMMouseMotionListener));
 }
 
 /*
@@ -696,15 +688,13 @@ nsresult
 nsSplitterFrameInner :: CaptureMouse(nsIPresContext* aPresContext, PRBool aGrabMouseEvents)
 {
     // get its view
-  nsIView* view = mOuter->GetView(aPresContext);
-  nsCOMPtr<nsIViewManager> viewMan;
+  nsIView* view = mOuter->GetView();
   PRBool result;
-  //nsCOMPtr<nsIWidget> widget;
 
   if (view) {
-    view->GetViewManager(*getter_AddRefs(viewMan));
-    //view->GetWidget(*getter_AddRefs(widget));
+    nsIViewManager* viewMan = view->GetViewManager();
     if (viewMan) {
+      // nsIWidget* widget = view->GetWidget();
       if (aGrabMouseEvents) {
         viewMan->GrabMouseEvents(view,result);
       //  if (widget)
@@ -725,11 +715,10 @@ PRBool
 nsSplitterFrameInner :: IsMouseCaptured(nsIPresContext* aPresContext)
 {
     // get its view
-  nsIView* view = mOuter->GetView(aPresContext);
-  nsCOMPtr<nsIViewManager> viewMan;
+  nsIView* view = mOuter->GetView();
   
   if (view) {
-    view->GetViewManager(*getter_AddRefs(viewMan));
+    nsIViewManager* viewMan = view->GetViewManager();
 
     if (viewMan) {
         nsIView* grabbingView;
@@ -765,11 +754,9 @@ nsSplitterFrameInner::MouseDown(nsIDOMEvent* aMouseEvent)
   if (button != 0)
      return NS_OK;
 
-  nsCOMPtr<nsIContent> content;  
-  mOuter->GetContent(getter_AddRefs(content));
-
   nsAutoString disabled;
-  content->GetAttr(kNameSpaceID_None, nsHTMLAtoms::disabled, disabled);
+  mOuter->GetContent()->GetAttr(kNameSpaceID_None,
+                                nsHTMLAtoms::disabled, disabled);
   if (disabled.Equals(NS_LITERAL_STRING("true")))
     return NS_OK;
 
@@ -816,7 +803,7 @@ nsSplitterFrameInner::MouseDown(nsIDOMEvent* aMouseEvent)
     nsIFrame* childFrame = nsnull;
     childBox->GetFrame(&childFrame);
 
-    childFrame->GetContent(getter_AddRefs(content));
+    nsIContent* content = childFrame->GetContent();
     nsCOMPtr<nsIAtom> atom;
     nsresult rv;
     nsCOMPtr<nsIXBLService> xblService = 
@@ -919,9 +906,7 @@ nsSplitterFrameInner::MouseDown(nsIDOMEvent* aMouseEvent)
   if (resizeAfter == Grow)
      mChildInfosAfterCount = 0;
 
-  nsRect vr(0,0,0,0);
-  nsIView *v = mOuter->GetView(mOuter->mPresContext);
-  v->GetBounds(vr);
+  nsRect vr = mOuter->GetView()->GetBounds();
 
   PRInt32 c = 0;
   if (isHorizontal) {
@@ -1019,9 +1004,8 @@ nsSplitterFrameInner::UpdateState()
       if (splitterSibling) {
         nsIFrame* splitterSiblingFrame = nsnull;
         splitterSibling->GetFrame(&splitterSiblingFrame);
-        nsCOMPtr<nsIContent> sibling;
-        if (NS_SUCCEEDED(splitterSiblingFrame->GetContent(getter_AddRefs(sibling)))
-            && sibling) {
+        nsIContent* sibling = splitterSiblingFrame->GetContent();
+        if (sibling) {
           if (mState == Collapsed) {
             // Collapsed -> Open
             // Collapsed -> Dragging
@@ -1046,10 +1030,8 @@ nsSplitterFrameInner::EnsureOrient()
 {
   nsIFrame* frame = nsnull;
   mParentBox->GetFrame(&frame);
-  nsFrameState state;
-  frame->GetFrameState(&state);
 
-  PRBool isHorizontal = !(state & NS_STATE_IS_HORIZONTAL);
+  PRBool isHorizontal = !(frame->GetStateBits() & NS_STATE_IS_HORIZONTAL);
   if (isHorizontal)
       mOuter->mState |= NS_STATE_IS_HORIZONTAL;
   else
@@ -1075,15 +1057,14 @@ if (realTimeDrag) {
     nsIFrame* frame = nsnull;
     mParentBox->GetFrame(&frame);
 
-    nsCOMPtr<nsIViewManager> viewManager;
-    nsIView* view = frame->GetView(aPresContext);
+    nsIView* view = frame->GetView();
 
     if (!view) {
         nsPoint   offset;
         frame->GetOffsetFromView(aPresContext, offset, &view);
         NS_ASSERTION(nsnull != view, "no view");
     }
-    view->GetViewManager(*getter_AddRefs(viewManager));
+    nsIViewManager* viewManager = view->GetViewManager();
 
     viewManager->DisableRefresh();
     shell->FlushPendingNotifications(PR_FALSE);
@@ -1171,8 +1152,7 @@ nsSplitterFrameInner::SetPreferredSize(nsBoxLayoutState& aState, nsIBox* aChildB
   nsIFrame* childFrame = nsnull;
   aChildBox->GetFrame(&childFrame);
 
-  nsCOMPtr<nsIContent> content;
-  childFrame->GetContent(getter_AddRefs(content));
+  nsIContent* content = childFrame->GetContent();
 
   // set its preferred size.
   nsAutoString prefValue;
@@ -1258,20 +1238,18 @@ void
 nsSplitterFrameInner::MoveSplitterBy(nsIPresContext* aPresContext, nscoord aDiff)
 {
   const nsRect& r = mOuter->mRect;
-  nsRect vr;
-  nsCOMPtr<nsIViewManager> vm;
-  nsIView *v = mOuter->GetView(aPresContext);
-  v->GetViewManager(*getter_AddRefs(vm));
-  v->GetBounds(vr);
+  nsIView *v = mOuter->GetView();
+  nsIViewManager* vm = v->GetViewManager();
+  nsRect vr = v->GetBounds();
   nsRect invalid;
   EnsureOrient();
   PRBool isHorizontal = !mOuter->IsHorizontal();
   if (isHorizontal) {
-      mOuter->MoveTo(aPresContext, mSplitterPos + aDiff, r.y);
+      mOuter->SetPosition(nsPoint(mSplitterPos + aDiff, r.y));
       vm->MoveViewTo(v, mSplitterViewPos + aDiff, vr.y);
       invalid.UnionRect(r,mOuter->mRect);
   } else {
-      mOuter->MoveTo(aPresContext, r.x, mSplitterPos + aDiff);
+      mOuter->SetPosition(nsPoint(r.x, mSplitterPos + aDiff));
       vm->MoveViewTo(v, vr.x, mSplitterViewPos + aDiff);
       invalid.UnionRect(r,mOuter->mRect);
   }
