@@ -171,9 +171,7 @@ nsTitleBarFrame::HandleEvent(nsIPresContext* aPresContext,
 			   aPresContext->GetShell(getter_AddRefs(presShell));
 			   nsCOMPtr<nsIDocument> document;
 			   presShell->GetDocument(getter_AddRefs(document));
-			   nsCOMPtr<nsIScriptGlobalObject> scriptGlobalObject;
-			   document->GetScriptGlobalObject(getter_AddRefs(scriptGlobalObject));
-			   nsCOMPtr<nsIDOMWindowInternal> window(do_QueryInterface(scriptGlobalObject));
+			   nsCOMPtr<nsIDOMWindowInternal> window(do_QueryInterface(document->GetScriptGlobalObject()));
 
 
 
@@ -209,16 +207,13 @@ NS_IMETHODIMP
 nsTitleBarFrame::CaptureMouseEvents(nsIPresContext* aPresContext,PRBool aGrabMouseEvents)
 {
 	// get its view
-  nsIView* view = GetView(aPresContext);
-  nsCOMPtr<nsIViewManager> viewMan;
+  nsIView* view = GetView();
   PRBool result;
 
-  nsCOMPtr<nsIWidget> widget;
-
   if (view) {
-    view->GetViewManager(*getter_AddRefs(viewMan));
+    nsIViewManager* viewMan = view->GetViewManager();
     if (viewMan) {
-      view->GetWidget(*getter_AddRefs(widget));
+      // nsIWidget* widget = view->GetWidget();
       if (aGrabMouseEvents) {
         viewMan->GrabMouseEvents(view,result);
         //mIsCapturingMouseEvents = PR_TRUE;
