@@ -20,8 +20,8 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
- *   Jan Varga (varga@utcru.sk)
- *   Brian Ryner <bryner@netscape.com>
+ *   Jan Varga (varga@nixcorp.com)
+ *   Brian Ryner <bryner@brianryner.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or 
@@ -551,8 +551,7 @@ nsTreeContentView::SetTree(nsITreeBoxObject* aTree)
     mRoot = do_QueryInterface(element);
 
     // Add ourselves to document's observers.
-    nsCOMPtr<nsIDocument> document;
-    mRoot->GetDocument(getter_AddRefs(document));
+    nsIDocument* document = mRoot->GetDocument();
     if (document) {
       document->AddObserver(this);
       mDocument = document;
@@ -688,13 +687,13 @@ NS_IMPL_NSIDOCUMENTOBSERVER_REFLOW_STUB(nsTreeContentView)
 NS_IMPL_NSIDOCUMENTOBSERVER_STYLE_STUB(nsTreeContentView)
 
 NS_IMETHODIMP
-nsTreeContentView::BeginUpdate(nsIDocument *aDocument)
+nsTreeContentView::BeginUpdate(nsIDocument *aDocument, nsUpdateType aUpdateType)
 {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsTreeContentView::EndUpdate(nsIDocument *aDocument)
+nsTreeContentView::EndUpdate(nsIDocument *aDocument, nsUpdateType aUpdateType)
 {
   return NS_OK;
 }
@@ -735,8 +734,7 @@ nsTreeContentView::AttributeChanged(nsIDocument *aDocument,
                                     nsIContent*  aContent,
                                     PRInt32      aNameSpaceID,
                                     nsIAtom*     aAttribute,
-                                    PRInt32      aModType, 
-                                    nsChangeHint aHint)
+                                    PRInt32      aModType)
 {
   // Make sure this notification concerns us.
   // First check the tag to see if it's one that we care about.
@@ -758,8 +756,7 @@ nsTreeContentView::AttributeChanged(nsIDocument *aDocument,
   nsCOMPtr<nsIContent> parent = aContent;
   nsCOMPtr<nsIAtom> parentTag;
   do {
-    nsCOMPtr<nsIContent> temp = parent;
-    temp->GetParent(getter_AddRefs(parent));
+    parent = parent->GetParent();
     if (parent)
       parent->GetTag(getter_AddRefs(parentTag));
   } while (parent && parentTag != nsXULAtoms::tree);
@@ -784,11 +781,9 @@ nsTreeContentView::AttributeChanged(nsIDocument *aDocument,
     }
     else if (!hidden && index < 0) {
       // Show this row along with its children.
-      nsCOMPtr<nsIContent> container;
-      aContent->GetParent(getter_AddRefs(container));
+      nsCOMPtr<nsIContent> container = aContent->GetParent();
       if (container) {
-        nsCOMPtr<nsIContent> parent;
-        container->GetParent(getter_AddRefs(parent));
+        nsCOMPtr<nsIContent> parent = container->GetParent();
         if (parent)
           InsertRowFor(parent, container, aContent);
       }
@@ -844,8 +839,7 @@ nsTreeContentView::AttributeChanged(nsIDocument *aDocument,
   }
   else if (tag == nsXULAtoms::treerow) {
     if (aAttribute == nsXULAtoms::properties) {
-      nsCOMPtr<nsIContent> parent;
-      aContent->GetParent(getter_AddRefs(parent));
+      nsCOMPtr<nsIContent> parent = aContent->GetParent();
       if (parent) {
         PRInt32 index = FindContent(parent);
         if (index >= 0) {
@@ -860,11 +854,9 @@ nsTreeContentView::AttributeChanged(nsIDocument *aDocument,
         aAttribute == nsXULAtoms::mode ||
         aAttribute == nsHTMLAtoms::value ||
         aAttribute == nsHTMLAtoms::label) {
-      nsCOMPtr<nsIContent> parent;
-      aContent->GetParent(getter_AddRefs(parent));
+      nsIContent* parent = aContent->GetParent();
       if (parent) {
-        nsCOMPtr<nsIContent> grandParent;
-        parent->GetParent(getter_AddRefs(grandParent));
+        nsCOMPtr<nsIContent> grandParent = parent->GetParent();
         if (grandParent) {
           PRInt32 index = FindContent(grandParent);
           if (index >= 0) {
@@ -884,8 +876,7 @@ nsTreeContentView::ContentAppended(nsIDocument *aDocument,
                                    nsIContent* aContainer,
                                    PRInt32     aNewIndexInContainer)
 {
-  nsCOMPtr<nsIContent> child;
-  aContainer->ChildAt(aNewIndexInContainer, getter_AddRefs(child));
+  nsIContent *child = aContainer->GetChildAt(aNewIndexInContainer);
   ContentInserted(aDocument, aContainer, child, aNewIndexInContainer);
 
   return NS_OK;
@@ -920,10 +911,9 @@ nsTreeContentView::ContentInserted(nsIDocument *aDocument,
 
   // If we have a legal tag, go up to the tree/select and make sure
   // that it's ours.
-  nsCOMPtr<nsIContent> element = aContainer;
   nsCOMPtr<nsIAtom> parentTag;
   
-  while (element) {
+  for (nsIContent* element = aContainer; element; element = element->GetParent()) {
     element->GetTag(getter_AddRefs(parentTag));
     if ((element->IsContentOfType(nsIContent::eXUL) && parentTag == nsXULAtoms::tree) ||
         (element->IsContentOfType(nsIContent::eHTML) && parentTag == nsHTMLAtoms::select))
@@ -931,14 +921,11 @@ nsTreeContentView::ContentInserted(nsIDocument *aDocument,
         break;
       else // this is not for us, we can bail out
         return NS_OK;
-    nsCOMPtr<nsIContent> temp = element;
-    temp->GetParent(getter_AddRefs(element));
   }
 
   if (childTag == nsXULAtoms::treeitem ||
       childTag == nsXULAtoms::treeseparator) {
-    nsCOMPtr<nsIContent> parent;
-    aContainer->GetParent(getter_AddRefs(parent));
+    nsCOMPtr<nsIContent> parent = aContainer->GetParent();
     if (parent)
       InsertRowFor(parent, aContainer, aChild);
   }
@@ -970,8 +957,7 @@ nsTreeContentView::ContentInserted(nsIDocument *aDocument,
       mBoxObject->InvalidateRow(index);
   }
   else if (childTag == nsXULAtoms::treecell) {
-    nsCOMPtr<nsIContent> parent;
-    aContainer->GetParent(getter_AddRefs(parent));
+    nsCOMPtr<nsIContent> parent = aContainer->GetParent();
     if (parent) {
       PRInt32 index = FindContent(parent);
       if (index >= 0)
@@ -1024,10 +1010,9 @@ nsTreeContentView::ContentRemoved(nsIDocument *aDocument,
 
   // If we have a legal tag, go up to the tree/select and make sure
   // that it's ours.
-  nsCOMPtr<nsIContent> element = aContainer;
   nsCOMPtr<nsIAtom> parentTag;
   
-  while (element) {
+  for (nsIContent* element = aContainer; element; element = element->GetParent()) {
     element->GetTag(getter_AddRefs(parentTag));
     if ((element->IsContentOfType(nsIContent::eXUL) && parentTag == nsXULAtoms::tree) || 
         (element->IsContentOfType(nsIContent::eHTML) && parentTag == nsHTMLAtoms::select))
@@ -1035,8 +1020,6 @@ nsTreeContentView::ContentRemoved(nsIDocument *aDocument,
         break;
       else // this is not for us, we can bail out
         return NS_OK;
-    nsCOMPtr<nsIContent> temp = element;
-    temp->GetParent(getter_AddRefs(element));
   }
 
   if (tag == nsXULAtoms::treeitem ||
@@ -1075,8 +1058,7 @@ nsTreeContentView::ContentRemoved(nsIDocument *aDocument,
       mBoxObject->InvalidateRow(index);
   }
   else if (tag == nsXULAtoms::treecell) {
-    nsCOMPtr<nsIContent> parent;
-    aContainer->GetParent(getter_AddRefs(parent));
+    nsCOMPtr<nsIContent> parent = aContainer->GetParent();
     if (parent) {
       PRInt32 index = FindContent(parent);
       if (index >= 0)
@@ -1219,13 +1201,12 @@ nsTreeContentView::SerializeOptGroup(nsIContent* aContent, PRInt32 aParentIndex,
 }
 
 void
-nsTreeContentView::GetIndexInSubtree(nsIContent* aContainer, nsIContent* aContent, PRInt32* aIndex)
+nsTreeContentView::GetIndexInSubtree(nsIContent* aContainer,
+                                     nsIContent* aContent, PRInt32* aIndex)
 {
-  PRInt32 childCount;
-  aContainer->ChildCount(childCount);
-  for (PRInt32 i = 0; i < childCount; i++) {
-    nsCOMPtr<nsIContent> content;
-    aContainer->ChildAt(i, getter_AddRefs(content));
+  PRUint32 childCount = aContainer->GetChildCount();
+  for (PRUint32 i = 0; i < childCount; i++) {
+    nsIContent *content = aContainer->GetChildAt(i);
 
     if (content == aContent)
       break;
