@@ -111,17 +111,27 @@ function HandleColumnClick(columnID)
 
   // if sortType is 0, this is an unsupported sort type
   // return, since we can't sort by that column.
-  if (sortType == 0) {
+  if (sortType == 0)
     return;
-  }
 
   var dbview = GetDBView();
+  // if we're already threaded, clicking the thread icon does a reverse sort by id.
+  if (sortType == nsMsgViewSortType.byThread && (dbview.viewFlags & nsMsgViewFlagsType.kThreadedDisplay))
+    sortType = nsMsgViewSortType.byId;
   if (sortType == nsMsgViewSortType.byThread && !dbview.supportsThreading)
       return;
   if (dbview.sortType == sortType) {
     MsgReverseSortThreadPane();
   }
   else {
+    try {
+      var flatSort = pref.getBoolPref("mailnews.thread_pane_column_unthreads"); 
+      if (flatSort)
+        dbview.viewFlags &= ~nsMsgViewFlagsType.kThreadedDisplay;
+    }
+    catch (ex) {
+    }
+
     MsgSortThreadPane(sortType);
   }
 }
@@ -160,14 +170,14 @@ function MsgSortByDate()
     MsgSortThreadPane(nsMsgViewSortType.byDate);
 }
 
-function MsgSortBySenderOrRecipient()
+function MsgSortBySender()
 {
-    if (IsSpecialFolderSelected(MSG_FOLDER_FLAG_SENTMAIL | MSG_FOLDER_FLAG_DRAFTS | MSG_FOLDER_FLAG_QUEUE)) {
-      MsgSortThreadPane(nsMsgViewSortType.byRecipient);
-    }
-    else {
-      MsgSortThreadPane(nsMsgViewSortType.byAuthor);
-    }
+    MsgSortThreadPane(nsMsgViewSortType.byAuthor);
+}
+
+function MsgSortByRecipient()
+{
+    MsgSortThreadPane(nsMsgViewSortType.byRecipient);
 }
 
 function MsgSortByStatus()
@@ -253,6 +263,15 @@ function MsgReverseSortThreadPane()
   }
 }
 
+function MsgToggleThreaded()
+{
+  var dbview = GetDBView();
+	curFlags = dbview.viewFlags;
+	dbview.viewFlags = dbview.viewFlags ^ nsMsgViewFlagsType.kThreadedDisplay;
+	dbview.sort(dbview.sortType, dbview.sortOrder); // resort
+  UpdateSortIndicators(dbview.sortType, nsMsgViewSortOrder.ascending);
+}
+
 function MsgSortAscending()
 {
   var dbview = GetDBView();
@@ -271,7 +290,7 @@ function UpdateSortIndicators(sortType, sortOrder)
 {
   // show the twisties if the view is threaded
   var currCol = document.getElementById("subjectCol");
-  var primary = (sortType == nsMsgViewSortType.byThread) && gDBView.supportsThreading;
+  var primary = (gDBView.viewFlags & nsMsgViewFlagsType.kThreadedDisplay) && gDBView.supportsThreading;
   currCol.setAttribute("primary", primary);
 
   // remove the sort indicator from all the columns
@@ -292,6 +311,17 @@ function UpdateSortIndicators(sortType, sortOrder)
       else {
         sortedColumn.setAttribute("sortDirection","descending");
       }
+			if (sortedColumn != "threadCol")
+			{
+			  currCol = document.getElementById("threadCol");
+				if (currCol)
+				{
+					if (gDBView.viewFlags & nsMsgViewFlagsType.kThreadedDisplay)
+						currCol.setAttribute("sortDirection", "ascending");
+					else
+						currCol.removeAttribute("sortDirection");
+				}
+			}
     }
   }
 }
@@ -343,7 +373,7 @@ function ThreadPaneOnLoad()
   var tree = GetThreadTree();
 
   tree.addEventListener("click",ThreadPaneOnClick,true);
-
+  
   // The mousedown event listener below should only be added in the thread
   // pane of the mailnews 3pane window, not in the advanced search window.
   if(tree.parentNode.id == "searchResultListBox")
