@@ -74,7 +74,7 @@ nsMsgRuleAction::~nsMsgRuleAction()
 
 NS_IMPL_ISUPPORTS1(nsMsgRuleAction, nsIMsgRuleAction)
 
-NS_IMPL_GETSET(nsMsgRuleAction, Type, nsMsgRuleActionType, m_type);
+NS_IMPL_GETSET(nsMsgRuleAction, Type, nsMsgRuleActionType, m_type)
 
 NS_IMETHODIMP nsMsgRuleAction::SetPriority(nsMsgPriorityValue aPriority)
 {
@@ -150,10 +150,10 @@ nsMsgFilter::~nsMsgFilter()
 
 NS_IMPL_ISUPPORTS1(nsMsgFilter, nsIMsgFilter)
 
-NS_IMPL_GETSET(nsMsgFilter, FilterType, nsMsgFilterTypeType, m_type);
-NS_IMPL_GETSET(nsMsgFilter, Enabled, PRBool, m_enabled);
-NS_IMPL_GETSET(nsMsgFilter, Temporary, PRBool, m_temporary);
-NS_IMPL_GETSET(nsMsgFilter, Unparseable, PRBool, m_unparseable);
+NS_IMPL_GETSET(nsMsgFilter, FilterType, nsMsgFilterTypeType, m_type)
+NS_IMPL_GETSET(nsMsgFilter, Enabled, PRBool, m_enabled)
+NS_IMPL_GETSET(nsMsgFilter, Temporary, PRBool, m_temporary)
+NS_IMPL_GETSET(nsMsgFilter, Unparseable, PRBool, m_unparseable)
 
 NS_IMETHODIMP nsMsgFilter::GetFilterName(PRUnichar **name)
 {
@@ -434,6 +434,7 @@ NS_IMETHODIMP nsMsgFilter::LogRuleHit(nsIMsgRuleAction *aFilterAction, nsIMsgDBH
 NS_IMETHODIMP nsMsgFilter::MatchHdr(nsIMsgDBHdr	*msgHdr, nsIMsgFolder *folder, nsIMsgDatabase *db, 
 									const char *headers, PRUint32 headersSize, PRBool *pResult)
 {
+  NS_ENSURE_ARG_POINTER(folder);
     // use offlineMail because
     nsMsgSearchScopeTerm* scope = new nsMsgSearchScopeTerm(nsnull, nsMsgSearchScope::offlineMail, folder);
     if (!scope) return NS_ERROR_OUT_OF_MEMORY;
