@@ -78,7 +78,6 @@ const char *nsMsgSearchAdapter::m_kImapNotSeen = " UNSEEN ";
 const char *nsMsgSearchAdapter::m_kImapNotAnswered = " UNANSWERED ";
 const char *nsMsgSearchAdapter::m_kImapCharset = " CHARSET ";
 
-static NS_DEFINE_CID(kIOServiceCID, NS_IOSERVICE_CID);
 #define PREF_CUSTOM_HEADERS "mailnews.customHeaders"
 
 NS_IMETHODIMP nsMsgSearchAdapter::FindTargetFolder(const nsMsgResultElement *,nsIMsgFolder * *)
@@ -349,6 +348,7 @@ nsMsgSearchAdapter::GetSearchCharsets(PRUnichar **srcCharset, PRUnichar **dstCha
         {
             nsXPIDLCString folderCharset;
             folder->GetCharset(getter_Copies(folderCharset));
+      PR_Free(*dstCharset);
             *dstCharset = ToNewUnicode(folderCharset);
         }
 	}
@@ -360,7 +360,10 @@ nsMsgSearchAdapter::GetSearchCharsets(PRUnichar **srcCharset, PRUnichar **dstCha
 	// was undefined or unavailable.)
   // ### well, it's not really anymore. Is there an equivalent?
   if (!nsCRT::strcmp(*dstCharset, m_defaultCharset.get()))
+  {
+    PR_Free(*dstCharset);
     *dstCharset = nsCRT::strdup(*srcCharset);
+  }
 
 	if (m_forceAsciiSearch)
 	{
@@ -371,6 +374,7 @@ nsMsgSearchAdapter::GetSearchCharsets(PRUnichar **srcCharset, PRUnichar **dstCha
 		// If the dest csid is ISO Latin 1 or MacRoman, attempt to convert the 
 		// source text to US-ASCII. (Not for now.)
 		// if ((dst_csid == CS_LATIN1) || (dst_csid == CS_MAC_ROMAN))
+    PR_Free(*dstCharset);
     *dstCharset = nsCRT::strdup(NS_LITERAL_STRING("us-ascii").get());
 	}
   return NS_OK;
