@@ -138,13 +138,13 @@
 #include "nsIPrefBranch.h"
 #include "nsIPrefBranchInternal.h"
 #include "nsCExternalHandlerService.h"
+#include "nsIExternalProtocolService.h"
 #include "nsIMIMEService.h"
 
 #include "nsILinkHandler.h"                                                                              
 
 static NS_DEFINE_CID(kRDFServiceCID,	NS_RDFSERVICE_CID);
 static NS_DEFINE_CID(kMsgSendLaterCID, NS_MSGSENDLATER_CID); 
-static NS_DEFINE_CID(kMsgPrintEngineCID,		NS_MSG_PRINTENGINE_CID);
 
 #define FOUR_K 4096
 #define MESSENGER_SAVE_DIR_PREF_NAME "messenger.save.dir"
@@ -317,7 +317,7 @@ nsMessenger::~nsMessenger()
 
 
 NS_IMPL_ISUPPORTS3(nsMessenger, nsIMessenger, nsIObserver, nsISupportsWeakReference)
-NS_IMPL_GETSET(nsMessenger, SendingUnsentMsgs, PRBool, mSendingUnsentMsgs);
+NS_IMPL_GETSET(nsMessenger, SendingUnsentMsgs, PRBool, mSendingUnsentMsgs)
 
 NS_IMETHODIMP    
 nsMessenger::SetWindow(nsIDOMWindowInternal *aWin, nsIMsgWindow *aMsgWindow)
@@ -331,80 +331,82 @@ nsMessenger::SetWindow(nsIDOMWindowInternal *aWin, nsIMsgWindow *aMsgWindow)
     if (prefBranch)
       pbi = do_QueryInterface(prefBranch);
   }
-	if(!aWin)
-	{
+  if(!aWin)
+  {
     // it isn't an error to pass in null for aWin, in fact it means we are shutting
     // down and we should start cleaning things up...
-
-		if (mMsgWindow)
-		{
-			nsCOMPtr<nsIMsgStatusFeedback> aStatusFeedback;
-
-			mMsgWindow->GetStatusFeedback(getter_AddRefs(aStatusFeedback));
-			if (aStatusFeedback)
-				aStatusFeedback->SetDocShell(nsnull, nsnull);
-
+    
+    if (mMsgWindow)
+    {
+      nsCOMPtr<nsIMsgStatusFeedback> aStatusFeedback;
+      
+      mMsgWindow->GetStatusFeedback(getter_AddRefs(aStatusFeedback));
+      if (aStatusFeedback)
+        aStatusFeedback->SetDocShell(nsnull, nsnull);
+      
       // Remove pref observer
       if (pbi)
         pbi->RemoveObserver(MAILNEWS_ALLOW_PLUGINS_PREF_NAME, this);
-		}
-
-		return NS_OK;
-	}
-
+    }
+    
+    return NS_OK;
+  }
+  
   mMsgWindow = aMsgWindow;
-
+  
   NS_IF_RELEASE(mWindow);
   mWindow = aWin;
   NS_ADDREF(aWin);
-
+  
   nsCOMPtr<nsIScriptGlobalObject> globalObj( do_QueryInterface(aWin) );
   NS_ENSURE_TRUE(globalObj, NS_ERROR_FAILURE);
-
+  
   nsCOMPtr<nsIDocShell> docShell;
   globalObj->GetDocShell(getter_AddRefs(docShell));
   nsCOMPtr<nsIDocShellTreeItem> docShellAsItem(do_QueryInterface(docShell));
   NS_ENSURE_TRUE(docShellAsItem, NS_ERROR_FAILURE);
-
+  
   nsCOMPtr<nsIDocShellTreeItem> rootDocShellAsItem;
   docShellAsItem->GetSameTypeRootTreeItem(getter_AddRefs(rootDocShellAsItem));
+  
+  nsCOMPtr<nsIDocShellTreeNode> rootDocShellAsNode(do_QueryInterface(rootDocShellAsItem));
 
-  nsCOMPtr<nsIDocShellTreeNode> 
-                     rootDocShellAsNode(do_QueryInterface(rootDocShellAsItem));
   if (rootDocShellAsNode) 
   {
     nsCOMPtr<nsIDocShellTreeItem> childAsItem;
     nsresult rv = rootDocShellAsNode->FindChildWithName(NS_LITERAL_STRING("messagepane").get(),
       PR_TRUE, PR_FALSE, nsnull, getter_AddRefs(childAsItem));
-
+    
     mDocShell = do_QueryInterface(childAsItem);
-
+    
     if (NS_SUCCEEDED(rv) && mDocShell) {
+      
+      mCurrentDisplayCharset = ""; // Important! Clear out mCurrentDisplayCharset so we reset a default charset on mDocshell the next time we try to load something into it.
+      
+      if (aMsgWindow) 
+      {
+        nsCOMPtr<nsIMsgStatusFeedback> aStatusFeedback;
+        
+        aMsgWindow->GetStatusFeedback(getter_AddRefs(aStatusFeedback));
+        if (aStatusFeedback)
+          aStatusFeedback->SetDocShell(mDocShell, mWindow);
 
-        if (aMsgWindow) {
-            nsCOMPtr<nsIMsgStatusFeedback> aStatusFeedback;
-            
-            aMsgWindow->GetStatusFeedback(getter_AddRefs(aStatusFeedback));
-            if (aStatusFeedback)
-            {
-                aStatusFeedback->SetDocShell(mDocShell, mWindow);
-            }
-            aMsgWindow->GetTransactionManager(getter_AddRefs(mTxnMgr));
-
-            // Add pref observer
-            if (pbi)
-              pbi->AddObserver(MAILNEWS_ALLOW_PLUGINS_PREF_NAME, this, PR_TRUE);
-            SetDisplayProperties();
-        }
+        aMsgWindow->GetTransactionManager(getter_AddRefs(mTxnMgr));
+        
+        // Add pref observer
+        if (pbi)
+          pbi->AddObserver(MAILNEWS_ALLOW_PLUGINS_PREF_NAME, this, PR_TRUE);
+        SetDisplayProperties();
+      }
     }
   }
-
+  
   // we don't always have a message pane, like in the addressbook
   // so if we don't havea docshell, use the one for the xul window.
   // we do this so OpenURL() will work.
   if (!mDocShell)
     mDocShell = docShell;
-
+  
   return NS_OK;
 }
 
@@ -421,11 +423,9 @@ NS_IMETHODIMP nsMessenger::SetDisplayCharset(const char * aCharset)
     if (cv) 
     {
       nsCOMPtr<nsIMarkupDocumentViewer> muDV = do_QueryInterface(cv);
-      if (muDV) {
+      if (muDV) 
         muDV->SetForceCharacterSet(nsDependentCString(aCharset));
 
-      }
-
       mCurrentDisplayCharset = aCharset;
     }
   }
@@ -581,7 +581,7 @@ nsMessenger::OpenURL(const char *aURL)
     if(webNav)
       rv = webNav->LoadURI(NS_ConvertASCIItoUCS2(unescapedUrl).get(), // URI string
       nsIWebNavigation::LOAD_FLAGS_NONE,  // Load flags
-      nsnull,                             // Refering URI
+      nsnull,                             // Referring URI
       nsnull,                             // Post stream
       nsnull);                            // Extra headers
   }
@@ -589,6 +589,19 @@ nsMessenger::OpenURL(const char *aURL)
   return rv;
 }
 
+NS_IMETHODIMP nsMessenger::LaunchExternalURL(const char * aURL)
+{
+  nsresult rv = NS_OK;
+  
+  nsCOMPtr<nsIURI> uri;
+  rv = NS_NewURI(getter_AddRefs(uri), aURL);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr<nsIExternalProtocolService> extProtService = do_GetService(NS_EXTERNALPROTOCOLSERVICE_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv,rv);
+  return extProtService->LoadUrl(uri); 
+}
+
 NS_IMETHODIMP
 nsMessenger::LoadURL(nsIDOMWindowInternal *aWin, const char *aURL)
 {
@@ -606,24 +619,8 @@ nsMessenger::LoadURL(nsIDOMWindowInternal *aWin, const char *aURL)
   nsCOMPtr<nsIURI> uri;
   nsresult rv = NS_NewURI(getter_AddRefs(uri), uriString);
   NS_ENSURE_SUCCESS(rv, rv);
-  
-  // cheat....if we were given a dom window, then use the docshell from it
-  // and pass the url out as a link...this is really just used by stand alone
-  // mail right now and could be wrapped in a MOZ_THUNDERBIRD ifdef if we needed to.
-  if (aWin)
-  {
-    nsCOMPtr<nsIScriptGlobalObject> globalObj = do_QueryInterface(aWin, &rv);    
-    NS_ENSURE_SUCCESS(rv,rv);                                                    
-    nsCOMPtr <nsIDocShell> docShell; 
-    rv = globalObj->GetDocShell(getter_AddRefs(docShell));  
-    NS_ENSURE_SUCCESS(rv,rv);
-    nsCOMPtr<nsILinkHandler> lh = do_QueryInterface(docShell, &rv);              
-    NS_ENSURE_SUCCESS(rv,rv); 
-    return rv = lh->OnLinkClick(nsnull, eLinkVerb_Replace, uri,nsnull,nsnull,nsnull);                                                     
-  }
-  else
-  {
-    NS_ENSURE_TRUE(mDocShell, NS_ERROR_FAILURE);
+
+  NS_ENSURE_TRUE(mDocShell, NS_ERROR_FAILURE);
   nsCOMPtr<nsIMsgMailNewsUrl> msgurl = do_QueryInterface(uri);
   if (msgurl)
     msgurl->SetMsgWindow(mMsgWindow);
@@ -633,7 +630,6 @@ nsMessenger::LoadURL(nsIDOMWindowInternal *aWin, const char *aURL)
   NS_ENSURE_SUCCESS(rv, rv);
   loadInfo->SetLoadType(nsIDocShellLoadInfo::loadNormal);
   return mDocShell->LoadURI(uri, loadInfo, 0, PR_TRUE);
-  }
 }
 
 nsresult
@@ -889,7 +885,7 @@ enum MESSENGER_SAVEAS_FILE_TYPE
 #define EML_FILE_EXTENSION  ".eml"
 
 NS_IMETHODIMP
-nsMessenger::SaveAs(const char *aURI, PRBool aAsFile, nsIMsgIdentity *aIdentity, nsIMsgWindow *aMsgWindow)
+nsMessenger::SaveAs(const char *aURI, PRBool aAsFile, nsIMsgIdentity *aIdentity, const PRUnichar *aMsgFilename)
 {
   NS_ENSURE_ARG_POINTER(aURI);
   
@@ -911,8 +907,12 @@ nsMessenger::SaveAs(const char *aURI, PRBool aAsFile, nsIMsgIdentity *aIdentity,
       goto done;
     
     filePicker->Init(nsnull, GetString(NS_LITERAL_STRING("SaveMailAs").get()), nsIFilePicker::modeSave);
-    
-    filePicker->SetDefaultString(GetString(NS_LITERAL_STRING("defaultSaveMessageAsFileName").get()));
+
+    // if we have a non-null filename use it, otherwise use default save message one
+    if (aMsgFilename)    
+      filePicker->SetDefaultString(aMsgFilename);
+    else
+      filePicker->SetDefaultString(GetString(NS_LITERAL_STRING("defaultSaveMessageAsFileName").get()));
     
     // because we will be using GetFilterIndex()
     // we must call AppendFilters() one at a time, 
@@ -1198,36 +1198,6 @@ nsMessenger::DoCommand(nsIRDFCompositeDataSource* db, const nsACString& command,
 
 }
 
-NS_IMETHODIMP
-nsMessenger::DeleteMessages(nsIRDFCompositeDataSource *database,
-                            nsIRDFResource *srcFolderResource,
-                            nsISupportsArray *resourceArray,
-							PRBool reallyDelete)
-{
-	nsresult rv;
-
-	if(!database || !srcFolderResource || !resourceArray)
-		return NS_ERROR_NULL_POINTER;
-
-	nsCOMPtr<nsISupportsArray> folderArray;
-
-	rv = NS_NewISupportsArray(getter_AddRefs(folderArray));
-	if(NS_FAILED(rv))
-	{
-		return NS_ERROR_OUT_OF_MEMORY;
-	}
-
-	folderArray->AppendElement(srcFolderResource);
-	
-	if(reallyDelete)
-		rv = DoCommand(database, NS_LITERAL_CSTRING(NC_RDF_REALLY_DELETE), folderArray, resourceArray);
-	else
-		rv = DoCommand(database, NS_LITERAL_CSTRING(NC_RDF_DELETE), folderArray, resourceArray);
-
-
-	return rv;
-}
-
 NS_IMETHODIMP nsMessenger::DeleteFolders(nsIRDFCompositeDataSource *db,
                                          nsIRDFResource *parentResource,
                                          nsIRDFResource *deletedFolderResource)
@@ -1571,7 +1541,7 @@ NS_IMPL_ISUPPORTS1(SendLaterListener, nsIMsgSendLaterListener)
 
 SendLaterListener::SendLaterListener(nsIMessenger *aMessenger)
 {
-  m_messenger = getter_AddRefs(NS_GetWeakReference(aMessenger));
+  m_messenger = do_GetWeakReference(aMessenger);
 }
 
 SendLaterListener::~SendLaterListener()
@@ -1808,7 +1778,7 @@ nsSaveMsgListener::OnStartRequest(nsIRequest* request, nsISupports* aSupport)
       if (mimeService)
       {
         nsCOMPtr<nsIMIMEInfo> mimeinfo;
-        if (NS_SUCCEEDED(mimeService->GetFromMIMEType(m_contentType.get(), getter_AddRefs(mimeinfo))))
+        if (NS_SUCCEEDED(mimeService->GetFromTypeAndExtension(m_contentType.get(), nsnull, getter_AddRefs(mimeinfo))))
         {
           PRUint32 aMacType;
           PRUint32 aMacCreator;
