@@ -38,7 +38,6 @@
 #ifndef __nsMsgAppCore_h
 #define __nsMsgAppCore_h
 
-#include "nsCom.h"
 #include "nscore.h"
 #include "nsIMessenger.h"
 #include "nsCOMPtr.h"
@@ -69,7 +68,7 @@ public:
 
 protected:
   nsresult DoDelete(nsIRDFCompositeDataSource* db, nsISupportsArray *srcArray, nsISupportsArray *deletedArray);
-  nsresult DoCommand(nsIRDFCompositeDataSource *db, const char * command, nsISupportsArray *srcArray, 
+  nsresult DoCommand(nsIRDFCompositeDataSource *db, const nsACString& command, nsISupportsArray *srcArray, 
 					   nsISupportsArray *arguments);
   PRUnichar *GetString(const PRUnichar *aStringName);
   nsresult InitStringBundle();
@@ -93,7 +92,7 @@ private:
   // String bundles...
   nsCOMPtr<nsIStringBundle>   mStringBundle;
 
-  nsString mCurrentDisplayCharset;
+  nsCString mCurrentDisplayCharset;
 
   nsCOMPtr<nsISupports>  mSearchContext;
   nsCString   mLastDisplayURI; // this used when the user attempts to force a charset reload of a message...we need to get the last displayed
