@@ -264,8 +264,8 @@ private:
 
 nsMessengerWinIntegration::nsMessengerWinIntegration()
 {
-  mDefaultServerAtom = getter_AddRefs(NS_NewAtom("DefaultServer"));
-  mTotalUnreadMessagesAtom = getter_AddRefs(NS_NewAtom("TotalUnreadMessages"));
+  mDefaultServerAtom = do_GetAtom("DefaultServer");
+  mTotalUnreadMessagesAtom = do_GetAtom("TotalUnreadMessages");
 
   mFirstTimeFolderUnreadCountChanged = PR_TRUE;
   mInboxURI = nsnull;
@@ -273,7 +273,7 @@ nsMessengerWinIntegration::nsMessengerWinIntegration()
   mStoreUnreadCounts = PR_FALSE; 
   mIntervalTime = 0;
 
-  mBiffStateAtom = getter_AddRefs(NS_NewAtom("BiffState"));
+  mBiffStateAtom = do_GetAtom("BiffState");
   mBiffIconVisible = PR_FALSE;
   mSuppressBiffIcon = PR_FALSE;
   mAlertInProgress = PR_FALSE;
@@ -299,8 +299,8 @@ nsMessengerWinIntegration::~nsMessengerWinIntegration()
   DestroyBiffIcon(); 
 }
 
-NS_IMPL_ADDREF(nsMessengerWinIntegration);
-NS_IMPL_RELEASE(nsMessengerWinIntegration);
+NS_IMPL_ADDREF(nsMessengerWinIntegration)
+NS_IMPL_RELEASE(nsMessengerWinIntegration)
 
 NS_INTERFACE_MAP_BEGIN(nsMessengerWinIntegration)
    NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIMessengerOSIntegration)
@@ -454,7 +454,7 @@ nsMessengerWinIntegration::Init()
   // get application path 
   char appPath[_MAX_PATH] = {0};
   GetModuleFileName(nsnull, appPath, sizeof(appPath));
-  WORD wideFormatAppPath[_MAX_PATH*2] = {0};
+  WCHAR wideFormatAppPath[_MAX_PATH*2] = {0};
   MultiByteToWideChar(CP_ACP, 0, appPath, strlen(appPath), wideFormatAppPath, _MAX_PATH*2);
   mAppName.Assign((PRUnichar *)wideFormatAppPath);
 
