@@ -64,8 +64,6 @@
 #include "nsIMsgFolder.h" // TO include biffState enum. Change to bool later...
 #include "nsArray.h"
 
-static NS_DEFINE_CID(kRDFServiceCID,            NS_RDFSERVICE_CID);
-
 nsIRDFResource* nsMsgFolderDataSource::kNC_Child = nsnull;
 nsIRDFResource* nsMsgFolderDataSource::kNC_Folder= nsnull;
 nsIRDFResource* nsMsgFolderDataSource::kNC_Name= nsnull;
@@ -1943,12 +1941,10 @@ nsMsgFolderDataSource::GetFolderSizeNode(PRInt32 aFolderSize, nsIRDFNode **aNode
     // XXX todo
     // can we catch this problem at compile time?
     // see #179234
-    nsAutoString units;
     if (sizeInMB)
-      units = NS_LITERAL_STRING(" MB");
+      sizeString.Append(NS_LITERAL_STRING(" MB"));
     else
-      units = NS_LITERAL_STRING(" KB");
-    sizeString.Append(units);
+      sizeString.Append(NS_LITERAL_STRING(" KB"));
     createNode(sizeString.get(), aNode, getRDFService());
   }
   return NS_OK;
