@@ -87,7 +87,7 @@ NS_IMPL_ISUPPORTS4(nsMsgPrintEngine,
                          nsIMsgPrintEngine, 
                          nsIWebProgressListener, 
                          nsIObserver,
-                         nsISupportsWeakReference);
+                         nsISupportsWeakReference)
 
 // nsIWebProgressListener implementation
 NS_IMETHODIMP
@@ -556,7 +556,7 @@ nsMsgPrintEngine::FireThatLoadOperation(nsString *uri)
     if (webNav)
       rv = webNav->LoadURI(uri->get(),                        // URI string
                            nsIWebNavigation::LOAD_FLAGS_NONE, // Load flags
-                           nsnull,                            // Refering URI
+                           nsnull,                            // Referring URI
                            nsnull,                            // Post data
                            nsnull);                           // Extra headers
   }
@@ -763,7 +763,13 @@ FireEvent(nsMsgPrintEngine* aMPE, PLHandleEventProc handler, PLDestroyEventProc
   // The event owns the msgPrintEngine pointer now.
   NS_ADDREF(aMPE);
 
-  event_queue->PostEvent(event);
+  if (NS_FAILED(event_queue->PostEvent(event)))
+  {
+    NS_WARNING("Failed to post event");
+    PL_DestroyEvent(event);
+    return PR_FALSE;
+  }
+
   return PR_TRUE;
 }
 
