@@ -68,8 +68,8 @@ public:
   
   NS_IMETHOD StartFolderLoading(void);
   NS_IMETHOD EndFolderLoading(void);
-  NS_IMETHOD GetCharset(PRUnichar * *aCharset);
-  NS_IMETHOD SetCharset(const PRUnichar * aCharset);
+  NS_IMETHOD GetCharset(char * *aCharset);
+  NS_IMETHOD SetCharset(const char * aCharset);
   NS_IMETHOD GetCharsetOverride(PRBool *aCharsetOverride);
   NS_IMETHOD SetCharsetOverride(PRBool aCharsetOverride);
   NS_IMETHOD GetFirstNewMessage(nsIMsgDBHdr **firstNewMessage);
@@ -122,7 +122,7 @@ public:
   NS_IMETHOD SetDBTransferInfo(nsIDBFolderInfo *aTransferInfo);
   NS_IMETHOD GetStringProperty(const char *propertyName, char **propertyValue);
   NS_IMETHOD SetStringProperty(const char *propertyName, const char *propertyValue);
-  NS_IMETHOD CallFilterPlugins(nsIMsgWindow *aMsgWindow);
+  NS_IMETHOD CallFilterPlugins(nsIMsgWindow *aMsgWindow, PRBool *aFiltersRun);
   NS_IMETHOD GetLastMessageLoaded(nsMsgKey *aMsgKey);
   NS_IMETHOD SetLastMessageLoaded(nsMsgKey aMsgKey);
 
@@ -138,8 +138,8 @@ protected:
 
   nsresult GetFolderCacheKey(nsIFileSpec **aFileSpec);
   nsresult GetFolderCacheElemFromFileSpec(nsIFileSpec *fileSpec, nsIMsgFolderCacheElement **cacheElement);
-  nsresult NotifyStoreClosedAllHeaders();
 
+  nsresult PromptForCachePassword(nsIMsgIncomingServer *server, nsIMsgWindow *aWindow, PRBool &passwordCorrect);
   // offline support methods.
   nsresult StartNewOfflineMessage();
   nsresult WriteStartOfNewLocalMessage();
@@ -151,6 +151,8 @@ protected:
   nsresult GetPromptPurgeThreshold(PRBool *aPrompt);
   nsresult GetPurgeThreshold(PRInt32 *aThreshold);
 
+  nsresult PerformBiffNotifications(void); // if there are new, non spam messages, do biff
+
   virtual nsresult SpamFilterClassifyMessage(const char *aURI, nsIMsgWindow *aMsgWindow, nsIJunkMailPlugin *aJunkMailPlugin);
   virtual nsresult SpamFilterClassifyMessages(const char **aURIArray, PRUint32 aURICount, nsIMsgWindow *aMsgWindow, nsIJunkMailPlugin *aJunkMailPlugin);
 
@@ -173,6 +175,7 @@ protected:
   static nsIAtom* mFolderLoadedAtom;
   static nsIAtom* mDeleteOrMoveMsgCompletedAtom;
   static nsIAtom* mDeleteOrMoveMsgFailedAtom;
+  static nsIAtom* mJunkStatusChangedAtom;
   static nsrefcnt mInstanceCount;
 };
 
