@@ -316,12 +316,11 @@ nsresult NS_MsgCreatePathStringFromFolderURI(const char *folderURI, nsCString& p
     while (startSlashPos != -1) {
 	  oldPath.Mid(pathPiece, startSlashPos + 1, endSlashPos - startSlashPos);
       // skip leading '/' (and other // style things)
-      if (pathPiece.Length() > 0) {
+      if (!pathPiece.IsEmpty()) {
 
         // add .sbd onto the previous path
         if (haveFirst) {
-          pathString+=".sbd";
-          pathString += "/";
+          pathString += ".sbd/";
         }
         
         NS_MsgHashIfNecessary(pathPiece);
@@ -812,3 +811,29 @@ nsresult MSGCramMD5(const char *text, PRInt32 text_len, const char *key, PRInt32
 
 }
 
+
+// digest needs to be a pointer to a DIGEST_LENGTH (16) byte buffer
+nsresult MSGApopMD5(const char *text, PRInt32 text_len, const char *password, PRInt32 password_len, unsigned char *digest)
+{
+  nsresult rv;
+  unsigned char result[DIGEST_LENGTH];
+  unsigned char *presult = result;
+
+  nsCOMPtr<nsISignatureVerifier> verifier = do_GetService(SIGNATURE_VERIFIER_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+
+  HASHContextStr      *context;
+  PRUint32 resultLen;
+
+  rv = verifier->HashBegin(nsISignatureVerifier::MD5, &context);
+  NS_ENSURE_SUCCESS(rv, rv);
+  rv = verifier->HashUpdate(context, text, text_len);
+  NS_ENSURE_SUCCESS(rv, rv);
+  rv = verifier->HashUpdate(context, password, password_len);
+  NS_ENSURE_SUCCESS(rv, rv);
+  rv = verifier->HashEnd(context, &presult, &resultLen, DIGEST_LENGTH);
+  NS_ENSURE_SUCCESS(rv, rv);
+  memcpy(digest, result, DIGEST_LENGTH);
+  return rv;
+}
