@@ -95,6 +95,7 @@ var gCharsetConvertManager;
 
 var gLastWindowToHaveFocus;
 var gReceiptOptionChanged;
+var gAttachVCardOptionChanged;
 
 var gMailSession;
 
@@ -130,7 +131,7 @@ function InitializeGlobalVariables()
   gCurrentMailSendCharset = null;
   gSendDefaultCharset = null;
   gCharsetTitle = null;
-  gCharsetConvertManager = Components.classes['@mozilla.org/charset-converter-manager;1'].getService(Components.interfaces.nsICharsetConverterManager2);
+  gCharsetConvertManager = Components.classes['@mozilla.org/charset-converter-manager;1'].getService(Components.interfaces.nsICharsetConverterManager);
   gMailSession = Components.classes["@mozilla.org/messenger/services/session;1"].getService(Components.interfaces.nsIMsgMailSession);
   gHideMenus = false;
   // We are storing the value of the bool logComposePerformance inorder to avoid logging unnecessarily.
@@ -139,6 +140,7 @@ function InitializeGlobalVariables()
 
   gLastWindowToHaveFocus = null;
   gReceiptOptionChanged = false;
+  gAttachVCardOptionChanged = false;
 }
 InitializeGlobalVariables();
 
@@ -579,6 +581,7 @@ function updateComposeItems() {
 }
 
 function updateEditItems() {
+  goUpdateCommand("cmd_pasteNoFormatting");
   goUpdateCommand("cmd_pasteQuote");
   goUpdateCommand("cmd_delete");
   goUpdateCommand("cmd_selectAll");
@@ -1216,6 +1219,8 @@ function ComposeStartup(recycled, aParams)
       }
       document.getElementById("returnReceiptMenu").setAttribute('checked', 
                                          gMsgCompose.compFields.returnReceipt);
+      document.getElementById("cmd_attachVCard").setAttribute('checked', 
+                                         gMsgCompose.compFields.attachVCard);
 
       // If recycle, editor is already created
       if (!recycled) 
@@ -1449,8 +1454,8 @@ function UpdateMailEditCharset()
 //  dump("gMsgCompose.compFields is " + compFieldsCharset + "\n");
 
   if (gCharsetConvertManager) {
-    var charsetAtom = gCharsetConvertManager.GetCharsetAtom(compFieldsCharset);
-    if (charsetAtom && (charsetAtom.equals("us-ascii")))
+    var charsetAlias = gCharsetConvertManager.getCharsetAlias(compFieldsCharset);
+    if (charsetAlias == "us-ascii")
       compFieldsCharset = "ISO-8859-1";   // no menu item for "us-ascii"
   }
 
@@ -1499,17 +1504,14 @@ function GetCharsetUIString()
     if (gCharsetTitle == null) {
       try {
         // check if we have a converter for this charset
-        var charsetAtom = gCharsetConvertManager.GetCharsetAtom(charset);
+        var charsetAlias = gCharsetConvertManager.GetCharsetAlias(charset);
         var encoderList = gCharsetConvertManager.GetEncoderList();
-        var n = encoderList.Count();
         var found = false;
-        for (var i = 0; i < n; i++)
-        {
-          if (charsetAtom == encoderList.GetElementAt(i))
-          {
-            found = true;
-            break;
-          }
+        while (encoderList.hasMore()) {
+            if (charsetAlias == encoderList.getNext()) {
+                found = true;
+                break;
+            }
         }
         if (!found)
         {
@@ -1520,7 +1522,7 @@ function GetCharsetUIString()
         }
 
         // get a localized string
-        gCharsetTitle = gCharsetConvertManager.GetCharsetTitle(charsetAtom);
+        gCharsetTitle = gCharsetConvertManager.getCharsetTitle(charsetAlias);
       }
       catch (ex) {
         dump("failed to get a charset title of " + charset + "!\n");
@@ -1733,6 +1735,8 @@ function Save()
 function SaveAsFile(saveAs)
 {
   dump("SaveAsFile from XUL\n");
+  var subject = document.getElementById('msgSubject').value;
+  GetCurrentEditor().setDocumentTitle(subject);
   if (gMsgCompose.bodyConvertible() == nsIMsgCompConvertible.Plain)
     SaveDocument(saveAs, false, "text/plain");
   else
@@ -1861,6 +1865,17 @@ function ToggleReturnReceipt(target)
     }
 }
 
+function ToggleAttachVCard(target)
+{
+    var msgCompFields = gMsgCompose.compFields;
+    if (msgCompFields)
+    {
+        msgCompFields.attachVCard = ! msgCompFields.attachVCard;
+        target.setAttribute('checked', msgCompFields.attachVCard);
+        gAttachVCardOptionChanged = true;
+    }
+}
+
 function queryISupportsArray(supportsArray, iid) {
     var result = new Array;
     for (var i=0; i<supportsArray.Count(); i++) {
@@ -1920,6 +1935,8 @@ function FillIdentityListPopup(popup)
 
   for (var i in accounts) {
     var server = accounts[i].incomingServer;
+    if (!server)
+       continue;
     var identites = queryISupportsArray(accounts[i].identities, Components.interfaces.nsIMsgIdentity);
     for (var j in identites) {
       var identity = identites[j];
@@ -2297,11 +2314,6 @@ function AttachmentElementHasItems()
   return element ? element.childNodes.length : 0;
 }  
 
-function AttachVCard()
-{
-  dump("AttachVCard()\n");
-}
-
 function DetermineHTMLAction(convertible)
 {
     var obj;
@@ -2412,6 +2424,7 @@ function LoadIdentity(startup)
           var prevReplyTo = prevIdentity.replyTo;
           var prevBcc = "";
           var prevReceipt = prevIdentity.requestReturnReceipt;
+          var prevAttachVCard = prevIdentity.attachVCard;
 
           if (prevIdentity.doBcc)
             prevBcc += prevIdentity.doBccList;
@@ -2419,6 +2432,8 @@ function LoadIdentity(startup)
           var newReplyTo = gCurrentIdentity.replyTo;
           var newBcc = "";
           var newReceipt = gCurrentIdentity.requestReturnReceipt;
+          var newAttachVCard = gCurrentIdentity.attachVCard;
+
           if (gCurrentIdentity.doBcc)
             newBcc += gCurrentIdentity.doBccList;          
 
@@ -2433,6 +2448,13 @@ function LoadIdentity(startup)
             document.getElementById("returnReceiptMenu").setAttribute('checked',msgCompFields.returnReceipt);
           }
 
+          if (!gAttachVCardOptionChanged &&
+              prevAttachVCard == msgCompFields.attachVCard &&
+              prevAttachVCard != newAttachVCard)
+          {
+            msgCompFields.attachVCard = newAttachVCard;
+            document.getElementById("cmd_attachVCard").setAttribute('checked',msgCompFields.attachVCard);
+          }
 
           if (newReplyTo != prevReplyTo)
           {
@@ -2539,18 +2561,10 @@ function subjectKeyPress(event)
   }
 }
 
-function editorKeyPress(event)
-{
-  if (event.keyCode == 9) {
-    if (event.shiftKey && !event.getPreventDefault()) {
-      document.getElementById('msgSubject').focus();
-      event.preventDefault();
-    }
-  }
-}
-
 function AttachmentBucketClicked(event)
 {
+  event.currentTarget.focus();
+
   if (event.button != 0)
     return;
 
