@@ -618,7 +618,10 @@ RRT_HEADER:
 
   // If we don't have disclosed recipient (only Bcc), address the message to
   // undisclosed-recipients to prevent problem with some servers
-  if (!hasDisclosedRecipient) {
+
+  // If we are saving the message as a draft, don't bother inserting the undisclosed recipients field. We'll take care of that when we
+  // really send the message.
+  if (!hasDisclosedRecipient && !isDraft) {
     PRBool bAddUndisclosedRecipients = PR_TRUE;
     prefs->GetBoolPref("mail.compose.add_undisclosed_recipients", &bAddUndisclosedRecipients);
     if (bAddUndisclosedRecipients) {
@@ -778,7 +781,16 @@ mime_generate_attachment_headers (const char *type,
   nsresult rv;
   nsCOMPtr<nsIPref> prefs(do_GetService(kPrefCID, &rv)); 
 
-  nsCString buf;
+  PRInt32 buffer_size = 2048 + (real_name ? 2*PL_strlen(real_name) : 0) + (base_url ? 2*PL_strlen(base_url) : 0) +
+                        (type_param ? PL_strlen(type_param) : 0) + (encoding ? PL_strlen(encoding) : 0) +
+                        (description ? PL_strlen(description) : 0) + (x_mac_type ? PL_strlen(x_mac_type) : 0) +
+                        (x_mac_creator ? PL_strlen(x_mac_creator) : 0) + (attachmentCharset ? PL_strlen(attachmentCharset) : 0) +
+                        (bodyCharset ? PL_strlen(bodyCharset) : 0) + (content_id ? PL_strlen(content_id) : 0);
+  char *buffer = (char *) PR_Malloc (buffer_size);
+  char *buffer_tail = buffer;
+
+  if (! buffer)
+    return 0; /* NS_ERROR_OUT_OF_MEMORY */
 
   NS_ASSERTION (encoding, "null encoding");
 
@@ -822,13 +834,14 @@ mime_generate_attachment_headers (const char *type,
     }
   }
 
-  buf.Append("Content-Type: ");
-  buf.Append(type);
+  PUSH_STRING ("Content-Type: ");
+  PUSH_STRING (type);
+
   if (type_param && *type_param)
   {
     if (*type_param != ';')
-      buf.Append("; ");
-    buf.Append(type_param);
+      PUSH_STRING("; ");
+    PUSH_STRING(type_param);
   }
 
   if (mime_type_needs_charset (type)) 
@@ -865,8 +878,8 @@ mime_generate_attachment_headers (const char *type,
          (PL_strcasecmp(encoding, ENCODING_BASE64) != 0)) &&
          (*charset_label))
     {
-      buf.Append("; charset=");
-      buf.Append(charset_label);
+      PUSH_STRING ("; charset=");
+      PUSH_STRING (charset_label);
     }
   }
 
@@ -877,7 +890,7 @@ mime_generate_attachment_headers (const char *type,
     if(type && !PL_strcasecmp(type, "text/plain"))
     {
       if(UseFormatFlowed(bodyCharset))
-        buf.Append("; format=flowed");
+        PUSH_STRING ("; format=flowed");
       // else
       // {
       // Don't add a markup. Could use 
@@ -889,59 +902,59 @@ mime_generate_attachment_headers (const char *type,
   }    
 
   if (x_mac_type && *x_mac_type) {
-    buf.Append("; x-mac-type=\"");
-    buf.Append(x_mac_type);
-    buf.Append("\"");
+    PUSH_STRING ("; x-mac-type=\"");
+    PUSH_STRING (x_mac_type);
+    PUSH_STRING ("\"");
   }
 
   if (x_mac_creator && *x_mac_creator) {
-    buf.Append("; x-mac-creator=\"");
-    buf.Append(x_mac_creator);
-    buf.Append("\"");
+    PUSH_STRING ("; x-mac-creator=\"");
+    PUSH_STRING (x_mac_creator);
+    PUSH_STRING ("\"");
   }
 
 #ifdef EMIT_NAME_IN_CONTENT_TYPE
   if (encodedRealName && *encodedRealName) {
     if (parmFolding == 0 || parmFolding == 1) {
-      buf.Append(";\r\n name=\"");
-      buf.Append(encodedRealName);
-      buf.Append("\"");
+      PUSH_STRING (";\r\n name=\"");
+      PUSH_STRING (encodedRealName);
+      PUSH_STRING ("\"");
     }
     else // if (parmFolding == 2)
     {
       char *rfc2231Parm = RFC2231ParmFolding("name", charset.get(),
                          nsMsgI18NGetAcceptLanguage(), encodedRealName);
       if (rfc2231Parm) {
-        buf.Append(";\r\n ");
-        buf.Append(rfc2231Parm);
+        PUSH_STRING(";\r\n ");
+        PUSH_STRING(rfc2231Parm);
         PR_Free(rfc2231Parm);
       }
     }
   }
 #endif /* EMIT_NAME_IN_CONTENT_TYPE */
-  buf.Append(CRLF);
 
-  buf.Append("Content-Transfer-Encoding: ");
-  buf.Append(encoding);
+  PUSH_NEWLINE ();
 
-	buf.Append(CRLF);
+  PUSH_STRING ("Content-Transfer-Encoding: ");
+  PUSH_STRING (encoding);
+  PUSH_NEWLINE ();
 
   if (description && *description) {
     char *s = mime_fix_header (description);
     if (s) {
-      buf.Append("Content-Description: ");
-      buf.Append(s);
-      buf.Append(CRLF);
+      PUSH_STRING ("Content-Description: ");
+      PUSH_STRING (s);
+      PUSH_NEWLINE ();
       PR_Free(s);
     }
   }
 
   if ( (content_id) && (*content_id) )
   {
-    buf.Append("Content-ID: <");
-    buf.Append(content_id);
-    buf.Append(">");  
-    buf.Append(CRLF);
+    PUSH_STRING ("Content-ID: <");
+    PUSH_STRING (content_id);
+    PUSH_STRING (">");
+    PUSH_NEWLINE ();
   }
 
   if (encodedRealName && *encodedRealName) {
@@ -951,15 +964,15 @@ mime_generate_attachment_headers (const char *type,
         rv = prefs->GetIntPref("mail.content_disposition_type", &pref_content_disposition);
         NS_ASSERTION(NS_SUCCEEDED(rv), "failed to get mail.content_disposition_type");
 
-    buf.Append("Content-Disposition: ");
+    PUSH_STRING ("Content-Disposition: ");
 
     if (pref_content_disposition == 1)
-      buf.Append("attachment");
+      PUSH_STRING ("attachment");
     else
       if (pref_content_disposition == 2 && 
           (!PL_strcasecmp(type, TEXT_PLAIN) || 
           (period && !PL_strcasecmp(period, ".txt"))))
-        buf.Append("attachment");
+        PUSH_STRING("attachment");
 
       /* If this document is an anonymous binary file or a vcard, 
       then always show it as an attachment, never inline. */
@@ -967,23 +980,23 @@ mime_generate_attachment_headers (const char *type,
         if (!PL_strcasecmp(type, APPLICATION_OCTET_STREAM) || 
             !PL_strcasecmp(type, TEXT_VCARD) ||
             !PL_strcasecmp(type, APPLICATION_DIRECTORY)) /* text/x-vcard synonym */
-          buf.Append("attachment");
+          PUSH_STRING ("attachment");
         else
-          buf.Append("inline");
+          PUSH_STRING ("inline");
 
     if (parmFolding == 0 || parmFolding == 1) {
-      buf.Append(";\r\n filename=\"");
-      buf.Append(encodedRealName);
-      buf.Append("\"" CRLF);
+      PUSH_STRING (";\r\n filename=\"");
+      PUSH_STRING (encodedRealName);
+      PUSH_STRING ("\"" CRLF);
     }
     else // if (parmFolding == 2)
     {
       char *rfc2231Parm = RFC2231ParmFolding("filename", charset.get(),
                        nsMsgI18NGetAcceptLanguage(), encodedRealName);
       if (rfc2231Parm) {
-        buf.Append(";\r\n ");
-        buf.Append(rfc2231Parm);
-        buf.Append(CRLF);
+        PUSH_STRING(";\r\n ");
+        PUSH_STRING(rfc2231Parm);
+        PUSH_NEWLINE ();
         PR_Free(rfc2231Parm);
       }
     }
@@ -992,7 +1005,7 @@ mime_generate_attachment_headers (const char *type,
     if (type &&
         (!PL_strcasecmp (type, MESSAGE_RFC822) ||
         !PL_strcasecmp (type, MESSAGE_NEWS)))
-      buf.Append("Content-Disposition: inline" CRLF);
+      PUSH_STRING ("Content-Disposition: inline" CRLF);
   
 #ifdef GENERATE_CONTENT_BASE
   /* If this is an HTML document, and we know the URL it originally
@@ -1026,9 +1039,9 @@ mime_generate_attachment_headers (const char *type,
       prefs->GetBoolPref("mail.use_content_location_on_send", &useContentLocation);
 
     if (useContentLocation)
-      buf.Append("Content-Location: \"");
+      PUSH_STRING ("Content-Location: \"");
     else
-      buf.Append("Content-Base: \"");
+      PUSH_STRING ("Content-Base: \"");
     /* rhp - Pref for Content-Location usage */
 
 /* rhp: this is to work with the Content-Location stuff */
@@ -1036,34 +1049,34 @@ CONTENT_LOC_HACK:
 
     while (*s != 0 && *s != '#')
     {
-      PRUint32 ot=buf.Length();
-      char tmp[]="\x00\x00";
+      const char *ot = buffer_tail;
+
       /* URLs must be wrapped at 40 characters or less. */
       if (col >= 38) {
-        buf.Append(CRLF "\t");
+        PUSH_STRING(CRLF "\t");
         col = 0;
       }
 
       if (*s == ' ')
-        buf.Append("%20");
+        PUSH_STRING("%20");
       else if (*s == '\t')
-        buf.Append("%09");
+        PUSH_STRING("%09");
       else if (*s == '\n')
-        buf.Append("%0A");
+        PUSH_STRING("%0A");
       else if (*s == '\r')
-        buf.Append("%0D");
+        PUSH_STRING("%0D");
       else {
-	      tmp[0]=*s;
-	      buf.Append(tmp);
+        *buffer_tail++ = *s;
+        *buffer_tail = '\0';
       }
       s++;
-      col += (buf.Length() - ot);
+      col += (buffer_tail - ot);
     }
-    buf.Append("\"" CRLF);
+    PUSH_STRING ("\"" CRLF);
 
     /* rhp: this is to try to get around this fun problem with Content-Location */
     if (!useContentLocation) {
-      buf.Append("Content-Location: \"");
+      PUSH_STRING ("Content-Location: \"");
       s = base_url;
       col = 0;
       useContentLocation = PR_TRUE;
@@ -1077,9 +1090,10 @@ GIVE_UP_ON_CONTENT_BASE:
 #endif /* GENERATE_CONTENT_BASE */
 
   /* realloc it smaller... */
+  buffer = (char*) PR_REALLOC (buffer, buffer_tail - buffer + 1);
 
   PR_FREEIF(encodedRealName);
-  return PL_strdup(buf.get());
+  return buffer;
 }
 
 static PRBool isValidHost( const char* host )
@@ -1692,7 +1706,7 @@ nsMsgNewURL(nsIURI** aInstancePtrResult, const char * aSpec)
   nsCOMPtr<nsIIOService> pNetService(do_GetService(kIOServiceCID, &rv)); 
   if (NS_SUCCEEDED(rv) && pNetService)
   {
-    if (PL_strstr(aSpec, "://") == nsnull)
+    if (PL_strstr(aSpec, "://") == nsnull && strncmp(aSpec, "data:", 5))
     {
       //XXXjag Temporary fix for bug 139362 until the real problem(bug 70083) get fixed
       nsCAutoString uri(NS_LITERAL_CSTRING("http://") + nsDependentCString(aSpec));
