@@ -73,6 +73,7 @@
 #include "nsIPrefService.h"
 #include "nsISignatureVerifier.h"
 #include "nsISSLSocketControl.h"
+#include "nsPrintfCString.h"
 
 #ifndef XP_UNIX
 #include <stdarg.h>
@@ -102,7 +103,7 @@ nsresult nsExplainErrorDetails(nsISmtpUrl * aSmtpUrl, int code, ...)
 
   nsresult rv = NS_OK;
   va_list args;
-	
+
   nsCOMPtr<nsIPrompt> dialog;
   aSmtpUrl->GetPrompt(getter_AddRefs(dialog));
   NS_ENSURE_TRUE(dialog, NS_ERROR_FAILURE);
@@ -113,20 +114,24 @@ nsresult nsExplainErrorDetails(nsISmtpUrl * aSmtpUrl, int code, ...)
   
   va_start (args, code);
   
-  switch (code)	{
-		case NS_ERROR_SMTP_SERVER_ERROR:
-                case NS_ERROR_TCP_READ_ERROR:
-                case NS_ERROR_SENDING_FROM_COMMAND:
-                case NS_ERROR_SENDING_RCPT_COMMAND:
-                case NS_ERROR_SENDING_DATA_COMMAND:
-                case NS_ERROR_SENDING_MESSAGE:   
-                  smtpBundle->GetStringByID(code, getter_Copies(eMsg));
-                  msg = nsTextFormatter::vsmprintf(eMsg, args);
-                  break;
-                default:
-                  smtpBundle->GetStringByID(NS_ERROR_COMMUNICATIONS_ERROR, getter_Copies(eMsg));
-                  msg = nsTextFormatter::smprintf(eMsg, code);
-                  break;
+  switch (code)
+  {
+      case NS_ERROR_SMTP_SERVER_ERROR:
+      case NS_ERROR_TCP_READ_ERROR:
+      case NS_ERROR_SMTP_TEMP_SIZE_EXCEEDED:
+      case NS_ERROR_SMTP_PERM_SIZE_EXCEEDED_1:
+      case NS_ERROR_SMTP_PERM_SIZE_EXCEEDED_2:
+      case NS_ERROR_SENDING_FROM_COMMAND:
+      case NS_ERROR_SENDING_RCPT_COMMAND:
+      case NS_ERROR_SENDING_DATA_COMMAND:
+      case NS_ERROR_SENDING_MESSAGE:   
+           smtpBundle->GetStringByID(code, getter_Copies(eMsg));
+           msg = nsTextFormatter::vsmprintf(eMsg, args);
+           break;
+      default:
+           smtpBundle->GetStringByID(NS_ERROR_COMMUNICATIONS_ERROR, getter_Copies(eMsg));
+           msg = nsTextFormatter::smprintf(eMsg, code);
+           break;
   }
   
   if (msg) 
@@ -246,7 +251,7 @@ void nsSmtpProtocol::Initialize(nsIURI * aURL)
     m_origAuthFlags = 0;
     m_prefAuthMethod = PREF_AUTH_NONE;
     m_usernamePrompted = PR_FALSE;
-    m_prefTrySSL = PREF_SSL_TRY;
+    m_prefTrySSL = PREF_SECURE_TRY_STARTTLS;
     m_prefTrySecAuth = PR_TRUE;
     m_tlsInitiated = PR_FALSE;
 
@@ -278,12 +283,19 @@ void nsSmtpProtocol::Initialize(nsIURI * aURL)
     m_addressCopy = nsnull;
     m_addresses = nsnull;
 
-  	m_addressesLeft = nsnull;
+    m_addressesLeft = nsnull;
     m_verifyAddress = nsnull;
 #ifdef UNREADY_CODE 
     m_totalAmountWritten = 0;
-    m_totalMessageSize = 0;
 #endif /* UNREADY_CODE */
+
+    m_sizelimit = 0;
+    m_totalMessageSize = 0;
+    nsCOMPtr<nsIFileSpec> fileSpec;
+    m_runningURL->GetPostMessageFile(getter_AddRefs(fileSpec));
+    if (fileSpec)
+        fileSpec->GetFileSize(&m_totalMessageSize);
+
     m_originalContentLength = 0;
     m_totalAmountRead = 0;
 
@@ -315,15 +327,19 @@ void nsSmtpProtocol::Initialize(nsIURI * aURL)
     if (smtpUrl)
         smtpUrl->GetNotificationCallbacks(getter_AddRefs(callbacks));
 
-    if (m_prefTrySSL != PREF_SSL_NEVER) {
+    if (m_prefTrySSL == PREF_SECURE_ALWAYS_SMTPS)
+        rv = OpenNetworkSocket(aURL, "ssl", callbacks);  
+    else if (m_prefTrySSL != PREF_SECURE_NEVER)
+    {
         rv = OpenNetworkSocket(aURL, "starttls", callbacks);  
-        if (NS_FAILED(rv) && m_prefTrySSL == PREF_SSL_TRY) {
-            m_prefTrySSL = PREF_SSL_NEVER;
+        if (NS_FAILED(rv) && m_prefTrySSL == PREF_SECURE_TRY_STARTTLS)
+        {
+            m_prefTrySSL = PREF_SECURE_NEVER;
             rv = OpenNetworkSocket(aURL, nsnull, callbacks);
         }
-    } else {
-        rv = OpenNetworkSocket(aURL, nsnull, callbacks);
     }
+    else
+        rv = OpenNetworkSocket(aURL, nsnull, callbacks);
     
     if (NS_FAILED(rv))
         return;
@@ -538,8 +554,8 @@ PRInt32 nsSmtpProtocol::SendHeloResponse(nsIInputStream * inputStream, PRUint32
   }
   
   /* don't check for a HELO response because it can be bogus and
-	 * we don't care
-	 */
+   * we don't care
+   */
   
   if(!((const char *)emailAddress) || CHECK_SIMULATED_ERROR(SIMULATED_SEND_ERROR_16))
   {
@@ -595,7 +611,10 @@ PRInt32 nsSmtpProtocol::SendHeloResponse(nsIInputStream * inputStream, PRUint32
     {
       buffer = "MAIL FROM:<";
       buffer += fullAddress;
-      buffer += ">" CRLF;
+      buffer += ">";
+      if(TestFlag(SMTP_EHLO_SIZE_ENABLED))
+        buffer += nsPrintfCString(" SIZE=%d", m_totalMessageSize);;
+      buffer += CRLF;
     }
     PR_FREEIF (fullAddress);
   }
@@ -626,7 +645,7 @@ PRInt32 nsSmtpProtocol::SendEhloResponse(nsIInputStream * inputStream, PRUint32
         /* EHLO must not be implemented by the server so fall back to the HELO case */
         if (m_responseCode >= 500 && m_responseCode < 550)
         {
-            if (m_prefTrySSL == PREF_SSL_ALWAYS)
+            if (m_prefTrySSL == PREF_SECURE_ALWAYS_STARTTLS)
             {
                 m_nextState = SMTP_ERROR_DONE;
                 m_urlErrorState = NS_ERROR_COULD_NOT_LOGIN_TO_SMTP_SERVER;
@@ -690,22 +709,40 @@ PRInt32 nsSmtpProtocol::SendEhloResponse(nsIInputStream * inputStream, PRUint32
 
             if(m_prefTrySecAuth)
             {
-            if (m_responseText.Find("CRAM-MD5", PR_TRUE, 5) >= 0)
-            {
-                nsresult rv;
-                nsCOMPtr<nsISignatureVerifier> verifier = do_GetService(SIGNATURE_VERIFIER_CONTRACTID, &rv);
-                // this checks if psm is installed...
-                if (NS_SUCCEEDED(rv))
-                  SetFlag(SMTP_AUTH_CRAM_MD5_ENABLED);
+                if (m_responseText.Find("CRAM-MD5", PR_TRUE, 5) >= 0)
+                {
+                    nsresult rv;
+                    nsCOMPtr<nsISignatureVerifier> verifier = do_GetService(SIGNATURE_VERIFIER_CONTRACTID, &rv);
+                    // this checks if psm is installed...
+                    if (NS_SUCCEEDED(rv))
+                      SetFlag(SMTP_AUTH_CRAM_MD5_ENABLED);
                 }
             }
 
             // for use after mechs disabled fallbacks when login failed
             BackupAuthFlags();
         }
+        else if (responseLine.Compare("SIZE", PR_TRUE, 4) == 0)
+        {
+            SetFlag(SMTP_EHLO_SIZE_ENABLED);
+
+            m_sizelimit = atol((responseLine.get()) + 4);
+        }
+
         startPos = endPos + 1;
     } while (endPos >= 0);
 
+    if(TestFlag(SMTP_EHLO_SIZE_ENABLED) &&
+       m_sizelimit > 0 && (PRInt32)m_totalMessageSize > m_sizelimit)
+    {
+        nsresult rv = nsExplainErrorDetails(m_runningURL,
+                NS_ERROR_SMTP_PERM_SIZE_EXCEEDED_1, m_sizelimit);
+        NS_ASSERTION(NS_SUCCEEDED(rv), "failed to explain SMTP error");
+
+        m_urlErrorState = NS_ERROR_BUT_DONT_SHOW_ALERT;
+        return(NS_ERROR_SENDING_FROM_COMMAND);
+    }
+
     m_nextState = SMTP_AUTH_PROCESS_STATE;
     return status;
 }
@@ -741,6 +778,7 @@ PRInt32 nsSmtpProtocol::SendTLSResponse()
       return rv;
     }
   }
+
   ClearFlag(SMTP_EHLO_STARTTLS_ENABLED);
   m_tlsInitiated = PR_FALSE;
   m_nextState = SMTP_AUTH_PROCESS_STATE;
@@ -758,7 +796,13 @@ PRInt32 nsSmtpProtocol::ProcessAuth()
     {
         if(TestFlag(SMTP_EHLO_STARTTLS_ENABLED))
         {
-            if (m_prefTrySSL != PREF_SSL_NEVER)
+            // Do not try to combine SMTPS with STARTTLS.
+            // PREF_SECURE_ALWAYS_SMTPS is used,
+            // we are alrady using a secure connection.
+            // Do not attempt to do STARTTLS,
+            // even if server offers it.
+            if (m_prefTrySSL == PREF_SECURE_TRY_STARTTLS ||
+                m_prefTrySSL == PREF_SECURE_ALWAYS_STARTTLS)
             {
                 buffer = "STARTTLS";
                 buffer += CRLF;
@@ -773,7 +817,8 @@ PRInt32 nsSmtpProtocol::ProcessAuth()
                 return status;
             }
         }
-        else if (m_prefTrySSL == PREF_SSL_ALWAYS)
+        else
+        if (m_prefTrySSL == PREF_SECURE_ALWAYS_STARTTLS)
         {
             m_nextState = SMTP_ERROR_DONE;
             m_urlErrorState = NS_ERROR_COULD_NOT_LOGIN_TO_SMTP_SERVER;
@@ -1071,16 +1116,22 @@ PRInt32 nsSmtpProtocol::SendMailResponse()
 {
   PRInt32 status = 0;
   nsCAutoString buffer;
-  
+  nsresult rv;
+
   if(m_responseCode != 250 || CHECK_SIMULATED_ERROR(SIMULATED_SEND_ERROR_11))
   {
-    nsresult rv = nsExplainErrorDetails(m_runningURL, NS_ERROR_SENDING_FROM_COMMAND, m_responseText.get());
+    rv = nsExplainErrorDetails(m_runningURL,
+     (m_responseCode == 452) ? NS_ERROR_SMTP_TEMP_SIZE_EXCEEDED :
+    ((m_responseCode == 552) ? NS_ERROR_SMTP_PERM_SIZE_EXCEEDED_2 :
+                               NS_ERROR_SENDING_FROM_COMMAND),
+                               m_responseText.get());
+
     NS_ASSERTION(NS_SUCCEEDED(rv), "failed to explain SMTP error");
     
     m_urlErrorState = NS_ERROR_BUT_DONT_SHOW_ALERT;
     return(NS_ERROR_SENDING_FROM_COMMAND);
   }
-  
+
   /* Send the RCPT TO: command */
 #ifdef UNREADY_CODE
   if (TestFlag(SMTP_EHLO_DSN_ENABLED) &&
@@ -1132,10 +1183,16 @@ PRInt32 nsSmtpProtocol::SendRecipientResponse()
 {
   PRInt32 status = 0;
   nsCAutoString buffer;
-  
+  nsresult rv;
+
   if(m_responseCode != 250 && m_responseCode != 251)
   {
-    nsresult rv = nsExplainErrorDetails(m_runningURL, NS_ERROR_SENDING_RCPT_COMMAND, m_responseText.get());
+    rv = nsExplainErrorDetails(m_runningURL,
+     (m_responseCode == 452) ? NS_ERROR_SMTP_TEMP_SIZE_EXCEEDED :
+    ((m_responseCode == 552) ? NS_ERROR_SMTP_PERM_SIZE_EXCEEDED_2 :
+                               NS_ERROR_SENDING_FROM_COMMAND),
+                               m_responseText.get());
+
     NS_ASSERTION(NS_SUCCEEDED(rv), "failed to explain SMTP error");
     
     m_urlErrorState = NS_ERROR_BUT_DONT_SHOW_ALERT;
@@ -1144,7 +1201,7 @@ PRInt32 nsSmtpProtocol::SendRecipientResponse()
   
   if(m_addressesLeft > 0)
   {
-		/* more senders to RCPT to 
+    /* more senders to RCPT to 
     */
     // fake to 250 because SendMailResponse() can't handle 251
     m_responseCode = 250;
@@ -1652,28 +1709,30 @@ nsSmtpProtocol::GetPassword(char **aPassword)
     rv = smtpServer->GetUsername(getter_Copies(username));
     NS_ENSURE_SUCCESS(rv, rv);
     
-    nsCAutoString promptValue(username);
+    const PRUnichar *formatStrings[] =
+    {
+      NS_ConvertASCIItoUCS2(username).get(),
+      nsnull  // this will be overwritten in some cases.
+    };
 
     PRBool hideHostnameForPassword = PR_FALSE;
     rv = prefBranch->GetBoolPref(prefName.get(), &hideHostnameForPassword);
-    if (NS_SUCCEEDED(rv) && hideHostnameForPassword) {
-      // for certain redirector types, we don't want to show the
-      // hostname to the user when prompting for password
-    }
-    else {
+    // for certain redirector types, we don't want to show the
+    // hostname to the user when prompting for password
+    if (!hideHostnameForPassword) 
+    {
       nsXPIDLCString hostname;      
       rv = smtpServer->GetHostname(getter_Copies(hostname));
       NS_ENSURE_SUCCESS(rv, rv);
-      promptValue.Append("@");
-      promptValue.Append(hostname);
+      formatStrings[1] = NS_ConvertASCIItoUCS2(hostname).get();
     }
-
-    rv = PromptForPassword(smtpServer, smtpUrl, NS_ConvertASCIItoUCS2(promptValue).get(), aPassword);
+    rv = PromptForPassword(smtpServer, smtpUrl, formatStrings, aPassword);
     NS_ENSURE_SUCCESS(rv,rv);
     return rv;
 }
 
-nsresult nsSmtpProtocol::PromptForPassword(nsISmtpServer *aSmtpServer, nsISmtpUrl *aSmtpUrl, const PRUnichar *aPromptValue, char **aPassword)
+nsresult
+nsSmtpProtocol::PromptForPassword(nsISmtpServer *aSmtpServer, nsISmtpUrl *aSmtpUrl, const PRUnichar **formatStrings, char **aPassword)
 {
   nsresult rv;
   nsCOMPtr<nsIStringBundleService> stringService = do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
@@ -1683,15 +1742,15 @@ nsresult nsSmtpProtocol::PromptForPassword(nsISmtpServer *aSmtpServer, nsISmtpUr
   rv = stringService->CreateBundle("chrome://messenger/locale/messengercompose/composeMsgs.properties", getter_AddRefs(composeStringBundle));
   NS_ENSURE_SUCCESS(rv,rv);
   
-  const PRUnichar *formatStrings[] =
-  {
-    aPromptValue,
-  };
- 
   nsXPIDLString passwordPromptString;
-  rv = composeStringBundle->FormatStringFromID(NS_SMTP_PASSWORD_PROMPT,
-    formatStrings, 1,
-    getter_Copies(passwordPromptString));
+  if(formatStrings[1])
+    rv = composeStringBundle->FormatStringFromID(NS_SMTP_PASSWORD_PROMPT2,
+      formatStrings, 2,
+      getter_Copies(passwordPromptString));
+  else
+    rv = composeStringBundle->FormatStringFromID(NS_SMTP_PASSWORD_PROMPT1,
+      formatStrings, 1,
+      getter_Copies(passwordPromptString));
   NS_ENSURE_SUCCESS(rv, rv);
   
   nsCOMPtr<nsIAuthPrompt> netPrompt;
@@ -1745,7 +1804,13 @@ nsSmtpProtocol::GetUsernamePassword(char **aUsername, char **aPassword)
     rv = smtpServer->GetHostname(getter_Copies(hostname));
     NS_ENSURE_SUCCESS(rv, rv);
 
-    rv = PromptForPassword(smtpServer, smtpUrl, NS_ConvertASCIItoUCS2(hostname).get(), aPassword);
+    const PRUnichar *formatStrings[] =
+    {
+      NS_ConvertASCIItoUCS2(hostname).get(),
+      nsnull
+    };
+
+    rv = PromptForPassword(smtpServer, smtpUrl, formatStrings, aPassword);
     NS_ENSURE_SUCCESS(rv,rv);
     return rv;
 }
