@@ -48,7 +48,10 @@
 #include "prlog.h"
 #include "prprf.h"
 #include "nsIFileSpec.h"
-
+#ifdef PUTUP_ALERT_ON_INVALID_DB
+#include "nsIPrompt.h"
+#include "nsIWindowWatcher.h"
+#endif
 const char *kOfflineOpsScope = "ns:msg:db:row:scope:ops:all";	// scope for all offine ops table
 const char *kOfflineOpsTableKind = "ns:msg:db:table:kind:ops";
 struct mdbOid gAllOfflineOpsTableOID;
@@ -64,8 +67,6 @@ nsMailDatabase::~nsMailDatabase()
 {
   if(m_folderSpec)
     delete m_folderSpec;
-  if (m_mdbAllOfflineOpsTable)
-    m_mdbAllOfflineOpsTable->Release();
 }
 
 NS_IMETHODIMP nsMailDatabase::SetFolderStream(nsIOFileStream *aFileStream)
@@ -76,6 +77,19 @@ NS_IMETHODIMP nsMailDatabase::SetFolderStream(nsIOFileStream *aFileStream)
   return NS_OK;
 }
 
+static PRBool gGotGlobalPrefs = PR_FALSE;
+static PRBool gThreadWithoutRe = PR_TRUE;
+static PRInt32 gTimeStampLeeway;
+
+void nsMailDatabase::GetGlobalPrefs()
+{
+  if (!gGotGlobalPrefs)
+  {
+    GetBoolPref("mail.thread_without_re", &gThreadWithoutRe);
+    GetIntPref("mail.db_timestamp_leeway", &gTimeStampLeeway);
+    gGotGlobalPrefs = PR_TRUE;
+  }
+}
 
 NS_IMETHODIMP nsMailDatabase::Open(nsIFileSpec *aFolderName, PRBool create, PRBool upgrading, nsIMsgDatabase** pMessageDB)
 {
@@ -187,11 +201,7 @@ NS_IMETHODIMP nsMailDatabase::Open(nsIFileSpec *aFolderName, PRBool create, PRBo
 
 NS_IMETHODIMP nsMailDatabase::ForceClosed()
 {
-  if (m_mdbAllOfflineOpsTable)
-  {
-    m_mdbAllOfflineOpsTable->Release();
-    m_mdbAllOfflineOpsTable = nsnull;
-  }
+  m_mdbAllOfflineOpsTable = nsnull;
   return nsMsgDatabase::ForceClosed();
 }
 
@@ -201,42 +211,11 @@ nsresult nsMailDatabase::GetAllOfflineOpsTable()
 {
   nsresult rv = NS_OK;
   if (!m_mdbAllOfflineOpsTable)
-  {
-		mdb_err err	= GetStore()->StringToToken(GetEnv(), kOfflineOpsScope, &m_offlineOpsRowScopeToken); 
-    err = GetStore()->StringToToken(GetEnv(), kOfflineOpsTableKind, &m_offlineOpsTableKindToken); 
-		gAllOfflineOpsTableOID.mOid_Scope = m_offlineOpsRowScopeToken;
-		gAllOfflineOpsTableOID.mOid_Id = 1;
-
-    rv = GetStore()->GetTable(GetEnv(), &gAllOfflineOpsTableOID, &m_mdbAllOfflineOpsTable);
-    if (rv != NS_OK)
-      rv = NS_ERROR_FAILURE;
-
-    // create new all msg hdrs table, if it doesn't exist.
-    if (NS_SUCCEEDED(rv) && !m_mdbAllOfflineOpsTable)
-    {
-	    nsIMdbStore *store = GetStore();
-	    mdb_err mdberr = (nsresult) store->NewTable(GetEnv(), m_offlineOpsRowScopeToken, 
-		    m_offlineOpsTableKindToken, PR_FALSE, nsnull, &m_mdbAllOfflineOpsTable);
-      if (mdberr != NS_OK || !m_mdbAllOfflineOpsTable)
-        rv = NS_ERROR_FAILURE;
-    }
-    NS_ASSERTION(NS_SUCCEEDED(rv), "couldn't create offline ops table");
-  }
+    rv = GetTableCreateIfMissing(kOfflineOpsScope, kOfflineOpsTableKind, getter_AddRefs(m_mdbAllOfflineOpsTable), 
+                                                m_offlineOpsRowScopeToken, m_offlineOpsTableKindToken) ;
   return rv;
 }
 
-/* static */ nsresult nsMailDatabase::CloneInvalidDBInfoIntoNewDB(nsFileSpec &pathName, nsMailDatabase** pMailDB)
-{
-	nsresult ret = NS_OK;
-	return ret;
-}
-
-nsresult nsMailDatabase::OnNewPath (nsFileSpec &newPath)
-{
-	nsresult ret = NS_OK;
-	return ret;
-}
-
 // cache m_folderStream to make updating mozilla status flags fast
 NS_IMETHODIMP nsMailDatabase::StartBatch()
 {
@@ -496,7 +475,10 @@ NS_IMETHODIMP nsMailDatabase::GetSummaryValid(PRBool *aResult)
   PRUint32  folderDate;
   nsFileSpec::TimeStamp actualFolderTimeStamp;
   PRInt32 numNewMessages;
+  nsAutoString errorMsg;
+
         
+  *aResult = PR_FALSE;
   
   if (m_folderSpec && m_dbFolderInfo)
   {
@@ -506,18 +488,55 @@ NS_IMETHODIMP nsMailDatabase::GetSummaryValid(PRBool *aResult)
     m_dbFolderInfo->GetFolderSize(&folderSize);
     m_dbFolderInfo->GetFolderDate(&folderDate);
 
-    // compare current version of db versus filed out version info.
+    // compare current version of db versus filed out version info, 
+    // and file size in db vs file size on disk.
     PRUint32 version;
+
     m_dbFolderInfo->GetVersion(&version);
     if (folderSize == m_folderSpec->GetFileSize() &&
-      folderDate == actualFolderTimeStamp &&
         numNewMessages >= 0 && GetCurVersion() == version)
     {
-      *aResult = PR_TRUE;
-      return NS_OK;
+      GetGlobalPrefs();
+      // if those values are ok, check time stamp
+      if (gTimeStampLeeway == 0)
+        *aResult = folderDate == actualFolderTimeStamp;
+      else
+        *aResult = PR_ABS((PRInt32) (actualFolderTimeStamp - folderDate)) <= gTimeStampLeeway;
+#ifndef PUTUP_ALERT_ON_INVALID_DB
     }
   }
-  *aResult = PR_FALSE;
+#else
+      if (!*aResult)
+      {
+        errorMsg.AppendWithConversion("time stamp didn't match delta = ");
+        errorMsg.AppendInt(actualFolderTimeStamp - folderDate);
+        errorMsg.AppendWithConversion(" leeway = ");
+        errorMsg.AppendInt(gTimeStampLeeway);
+      }
+    }
+    else if (folderSize != m_folderSpec->GetFileSize())
+    {
+      errorMsg.AppendWithConversion("folder size didn't match db size = ");
+      errorMsg.AppendInt(folderSize);
+      errorMsg.AppendWithConversion(" actual size = ");
+      errorMsg.AppendInt(m_folderSpec->GetFileSize());
+    }
+    else if (numNewMessages < 0)
+    {
+      errorMsg.AppendWithConversion("numNewMessages < 0");
+    }
+  }
+  if (errorMsg.Length())
+  {
+    nsCOMPtr<nsIPrompt> dialog;
+
+    nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService(NS_WINDOWWATCHER_CONTRACTID));
+    if (wwatch)
+      wwatch->GetNewPrompter(0, getter_AddRefs(dialog));
+    if (dialog)
+      dialog->Alert(nsnull, errorMsg.get());
+  }
+#endif // PUTUP_ALERT_ON_INVALID_DB
   return NS_OK;
 }
 
@@ -568,22 +587,8 @@ NS_IMETHODIMP  nsMailDatabase::RemoveOfflineOp(nsIMsgOfflineImapOperation *op)
   return rv;
 }
 
-nsresult SetSourceMailbox(nsOfflineImapOperation *op, const char *mailbox, nsMsgKey key)
-{
-	nsresult ret = NS_OK;
-	return ret;
-}
-
-	
-nsresult nsMailDatabase::GetIdsWithNoBodies (nsMsgKeyArray &bodylessIds)
-{
-	nsresult ret = NS_OK;
-	return ret;
-}
-
 NS_IMETHODIMP nsMailDatabase::GetOfflineOpForKey(nsMsgKey msgKey, PRBool create, nsIMsgOfflineImapOperation **offlineOp)
 {
-  PRBool newOp = PR_FALSE;
   mdb_bool	hasOid;
   mdbOid		rowObjectId;
   mdb_err   err;
@@ -612,10 +617,7 @@ NS_IMETHODIMP nsMailDatabase::GetOfflineOpForKey(nsMsgKey msgKey, PRBool create,
         NS_ENSURE_SUCCESS(err, err);
       }
       if (offlineOpRow && !hasOid)
-      {
         m_mdbAllOfflineOpsTable->AddRow(GetEnv(), offlineOpRow);
-        newOp = PR_TRUE;
-      }
     }
     
     if (err == NS_OK && offlineOpRow)
@@ -798,7 +800,6 @@ nsresult nsMailDatabase::SetFolderInfoValid(nsFileSpec *folderName, int num, int
     
     pMessageDB->m_dbFolderInfo->SetFolderSize(folderName->GetFileSize());
     pMessageDB->m_dbFolderInfo->SetFolderDate(actualFolderTimeStamp);
-    pMessageDB->m_dbFolderInfo->ChangeNumVisibleMessages(num);
     pMessageDB->m_dbFolderInfo->ChangeNumNewMessages(numunread);
     pMessageDB->m_dbFolderInfo->ChangeNumMessages(num);
   }
@@ -821,26 +822,16 @@ nsresult nsMailDatabase::SetFolderInfoValid(nsFileSpec *folderName, int num, int
 // and needs to be regenerated.
 void nsMailDatabase::SetReparse(PRBool reparse)
 {
-	m_reparse = reparse;
+  m_reparse = reparse;
 }
 
 
-static PRBool gGotThreadingPrefs = PR_FALSE;
-static PRBool gThreadWithoutRe = PR_TRUE;
-
-
 // should we thread messages with common subjects that don't start with Re: together?
 // I imagine we might have separate preferences for mail and news, so this is a virtual method.
 PRBool	nsMailDatabase::ThreadBySubjectWithoutRe()
 {
-	if (!gGotThreadingPrefs)
-	{
-		GetBoolPref("mail.thread_without_re", &gThreadWithoutRe);
-		gGotThreadingPrefs = PR_TRUE;
-	}
-
-	gThreadWithoutRe = PR_TRUE;	// we need to this to be true for now.
-	return gThreadWithoutRe;
+  GetGlobalPrefs();
+  return gThreadWithoutRe;
 }
 
 class nsMsgOfflineOpEnumerator : public nsISimpleEnumerator {
