@@ -199,6 +199,9 @@ PRBool nsMsgMdnGenerator::ProcessSendMode()
         if (!accountDomain)
             return m_reallySendMdn;
 
+        if (MailAddrMatch(m_email.get(), m_dntRrt.get())) // return address is self, don't send
+          return PR_FALSE;
+
         // *** fix me see Bug 132504 for more information
         // *** what if the message has been filtered to different account
         if (!PL_strcasestr(m_dntRrt, accountDomain))
@@ -499,7 +502,7 @@ nsresult nsMsgMdnGenerator::CreateFirstPart()
       compUtils->GetMsgMimeConformToStandard(&conformToStandard);
 
     convbuf = nsMsgI18NEncodeMimePartIIStr(
-        m_email.get(), PR_TRUE, NS_LossyConvertUCS2toASCII(m_charset).get(), 0,
+        m_email.get(), PR_TRUE, m_charset.get(), 0,
         conformToStandard);
     parm = PR_smprintf("From: %s" CRLF, convbuf ? convbuf : m_email.get());
 
@@ -564,7 +567,7 @@ nsresult nsMsgMdnGenerator::CreateFirstPart()
     m_headers->ExtractHeader(HEADER_SUBJECT, PR_FALSE, getter_Copies(subject));
     convbuf = nsMsgI18NEncodeMimePartIIStr(
         subject.Length() ? subject.get() : "[no subject]", 
-        PR_TRUE, NS_LossyConvertUCS2toASCII(m_charset).get(), 0,
+        PR_TRUE, m_charset.get(), 0,
         conformToStandard);
     tmpBuffer = PR_smprintf("Subject: %s - %s" CRLF, 
                             (receipt_string ? 
@@ -578,7 +581,7 @@ nsresult nsMsgMdnGenerator::CreateFirstPart()
     PR_Free(convbuf);
 
     convbuf = nsMsgI18NEncodeMimePartIIStr(
-        m_dntRrt, PR_TRUE, NS_LossyConvertUCS2toASCII(m_charset).get(), 0,
+        m_dntRrt, PR_TRUE, m_charset.get(), 0,
         conformToStandard);
 
     tmpBuffer = PR_smprintf("To: %s" CRLF, convbuf ? convbuf :
@@ -610,7 +613,7 @@ report-type=disposition-notification;\r\n\tboundary=\"%s\"" CRLF CRLF,
     PUSH_N_FREE_STRING(tmpBuffer);
 
     tmpBuffer = PR_smprintf("Content-Type: text/plain; charset=%s" CRLF,
-                            NS_LossyConvertUCS2toASCII(m_charset).get());
+                            m_charset.get());
     PUSH_N_FREE_STRING(tmpBuffer);
 
     tmpBuffer = PR_smprintf("Content-Transfer-Encoding: %s" CRLF CRLF,
@@ -727,7 +730,7 @@ nsresult nsMsgMdnGenerator::CreateSecondPart()
       compUtils->GetMsgMimeConformToStandard(&conformToStandard);
 
     convbuf = nsMsgI18NEncodeMimePartIIStr(
-        m_email.get(), PR_TRUE, NS_LossyConvertUCS2toASCII(m_charset).get(), 0,
+        m_email.get(), PR_TRUE, m_charset.get(), 0,
         conformToStandard);
     tmpBuffer = PR_smprintf("Final-Recipient: rfc822;%s" CRLF, convbuf ?
                             convbuf : m_email.get()); 
@@ -841,11 +844,10 @@ nsresult nsMsgMdnGenerator::OutputAllHeaders()
     
         if (end > start && *end == 0)
         {
-            // strip out private X-Mozilla-Status header & X-Mozilla-Draft-Info
+            // strip out private X-Mozilla-Status header & X-Mozilla-Draft-Info && envelope header
             if (!PL_strncasecmp(start, X_MOZILLA_STATUS, X_MOZILLA_STATUS_LEN)
-                || 
-                !PL_strncasecmp(start, X_MOZILLA_DRAFT_INFO,
-                                X_MOZILLA_DRAFT_INFO_LEN))
+                || !PL_strncasecmp(start, X_MOZILLA_DRAFT_INFO, X_MOZILLA_DRAFT_INFO_LEN)
+                || !PL_strncasecmp(start, "From ", 5))
             {
                 while ( end < buf_end && 
                         (*end == nsCRT::LF || *end == nsCRT::CR || *end == 0))
