@@ -81,7 +81,7 @@ nsNoIncomingServer::SetFlagsOnDefaultMailboxes()
 {
     nsresult rv;
     
-    nsCOMPtr<nsIFolder> rootFolder;
+    nsCOMPtr<nsIMsgFolder> rootFolder;
     rv = GetRootFolder(getter_AddRefs(rootFolder));
     NS_ENSURE_SUCCESS(rv, rv);
 
