@@ -75,7 +75,6 @@
 #include "nsMsgSearchCore.h"
 
 static NS_DEFINE_CID(kCMailDB, NS_MAILDB_CID);
-static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID);
 static NS_DEFINE_CID(kIOServiceCID,              NS_IOSERVICE_CID);
 static NS_DEFINE_CID(kRDFServiceCID, NS_RDFSERVICE_CID);
 
@@ -228,7 +227,7 @@ nsMsgMailboxParser::nsMsgMailboxParser() : nsMsgLineBuffer(nsnull, PR_FALSE)
 nsMsgMailboxParser::nsMsgMailboxParser(nsIMsgFolder *aFolder) : nsMsgLineBuffer(nsnull, PR_FALSE)
 {
   Init();
-  m_folder = getter_AddRefs(NS_GetWeakReference(aFolder));
+  m_folder = do_GetWeakReference(aFolder);
 }
 
 nsMsgMailboxParser::~nsMsgMailboxParser()
@@ -370,9 +369,7 @@ PRInt32 nsMsgMailboxParser::PublishMsgHeader(nsIMsgWindow *msgWindow)
     }
     else if (m_mailDB)
     {
-      // add hdr but don't notify - shouldn't be requiring notifications 
-      // during summary file rebuilding
-      m_mailDB->AddNewHdrToDB(m_newMsgHdr, PR_FALSE);
+      m_mailDB->AddNewHdrToDB(m_newMsgHdr, PR_TRUE);
       m_newMsgHdr = nsnull;
     }
     else
@@ -524,17 +521,17 @@ NS_IMETHODIMP nsParseMailMessageState::Clear()
 
 NS_IMETHODIMP nsParseMailMessageState::SetState(nsMailboxParseState aState)
 {
-	m_state = aState;
-	return NS_OK;
+  m_state = aState;
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsParseMailMessageState::GetState(nsMailboxParseState *aState)
 {
-	if (!aState) 
-		return NS_ERROR_NULL_POINTER;
-
-	*aState = m_state;
-	return NS_OK;
+  if (!aState) 
+    return NS_ERROR_NULL_POINTER;
+  
+  *aState = m_state;
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -548,10 +545,10 @@ nsParseMailMessageState::GetEnvelopePos(PRUint32 *aEnvelopePos)
 
 NS_IMETHODIMP nsParseMailMessageState::SetEnvelopePos(PRUint32 aEnvelopePos)
 {
-	m_envelope_pos = aEnvelopePos;
-    m_position = m_envelope_pos;
-    m_headerstartpos = m_position;
-	return NS_OK;
+  m_envelope_pos = aEnvelopePos;
+  m_position = m_envelope_pos;
+  m_headerstartpos = m_position;
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsParseMailMessageState::GetNewMsgHdr(nsIMsgDBHdr ** aMsgHeader)
@@ -566,52 +563,52 @@ NS_IMETHODIMP nsParseMailMessageState::GetNewMsgHdr(nsIMsgDBHdr ** aMsgHeader)
 
 NS_IMETHODIMP nsParseMailMessageState::ParseAFolderLine(const char *line, PRUint32 lineLength)
 {
-	ParseFolderLine(line, lineLength);
-	return NS_OK;
+  ParseFolderLine(line, lineLength);
+  return NS_OK;
 }
 
 PRInt32 nsParseMailMessageState::ParseFolderLine(const char *line, PRUint32 lineLength)
 {
-	int status = 0;
-
-	if (m_state == nsIMsgParseMailMsgState::ParseHeadersState)
-	{
-		if (EMPTY_MESSAGE_LINE(line))
-		{
-		  /* End of headers.  Now parse them. */
-			status = ParseHeaders();
-			 NS_ASSERTION(status >= 0, "error parsing headers parsing mailbox");
-			if (status < 0)
-				return status;
-
-			 status = FinalizeHeaders();
-			 NS_ASSERTION(status >= 0, "error finalizing headers parsing mailbox");
-			if (status < 0)
-				return status;
-			 m_state = nsIMsgParseMailMsgState::ParseBodyState;
-		}
-		else
-		{
+  int status = 0;
+  
+  if (m_state == nsIMsgParseMailMsgState::ParseHeadersState)
+  {
+    if (EMPTY_MESSAGE_LINE(line))
+    {
+      /* End of headers.  Now parse them. */
+      status = ParseHeaders();
+      NS_ASSERTION(status >= 0, "error parsing headers parsing mailbox");
+      if (status < 0)
+        return status;
+      
+      status = FinalizeHeaders();
+      NS_ASSERTION(status >= 0, "error finalizing headers parsing mailbox");
+      if (status < 0)
+        return status;
+      m_state = nsIMsgParseMailMsgState::ParseBodyState;
+    }
+    else
+    {
 		  /* Otherwise, this line belongs to a header.  So append it to the
-			 header data, and stay in MBOX `MIME_PARSE_HEADERS' state.
-		   */
-			m_headers.AppendBuffer(line, lineLength);
-		}
-	}
-	else if ( m_state == nsIMsgParseMailMsgState::ParseBodyState)
-	{
-		m_body_lines++;
-	}
-
-	m_position += lineLength;
-
-	return 0;
+                  header data, and stay in MBOX `MIME_PARSE_HEADERS' state.
+      */
+      m_headers.AppendBuffer(line, lineLength);
+    }
+  }
+  else if ( m_state == nsIMsgParseMailMsgState::ParseBodyState)
+  {
+    m_body_lines++;
+  }
+  
+  m_position += lineLength;
+  
+  return 0;
 }
 
 NS_IMETHODIMP nsParseMailMessageState::SetMailDB(nsIMsgDatabase *mailDB)
 {
-	m_mailDB = dont_QueryInterface(mailDB);
-	return NS_OK;
+  m_mailDB = mailDB;
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsParseMailMessageState::SetDBFolderStream(nsIOFileStream *fileStream)
@@ -1560,160 +1557,157 @@ nsresult nsParseNewMailState::GetTrashFolder(nsIMsgFolder **pTrashFolder)
 
 void nsParseNewMailState::ApplyFilters(PRBool *pMoved, nsIMsgWindow *msgWindow)
 {
-	m_msgMovedByFilter = PR_FALSE;
-
-	nsCOMPtr<nsIMsgDBHdr> msgHdr = m_newMsgHdr;
-	nsCOMPtr<nsIMsgFolder> inbox;
-	nsCOMPtr <nsIMsgFolder> rootMsgFolder = do_QueryInterface(m_rootFolder);
-	if (rootMsgFolder)
-	{
-		PRUint32 numFolders;
-		rootMsgFolder->GetFoldersWithFlag(MSG_FOLDER_FLAG_INBOX, 1, &numFolders, getter_AddRefs(inbox));
+  m_msgMovedByFilter = PR_FALSE;
+  
+  nsCOMPtr<nsIMsgDBHdr> msgHdr = m_newMsgHdr;
+  nsCOMPtr<nsIMsgFolder> inbox;
+  nsCOMPtr <nsIMsgFolder> rootMsgFolder = do_QueryInterface(m_rootFolder);
+  if (rootMsgFolder)
+  {
+    PRUint32 numFolders;
+    rootMsgFolder->GetFoldersWithFlag(MSG_FOLDER_FLAG_INBOX, 1, &numFolders, getter_AddRefs(inbox));
     if (inbox)
       inbox->GetURI(getter_Copies(m_inboxUri));
-		char * headers = m_headers.GetBuffer();
-		PRUint32 headersSize = m_headers.GetBufferPos();
-		nsresult matchTermStatus;
-        if (m_filterList)
-            matchTermStatus =
-                m_filterList->ApplyFiltersToHdr(nsMsgFilterType::InboxRule,
-                                                msgHdr, inbox, m_mailDB,
-                                                headers, headersSize, this,
-                                                msgWindow);
-	}
-
-	if (pMoved)
-		*pMoved = m_msgMovedByFilter;
+    char * headers = m_headers.GetBuffer();
+    PRUint32 headersSize = m_headers.GetBufferPos();
+    nsresult matchTermStatus;
+    if (m_filterList)
+      matchTermStatus = m_filterList->ApplyFiltersToHdr(nsMsgFilterType::InboxRule,
+                  msgHdr, inbox, m_mailDB, headers, headersSize, this, msgWindow);
+  }
+  
+  if (pMoved)
+    *pMoved = m_msgMovedByFilter;
 }
 
 NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWindow *msgWindow, PRBool *applyMore)
 {
   NS_ENSURE_ARG_POINTER(applyMore);
-
-	nsMsgRuleActionType actionType;
+  
+  nsMsgRuleActionType actionType;
   nsXPIDLCString actionTargetFolderUri;
-	PRUint32	newFlags;
-	nsresult rv = NS_OK;
-
-	*applyMore = PR_TRUE;
-
+  PRUint32	newFlags;
+  nsresult rv = NS_OK;
+  
+  *applyMore = PR_TRUE;
+  
   nsCOMPtr<nsIMsgDBHdr> msgHdr = m_newMsgHdr;
-
+  
   nsCOMPtr<nsISupportsArray> filterActionList;
   rv = NS_NewISupportsArray(getter_AddRefs(filterActionList));
   NS_ENSURE_SUCCESS(rv, rv);
   rv = filter->GetSortedActionList(filterActionList);
   NS_ENSURE_SUCCESS(rv, rv);
-
+  
   // look at action - currently handle move
 #ifdef DEBUG_bienvenu
-	printf("got a rule hit!\n");
+  printf("got a rule hit!\n");
 #endif
-
+  
   PRUint32 numActions;
   rv = filterActionList->Count(&numActions);
   NS_ENSURE_SUCCESS(rv, rv);
-
+  
   PRBool loggingEnabled = PR_FALSE;
   if (m_filterList && numActions)
-	  m_filterList->GetLoggingEnabled(&loggingEnabled);
-
+    m_filterList->GetLoggingEnabled(&loggingEnabled);
+  
   for (PRUint32 actionIndex =0; actionIndex < numActions && *applyMore; actionIndex++)
   {
     nsCOMPtr<nsIMsgRuleAction> filterAction;
     filterActionList->QueryElementAt(actionIndex, NS_GET_IID(nsIMsgRuleAction), getter_AddRefs(filterAction));
     if (!filterAction)
       continue;
-
-	  if (NS_SUCCEEDED(filterAction->GetType(&actionType)))
+    
+    if (NS_SUCCEEDED(filterAction->GetType(&actionType)))
     {
       if (actionType == nsMsgFilterAction::MoveToFolder)
       {
-         filterAction->GetTargetFolderUri(getter_Copies(actionTargetFolderUri));
-         if (actionTargetFolderUri.IsEmpty())
-         {
-           NS_ASSERTION(PR_FALSE, "actionTargetFolderUri is empty");
-           continue;
-         }
-      }
-		  switch (actionType)
-      {
-		    case nsMsgFilterAction::Delete:
+        filterAction->GetTargetFolderUri(getter_Copies(actionTargetFolderUri));
+        if (actionTargetFolderUri.IsEmpty())
         {
-			    nsCOMPtr <nsIMsgFolder> trash;
-			    // set value to trash folder
-			    rv = GetTrashFolder(getter_AddRefs(trash));
-			    if (NS_SUCCEEDED(rv) && trash)
-				    rv = trash->GetURI(getter_Copies(actionTargetFolderUri));
-
+          NS_ASSERTION(PR_FALSE, "actionTargetFolderUri is empty");
+          continue;
+        }
+      }
+      switch (actionType)
+      {
+      case nsMsgFilterAction::Delete:
+        {
+          nsCOMPtr <nsIMsgFolder> trash;
+          // set value to trash folder
+          rv = GetTrashFolder(getter_AddRefs(trash));
+          if (NS_SUCCEEDED(rv) && trash)
+            rv = trash->GetURI(getter_Copies(actionTargetFolderUri));
+          
           msgHdr->OrFlags(MSG_FLAG_READ, &newFlags);	// mark read in trash.
         }
-		    case nsMsgFilterAction::MoveToFolder:
-			    // if moving to a different file, do it.
-			    if ((const char*)actionTargetFolderUri &&
-              nsCRT::strcasecmp(m_inboxUri, actionTargetFolderUri))
+      case nsMsgFilterAction::MoveToFolder:
+        // if moving to a different file, do it.
+        if ((const char*)actionTargetFolderUri &&
+          nsCRT::strcasecmp(m_inboxUri, actionTargetFolderUri))
+        {
+          nsresult err = MoveIncorporatedMessage(msgHdr, m_mailDB, actionTargetFolderUri, filter, msgWindow);
+          if (NS_SUCCEEDED(err))
           {
-            nsresult err = MoveIncorporatedMessage(msgHdr, m_mailDB, actionTargetFolderUri, filter, msgWindow);
-            if (NS_SUCCEEDED(err))
-            {
-              if (loggingEnabled)
-                (void)filter->LogRuleHit(filterAction, msgHdr); 
-
-              // cleanup after mailHdr in source DB because we moved the message.
-              m_mailDB->RemoveHeaderMdbRow(msgHdr);
-              m_msgMovedByFilter = PR_TRUE;
-            }
+            if (loggingEnabled)
+              (void)filter->LogRuleHit(filterAction, msgHdr); 
+            
+            // cleanup after mailHdr in source DB because we moved the message.
+            m_mailDB->RemoveHeaderMdbRow(msgHdr);
+            m_msgMovedByFilter = PR_TRUE;
           }
-          *applyMore = PR_FALSE; 
-          break;
-		    case nsMsgFilterAction::MarkRead:
-			    MarkFilteredMessageRead(msgHdr);
-			    break;
-		    case nsMsgFilterAction::KillThread:
-			    // The db will check for this flag when a hdr gets added to the db, and set the flag appropriately on the thread object
-			    msgHdr->OrFlags(MSG_FLAG_IGNORED, &newFlags);
-			    break;
-		    case nsMsgFilterAction::WatchThread:
-			    msgHdr->OrFlags(MSG_FLAG_WATCHED, &newFlags);
-			    break;
-        case nsMsgFilterAction::MarkFlagged:
-          msgHdr->MarkFlagged(PR_TRUE);
-          break;
-		    case nsMsgFilterAction::ChangePriority:
-          nsMsgPriorityValue filterPriority;
-          filterAction->GetPriority(&filterPriority);
-          msgHdr->SetPriority(filterPriority);
-          break;
-		    case nsMsgFilterAction::Label:
-          nsMsgLabelValue filterLabel;
-          filterAction->GetLabel(&filterLabel);
-          nsMsgKey msgKey;
-          msgHdr->GetMessageKey(&msgKey);
-          m_mailDB->SetLabel(msgKey, filterLabel);
-			    break;
-		    default:
-			    break;
+        }
+        *applyMore = PR_FALSE; 
+        break;
+      case nsMsgFilterAction::MarkRead:
+        MarkFilteredMessageRead(msgHdr);
+        break;
+      case nsMsgFilterAction::KillThread:
+        // The db will check for this flag when a hdr gets added to the db, and set the flag appropriately on the thread object
+        msgHdr->OrFlags(MSG_FLAG_IGNORED, &newFlags);
+        break;
+      case nsMsgFilterAction::WatchThread:
+        msgHdr->OrFlags(MSG_FLAG_WATCHED, &newFlags);
+        break;
+      case nsMsgFilterAction::MarkFlagged:
+        msgHdr->MarkFlagged(PR_TRUE);
+        break;
+      case nsMsgFilterAction::ChangePriority:
+        nsMsgPriorityValue filterPriority;
+        filterAction->GetPriority(&filterPriority);
+        msgHdr->SetPriority(filterPriority);
+        break;
+      case nsMsgFilterAction::Label:
+        nsMsgLabelValue filterLabel;
+        filterAction->GetLabel(&filterLabel);
+        nsMsgKey msgKey;
+        msgHdr->GetMessageKey(&msgKey);
+        m_mailDB->SetLabel(msgKey, filterLabel);
+        break;
+      default:
+        break;
       }
       if (loggingEnabled && actionType != nsMsgFilterAction::MoveToFolder && actionType != nsMsgFilterAction::Delete)  
         (void)filter->LogRuleHit(filterAction, msgHdr); 
     }
-	}
-	return rv;
+  }
+  return rv;
 }
 
 int nsParseNewMailState::MarkFilteredMessageRead(nsIMsgDBHdr *msgHdr)
 {
-	PRUint32 newFlags;
-	if (m_mailDB)
-		m_mailDB->MarkHdrRead(msgHdr, PR_TRUE, nsnull);
-	else
-		msgHdr->OrFlags(MSG_FLAG_READ, &newFlags);
-	return 0;
+  PRUint32 newFlags;
+  if (m_mailDB)
+    m_mailDB->MarkHdrRead(msgHdr, PR_TRUE, nsnull);
+  else
+    msgHdr->OrFlags(MSG_FLAG_READ, &newFlags);
+  return 0;
 }
 
 nsresult nsParseNewMailState::MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr, 
-											   nsIMsgDatabase *sourceDB, 
-											   const nsACString& destFolderUri,
+                                                      nsIMsgDatabase *sourceDB, 
+                                                      const nsACString& destFolderUri,
                                                       nsIMsgFilter *filter,
                                                       nsIMsgWindow *msgWindow)
 {
@@ -1721,6 +1715,7 @@ nsresult nsParseNewMailState::MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr,
   nsIOFileStream *destFile;
   
   nsCOMPtr<nsIRDFService> rdf(do_GetService(kRDFServiceCID, &err)); 
+  NS_ENSURE_SUCCESS(err, err);
   nsCOMPtr<nsIRDFResource> res;
   err = rdf->GetResource(destFolderUri, getter_AddRefs(res));
   if (NS_FAILED(err))
