@@ -232,6 +232,8 @@ nsresult nsPop3Service::RunPopUrl(nsIMsgIncomingServer * aServer, nsIURI * aUrlT
         protocol->SetUsername(userName);
         rv = protocol->LoadUrl(aUrlToRun);
         NS_RELEASE(protocol);
+        if (NS_FAILED(rv))
+          aServer->SetServerBusy(PR_FALSE);
       }
     } 
   } // if server
@@ -289,8 +291,7 @@ NS_IMETHODIMP nsPop3Service::NewURI(const nsACString &aSpec,
 
     nsCOMPtr<nsIRDFService> rdfService(do_GetService(kRDFServiceCID, &rv)); 
     if (NS_FAILED(rv)) return rv;
-    rv = rdfService->GetResource(folderUri,
-                                 getter_AddRefs(resource));
+    rv = rdfService->GetResource(folderUri, getter_AddRefs(resource));
     if (NS_FAILED(rv)) return rv;
     nsCOMPtr<nsIMsgFolder> folder = do_QueryInterface(resource, &rv);
     if (NS_FAILED(rv)) return rv;
