@@ -350,13 +350,8 @@ static int do_lzw(gif_struct *gs, const PRUint8 *q)
         continue;
       }
 
-      /* Check for a code not defined in the dictionary yet. */
-      if (code > avail)
-        return -1;
-
       incode = code;
-      if (code == avail) {
-        /* the first code is always < avail */
+      if (code >= avail) {
         *stackp++ = firstchar;
         code = oldcode;
 
@@ -364,10 +359,8 @@ static int do_lzw(gif_struct *gs, const PRUint8 *q)
           return -1;
       }
 
-      int code2 = 0;
-      while (code > clear_code)
+      while (code >= clear_code)
       {
-        code2 = code;
         if (code == prefix[code])
           return -1;
 
@@ -376,9 +369,6 @@ static int do_lzw(gif_struct *gs, const PRUint8 *q)
 
         if (stackp == stack + MAX_BITS)
           return -1;
-
-        if (code2 == prefix[code])
-          return -1;
       }
 
       *stackp++ = firstchar = suffix[code];
@@ -428,18 +418,6 @@ static int do_lzw(gif_struct *gs, const PRUint8 *q)
   return 0;
 }
 
-PRBool gif_create(gif_struct **gs)
-{
-  gif_struct *ret = PR_NEWZAP(gif_struct);
-
-  if (!ret)
-    return PR_FALSE;
-
-  *gs = ret;
-
-  return PR_TRUE;
-}
-
 static inline void *gif_calloc(size_t n, size_t s)
 {
   if (!gGifAllocator)
