@@ -38,7 +38,7 @@
 // GIF Decoder Implementation
 // This is an adaptor between GIF2 and imgIDecoder
 
-NS_IMPL_ISUPPORTS1(nsGIFDecoder2, imgIDecoder);
+NS_IMPL_ISUPPORTS1(nsGIFDecoder2, imgIDecoder)
 
 nsGIFDecoder2::nsGIFDecoder2()
   : mCurrentRow(-1)
@@ -83,9 +83,9 @@ NS_IMETHODIMP nsGIFDecoder2::Init(imgILoad *aLoad)
   /* do gif init stuff */
   /* Always decode to 24 bit pixdepth */
   
-  PRBool created = gif_create(&mGIFStruct);
-  NS_ASSERTION(created, "gif_create failed");
-  if (!created)
+  mGIFStruct = (gif_struct *)PR_CALLOC(sizeof(gif_struct));
+  NS_ASSERTION(mGIFStruct, "gif_create failed");
+  if (!mGIFStruct)
     return NS_ERROR_FAILURE;
 
   // Call GIF decoder init routine
@@ -436,7 +436,7 @@ int nsGIFDecoder2::HaveDecodedRow(
       bgColor |= cmap[bgIndex + 2] << 16;
       decoder->mImageFrame->SetBackgroundColor(bgColor);
     }
-    if(decoder->mGIFStruct->local_colormap) {
+    if (decoder->mGIFStruct->is_local_colormap_defined) {
       cmapsize = decoder->mGIFStruct->local_colormap_size;
       cmap = decoder->mGIFStruct->local_colormap;
     }
@@ -457,15 +457,22 @@ int nsGIFDecoder2::HaveDecodedRow(
       
       switch (format) {
         case gfxIFormats::RGB:
+        case gfxIFormats::BGR:
         {
           while (rowBufIndex != decoder->mGIFStruct->rowend) {
-            PRUint32 colorIndex = *rowBufIndex * 3;
+            PRUint32 colorIndex = (*rowBufIndex < cmapsize) ? (*rowBufIndex * 3) : 0;
+#if defined(XP_WIN) || defined(XP_OS2) || defined(XP_BEOS) || defined(MOZ_WIDGET_PHOTON)
+            *rgbRowIndex++ = cmap[colorIndex + 2]; // blue
+            *rgbRowIndex++ = cmap[colorIndex + 1]; // green
+            *rgbRowIndex++ = cmap[colorIndex];     // red
+#else
 #if defined(XP_MAC) || defined(XP_MACOSX)
             *rgbRowIndex++ = 0; // Mac is always 32bits per pixel, this is pad
 #endif
             *rgbRowIndex++ = cmap[colorIndex];     // red
             *rgbRowIndex++ = cmap[colorIndex + 1]; // green
             *rgbRowIndex++ = cmap[colorIndex + 2]; // blue
+#endif
             ++rowBufIndex;
           }  
           for (int i=0; i<aDuplicateCount; i++) {
@@ -474,21 +481,6 @@ int nsGIFDecoder2::HaveDecodedRow(
           }
           break;
         }
-        case gfxIFormats::BGR:
-        {
-          while (rowBufIndex != decoder->mGIFStruct->rowend) {
-            PRUint32 colorIndex = *rowBufIndex * 3;
-            *rgbRowIndex++ = cmap[colorIndex + 2]; // blue
-            *rgbRowIndex++ = cmap[colorIndex + 1]; // green
-            *rgbRowIndex++ = cmap[colorIndex];     // red
-            ++rowBufIndex;
-          }
-          for (int i = 0; i < aDuplicateCount; ++i) {
-            decoder->mImageFrame->SetImageData(decoder->mRGBLine,
-                                               bpr, (aRowNumber+i)*bpr);
-          }
-          break;
-        }
         case gfxIFormats::RGB_A1:
         case gfxIFormats::BGR_A1:
         {
@@ -496,7 +488,7 @@ int nsGIFDecoder2::HaveDecodedRow(
           memset(decoder->mAlphaLine, 0, abpr);
           for (PRUint32 x = 0; x < (PRUint32)width; ++x) {
             if (*rowBufIndex != decoder->mGIFStruct->tpixel) {
-              PRUint32 colorIndex = *rowBufIndex * 3;
+              PRUint32 colorIndex = (*rowBufIndex < cmapsize) ? (*rowBufIndex * 3) : 0;
 #if defined(XP_WIN) || defined(XP_OS2) || defined(XP_BEOS) || defined(MOZ_WIDGET_PHOTON)
               *rgbRowIndex++ = cmap[colorIndex + 2]; // blue
               *rgbRowIndex++ = cmap[colorIndex + 1]; // green
