@@ -39,9 +39,6 @@
 pref("browser.drag_out_of_frame_style", 1);
 pref("ui.key.saveLink.shift", false); // true = shift, false = meta
 
-pref("editor.use_html_editor",              false);
-pref("editor.use_image_editor",             false);
-
 // should a GURL event open a new window or re-use (4.x compat)
 pref("browser.always_reuse_window", false);
 
@@ -137,6 +134,16 @@ pref("font.name.monospace.zh-TW", "Apple LiGothic Medium");
 pref("font.name.cursive.zh-TW", "XXX.cursive");
 pref("font.name.fantasy.zh-TW", "XXX.fantasy");
 
+// ming_uni.ttf (HKSCS-2001) 
+// http://www.info.gov.hk/digital21/eng/hkscs/download/uime.exe
+// use Stuffit Expander to unzip it and install ming_uni.ttf
+pref("font.name.serif.zh-HK", "Ming(for ISO10646)");
+pref("font.name.sans-serif.zh-HK", "Apple LiGothic Medium");  
+pref("font.name.monospace.zh-HK", "Apple LiGothic Medium");  
+pref("font.name.cursive.zh-HK", "XXX.cursive");
+pref("font.name.fantasy.zh-HK", "XXX.fantasy");
+
+pref("font.default", "serif");
 pref("font.default", "serif");
 pref("font.size.variable.ar", 16);
 pref("font.size.fixed.ar", 13);
@@ -180,6 +187,9 @@ pref("font.size.fixed.zh-CN", 16);
 pref("font.size.variable.zh-TW", 15);
 pref("font.size.fixed.zh-TW", 16);
 
+pref("font.size.variable.zh-HK", 15);
+pref("font.size.fixed.zh-HK", 16);
+
 pref("browser.urlbar.clickAtEndSelects", false);
 
 // Tab focus model bit field:
