@@ -56,6 +56,7 @@
 #include "pldhash.h"
 
 #include "prefapi.h"
+#include "prefread.h"
 #include "prefapi_private_data.h"
 
 // supporting PREF_Init()
@@ -63,31 +64,45 @@
 
 #include "nsITimelineService.h"
 
+#ifdef MOZ_PROFILESHARING
+#include "nsIProfileSharingSetup.h"
+#include "nsSharedPrefHandler.h"
+#endif
+
 // Definitions
 #define INITIAL_MAX_DEFAULT_PREF_FILES 10
-
+#define PREF_READ_BUFFER_SIZE 4096
 
 // Prototypes
-static nsresult openPrefFile(nsIFile* aFile, PRBool aIsErrorFatal,
-                             PRBool aIsGlobalContext, PRBool aSkipFirstLine);
+#ifdef MOZ_PROFILESHARING
+static PRBool isSharingEnabled();
+#endif
 
+static nsresult openPrefFile(nsIFile* aFile);
+static nsresult pref_InitInitialObjects(void);
 
   // needed so we can still get the JS Runtime Service during XPCOM shutdown
 static nsIJSRuntimeService* gJSRuntimeService = nsnull; // owning reference
 
+//-----------------------------------------------------------------------------
 
 /*
  * Constructor/Destructor
  */
 
 nsPrefService::nsPrefService()
-: mCurrentFile(nsnull)
+: mCurrentFile(nsnull),
+  mErrorOpeningUserPrefs(PR_FALSE)
 {
   nsPrefBranch *rootBranch;
 
   rootBranch = new nsPrefBranch("", PR_FALSE); 
   mRootBranch = (nsIPrefBranch *)rootBranch;
-
+  
+#if MOZ_PROFILESHARING
+  mCurrentSharedFile = nsnull;
+  mErrorOpeningSharedUserPrefs = PR_FALSE;
+#endif
 }
 
 nsPrefService::~nsPrefService()
@@ -95,6 +110,11 @@ nsPrefService::~nsPrefService()
   PREF_Cleanup();
   NS_IF_RELEASE(mCurrentFile);
   NS_IF_RELEASE(gJSRuntimeService);
+
+#ifdef MOZ_PROFILESHARING
+  NS_IF_RELEASE(mCurrentSharedFile);
+  NS_IF_RELEASE(gSharedPrefHandler);
+#endif
 }
 
 
@@ -124,8 +144,11 @@ nsresult nsPrefService::Init()
   nsXPIDLCString lockFileName;
   nsresult rv;
 
-  if (!PREF_Init(nsnull))
-    return NS_ERROR_FAILURE;
+  rv = PREF_Init();
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  rv = pref_InitInitialObjects();
+  NS_ENSURE_SUCCESS(rv, rv);
 
   /*
    * The following is a small hack which will allow us to only load the library
@@ -149,6 +172,11 @@ nsresult nsPrefService::Init()
       rv = observerService->AddObserver(this, "profile-do-change", PR_TRUE);
     }
   }
+
+#ifdef MOZ_PROFILESHARING  
+  rv = NS_CreateSharedPrefHandler(this);
+#endif
+
   return(rv);
 }
 
@@ -164,10 +192,22 @@ NS_IMETHODIMP nsPrefService::Observe(nsISupports *aSubject, const char *aTopic,
       }
     } else {
       rv = SavePrefFile(nsnull);
+#ifdef MOZ_PROFILESHARING
+      if (isSharingEnabled())
+        rv = gSharedPrefHandler->OnSessionEnd();
+#endif
     }
   } else if (!nsCRT::strcmp(aTopic, "profile-do-change")) {
-    ResetUserPrefs();
-    rv = ReadUserPrefs(nsnull);
+  
+#ifdef MOZ_PROFILESHARING
+    if (isSharingEnabled())
+      rv = gSharedPrefHandler->OnSessionBegin();
+    else
+#endif
+    {
+      ResetUserPrefs();
+      rv = ReadUserPrefs(nsnull);
+    }
   }
   return rv;
 }
@@ -182,19 +222,9 @@ NS_IMETHODIMP nsPrefService::ReadUserPrefs(nsIFile *aFile)
     UseUserPrefFile();
 
     NotifyServiceObservers(NS_PREFSERVICE_READ_TOPIC_ID);
-    
-    JS_MaybeGC(gMochaContext);
+
   } else {
-    if (mCurrentFile == aFile)
-      return NS_OK;
-
-    NS_IF_RELEASE(mCurrentFile);
-    mCurrentFile = aFile;
-    NS_ADDREF(mCurrentFile);
-    
-    gErrorOpeningUserPrefs = PR_FALSE;
-
-    rv = openPrefFile(mCurrentFile, PR_TRUE, PR_FALSE, PR_TRUE);
+    rv = ReadAndOwnUserPrefFile(aFile);
   }
   return rv;
 }
@@ -204,10 +234,10 @@ NS_IMETHODIMP nsPrefService::ResetPrefs()
   NotifyServiceObservers(NS_PREFSERVICE_RESET_TOPIC_ID);
   PREF_CleanupPrefs();
 
-  if (!PREF_Init(nsnull))
-    return NS_ERROR_FAILURE;
+  nsresult rv = PREF_Init();
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  return NS_OK;
+  return pref_InitInitialObjects();
 }
 
 NS_IMETHODIMP nsPrefService::ResetUserPrefs()
@@ -218,19 +248,12 @@ NS_IMETHODIMP nsPrefService::ResetUserPrefs()
 
 NS_IMETHODIMP nsPrefService::SavePrefFile(nsIFile *aFile)
 {
-  if (nsnull == aFile) {
-    // the gDirty flag tells us if we should write to mCurrentFile
-    // we only check this flag when the caller wants to write to the default
-    if (!gDirty) {
-      NS_WARNING("not writing prefs because they haven't changed");
-      return NS_OK;
-    }
-    
-    // It's possible that we never got a prefs file.
-    return mCurrentFile ? WritePrefFile(mCurrentFile) : NS_OK;
-  } else {
-    return WritePrefFile(aFile);
-  }
+#ifdef MOZ_PROFILESHARING
+  // sharing only applies to the default prefs file
+  if (aFile == nsnull && isSharingEnabled())
+    return gSharedPrefHandler->OnSavePrefs();
+#endif
+  return SavePrefFileInternal(aFile);
 }
 
 NS_IMETHODIMP nsPrefService::GetBranch(const char *aPrefRoot, nsIPrefBranch **_retval)
@@ -305,29 +328,39 @@ nsresult nsPrefService::NotifyServiceObservers(const char *aTopic)
 
 nsresult nsPrefService::UseDefaultPrefFile()
 {
-  nsresult rv;
+  nsresult rv, rv2;
   nsCOMPtr<nsIFile> aFile;
 
-  // Anything which calls NS_InitXPCOM will have this
-  rv = NS_GetSpecialDirectory(NS_APP_PREFS_50_FILE, getter_AddRefs(aFile));
-
-  if (!aFile) {
-    // We know we have XPCOM directory services, but we might not have a provider which
-    // knows about NS_APP_PREFS_50_FILE. Put the file in NS_XPCOM_CURRENT_PROCESS_DIR.
-    rv = NS_GetSpecialDirectory(NS_XPCOM_CURRENT_PROCESS_DIR, getter_AddRefs(aFile));
-    if (NS_FAILED(rv)) return rv;
-    rv = aFile->AppendNative(NS_LITERAL_CSTRING("default_prefs.js"));
-    if (NS_FAILED(rv)) return rv;
-  } 
-
-  rv = ReadUserPrefs(aFile);
-  if (NS_SUCCEEDED(rv)) {
-    return rv;
+#ifdef MOZ_PROFILESHARING
+  // First, read the shared file.
+  if (isSharingEnabled()) {
+    rv = NS_GetSpecialDirectory(NS_SHARED NS_APP_PREFS_50_FILE, getter_AddRefs(aFile));
+    if (NS_SUCCEEDED(rv)) {
+      rv = ReadAndOwnSharedUserPrefFile(aFile);
+      // Most likely cause of failure here is that the file didn't
+      // exist, so save a new one. mSharedUserPrefReadFailed will be
+      // used to catch an error in actually reading the file.
+      if (NS_FAILED(rv)) {
+        rv2 = SavePrefFileInternal(aFile);
+        NS_ASSERTION(NS_SUCCEEDED(rv2), "Failed to save new shared pref file");
+      }
+    }
   }
+  // Continue on to read the nonshared file.
+#endif
 
-  // need to save the prefs now
-  SavePrefFile(aFile); 
-
+  rv = NS_GetSpecialDirectory(NS_APP_PREFS_50_FILE, getter_AddRefs(aFile));
+  if (NS_SUCCEEDED(rv)) {
+    rv = ReadAndOwnUserPrefFile(aFile);
+    // Most likely cause of failure here is that the file didn't
+    // exist, so save a new one. mUserPrefReadFailed will be
+    // used to catch an error in actually reading the file.
+    if (NS_FAILED(rv)) {
+      rv2 = SavePrefFileInternal(aFile);
+      NS_ASSERTION(NS_SUCCEEDED(rv2), "Failed to save new shared pref file");
+    }
+  }
+  
   return rv;
 }
 
@@ -336,23 +369,133 @@ nsresult nsPrefService::UseUserPrefFile()
   nsresult rv = NS_OK;
   nsCOMPtr<nsIFile> aFile;
 
-  rv = NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR, getter_AddRefs(aFile));
+#ifdef MOZ_PROFILESHARING
+  nsCAutoString prefsDirProp(NS_APP_PREFS_50_DIR);
+  if (isSharingEnabled())
+    prefsDirProp.Insert(NS_SHARED, 0); // Prepend modifier so we get shared file
+#else
+  nsDependentCString prefsDirProp(NS_APP_PREFS_50_DIR);
+#endif
+
+  rv = NS_GetSpecialDirectory(prefsDirProp.get(), getter_AddRefs(aFile));
   if (NS_SUCCEEDED(rv) && aFile) {
     rv = aFile->AppendNative(NS_LITERAL_CSTRING("user.js"));
     if (NS_SUCCEEDED(rv)) {
-      rv = openPrefFile(aFile, PR_FALSE, PR_FALSE, PR_FALSE);
+      rv = openPrefFile(aFile);
     }
   }
   return rv;
 }
 
+nsresult nsPrefService::ReadAndOwnUserPrefFile(nsIFile *aFile)
+{
+  NS_ENSURE_ARG(aFile);
+  
+  if (mCurrentFile == aFile)
+    return NS_OK;
+  NS_IF_RELEASE(mCurrentFile);
+  mCurrentFile = aFile;
+  NS_ADDREF(mCurrentFile);
+
+#ifdef MOZ_PROFILESHARING
+  // We don't want prefs set here to cause transactions
+  gSharedPrefHandler->ReadingUserPrefs(PR_TRUE);
+#endif
+
+  // We need to track errors in reading the shared and the
+  // non-shared files independently. 
+  // Set the appropriate member variable from it after reading.
+  nsresult rv = openPrefFile(mCurrentFile);
+  mErrorOpeningUserPrefs = NS_FAILED(rv);
+
+#ifdef MOZ_PROFILESHARING
+  gSharedPrefHandler->ReadingUserPrefs(PR_FALSE);
+#endif
+
+  return rv;
+}
+
+#ifdef MOZ_PROFILESHARING
+nsresult nsPrefService::ReadAndOwnSharedUserPrefFile(nsIFile *aFile)
+{
+  NS_ENSURE_ARG(aFile);
+
+  if (mCurrentSharedFile == aFile)
+    return NS_OK;
+  NS_IF_RELEASE(mCurrentSharedFile);
+  mCurrentSharedFile = aFile;
+  NS_ADDREF(mCurrentSharedFile);
+
+#ifdef MOZ_PROFILESHARING
+  // We don't want prefs set here to cause transactions
+  gSharedPrefHandler->ReadingUserPrefs(PR_TRUE);
+#endif
+
+  // We need to track errors in reading the shared and the
+  // non-shared files independently. 
+  // Set the appropriate member variable from it after reading.
+  nsresult rv = openPrefFile(mCurrentSharedFile);
+  mErrorOpeningSharedUserPrefs = NS_FAILED(rv);
+
+#ifdef MOZ_PROFILESHARING
+  gSharedPrefHandler->ReadingUserPrefs(PR_FALSE);
+#endif
+
+  return rv;
+}
+#endif
+
+nsresult nsPrefService::SavePrefFileInternal(nsIFile *aFile)
+{
+  if (nsnull == aFile) {
+    // the gDirty flag tells us if we should write to mCurrentFile
+    // we only check this flag when the caller wants to write to the default
+    if (!gDirty)
+      return NS_OK;
+    
+    // It's possible that we never got a prefs file.
+    nsresult rv = NS_OK;
+    if (mCurrentFile)
+      rv = WritePrefFile(mCurrentFile);
+
+#ifdef MOZ_PROFILESHARING
+    if (mCurrentSharedFile) {
+      nsresult rv2 = WritePrefFile(mCurrentSharedFile);
+      if (NS_SUCCEEDED(rv))
+        rv = rv2;
+    }
+#endif
+
+    return rv;
+  } else {
+    return WritePrefFile(aFile);
+  }
+}
+
 nsresult nsPrefService::WritePrefFile(nsIFile* aFile)
 {
-  const char                outHeader[] = "# Mozilla User Preferences" 
-                                          NS_LINEBREAK 
-                                          "// This is a generated file!" 
-                                          NS_LINEBREAK 
-                                          NS_LINEBREAK;
+  const char                outHeader[] =
+    "# Mozilla User Preferences"
+    NS_LINEBREAK
+    NS_LINEBREAK
+    "/* Do not edit this file."
+    NS_LINEBREAK
+    " *"
+    NS_LINEBREAK
+    " * If you make changes to this file while the browser is running,"
+    NS_LINEBREAK
+    " * the changes will be overwritten when the browser exits."
+    NS_LINEBREAK
+    " *"
+    NS_LINEBREAK
+    " * To make a manual change to preferences, you can visit the URL about:config"
+    NS_LINEBREAK
+    " * For more information, see http://www.mozilla.org/unix/customizing.html#prefs"
+    NS_LINEBREAK
+    " */"
+    NS_LINEBREAK
+    NS_LINEBREAK;
+
   nsCOMPtr<nsIOutputStream> outStream;
   PRUint32                  writeAmount;
   nsresult                  rv;
@@ -361,8 +504,12 @@ nsresult nsPrefService::WritePrefFile(nsIFile* aFile)
     return NS_ERROR_NOT_INITIALIZED;
 
   /* ?! Don't save (blank) user prefs if there was an error reading them */
-  if (gErrorOpeningUserPrefs)
+  if (aFile == mCurrentFile && mErrorOpeningUserPrefs)
     return NS_OK;
+#if MOZ_PROFILESHARING
+  if (aFile == mCurrentSharedFile && mErrorOpeningSharedUserPrefs)
+    return NS_OK;
+#endif
 
   // execute a "safe" save by saving through a tempfile
   PRInt32 numCopies = 1;
@@ -380,12 +527,25 @@ nsresult nsPrefService::WritePrefFile(nsIFile* aFile)
   rv = NS_NewLocalFileOutputStream(getter_AddRefs(outStream), aFile);
   if (NS_FAILED(rv)) 
       return rv;
-
+  
   // write out the file header
   rv = outStream->Write(outHeader, sizeof(outHeader) - 1, &writeAmount);
 
+  pref_saveArgs saveArgs;
+  saveArgs.prefArray = valueArray;
+  saveArgs.saveTypes = SAVE_ALL;
+  
+#if MOZ_PROFILESHARING
+  if (isSharingEnabled()) {
+    if (aFile == mCurrentSharedFile)
+      saveArgs.saveTypes = SAVE_SHARED;
+    else if (aFile == mCurrentFile)
+      saveArgs.saveTypes = SAVE_NONSHARED;
+  }
+#endif
+  
   // get the lines that we're supposed to be writing to the file
-  PL_DHashTableEnumerate(&gHashTable, pref_savePref, valueArray);
+  PL_DHashTableEnumerate(&gHashTable, pref_savePref, &saveArgs);
     
   /* Sort the preferences to make a readable file on disk */
   NS_QuickSort(valueArray, gHashTable.entryCount, sizeof(char *), pref_CompareStrings, NULL);
@@ -419,14 +579,26 @@ nsresult nsPrefService::WritePrefFile(nsIFile* aFile)
   return rv;
 }
 
-static nsresult openPrefFile(nsIFile* aFile, PRBool aIsErrorFatal,
-                             PRBool aIsGlobalContext, PRBool aSkipFirstLine)
+#ifdef MOZ_PROFILESHARING
+static PRBool isSharingEnabled()
+{
+  static PRBool gSharingEnabled = PR_FALSE;
+  
+  // If FALSE, query again. It may not have been set yet.
+  if (!gSharingEnabled) {
+    nsCOMPtr<nsIProfileSharingSetup> sharingSetup =
+        do_GetService("@mozilla.org/embedcomp/profile-sharing-setup;1");
+    if (sharingSetup)
+      sharingSetup->GetIsSharingEnabled(&gSharingEnabled);
+  }
+  return gSharingEnabled;
+}
+#endif
+
+static nsresult openPrefFile(nsIFile* aFile)
 {
   nsCOMPtr<nsIInputStream> inStr;
-  char *readBuf;
-  PRInt64 llFileSize;
-  PRUint32 fileSize;
-  nsresult rv;
+  char      readBuf[PREF_READ_BUFFER_SIZE];
 
 #if MOZ_TIMELINE
   {
@@ -436,54 +608,24 @@ static nsresult openPrefFile(nsIFile* aFile, PRBool aIsErrorFatal,
   }
 #endif
 
-  rv = aFile->GetFileSize(&llFileSize);
-  if (NS_FAILED(rv))
-    return rv;        
-  LL_L2UI(fileSize, llFileSize); // Converting 64 bit structure to unsigned int
-
-  // Now that we know the file exists, set this flag until we have
-  // successfully read and evaluated the prefs file. This will
-  // prevent us from writing an empty or partial prefs.js.
-  
-  gErrorOpeningUserPrefs = aIsErrorFatal;
-
-  rv = NS_NewLocalFileInputStream(getter_AddRefs(inStr), aFile);
+  nsresult rv = NS_NewLocalFileInputStream(getter_AddRefs(inStr), aFile);
   if (NS_FAILED(rv)) 
     return rv;        
 
-  readBuf = (char *)PR_Malloc(fileSize);
-  if (!readBuf) 
-    return NS_ERROR_OUT_OF_MEMORY;
+  PrefParseState ps;
+  PREF_InitParseState(&ps, PREF_ReaderCallback, NULL);
+  for (;;) {
+    PRUint32 amtRead = 0;
+    rv = inStr->Read(readBuf, sizeof(readBuf), &amtRead);
+    if (NS_FAILED(rv) || amtRead == 0)
+      break;
 
-  JS_BeginRequest(gMochaContext);
-
-  PRUint32 amtRead = 0;
-  rv = inStr->Read(readBuf, fileSize, &amtRead);
-  NS_ASSERTION((amtRead == fileSize), "failed to read the entire prefs file!!");
-  if (amtRead != fileSize)
-    return NS_ERROR_FAILURE;
- #ifdef XP_OS2 /* OS/2 workaround - our system editor adds an EOF character */
-     if (readBuf[amtRead - 1] == 0x1A) {
-        amtRead--;
-     }
- #endif
-  if (NS_SUCCEEDED(rv)) {
-    nsCAutoString leafName;
-    aFile->GetNativeLeafName(leafName);
-    if (PREF_EvaluateConfigScript(readBuf, amtRead, leafName.get(), aIsGlobalContext, PR_TRUE,
-                                   aSkipFirstLine))
-      gErrorOpeningUserPrefs = PR_FALSE;
-    else
-      rv = NS_ERROR_FAILURE;
+    PREF_ParseBuf(&ps, readBuf, amtRead); 
   }
-
-  PR_Free(readBuf);
-  JS_EndRequest(gMochaContext);
-
+  PREF_FinalizeParseState(&ps);
   return rv;        
 }
 
-
 /*
  * some stuff that gets called from Pref_Init()
  */
@@ -516,10 +658,10 @@ inplaceSortCallback(const void *data1, const void *data2, void *privateData)
 }
 
 //----------------------------------------------------------------------------------------
-JSBool pref_InitInitialObjects()
 // Initialize default preference JavaScript buffers from
 // appropriate TEXT resources
 //----------------------------------------------------------------------------------------
+static nsresult pref_InitInitialObjects()
 {
   nsCOMPtr<nsIFile> aFile;
   nsCOMPtr<nsIFile> defaultPrefDir;
@@ -549,8 +691,7 @@ JSBool pref_InitInitialObjects()
   };
 
   rv = NS_GetSpecialDirectory(NS_APP_PREF_DEFAULTS_50_DIR, getter_AddRefs(defaultPrefDir));
-  if (NS_FAILED(rv))
-    return JS_FALSE;
+  NS_ENSURE_SUCCESS(rv, rv);
 
   nsIFile **defaultPrefFiles = (nsIFile **)nsMemory::Alloc(INITIAL_MAX_DEFAULT_PREF_FILES * sizeof(nsIFile *));
   int maxDefaultPrefFiles = INITIAL_MAX_DEFAULT_PREF_FILES;
@@ -558,16 +699,16 @@ JSBool pref_InitInitialObjects()
 
   // Parse all the random files that happen to be in the components directory.
   nsCOMPtr<nsISimpleEnumerator> dirIterator;
-  rv = defaultPrefDir->GetDirectoryEntries(getter_AddRefs(dirIterator));
+  defaultPrefDir->GetDirectoryEntries(getter_AddRefs(dirIterator));
   if (!dirIterator) {
-    NS_ASSERTION(NS_SUCCEEDED(rv), "ERROR: Could not make a directory iterator.");
-    return JS_FALSE;
+    NS_ERROR("ERROR: Could not make a directory iterator.");
+    return NS_ERROR_FAILURE;
   }
 
   dirIterator->HasMoreElements(&hasMoreElements);
   if (!hasMoreElements) {
-    NS_ASSERTION(NS_SUCCEEDED(rv), "ERROR: Prefs directory is empty.");
-    return JS_FALSE;
+    NS_ERROR("ERROR: Prefs directory is empty.");
+    return NS_ERROR_FAILURE;
   }
 
   while (hasMoreElements) {
@@ -580,7 +721,7 @@ JSBool pref_InitInitialObjects()
     rv = aFile->GetNativeLeafName(leafName);
     if (NS_SUCCEEDED(rv)) {
       // Skip non-js files
-      if ((leafName.Length() < 3) || !Substring(leafName, leafName.Length() - 3, 3).Equals(NS_LITERAL_CSTRING(".js")))
+      if (!StringEndsWith(leafName, NS_LITERAL_CSTRING(".js")))
         shouldParse = PR_FALSE;
       // Skip files in the special list.
       if (shouldParse) {
@@ -600,13 +741,13 @@ JSBool pref_InitInitialObjects()
         }
       }
     }
-  };
+  }
 
   NS_QuickSort((void *)defaultPrefFiles, numFiles, sizeof(nsIFile *), inplaceSortCallback, nsnull);
 
   int k;
   for (k = 0; k < numFiles; k++) {
-    rv = openPrefFile(defaultPrefFiles[k], PR_FALSE, PR_FALSE, PR_FALSE);
+    rv = openPrefFile(defaultPrefFiles[k]);
     NS_ASSERTION(NS_SUCCEEDED(rv), "Config file not parsed successfully");
     NS_RELEASE(defaultPrefFiles[k]);
   }
@@ -620,35 +761,12 @@ JSBool pref_InitInitialObjects()
     if (NS_SUCCEEDED(rv)) {
       rv = aFile->AppendNative(nsDependentCString(specialFiles[k]));
       if (NS_SUCCEEDED(rv)) {
-        rv = openPrefFile(aFile, PR_FALSE, PR_FALSE, PR_FALSE);
+        rv = openPrefFile(aFile);
         NS_ASSERTION(NS_SUCCEEDED(rv), "<platform>.js was not parsed successfully");
       }
     }
   }
 
-  JS_MaybeGC(gMochaContext);
-  return JS_TRUE;
-}
-
-
-JSRuntime* PREF_GetJSRuntime()
-{
-  nsresult rv;
-
-  if (!gJSRuntimeService) {
-    rv = CallGetService("@mozilla.org/js/xpc/RuntimeService;1",
-                        &gJSRuntimeService);
-    if (NS_FAILED(rv)) {
-      NS_WARNING("nsJSRuntimeService is missing");
-      gJSRuntimeService = nsnull;
-      return nsnull;
-    }
-  }
-
-  JSRuntime* rt;
-  rv = gJSRuntimeService->GetRuntime(&rt);
-  if (NS_SUCCEEDED(rv))
-    return rt;
-  return nsnull;
+  return NS_OK;
 }
 
