@@ -166,12 +166,6 @@ map_jsj_thread_to_js_context_impl(JSJavaThreadState *jsj_env, void* java_applet_
 #endif
 }
 
-JS_STATIC_DLL_CALLBACK(void) detach_jsjava_thread_state(void* env)
-{
-    JSJavaThreadState *jsj_env = NS_REINTERPRET_CAST(JSJavaThreadState*, env);
-    JSJ_DetachCurrentThreadFromJava(jsj_env);
-}
-
 /*
 ** This callback is called to map a JSContext to a JSJavaThreadState which
 ** is a wrapper around JNIEnv. Hence this callback essentially maps a JSContext
@@ -358,7 +352,7 @@ get_JSPrincipals_from_java_caller_impl(JNIEnv *pJNIEnv, JSContext *pJSContext, v
                     rv = ssm->GetCodebasePrincipal(codebaseURI, getter_AddRefs(principal));
                     if (NS_SUCCEEDED(rv)) {
                         JSPrincipals* jsprincipals;
-                        principal->GetJSPrincipals(&jsprincipals);
+                        principal->GetJsPrincipals(&jsprincipals);
                         return jsprincipals;
                     }
                 }
@@ -368,7 +362,7 @@ get_JSPrincipals_from_java_caller_impl(JNIEnv *pJNIEnv, JSContext *pJSContext, v
         nsCOMPtr<nsIPrincipal> principal = do_QueryInterface(credentials);
         if (principal) {
             JSPrincipals* jsprincipals;
-            principal->GetJSPrincipals(&jsprincipals);
+            principal->GetJsPrincipals(&jsprincipals);
             return jsprincipals;
         }
     }
@@ -430,7 +424,10 @@ enter_js_from_java_impl(JNIEnv *jEnv, char **errp,
     nsCOMPtr<nsISecurityContext> javaSecurityContext = do_QueryInterface(credentials);
     if (javaSecurityContext) {
         if (pJSCX) {
-            nsCOMPtr<nsIScriptContext> scriptContext = NS_REINTERPRET_CAST(nsIScriptContext*, JS_GetContextPrivate(pJSCX));
+            nsCOMPtr<nsIScriptContext> scriptContext;
+            GetScriptContextFromJSContext(pJSCX,
+                                          getter_AddRefs(scriptContext));
+
             if (scriptContext) {
                 nsCOMPtr<nsIScriptGlobalObject> global;
                 scriptContext->GetGlobalObject(getter_AddRefs(global));
@@ -522,9 +519,8 @@ exit_js_impl(JNIEnv *jEnv, JSContext *cx)
     // The main idea is to execute terminate function if have any;
     if (cx)
     {
-        nsISupports* supports = NS_REINTERPRET_CAST(nsIScriptContext*, 
-                                                    JS_GetContextPrivate(cx));
-        nsCOMPtr<nsIScriptContext> scriptContext = do_QueryInterface(supports);
+        nsCOMPtr<nsIScriptContext> scriptContext;
+        GetScriptContextFromJSContext(cx, getter_AddRefs(scriptContext));
 
         if (scriptContext)
         {
