@@ -53,12 +53,7 @@
 
 // For GetOrigin()
 
-#include "nsCOMPtr.h"
-#include "nsJSPrincipals.h"
-#include "nsSystemPrincipal.h"
-#include "nsCodebasePrincipal.h"
-#include "nsCertificatePrincipal.h"
-#include "nsScriptSecurityManager.h"
+#include "nsIScriptSecurityManager.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIServiceManager.h"
 #include "nsIScriptObjectPrincipal.h"
@@ -75,8 +70,8 @@ static NS_DEFINE_IID(kISupportsIID, NS_ISUPPORTS_IID);
 
 // Thes macro expands to the aggregated query interface scheme.
 
-NS_IMPL_ADDREF(nsCSecurityContext);
-NS_IMPL_RELEASE(nsCSecurityContext);
+NS_IMPL_ADDREF(nsCSecurityContext)
+NS_IMPL_RELEASE(nsCSecurityContext)
 
 NS_METHOD
 nsCSecurityContext::QueryInterface(const nsIID& aIID, void** aInstancePtr)
@@ -120,81 +115,42 @@ nsCSecurityContext::Implies(const char* target, const char* action, PRBool *bAll
 NS_METHOD 
 nsCSecurityContext::GetOrigin(char* buf, int buflen)
 {
-    // Get the Script Security Manager.
-
-    nsresult rv      = NS_OK;
-    nsCOMPtr<nsIScriptSecurityManager> secMan = 
-             do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
-    if (NS_FAILED(rv) || !secMan) return NS_ERROR_FAILURE;
-
-
-    // First, try to get the principal from the security manager.
-    // Next, try to get it from the dom.
-    // If neither of those work, the qi for codebase will fail.
-
     if (!m_pPrincipal) {
-        if (NS_FAILED(secMan->GetSubjectPrincipal(&m_pPrincipal))) 
-        // return NS_ERROR_FAILURE;
-        ; // Don't return here because the security manager returns 
-          // NS_ERROR_FAILURE when there is no subject principal. In
-          // that case we are not done.
-        
-        if (!m_pPrincipal && m_pJSCX ) {
-            nsCOMPtr<nsIScriptContext> scriptContext;
-            GetScriptContextFromJSContext(m_pJSCX,
-                                          getter_AddRefs(scriptContext));
-
-            if (scriptContext) {
-                nsCOMPtr<nsIScriptGlobalObject> global;
-                scriptContext->GetGlobalObject(getter_AddRefs(global));
-                NS_ASSERTION(global, "script context has no global object");
-
-                if (global) {
-                    nsCOMPtr<nsIScriptObjectPrincipal> globalData = do_QueryInterface(global);
-                    if (globalData) {
-                        // ISSUE: proper ref counting.
-                        if (NS_FAILED(globalData->GetPrincipal(&m_pPrincipal)))
-                            return NS_ERROR_FAILURE; 
-                       
-                   }
-                }
-            }
-        }
-    }
-
-    nsCOMPtr<nsICodebasePrincipal> codebase = do_QueryInterface(m_pPrincipal);
-    if (!codebase) 
-        return NS_ERROR_FAILURE;
-
-    char* origin=nsnull;
-    codebase->GetOrigin(&origin);
-
-    if (origin) {
-        PRInt32 originlen = (PRInt32) strlen(origin);
-        if (!buf || buflen<=originlen) {
-            if (origin) {
-                nsCRT::free(origin);
-            }
+        // Get the Script Security Manager.
+        nsresult rv = NS_OK;
+        nsCOMPtr<nsIScriptSecurityManager> secMan =
+             do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
+        if (NS_FAILED(rv) || !secMan) {
             return NS_ERROR_FAILURE;
         }
 
-        // Copy the string into to user supplied buffer. Is there a better
-        // way to do this?
-
-        memcpy(buf,origin,originlen);
-        buf[originlen]=nsnull; // Gotta terminate it.
-        nsCRT::free(origin);
-    } else {
-        *buf = nsnull;
+        secMan->GetSubjectPrincipal(getter_AddRefs(m_pPrincipal));
+        if (!m_pPrincipal) {
+            return NS_ERROR_FAILURE;
+        }
     }
 
+    nsXPIDLCString origin;
+    m_pPrincipal->GetOrigin(getter_Copies(origin));
+
+    PRInt32 originlen = origin.Length();
+    if (origin.IsEmpty() || originlen > buflen - 1) {
+        return NS_ERROR_FAILURE;
+    }
+
+    // Copy the string into to user supplied buffer. Is there a better
+    // way to do this?
+
+    memcpy(buf, origin, originlen);
+    buf[originlen] = nsnull; // Gotta terminate it.
+
     return NS_OK;
 }
 
 NS_METHOD 
 nsCSecurityContext::GetCertificateID(char* buf, int buflen)
 {
-    nsCOMPtr<nsIPrincipal> principal = NULL;
+    nsCOMPtr<nsIPrincipal> principal;
   
     // Get the Script Security Manager.
 
@@ -203,28 +159,22 @@ nsCSecurityContext::GetCertificateID(char* buf, int buflen)
              do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
     if (NS_FAILED(rv) || !secMan) return NS_ERROR_FAILURE;
 
-
     secMan->GetSubjectPrincipal(getter_AddRefs(principal));
-    nsCOMPtr<nsICertificatePrincipal> cprincipal = do_QueryInterface(principal);
-    if (!cprincipal) 
+    if (!principal) {
         return NS_ERROR_FAILURE;
-
-    char* certificate = nsnull;
-    cprincipal->GetCertificateID(&certificate);
-
-    if (certificate) {
-        PRInt32 certlen = (PRInt32) strlen(certificate);
-        if( buflen<=certlen ) {
-            nsCRT::free(certificate);
-            return NS_ERROR_FAILURE;
-        }
-        memcpy(buf,certificate,certlen);
-        buf[certlen]=nsnull;
-        nsCRT::free(certificate);
-    } else {
-        *buf = nsnull;
     }
 
+    nsXPIDLCString certificate;
+    principal->GetCertificateID(getter_Copies(certificate));
+
+    PRInt32 certlen = certificate.Length();
+    if (buflen <= certlen) {
+        return NS_ERROR_FAILURE;
+    }
+
+    memcpy(buf, certificate.get(), certlen);
+    buf[certlen] = nsnull;
+
     return NS_OK;
 }
 
@@ -298,9 +248,7 @@ nsCSecurityContext::nsCSecurityContext(nsIPrincipal *principal)
 
     // Do early evaluation of "UniversalJavaPermission" capability.
 
-    PRBool equals;
-    if (!m_pPrincipal || 
-        NS_SUCCEEDED(m_pPrincipal->Equals(sysprincipal, &equals)) && equals) {
+    if (!m_pPrincipal || m_pPrincipal == sysprincipal) {
         // We have native code or the system principal: just allow general access
         m_HasUniversalBrowserReadCapability = PR_TRUE;
         m_HasUniversalJavaCapability = PR_TRUE;
