@@ -88,10 +88,6 @@ enum eNPPStreamTypeInternal {
 // CID's && IID's
 static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID);
 static NS_DEFINE_IID(kCPluginManagerCID, NS_PLUGINMANAGER_CID);
-static NS_DEFINE_IID(kIPluginIID, NS_IPLUGIN_IID); 
-static NS_DEFINE_IID(kIFactoryIID, NS_IFACTORY_IID);
-static NS_DEFINE_IID(kISupportsIID, NS_ISUPPORTS_IID);
-static NS_DEFINE_IID(kIWindowlessPluginInstancePeerIID, NS_IWINDOWLESSPLUGININSTANCEPEER_IID);
 static NS_DEFINE_IID(kPluginManagerCID, NS_PLUGINMANAGER_CID);
 static NS_DEFINE_IID(kIPluginManagerIID, NS_IPLUGINMANAGER_IID); 
 static NS_DEFINE_IID(kMemoryCID, NS_MEMORY_CID);
@@ -304,7 +300,7 @@ ns4xPlugin::ns4xPlugin(NPPluginFuncs* callbacks, PRLibrary* aLibrary, NP_PLUGINS
   gServiceMgr = serviceMgr;
   fLibrary = nsnull;
 
-#if defined(XP_WIN) || defined(XP_OS2)
+#if defined(XP_WIN)
   // On Windows (and Mac) we need to keep a direct reference to the fCallbacks and NOT
   // just copy the struct. See Bugzilla 85334
 
@@ -545,8 +541,27 @@ ns4xPlugin::CreatePlugin(nsIServiceManagerObsolete* aServiceMgr,
 #endif
 
 #ifdef XP_OS2
+  // XXX Do we need to do this on OS/2 or can we look more like Windows?
+  NP_GETENTRYPOINTS pfnGetEntryPoints = (NP_GETENTRYPOINTS)PR_FindSymbol(aLibrary, "NP_GetEntryPoints");
+
+  if (pfnGetEntryPoints == NULL)
+    return NS_ERROR_FAILURE;
+
+  NPPluginFuncs callbacks;
+  memset((void*) &callbacks, 0, sizeof(callbacks));
+
+  callbacks.size = sizeof(callbacks);
+
+  if (pfnGetEntryPoints(&callbacks) != NS_OK)
+    return NS_ERROR_FAILURE; // XXX
+
+  if (HIBYTE(callbacks.version) < NP_VERSION_MAJOR)
+    return NS_ERROR_FAILURE;
+
+  NP_PLUGINSHUTDOWN pfnShutdown = (NP_PLUGINSHUTDOWN)PR_FindSymbol(aLibrary, "NP_Shutdown");
+
   // create the new plugin handler
-  *aResult = new ns4xPlugin(nsnull, aLibrary, nsnull, aServiceMgr);
+  *aResult = new ns4xPlugin(&callbacks, aLibrary, pfnShutdown, aServiceMgr);
 
   if (*aResult == NULL)
     return NS_ERROR_OUT_OF_MEMORY;
@@ -1210,6 +1225,17 @@ _getvalue(NPP npp, NPNVariable variable, void *result)
   switch(variable) {
 #if defined(XP_UNIX) && !defined(XP_MACOSX)
   case NPNVxDisplay : {
+#ifdef MOZ_WIDGET_GTK2
+    if(npp) {
+      ns4xPluginInstance *inst = (ns4xPluginInstance *) npp->ndata;
+      NPBool rtv = PR_FALSE;
+      inst->GetValue((nsPluginInstanceVariable)NPPVpluginNeedsXEmbed, &rtv);
+      if(rtv) {
+        (*(Display **)result) = GDK_DISPLAY();
+        return NPERR_NO_ERROR;
+      }
+    }
+#endif
 #if defined(MOZ_WIDGET_GTK) || defined(MOZ_WIDGET_GTK2)
     // adobe nppdf calls XtGetApplicationNameAndClass(display, &instance, &class)
     // we have to init Xt toolkit before get XtDisplay
@@ -1310,9 +1336,7 @@ _getvalue(NPP npp, NPNVariable variable, void *result)
         nsCOMPtr<nsIDocument> doc;
         owner->GetDocument(getter_AddRefs(doc));
         if (doc) {
-          nsCOMPtr<nsIScriptGlobalObject> globalScript;
-          doc->GetScriptGlobalObject(getter_AddRefs(globalScript));
-          nsCOMPtr<nsIDOMWindow> domWindow (do_QueryInterface(globalScript));
+          nsCOMPtr<nsIDOMWindow> domWindow (do_QueryInterface(doc->GetScriptGlobalObject()));
           if (domWindow) {
             NS_ADDREF(*(nsIDOMWindow**)result = domWindow.get());
             return NPERR_NO_ERROR;
@@ -1322,6 +1346,30 @@ _getvalue(NPP npp, NPNVariable variable, void *result)
     }
     return NPERR_GENERIC_ERROR;
   }
+
+  case NPNVToolkit: {
+#ifdef MOZ_WIDGET_GTK
+    *((NPNToolkitType*)result) = NPNVGtk12;
+#endif
+
+#ifdef MOZ_WIDGET_GTK2
+    *((NPNToolkitType*)result) = NPNVGtk2;
+#endif
+
+    if (result)
+        return NPERR_NO_ERROR;
+
+    return NPERR_GENERIC_ERROR;
+  }
+
+  case NPNVSupportsXEmbedBool: {
+#ifdef MOZ_WIDGET_GTK2
+    *(NPBool*)result = PR_TRUE; 
+#else
+    *(NPBool*)result = PR_FALSE; 
+#endif
+    return NPERR_NO_ERROR;
+  }
   default : return NPERR_GENERIC_ERROR;
   }
 }
