@@ -49,6 +49,10 @@
 #include "nsIPref.h" // needed for NS_TRY_SAFE_CALL_*
 #include "nsPluginLogging.h"
 
+#ifdef XP_OS2
+#include "nsILegacyPluginWrapperOS2.h"
+#endif
+
 #ifdef MOZ_WIDGET_GTK
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
@@ -69,12 +73,8 @@
 
 ////////////////////////////////////////////////////////////////////////
 // CID's && IID's
-static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID); // needed for NS_TRY_SAFE_CALL_*
-static NS_DEFINE_IID(kCPluginManagerCID, NS_PLUGINMANAGER_CID);
+static NS_DEFINE_IID(kCPluginManagerCID, NS_PLUGINMANAGER_CID); // needed for NS_TRY_SAFE_CALL
 static NS_DEFINE_IID(kIPluginStreamListenerIID, NS_IPLUGINSTREAMLISTENER_IID);
-static NS_DEFINE_IID(kIPluginInstanceIID, NS_IPLUGININSTANCE_IID); 
-static NS_DEFINE_IID(kIPluginTagInfoIID, NS_IPLUGINTAGINFO_IID); 
-static NS_DEFINE_IID(kISupportsIID, NS_ISUPPORTS_IID);
 
 ///////////////////////////////////////////////////////////////////////////////
 // ns4xPluginStreamListener Methods
@@ -864,6 +864,7 @@ NS_IMETHODIMP ns4xPluginInstance::SetWindow(nsPluginWindow* window)
   NPError error;
   
 #if defined (MOZ_WIDGET_GTK) || defined (MOZ_WIDGET_GTK2)
+  PRBool isXembed = PR_FALSE;
   // bug 108337, flash plugin on linux doesn't like window->width <= 0
   if ((PRInt32) window->width <= 0 || (PRInt32) window->height <= 0)
     return NS_OK;
@@ -885,6 +886,15 @@ NS_IMETHODIMP ns4xPluginInstance::SetWindow(nsPluginWindow* window)
     GdkWindow *win = gdk_window_lookup((XID)window->window);
     if (!win)
       return NS_ERROR_FAILURE;
+    gpointer user_data = nsnull;
+    gdk_window_get_user_data(win, &user_data);
+    if (user_data) {
+      GtkWidget* widget = GTK_WIDGET(user_data);
+      if (GTK_IS_SOCKET(widget)) {
+	isXembed = PR_TRUE;
+      }
+    }
+    if (!isXembed)
     {  
 #ifdef NS_DEBUG      
       printf("About to create new xtbin of %i X %i from %p...\n",
@@ -933,7 +943,10 @@ NS_IMETHODIMP ns4xPluginInstance::SetWindow(nsPluginWindow* window)
     ws->type = 0; // OK, that was a guess!!
 #ifdef MOZ_X11
     ws->depth = gdk_window_get_visual(win)->depth;
-    ws->display = GTK_XTBIN(mXtBin)->xtdisplay;
+    if (!isXembed)
+      ws->display = GTK_XTBIN(mXtBin)->xtdisplay;
+    else
+      ws->display = GDK_WINDOW_XDISPLAY(win);
     ws->visual = GDK_VISUAL_XVISUAL(gdk_window_get_visual(win));
     ws->colormap = GDK_COLORMAP_XCOLORMAP(gdk_window_get_colormap(win));
 
@@ -941,14 +954,16 @@ NS_IMETHODIMP ns4xPluginInstance::SetWindow(nsPluginWindow* window)
 #endif
   } // !window->ws_info
 
-  if (!mXtBin)
+  if (!mXtBin && !isXembed)
     return NS_ERROR_FAILURE;
 
-  // And now point the NPWindow structures window 
-  // to the actual X window
-  window->window = (nsPluginPort *)GTK_XTBIN(mXtBin)->xtwindow;
-  
-  gtk_xtbin_resize(mXtBin, window->width, window->height);
+  if (!isXembed) {
+    // And now point the NPWindow structures window 
+    // to the actual X window
+    window->window = (nsPluginPort *)GTK_XTBIN(mXtBin)->xtwindow;
+    
+    gtk_xtbin_resize(mXtBin, window->width, window->height);
+  }
   
 #elif defined(MOZ_WIDGET_XLIB)
 
@@ -1176,6 +1191,24 @@ nsresult ns4xPluginInstance::GetValueInternal(NPPVariable variable, void* value)
     NPP_PLUGIN_LOG(PLUGIN_LOG_NORMAL,
     ("NPP GetValue called: this=%p, npp=%p, var=%d, value=%d, return=%d\n", 
     this, &fNPP, variable, value, res));
+
+#ifdef XP_OS2
+    /* Query interface for legacy Flash plugin */
+    if (res == NS_OK && variable == NPPVpluginScriptableInstance)
+    {
+      nsCOMPtr<nsILegacyPluginWrapperOS2> wrapper =
+               do_GetService(NS_LEGACY_PLUGIN_WRAPPER_CONTRACTID, &res);
+      if (res == NS_OK)
+      {
+        nsIID *iid = nsnull; 
+        res = CallNPP_GetValueProc(fCallbacks->getvalue, &fNPP, 
+                                   NPPVpluginScriptableIID, (void *)&iid);
+        if (res == NS_OK)
+          res = wrapper->MaybeWrap(*iid, *(nsISupports**)value,
+                                   (nsISupports**)value);
+      }
+    }
+#endif
   }
 
   return res;
