@@ -128,9 +128,9 @@ private:
                     const char *challenge,
                     const nsHttpAuthIdentity &ident,
                     nsISupports *metadata)
-        : mRealm(nsnull)
-        , mRoot(nsnull)
+        : mRoot(nsnull)
         , mTail(nsnull)
+        , mRealm(nsnull)
     {
         Set(path, realm, creds, challenge, ident, metadata);
     }
@@ -208,8 +208,7 @@ public:
     // |host| and |port| are required
     // |path| can be null
     // |entry| is either null or a weak reference
-    nsresult GetAuthEntryForPath(const char *scheme,
-                                 const char *host,
+    nsresult GetAuthEntryForPath(const char *host,
                                  PRInt32     port,
                                  const char *path,
                                  nsHttpAuthEntry **entry);
@@ -217,8 +216,7 @@ public:
     // |host| and |port| are required
     // |realm| must not be null
     // |entry| is either null or a weak reference
-    nsresult GetAuthEntryForDomain(const char *scheme,
-                                   const char *host,
+    nsresult GetAuthEntryForDomain(const char *host,
                                    PRInt32     port,
                                    const char *realm,
                                    nsHttpAuthEntry **entry);
@@ -228,8 +226,7 @@ public:
     // |realm| must not be null
     // if |credentials|, |user|, |pass|, and |challenge| are each
     // null, then the entry is deleted.
-    nsresult SetAuthEntry(const char *scheme,
-                          const char *host,
+    nsresult SetAuthEntry(const char *host,
                           PRInt32     port,
                           const char *directory,
                           const char *realm,
@@ -238,8 +235,7 @@ public:
                           const nsHttpAuthIdentity &ident,
                           nsISupports *metadata);
 
-    void ClearAuthEntry(const char *scheme,
-                        const char *host,
+    void ClearAuthEntry(const char *host,
                         PRInt32     port,
                         const char *realm);
 
@@ -247,10 +243,7 @@ public:
     nsresult ClearAll();
 
 private:
-    nsHttpAuthNode *LookupAuthNode(const char *scheme,
-                                   const char *host,
-                                   PRInt32     port,
-                                   nsCString  &key);
+    nsHttpAuthNode *LookupAuthNode(const char *host, PRInt32 port, nsCString &key);
 
     // hash table allocation functions
     static void*        PR_CALLBACK AllocTable(void *, PRSize size);
