@@ -36,14 +36,11 @@
 #include "nsHttpAuthCache.h"
 #include "nsStandardURL.h"
 #include "nsIHttpChannel.h"
-#include "nsIHttpNotify.h"
 #include "nsIURL.h"
 #include "nsIStandardURL.h"
 #include "nsICacheService.h"
 #include "nsICategoryManager.h"
 #include "nsCategoryManagerUtils.h"
-#include "nsIObserverService.h"
-#include "nsINetModRegEntry.h"
 #include "nsICacheService.h"
 #include "nsIPrefService.h"
 #include "nsIPrefBranchInternal.h"
@@ -75,9 +72,8 @@ extern PRThread *gSocketThread;
 #endif
 
 static NS_DEFINE_CID(kIOServiceCID, NS_IOSERVICE_CID);
-static NS_DEFINE_CID(kStandardURLCID, NS_STANDARDURL_CID);
-static NS_DEFINE_CID(kNetModuleMgrCID, NS_NETMODULEMGR_CID);
 static NS_DEFINE_CID(kStreamConverterServiceCID, NS_STREAMCONVERTERSERVICE_CID);
+static NS_DEFINE_CID(kCookieServiceCID, NS_COOKIESERVICE_CID);
 static NS_DEFINE_CID(kCacheServiceCID, NS_CACHESERVICE_CID);
 static NS_DEFINE_CID(kEventQueueServiceCID, NS_EVENTQUEUESERVICE_CID);
 static NS_DEFINE_CID(kSocketProviderServiceCID, NS_SOCKETPROVIDERSERVICE_CID);
@@ -91,9 +87,11 @@ static NS_DEFINE_CID(kSocketProviderServiceCID, NS_SOCKETPROVIDERSERVICE_CID);
 #define INTL_ACCEPT_LANGUAGES   "intl.accept_languages"
 #define INTL_ACCEPT_CHARSET     "intl.charset.default"
 #define NETWORK_ENABLEIDN       "network.enableIDN"
+#define BROWSER_PREF_PREFIX     "browser.cache."
 
 #define UA_PREF(_pref) UA_PREF_PREFIX _pref
 #define HTTP_PREF(_pref) HTTP_PREF_PREFIX _pref
+#define BROWSER_PREF(_pref) BROWSER_PREF_PREFIX _pref
 
 //-----------------------------------------------------------------------------
 
@@ -147,6 +145,7 @@ nsHttpHandler::nsHttpHandler()
     , mUserAgentIsDirty(PR_TRUE)
     , mUseCache(PR_TRUE)
     , mSendSecureXSiteReferrer(PR_TRUE)
+    , mEnablePersistentHttpsCaching(PR_FALSE)
 {
 #if defined(PR_LOGGING)
     gHttpLog = PR_NewLogModule("nsHttp");
@@ -202,10 +201,13 @@ nsHttpHandler::Init()
             pbi->AddObserver(INTL_ACCEPT_LANGUAGES, this, PR_TRUE); 
             pbi->AddObserver(INTL_ACCEPT_CHARSET, this, PR_TRUE);
             pbi->AddObserver(NETWORK_ENABLEIDN, this, PR_TRUE);
+            pbi->AddObserver(BROWSER_PREF("disk_cache_ssl"), this, PR_TRUE);
         }
         PrefsChanged(prefBranch, nsnull);
     }
 
+    mMisc = NS_LITERAL_CSTRING("rv:" MOZILLA_VERSION);
+
 #if DEBUG
     // dump user agent prefs
     LOG(("> app-name = %s\n", mAppName.get()));
@@ -238,13 +240,12 @@ nsHttpHandler::Init()
                                   NS_STATIC_CAST(nsISupports*,NS_STATIC_CAST(void*,this)),
                                   NS_HTTP_STARTUP_TOPIC);    
     
-    nsCOMPtr<nsIObserverService> observerSvc =
-        do_GetService("@mozilla.org/observer-service;1", &rv);
-    if (observerSvc) {
-        observerSvc->AddObserver(this, "profile-change-net-teardown", PR_TRUE);
-        observerSvc->AddObserver(this, "profile-change-net-restore", PR_TRUE);
-        observerSvc->AddObserver(this, "session-logout", PR_TRUE);
-        observerSvc->AddObserver(this, NS_XPCOM_SHUTDOWN_OBSERVER_ID, PR_TRUE);
+    mObserverService = do_GetService("@mozilla.org/observer-service;1");
+    if (mObserverService) {
+        mObserverService->AddObserver(this, "profile-change-net-teardown", PR_TRUE);
+        mObserverService->AddObserver(this, "profile-change-net-restore", PR_TRUE);
+        mObserverService->AddObserver(this, "session-logout", PR_TRUE);
+        mObserverService->AddObserver(this, NS_XPCOM_SHUTDOWN_OBSERVER_ID, PR_TRUE);
     }
  
     StartPruneDeadConnectionsTimer();
@@ -415,19 +416,6 @@ nsHttpHandler::GetCacheSession(nsCacheStoragePolicy storagePolicy,
     return NS_OK;
 }
 
-nsresult
-nsHttpHandler::GetProxyObjectManager(nsIProxyObjectManager **result)
-{
-    if (!mProxyMgr) {
-        nsresult rv;
-        mProxyMgr = do_GetService(NS_XPCOMPROXY_CONTRACTID, &rv);
-        if (NS_FAILED(rv)) return rv;
-    }
-    *result = mProxyMgr;
-    NS_ADDREF(*result);
-    return NS_OK;
-}
-
 nsresult
 nsHttpHandler::GetEventQueueService(nsIEventQueueService **result)
 {
@@ -454,6 +442,14 @@ nsHttpHandler::GetStreamConverterService(nsIStreamConverterService **result)
     return NS_OK;
 }
 
+nsICookieService *
+nsHttpHandler::GetCookieService()
+{
+    if (!mCookieService)
+        mCookieService = do_GetService(kCookieServiceCID);
+    return mCookieService;
+}
+
 nsresult
 nsHttpHandler::GetMimeService(nsIMIMEService **result)
 {
@@ -478,80 +474,18 @@ nsHttpHandler::GetIOService(nsIIOService** result)
 nsresult
 nsHttpHandler::OnModifyRequest(nsIHttpChannel *chan)
 {
-    nsresult rv;
-
     LOG(("nsHttpHandler::OnModifyRequest [chan=%x]\n", chan));
-
-    if (!mNetModuleMgr) {
-        mNetModuleMgr = do_GetService(kNetModuleMgrCID, &rv);
-        if (NS_FAILED(rv)) return rv;
-    }
-
-    nsCOMPtr<nsISimpleEnumerator> modules;
-    rv = mNetModuleMgr->EnumerateModules(
-            NS_NETWORK_MODULE_MANAGER_HTTP_REQUEST_CONTRACTID,
-            getter_AddRefs(modules));
-    if (NS_FAILED(rv)) return rv;
-
-    nsCOMPtr<nsISupports> sup;
-
-    // notify each module...
-    while (NS_SUCCEEDED(modules->GetNext(getter_AddRefs(sup)))) {
-        nsCOMPtr<nsINetModRegEntry> entry = do_QueryInterface(sup, &rv);
-        if (NS_FAILED(rv)) return rv;
-
-        nsCOMPtr<nsINetNotify> netNotify;
-        rv = entry->GetSyncProxy(getter_AddRefs(netNotify));
-        if (NS_FAILED(rv)) return rv;
-
-        nsCOMPtr<nsIHttpNotify> httpNotify = do_QueryInterface(netNotify, &rv);
-        if (NS_FAILED(rv)) return rv;
-
-        // fire off the notification, ignore the return code.
-        httpNotify->OnModifyRequest(chan);
-    }
-    
+    if (mObserverService)
+        mObserverService->NotifyObservers(chan, NS_HTTP_ON_MODIFY_REQUEST_TOPIC, nsnull);
     return NS_OK;
 }
 
 nsresult
 nsHttpHandler::OnExamineResponse(nsIHttpChannel *chan)
 {
-    nsresult rv;
-
     LOG(("nsHttpHandler::OnExamineResponse [chan=%x]\n", chan));
-
-    if (!mNetModuleMgr) {
-        mNetModuleMgr = do_GetService(kNetModuleMgrCID, &rv);
-        if (NS_FAILED(rv)) return rv;
-    }
-
-    nsCOMPtr<nsISimpleEnumerator> modules;
-    rv = mNetModuleMgr->EnumerateModules(
-            NS_NETWORK_MODULE_MANAGER_HTTP_RESPONSE_CONTRACTID,
-            getter_AddRefs(modules));
-    if (NS_FAILED(rv)) return rv;
-
-    nsCOMPtr<nsISupports> sup;
-    nsCOMPtr<nsINetModRegEntry> entry;
-    nsCOMPtr<nsINetNotify> netNotify;
-    nsCOMPtr<nsIHttpNotify> httpNotify;
-
-    // notify each module...
-    while (NS_SUCCEEDED(modules->GetNext(getter_AddRefs(sup)))) {
-        entry = do_QueryInterface(sup, &rv);
-        if (NS_FAILED(rv)) return rv;
-
-        rv = entry->GetSyncProxy(getter_AddRefs(netNotify));
-        if (NS_FAILED(rv)) return rv;
-
-        httpNotify = do_QueryInterface(netNotify, &rv);
-        if (NS_FAILED(rv)) return rv;
-
-        // fire off the notification, ignore the return code.
-        httpNotify->OnExamineResponse(chan);
-    }
-    
+    if (mObserverService)
+        mObserverService->NotifyObservers(chan, NS_HTTP_ON_EXAMINE_RESPONSE_TOPIC, nsnull);
     return NS_OK;
 }
 
@@ -662,10 +596,6 @@ void
 nsHttpHandler::InitUserAgentComponents()
 {
 
-    // Gather Application name and Version.
-    mAppName.Adopt(nsCRT::strdup(UA_APPNAME));
-    mAppVersion.Adopt(nsCRT::strdup(UA_APPVERSION));
-
       // Gather platform.
     mPlatform.Adopt(nsCRT::strdup(
 #if defined(MOZ_WIDGET_PHOTON)
@@ -772,6 +702,22 @@ nsHttpHandler::PrefsChanged(nsIPrefBranch *prefs, const char *pref)
     // UA components
     //
 
+    // Gather application values.
+    if (PREF_CHANGED(UA_PREF("appName"))) {
+        prefs->GetCharPref(UA_PREF("appName"),
+            getter_Copies(mAppName));
+        if (mAppName.IsEmpty())
+            mAppName.Adopt(nsCRT::strdup(UA_APPNAME));
+        mUserAgentIsDirty = PR_TRUE;
+    }
+    if (PREF_CHANGED(UA_PREF("appVersion"))) {
+        prefs->GetCharPref(UA_PREF("appVersion"),
+            getter_Copies(mAppVersion));
+        if (mAppVersion.IsEmpty())
+            mAppVersion.Adopt(nsCRT::strdup(UA_APPVERSION));
+        mUserAgentIsDirty = PR_TRUE;
+    }
+
     // Gather vendor values.
     if (PREF_CHANGED(UA_PREF("vendor"))) {
         prefs->GetCharPref(UA_PREF("vendor"),
@@ -806,12 +752,6 @@ nsHttpHandler::PrefsChanged(nsIPrefBranch *prefs, const char *pref)
         mUserAgentIsDirty = PR_TRUE;
     }
 
-    // Gather misc value.
-    if (PREF_CHANGED(UA_PREF("misc"))) {
-        prefs->GetCharPref(UA_PREF("misc"), getter_Copies(mMisc));
-        mUserAgentIsDirty = PR_TRUE;
-    }
-
     // Get Security level supported
     if (PREF_CHANGED(UA_PREF("security"))) {
         prefs->GetCharPref(UA_PREF("security"), getter_Copies(mSecurity));
@@ -1016,11 +956,11 @@ nsHttpHandler::PrefsChanged(nsIPrefBranch *prefs, const char *pref)
     }
 
     if (PREF_CHANGED(HTTP_PREF("default-socket-type"))) {
-        nsXPIDLCString val;
+        nsXPIDLCString sval;
         rv = prefs->GetCharPref(HTTP_PREF("default-socket-type"),
-                                getter_Copies(val));
+                                getter_Copies(sval));
         if (NS_SUCCEEDED(rv)) {
-            if (val.IsEmpty())
+            if (sval.IsEmpty())
                 mDefaultSocketType.Adopt(0);
             else {
                 // verify that this socket type is actually valid
@@ -1028,16 +968,24 @@ nsHttpHandler::PrefsChanged(nsIPrefBranch *prefs, const char *pref)
                         do_GetService(kSocketProviderServiceCID, &rv));
                 if (NS_SUCCEEDED(rv)) {
                     nsCOMPtr<nsISocketProvider> sp;
-                    rv = sps->GetSocketProvider(val, getter_AddRefs(sp));
+                    rv = sps->GetSocketProvider(sval, getter_AddRefs(sp));
                     if (NS_SUCCEEDED(rv)) {
                         // OK, this looks like a valid socket provider.
-                        mDefaultSocketType.Assign(val);
+                        mDefaultSocketType.Assign(sval);
                     }
                 }
             }
         }
     }
 
+    // enable Persistent caching for HTTPS - bug#205921    
+    if (PREF_CHANGED(BROWSER_PREF("disk_cache_ssl"))) {
+        cVar = PR_FALSE;
+        rv = prefs->GetBoolPref(BROWSER_PREF("disk_cache_ssl"), &cVar);
+        if (NS_SUCCEEDED(rv))
+            mEnablePersistentHttpsCaching = cVar;
+    }
+
     //
     // INTL options
     //
@@ -1145,9 +1093,9 @@ PrepareAcceptLanguages(const char *i_AcceptLanguages, nsACString &o_AcceptLangua
          token != (char *) 0;
          token = nsCRT::strtok(p, ",", &p))
     {
-        while (*token == ' ' || *token == '\x9') token++;
+        token = net_FindCharNotInSet(token, HTTP_LWS);
         char* trim;
-        trim = PL_strpbrk(token, "; \x9");
+        trim = net_FindCharInSet(token, ";" HTTP_LWS);
         if (trim != (char*)0)  // remove "; q=..." if present
             *trim = '\0';
 
@@ -1243,9 +1191,9 @@ PrepareAcceptCharsets(const char *i_AcceptCharset, nsACString &o_AcceptCharset)
     for (token = nsCRT::strtok(o_Accept, ",", &p);
          token != (char *) 0;
          token = nsCRT::strtok(p, ",", &p)) {
-        while (*token == ' ' || *token == '\x9') token++;
+        token = net_FindCharNotInSet(token, HTTP_LWS);
         char* trim;
-        trim = PL_strpbrk(token, "; \x9");
+        trim = net_FindCharInSet(token, ";" HTTP_LWS);
         if (trim != (char*)0)  // remove "; q=..." if present
             *trim = '\0';
 
@@ -1619,7 +1567,7 @@ nsHttpHandler::Observe(nsISupports *subject,
                        const char *topic,
                        const PRUnichar *data)
 {
-    LOG(("nsHttpHandler::Observe [topic=\"%s\")]\n", topic));
+    LOG(("nsHttpHandler::Observe [topic=\"%s\"]\n", topic));
 
     if (strcmp(topic, NS_PREFBRANCH_PREFCHANGE_TOPIC_ID) == 0) {
         nsCOMPtr<nsIPrefBranch> prefBranch = do_QueryInterface(subject);
