@@ -51,14 +51,14 @@ class nsHttpChunkedDecoder;
 //-----------------------------------------------------------------------------
 
 class nsHttpTransaction : public nsAHttpTransaction
-                        , public nsIOutputStreamNotify
-                        , public nsISocketEventHandler
+                        , public nsIInputStreamCallback
+                        , public nsIOutputStreamCallback
 {
 public:
     NS_DECL_ISUPPORTS
     NS_DECL_NSAHTTPTRANSACTION
-    NS_DECL_NSIOUTPUTSTREAMNOTIFY
-    NS_DECL_NSISOCKETEVENTHANDLER
+    NS_DECL_NSIINPUTSTREAMCALLBACK
+    NS_DECL_NSIOUTPUTSTREAMCALLBACK
 
     nsHttpTransaction();
     virtual ~nsHttpTransaction();
@@ -90,7 +90,7 @@ public:
                   nsHttpRequestHead     *reqHeaders,
                   nsIInputStream        *reqBody,
                   PRBool                 reqBodyIncludesHeaders,
-                  nsIEventQueue         *eventQ,
+                  nsIEventQueue         *consumerEventQ,
                   nsIInterfaceRequestor *callbacks,
                   nsITransportEventSink *eventsink,
                   nsIAsyncInputStream  **responseBody);
@@ -101,7 +101,6 @@ public:
     nsHttpRequestHead     *RequestHead()    { return mRequestHead; }
     nsHttpResponseHead    *ResponseHead()   { return mHaveAllHeaders ? mResponseHead : nsnull; }
     nsISupports           *SecurityInfo()   { return mSecurityInfo; }
-    nsresult               TransportStatus(){ return mTransportStatus; }
 
     nsIInterfaceRequestor *Callbacks()      { return mCallbacks; } 
     nsIEventQueue         *ConsumerEventQ() { return mConsumerEventQ; }
@@ -114,9 +113,6 @@ public:
     // Called to find out if the transaction generated a complete response.
     PRBool ResponseIsComplete() { return mResponseIsComplete; }
 
-    void   SetSSLConnectFailed() { mSSLConnectFailed = PR_TRUE; }
-    PRBool    SSLConnectFailed() { return mSSLConnectFailed; }
-
 private:
     nsresult Restart();
     void     ParseLine(char *line);
@@ -166,15 +162,6 @@ private:
 
     nsresult                        mStatus;
 
-    // this lock is used to protect access to members which may be accessed
-    // from both the main thread as well as the socket thread.
-    PRLock                         *mLock;
-
-    // these transport status fields are protected by mLock
-    nsresult                        mTransportStatus;
-    PRUint32                        mTransportProgress;
-    PRUint32                        mTransportProgressMax;
-
     PRUint16                        mRestartCount;        // the number of times this transaction has been restarted
     PRUint8                         mCaps;
 
@@ -191,8 +178,6 @@ private:
     PRUint32                        mSentData           : 1;
     PRUint32                        mReceivedData       : 1;
     PRUint32                        mStatusEventPending : 1;
-    PRUint32                        mHasRequestBody     : 1;
-    PRUint32                        mSSLConnectFailed   : 1;
 
     // mClosed           := transaction has been explicitly closed
     // mTransactionDone  := transaction ran to completion or was interrupted
