@@ -22,13 +22,12 @@
 #include "nsMimeTypes.h"
 #include "nsNetUtil.h"
 
-#include "nsScriptSecurityManager.h"
-#include "nsIAggregatePrincipal.h"
+#include "nsIScriptSecurityManager.h"
+#include "nsIPrincipal.h"
 #include "nsIFileURL.h"
 #include "nsIJAR.h"
 
-static NS_DEFINE_CID(kScriptSecurityManagerCID, NS_SCRIPTSECURITYMANAGER_CID);
-static NS_DEFINE_CID(kInputStreamChannelCID, NS_INPUTSTREAMCHANNEL_CID);
+static NS_DEFINE_CID(kZipReaderCID, NS_ZIPREADER_CID);
 
 //-----------------------------------------------------------------------------
 
@@ -63,7 +62,12 @@ public:
     {
         NS_ASSERTION(mJarFile, "no jar file");
     }
-    virtual ~nsJARInputThunk() {}
+
+    virtual ~nsJARInputThunk()
+    {
+        if (!mJarCache && mJarReader)
+            mJarReader->Close();
+    }
 
     void GetJarReader(nsIZipReader **result)
     {
@@ -96,19 +100,26 @@ nsJARInputThunk::EnsureJarStream()
         return NS_OK;
 
     nsresult rv;
-    
-    rv = mJarCache->GetZip(mJarFile, getter_AddRefs(mJarReader));
+    if (mJarCache)
+        rv = mJarCache->GetZip(mJarFile, getter_AddRefs(mJarReader));
+    else {
+        // create an uncached jar reader
+        mJarReader = do_CreateInstance(kZipReaderCID, &rv);
+        if (NS_FAILED(rv)) return rv;
+
+        rv = mJarReader->Init(mJarFile);
+        if (NS_FAILED(rv)) return rv;
+
+        rv = mJarReader->Open();
+    }
     if (NS_FAILED(rv)) return rv;
 
     rv = mJarReader->GetInputStream(mJarEntry.get(),
                                     getter_AddRefs(mJarStream));
     if (NS_FAILED(rv)) return rv;
 
-    // ask the zip entry for the content length
-    nsCOMPtr<nsIZipEntry> entry;
-    mJarReader->GetEntry(mJarEntry.get(), getter_AddRefs(entry));
-    if (entry)
-        entry->GetRealSize((PRUint32 *) &mContentLength);
+    // ask the JarStream for the content length
+    mJarStream->Available((PRUint32 *) &mContentLength);
 
     return NS_OK;
 }
@@ -205,7 +216,7 @@ nsJARChannel::Init(nsIURI *uri)
 }
 
 nsresult
-nsJARChannel::CreateJarInput()
+nsJARChannel::CreateJarInput(nsIZipReaderCache *jarCache)
 {
     // important to pass a clone of the file since the nsIFile impl is not
     // necessarily MT-safe
@@ -213,7 +224,7 @@ nsJARChannel::CreateJarInput()
     nsresult rv = mJarFile->Clone(getter_AddRefs(clonedFile));
     if (NS_FAILED(rv)) return rv;
 
-    mJarInput = new nsJARInputThunk(clonedFile, mJarEntry, gJarHandler->JarCache());
+    mJarInput = new nsJARInputThunk(clonedFile, mJarEntry, jarCache);
     if (!mJarInput)
         return NS_ERROR_OUT_OF_MEMORY;
     NS_ADDREF(mJarInput);
@@ -242,7 +253,7 @@ nsJARChannel::EnsureJarInput(PRBool blocking)
     }
 
     if (mJarFile) {
-        rv = CreateJarInput();
+        rv = CreateJarInput(gJarHandler->JarCache());
     }
     else if (blocking) {
         NS_NOTREACHED("need sync downloader");
@@ -250,9 +261,11 @@ nsJARChannel::EnsureJarInput(PRBool blocking)
     }
     else {
         // kick off an async download of the base URI...
-        rv = NS_NewDownloader(getter_AddRefs(mDownloader),
-                              mJarBaseURI, this, nsnull, PR_FALSE,
-                              mLoadGroup, mCallbacks, mLoadFlags);
+        rv = NS_NewDownloader(getter_AddRefs(mDownloader), this);
+        if (NS_SUCCEEDED(rv))
+            rv = NS_OpenURI(mDownloader, nsnull, mJarBaseURI, nsnull,
+                            mLoadGroup, mCallbacks,
+                            mLoadFlags & ~LOAD_DOCUMENT_URI);
     }
     return rv;
 
@@ -405,24 +418,19 @@ nsJARChannel::GetOwner(nsISupports **result)
     if (NS_FAILED(rv)) return rv;
 
     if (cert) {
-        // Get the codebase principal
+        nsXPIDLCString certID;
+        rv = cert->GetCertificateID(getter_Copies(certID));
+        if (NS_FAILED(rv)) return rv;
+
         nsCOMPtr<nsIScriptSecurityManager> secMan = 
-                 do_GetService(kScriptSecurityManagerCID, &rv);
+                 do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
         if (NS_FAILED(rv)) return rv;
 
-        nsCOMPtr<nsIPrincipal> codebase;
-        rv = secMan->GetCodebasePrincipal(mJarBaseURI, 
-                                          getter_AddRefs(codebase));
-        if (NS_FAILED(rv)) return rv;
-    
-        // Join the certificate and the codebase
-        nsCOMPtr<nsIAggregatePrincipal> agg = do_QueryInterface(cert, &rv);
+        rv = secMan->GetCertificatePrincipal(certID, mJarBaseURI,
+                                             getter_AddRefs(cert));
         if (NS_FAILED(rv)) return rv;
 
-        rv = agg->SetCodebase(codebase);
-        if (NS_FAILED(rv)) return rv;
-
-        mOwner = do_QueryInterface(agg, &rv);
+        mOwner = do_QueryInterface(cert, &rv);
         if (NS_FAILED(rv)) return rv;
 
         NS_ADDREF(*result = mOwner);
@@ -507,6 +515,9 @@ nsJARChannel::GetContentType(nsACString &result)
 NS_IMETHODIMP
 nsJARChannel::SetContentType(const nsACString &aContentType)
 {
+    // If someone gives us a type hint we should just use that type instead of
+    // doing our guessing.  So we don't care when this is being called.
+
     // mContentCharset is unchanged if not parsed
     NS_ParseContentType(aContentType, mContentType, mContentCharset);
     return NS_OK;
@@ -515,6 +526,8 @@ nsJARChannel::SetContentType(const nsACString &aContentType)
 NS_IMETHODIMP
 nsJARChannel::GetContentCharset(nsACString &aContentCharset)
 {
+    // If someone gives us a charset hint we should just use that charset.
+    // So we don't care when this is being called.
     aContentCharset = mContentCharset;
     return NS_OK;
 }
@@ -601,14 +614,13 @@ nsJARChannel::AsyncOpen(nsIStreamListener *listener, nsISupports *ctx)
 
 NS_IMETHODIMP
 nsJARChannel::OnDownloadComplete(nsIDownloader *downloader,
-                                 nsISupports *closure,
-                                 nsresult status,
-                                 nsIFile *file)
+                                 nsresult       status,
+                                 nsIFile       *file)
 {
     if (NS_SUCCEEDED(status)) {
         mJarFile = file;
     
-        nsresult rv = CreateJarInput();
+        nsresult rv = CreateJarInput(nsnull);
         if (NS_SUCCEEDED(rv)) {
             // create input stream pump
             rv = NS_NewInputStreamPump(getter_AddRefs(mPump), mJarInput);
@@ -623,7 +635,6 @@ nsJARChannel::OnDownloadComplete(nsIDownloader *downloader,
         OnStopRequest(nsnull, nsnull, status);
     }
 
-    mDownloader = 0;
     return NS_OK;
 }
 
@@ -660,6 +671,7 @@ nsJARChannel::OnStopRequest(nsIRequest *req, nsISupports *ctx, nsresult status)
     mPump = 0;
     NS_IF_RELEASE(mJarInput);
     mIsPending = PR_FALSE;
+    mDownloader = 0; // this may delete the underlying jar file
     return NS_OK;
 }
 
