@@ -45,7 +45,8 @@
 #include "nsMimeTypes.h"
 #include "netCore.h"
 #include "nsXPIDLString.h"
-#include "nsIPref.h"
+#include "nsIPrefService.h"
+#include "nsIPrefBranch.h"
 #include "imgILoader.h"
 
 #include "nsCRT.h"
@@ -58,8 +59,6 @@
 
 #define MAX_BUFFER_SIZE 1024
 
-static NS_DEFINE_IID(kPrefServiceCID, NS_PREF_CID);
-
 #if defined WORDS_BIGENDIAN || defined IS_BIG_ENDIAN
 #define LITTLE_TO_NATIVE16(x) ((((x) & 0xFF) << 8) | ((x) >> 8))
 #else
@@ -67,18 +66,16 @@ static NS_DEFINE_IID(kPrefServiceCID, NS_PREF_CID);
 #endif
 
 nsUnknownDecoder::nsUnknownDecoder()
+  : mBuffer(nsnull)
+  , mBufferLen(0)
+  , mRequireHTMLsuffix(PR_FALSE)
 {
-  mBuffer = nsnull;
-  mBufferLen = 0;
-  mRequireHTMLsuffix = PR_FALSE;
-
-  nsresult rv;
-  nsCOMPtr<nsIPref> pPrefService = do_GetService(kPrefServiceCID, &rv);
-  if (NS_SUCCEEDED(rv)) {
-    rv = pPrefService->GetBoolPref("security.requireHTMLsuffix", &mRequireHTMLsuffix);
+  nsCOMPtr<nsIPrefBranch> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
+  if (prefs) {
+    PRBool val;
+    if (NS_SUCCEEDED(prefs->GetBoolPref("security.requireHTMLsuffix", &val)))
+      mRequireHTMLsuffix = val;
   }
-
-
 }
 
 nsUnknownDecoder::~nsUnknownDecoder()
@@ -389,49 +386,39 @@ PRBool nsUnknownDecoder::SniffForHTML(nsIRequest* aRequest)
     return PR_FALSE;
   }
   
-  // Now look for HTML.  First, we get us a nice nsCAutoString
-  // containing our data in a readonly-ish manner...
-  const CBufDescriptor bufDesc((const char*)mBuffer, PR_TRUE, mBufferLen, mBufferLen);
-  const nsCAutoString str(bufDesc);
-
-  nsCAutoString::const_iterator start, end;
-  str.BeginReading(start);
-  str.EndReading(end);
-  PRUint32 pos = 0; // for Substring ease
+  // Now look for HTML.
+  const char* str = mBuffer;
+  const char* end = mBuffer + mBufferLen;
 
   // skip leading whitespace
-  while (start != end && nsCRT::IsAsciiSpace(*start)) {
-    ++start;
-    ++pos;
+  while (str != end && nsCRT::IsAsciiSpace(*str)) {
+    ++str;
   }
 
   // did we find something like a start tag?
-  if (start == end || *start != '<' || ++start == end) {
+  if (str == end || *str != '<' || ++str == end) {
     return PR_FALSE;
   }
 
-  // advance pos to keep synch with |start|
-  ++pos;
-
   // If we seem to be SGML or XML and we got down here, just pretend we're HTML
-  if (*start == '!' || *start == '?') {
+  if (*str == '!' || *str == '?') {
     mContentType = TEXT_HTML;
     return PR_TRUE;
   }
-
-  const char* strPtr = str.get() + pos;
+  
+  PRUint32 bufSize = end - str;
   // We use sizeof(_tagstr) below because that's the length of _tagstr
   // with the one char " " or ">" appended.
 #define MATCHES_TAG(_tagstr)                                              \
-  (PL_strncasecmp(strPtr, _tagstr " ", sizeof(_tagstr)) == 0 ||  \
-   PL_strncasecmp(strPtr, _tagstr ">", sizeof(_tagstr)) == 0)
+  (bufSize >= sizeof(_tagstr) &&                                          \
+   (PL_strncasecmp(str, _tagstr " ", sizeof(_tagstr)) == 0 ||             \
+    PL_strncasecmp(str, _tagstr ">", sizeof(_tagstr)) == 0))
     
   if (MATCHES_TAG("html")     ||
       MATCHES_TAG("frameset") ||
       MATCHES_TAG("body")     ||
       MATCHES_TAG("head")     ||
       MATCHES_TAG("script")   ||
-      MATCHES_TAG("iframe")   ||
       MATCHES_TAG("a")        ||
       MATCHES_TAG("img")      ||
       MATCHES_TAG("table")    ||
