@@ -119,7 +119,7 @@
 #define CONTENTLOCALE_CMD_LINE_ARG "-contentLocale"   
 
 #define PREF_CONFIRM_AUTOMIGRATION     "profile.confirm_automigration"
-#define SHRIMP_PREF                    "shrimp.startup.enable"
+#define PREF_AUTOMIGRATION             "profile.allow_automigration"
 
 #if defined (XP_MAC)
 #define CHROME_STYLE nsIWebBrowserChrome::CHROME_WINDOW_BORDERS | nsIWebBrowserChrome::CHROME_WINDOW_CLOSE | nsIWebBrowserChrome::CHROME_CENTER_SCREEN
@@ -153,17 +153,9 @@ static nsHashtable *gLocaleProfiles = nsnull;
 static nsProfileDirServiceProvider *gDirServiceProvider = nsnull;
 
 // IID and CIDs of all the services needed
-static NS_DEFINE_CID(kIProfileIID, NS_IPROFILE_IID);
-static NS_DEFINE_CID(kComponentManagerCID, NS_COMPONENTMANAGER_CID);
-static NS_DEFINE_CID(kRegistryCID, NS_REGISTRY_CID);
-static NS_DEFINE_IID(kIFactoryIID,  NS_IFACTORY_IID);
-static NS_DEFINE_IID(kIIOServiceIID, NS_IIOSERVICE_IID);
-static NS_DEFINE_CID(kIOServiceCID, NS_IOSERVICE_CID);
 static NS_DEFINE_CID(kPrefMigrationCID, NS_PREFMIGRATION_CID);
 static NS_DEFINE_CID(kPrefConverterCID, NS_PREFCONVERTER_CID);
-static NS_DEFINE_IID(kCookieServiceCID, NS_COOKIESERVICE_CID);
 
-static NS_DEFINE_CID(kStringBundleServiceCID, NS_STRINGBUNDLESERVICE_CID);
 
 
 /*
@@ -459,7 +451,7 @@ nsProfile::LoadDefaultProfileDir(nsCString & profileURLStr, PRBool canInteract)
     
     GetProfileCount(&numProfiles);
 
-    if (profileURLStr.Length() == 0)
+    if (profileURLStr.IsEmpty())
     {
         // If this flag is TRUE, it makes the multiple profile case
         // just like the single profile case - the profile will be
@@ -515,7 +507,7 @@ nsProfile::LoadDefaultProfileDir(nsCString & profileURLStr, PRBool canInteract)
             profileURLStr = PROFILE_SELECTION_URL;
     }
 
-    if (profileURLStr.Length() != 0)
+    if (!profileURLStr.IsEmpty())
     {
         if (!canInteract) return NS_ERROR_PROFILE_REQUIRES_INTERACTION;
 
@@ -742,7 +734,7 @@ nsProfile::ProcessArgs(nsICmdLineService *cmdLineArgs,
             }
             else {
                 // get a platform charset
-                nsAutoString charSet;
+                nsCAutoString charSet;
                 rv = GetPlatformCharset(charSet);
                 NS_ASSERTION(NS_SUCCEEDED(rv), "failed to get a platform charset");
 
@@ -811,7 +803,7 @@ nsProfile::ProcessArgs(nsICmdLineService *cmdLineArgs,
             }
             else {
                 // get a platform charset
-                nsAutoString charSet;
+                nsCAutoString charSet;
                 rv = GetPlatformCharset(charSet);
                 NS_ASSERTION(NS_SUCCEEDED(rv), "failed to get a platform charset");
 
@@ -895,11 +887,20 @@ nsProfile::ProcessArgs(nsICmdLineService *cmdLineArgs,
 		NS_ASSERTION(NS_SUCCEEDED(rv),"failed to determine if we should force migration");
 	}
 
-#ifndef MOZ_XUL_APP // The phoenix/thunderbird doesn't use old profiles.
+    nsCOMPtr<nsIPrefBranch> prefBranch;
+   
+    // First check PREF_AUTOMIGRATION. 
+    PRBool allowAutoMigration = PR_TRUE;
+    nsCOMPtr<nsIPrefService> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
+    if (NS_FAILED(rv)) return rv;
+    rv = prefs->GetBranch(nsnull, getter_AddRefs(prefBranch));
+    if (NS_FAILED(rv)) return rv;
+
+    (void)prefBranch->GetBoolPref(PREF_AUTOMIGRATION, &allowAutoMigration);
 
     // Start Migaration activity
     rv = cmdLineArgs->GetCmdLineValue(INSTALLER_CMD_LINE_ARG, getter_Copies(cmdResult));
-    if (NS_SUCCEEDED(rv) || forceMigration)
+    if (allowAutoMigration && (NS_SUCCEEDED(rv) || forceMigration))
     {        
         if (cmdResult || forceMigration) {
             rv = MigrateProfileInfo();
@@ -937,8 +938,6 @@ nsProfile::ProcessArgs(nsICmdLineService *cmdLineArgs,
         }
     }
 
-#endif // MOZ_XUL_APP
-
 #ifdef DEBUG_profile_verbose
     printf("Profile Manager : Command Line Options : End\n");
 #endif
@@ -1455,7 +1454,7 @@ nsProfile::AddLevelOfIndirection(nsIFile *aDir)
   rv = aDir->Exists(&exists);
   NS_ENSURE_SUCCESS(rv,rv);
   if (!exists) {
-    rv = aDir->Create(nsIFile::DIRECTORY_TYPE, 0700);
+    rv = aDir->Create(nsIFile::DIRECTORY_TYPE, 0775);
     NS_ENSURE_SUCCESS(rv,rv);
   }
 	
@@ -1529,7 +1528,7 @@ nsresult nsProfile::SetProfileDir(const PRUnichar *profileName, nsIFile *profile
     PRBool exists;
     rv = profileDir->Exists(&exists);
     if (NS_SUCCEEDED(rv) && !exists)
-        rv = profileDir->Create(nsIFile::DIRECTORY_TYPE, 0700);
+        rv = profileDir->Create(nsIFile::DIRECTORY_TYPE, 0775);
     if (NS_FAILED(rv)) 
         return rv;
     
@@ -1591,7 +1590,7 @@ nsProfile::CreateNewProfileWithLocales(const PRUnichar* profileName,
         rv = profileDir->Exists(&exists);
         if (NS_FAILED(rv)) return rv;        
         if (!exists)
-            profileDir->Create(nsIFile::DIRECTORY_TYPE, 0700);
+            profileDir->Create(nsIFile::DIRECTORY_TYPE, 0775);
 
         // append profile name
         profileDir->Append(nsDependentString(profileName));
@@ -1610,7 +1609,7 @@ nsProfile::CreateNewProfileWithLocales(const PRUnichar* profileName,
     // Make profile directory unique only when the user 
     // decides to not use an already existing profile directory
     if (!useExistingDir) {
-        rv = profileDir->CreateUnique(nsIFile::DIRECTORY_TYPE, 0700);
+        rv = profileDir->CreateUnique(nsIFile::DIRECTORY_TYPE, 0775);
         if (NS_FAILED(rv)) return rv;
     }
 
@@ -1622,7 +1621,7 @@ nsProfile::CreateNewProfileWithLocales(const PRUnichar* profileName,
     if (NS_FAILED(rv)) return rv;        
     if (!exists)
     {
-        rv = profileDir->Create(nsIFile::DIRECTORY_TYPE, 0700);
+        rv = profileDir->Create(nsIFile::DIRECTORY_TYPE, 0775);
         if (NS_FAILED(rv)) return rv;
         useExistingDir = PR_FALSE;
     }
@@ -2227,7 +2226,7 @@ nsProfile::MigrateProfile(const PRUnichar* profileName)
     if (NS_FAILED(rv)) 
       return rv;
     
-    rv = newProfDir->CreateUnique(nsIFile::DIRECTORY_TYPE, 0700);
+    rv = newProfDir->CreateUnique(nsIFile::DIRECTORY_TYPE, 0775);
     if (NS_FAILED(rv)) 
       return rv;
     
@@ -2270,7 +2269,7 @@ nsProfile::RemigrateProfile(const PRUnichar* profileName)
     NS_ENSURE_SUCCESS(rv,rv);
     
     // Create a new directory for the remigrated profile
-    rv = newProfileDir->CreateUnique(nsIFile::DIRECTORY_TYPE, 0700);
+    rv = newProfileDir->CreateUnique(nsIFile::DIRECTORY_TYPE, 0775);
     NS_ASSERTION(NS_SUCCEEDED(rv), "failed to create new directory for the remigrated profile");
     if (NS_SUCCEEDED(rv)) {
         rv = MigrateProfileInternal(profileName, oldProfileDir, newProfileDir);
@@ -2375,7 +2374,7 @@ NS_IMETHODIMP nsProfile::CloneProfile(const PRUnichar* newProfile)
         destDir->AppendRelativePath(nsDependentString(newProfile));
 
         // Find a unique name in the dest dir
-        rv = destDir->CreateUnique(nsIFile::DIRECTORY_TYPE, 0700);
+        rv = destDir->CreateUnique(nsIFile::DIRECTORY_TYPE, 0775);
         if (NS_FAILED(rv)) return rv;
         
         rv = RecursiveCopy(currProfileDir, destDir);
