@@ -23,7 +23,7 @@
  *   Hubbie Shaw
  *   Doug Turner <dougt@netscape.com>
  *   Stuart Parmenter <pavlov@netscape.com>
- *   Brian Ryner <bryner@netscape.com>
+ *   Brian Ryner <bryner@brianryner.com>
  *   Terry Hayes <thayes@netscape.com>
  *   Kai Engert <kaie@netscape.com>
  *
@@ -106,12 +106,13 @@ RequestMapMatchEntry(PLDHashTable *table, const PLDHashEntryHdr *hdr,
   return entry->r == key;
 }
 
-PR_STATIC_CALLBACK(void)
+PR_STATIC_CALLBACK(PRBool)
 RequestMapInitEntry(PLDHashTable *table, PLDHashEntryHdr *hdr,
                      const void *key)
 {
   RequestHashEntry *entry = NS_STATIC_CAST(RequestHashEntry*, hdr);
   entry->r = (void*)key;
+  return PR_TRUE;
 }
 
 static PLDHashTableOps gMapOps = {
@@ -128,8 +129,8 @@ static PLDHashTableOps gMapOps = {
 
 
 nsSecureBrowserUIImpl::nsSecureBrowserUIImpl()
-  : mPreviousSecurityState(lis_no_security),
-    mIsViewSource(PR_FALSE)
+  : mIsViewSource(PR_FALSE),
+    mPreviousSecurityState(lis_no_security)
 {
   mTransferringRequests.ops = nsnull;
   mNewToplevelSecurityState = STATE_IS_INSECURE;
@@ -325,16 +326,12 @@ nsSecureBrowserUIImpl::Notify(nsIContent* formNode,
   if (!window || !actionURL || !formNode)
     return NS_OK;
   
-  nsCOMPtr<nsIDocument> document;
-  formNode->GetDocument(*getter_AddRefs(document));
+  nsCOMPtr<nsIDocument> document = formNode->GetDocument();
   if (!document) return NS_OK;
 
-  nsCOMPtr<nsIURI> formURL;
-  document->GetBaseURL(*getter_AddRefs(formURL));
-  
-  nsCOMPtr<nsIScriptGlobalObject> globalObject;
-  document->GetScriptGlobalObject(getter_AddRefs(globalObject));
-  nsCOMPtr<nsIDOMWindow> postingWindow(do_QueryInterface(globalObject));
+  nsIURI *formURL = document->GetBaseURL();
+
+  nsCOMPtr<nsIDOMWindow> postingWindow(do_QueryInterface(document->GetScriptGlobalObject()));
   
   PRBool isChild;
   IsChildOfDomWindow(mWindow, postingWindow, &isChild);
@@ -539,6 +536,22 @@ nsSecureBrowserUIImpl::OnStateChange(nsIWebProgress* aWebProgress,
         // If a document loading gets triggered, we will see more events.
         return NS_OK;
       }
+
+      if (NS_SUCCEEDED(uri->SchemeIs("wyciwyg", &vs)) && vs)
+      {
+        // We ignore everything caused by wycywig == document.write/writeln
+        // and assume the same security status
+        // Unfortunately, this results in different lock icon states
+        // when using "back".
+        // 1) goto secure page => secure lock icon
+        // 2) trigger document.writeln() => still secure lock icon
+        //    (because we not change lock icon)
+        // 3) go to a different insecure page => insecure lock icon
+        // 4) press "back" button => still insecure lock icon
+        // To fix this, we could try to remember the security state in the 
+        // wyciwyg channel object.
+        return NS_OK;
+      }
     }
   }
 
@@ -1123,23 +1136,21 @@ nsSecureBrowserUIImpl::OnLocationChange(nsIWebProgress* aWebProgress,
                                         nsIRequest* aRequest,
                                         nsIURI* aLocation)
 {
-  if (aLocation)
+  mCurrentURI = aLocation;
+
+  if (mCurrentURI)
   {
     PRBool vs;
-
-    nsresult rv = aLocation->SchemeIs("view-source", &vs);
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    if (vs) {
+    
+    if (NS_SUCCEEDED(mCurrentURI->SchemeIs("view-source", &vs)) && vs)
+    {
       PR_LOG(gSecureDocLog, PR_LOG_DEBUG,
              ("SecureUI:%p: OnLocationChange: view-source\n", this));
+
+      mIsViewSource = PR_TRUE;
     }
-
-    mIsViewSource = vs;
   }
 
-  mCurrentURI = aLocation;
-
   return NS_OK;
 }
 
