@@ -47,14 +47,12 @@
 #include "nsIPrompt.h"
 #include "nsIFilePicker.h"
 #include "nsJSPrincipals.h"
-#include "nsScriptSecurityManager.h"
 #include "nsIPrincipal.h"
 #include "nsXPIDLString.h"
 #include "nsIGenKeypairInfoDlg.h"
 #include "nsIDOMCryptoDialogs.h"
 #include "jsapi.h"
 #include "jsdbgapi.h"
-#include "jscntxt.h"
 #include <ctype.h>
 #include "nsReadableUtils.h"
 #include "pk11func.h"
@@ -376,20 +374,19 @@ nsCrypto::GetScriptPrincipal(JSContext *cx)
   if (principal)
     return principal;
 
-  if (JS_GetOptions(cx) & JSOPTION_PRIVATE_IS_NSISUPPORTS) {
-    nsISupports* scriptContextSupports =
-      NS_STATIC_CAST(nsISupports*, JS_GetContextPrivate(cx));
-    nsCOMPtr<nsIScriptContext> scriptContext(do_QueryInterface(scriptContextSupports));
-    if (scriptContext)
-    {
-      nsCOMPtr<nsIScriptGlobalObject> global;
-      scriptContext->GetGlobalObject(getter_AddRefs(global));
-      NS_ENSURE_TRUE(global, nsnull);
-      nsCOMPtr<nsIScriptObjectPrincipal> globalData = do_QueryInterface(global);
-      NS_ENSURE_TRUE(globalData, nsnull);
-      globalData->GetPrincipal(&principal);
-    }
+  nsCOMPtr<nsIScriptContext> scriptContext;
+  GetScriptContextFromJSContext(cx, getter_AddRefs(scriptContext));
+
+  if (scriptContext)
+  {
+    nsCOMPtr<nsIScriptGlobalObject> global;
+    scriptContext->GetGlobalObject(getter_AddRefs(global));
+    NS_ENSURE_TRUE(global, nsnull);
+    nsCOMPtr<nsIScriptObjectPrincipal> globalData = do_QueryInterface(global);
+    NS_ENSURE_TRUE(globalData, nsnull);
+    globalData->GetPrincipal(&principal);
   }
+
   return principal;
 }
 
@@ -1791,7 +1788,7 @@ nsCryptoRunnable::Run()
   nsNSSShutDownPreventionLock locker;
   JSPrincipals *principals;
 
-  nsresult rv = m_args->m_principals->GetJSPrincipals(&principals);
+  nsresult rv = m_args->m_principals->GetJsPrincipals(&principals);
   if (NS_FAILED(rv))
     return NS_ERROR_FAILURE;
 
@@ -2006,7 +2003,7 @@ nsCrypto::ImportUserCertificates(const nsAString& aNickname,
            node = CERT_LIST_NEXT(node), i++) {
         derCerts[i] = node->cert->derCert;
       }
-      nsNSSCertificateDB::ImportValidCACerts(numCAs, derCerts, ctx);
+      CERT_ImportCAChain(derCerts, numCAs, certUsageUserCertImport);
       nsMemory::Free(derCerts);
     }
     
