@@ -301,7 +301,7 @@ ProcessRawBytes(SECItem *data, nsString &text)
   char buffer[5];
   for (i=0; i<data->len; i++) {
     PR_snprintf(buffer, 5, "%02x ", data->data[i]);
-    text.Append(NS_ConvertASCIItoUCS2(buffer).get());
+    AppendASCIItoUTF16(buffer, text);
     if ((i+1)%16 == 0) {
       text.Append(NS_LITERAL_STRING(SEPARATOR).get());
     }
@@ -495,18 +495,23 @@ ProcessSingleExtension(CERTCertExtension *extension,
 PRUint32 
 getCertType(CERTCertificate *cert)
 {
+  char *nick = cert->nickname;
+  char *email = cert->emailAddr;
   nsNSSCertTrust trust(cert->trust);
-  if (cert->nickname && trust.HasAnyUser())
-    return nsIX509Cert::USER_CERT;
-  if (trust.HasAnyCA())
-    return nsIX509Cert::CA_CERT;
-  if (trust.HasPeer(PR_TRUE, PR_FALSE, PR_FALSE))
-    return nsIX509Cert::SERVER_CERT;
-  if (trust.HasPeer(PR_FALSE, PR_TRUE, PR_FALSE) && cert->emailAddr)
+  /*
+fprintf(stderr, "====> nick: %s  email: %s  has-any-user: %d  hash-any-ca: %d  has-peer100: %d  has-peer001: %d\n",
+  nick, email, (nick) ? trust.HasAnyUser() : 0, (nick) ? trust.HasAnyCA() : 0, (nick) ? trust.HasPeer(PR_TRUE, PR_FALSE, PR_FALSE) : 0, 
+  (email) ? trust.HasPeer(PR_FALSE, PR_TRUE, PR_FALSE) : 0 );
+*/
+  if (nick) {
+    if (trust.HasAnyUser())
+      return nsIX509Cert::USER_CERT;
+    if (trust.HasAnyCA() || CERT_IsCACert(cert,NULL))
+      return nsIX509Cert::CA_CERT;
+    if (trust.HasPeer(PR_TRUE, PR_FALSE, PR_FALSE))
+      return nsIX509Cert::SERVER_CERT;
+  }
+  if (email && trust.HasPeer(PR_FALSE, PR_TRUE, PR_FALSE))
     return nsIX509Cert::EMAIL_CERT;
-  if (CERT_IsCACert(cert,NULL))
-    return nsIX509Cert::CA_CERT;
-  if (cert->emailAddr)
-    return nsIX509Cert::EMAIL_CERT;
-  return nsIX509Cert::SERVER_CERT;
+  return nsIX509Cert::UNKNOWN_CERT;
 }
