@@ -21,7 +21,7 @@
  *   Hubbie Shaw
  *   Doug Turner <dougt@netscape.com>
  *   Mitch Stoltz <mstoltz@netscape.com>
- *   Brian Ryner <bryner@netscape.com>
+ *   Brian Ryner <bryner@brianryner.com>
  *   Kai Engert <kaie@netscape.com>
  */
 
@@ -58,7 +58,7 @@
 #include "nsIWindowWatcher.h"
 #include "nsIPrompt.h"
 #include "nsProxiedService.h"
-#include "nsICertificatePrincipal.h"
+#include "nsIPrincipal.h"
 #include "nsReadableUtils.h"
 #include "nsIDateTimeFormat.h"
 #include "prtypes.h"
@@ -91,7 +91,6 @@ extern "C" {
 PRLogModuleInfo* gPIPNSSLog = nsnull;
 #endif
 
-static NS_DEFINE_CID(kDateTimeFormatCID, NS_DATETIMEFORMAT_CID);
 static NS_DEFINE_CID(kNSSComponentCID, NS_NSSCOMPONENT_CID);
 int nsNSSComponent::mInstanceCount = 0;
 
@@ -1321,7 +1320,7 @@ NS_IMPL_THREADSAFE_ISUPPORTS5(nsNSSComponent,
                               nsIEntropyCollector,
                               nsINSSComponent,
                               nsIObserver,
-                              nsISupportsWeakReference);
+                              nsISupportsWeakReference)
 
 //---------------------------------------------
 // Functions Implementing nsISignatureVerifier
@@ -1459,17 +1458,19 @@ nsNSSComponent::VerifySignature(const char* aRSABuf, PRUint32 aRSABufLen,
     rv2 = pCert->GetSha1Fingerprint(fingerprint);
     NS_LossyConvertUCS2toASCII fingerprintStr(fingerprint);
     if (NS_FAILED(rv2)) return rv2;
-    rv2 = mScriptSecurityManager->GetCertificatePrincipal(fingerprintStr.get(), aPrincipal);
-    if (NS_FAILED(rv2) || !*aPrincipal) return rv2;
+    nsCOMPtr<nsIPrincipal> certPrincipal;
+    rv2 = mScriptSecurityManager->GetCertificatePrincipal(fingerprintStr.get(), nsnull,
+                                                          getter_AddRefs(certPrincipal));
+    if (NS_FAILED(rv2) || !certPrincipal) return rv2;
 
-    nsCOMPtr<nsICertificatePrincipal> certPrincipal = do_QueryInterface(*aPrincipal, &rv2);
-    if (NS_FAILED(rv2)) return rv2;
     nsAutoString orgName;
     rv2 = pCert->GetOrganization(orgName);
     if (NS_FAILED(rv2)) return rv2;
     NS_LossyConvertUCS2toASCII  orgNameStr(orgName);
     rv2 = certPrincipal->SetCommonName(orgNameStr.get());
     if (NS_FAILED(rv2)) return rv2;
+
+    *aPrincipal = certPrincipal;
   }
 
   if (p7_info) {
@@ -2189,7 +2190,7 @@ getPSMContentType(const char * aContentType)
 
 NS_IMPL_ISUPPORTS2(PSMContentListener,
                    nsIURIContentListener,
-                   nsISupportsWeakReference); 
+                   nsISupportsWeakReference) 
 
 PSMContentListener::PSMContentListener()
 {
