@@ -18,7 +18,7 @@
  * Rights Reserved.
  *
  * Contributor(s):
- *  Brian Ryner <bryner@netscape.com>
+ *  Brian Ryner <bryner@brianryner.com>
  *  Javier Delgadillo <javi@netscape.com>
  *
  * Alternatively, the contents of this file may be used under the
@@ -1199,7 +1199,8 @@ static void InitNSSMethods()
 }
 
 nsresult
-nsSSLIOLayerNewSocket(const char *host,
+nsSSLIOLayerNewSocket(PRInt32 family,
+                      const char *host,
                       PRInt32 port,
                       const char *proxyHost,
                       PRInt32 proxyPort,
@@ -1217,10 +1218,10 @@ nsSSLIOLayerNewSocket(const char *host,
     firstTime = PR_FALSE;
   }
 
-  PRFileDesc* sock = PR_OpenTCPSocket(PR_AF_INET6);
+  PRFileDesc* sock = PR_OpenTCPSocket(family);
   if (!sock) return NS_ERROR_OUT_OF_MEMORY;
 
-  nsresult rv = nsSSLIOLayerAddToSocket(host, port, proxyHost, proxyPort,
+  nsresult rv = nsSSLIOLayerAddToSocket(family, host, port, proxyHost, proxyPort,
                                         sock, info, forSTARTTLS);
   if (NS_FAILED(rv)) {
     PR_Close(sock);
@@ -2326,7 +2327,8 @@ nsSSLIOLayerSetOptions(PRFileDesc *fd, PRBool forSTARTTLS,
 }
 
 nsresult
-nsSSLIOLayerAddToSocket(const char* host,
+nsSSLIOLayerAddToSocket(PRInt32 family,
+                        const char* host,
                         PRInt32 port,
                         const char* proxyHost,
                         PRInt32 proxyPort,
