@@ -68,6 +68,14 @@ extern char *sys_errlist[];
 
 #define ERROR_BREAK rv = SECFailure;break;
 
+const SEC_ASN1Template SECKEY_PQGParamsTemplate[] = {
+    { SEC_ASN1_SEQUENCE, 0, NULL, sizeof(SECKEYPQGParams) },
+    { SEC_ASN1_INTEGER, offsetof(SECKEYPQGParams,prime) },
+    { SEC_ASN1_INTEGER, offsetof(SECKEYPQGParams,subPrime) },
+    { SEC_ASN1_INTEGER, offsetof(SECKEYPQGParams,base) },
+    { 0, }
+};
+
 /* returns 0 for success, -1 for failure (EOF encountered) */
 static int
 UpdateRNG(void)
@@ -126,8 +134,7 @@ UpdateRNG(void)
 	    rv = -1;
 	    break;
 	}
-	RNG_GetNoise(&randbuf[1], sizeof(randbuf)-1);
-	RNG_RandomUpdate(randbuf, sizeof(randbuf));
+	PK11_RandomUpdate(randbuf, sizeof(randbuf));
 	if (c != randbuf[0]) {
 	    randbuf[0] = c;
 	    FPS "\r|");
@@ -189,27 +196,27 @@ static unsigned char G[] = { 0x00, 0x62, 0x6d, 0x02, 0x78, 0x39, 0xea, 0x0a,
 			     0x8c, 0xc5, 0x72, 0xaf, 0x53, 0xe6, 0xd7, 0x88,
 			     0x02 };
 
-static PQGParams default_pqg_params = {
+static SECKEYPQGParams default_pqg_params = {
     NULL,
     { 0, P, sizeof(P) },
     { 0, Q, sizeof(Q) },
     { 0, G, sizeof(G) }
 };
 
-static PQGParams *
+static SECKEYPQGParams *
 decode_pqg_params(char *str)
 {
     char *buf;
     unsigned int len;
     PRArenaPool *arena;
-    PQGParams *params;
+    SECKEYPQGParams *params;
     SECStatus status;
  
     arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
     if (arena == NULL)
         return NULL;
  
-    params = PORT_ArenaZAlloc(arena, sizeof(PQGParams));
+    params = PORT_ArenaZAlloc(arena, sizeof(SECKEYPQGParams));
     if (params == NULL)
         goto loser;
     params->arena = arena;
@@ -229,11 +236,19 @@ loser:
         PORT_FreeArena(arena, PR_FALSE);
     return NULL;
 }
- 
+
+void  
+CERTUTIL_DestroyParamsPQG(SECKEYPQGParams *params)
+{
+    if (params->arena) {
+        PORT_FreeArena(params->arena, PR_FALSE);
+    }
+}
+
 static int
 pqg_prime_bits(char *str)
 {
-    PQGParams *params = NULL;
+    SECKEYPQGParams *params = NULL;
     int primeBits = 0, i;
  
     params = decode_pqg_params(str);
@@ -246,7 +261,7 @@ pqg_prime_bits(char *str)
  
 done:
     if (params != NULL)
-        PQG_DestroyParams(params);
+        CERTUTIL_DestroyParamsPQG(params);
     return primeBits;
 }
 
@@ -270,7 +285,7 @@ SECU_GetpqgString(char *filename)
     return NULL;
 }
 
-PQGParams*
+SECKEYPQGParams*
 getpqgfromfile(int keyBits, char *pqgFile)
 {
     char *end, *str, *pqgString;
@@ -302,6 +317,26 @@ found_match:
     return decode_pqg_params(str);
 }
 
+void CERTUTIL_FileForRNG(char *noise)
+{
+    char buf[2048];
+    PRFileDesc *fd;
+    PRInt32 count;
+
+    fd = PR_OpenFile(noise,PR_RDONLY,0666);
+    if (!fd) return;
+
+    do {
+	count = PR_Read(fd,buf,sizeof(buf));
+	if (count > 0) {
+	    PK11_RandomUpdate(buf,count);
+	}
+    } while (count > 0);
+
+    PR_Close(fd);
+   
+}
+
 SECKEYPrivateKey *
 CERTUTIL_GeneratePrivateKey(KeyType keytype, PK11SlotInfo *slot, int size,
 			    int publicExponent, char *noise, 
@@ -311,17 +346,16 @@ CERTUTIL_GeneratePrivateKey(KeyType keytype, PK11SlotInfo *slot, int size,
     CK_MECHANISM_TYPE mechanism;
     SECOidTag algtag;
     PK11RSAGenParams rsaparams;
-    PQGParams *dsaparams = NULL;
+    SECKEYPQGParams *dsaparams = NULL;
     void *params;
     PRArenaPool *dsaparena;
 
     /*
      * Do some random-number initialization.
      */
-    RNG_SystemInfoForRNG();
 
     if (noise) {
-    	RNG_FileForRNG(noise);
+    	CERTUTIL_FileForRNG(noise);
     } else {
 	int rv = UpdateRNG();
 	if (rv) {
@@ -346,7 +380,7 @@ CERTUTIL_GeneratePrivateKey(KeyType keytype, PK11SlotInfo *slot, int size,
 	} else {
 	    dsaparena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
 	    if (dsaparena == NULL) return NULL;
-	    dsaparams = PORT_ArenaZAlloc(dsaparena, sizeof(PQGParams));
+	    dsaparams = PORT_ArenaZAlloc(dsaparena, sizeof(SECKEYPQGParams));
 	    if (dsaparams == NULL) return NULL;
 	    dsaparams->arena = dsaparena;
 	    SECITEM_AllocItem(dsaparena, &dsaparams->prime, sizeof P);
