@@ -34,7 +34,7 @@
 /*
  * cmsutil -- A command to work with CMS data
  *
- * $Id: cmsutil.c,v 1.41.6.1 2004/01/23 18:56:32 mkaply%us.ibm.com Exp $
+ * $Id: cmsutil.c,v 1.44 2003/11/13 23:03:12 nelsonb%netscape.com Exp $
  */
 
 #include "nspr.h"
@@ -87,7 +87,6 @@ DigestFile(PLArenaPool *poolp, SECItem ***digests, SECItem *input,
            SECAlgorithmID **algids)
 {
     NSSCMSDigestContext *digcx;
-    SECStatus rv;
 
     digcx = NSS_CMSDigestContext_StartMultiple(algids);
     if (digcx == NULL)
@@ -95,8 +94,7 @@ DigestFile(PLArenaPool *poolp, SECItem ***digests, SECItem *input,
 
     NSS_CMSDigestContext_Update(digcx, input->data, input->len);
 
-    rv = NSS_CMSDigestContext_FinishMultiple(digcx, poolp, digests);
-    return rv;
+    return NSS_CMSDigestContext_FinishMultiple(digcx, poolp, digests);
 }
 
 
@@ -107,11 +105,9 @@ Usage(char *progName)
 "Usage:  %s [-C|-D|-E|-O|-S] [<options>] [-d dbdir] [-u certusage]\n"
 " -C            create a CMS encrypted data message\n"
 " -D            decode a CMS message\n"
-"  -b           decode a batch of files named in infile\n"
 "  -c content   use this detached content\n"
 "  -n           suppress output of content\n"
 "  -h num       display num levels of CMS message info as email headers\n"
-"  -k           keep decoded encryption certs in perm cert db\n"
 " -E            create a CMS enveloped data message\n"
 "  -r id,...    create envelope for these recipients,\n"
 "               where id can be a certificate nickname or email address\n"
@@ -159,12 +155,11 @@ struct optionsStr {
 
 struct decodeOptionsStr {
     struct optionsStr *options;
-    SECItem            content;
+    PRFileDesc *contentFile;
     int headerLevel;
     PRBool suppressContent;
     NSSCMSGetDecryptKeyCallback dkcb;
     PK11SymKey *bulkkey;
-    PRBool      keepCerts;
 };
 
 struct signOptionsStr {
@@ -200,14 +195,22 @@ struct encryptOptionsStr {
 };
 
 static NSSCMSMessage *
-decode(FILE *out, SECItem *input, const struct decodeOptionsStr *decodeOptions)
+decode(FILE *out, SECItem *output, SECItem *input, 
+       const struct decodeOptionsStr *decodeOptions)
 {
     NSSCMSDecoderContext *dcx;
     NSSCMSMessage *cmsg;
-    int nlevels, i;
+    NSSCMSContentInfo *cinfo;
+    NSSCMSSignedData *sigd = NULL;
+    NSSCMSEnvelopedData *envd;
+    NSSCMSEncryptedData *encd;
+    int nlevels, i, nsigners, j;
+    char *signercn;
+    NSSCMSSignerInfo *si;
+    SECOidTag typetag;
+    SECItem **digests;
     SECItem sitem = { 0, 0, 0 };
 
-    PORT_SetError(0);
     dcx = NSS_CMSDecoder_Start(NULL, 
                                NULL, NULL,         /* content callback     */
                                pwcb, pwcb_arg,     /* password callback    */
@@ -227,9 +230,6 @@ decode(FILE *out, SECItem *input, const struct decodeOptionsStr *decodeOptions)
 
     nlevels = NSS_CMSMessage_ContentLevelCount(cmsg);
     for (i = 0; i < nlevels; i++) {
-	NSSCMSContentInfo *cinfo;
-	SECOidTag typetag;
-
 	cinfo = NSS_CMSMessage_ContentLevel(cmsg, i);
 	typetag = NSS_CMSContentInfo_GetContentTypeTag(cinfo);
 
@@ -238,28 +238,25 @@ decode(FILE *out, SECItem *input, const struct decodeOptionsStr *decodeOptions)
 
 	switch (typetag) {
 	case SEC_OID_PKCS7_SIGNED_DATA:
-	  {
-	    NSSCMSSignedData *sigd = NULL;
-	    SECItem **digests;
-	    int nsigners;
-	    int j;
-
 	    if (decodeOptions->headerLevel >= 0)
 		fprintf(out, "type=signedData; ");
 	    sigd = (NSSCMSSignedData *)NSS_CMSContentInfo_GetContent(cinfo);
 	    if (sigd == NULL) {
-		SECU_PrintError(progName, "signedData component missing");
+		SECU_PrintError(progName, 
+		                "problem finding signedData component");
 		goto loser;
 	    }
 
 	    /* if we have a content file, but no digests for this signedData */
-	    if (decodeOptions->content.data != NULL && 
+	    if (decodeOptions->contentFile != NULL && 
 	        !NSS_CMSSignedData_HasDigests(sigd)) {
 		PLArenaPool     *poolp;
 		SECAlgorithmID **digestalgs;
 
 		/* detached content: grab content file */
-		sitem = decodeOptions->content;
+		if (!sitem.data) {
+		    SECU_FileToItem(&sitem, decodeOptions->contentFile);
+		}
 
 		if ((poolp = PORT_NewArena(1024)) == NULL) {
 		    fprintf(stderr, "cmsutil: Out of memory.\n");
@@ -270,14 +267,13 @@ decode(FILE *out, SECItem *input, const struct decodeOptionsStr *decodeOptions)
 		      != SECSuccess) {
 		    SECU_PrintError(progName, 
 		                    "problem computing message digest");
-		    PORT_FreeArena(poolp, PR_FALSE);
 		    goto loser;
 		}
 		if (NSS_CMSSignedData_SetDigests(sigd, digestalgs, digests) 
 		    != SECSuccess) {
+		    
 		    SECU_PrintError(progName, 
 		                    "problem setting message digests");
-		    PORT_FreeArena(poolp, PR_FALSE);
 		    goto loser;
 		}
 		PORT_FreeArena(poolp, PR_FALSE);
@@ -287,7 +283,7 @@ decode(FILE *out, SECItem *input, const struct decodeOptionsStr *decodeOptions)
 	    if (NSS_CMSSignedData_ImportCerts(sigd, 
 	                                   decodeOptions->options->certHandle, 
 	                                   decodeOptions->options->certUsage, 
-	                                   decodeOptions->keepCerts) 
+	                                   PR_FALSE) 
 	          != SECSuccess) {
 		SECU_PrintError(progName, "cert import failed");
 		goto loser;
@@ -298,11 +294,13 @@ decode(FILE *out, SECItem *input, const struct decodeOptionsStr *decodeOptions)
 	    if (decodeOptions->headerLevel >= 0)
 		fprintf(out, "nsigners=%d; ", nsigners);
 	    if (nsigners == 0) {
-		/* Might be a cert transport message
-		** or might be an invalid message, such as a QA test message
-		** or a message from an attacker.
-		*/
+		/* must be a cert transport message */
 		SECStatus rv;
+		/* XXX workaround for bug #54014 */
+		NSS_CMSSignedData_ImportCerts(sigd, 
+                                            decodeOptions->options->certHandle, 
+		                            decodeOptions->options->certUsage, 
+		                            PR_TRUE);
 		rv = NSS_CMSSignedData_VerifyCertsOnly(sigd, 
 		                            decodeOptions->options->certHandle, 
 		                            decodeOptions->options->certUsage);
@@ -320,23 +318,16 @@ decode(FILE *out, SECItem *input, const struct decodeOptionsStr *decodeOptions)
 	    }
 
 	    for (j = 0; j < nsigners; j++) {
-		const char * svs;
-		NSSCMSSignerInfo *si;
-		NSSCMSVerificationStatus vs;
 		SECStatus bad;
+		NSSCMSVerificationStatus vs;
+		const char * svs;
 
 		si = NSS_CMSSignedData_GetSignerInfo(sigd, j);
-		if (decodeOptions->headerLevel >= 0) {
-		    char *signercn;
-		    static char empty[] = { "" };
-
-		    signercn = NSS_CMSSignerInfo_GetSignerCommonName(si);
-		    if (signercn == NULL)
-			signercn = empty;
+		signercn = NSS_CMSSignerInfo_GetSignerCommonName(si);
+		if (signercn == NULL)
+		    signercn = "";
+		if (decodeOptions->headerLevel >= 0)
 		    fprintf(out, "\n\t\tsigner%d.id=\"%s\"; ", j, signercn);
-		    if (signercn != empty)
-		        PORT_Free(signercn);
-		}
 		bad = NSS_CMSSignedData_VerifySignerInfo(sigd, j, 
 		                           decodeOptions->options->certHandle, 
 		                           decodeOptions->options->certUsage);
@@ -350,32 +341,17 @@ decode(FILE *out, SECItem *input, const struct decodeOptionsStr *decodeOptions)
 		    goto loser;
 		}
 	    }
-	  }
-	  break;
+	    break;
 	case SEC_OID_PKCS7_ENVELOPED_DATA:
-	  {
-	    NSSCMSEnvelopedData *envd;
 	    if (decodeOptions->headerLevel >= 0)
 		fprintf(out, "type=envelopedData; ");
 	    envd = (NSSCMSEnvelopedData *)NSS_CMSContentInfo_GetContent(cinfo);
-	    if (envd == NULL) {
-		SECU_PrintError(progName, "envelopedData component missing");
-		goto loser;
-	    }
-	  }
-	  break;
+	    break;
 	case SEC_OID_PKCS7_ENCRYPTED_DATA:
-	  {
-	    NSSCMSEncryptedData *encd;
 	    if (decodeOptions->headerLevel >= 0)
 		fprintf(out, "type=encryptedData; ");
 	    encd = (NSSCMSEncryptedData *)NSS_CMSContentInfo_GetContent(cinfo);
-	    if (encd == NULL) {
-		SECU_PrintError(progName, "encryptedData component missing");
-		goto loser;
-	    }
-	  }
-	  break;
+	    break;
 	case SEC_OID_PKCS7_DATA:
 	    if (decodeOptions->headerLevel >= 0)
 		fprintf(out, "type=data; ");
@@ -387,12 +363,11 @@ decode(FILE *out, SECItem *input, const struct decodeOptionsStr *decodeOptions)
 	    fprintf(out, "\n");
     }
 
-    if (!decodeOptions->suppressContent && out) {
-	SECItem *item = (sitem.data ? &sitem 
-	                            : NSS_CMSMessage_GetContent(cmsg));
-	if (item && item->data && item->len) {
-	    fwrite(item->data, item->len, 1, out);
-    	}
+    if (!decodeOptions->suppressContent) {
+	SECItem *item = (sitem.data) 
+	                    ? &sitem 
+	                    : NSS_CMSMessage_GetContent(cmsg);
+	SECITEM_CopyItem(NULL, output, item);
     }
     return cmsg;
 
@@ -978,87 +953,8 @@ loser:
     return NULL;
 }
 
-static char *
-pl_fgets(char * buf, int size, PRFileDesc * fd)
-{
-    char * bp = buf;
-    int    nb = 0;;
-
-    while (size > 1) {
-    	nb = PR_Read(fd, bp, 1);
-	if (nb < 0) {
-	    /* deal with error */
-	    return NULL;
-	} else if (nb == 0) {
-	    /* deal with EOF */
-	    return NULL;
-	} else if (*bp == '\n') {
-	    /* deal with EOL */
-	    ++bp;  /* keep EOL character */
-	    break;
-	} else {
-	    /* ordinary character */
-	    ++bp;
-	    --size;
-	}
-    }
-    *bp = '\0';
-    return buf;
-}
-
 typedef enum { UNKNOWN, DECODE, SIGN, ENCRYPT, ENVELOPE, CERTSONLY } Mode;
 
-static int 
-doBatchDecode(FILE *outFile, PRFileDesc *batchFile, 
-              const struct decodeOptionsStr *decodeOptions)
-{
-    char * str;
-    int    exitStatus = 0;
-    char   batchLine[512];
-
-    while (NULL != (str = pl_fgets(batchLine, sizeof batchLine, batchFile))) {
-	NSSCMSMessage *cmsg = NULL;
-	PRFileDesc *   inFile;
-    	int            len = strlen(str);
-	SECStatus      rv;
-	SECItem        input = {0, 0, 0};
-	char           cc;
-
-	while (len > 0 && 
-	       ((cc = str[len - 1]) == '\n' || cc == '\r')) {
-	    str[--len] = '\0';
-	}
-	if (!len) /* skip empty line */
-	    continue;
-	if (str[0] == '#')
-	    continue;  /* skip comment line */
-	fprintf(outFile, "========== %s ==========\n", str);
-	inFile = PR_Open(str, PR_RDONLY, 00660);
-	if (inFile == NULL) {
-	    fprintf(outFile, "%s: unable to open \"%s\" for reading\n",
-		    progName, str);
-	    exitStatus = 1;
-	    continue;
-	}
-	rv = SECU_FileToItem(&input, inFile);
-	PR_Close(inFile);
-	if (rv != SECSuccess) {
-	    SECU_PrintError(progName, "unable to read infile");
-	    exitStatus = 1;
-	    continue;
-	}
-	cmsg = decode(outFile, &input, decodeOptions);
-	SECITEM_FreeItem(&input, PR_FALSE);
-	if (cmsg)
-	    NSS_CMSMessage_Destroy(cmsg);
-	else {
-	    SECU_PrintError(progName, "problem decoding");
-	    exitStatus = 1;
-	}
-    }
-    return exitStatus;
-}
-
 int
 main(int argc, char **argv)
 {
@@ -1080,17 +976,10 @@ main(int argc, char **argv)
     char *str, *tok;
     char *envFileName;
     SECItem input = { 0, 0, 0};
+    SECItem output = { 0, 0, 0};
+    SECItem dummy = { 0, 0, 0 };
     SECItem envmsg = { 0, 0, 0 };
     SECStatus rv;
-    PRFileDesc *contentFile = NULL;
-    PRBool      batch = PR_FALSE;
-
-#ifdef NISCC_TESTING
-    const char *ev = PR_GetEnv("NSS_DISABLE_ARENA_FREE_LIST");
-    PORT_Assert(ev); 
-    ev = PR_GetEnv("NSS_STRICT_SHUTDOWN");
-    PORT_Assert(ev); 
-#endif 
 
     progName = strrchr(argv[0], '/');
     if (!progName)
@@ -1101,11 +990,9 @@ main(int argc, char **argv)
     outFile = stdout;
     envFileName = NULL;
     mode = UNKNOWN;
-    decodeOptions.content.data = NULL;
-    decodeOptions.content.len  = 0;
+    decodeOptions.contentFile = NULL;
     decodeOptions.suppressContent = PR_FALSE;
     decodeOptions.headerLevel = -1;
-    decodeOptions.keepCerts = PR_FALSE;
     options.certUsage = certUsageEmailSigner;
     options.password = NULL;
     signOptions.nickname = NULL;
@@ -1126,7 +1013,7 @@ main(int argc, char **argv)
      * Parse command line arguments
      */
     optstate = PL_CreateOptState(argc, argv, 
-				 "CDEGH:N:OPSTY:bc:d:e:h:i:kno:p:r:s:u:v");
+				 "CDEGH:N:OPSTY:c:d:e:h:i:no:p:r:s:u:v");
     while ((status = PL_GetNextOpt(optstate)) == PL_OPT_OK) {
 	switch (optstate->option) {
 	case 'C':
@@ -1225,17 +1112,6 @@ main(int argc, char **argv)
 	    signOptions.encryptionKeyPreferenceNick = strdup(optstate->value);
 	    break;
 
-	case 'b':
-	    if (mode != DECODE) {
-		fprintf(stderr, 
-		        "%s: option -b only supported with option -D.\n", 
-		        progName);
-		Usage(progName);
-		exit(1);
-	    }
-	    batch = PR_TRUE;
-	    break;
-
 	case 'c':
 	    if (mode != DECODE) {
 		fprintf(stderr, 
@@ -1244,25 +1120,12 @@ main(int argc, char **argv)
 		Usage(progName);
 		exit(1);
 	    }
-	    contentFile = PR_Open(optstate->value, PR_RDONLY, 006600);
-	    if (contentFile == NULL) {
+	    if ((decodeOptions.contentFile = 
+	          PR_Open(optstate->value, PR_RDONLY, 006600)) == NULL) {
 		fprintf(stderr, "%s: unable to open \"%s\" for reading.\n",
 			progName, optstate->value);
 		exit(1);
 	    }
-
-	    rv = SECU_FileToItem(&decodeOptions.content, contentFile);
-	    PR_Close(contentFile);
-	    if (rv != SECSuccess) {
-		SECU_PrintError(progName, "problem reading content file");
-		exit(1);
-	    }
-	    if (!decodeOptions.content.data) {
-		/* file was zero length */
-		decodeOptions.content.data = (unsigned char *)PORT_Strdup("");
-		decodeOptions.content.len  = 0;
-	    }
-
 	    break;
 	case 'd':
 	    SECU_ConfigDirectory(optstate->value);
@@ -1295,17 +1158,6 @@ main(int argc, char **argv)
 	    }
 	    break;
 
-	case 'k':
-	    if (mode != DECODE) {
-		fprintf(stderr, 
-		        "%s: option -k only supported with option -D.\n", 
-		        progName);
-		Usage(progName);
-		exit(1);
-	    }
-	    decodeOptions.keepCerts = PR_TRUE;
-	    break;
-
 	case 'n':
 	    if (mode != DECODE) {
 		fprintf(stderr, 
@@ -1375,16 +1227,10 @@ main(int argc, char **argv)
     if (mode == UNKNOWN)
 	Usage(progName);
 
-    if (mode != CERTSONLY && !batch) {
-	rv = SECU_FileToItem(&input, inFile);
-	if (rv != SECSuccess) {
-	    SECU_PrintError(progName, "unable to read infile");
-	    exit(1);
-	}
-	if (inFile != PR_STDIN) {
-	    PR_Close(inFile);
-    	}
-    }
+    if (mode != CERTSONLY)
+	SECU_FileToItem(&input, inFile);
+    if (inFile != PR_STDIN)
+	PR_Close(inFile);
     if (cms_verbose) {
 	fprintf(stderr, "received commands\n");
     }
@@ -1443,7 +1289,8 @@ main(int argc, char **argv)
 	     */
 	    SECU_FileToItem(&envmsg, encryptOptions.envFile);
 	    decodeOptions.options = &options;
-	    encryptOptions.envmsg = decode(NULL, &envmsg, &decodeOptions);
+	    encryptOptions.envmsg = decode(NULL, &dummy, &envmsg, 
+	                                   &decodeOptions);
 	    if (!encryptOptions.envmsg) {
 		SECU_PrintError(progName, "problem decoding env msg");
 		exitstatus = 1;
@@ -1453,18 +1300,12 @@ main(int argc, char **argv)
 	    decodeOptions.dkcb = dkcb;
 	    decodeOptions.bulkkey = encryptOptions.bulkkey;
 	}
-	if (!batch) {
-	    cmsg = decode(outFile, &input, &decodeOptions);
-	    if (!cmsg) {
-		SECU_PrintError(progName, "problem decoding");
-		exitstatus = 1;
-	    }
-	} else {
-	    exitstatus = doBatchDecode(outFile, inFile, &decodeOptions);
-	    if (inFile != PR_STDIN) {
-		PR_Close(inFile);
-	    }
+	cmsg = decode(outFile, &output, &input, &decodeOptions);
+	if (!cmsg) {
+	    SECU_PrintError(progName, "problem decoding");
+	    exitstatus = 1;
 	}
+	fwrite(output.data, output.len, 1, outFile);
 	break;
     case SIGN:         /* -S */
 	signOptions.options = &options;
@@ -1497,7 +1338,8 @@ main(int argc, char **argv)
 	} else {
 	    SECU_FileToItem(&envmsg, encryptOptions.envFile);
 	    decodeOptions.options = &options;
-	    encryptOptions.envmsg = decode(NULL, &envmsg, &decodeOptions);
+	    encryptOptions.envmsg = decode(NULL, &dummy, &envmsg, 
+	                                   &decodeOptions);
 	    if (encryptOptions.envmsg == NULL) {
 	    	SECU_PrintError(progName, "problem decrypting env msg");
 		exitstatus = 1;
@@ -1591,6 +1433,7 @@ main(int argc, char **argv)
 	if (cms_verbose) {
 	    fprintf(stderr, "encoding passed\n");
 	}
+	/*PR_Write(output.data, output.len);*/
 	fwrite(output.data, output.len, 1, outFile);
 	if (cms_verbose) {
 	    fprintf(stderr, "wrote to file\n");
@@ -1602,13 +1445,10 @@ main(int argc, char **argv)
     if (outFile != stdout)
 	fclose(outFile);
 
-    SECITEM_FreeItem(&decodeOptions.content, PR_FALSE);
-    SECITEM_FreeItem(&envmsg, PR_FALSE);
-    SECITEM_FreeItem(&input, PR_FALSE);
+    if (decodeOptions.contentFile)
+	PR_Close(decodeOptions.contentFile);
     if (NSS_Shutdown() != SECSuccess) {
-	SECU_PrintError(progName, "NSS_Shutdown failed");
-	exitstatus = 1;
+	exit(1);
     }
-    PR_Cleanup();
-    return exitstatus;
+    exit(exitstatus);
 }
