@@ -34,7 +34,7 @@
 /*
  * CMS attributes.
  *
- * $Id: cmsattr.c,v 1.5.94.1 2004/01/23 18:56:54 mkaply%us.ibm.com Exp $
+ * $Id: cmsattr.c,v 1.6 2003/09/19 04:14:50 jpierre%netscape.com Exp $
  */
 
 #include "cmslocal.h"
@@ -86,11 +86,13 @@ NSS_CMSAttribute_Create(PRArenaPool *poolp, SECOidTag oidtag, SECItem *value, PR
 	goto loser;
 
     if (value != NULL) {
-	if ((copiedvalue = SECITEM_ArenaDupItem(poolp, value)) == NULL)
+	if ((copiedvalue = SECITEM_AllocItem(poolp, NULL, value->len)) == NULL)
 	    goto loser;
 
-	if (NSS_CMSArray_Add(poolp, (void ***)&(attr->values), (void *)copiedvalue) != SECSuccess)
+	if (SECITEM_CopyItem(poolp, copiedvalue, value) != SECSuccess)
 	    goto loser;
+
+	NSS_CMSArray_Add(poolp, (void ***)&(attr->values), (void *)copiedvalue);
     }
 
     attr->encoded = encoded;
@@ -111,22 +113,18 @@ loser:
 SECStatus
 NSS_CMSAttribute_AddValue(PLArenaPool *poolp, NSSCMSAttribute *attr, SECItem *value)
 {
-    SECItem *copiedvalue;
+    SECItem copiedvalue;
     void *mark;
 
     PORT_Assert (poolp != NULL);
 
     mark = PORT_ArenaMark(poolp);
 
-    if (value == NULL) {
-	PORT_SetError(SEC_ERROR_INVALID_ARGS);
-	goto loser;
-    }
-
-    if ((copiedvalue = SECITEM_ArenaDupItem(poolp, value)) == NULL)
+    /* XXX we need an object memory model #$%#$%! */
+    if (SECITEM_CopyItem(poolp, &copiedvalue, value) != SECSuccess)
 	goto loser;
 
-    if (NSS_CMSArray_Add(poolp, (void ***)&(attr->values), (void *)copiedvalue) != SECSuccess)
+    if (NSS_CMSArray_Add(poolp, (void ***)&(attr->values), (void *)&copiedvalue) != SECSuccess)
 	goto loser;
 
     PORT_ArenaUnmark(poolp, mark);
@@ -219,12 +217,9 @@ cms_attr_choose_attr_value_template(void *src_or_dest, PRBool encoding)
 
     attribute = (NSSCMSAttribute *)src_or_dest;
 
-    if (encoding && (!attribute->values || !attribute->values[0] ||
-        attribute->encoded)) {
-        /* we're encoding, and the attribute has no value or the attribute
-         * value is already encoded. */
-        return SEC_ASN1_GET(SEC_AnyTemplate);
-    }
+    if (encoding && attribute->encoded)
+	/* we're encoding, and the attribute value is already encoded. */
+	return SEC_ASN1_GET(SEC_AnyTemplate);
 
     /* get attribute's typeTag */
     oiddata = attribute->typeTag;
