@@ -34,7 +34,7 @@
 /*
  * CMS signerInfo methods.
  *
- * $Id: cmssiginfo.c,v 1.17.6.1 2004/01/23 18:56:54 mkaply%us.ibm.com Exp $
+ * $Id: cmssiginfo.c,v 1.19 2003/11/15 00:09:51 nelsonb%netscape.com Exp $
  */
 
 #include "cmslocal.h"
@@ -309,7 +309,7 @@ NSS_CMSSignerInfo_VerifyCertificate(NSSCMSSignerInfo *signerinfo, CERTCertDBHand
      * email profile.
      */
     if (NSS_CMSSignerInfo_GetSigningTime (signerinfo, &stime) != SECSuccess)
-	stime = PR_Now(); /* not found or conversion failed, so check against now */
+	stime = PR_Now();	/* not found or conversion failed, so check against now */
     
     /*
      * XXX  This uses the signing time, if available.  Additionally, we
@@ -319,8 +319,7 @@ NSS_CMSSignerInfo_VerifyCertificate(NSSCMSSignerInfo *signerinfo, CERTCertDBHand
      * in a time (and for non-S/MIME callers to pass in nothing, or
      * maybe make them pass in the current time, always?).
      */
-    if (CERT_VerifyCert(certdb, cert, PR_TRUE, certusage, stime, 
-                        signerinfo->cmsg->pwfn_arg, NULL) != SECSuccess) {
+    if (CERT_VerifyCert(certdb, cert, PR_TRUE, certusage, stime, signerinfo->cmsg->pwfn_arg, NULL) != SECSuccess) {
 	signerinfo->verificationStatus = NSSCMSVS_SigningCertNotTrusted;
 	return SECFailure;
     }
@@ -330,13 +329,11 @@ NSS_CMSSignerInfo_VerifyCertificate(NSSCMSSignerInfo *signerinfo, CERTCertDBHand
 /*
  * NSS_CMSSignerInfo_Verify - verify the signature of a single SignerInfo
  *
- * Just verifies the signature. The assumption is that verification of 
- * the certificate is done already.
+ * Just verifies the signature. The assumption is that verification of the certificate
+ * is done already.
  */
 SECStatus
-NSS_CMSSignerInfo_Verify(NSSCMSSignerInfo *signerinfo, 
-                         SECItem *digest,               /* may be NULL */
-                         SECItem *contentType)          /* may be NULL */
+NSS_CMSSignerInfo_Verify(NSSCMSSignerInfo *signerinfo, SECItem *digest, SECItem *contentType)
 {
     SECKEYPublicKey *publickey = NULL;
     NSSCMSAttribute *attr;
@@ -348,11 +345,9 @@ NSS_CMSSignerInfo_Verify(NSSCMSSignerInfo *signerinfo,
     if (signerinfo == NULL)
 	return SECFailure;
 
-    /* NSS_CMSSignerInfo_GetSigningCertificate will fail if 2nd parm is NULL 
-    ** and cert has not been verified 
-    */
-    cert = NSS_CMSSignerInfo_GetSigningCertificate(signerinfo, NULL);
-    if (cert == NULL) {
+    /* NSS_CMSSignerInfo_GetSigningCertificate will fail if 2nd parm is NULL and */
+    /* cert has not been verified */
+    if ((cert = NSS_CMSSignerInfo_GetSigningCertificate(signerinfo, NULL)) == NULL) {
 	vs = NSSCMSVS_SigningCertNotFound;
 	goto loser;
     }
@@ -395,9 +390,9 @@ NSS_CMSSignerInfo_Verify(NSSCMSSignerInfo *signerinfo,
 	     * be one for message digest which matches our message digest.
 	     * So check these things first.
 	     */
-	    attr = NSS_CMSAttributeArray_FindAttrByOidTag(signerinfo->authAttr,
-					SEC_OID_PKCS9_CONTENT_TYPE, PR_TRUE);
-	    if (attr == NULL) {
+	    if ((attr = NSS_CMSAttributeArray_FindAttrByOidTag(signerinfo->authAttr,
+					SEC_OID_PKCS9_CONTENT_TYPE, PR_TRUE)) == NULL)
+	    {
 		vs = NSSCMSVS_MalformedSignature;
 		goto loser;
 	    }
@@ -411,14 +406,12 @@ NSS_CMSSignerInfo_Verify(NSSCMSSignerInfo *signerinfo,
 	/*
 	 * Check digest
 	 */
-	attr = NSS_CMSAttributeArray_FindAttrByOidTag(signerinfo->authAttr, 
-	                              SEC_OID_PKCS9_MESSAGE_DIGEST, PR_TRUE);
-	if (attr == NULL) {
+	if ((attr = NSS_CMSAttributeArray_FindAttrByOidTag(signerinfo->authAttr, SEC_OID_PKCS9_MESSAGE_DIGEST, PR_TRUE)) == NULL)
+	{
 	    vs = NSSCMSVS_MalformedSignature;
 	    goto loser;
 	}
-	if (!digest || 
-	    NSS_CMSAttribute_CompareValue(attr, digest) == PR_FALSE) {
+	if (NSS_CMSAttribute_CompareValue(attr, digest) == PR_FALSE) {
 	    vs = NSSCMSVS_DigestMismatch;
 	    goto loser;
 	}
@@ -433,15 +426,14 @@ NSS_CMSSignerInfo_Verify(NSSCMSSignerInfo *signerinfo,
 	 *
 	 * The signature is based on a digest of the DER-encoded authenticated
 	 * attributes.  So, first we encode and then we digest/verify.
-	 * we trust the decoder to have the attributes in the right (sorted) 
-	 * order
+	 * we trust the decoder to have the attributes in the right (sorted) order
 	 */
 	encoded_attrs.data = NULL;
 	encoded_attrs.len = 0;
 
-	if (NSS_CMSAttributeArray_Encode(poolp, &(signerinfo->authAttr), 
-	                                 &encoded_attrs) == NULL ||
-		encoded_attrs.data == NULL || encoded_attrs.len == 0) {
+	if (NSS_CMSAttributeArray_Encode(poolp, &(signerinfo->authAttr), &encoded_attrs) == NULL ||
+		encoded_attrs.data == NULL || encoded_attrs.len == 0)
+	{
 	    vs = NSSCMSVS_ProcessingError;
 	    goto loser;
 	}
@@ -449,26 +441,21 @@ NSS_CMSSignerInfo_Verify(NSSCMSSignerInfo *signerinfo,
 	vs = (VFY_VerifyData (encoded_attrs.data, encoded_attrs.len,
 			publickey, &(signerinfo->encDigest),
 			SECOID_GetAlgorithmTag(&(signerinfo->digestEncAlg)),
-			signerinfo->cmsg->pwfn_arg) != SECSuccess) 
-			? NSSCMSVS_BadSignature : NSSCMSVS_GoodSignature;
+			signerinfo->cmsg->pwfn_arg) != SECSuccess) ? NSSCMSVS_BadSignature : NSSCMSVS_GoodSignature;
 
-	PORT_FreeArena(poolp, PR_FALSE);  /* awkward memory management :-( */
+	PORT_FreeArena(poolp, PR_FALSE);	/* awkward memory management :-( */
 
     } else {
 	SECItem *sig;
 
-	/* No authenticated attributes. 
-	** The signature is based on the plain message digest. 
-	*/
+	/* No authenticated attributes. The signature is based on the plain message digest. */
 	sig = &(signerinfo->encDigest);
 	if (sig->len == 0)
 	    goto loser;
 
-	vs = (!digest || 
-	      VFY_VerifyDigest(digest, publickey, sig,
+	vs = (VFY_VerifyDigest(digest, publickey, sig,
 			SECOID_GetAlgorithmTag(&(signerinfo->digestEncAlg)),
-			signerinfo->cmsg->pwfn_arg) != SECSuccess) 
-			? NSSCMSVS_BadSignature : NSSCMSVS_GoodSignature;
+			signerinfo->cmsg->pwfn_arg) != SECSuccess) ? NSSCMSVS_BadSignature : NSSCMSVS_GoodSignature;
     }
 
     if (vs == NSSCMSVS_BadSignature) {
@@ -526,11 +513,6 @@ NSS_CMSSignerInfo_GetDigestAlgTag(NSSCMSSignerInfo *signerinfo)
 {
     SECOidData *algdata;
 
-    if (!signerinfo) {
-        PORT_SetError(SEC_ERROR_INVALID_ARGS);
-        return SEC_OID_UNKNOWN;
-    }
-
     algdata = SECOID_FindOID (&(signerinfo->digestAlg.algorithm));
     if (algdata != NULL)
 	return algdata->offset;
@@ -584,7 +566,7 @@ NSS_CMSSignerInfo_GetSigningTime(NSSCMSSignerInfo *sinfo, PRTime *stime)
     /* XXXX multi-valued attributes NIH */
     if (attr == NULL || (value = NSS_CMSAttribute_GetValue(attr)) == NULL)
 	return SECFailure;
-    if (DER_DecodeTimeChoice(stime, value) != SECSuccess)
+    if (CERT_DecodeTimeChoice(stime, value) != SECSuccess)
 	return SECFailure;
     sinfo->signingTime = *stime;	/* make cached copy */
     return SECSuccess;
@@ -638,7 +620,7 @@ NSS_CMSSignerInfo_GetSigningCertificate(NSSCMSSignerInfo *signerinfo, CERTCertDB
  *
  * sinfo - signerInfo data for this signer
  *
- * Returns a pointer to allocated memory, which must be freed with PORT_Free.
+ * Returns a pointer to allocated memory, which must be freed.
  * A return value of NULL is an error.
  */
 char *
@@ -721,7 +703,7 @@ NSS_CMSSignerInfo_AddSigningTime(NSSCMSSignerInfo *signerinfo, PRTime t)
     mark = PORT_ArenaMark(poolp);
 
     /* create new signing time attribute */
-    if (DER_EncodeTimeChoice(NULL, &stime, t) != SECSuccess)
+    if (CERT_EncodeTimeChoice(NULL, &stime, t) != SECSuccess)
 	goto loser;
 
     if ((attr = NSS_CMSAttribute_Create(poolp, SEC_OID_PKCS9_SIGNING_TIME, &stime, PR_FALSE)) == NULL) {
