@@ -234,13 +234,25 @@ CONST_OID cmsRC2wrap[]  			= { PKCS9_SMIME_ALGS, 7 };
 CONST_OID smimeEncryptionKeyPreference[] 	= { PKCS9_SMIME_ATTRS, 11 };
 CONST_OID ms_smimeEncryptionKeyPreference[] 	= { MICROSOFT_OID, 0x10, 0x4 };
 
-CONST_OID x520CommonName[]          		= { X520_ATTRIBUTE_TYPE, 3 };
-CONST_OID x520CountryName[]         		= { X520_ATTRIBUTE_TYPE, 6 };
-CONST_OID x520LocalityName[]        		= { X520_ATTRIBUTE_TYPE, 7 };
-CONST_OID x520StateOrProvinceName[] 		= { X520_ATTRIBUTE_TYPE, 8 };
-CONST_OID x520OrgName[]             		= { X520_ATTRIBUTE_TYPE, 10 };
-CONST_OID x520OrgUnitName[]         		= { X520_ATTRIBUTE_TYPE, 11 };
-CONST_OID x520DnQualifier[]         		= { X520_ATTRIBUTE_TYPE, 46 };
+CONST_OID x520CommonName[]                      = { X520_ATTRIBUTE_TYPE, 3 };
+CONST_OID x520SurName[]                         = { X520_ATTRIBUTE_TYPE, 4 };
+CONST_OID x520SerialNumber[]                    = { X520_ATTRIBUTE_TYPE, 5 };
+CONST_OID x520CountryName[]                     = { X520_ATTRIBUTE_TYPE, 6 };
+CONST_OID x520LocalityName[]                    = { X520_ATTRIBUTE_TYPE, 7 };
+CONST_OID x520StateOrProvinceName[]             = { X520_ATTRIBUTE_TYPE, 8 };
+CONST_OID x520StreetAddress[]                   = { X520_ATTRIBUTE_TYPE, 9 };
+CONST_OID x520OrgName[]                         = { X520_ATTRIBUTE_TYPE, 10 };
+CONST_OID x520OrgUnitName[]                     = { X520_ATTRIBUTE_TYPE, 11 };
+CONST_OID x520Title[]                           = { X520_ATTRIBUTE_TYPE, 12 };
+CONST_OID x520PostalAddress[]                   = { X520_ATTRIBUTE_TYPE, 16 };
+CONST_OID x520PostalCode[]                      = { X520_ATTRIBUTE_TYPE, 17 };
+CONST_OID x520PostOfficeBox[]                   = { X520_ATTRIBUTE_TYPE, 18 };
+CONST_OID x520GivenName[]                       = { X520_ATTRIBUTE_TYPE, 42 };
+CONST_OID x520Initials[]                        = { X520_ATTRIBUTE_TYPE, 43 };
+CONST_OID x520GenerationQualifier[]             = { X520_ATTRIBUTE_TYPE, 44 };
+CONST_OID x520DnQualifier[]                     = { X520_ATTRIBUTE_TYPE, 46 };
+CONST_OID x520HouseIdentifier[]                 = { X520_ATTRIBUTE_TYPE, 51 };
+CONST_OID x520Pseudonym[]                       = { X520_ATTRIBUTE_TYPE, 65 };
 
 CONST_OID nsTypeGIF[]          			= { NETSCAPE_DATA_TYPE, 0x01 };
 CONST_OID nsTypeJPEG[]         			= { NETSCAPE_DATA_TYPE, 0x02 };
@@ -466,6 +478,48 @@ CONST_OID secgECsecp256k1[] = { SECG_OID, 0x0a };
 CONST_OID secgECsecp384r1[] = { SECG_OID, 0x22 };
 CONST_OID secgECsecp521r1[] = { SECG_OID, 0x23 };
 
+/* ANSI X9.62 characteristic two curve OIDs */
+CONST_OID ansiX962c2pnb163v1[] = { ANSI_X962_GF2m_OID, 0x01 };
+CONST_OID ansiX962c2pnb163v2[] = { ANSI_X962_GF2m_OID, 0x02 };
+CONST_OID ansiX962c2pnb163v3[] = { ANSI_X962_GF2m_OID, 0x03 };
+CONST_OID ansiX962c2pnb176v1[] = { ANSI_X962_GF2m_OID, 0x04 };
+CONST_OID ansiX962c2tnb191v1[] = { ANSI_X962_GF2m_OID, 0x05 };
+CONST_OID ansiX962c2tnb191v2[] = { ANSI_X962_GF2m_OID, 0x06 };
+CONST_OID ansiX962c2tnb191v3[] = { ANSI_X962_GF2m_OID, 0x07 };
+CONST_OID ansiX962c2onb191v4[] = { ANSI_X962_GF2m_OID, 0x08 };
+CONST_OID ansiX962c2onb191v5[] = { ANSI_X962_GF2m_OID, 0x09 };
+CONST_OID ansiX962c2pnb208w1[] = { ANSI_X962_GF2m_OID, 0x0a };
+CONST_OID ansiX962c2tnb239v1[] = { ANSI_X962_GF2m_OID, 0x0b };
+CONST_OID ansiX962c2tnb239v2[] = { ANSI_X962_GF2m_OID, 0x0c };
+CONST_OID ansiX962c2tnb239v3[] = { ANSI_X962_GF2m_OID, 0x0d };
+CONST_OID ansiX962c2onb239v4[] = { ANSI_X962_GF2m_OID, 0x0e };
+CONST_OID ansiX962c2onb239v5[] = { ANSI_X962_GF2m_OID, 0x0f };
+CONST_OID ansiX962c2pnb272w1[] = { ANSI_X962_GF2m_OID, 0x10 };
+CONST_OID ansiX962c2pnb304w1[] = { ANSI_X962_GF2m_OID, 0x11 };
+CONST_OID ansiX962c2tnb359v1[] = { ANSI_X962_GF2m_OID, 0x12 };
+CONST_OID ansiX962c2pnb368w1[] = { ANSI_X962_GF2m_OID, 0x13 };
+CONST_OID ansiX962c2tnb431r1[] = { ANSI_X962_GF2m_OID, 0x14 };
+
+/* SECG characterisitic two curve OIDs */
+CONST_OID secgECsect113r1[] = {SECG_OID, 0x04 };
+CONST_OID secgECsect113r2[] = {SECG_OID, 0x05 };
+CONST_OID secgECsect131r1[] = {SECG_OID, 0x16 };
+CONST_OID secgECsect131r2[] = {SECG_OID, 0x17 };
+CONST_OID secgECsect163k1[] = {SECG_OID, 0x01 };
+CONST_OID secgECsect163r1[] = {SECG_OID, 0x02 };
+CONST_OID secgECsect163r2[] = {SECG_OID, 0x0f };
+CONST_OID secgECsect193r1[] = {SECG_OID, 0x18 };
+CONST_OID secgECsect193r2[] = {SECG_OID, 0x19 };
+CONST_OID secgECsect233k1[] = {SECG_OID, 0x1a };
+CONST_OID secgECsect233r1[] = {SECG_OID, 0x1b };
+CONST_OID secgECsect239k1[] = {SECG_OID, 0x03 };
+CONST_OID secgECsect283k1[] = {SECG_OID, 0x10 };
+CONST_OID secgECsect283r1[] = {SECG_OID, 0x11 };
+CONST_OID secgECsect409k1[] = {SECG_OID, 0x24 };
+CONST_OID secgECsect409r1[] = {SECG_OID, 0x25 };
+CONST_OID secgECsect571k1[] = {SECG_OID, 0x26 };
+CONST_OID secgECsect571r1[] = {SECG_OID, 0x27 };
+
 #define OI(x) { siDEROID, (unsigned char *)x, sizeof x }
 #ifndef SECOID_NO_STRINGS
 #define OD(oid,tag,desc,mech,ext) { OI(oid), tag, desc, mech, ext }
@@ -1181,10 +1235,194 @@ const static SECOidData oids[] = {
 	"SECG elliptic curve secp521r1 (aka NIST P-521)", 
 	CKM_INVALID_MECHANISM,
 	INVALID_CERT_EXTENSION ),
+
+    /* ANSI X9.62 named elliptic curves (characteristic two field) */
+    OD( ansiX962c2pnb163v1, SEC_OID_ANSIX962_EC_C2PNB163V1,
+	"ANSI X9.62 elliptic curve c2pnb163v1", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( ansiX962c2pnb163v2, SEC_OID_ANSIX962_EC_C2PNB163V2,
+	"ANSI X9.62 elliptic curve c2pnb163v2", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( ansiX962c2pnb163v3, SEC_OID_ANSIX962_EC_C2PNB163V3,
+	"ANSI X9.62 elliptic curve c2pnb163v3", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( ansiX962c2pnb176v1, SEC_OID_ANSIX962_EC_C2PNB176V1,
+	"ANSI X9.62 elliptic curve c2pnb176v1", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( ansiX962c2tnb191v1, SEC_OID_ANSIX962_EC_C2TNB191V1,
+	"ANSI X9.62 elliptic curve c2tnb191v1", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( ansiX962c2tnb191v2, SEC_OID_ANSIX962_EC_C2TNB191V2,
+	"ANSI X9.62 elliptic curve c2tnb191v2", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( ansiX962c2tnb191v3, SEC_OID_ANSIX962_EC_C2TNB191V3,
+	"ANSI X9.62 elliptic curve c2tnb191v3", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( ansiX962c2onb191v4, SEC_OID_ANSIX962_EC_C2ONB191V4,
+	"ANSI X9.62 elliptic curve c2onb191v4", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( ansiX962c2onb191v5, SEC_OID_ANSIX962_EC_C2ONB191V5,
+	"ANSI X9.62 elliptic curve c2onb191v5", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( ansiX962c2pnb208w1, SEC_OID_ANSIX962_EC_C2PNB208W1,
+	"ANSI X9.62 elliptic curve c2pnb208w1", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( ansiX962c2tnb239v1, SEC_OID_ANSIX962_EC_C2TNB239V1,
+	"ANSI X9.62 elliptic curve c2tnb239v1", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( ansiX962c2tnb239v2, SEC_OID_ANSIX962_EC_C2TNB239V2,
+	"ANSI X9.62 elliptic curve c2tnb239v2", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( ansiX962c2tnb239v3, SEC_OID_ANSIX962_EC_C2TNB239V3,
+	"ANSI X9.62 elliptic curve c2tnb239v3", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( ansiX962c2onb239v4, SEC_OID_ANSIX962_EC_C2ONB239V4,
+	"ANSI X9.62 elliptic curve c2onb239v4", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( ansiX962c2onb239v5, SEC_OID_ANSIX962_EC_C2ONB239V5,
+	"ANSI X9.62 elliptic curve c2onb239v5", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( ansiX962c2pnb272w1, SEC_OID_ANSIX962_EC_C2PNB272W1,
+	"ANSI X9.62 elliptic curve c2pnb272w1", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( ansiX962c2pnb304w1, SEC_OID_ANSIX962_EC_C2PNB304W1,
+	"ANSI X9.62 elliptic curve c2pnb304w1", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( ansiX962c2tnb359v1, SEC_OID_ANSIX962_EC_C2TNB359V1,
+	"ANSI X9.62 elliptic curve c2tnb359v1", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( ansiX962c2pnb368w1, SEC_OID_ANSIX962_EC_C2PNB368W1,
+	"ANSI X9.62 elliptic curve c2pnb368w1", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( ansiX962c2tnb431r1, SEC_OID_ANSIX962_EC_C2TNB431R1,
+	"ANSI X9.62 elliptic curve c2tnb431r1", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+
+    /* SECG named elliptic curves (characterisitic two field) */
+    OD( secgECsect113r1, SEC_OID_SECG_EC_SECT113R1,
+	"SECG elliptic curve sect113r1", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( secgECsect113r2, SEC_OID_SECG_EC_SECT113R2,
+	"SECG elliptic curve sect113r2", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( secgECsect131r1, SEC_OID_SECG_EC_SECT131R1,
+	"SECG elliptic curve sect131r1", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( secgECsect131r2, SEC_OID_SECG_EC_SECT131R2,
+	"SECG elliptic curve sect131r2", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( secgECsect163k1, SEC_OID_SECG_EC_SECT163K1,
+	"SECG elliptic curve sect163k1 (aka NIST K-163)", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( secgECsect163r1, SEC_OID_SECG_EC_SECT163R1,
+	"SECG elliptic curve sect163r1", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( secgECsect163r2, SEC_OID_SECG_EC_SECT163R2,
+	"SECG elliptic curve sect163r2 (aka NIST B-163)", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( secgECsect193r1, SEC_OID_SECG_EC_SECT193R1,
+	"SECG elliptic curve sect193r1", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( secgECsect193r2, SEC_OID_SECG_EC_SECT193R2,
+	"SECG elliptic curve sect193r2", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( secgECsect233k1, SEC_OID_SECG_EC_SECT233K1,
+	"SECG elliptic curve sect233k1 (aka NIST K-233)", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( secgECsect233r1, SEC_OID_SECG_EC_SECT233R1,
+	"SECG elliptic curve sect233r1 (aka NIST B-233)", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( secgECsect239k1, SEC_OID_SECG_EC_SECT239K1,
+	"SECG elliptic curve sect239k1", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( secgECsect283k1, SEC_OID_SECG_EC_SECT283K1,
+	"SECG elliptic curve sect283k1 (aka NIST K-283)", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( secgECsect283r1, SEC_OID_SECG_EC_SECT283R1,
+	"SECG elliptic curve sect283r1 (aka NIST B-283)", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( secgECsect409k1, SEC_OID_SECG_EC_SECT409K1,
+	"SECG elliptic curve sect409k1 (aka NIST K-409)", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( secgECsect409r1, SEC_OID_SECG_EC_SECT409R1,
+	"SECG elliptic curve sect409r1 (aka NIST B-409)", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( secgECsect571k1, SEC_OID_SECG_EC_SECT571K1,
+	"SECG elliptic curve sect571k1 (aka NIST K-571)", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+    OD( secgECsect571r1, SEC_OID_SECG_EC_SECT571R1,
+	"SECG elliptic curve sect571r1 (aka NIST B-571)", 
+	CKM_INVALID_MECHANISM,
+	INVALID_CERT_EXTENSION ),
+
     OD( netscapeAOLScreenname, SEC_OID_NETSCAPE_AOLSCREENNAME,
 	"AOL Screenname", CKM_INVALID_MECHANISM,
 	INVALID_CERT_EXTENSION ),
 
+    OD( x520SurName, SEC_OID_AVA_SURNAME,
+    	"X520 Title",         CKM_INVALID_MECHANISM, INVALID_CERT_EXTENSION ),
+    OD( x520SerialNumber, SEC_OID_AVA_SERIAL_NUMBER,
+        "X520 Serial Number", CKM_INVALID_MECHANISM, INVALID_CERT_EXTENSION ),
+    OD( x520StreetAddress, SEC_OID_AVA_STREET_ADDRESS,
+        "X520 Street Address", CKM_INVALID_MECHANISM, INVALID_CERT_EXTENSION ),
+    OD( x520Title, SEC_OID_AVA_TITLE, 
+    	"X520 Title",         CKM_INVALID_MECHANISM, INVALID_CERT_EXTENSION ),
+    OD( x520PostalAddress, SEC_OID_AVA_POSTAL_ADDRESS,
+    	"X520 Postal Address", CKM_INVALID_MECHANISM, INVALID_CERT_EXTENSION ),
+    OD( x520PostalCode, SEC_OID_AVA_POSTAL_CODE,
+    	"X520 Postal Code",   CKM_INVALID_MECHANISM, INVALID_CERT_EXTENSION ),
+    OD( x520PostOfficeBox, SEC_OID_AVA_POST_OFFICE_BOX,
+    	"X520 Post Office Box", CKM_INVALID_MECHANISM, INVALID_CERT_EXTENSION ),
+    OD( x520GivenName, SEC_OID_AVA_GIVEN_NAME,
+    	"X520 Given Name",    CKM_INVALID_MECHANISM, INVALID_CERT_EXTENSION ),
+    OD( x520Initials, SEC_OID_AVA_INITIALS,
+    	"X520 Initials",      CKM_INVALID_MECHANISM, INVALID_CERT_EXTENSION ),
+    OD( x520GenerationQualifier, SEC_OID_AVA_GENERATION_QUALIFIER,
+    	"X520 Generation Qualifier", 
+	CKM_INVALID_MECHANISM, INVALID_CERT_EXTENSION ),
+    OD( x520HouseIdentifier, SEC_OID_AVA_HOUSE_IDENTIFIER,
+    	"X520 House Identifier", 
+	CKM_INVALID_MECHANISM, INVALID_CERT_EXTENSION ),
+    OD( x520Pseudonym, SEC_OID_AVA_PSEUDONYM,
+    	"X520 Pseudonym",     CKM_INVALID_MECHANISM, INVALID_CERT_EXTENSION ),
+
 };
 
 /*
@@ -1392,7 +1630,6 @@ SECOidData *
 SECOID_FindOIDByMechanism(unsigned long mechanism)
 {
     SECOidData *ret;
-    int rv;
 
     PR_ASSERT(oidhash != NULL);
 
@@ -1408,15 +1645,14 @@ SECOidData *
 SECOID_FindOID(SECItem *oid)
 {
     SECOidData *ret;
-    int rv;
-    
+
     PR_ASSERT(oidhash != NULL);
     
     ret = PL_HashTableLookupConst ( oidhash, oid );
     if ( ret == NULL ) {
 	ret  = secoid_FindDynamic(oid);
 	if (ret == NULL) {
-	    PORT_SetError(SEC_ERROR_LIBRARY_FAILURE);
+	    PORT_SetError(SEC_ERROR_UNRECOGNIZED_OID);
 	}
     }
 
