@@ -107,12 +107,13 @@ struct nsRequestInfo : public PLDHashEntryHdr
 };
 
 
-PR_STATIC_CALLBACK(void)
+PR_STATIC_CALLBACK(PRBool)
 RequestInfoHashInitEntry(PLDHashTable *table, PLDHashEntryHdr *entry,
                          const void *key)
 {
   // Initialize the entry with placement new
   new (entry) nsRequestInfo(key);
+  return PR_TRUE;
 }
 
 
@@ -417,30 +418,24 @@ NS_IMETHODIMP
 nsDocLoaderImpl::GetContentViewerContainer(nsISupports* aDocumentID,
                                            nsIContentViewerContainer** aResult)
 {
-  nsISupports* base = aDocumentID;
-  nsIDocument* doc;
-  nsresult rv;
+  nsCOMPtr<nsIDocument> doc(do_QueryInterface(aDocumentID));
 
-  rv = base->QueryInterface(kIDocumentIID, (void**)&doc);
-  if (NS_SUCCEEDED(rv)) {
-    nsCOMPtr<nsIPresShell> pres;
-    doc->GetShellAt(0, getter_AddRefs(pres));
-    if (nsnull != pres) {
-      nsIPresContext* presContext;
-      rv = pres->GetPresContext(&presContext);
-      if (NS_SUCCEEDED(rv) && nsnull != presContext) {
-        nsISupports* supp;
-        rv = presContext->GetContainer(&supp);
-        if (NS_SUCCEEDED(rv) && nsnull != supp) {          
-          rv = supp->QueryInterface(kIContentViewerContainerIID, (void**)aResult);          
-          NS_RELEASE(supp);
+  if (doc) {
+    nsIPresShell *pres = doc->GetShellAt(0);
+    if (pres) {
+      nsCOMPtr<nsIPresContext> presContext;
+      pres->GetPresContext(getter_AddRefs(presContext));
+      if (presContext) {
+        nsCOMPtr<nsISupports> supp;
+        presContext->GetContainer(getter_AddRefs(supp));
+        if (supp) {
+          return CallQueryInterface(supp, aResult);          
         }
-        NS_RELEASE(presContext);
       }
     }
-    NS_RELEASE(doc);
   }
-  return rv;
+
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -889,7 +884,7 @@ nsDocLoaderImpl::AddProgressListener(nsIWebProgressListener *aListener,
                                      PRUint32 aNotifyMask)
 {
   nsresult rv;
-  nsWeakPtr listener = getter_AddRefs(NS_GetWeakReference(aListener));
+  nsWeakPtr listener = do_GetWeakReference(aListener);
   if (!listener) {
     return NS_ERROR_INVALID_ARG;
   }
@@ -913,7 +908,7 @@ NS_IMETHODIMP
 nsDocLoaderImpl::RemoveProgressListener(nsIWebProgressListener *aListener)
 {
   nsresult rv;
-  nsWeakPtr listener = getter_AddRefs(NS_GetWeakReference(aListener));
+  nsWeakPtr listener = do_GetWeakReference(aListener);
   if (!listener) {
     return NS_ERROR_INVALID_ARG;
   }
