@@ -54,9 +54,7 @@ public:
 	// override nsIExternalHelperAppService methods....
 	NS_IMETHOD LaunchAppWithTempFile(nsIMIMEInfo *aMIMEInfo, nsIFile * aTempFile);
 
-	// method overrides for mime.types and mime.info look up steps
-	NS_IMETHODIMP GetFromExtension(const char *aFileExt, nsIMIMEInfo **_retval);
-	NS_IMETHODIMP GetFromMIMEType(const char *aMIMEType, nsIMIMEInfo **_retval);
+	already_AddRefed<nsIMIMEInfo> GetMIMEInfoFromOS(const char *aMIMEType, const char * aFileExt);
 
 	// override nsIExternalProtocolService methods
 	NS_IMETHOD ExternalProtocolHandlerExists(const char * aProtocolScheme, PRBool * aHandlerExists);
@@ -70,6 +68,8 @@ public:
 
 protected:
 	nsresult SetMIMEInfoForType(const char *aMIMEType, nsIMIMEInfo **_retval);
+	nsresult GetMimeInfoFromExtension(const char *aFileExt, nsIMIMEInfo **_retval);
+	nsresult GetMimeInfoFromMIMEType(const char *aMIMEType, nsIMIMEInfo **_retval);
 };
 
 #endif // nsOSHelperAppService_h__
