@@ -30,6 +30,8 @@
 #include "nsILocalFile.h"
 #include "nsIProcess.h"
 #include "plstr.h"
+#include "nsAutoPtr.h"
+#include "nsNativeCharsetUtils.h"
 
 // we need windows.h to read out registry information...
 #include <windows.h>
@@ -37,13 +39,30 @@
 // shellapi.h is needed to build with WIN32_LEAN_AND_MEAN
 #include <shellapi.h>
 
+#define LOG(args) PR_LOG(mLog, PR_LOG_DEBUG, args)
+
 // helper methods: forward declarations...
-BYTE * GetValueBytes( HKEY hKey, const char *pValueName, DWORD *pLen=0);
-nsresult GetExtensionFrom4xRegistryInfo(const char * aMimeType, nsCString& aFileExtension);
-nsresult GetExtensionFromWindowsMimeDatabase(const char * aMimeType, nsCString& aFileExtension);
+static BYTE * GetValueBytes( HKEY hKey, const char *pValueName, DWORD *pLen=0);
+static nsresult GetExtensionFrom4xRegistryInfo(const char * aMimeType, nsCString& aFileExtension);
+static nsresult GetExtensionFromWindowsMimeDatabase(const char * aMimeType, nsCString& aFileExtension);
+
+// static member
+PRBool nsOSHelperAppService::mIsNT = PR_FALSE;
 
 nsOSHelperAppService::nsOSHelperAppService() : nsExternalHelperAppService()
-{}
+{
+  OSVERSIONINFO osversion;
+  ::ZeroMemory(&osversion, sizeof(OSVERSIONINFO));
+  osversion.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
+
+  if (!GetVersionEx(&osversion)) {
+    // If the call failed, better be safe and assume *W functions don't work
+    mIsNT = PR_FALSE;
+  }
+  else {
+    mIsNT = (osversion.dwPlatformId == VER_PLATFORM_WIN32_NT);
+  }
+}
 
 nsOSHelperAppService::~nsOSHelperAppService()
 {}
@@ -96,9 +115,8 @@ NS_IMETHODIMP nsOSHelperAppService::LaunchAppWithTempFile(nsIMIMEInfo * aMIMEInf
 
 // The windows registry provides a mime database key which lists a set of mime types and corresponding "Extension" values. 
 // we can use this to look up our mime type to see if there is a preferred extension for the mime type.
-nsresult GetExtensionFromWindowsMimeDatabase(const char * aMimeType, nsCString& aFileExtension)
+static nsresult GetExtensionFromWindowsMimeDatabase(const char * aMimeType, nsCString& aFileExtension)
 {
-   nsresult rv = NS_OK;
    HKEY hKey;
    nsCAutoString mimeDatabaseKey ("MIME\\Database\\Content Type\\");
 
@@ -113,20 +131,19 @@ nsresult GetExtensionFromWindowsMimeDatabase(const char * aMimeType, nsCString&
         aFileExtension = (char * )pBytes;
       }
 
-      delete pBytes;
+      delete[] pBytes;
 
       ::RegCloseKey(hKey);
    }
 
    return NS_OK;
-
 }
 
 // We have a serious problem!! I have this content type and the windows registry only gives me
 // helper apps based on extension. Right now, we really don't have a good place to go for 
 // trying to figure out the extension for a particular mime type....One short term hack is to look
 // this information in 4.x (it's stored in the windows regsitry). 
-nsresult GetExtensionFrom4xRegistryInfo(const char * aMimeType, nsCString& aFileExtension)
+static nsresult GetExtensionFrom4xRegistryInfo(const char * aMimeType, nsCString& aFileExtension)
 {
    nsCAutoString command ("Software\\Netscape\\Netscape Navigator\\Suffixes");
    nsresult rv = NS_OK;
@@ -182,6 +199,43 @@ static BYTE * GetValueBytes( HKEY hKey, const char *pValueName, DWORD *pLen)
   return( pBytes);
 }
 
+/* static */
+PRBool nsOSHelperAppService::GetValueString(HKEY hKey, PRUnichar* pValueName, nsAString& result)
+{
+  if (!mIsNT) {
+    nsCAutoString cValueName;
+    if (pValueName)
+      NS_CopyUnicodeToNative(nsDependentString(pValueName), cValueName);
+    char* pBytes = (char*)GetValueBytes(hKey, cValueName.get());
+    if (pBytes) {
+      nsresult rv = NS_CopyNativeToUnicode(nsDependentCString(pBytes), result);
+      delete[] pBytes;
+      return NS_SUCCEEDED(rv);
+    }
+    return PR_FALSE;
+  }
+
+  DWORD bufSz;
+  LONG err = ::RegQueryValueExW( hKey, pValueName, NULL, NULL, NULL, &bufSz); 
+  if (err == ERROR_SUCCESS) {
+    PRUnichar* pBytes = new PRUnichar[bufSz];
+    if (!pBytes)
+      return PR_FALSE;
+
+    err = ::RegQueryValueExW( hKey, pValueName, NULL, NULL, (BYTE*)pBytes, &bufSz);
+    if (err != ERROR_SUCCESS) {
+      delete [] pBytes;
+      return PR_FALSE;
+    } else {
+      result.Assign(pBytes);
+      delete [] pBytes;
+      return PR_TRUE;
+    }
+  }
+
+  return PR_FALSE;
+}
+
 NS_IMETHODIMP nsOSHelperAppService::ExternalProtocolHandlerExists(const char * aProtocolScheme, PRBool * aHandlerExists)
 {
   // look up the protocol scheme in the windows registry....if we find a match then we have a handler for it...
@@ -257,144 +311,218 @@ nsresult nsOSHelperAppService::GetFileTokenForPath(const PRUnichar * platformApp
 // to be prompted.
 //
 // This function sets only the Description attribute of the input nsIMIMEInfo.
-static nsresult GetMIMEInfoFromRegistry( LPBYTE fileType, nsIMIMEInfo *pInfo )
+/* static */
+nsresult nsOSHelperAppService::GetMIMEInfoFromRegistry( const nsAFlatString& fileType, nsIMIMEInfo *pInfo )
 {
     nsresult rv = NS_OK;
 
-    NS_ENSURE_ARG( fileType );
     NS_ENSURE_ARG( pInfo );
 
     // Get registry key for the pointed-to "file type."
     HKEY fileTypeKey = 0;
-    LONG rc = ::RegOpenKeyEx( HKEY_CLASSES_ROOT, (const char *)fileType, 0, KEY_QUERY_VALUE, &fileTypeKey );
-    if ( rc == ERROR_SUCCESS )
-    {
-        // OK, the default value here is the description of the type.
-        DWORD lenDesc = 0;
-        LPBYTE pDesc = GetValueBytes( fileTypeKey, "", &lenDesc );
-        if ( pDesc )
-        {
-            PRUnichar *pUniDesc = new PRUnichar[lenDesc+1];
-            if (pUniDesc) {
-              int len = MultiByteToWideChar(CP_ACP, 0, (const char *)pDesc, -1, pUniDesc, lenDesc); 
-              pUniDesc[len] = 0;
-              pInfo->SetDescription( pUniDesc );
-              delete [] pUniDesc;
-            }
-            delete [] pDesc;
-        }
-    }
-    else
-    {
-        rv = NS_ERROR_FAILURE;
+    LONG rc;
+    if (mIsNT) {
+      rc = ::RegOpenKeyExW( HKEY_CLASSES_ROOT, fileType.get(), 0, KEY_QUERY_VALUE, &fileTypeKey );
     }
+    else {
+      nsCAutoString ansiKey;
+      rv = NS_CopyUnicodeToNative(fileType, ansiKey);
+      if (NS_FAILED(rv))
+        return rv;
 
-    return rv;
+      rc = ::RegOpenKeyEx( HKEY_CLASSES_ROOT, ansiKey.get(), 0, KEY_QUERY_VALUE, &fileTypeKey );
+    }
+    if ( rc != ERROR_SUCCESS )
+        return NS_ERROR_FAILURE;
+
+    // OK, the default value here is the description of the type.
+    nsAutoString description;
+    PRBool found = GetValueString(fileTypeKey, NULL, description);
+    if (found)
+        pInfo->SetDescription(description.get());
+    ::RegCloseKey(fileTypeKey);
+
+    return NS_OK;
 }
 
 /////////////////////////////////////////////////////////////////////////////////////////////////
 // method overrides used to gather information from the windows registry for
 // various mime types. 
 ////////////////////////////////////////////////////////////////////////////////////////////////
-nsresult nsOSHelperAppService::GetMIMEInfoForExtensionFromOS(const char *aFileExt, nsIMIMEInfo **_retval) 
-{
-  if (!aFileExt || !*aFileExt) return NS_ERROR_FAILURE;
 
-  nsresult rv = NS_OK;
+/// Looks up the type for the extension aExt and compares it to aType
+static PRBool typeFromExtEquals(const char *aExt, const char *aType)
+{
+  if (!aType)
+    return PR_FALSE;
+  nsCAutoString fileExtToUse;
+  if (aExt[0] != '.')
+    fileExtToUse = '.';
+
+  fileExtToUse.Append(aExt);
+
+  HKEY hKey;
+  PRBool eq = PR_FALSE;
+  LONG err = ::RegOpenKeyEx( HKEY_CLASSES_ROOT, fileExtToUse.get(), 0, KEY_QUERY_VALUE, &hKey);
+  if (err == ERROR_SUCCESS)
+  {
+     LPBYTE pBytes = GetValueBytes(hKey, "Content Type");
+     if (pBytes)
+     {
+       eq = strcmp((const char *)pBytes, aType) == 0;
+       delete[] pBytes;
+     }
+     ::RegCloseKey(hKey);
+  }
+  return eq;
+}
+
+already_AddRefed<nsIMIMEInfo> nsOSHelperAppService::GetByExtension(const char *aFileExt, const char *aTypeHint)
+{
+  if (!aFileExt || !*aFileExt)
+    return nsnull;
 
   // windows registry assumes your file extension is going to include the '.'.
   // so make sure it's there...
   nsCAutoString fileExtToUse;
-  if (aFileExt && aFileExt[0] != '.')
+  if (*aFileExt != '.')
     fileExtToUse = '.';
 
   fileExtToUse.Append(aFileExt);
 
   // o.t. try to get an entry from the windows registry.
-   HKEY hKey;
-   LONG err = ::RegOpenKeyEx( HKEY_CLASSES_ROOT, fileExtToUse.get(), 0, KEY_QUERY_VALUE, &hKey);
-   if (err == ERROR_SUCCESS)
-   {
+  HKEY hKey;
+  LONG err = ::RegOpenKeyEx( HKEY_CLASSES_ROOT, fileExtToUse.get(), 0, KEY_QUERY_VALUE, &hKey);
+  if (err == ERROR_SUCCESS)
+  {
+    nsCAutoString typeToUse;
+    if (aTypeHint) {
+      typeToUse.Assign(aTypeHint);
+    }
+    else {
       LPBYTE pBytes = GetValueBytes( hKey, "Content Type");
-      LPBYTE pFileDescription = GetValueBytes(hKey, "");
-
-      nsCOMPtr<nsIMIMEInfo> mimeInfo (do_CreateInstance(NS_MIMEINFO_CONTRACTID));
-      if (mimeInfo && pBytes)
-      {
-        mimeInfo->SetMIMEType((char *) pBytes);
-        // if the file extension includes the '.' then we don't want to include that when we append
-        // it to the mime info object.
-        if (aFileExt && *aFileExt == '.' && (aFileExt+1))
-          mimeInfo->AppendExtension(aFileExt+1);
-        else
-          mimeInfo->AppendExtension(aFileExt);
-
-        mimeInfo->SetPreferredAction(nsIMIMEInfo::useSystemDefault);
-
-        nsAutoString description;
-        description.AssignWithConversion((char *) pFileDescription);
-
-        PRInt32 pos = description.FindChar('.');
-        if (pos > 0) 
-          description.Truncate(pos); 
-        // the format of the description usually looks like appname.version.something.
-        // for now, let's try to make it pretty and just show you the appname.
-
-        mimeInfo->SetDefaultDescription(description.get());
-
-        // Get other nsIMIMEInfo fields from registry, if possible.
-        if ( pFileDescription )
-        {
-            GetMIMEInfoFromRegistry( pFileDescription, mimeInfo );
-        }
-
-        // if we got here, then set our return variable
-        *_retval = mimeInfo;
-        NS_ADDREF(*_retval);
-      }
-      else
-        rv = NS_ERROR_FAILURE; // we failed to really find an entry in the registry
-
+      if (pBytes)
+        typeToUse.Assign((const char*)pBytes);
       delete [] pBytes;
-      delete [] pFileDescription;
+    }
+    nsAutoString description;
+    PRBool found = GetValueString(hKey, NULL, description);
 
-      // close the key
-     ::RegCloseKey(hKey);
-   }
+    nsIMIMEInfo* mimeInfo = nsnull;
+    CallCreateInstance(NS_MIMEINFO_CONTRACTID, &mimeInfo);
+    if (mimeInfo)
+    {
+      if (!typeToUse.IsEmpty())
+        mimeInfo->SetMIMEType(typeToUse.get());
+      // don't append the '.'
+      mimeInfo->AppendExtension(fileExtToUse.get() + 1);
 
-   // we failed to find a mime type.
-   if (!*_retval) rv = NS_ERROR_FAILURE;
-    
-   return rv;
+      mimeInfo->SetPreferredAction(nsIMIMEInfo::useSystemDefault);
+
+      nsAutoString visibleDesc(description);
+      PRInt32 pos = visibleDesc.FindChar('.');
+      if (pos > 0) 
+        visibleDesc.Truncate(pos); 
+      // the format of the description usually looks like appname.version.something.
+      // for now, let's try to make it pretty and just show you the appname.
+
+      mimeInfo->SetDefaultDescription(visibleDesc.get());
+
+      // Get other nsIMIMEInfo fields from registry, if possible.
+      if ( found )
+      {
+          GetMIMEInfoFromRegistry( description, mimeInfo );
+      }
+    }
+    else {
+      NS_IF_RELEASE(mimeInfo); // we failed to really find an entry in the registry
+    }
+
+    // close the key
+    ::RegCloseKey(hKey);
+
+    return mimeInfo;
+  }
+
+  // we failed to find a mime type.
+  return nsnull;
 }
 
-nsresult nsOSHelperAppService::GetMIMEInfoForMimeTypeFromOS(const char *aMIMEType, nsIMIMEInfo ** _retval) 
+already_AddRefed<nsIMIMEInfo> nsOSHelperAppService::GetMIMEInfoFromOS(const char *aMIMEType, const char *aFileExt, PRBool *aFound)
 {
-  if (PL_strcasecmp(aMIMEType, APPLICATION_OCTET_STREAM) == 0) {
-    /* XXX Gross hack to wallpaper over the most common Win32
-     * extension issues caused by the fix for bug 116938.  See bug
-     * 120327, comment 271 for why this is needed.  Not even sure we
-     * want to remove this once we have fixed all this stuff to work
-     * right; any info we get from the OS on this type is pretty much
-     * useless....
-     */
-    return NS_ERROR_FAILURE;
-  }
-  
-  // (1) try to use the windows mime database to see if there is a mapping to a file extension
-  // (2) try to see if we have some left over 4.x registry info we can peek at...
+  *aFound = PR_TRUE;
   nsCAutoString fileExtension;
-  GetExtensionFromWindowsMimeDatabase(aMIMEType, fileExtension);
-  if (fileExtension.IsEmpty())
-    GetExtensionFrom4xRegistryInfo(aMIMEType, fileExtension);
-
-  // now look up based on the file extension.
-  if (!fileExtension.IsEmpty()) {
-    nsresult rv = GetFromExtension(fileExtension.get(), _retval);
-    if (NS_SUCCEEDED(rv) && *_retval)
-      (*_retval)->SetMIMEType(aMIMEType);
-    return rv;
+  /* XXX The strcasecmp is a gross hack to wallpaper over the most common Win32
+   * extension issues caused by the fix for bug 116938.  See bug
+   * 120327, comment 271 for why this is needed.  Not even sure we
+   * want to remove this once we have fixed all this stuff to work
+   * right; any info we get from the OS on this type is pretty much
+   * useless....
+   * We'll do extension-based lookup for this type later in this function.
+   */
+  if (aMIMEType && *aMIMEType && PL_strcasecmp(aMIMEType, APPLICATION_OCTET_STREAM) != 0) {
+    // (1) try to use the windows mime database to see if there is a mapping to a file extension
+    // (2) try to see if we have some left over 4.x registry info we can peek at...
+    GetExtensionFromWindowsMimeDatabase(aMIMEType, fileExtension);
+    LOG(("Windows mime database: extension '%s'\n", fileExtension.get()));
+    if (fileExtension.IsEmpty()) {
+      GetExtensionFrom4xRegistryInfo(aMIMEType, fileExtension);
+      LOG(("4.x Registry: extension '%s'\n", fileExtension.get()));
+    }
   }
+  // If we found an extension for the type, do the lookup
+  nsIMIMEInfo* mi = nsnull;
+  if (!fileExtension.IsEmpty())
+    mi = GetByExtension(fileExtension.get(), aMIMEType).get();
+  LOG(("Extension lookup on '%s' found: 0x%p\n", fileExtension.get(), mi));
 
-  return NS_ERROR_FAILURE;
+  PRBool hasDefault = PR_FALSE;
+  if (mi) {
+    mi->GetHasDefaultHandler(&hasDefault);
+    // OK. We might have the case that |aFileExt| is a valid extension for the
+    // mimetype we were given. In that case, we do want to append aFileExt
+    // to the mimeinfo that we have. (E.g.: We are asked for video/mpeg and
+    // .mpg, but the primary extension for video/mpeg is .mpeg. But because
+    // .mpg is an extension for video/mpeg content, we want to append it)
+    if (aFileExt && *aFileExt && typeFromExtEquals(aFileExt, aMIMEType)) {
+      LOG(("Appending extension '%s' to mimeinfo, because its mimetype is '%s'\n",
+           aFileExt, aMIMEType));
+      PRBool extExist = PR_FALSE;
+      mi->ExtensionExists(aFileExt, &extExist);
+      if (!extExist)
+        mi->AppendExtension(aFileExt);
+    }
+  }
+  if (!mi || !hasDefault) {
+    nsCOMPtr<nsIMIMEInfo> miByExt = GetByExtension(aFileExt, aMIMEType);
+    LOG(("Ext. lookup for '%s' found 0x%p\n", aFileExt, miByExt.get()));
+    if (!miByExt && mi)
+      return mi;
+    if (miByExt && !mi) {
+      miByExt.swap(mi);
+      return mi;
+    }
+    if (!miByExt && !mi) {
+      *aFound = PR_FALSE;
+      CallCreateInstance(NS_MIMEINFO_CONTRACTID, &mi);
+      if (mi) {
+        if (aMIMEType && *aMIMEType)
+          mi->SetMIMEType(aMIMEType);
+        if (aFileExt && *aFileExt)
+          mi->AppendExtension(aFileExt);
+      }
+      
+      return mi;
+    }
+
+    // if we get here, mi has no default app. copy from extension lookup.
+    nsCOMPtr<nsIFile> defaultApp;
+    nsXPIDLString desc;
+    miByExt->GetDefaultApplicationHandler(getter_AddRefs(defaultApp));
+    miByExt->GetDefaultDescription(getter_Copies(desc));
+
+    mi->SetDefaultApplicationHandler(defaultApp);
+    mi->SetDefaultDescription(desc.get());
+  }
+  return mi;
 }
