@@ -186,10 +186,6 @@ public:
 
   NS_IMETHOD  GetDeviceContext(nsIDeviceContext *&aContext);
 
-  NS_IMETHOD  ShowQuality(PRBool aShow);
-  NS_IMETHOD  GetShowQuality(PRBool &aResult);
-  NS_IMETHOD  SetQuality(nsContentQuality aQuality);
-
   NS_IMETHOD  DisableRefresh(void);
   NS_IMETHOD  EnableRefresh(PRUint32 aUpdateFlags);
 
@@ -280,13 +276,14 @@ private:
                            nscoord aOriginX, nscoord aOriginY,
                            PRBool aInsideRealView, nsView *aRealView, const nsRect *aDamageRect,
                            nsView *aTopView, nscoord aX, nscoord aY,
-                           PRBool aPaintFloaters, PRBool aEventProcessing);
+                           PRBool aPaintFloats, PRBool aEventProcessing);
   PRBool AddToDisplayList(nsView *aView,
                           DisplayZTreeNode* &aParent, nsRect &aClipRect,
                           nsRect& aDirtyRect, PRUint32 aFlags, nscoord aAbsX, nscoord aAbsY,
                           PRBool aAssumeIntersection);
   void OptimizeDisplayList(nsAutoVoidArray* aDisplayList, const nsRegion& aDirtyRegion,
-                           nsRect& aFinalTransparentRect, nsRegion& aOpaqueRgn);
+                           nsRect& aFinalTransparentRect, nsRegion& aOpaqueRgn,
+                           PRBool aTreatUniformAsOpaque);
     // Remove redundant PUSH/POP_CLIP pairs.
   void ComputeViewOffset(nsView *aView, nsPoint *aOrigin);
 
