@@ -767,10 +767,8 @@ nsWebCrawler::FindURLsIn(nsIDocument* aDocument, nsIContent* aNode)
     else {
       aNode->GetAttr(kNameSpaceID_None, mSrcAttr, src);
     }
-    nsCOMPtr<nsIURI> docURL;
-    aDocument->GetDocumentURL(getter_AddRefs(docURL));
     nsresult rv;
-    rv = NS_MakeAbsoluteURI(absURLSpec, src, docURL);
+    rv = NS_MakeAbsoluteURI(absURLSpec, src, aDocument->GetDocumentURL());
     if (NS_OK == rv) {
       nsCOMPtr<nsIAtom> urlAtom = do_GetAtom(absURLSpec);
       if (0 == mVisited->Get(urlAtom)) {
@@ -802,18 +800,9 @@ nsWebCrawler::FindURLsIn(nsIDocument* aDocument, nsIContent* aNode)
     }
   }
 
-  PRBool canHaveKids;
-  aNode->CanContainChildren(canHaveKids);
-  if (canHaveKids) {
-    PRInt32 i, n;
-    aNode->ChildCount(n);
-    for (i = 0; i < n; i++) {
-      nsCOMPtr<nsIContent> kid;
-      aNode->ChildAt(i, getter_AddRefs(kid));
-      if (nsnull != kid) {
-        FindURLsIn(aDocument, kid);
-      }
-    }
+  PRUint32 i, n = aNode->GetChildCount();
+  for (i = 0; i < n; ++i) {
+    FindURLsIn(aDocument, aNode->GetChildAt(i));
   }
 }
 
@@ -833,8 +822,7 @@ nsWebCrawler::FindMoreURLs()
         nsCOMPtr<nsIDocument> doc;
         docv->GetDocument(getter_AddRefs(doc));
         if (doc) {
-          nsCOMPtr<nsIContent> root;
-          doc->GetRootContent(getter_AddRefs(root));
+          nsIContent *root = doc->GetRootContent();
           if (root) {
             FindURLsIn(doc, root);
           }
