@@ -81,14 +81,12 @@
 //#include "nsUnitConversion.h"
 //#include "nsIDeviceContext.h"
 
-static NS_DEFINE_IID(kXPBaseWindowCID, NS_XPBASE_WINDOW_CID);
 static NS_DEFINE_IID(kWebShellCID, NS_WEB_SHELL_CID);
 static NS_DEFINE_IID(kWindowCID, NS_WINDOW_CID);
 
 
 static NS_DEFINE_IID(kIXPBaseWindowIID, NS_IXPBASE_WINDOW_IID);
 static NS_DEFINE_IID(kISupportsIID, NS_ISUPPORTS_IID);
-static NS_DEFINE_IID(kIDOMDocumentIID, NS_IDOMDOCUMENT_IID);
 static NS_DEFINE_IID(kIFactoryIID, NS_IFACTORY_IID);
 static NS_DEFINE_IID(kIWebShellIID, NS_IWEB_SHELL_IID);
 static NS_DEFINE_IID(kIWebShellContainerIID, NS_IWEB_SHELL_CONTAINER_IID);
@@ -97,7 +95,6 @@ static NS_DEFINE_IID(kIWidgetIID, NS_IWIDGET_IID);
 
 static NS_DEFINE_IID(kIDOMMouseListenerIID,   NS_IDOMMOUSELISTENER_IID);
 static NS_DEFINE_IID(kIDOMEventReceiverIID,   NS_IDOMEVENTRECEIVER_IID);
-static NS_DEFINE_IID(kIDOMElementIID, NS_IDOMELEMENT_IID);
 static NS_DEFINE_IID(kIDOMHTMLDocumentIID, NS_IDOMHTMLDOCUMENT_IID);
 
 //----------------------------------------------------------------------
@@ -411,7 +408,7 @@ NS_IMETHODIMP nsXPBaseWindow::EndLoadURL(nsIWebShell* aShell, const PRUnichar* a
     nsCOMPtr<nsIDocument> doc;
     shell->GetDocument(getter_AddRefs(doc));
     if (doc) {
-      doc->GetRootContent(&mContentRoot);
+      NS_IF_ADDREF(mContentRoot = doc->GetRootContent());
       mDocIsLoaded = PR_TRUE;
       if (nsnull != mWindowListener) {
         mWindowListener->Initialize(this);
@@ -696,8 +693,8 @@ nsXPBaseWindowFactory::QueryInterface(const nsIID &aIID, void **aResult)
   return NS_OK;
 }
 
-NS_IMPL_ADDREF(nsXPBaseWindowFactory);
-NS_IMPL_RELEASE(nsXPBaseWindowFactory);
+NS_IMPL_ADDREF(nsXPBaseWindowFactory)
+NS_IMPL_RELEASE(nsXPBaseWindowFactory)
 
 //----------------------------------------------------------------------
 nsresult
