@@ -150,7 +150,7 @@ NS_METHOD
 nsMenuX::Create(nsISupports * aParent, const nsAString &aLabel, const nsAString &aAccessKey, 
                 nsIChangeManager* aManager, nsIWebShell* aShell, nsIContent* aNode )
 {
-  mWebShellWeakRef = getter_AddRefs(NS_GetWeakReference(aShell));
+  mWebShellWeakRef = do_GetWeakReference(aShell);
   mMenuContent = aNode;
 
   // register this menu to be notified when changes are made to our content object
@@ -341,7 +341,7 @@ NS_METHOD nsMenuX::AddMenu(nsIMenu * aMenu)
     ::DisableMenuItem(mMacMenuHandle, currItemIndex);	    
 
   MenuHandle childMenu;
-  if (aMenu->GetNativeData(&(void*)childMenu) == NS_OK)
+  if (aMenu->GetNativeData((void**)&childMenu) == NS_OK)
     ::SetMenuItemHierarchicalMenu((MenuHandle) mMacMenuHandle, currItemIndex, childMenu);
   
   return NS_OK;
@@ -575,11 +575,9 @@ nsEventStatus nsMenuX::MenuConstruct(
     return nsEventStatus_eIgnore;
       
   // Iterate over the kids
-  PRInt32 count;
-  menuPopup->ChildCount(count);
-  for ( PRInt32 i = 0; i < count; ++i ) {
-    nsCOMPtr<nsIContent> child;
-    menuPopup->ChildAt(i, getter_AddRefs(child));
+  PRUint32 count = menuPopup->GetChildCount();
+  for ( PRUint32 i = 0; i < count; ++i ) {
+    nsIContent *child = menuPopup->GetChildAt(i);
     if ( child ) {
       // depending on the type, create a menu item, separator, or submenu
       nsCOMPtr<nsIAtom> tag;
@@ -621,11 +619,9 @@ nsEventStatus nsMenuX::HelpMenuConstruct(
     return nsEventStatus_eIgnore;
       
   // Iterate over the kids
-  PRInt32 count;
-  menuPopup->ChildCount(count);
-  for ( PRInt32 i = 0; i < count; ++i ) {
-    nsCOMPtr<nsIContent> child;
-    menuPopup->ChildAt(i, getter_AddRefs(child));
+  PRUint32 count = menuPopup->GetChildCount();
+  for ( PRUint32 i = 0; i < count; ++i ) {
+    nsIContent *child = menuPopup->GetChildAt(i);
     if ( child ) {      
       // depending on the type, create a menu item, separator, or submenu
       nsCOMPtr<nsIAtom> tag;
@@ -871,11 +867,8 @@ void nsMenuX::LoadMenuItem( nsIMenu* inParentMenu, nsIContent* inMenuItemContent
     inMenuItemContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::key, keyValue);
 
     // Try to find the key node. Get the document so we can do |GetElementByID|
-    nsCOMPtr<nsIDocument> document;
-    inMenuItemContent->GetDocument(getter_AddRefs(document));
-    if ( !document ) 
-      return;
-    nsCOMPtr<nsIDOMDocument> domDocument = do_QueryInterface(document);
+    nsCOMPtr<nsIDOMDocument> domDocument =
+      do_QueryInterface(inMenuItemContent->GetDocument());
     if ( !domDocument )
       return;
   
@@ -1020,18 +1013,14 @@ nsMenuX::OnCreate()
   // a command attribute. If so, several apptributes must potentially
   // be updated.
   if (popupContent) {
-    nsCOMPtr<nsIDocument> doc;
-    popupContent->GetDocument(getter_AddRefs(doc));
-    nsCOMPtr<nsIDOMDocument> domDoc(do_QueryInterface(doc));
+    nsCOMPtr<nsIDOMDocument> domDoc(do_QueryInterface(popupContent->GetDocument()));
 
-    PRInt32 count;
-    popupContent->ChildCount(count);
-    for (PRInt32 i = 0; i < count; i++) {
-      nsCOMPtr<nsIContent> grandChild;
-      popupContent->ChildAt(i, getter_AddRefs(grandChild));
+    PRUint32 count = popupContent->GetChildCount();
+    for (PRUint32 i = 0; i < count; i++) {
+      nsIContent *grandChild = popupContent->GetChildAt(i);
       nsCOMPtr<nsIAtom> tag;
       grandChild->GetTag(getter_AddRefs(tag));
-      if (tag.get() == nsWidgetAtoms::menuitem) {
+      if (tag == nsWidgetAtoms::menuitem) {
         // See if we have a command attribute.
         nsAutoString command;
         grandChild->GetAttr(kNameSpaceID_None, nsWidgetAtoms::command, command);
@@ -1218,17 +1207,14 @@ nsMenuX::GetMenuPopupContent(nsIContent** aResult)
   if ( !xblService )
     return;
   
-  PRInt32 count;
-  mMenuContent->ChildCount(count);
-
-  for (PRInt32 i = 0; i < count; i++) {
+  PRUint32 count = mMenuContent->GetChildCount();
+  for (PRUint32 i = 0; i < count; i++) {
     PRInt32 dummy;
-    nsCOMPtr<nsIContent> child;
-    mMenuContent->ChildAt(i, getter_AddRefs(child));
+    nsIContent *child = mMenuContent->GetChildAt(i);
     nsCOMPtr<nsIAtom> tag;
     xblService->ResolveTag(child, &dummy, getter_AddRefs(tag));
-    if (tag && tag.get() == nsWidgetAtoms::menupopup) {
-      *aResult = child.get();
+    if (tag == nsWidgetAtoms::menupopup) {
+      *aResult = child;
       NS_ADDREF(*aResult);
       return;
     }
@@ -1297,8 +1283,7 @@ nsMenuX :: CountVisibleBefore ( PRUint32* outVisibleBefore )
 
 
 NS_IMETHODIMP
-nsMenuX::AttributeChanged(nsIDocument *aDocument, PRInt32 aNameSpaceID, nsIAtom *aAttribute,
-                               PRInt32 aHint)
+nsMenuX::AttributeChanged(nsIDocument *aDocument, PRInt32 aNameSpaceID, nsIAtom *aAttribute )
 {
   if (gConstructingMenu)
     return NS_OK;
