@@ -124,7 +124,7 @@ static nsModuleComponentInfo components[] =
 		nsToolkitConstructor },
 	{	"Look And Feel",
 		NS_LOOKANDFEEL_CID,
-		"@mozilla.org/widget/lookandfeel/mac;1",
+		"@mozilla.org/widget/lookandfeel;1",
 		nsLookAndFeelConstructor },
 	{	"Menubar",
 		NS_MENUBAR_CID,
@@ -148,7 +148,7 @@ static nsModuleComponentInfo components[] =
 		nsTransferableConstructor },
 	{	"HTML Format Converter",
 		NS_HTMLFORMATCONVERTER_CID,
-		"@mozilla.org/widget/htmlformatconverter/mac;1",
+		"@mozilla.org/widget/htmlformatconverter;1",
 		nsHTMLFormatConverterConstructor },
 	{	"Clipboard",
 		NS_CLIPBOARD_CID,
