@@ -70,7 +70,7 @@ GtkWidget* nsClipboard::sWidget = 0;
 
 static GdkAtom GDK_SELECTION_CLIPBOARD;
 
-NS_IMPL_ISUPPORTS1(nsClipboard, nsIClipboard);
+NS_IMPL_ISUPPORTS1(nsClipboard, nsIClipboard)
 
 //-------------------------------------------------------------------------
 //
@@ -623,6 +623,11 @@ nsClipboard::SelectionReceiver (GtkWidget *aWidget,
     nsCOMPtr<nsICharsetConverterManager> ccm = do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
     rv = ccm->GetUnicodeDecoderRaw(platformCharset.get(),
                                    getter_AddRefs(decoder));
+    NS_ASSERTION(NS_SUCCEEDED(rv), "GetUnicodeEncoderRaw failed");
+    if (NS_FAILED(rv)) {
+      if (tmpData) XFreeStringList(tmpData);
+      return;
+    }
       
     // Estimate out length and allocate the buffer based on a worst-case estimate, then do
     // the conversion. 
@@ -644,6 +649,7 @@ nsClipboard::SelectionReceiver (GtkWidget *aWidget,
 
     mSelectionData.data = NS_REINTERPRET_CAST(guchar*,unicodeData);
     mSelectionData.length = outUnicodeLen * 2;
+    if (tmpData) XFreeStringList(tmpData);
   }
   else if (type.Equals("UTF8_STRING")) {
     mSelectionData = *aSD;
@@ -664,6 +670,8 @@ nsClipboard::SelectionReceiver (GtkWidget *aWidget,
     nsCOMPtr<nsICharsetConverterManager> ccm = do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
     rv = ccm->GetUnicodeDecoderRaw("UTF-8", getter_AddRefs(decoder));
 
+    g_return_if_fail(NS_SUCCEEDED(rv));
+
     decoder->GetMaxLength(data, numberOfBytes, &outUnicodeLen);   // |outUnicodeLen| is number of chars
     if (outUnicodeLen) {
       unicodeData = NS_REINTERPRET_CAST(PRUnichar*, nsMemory::Alloc((outUnicodeLen + 1) * sizeof(PRUnichar)));
@@ -970,6 +978,11 @@ void nsClipboard::SelectionGetCB(GtkWidget        *widget,
       // get the encoder
       nsCOMPtr<nsICharsetConverterManager> ccm = do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
       rv = ccm->GetUnicodeEncoderRaw(platformCharset.get(), getter_AddRefs(encoder));
+      NS_ASSERTION(NS_SUCCEEDED(rv), "GetUnicodeEncoderRaw failed");
+      if (NS_FAILED(rv)) {
+        nsMemory::Free(NS_REINTERPRET_CAST(char*, clipboardData));
+        return;
+      }
 
       encoder->SetOutputErrorBehavior(nsIUnicodeEncoder::kOnError_Replace, nsnull, '?');
 
@@ -1034,7 +1047,7 @@ void nsClipboard::SelectionGetCB(GtkWidget        *widget,
       gtk_selection_data_set(aSelectionData,
                              gdk_atom_intern("NULL", FALSE), 8,
                              nsnull, 0);
-    nsCRT::free ( NS_REINTERPRET_CAST(char*, clipboardData) );
+    nsMemory::Free ( NS_REINTERPRET_CAST(char*, clipboardData) );
   }
 #ifdef DEBUG_pavlov
   else
