@@ -1,24 +1,39 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: NPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Netscape Public License
+ * Version 1.1 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/NPL/
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is Netscape
- * Communications Corporation.  Portions created by Netscape are
- * Copyright (C) 1998 Netscape Communications Corporation. All
- * Rights Reserved.
+ * The Initial Developer of the Original Code is 
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1998
+ * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): 
- */
+ * Contributor(s):
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or 
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the NPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the NPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
  
 #include "nsDynamicMDEF.h"
 
@@ -85,6 +100,7 @@ nsMenuStack::~nsMenuStack()
 nsresult
 nsMenuStack::GetMenuAt(PRInt32 aIndex, nsIMenu **outMenu)
 {
+  // I'd prefer to fix the caller, but this will be safe
   nsCOMPtr<nsISupports> elementPtr = getter_AddRefs(mMenuArray.ElementAt(aIndex));
   if (!elementPtr)
     return NS_ERROR_NULL_POINTER;
@@ -96,6 +112,7 @@ nsMenuStack::GetMenuAt(PRInt32 aIndex, nsIMenu **outMenu)
 PRBool
 nsMenuStack::HaveMenuAt(PRInt32 aIndex)
 {
+  // I'd prefer to fix the caller, but this will be safe
   nsCOMPtr<nsISupports> elementPtr = getter_AddRefs(mMenuArray.ElementAt(aIndex));
   if (!elementPtr)
     return NS_ERROR_NULL_POINTER;
@@ -278,7 +295,7 @@ void nsCheckDestroy(MenuHandle theMenu, short * whichItem)
   if(gCurrentMenuItem == *whichItem)
     changeOccured = false;
 
-  if(!gPreviousMenuHandleStack[gPreviousMenuHandleStack.Count() - 1])
+  if(gPreviousMenuHandleStack.Count() == 0 || !gPreviousMenuHandleStack[gPreviousMenuHandleStack.Count() - 1])
     changeOccured = false;
   
   if(nsIsHierChild(theMenu))
@@ -404,7 +421,7 @@ void nsBuildMenu(MenuHandle theMenu, PRBool isChild)
               gPreviousTopLevelMenuHandle = theMenu;
               gPreviousTopLevelMenu = menu;
               
-              gPreviousMenuBar = getter_AddRefs(NS_GetWeakReference(menubar));
+              gPreviousMenuBar = do_GetWeakReference(menubar);
 	            //printf("exit BuildMenu \n");
 	            return;   
 	          }
@@ -447,7 +464,8 @@ void nsBuildMenu(MenuHandle theMenu, PRBool isChild)
 void nsPostBuild(nsIMenu * menu, MenuHandle theMenu, PRBool isChild)
 {
   // it is built now
-  if(isChild || (gPreviousMenuHandleStack[gPreviousMenuHandleStack.Count() - 1] != theMenu))
+  if (isChild || (gPreviousMenuHandleStack.Count() == 0 ||
+                  gPreviousMenuHandleStack[gPreviousMenuHandleStack.Count() - 1] != theMenu))
   {
     nsPushMenu(menu);  
     nsPushMenuHandle(theMenu);    
