@@ -19,6 +19,7 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
+ *   Rich Walsh <dragtext@e-vertise.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or 
@@ -46,12 +47,13 @@
 #include "nsIURL.h"
 #include "nsNetUtil.h"
 #include "nsOS2Uni.h"
+#include "nsdefs.h"
+#include "resource.h"
 
 // --------------------------------------------------------------------------
 // helper functions
 
 void    WriteTypeEA(const char *filename, const char *type);
-nsresult GetURLObjectContents(const char *pszPath, char **ppszURL);
 int     UnicodeToCodepage( const nsAString& inString, char **outText);
 
 // --------------------------------------------------------------------------
@@ -60,21 +62,16 @@ int     UnicodeToCodepage( const nsAString& inString, char **outText);
 #define MAXTITLELTH 31
 #define TITLESEPARATOR (L' ')
 
-// from Toolkit45's unidef.h
-#ifndef CT_ALNUM
-extern "C" {
-  #define CT_ALNUM  0x0200
-  int _System UniQueryChar( PRUnichar uc, ULONG attr);
-}
-#endif
-
 #ifndef DC_PREPAREITEM
 #define DC_PREPAREITEM 0x0040;
 #endif
 
 // --------------------------------------------------------------------------
+// Global data
 
-PRBool nsDragService::sDoingDrag = PR_FALSE;
+static HPOINTER gPtrArray[IDC_DNDCOUNT];
+
+// --------------------------------------------------------------------------
 
 nsDragService::nsDragService()
 {
@@ -82,6 +79,11 @@ nsDragService::nsDragService()
    mDragWnd = WinCreateWindow( HWND_DESKTOP, WC_STATIC, 0, 0, 0, 0, 0, 0,
                                HWND_DESKTOP, HWND_BOTTOM, 0, 0, 0);
    WinSubclassWindow( mDragWnd, nsDragWindowProc);
+
+   HMODULE hModResources = NULLHANDLE;
+   DosQueryModFromEIP(&hModResources, NULL, 0, NULL, NULL, (ULONG) &gPtrArray);
+   for (int i = 0; i < IDC_DNDCOUNT; i++)
+     gPtrArray[i] = ::WinLoadPointer(HWND_DESKTOP, hModResources, i+IDC_DNDBASE);
 }
 
 // --------------------------------------------------------------------------
@@ -191,16 +193,22 @@ nsDragService::~nsDragService()
 {
   /* destructor code */
   WinDestroyWindow(mDragWnd);
+
+  for (int i = 0; i < IDC_DNDCOUNT; i++) {
+    WinDestroyPointer(gPtrArray[i]);
+    gPtrArray[i] = 0;
+  }
 }
 
 // --------------------------------------------------------------------------
 
 NS_IMETHODIMP nsDragService::InvokeDragSession(nsIDOMNode *aDOMNode, nsISupportsArray *aTransferables, nsIScriptableRegion *aRegion, PRUint32 aActionType)
 {
-  // Utter hack for drag drop - provide a way for nsWindow
-  // to set mSourceDataItems 
+  // Utter hack for drag drop - provide a way for nsWindow to set
+  // mSourceDataItems & to clear mSourceNode so we can identify native drags
   if (!aDOMNode && !aRegion && !aActionType) {
     mSourceDataItems = aTransferables;
+    mSourceNode = 0;
     return NS_OK;
   }
   nsBaseDragService::InvokeDragSession ( aDOMNode, aTransferables, aRegion, aActionType );
@@ -232,6 +240,7 @@ NS_IMETHODIMP nsDragService::InvokeDragSession(nsIDOMNode *aDOMNode, nsISupports
   dragitem.hstrSourceName      = NULLHANDLE;
 
   nsresult rv = NS_ERROR_FAILURE;
+  ULONG idIcon = 0;
 
   // reduce our footprint by ensuring any intermediate objects
   // go out of scope before the drag begins
@@ -240,9 +249,6 @@ NS_IMETHODIMP nsDragService::InvokeDragSession(nsIDOMNode *aDOMNode, nsISupports
     mSourceDataItems->GetElementAt(0, getter_AddRefs(genericItem));
     nsCOMPtr<nsITransferable> transItem (do_QueryInterface(genericItem));
 
-    // fix a bug in v1.4 which misidentifies text as a URL
-    FixTransferable(transItem);
-
     nsCOMPtr<nsISupports> genericData;
     PRUint32 len = 0;
     char * targetName = 0;
@@ -261,6 +267,7 @@ NS_IMETHODIMP nsDragService::InvokeDragSession(nsIDOMNode *aDOMNode, nsISupports
         dragitem.hstrRMF        = DrgAddStrHandle("<DRM_OS2FILE,DRF_UNKNOWN>");
         dragitem.hstrTargetName = DrgAddStrHandle(targetName);
         nsMemory::Free(targetName);
+        idIcon = IDC_DNDURL;
       }
     }
     else
@@ -272,6 +279,7 @@ NS_IMETHODIMP nsDragService::InvokeDragSession(nsIDOMNode *aDOMNode, nsISupports
         dragitem.hstrRMF        = DrgAddStrHandle("<DRM_OS2FILE,DRF_TEXT>");
         dragitem.hstrTargetName = DrgAddStrHandle(targetName);
         nsMemory::Free(targetName);
+        idIcon = IDC_DNDTEXT;
       }
     }
   }
@@ -289,13 +297,20 @@ NS_IMETHODIMP nsDragService::InvokeDragSession(nsIDOMNode *aDOMNode, nsISupports
   DRAGIMAGE dragimage;
   memset(&dragimage, 0, sizeof(DRAGIMAGE));
   dragimage.cb = sizeof(DRAGIMAGE);
-  dragimage.hImage = WinQuerySysPointer(HWND_DESKTOP, SPTR_FILE, FALSE);
   dragimage.fl = DRG_ICON;
+  if (idIcon)
+    dragimage.hImage = gPtrArray[idIcon-IDC_DNDBASE];
+  if (dragimage.hImage) {
+    dragimage.cyOffset = 8;
+    dragimage.cxOffset = 2;
+  }
+  else
+    dragimage.hImage  = WinQuerySysPointer(HWND_DESKTOP, SPTR_FILE, FALSE);
     
-  mDoingDrag = sDoingDrag = PR_TRUE;
+  mDoingDrag = PR_TRUE;
   HWND hwndDest = DrgDrag(mDragWnd, pDragInfo, &dragimage, 1, VK_BUTTON2,
                   (void*)0x80000000L); // Don't lock the desktop PS
-  mDoingDrag = sDoingDrag = PR_FALSE;
+  mDoingDrag = PR_FALSE;
 
   // do clean up;  if the drop completed, the target will delete
   // the string handles
@@ -303,6 +318,7 @@ NS_IMETHODIMP nsDragService::InvokeDragSession(nsIDOMNode *aDOMNode, nsISupports
       DrgDeleteDraginfoStrHandles(pDragInfo);
   DrgFreeDraginfo(pDragInfo);
 
+  mSourceNode = 0;
   mSourceDataItems = 0;
   mSourceData = 0;
   mMimeType = 0;
@@ -312,115 +328,6 @@ NS_IMETHODIMP nsDragService::InvokeDragSession(nsIDOMNode *aDOMNode, nsISupports
 
 // --------------------------------------------------------------------------
 
-// This method compensates for a bug in Mozilla v1.4's nsContentAreaDragDrop.
-// If selected text happens to include an anchor, the entire selection is
-// misidentified as a URL and the selection is used as the URL's title.
-// FixTransferable() examines the HTML, and based on the quirks known to
-// exist in nsContentAreaDragDrop's rendering, determines if the bug is
-// active.  If so, it removes the invalid entries from the transferable
-// and reinserts the URL's title as the new Unicode text flavor.
-// Note:  this works perfectly(?) in v1.4 but doesn't work at all in v1.5
-
-nsresult nsDragService::FixTransferable(nsITransferable* aTransItem)
-
-{
-  nsresult rv = NS_ERROR_FAILURE;
-  PRUint32 len = 0;
-
-  // begin testing - everything we alloc here will go out of scope
-  // if this proves to be text misidentified as a URL
-  {
-    PRBool isValid;
-
-    // exit if there isn't a URL in the transferable
-    if (NS_FAILED(IsDataFlavorSupported(kURLMime, &isValid)) || !isValid)
-      return (rv);
-
-    // exit if the URL points to an image
-    if (NS_FAILED(IsDataFlavorSupported(kNativeImageMime, &isValid)) || isValid)
-      return (rv);
-
-    nsCOMPtr<nsISupports> htmlData;
-
-    // get the HTML from the transferable
-    if (NS_FAILED(aTransItem->GetTransferData(kHTMLMime,
-                                getter_AddRefs(htmlData), &len)))
-      return (rv);
-
-    nsCOMPtr<nsISupportsString> strObject ( do_QueryInterface(htmlData));
-    if (!strObject)
-      return (rv);
-
-    nsXPIDLString htmlStr;
-    strObject->ToString(getter_Copies(htmlStr));
-
-    PRBool found;
-    nsXPIDLString::const_iterator start, end, first, last;
-    htmlStr.BeginReading(first);
-    htmlStr.EndReading(last);
-
-    // find the first anchor tag
-    start = first;
-    end = last;
-    found = FindInReadable( NS_LITERAL_STRING("<a "), start, end);
-    if (!found)
-    {
-      start = first;
-      end = last;
-      found = FindInReadable( NS_LITERAL_STRING("<A "), start, end);
-    }
-
-    // if there's no anchor tag at all, this really is a URL
-    // (this is a bug in v1.4's nsContentArea that's fixed in v1.5)
-    if (!found)
-      return (rv);
-
-    // if there's an anchor & it's at the beginning of the string,
-    // see if the matching end-of-anchor is at the end of the string;
-    // if so, exit because this is a valid URL
-    if (start == first)
-    {
-      end = last;
-      found = FindInReadable( NS_LITERAL_STRING("</a>"), start, end);
-      if (!found)
-      {
-        start = first;
-        end = last;
-        found = FindInReadable( NS_LITERAL_STRING("</A>"), start, end);
-      }
-      if (found && end == last)
-          return (rv);
-    }
-  } // end testing
-
-// we've got text that's been misidentified as a URL;
-// the title portion of the URL string is the text we want
-
-  // get the URL title
-  nsCOMPtr<nsISupports> textData;
-  if (NS_FAILED(aTransItem->GetTransferData(kURLDescriptionMime,
-                              getter_AddRefs(textData), &len)))
-    return (rv);
-
-  nsCOMPtr<nsISupportsString> strObject(do_QueryInterface(textData));
-  if (!strObject)
-    return (rv);
-
-  // remove all of the invalid flavors
-  aTransItem->RemoveDataFlavor( kURLMime);
-  aTransItem->RemoveDataFlavor( kURLDataMime);
-  aTransItem->RemoveDataFlavor( kURLDescriptionMime);
-  aTransItem->RemoveDataFlavor( kUnicodeMime);
-
-  // reinsert the string as the new Unicode flavor
-  aTransItem->SetTransferData(kUnicodeMime, strObject, len);
-
-  rv = NS_OK;
-  return (rv);
-}
-
-// --------------------------------------------------------------------------
-
 NS_IMETHODIMP nsDragService::GetNumDropItems(PRUint32 *aNumDropItems)
 {
   mSourceDataItems->Count(aNumDropItems);
@@ -769,44 +676,6 @@ void WriteTypeEA(const char *filename, const char *type)
 
 // --------------------------------------------------------------------------
 
-/* GetURLObjectContents -- This function returns the contents of a WPSH URL
- * object by reading the file name specified in the dragitem.  This is
- * necessary so that dropping URL objects onto the browser displays the
- * page at the URL, rather than displaying the URL itself.
- */
-
-nsresult GetURLObjectContents(const char *pszPath, char **ppszURL)
-{
-   nsresult rv = NS_ERROR_FAILURE;
-   size_t readsize;
-   ULONG filesize;
-   FILE *fp = 0;
-
-   fp = fopen(pszPath, "r");
-   if (fp) {
-     fseek(fp, 0, SEEK_END);
-     filesize = ftell(fp);
-     fseek(fp, 0, SEEK_SET);
-     if (filesize > 0) {
-        readsize = (size_t)filesize;
-        *ppszURL = (char*)nsMemory::Alloc(readsize+1);
-        if (*ppszURL) {
-          readsize = fread((void *)*ppszURL, 1, readsize, fp);
-          if (readsize) {
-            (*ppszURL)[readsize] = '\0';
-            rv = NS_OK;
-          }
-          else
-            nsMemory::Free(*ppszURL);
-        }
-     }
-     fclose(fp);
-   }
-   return (rv);
-}
-
-// --------------------------------------------------------------------------
-
 // to do:  this needs to take into account the current page's encoding
 // if it is different than the PM codepage
 
