@@ -56,10 +56,6 @@ NS_IMPL_ISUPPORTS1(nsFilePicker, nsIFilePicker)
 
 char nsFilePicker::mLastUsedDirectory[MAX_PATH+1] = { 0 };
 
-static char* gpszFDSaveCaption = 0;
-static char* gpszFDFileExists = 0;
-static char* gpszFDFileReadOnly = 0;
-
 MRESULT EXPENTRY DirDialogProc( HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2);
 MRESULT EXPENTRY FileDialogProc( HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2);
 
@@ -75,6 +71,7 @@ nsFilePicker::nsFilePicker()
   mUnicodeDecoder = nsnull;
   mSelectedType   = 0;
   mDisplayDirectory = do_CreateInstance("@mozilla.org/file/local;1");
+  pszFDFileExists[0] = '\0';
 }
 
 //-------------------------------------------------------------------------
@@ -91,16 +88,6 @@ nsFilePicker::~nsFilePicker()
   NS_IF_RELEASE(mUnicodeDecoder);
 }
 
-/* static */ void
-nsFilePicker::ReleaseGlobals()
-{
-  if (gpszFDSaveCaption) {
-     free(gpszFDSaveCaption);
-     free(gpszFDFileExists);
-     free(gpszFDFileReadOnly);
-  }
-}
-
 //-------------------------------------------------------------------------
 //
 // Show - Display the file dialog
@@ -218,31 +205,16 @@ NS_IMETHODIMP nsFilePicker::Show(PRInt16 *retval)
             fileExists = PR_FALSE;
          }
          if (fileExists) {
-            if (!gpszFDSaveCaption) {
+            if (!pszFDFileExists[0]) {
               HMODULE hmod;
               char LoadError[CCHMAXPATH];
-              char loadedString[256];
-              int length;
               DosLoadModule(LoadError, CCHMAXPATH, "PMSDMRI", &hmod);
-              length = WinLoadString((HAB)0, hmod, 1110, 256, loadedString);
-              gpszFDSaveCaption = (char*)malloc(length+1);
-              strcpy(gpszFDSaveCaption, loadedString);
-              length = WinLoadString((HAB)0, hmod, 1135, 256, loadedString);
-              gpszFDFileExists = (char*)malloc(length+1);
-              strcpy(gpszFDFileExists, loadedString);
-              length = WinLoadString((HAB)0, hmod, 1136, 256, loadedString);
-              gpszFDFileReadOnly = (char*)malloc(length+1);
-              strcpy(gpszFDFileReadOnly, loadedString);
+              WinLoadString((HAB)0, hmod, 1110, 256, pszFDSaveCaption);
+              WinLoadString((HAB)0, hmod, 1135, 256, pszFDFileExists);
               int i;
-              for (i=0;i<256 && gpszFDFileExists[i];i++ ) {
-                if (gpszFDFileExists[i] == '%') {
-                  gpszFDFileExists[i+1] = 's';
-                  break;
-                }
-              }
-              for (i=0;i<256 && gpszFDFileReadOnly[i];i++ ) {
-                if (gpszFDFileReadOnly[i] == '%') {
-                  gpszFDFileReadOnly[i+1] = 's';
+              for (i=0;i<256 && pszFDFileExists[i];i++ ) {
+                if (pszFDFileExists[i] == '%') {
+                  pszFDFileExists[i+1] = 's';
                   break;
                 }
               }
@@ -250,21 +222,10 @@ NS_IMETHODIMP nsFilePicker::Show(PRInt16 *retval)
 
             }
             char pszFullText[256+CCHMAXPATH];
-            FILESTATUS3 fsts3;
-            ULONG ulResponse;
-            DosQueryPathInfo( filedlg.szFullFile, FIL_STANDARD, &fsts3, sizeof(FILESTATUS3));
-            if (fsts3.attrFile & FILE_READONLY) {
-              sprintf(pszFullText, gpszFDFileReadOnly, filedlg.szFullFile);
-              ulResponse = WinMessageBox(HWND_DESKTOP, mWnd, pszFullText,
-                                               gpszFDSaveCaption, 0,
-                                               MB_OK | MB_MOVEABLE | MB_WARNING);
-            } else {
-              sprintf(pszFullText, gpszFDFileExists, filedlg.szFullFile);
-              ulResponse = WinMessageBox(HWND_DESKTOP, mWnd, pszFullText,
-                                               gpszFDSaveCaption, 0,
-                                               MB_YESNO | MB_MOVEABLE | MB_WARNING);
-            }
-
+            sprintf(pszFullText, pszFDFileExists, filedlg.szFullFile);
+            ULONG ulResponse = WinMessageBox(HWND_DESKTOP, mWnd, pszFullText,
+                                             pszFDSaveCaption, 0,
+                                             MB_YESNO | MB_MOVEABLE | MB_WARNING);
             if (ulResponse == MBID_YES) {
                fileExists = PR_FALSE;
             }
@@ -529,9 +490,9 @@ NS_IMETHODIMP nsFilePicker::InitNative(nsIWidget *aParent,
 
 
 //-------------------------------------------------------------------------
-void nsFilePicker::GetFileSystemCharset(nsString & fileSystemCharset)
+void nsFilePicker::GetFileSystemCharset(nsCString & fileSystemCharset)
 {
-  static nsAutoString aCharset;
+  static nsCAutoString aCharset;
   nsresult rv;
 
   if (aCharset.Length() < 1) {
@@ -541,7 +502,7 @@ void nsFilePicker::GetFileSystemCharset(nsString & fileSystemCharset)
 
     NS_ASSERTION(NS_SUCCEEDED(rv), "error getting platform charset");
 	  if (NS_FAILED(rv)) 
-		  aCharset.Assign(NS_LITERAL_STRING("windows-1252"));
+		  aCharset.Assign(NS_LITERAL_CSTRING("IBM850"));
   }
   fileSystemCharset = aCharset;
 }
@@ -554,13 +515,13 @@ char * nsFilePicker::ConvertToFileSystemCharset(const PRUnichar *inString)
 
   // get file system charset and create a unicode encoder
   if (nsnull == mUnicodeEncoder) {
-    nsAutoString fileSystemCharset;
+    nsCAutoString fileSystemCharset;
     GetFileSystemCharset(fileSystemCharset);
 
     nsCOMPtr<nsICharsetConverterManager> ccm = 
              do_GetService(kCharsetConverterManagerCID, &rv); 
     if (NS_SUCCEEDED(rv)) {
-      rv = ccm->GetUnicodeEncoder(&fileSystemCharset, &mUnicodeEncoder);
+      rv = ccm->GetUnicodeEncoderRaw(fileSystemCharset.get(), &mUnicodeEncoder);
     }
   }
 
@@ -592,13 +553,13 @@ PRUnichar * nsFilePicker::ConvertFromFileSystemCharset(const char *inString)
 
   // get file system charset and create a unicode encoder
   if (nsnull == mUnicodeDecoder) {
-    nsAutoString fileSystemCharset;
+    nsCAutoString fileSystemCharset;
     GetFileSystemCharset(fileSystemCharset);
 
     nsCOMPtr<nsICharsetConverterManager> ccm = 
              do_GetService(kCharsetConverterManagerCID, &rv); 
     if (NS_SUCCEEDED(rv)) {
-      rv = ccm->GetUnicodeDecoder(&fileSystemCharset, &mUnicodeDecoder);
+      rv = ccm->GetUnicodeDecoderRaw(fileSystemCharset.get(), &mUnicodeDecoder);
     }
   }
 
