@@ -31,6 +31,7 @@
  * 06/15/2000       IBM Corp.      Added NS2PM for rectangles
  * 06/21/2000       IBM Corp.      Added CaptureMouse
  *
+ * Rich Walsh <dragtext@e-vertise.com>
  */
 
 #ifndef _nswindow_h
@@ -66,7 +67,9 @@
    #define      nsWindowState_eDoingDelete 0x00000010
    // window destroyed 
    #define      nsWindowState_eDead        0x00000100         
-   
+
+MRESULT EXPENTRY fnwpNSWindow( HWND, ULONG, MPARAM, MPARAM);
+MRESULT EXPENTRY fnwpFrame( HWND, ULONG, MPARAM, MPARAM);
 
 class nsWindow : public nsBaseWidget,
                  public nsSwitchToUIThread
@@ -120,7 +123,8 @@ class nsWindow : public nsBaseWidget,
    NS_IMETHOD SetFocus(PRBool aRaise);
    NS_IMETHOD GetBounds(nsRect &aRect);
    NS_IMETHOD IsVisible( PRBool &aState);
-   NS_IMETHOD PlaceBehind(nsIWidget *aWidget, PRBool aActivate);
+   NS_IMETHOD PlaceBehind(nsTopLevelWidgetZPlacement aPlacement,
+                          nsIWidget *aWidget, PRBool aActivate);
 
    NS_IMETHOD CaptureMouse(PRBool aCapture);
 
@@ -242,8 +246,6 @@ protected:
    PSWP      mSWPs;           // SWPs for deferred window positioning
    ULONG     mlHave, mlUsed;  // description of mSWPs array
    HPOINTER  mFrameIcon;      // current frame icon
-   HPS       mPS;             // cache PS for window
-   ULONG     mPSRefs;         // number of refs to cache ps
    BOOL      mNativeDrag;     // is the drag from outside Mozilla
    VDKEY     mDeadKey;        // dead key from previous keyevent
    BOOL      mHaveDeadKey;    // is mDeadKey valid [0 may be a valid dead key, for all I know]
@@ -252,7 +254,9 @@ protected:
    PRBool    mIsTopWidgetWindow;
    BOOL      mIsScrollBar;
    BOOL      mInSetFocus;
+   BOOL      mChromeHidden;
    nsContentType mContentType;
+   HPS       mDragHps;        // retrieved by DrgGetPS() during a drag
 
    HWND      GetParentHWND() const;
    HWND      GetHWND() const   { return mWnd; }
@@ -294,8 +298,10 @@ protected:
    virtual PRBool DispatchResizeEvent( PRInt32 aClientX, PRInt32 aClientY);
    void GetNonClientBounds(nsRect &aRect);
    void    DeferPosition( HWND, HWND, long, long, long, long, ULONG);
+   void ConstrainZLevel(HWND *aAfter);
 
-    void ConstrainZLevel(HWND *aAfter);
+   PRUint32 GetDragStatus(PRUint32 aState, HPS * oHps);
+   PRBool   ReleaseIfDragHPS(HPS hps);
 
    // Enumeration of the methods which are accessable on the PM thread
    enum {
