@@ -327,23 +327,24 @@ nsFilePicker::AppendFilter(const PRUnichar *aTitle, const PRUnichar *aFilter)
 
 
 //-------------------------------------------------------------------------
-void nsFilePicker::GetFileSystemCharset(nsString & fileSystemCharset)
+void nsFilePicker::GetFileSystemCharset(nsCString & fileSystemCharset)
 {
-  static nsAutoString aCharset;
+  static nsCAutoString aCharset;
   nsresult rv;
 
   if (aCharset.Length() < 1) {
     nsCOMPtr <nsIPlatformCharset> platformCharset = do_GetService(NS_PLATFORMCHARSET_CONTRACTID, &rv);
-	  if (NS_SUCCEEDED(rv)) 
-		  rv = platformCharset->GetCharset(kPlatformCharsetSel_FileName, aCharset);
+    if (NS_SUCCEEDED(rv))
+      rv = platformCharset->GetCharset(kPlatformCharsetSel_FileName, aCharset);
 
     NS_ASSERTION(NS_SUCCEEDED(rv), "error getting platform charset");
-	  if (NS_FAILED(rv)) 
-		  aCharset.Assign(NS_LITERAL_STRING("windows-1252"));
+    if (NS_FAILED(rv))
+      aCharset.Assign(NS_LITERAL_CSTRING("windows-1252"));
   }
   fileSystemCharset = aCharset;
 }
 
+
 //-------------------------------------------------------------------------
 char * nsFilePicker::ConvertToFileSystemCharset(const PRUnichar *inString)
 {
@@ -352,13 +353,13 @@ char * nsFilePicker::ConvertToFileSystemCharset(const PRUnichar *inString)
 
   // get file system charset and create a unicode encoder
   if (nsnull == mUnicodeEncoder) {
-    nsAutoString fileSystemCharset;
+    nsCAutoString fileSystemCharset;
     GetFileSystemCharset(fileSystemCharset);
 
     nsCOMPtr<nsICharsetConverterManager> ccm = 
              do_GetService(kCharsetConverterManagerCID, &rv); 
     if (NS_SUCCEEDED(rv)) {
-      rv = ccm->GetUnicodeEncoder(&fileSystemCharset, &mUnicodeEncoder);
+      rv = ccm->GetUnicodeEncoderRaw(fileSystemCharset.get(), &mUnicodeEncoder);
     }
   }
 
@@ -390,13 +391,13 @@ PRUnichar * nsFilePicker::ConvertFromFileSystemCharset(const char *inString)
 
   // get file system charset and create a unicode encoder
   if (nsnull == mUnicodeDecoder) {
-    nsAutoString fileSystemCharset;
+    nsCAutoString fileSystemCharset;
     GetFileSystemCharset(fileSystemCharset);
 
     nsCOMPtr<nsICharsetConverterManager> ccm = 
              do_GetService(kCharsetConverterManagerCID, &rv); 
     if (NS_SUCCEEDED(rv)) {
-      rv = ccm->GetUnicodeDecoder(&fileSystemCharset, &mUnicodeDecoder);
+      rv = ccm->GetUnicodeDecoderRaw(fileSystemCharset.get(), &mUnicodeDecoder);
     }
   }
 
