@@ -38,7 +38,7 @@
 #include "nsEnumeratorUtils.h"
 #include "nsCRT.h"
 #include <windows.h>
-#include <SHLOBJ.H>
+#include <shlobj.h>
 
 // commdlg.h and cderr.h are needed to build with WIN32_LEAN_AND_MEAN
 #include <commdlg.h>
@@ -98,8 +98,8 @@ NS_IMETHODIMP nsFilePicker::ShowW(PRInt16 *aReturnVal)
   }
 
   PRBool result = PR_FALSE;
-  PRUnichar fileBuffer[MAX_PATH+1];
-  wcsncpy(fileBuffer,  mDefault.get(), MAX_PATH);
+  PRUnichar fileBuffer[FILE_BUFFER_SIZE+1];
+  wcsncpy(fileBuffer,  mDefault.get(), FILE_BUFFER_SIZE);
 
   nsAutoString htmExt(NS_LITERAL_STRING("html"));
   PRUnichar *title = ToNewUnicode(mTitle);
@@ -165,9 +165,9 @@ NS_IMETHODIMP nsFilePicker::ShowW(PRInt16 *aReturnVal)
     ofn.hwndOwner    = (HWND)
       (mParentWidget ? mParentWidget->GetNativeData(NS_NATIVE_WINDOW) : 0); 
     ofn.lpstrFile    = fileBuffer;
-    ofn.nMaxFile     = MAX_PATH;
+    ofn.nMaxFile     = FILE_BUFFER_SIZE;
 
-    ofn.Flags = OFN_NOCHANGEDIR | OFN_SHAREAWARE | OFN_LONGNAMES | OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY;
+    ofn.Flags = OFN_NOCHANGEDIR | OFN_SHAREAWARE | OFN_LONGNAMES | OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY | OFN_PATHMUSTEXIST;
 
     if (!mDefaultExtension.IsEmpty()) {
       ofn.lpstrDefExt = mDefaultExtension.get();
