@@ -46,7 +46,7 @@
 #include "nsIComponentRegistrar.h"
 #include "nsIComponentManagerObsolete.h"
 #include "nsIComponentLoaderManager.h"
-#include "nsICategoryManager.h"
+#include "nsCategoryManager.h"
 #include "nsIServiceManager.h"
 #include "nsIFactory.h"
 #include "nsIInterfaceRequestor.h"
@@ -140,21 +140,14 @@ public:
 
     nsresult WritePersistentRegistry();
     nsresult ReadPersistentRegistry();
-private:
-    nsresult WriteCategoryManagerToRegistry(PRFileDesc* fd);
-public:
 
     nsresult Shutdown(void);
 
     nsresult FreeServices();
 
-    friend class nsFactoryEntry;
-    friend class nsServiceManager;
-
-    friend nsresult
+    nsresult
     NS_GetService(const char *aContractID, const nsIID& aIID, PRBool aDontCreate, nsISupports** result);
 
-protected:
     nsresult RegisterComponentCommon(const nsCID &aClass,
                                      const char *aClassName,
                                      const char *aContractID,
@@ -198,18 +191,15 @@ protected:
     // there was an error
     int AddLoaderType(const char *typeStr);
 
-public:
     int GetLoaderCount() { return mNLoaderData + 1; }
 
     // registers only the files in spec's location by loaders other than the
     // native loader.  This is an optimization method only.
     nsresult AutoRegisterNonNativeComponents(nsIFile* spec);
 
-
-private:
     nsresult AutoRegisterImpl(PRInt32 when, nsIFile *inDirSpec, PRBool fileIsCompDir=PR_TRUE);
+    nsresult RemoveEntries(nsIFile* file);
 
-protected:
     PLDHashTable        mFactories;
     PLDHashTable        mContractIDs;
     PRMonitor*          mMon;
@@ -238,7 +228,7 @@ protected:
 
     PRBool              mRegistryDirty;
     nsHashtable         mAutoRegEntries;
-    nsCOMPtr<nsICategoryManager>  mCategoryManager;
+    nsCOMPtr<nsCategoryManager>  mCategoryManager;
 
     PLArenaPool   mArena;
 
@@ -256,32 +246,6 @@ protected:
 
 #define NS_ERROR_IS_DIR NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_XPCOM, 24)
 
-#if defined(XP_UNIX) && !defined(XP_MACOSX)
-/* The default registry on the unix system is $HOME/.mozilla/registry per
- * vr_findGlobalRegName(). vr_findRegFile() will create the registry file
- * if it doesn't exist. But it wont create directories.
- *
- * Hence we need to create the directory if it doesn't exist already.
- *
- * Why create it here as opposed to the app ?
- * ------------------------------------------
- * The app cannot create the directory in main() as most of the registry
- * and initialization happens due to use of static variables.
- * And we dont want to be dependent on the order in which
- * these static stuff happen.
- *
- * Permission for the $HOME/.mozilla will be Read,Write,Execute
- * for user only. Nothing to group and others.
- */
-#define NS_MOZILLA_DIR_NAME		".mozilla"
-#define NS_MOZILLA_DIR_PERMISSION	00700
-#endif /* XP_UNIX */
-
-#ifdef XP_BEOS
-#define NS_MOZILLA_DIR_NAME		"Mozilla"
-#define NS_MOZILLA_DIR_PERMISSION	00700
-#endif /* XP_BEOS */
-
 ////////////////////////////////////////////////////////////////////////////////
 /**
  * Class: nsFactoryEntry()
