@@ -41,7 +41,6 @@
 
 #include "nsVoidArray.h"
 #include "nsISupports.h"
-#include "nsCOMPtr.h"
 
 // See below for the definition of nsCOMArray<T>
 
@@ -70,6 +69,10 @@ protected:
         return mArray.EnumerateBackwards(aFunc, aData);
     }
     
+    void Sort(nsVoidArrayComparatorFunc aFunc, void* aData) {
+        mArray.Sort(aFunc, aData);
+    }
+    
     // any method which is not a direct forward to mArray should
     // avoid inline bodies, so that the compiler doesn't inline them
     // all over the place
@@ -93,11 +96,11 @@ public:
         return mArray.Count();
     }
 
-    nsDerivedSafe<nsISupports>* ObjectAt(PRInt32 aIndex) const {
-        return NS_REINTERPRET_CAST(nsDerivedSafe<nsISupports>*, mArray.ElementAt(aIndex));
+    nsISupports* ObjectAt(PRInt32 aIndex) const {
+        return NS_STATIC_CAST(nsISupports*, mArray.ElementAt(aIndex));
     }
     
-    nsDerivedSafe<nsISupports>* operator[](PRInt32 aIndex) const {
+    nsISupports* operator[](PRInt32 aIndex) const {
         return ObjectAt(aIndex);
     }
 
@@ -142,12 +145,12 @@ class nsCOMArray : public nsCOMArray_base
     ~nsCOMArray() {}
 
     // these do NOT refcount on the way out, for speed
-    nsDerivedSafe<T>* ObjectAt(PRInt32 aIndex) const {
-        return NS_REINTERPRET_CAST(nsDerivedSafe<T>*,nsCOMArray_base::ObjectAt(aIndex));
+    T* ObjectAt(PRInt32 aIndex) const {
+        return NS_STATIC_CAST(T*,nsCOMArray_base::ObjectAt(aIndex));
     }
 
     // indexing operator for syntactic sugar
-    nsDerivedSafe<T>* operator[](PRInt32 aIndex) const {
+    T* operator[](PRInt32 aIndex) const {
         return ObjectAt(aIndex);
     }
 
@@ -214,6 +217,13 @@ class nsCOMArray : public nsCOMArray_base
         return nsCOMArray_base::EnumerateBackwards(nsVoidArrayEnumFunc(aFunc),
                                                   aData);
     }
+    
+    typedef int (* PR_CALLBACK nsCOMArrayComparatorFunc)
+        (const T* aElement1, const T* aElement2, void* aData);
+        
+    void Sort(nsCOMArrayComparatorFunc aFunc, void* aData) {
+        nsCOMArray_base::Sort(nsVoidArrayComparatorFunc(aFunc), aData);
+    }
 
     // append an object, growing the array as necessary
     PRBool AppendObject(T *aObject) {
