@@ -89,11 +89,11 @@
   #pragma warning( disable: 4514 )
 #endif
 
-#define NSCAP_FEATURE_FACTOR_DESTRUCTOR
+#define NSCAP_FEATURE_USE_BASE
 
 #ifdef NS_DEBUG
   #define NSCAP_FEATURE_TEST_DONTQUERY_CASES
-  #define NSCAP_FEATURE_DEBUG_PTR_TYPES
+  #undef NSCAP_FEATURE_USE_BASE
 //#define NSCAP_FEATURE_TEST_NONNULL_QUERY_SUCCEEDS
 #endif
 
@@ -107,12 +107,14 @@
   #undef NSCAP_FEATURE_TEST_DONTQUERY_CASES
 #endif
 
-#if defined(NSCAP_DISABLE_DEBUG_PTR_TYPES) || !defined(NS_DEBUG)
-  #undef NSCAP_FEATURE_DEBUG_PTR_TYPES
+#if defined(__GNUC__) && __GNUC__ >= 3
+  // Without this, we violate the C++ standard's aliasing rules.  See
+  // http://bugzilla.mozilla.org/show_bug.cgi?id=212082
+  #undef NSCAP_FEATURE_USE_BASE
 #endif
 
-#ifdef NSCAP_FEATURE_DEBUG_PTR_TYPES
-  #undef NSCAP_FEATURE_FACTOR_DESTRUCTOR
+#if defined(NSCAP_DISABLE_DEBUG_PTR_TYPES)
+  #define NSCAP_FEATURE_USE_BASE
 #endif
 
 
@@ -196,7 +198,7 @@ class nsDerivedSafe : public T
       nsrefcnt Release(void);
 #endif
 
-#if !defined(XP_OS2_VACPP) && !defined(AIX) && !defined(IRIX)
+#if !defined(AIX) && !defined(IRIX)
       void operator delete( void*, size_t );                  // NOT TO BE IMPLEMENTED
         // declaring |operator delete| private makes calling delete on an interface pointer a compile error
 #endif
@@ -210,6 +212,13 @@ class nsDerivedSafe : public T
           |operator=()|, and that's _bad_.  So bad, in fact, that this declaration exists explicitly
           to stop people from doing it.
         */
+
+    protected:
+      nsDerivedSafe();                                        // NOT TO BE IMPLEMENTED
+        /*
+          This ctor exists to avoid compile errors and warnings about nsDeriviedSafe using the
+          default ctor but inheriting classes without an empty ctor. See bug 209667.
+        */
   };
 
 #if !defined(HAVE_CPP_ACCESS_CHANGING_USING) && defined(NEED_CPP_UNUSED_IMPLEMENTATIONS)
@@ -298,25 +307,6 @@ dont_AddRef( const already_AddRefed<T> aAlreadyAddRefedPtr )
 
 
 
-
-  /*
-    There used to be machinery to allow |dont_QueryInterface()| to work, but
-     since it is now equivalent to using a raw pointer ... all that machinery
-     has gone away.  For pointer arguments, the following definition should
-     optimize away.  This is better than using a |#define| because it is
-     scoped.
-  */
-
-template <class T>
-inline
-T*
-dont_QueryInterface( T* expr )
-  {
-    return expr;
-  }
-
-
-
 class nsCOMPtr_helper
     /*
       An |nsCOMPtr_helper| transforms commonly called getters into typesafe forms
@@ -406,13 +396,7 @@ class nsCOMPtr_base
           // nothing else to do here
         }
 
-#ifdef NSCAP_FEATURE_FACTOR_DESTRUCTOR
       NS_COM ~nsCOMPtr_base();
-#else
-      // Allow debug builds to link with optimized versions of nsCOMPtr-using
-      // plugins (e.g., JVMs).
-      NS_COM ~nsCOMPtr_base() { }
-#endif
 
       NS_COM void    assign_with_AddRef( nsISupports* );
       NS_COM void    assign_from_helper( const nsCOMPtr_helper&, const nsIID& );
@@ -445,23 +429,16 @@ class nsCOMPtr_base
 
 template <class T>
 class nsCOMPtr
-#ifndef NSCAP_FEATURE_DEBUG_PTR_TYPES
+#ifdef NSCAP_FEATURE_USE_BASE
     : private nsCOMPtr_base
 #endif
   {
-    enum { _force_even_compliant_compilers_to_fail_ = sizeof(T) };
-      /*
-        The declaration above exists specifically to make |nsCOMPtr<T>| _not_ compile with only
-        a forward declaration of |T|.  This should prevent Windows and Mac engineers from
-        breaking Solaris and other compilers that naturally have this behavior.  Thank
-        <law@netscape.com> for inventing this specific trick.
 
-        Of course, if you're using |nsCOMPtr| outside the scope of wanting to compile on
-        Solaris and old GCC, you probably want to remove the enum so you can exploit forward
-        declarations.
-      */
+#ifdef NSCAP_FEATURE_USE_BASE
+  #define NSCAP_CTOR_BASE(x) nsCOMPtr_base(x)
+#else
+  #define NSCAP_CTOR_BASE(x) mRawPtr(x)
 
-#ifdef NSCAP_FEATURE_DEBUG_PTR_TYPES
     private:
       void    assign_with_AddRef( nsISupports* );
       void    assign_from_helper( const nsCOMPtr_helper&, const nsIID& );
@@ -480,16 +457,12 @@ class nsCOMPtr
 
     private:
       T* mRawPtr;
-
-  #define NSCAP_CTOR_BASE(x) mRawPtr(x)
-#else
-  #define NSCAP_CTOR_BASE(x) nsCOMPtr_base(x)
 #endif
 
     public:
       typedef T element_type;
       
-#ifndef NSCAP_FEATURE_FACTOR_DESTRUCTOR
+#ifndef NSCAP_FEATURE_USE_BASE
      ~nsCOMPtr()
         {
           NSCAP_LOG_RELEASE(this, mRawPtr);
@@ -628,10 +601,10 @@ class nsCOMPtr
       swap( nsCOMPtr<T>& rhs )
           // ...exchange ownership with |rhs|; can save a pair of refcount operations
         {
-#ifdef NSCAP_FEATURE_DEBUG_PTR_TYPES
-          T* temp = rhs.mRawPtr;
-#else
+#ifdef NSCAP_FEATURE_USE_BASE
           nsISupports* temp = rhs.mRawPtr;
+#else
+          T* temp = rhs.mRawPtr;
 #endif
           NSCAP_LOG_ASSIGNMENT(&rhs, mRawPtr);
           NSCAP_LOG_ASSIGNMENT(this, temp);
@@ -646,10 +619,10 @@ class nsCOMPtr
       swap( T*& rhs )
           // ...exchange ownership with |rhs|; can save a pair of refcount operations
         {
-#ifdef NSCAP_FEATURE_DEBUG_PTR_TYPES
-          T* temp = rhs;
-#else
+#ifdef NSCAP_FEATURE_USE_BASE
           nsISupports* temp = rhs;
+#else
+          T* temp = rhs;
 #endif
           NSCAP_LOG_ASSIGNMENT(this, temp);
           NSCAP_LOG_RELEASE(this, mRawPtr);
@@ -766,16 +739,6 @@ class nsCOMPtr<nsISupports>
     public:
       typedef nsISupports element_type;
 
-#ifndef NSCAP_FEATURE_FACTOR_DESTRUCTOR
-     ~nsCOMPtr()
-        {
-          NSCAP_LOG_RELEASE(this, mRawPtr);
-          if ( mRawPtr )
-            NSCAP_RELEASE(this, mRawPtr);
-        }
-#endif
-
-
         // Constructors
 
       nsCOMPtr()
@@ -970,7 +933,7 @@ class nsCOMPtr<nsISupports>
         }
   };
 
-#ifdef NSCAP_FEATURE_DEBUG_PTR_TYPES
+#ifndef NSCAP_FEATURE_USE_BASE
 template <class T>
 void
 nsCOMPtr<T>::assign_with_AddRef( nsISupports* rawPtr )
@@ -1158,7 +1121,7 @@ inline
 NSCAP_BOOL
 operator==( const nsCOMPtr<T>& lhs, const nsCOMPtr<U>& rhs )
   {
-    return NS_STATIC_CAST(const void*, lhs.get()) == NS_STATIC_CAST(const void*, rhs.get());
+    return NS_STATIC_CAST(const T*, lhs.get()) == NS_STATIC_CAST(const U*, rhs.get());
   }
 
 
@@ -1167,7 +1130,7 @@ inline
 NSCAP_BOOL
 operator!=( const nsCOMPtr<T>& lhs, const nsCOMPtr<U>& rhs )
   {
-    return NS_STATIC_CAST(const void*, lhs.get()) != NS_STATIC_CAST(const void*, rhs.get());
+    return NS_STATIC_CAST(const T*, lhs.get()) != NS_STATIC_CAST(const U*, rhs.get());
   }
 
 
@@ -1178,7 +1141,7 @@ inline
 NSCAP_BOOL
 operator==( const nsCOMPtr<T>& lhs, const U* rhs )
   {
-    return NS_STATIC_CAST(const void*, lhs.get()) == NS_STATIC_CAST(const void*, rhs);
+    return NS_STATIC_CAST(const T*, lhs.get()) == rhs;
   }
 
 template <class T, class U>
@@ -1186,7 +1149,7 @@ inline
 NSCAP_BOOL
 operator==( const U* lhs, const nsCOMPtr<T>& rhs )
   {
-    return NS_STATIC_CAST(const void*, lhs) == NS_STATIC_CAST(const void*, rhs.get());
+    return lhs == NS_STATIC_CAST(const T*, rhs.get());
   }
 
 template <class T, class U>
@@ -1194,7 +1157,7 @@ inline
 NSCAP_BOOL
 operator!=( const nsCOMPtr<T>& lhs, const U* rhs )
   {
-    return NS_STATIC_CAST(const void*, lhs.get()) != NS_STATIC_CAST(const void*, rhs);
+    return NS_STATIC_CAST(const T*, lhs.get()) != rhs;
   }
 
 template <class T, class U>
@@ -1202,7 +1165,7 @@ inline
 NSCAP_BOOL
 operator!=( const U* lhs, const nsCOMPtr<T>& rhs )
   {
-    return NS_STATIC_CAST(const void*, lhs) != NS_STATIC_CAST(const void*, rhs.get());
+    return lhs != NS_STATIC_CAST(const T*, rhs.get());
   }
 
   // To avoid ambiguities caused by the presence of builtin |operator==|s
@@ -1215,7 +1178,7 @@ operator!=( const U* lhs, const nsCOMPtr<T>& rhs )
 // This is defined by an autoconf test, but VC++ also has a bug that
 // prevents us from using these.  (It also, fortunately, has the bug
 // that we don't need them either.)
-#ifdef _MSC_VER
+#if defined(_MSC_VER) && (_MSC_VER < 1310)
 #define NSCAP_DONT_PROVIDE_NONCONST_OPEQ
 #endif
 
@@ -1225,7 +1188,7 @@ inline
 NSCAP_BOOL
 operator==( const nsCOMPtr<T>& lhs, U* rhs )
   {
-    return NS_STATIC_CAST(const void*, lhs.get()) == NS_STATIC_CAST(void*, rhs);
+    return NS_STATIC_CAST(const T*, lhs.get()) == NS_CONST_CAST(const U*, rhs);
   }
 
 template <class T, class U>
@@ -1233,7 +1196,7 @@ inline
 NSCAP_BOOL
 operator==( U* lhs, const nsCOMPtr<T>& rhs )
   {
-    return NS_STATIC_CAST(void*, lhs) == NS_STATIC_CAST(const void*, rhs.get());
+    return NS_CONST_CAST(const U*, lhs) == NS_STATIC_CAST(const T*, rhs.get());
   }
 
 template <class T, class U>
@@ -1241,7 +1204,7 @@ inline
 NSCAP_BOOL
 operator!=( const nsCOMPtr<T>& lhs, U* rhs )
   {
-    return NS_STATIC_CAST(const void*, lhs.get()) != NS_STATIC_CAST(void*, rhs);
+    return NS_STATIC_CAST(const T*, lhs.get()) != NS_CONST_CAST(const U*, rhs);
   }
 
 template <class T, class U>
@@ -1249,7 +1212,7 @@ inline
 NSCAP_BOOL
 operator!=( U* lhs, const nsCOMPtr<T>& rhs )
   {
-    return NS_STATIC_CAST(void*, lhs) != NS_STATIC_CAST(const void*, rhs.get());
+    return NS_CONST_CAST(const U*, lhs) != NS_STATIC_CAST(const T*, rhs.get());
   }
 #endif
 
