@@ -45,35 +45,20 @@
 #include "nsISupportsBase.h"
 #endif
 
-#include "pratom.h" /* needed for PR_AtomicIncrement and PR_AtomicDecrement */
+#include "prthread.h" /* needed for thread-safety checks */
+#include "pratom.h"   /* needed for PR_AtomicIncrement and PR_AtomicDecrement */
 
-#ifdef XPCOM_GLUE
-// nsTraceRefcnt needs a cleaning...
-#define NS_LOG_ADDREF(_p, _rc, _type, _size)
-#define NS_LOG_RELEASE(_p, _rc, _type)
-#define NS_LOG_NEW_XPCOM(_p,_type,_size,_file,_line)
-#define NS_LOG_DELETE_XPCOM(_p,_file,_line)
-#define NS_LOG_ADDREF_CALL(_p,_rc,_file,_line) _rc
-#define NS_LOG_RELEASE_CALL(_p,_rc,_file,_line) _rc
-#define MOZ_DECL_CTOR_COUNTER(_type)
-#define MOZ_COUNT_CTOR(_type)
-#define MOZ_COUNT_DTOR(_type)
-#else
+#include "nsDebug.h"
 #include "nsTraceRefcnt.h" 
-#endif
 
 ////////////////////////////////////////////////////////////////////////////////
 // Macros to help detect thread-safety:
 
-#if defined(NS_DEBUG) && defined(NS_MT_SUPPORTED) && !defined(XPCOM_GLUE)
-
-extern "C" NS_COM void* NS_CurrentThread(void);
-extern "C" NS_COM void NS_CheckThreadSafe(void* owningThread,
-                                             const char* msg);
+#if defined(NS_DEBUG)
 
 class nsAutoOwningThread {
 public:
-    nsAutoOwningThread() { mThread = NS_CurrentThread(); }
+    nsAutoOwningThread() { mThread = PR_GetCurrentThread(); }
     void *GetThread() const { return mThread; }
 
 private:
@@ -84,12 +69,12 @@ private:
 #define NS_ASSERT_OWNINGTHREAD(_class) \
   NS_CheckThreadSafe(_mOwningThread.GetThread(), #_class " not thread-safe")
 
-#else // !(defined(NS_DEBUG) && defined(NS_MT_SUPPORTED))
+#else // !(defined(NS_DEBUG))
 
 #define NS_DECL_OWNINGTHREAD            /* nothing */
 #define NS_ASSERT_OWNINGTHREAD(_class)  ((void)0)
 
-#endif // !(defined(NS_DEBUG) && defined(NS_MT_SUPPORTED))
+#endif // !(defined(NS_DEBUG))
 
 class nsAutoRefCnt {
 
@@ -696,8 +681,6 @@ NS_IMETHODIMP_(nsrefcnt) Class::Release(void)                                 \
  *
  */
 
-#if defined(NS_MT_SUPPORTED)
-
 /**
  * Use this macro to implement the AddRef method for a given <i>_class</i>
  * @param _class The name of the class implementing the method
@@ -735,22 +718,6 @@ NS_IMETHODIMP_(nsrefcnt) _class::Release(void)                                \
   return count;                                                               \
 }
 
-#else  // defined(NS_MT_SUPPORTED)
-
-/**
- * Convenience macro for implementing all nsISupports methods for
- * a simple class.
- * @param _class The name of the class implementing the method
- * @param _classiiddef The name of the #define symbol that defines the IID
- * for the class (e.g. NS_ISUPPORTS_IID)
- */
-
-#define NS_IMPL_THREADSAFE_ADDREF(_class)  NS_IMPL_ADDREF(_class)
-
-#define NS_IMPL_THREADSAFE_RELEASE(_class) NS_IMPL_RELEASE(_class)
-
-#endif /* !NS_MT_SUPPORTED */
-
 #define NS_IMPL_THREADSAFE_ISUPPORTS0(_class)                                 \
   NS_IMPL_THREADSAFE_ADDREF(_class)                                           \
   NS_IMPL_THREADSAFE_RELEASE(_class)                                          \
