@@ -112,10 +112,10 @@ nsGREDirServiceProvider::GetFile(const char *prop, PRBool *persistant, nsIFile *
   *persistant = PR_TRUE;
 
   //---------------------------------------------------------------
-  // Note that by returning a valid localFile's for NS_GRE_DIR and
-  // NS_GRE_COMPONENT_DIR, your app is indicating to XPCOM that 
-  // it found a GRE version with which it's compatible with and 
-  // intends to be "run against" that GRE.
+  // Note that by returning a valid localFile's for NS_GRE_DIR,
+  // your app is indicating to XPCOM that it found a GRE version
+  // with which it's compatible with and intends to be "run against"
+  // that GRE.
   //
   // Please see http://www.mozilla.org/projects/embedding/MRE.html
   // for more info on GRE.
@@ -124,16 +124,7 @@ nsGREDirServiceProvider::GetFile(const char *prop, PRBool *persistant, nsIFile *
   {
     rv = GetGreDirectory(getter_AddRefs(localFile));
   }    
-  else if(strcmp(prop, NS_GRE_COMPONENT_DIR) == 0)
-  {
-    rv = GetGreDirectory(getter_AddRefs(localFile));
-    if(NS_SUCCEEDED(rv)) {
-      nsEmbedCString leaf;
-      leaf.Assign("components");
-      rv = localFile->AppendRelativeNativePath(leaf);
-    }
-  }    
-
+  
   if(!localFile || NS_FAILED(rv))
     return rv;
 
@@ -192,7 +183,7 @@ char* GetCurrentProcessDirectory()
     // us try this for unix:
     //	- if MOZILLA_FIVE_HOME is defined, that is it
     //	- else give the current directory
-    char buf[1024];
+    char buf[MAXPATHLEN];
 
     // The MOZ_DEFAULT_MOZILLA_FIVE_HOME variable can be set at configure time with
     // a --with-default-mozilla-five-home=foo autoconf flag.
@@ -214,7 +205,10 @@ char* GetCurrentProcessDirectory()
 
     if (moz5)
     {
-        resultPath = strdup(moz5);
+        if (realpath(moz5, buf))
+            resultPath = strdup(buf);
+        else
+            resultPath = strdup(moz5);
         return resultPath;
     }
     else
@@ -523,7 +517,7 @@ nsGREDirServiceProvider::AddGRELocationToPath()
   }
 
   if (mPathEnvString)
-    PR_Free(mPathEnvString);
+    PR_smprintf_free(mPathEnvString);
 
   mPathEnvString = PR_smprintf("%s=%s;%s",
                                XPCOM_SEARCH_KEY,
