@@ -235,7 +235,6 @@ nsresult nsWebShellWindow::Initialize(nsIXULWindow* aParent,
                                       nsIAppShell* aShell, nsIURI* aUrl, 
                                       PRBool aCreatedVisible,
                                       PRBool aLoadDefaultPage,
-                                      PRUint32 aZlevel,
                                       PRInt32 aInitialWidth, PRInt32 aInitialHeight,
                                       PRBool aIsHiddenWindow, nsWidgetInitData& widgetInitData)
 {
@@ -246,7 +245,6 @@ nsresult nsWebShellWindow::Initialize(nsIXULWindow* aParent,
   
   mShowAfterLoad = aCreatedVisible;
   mLoadDefaultPage = aLoadDefaultPage;
-  mZlevel = aZlevel;
   
   // XXX: need to get the default window size from prefs...
   // Doesn't come from prefs... will come from CSS/XUL/RDF
@@ -272,7 +270,7 @@ nsresult nsWebShellWindow::Initialize(nsIXULWindow* aParent,
   nsCOMPtr<nsIBaseWindow> parentAsWin(do_QueryInterface(aParent));
   if (parentAsWin) {
     parentAsWin->GetMainWidget(getter_AddRefs(parentWidget));
-    mParentWindow = getter_AddRefs(NS_GetWeakReference(aParent));
+    mParentWindow = do_GetWeakReference(aParent);
   }
 
   mWindow->SetClientData(this);
@@ -406,7 +404,7 @@ nsWebShellWindow::HandleEvent(nsGUIEvent *aEvent)
       case NS_MOVE: {
         // persist position, but not immediately, in case this OS is firing
         // repeated move events as the user drags the window
-        eventWindow->SetPersistenceTimer(PR_FALSE, PR_TRUE, PR_FALSE);
+        eventWindow->SetPersistenceTimer(PAD_POSITION);
         break;
       }
       case NS_SIZE: {
@@ -418,18 +416,32 @@ nsWebShellWindow::HandleEvent(nsGUIEvent *aEvent)
         // persist size, but not immediately, in case this OS is firing
         // repeated size events as the user drags the sizing handle
         if (NS_FAILED(eventWindow->GetLockedState(chromeLock)) || !chromeLock)
-          eventWindow->SetPersistenceTimer(PR_TRUE, PR_FALSE, PR_TRUE);
+          eventWindow->SetPersistenceTimer(PAD_SIZE | PAD_MISC);
         result = nsEventStatus_eConsumeNoDefault;
         break;
       }
       case NS_SIZEMODE: {
         nsSizeModeEvent* modeEvent = (nsSizeModeEvent*)aEvent;
+
+        // an alwaysRaised (or higher) window will hide any newly opened
+        // normal browser windows. here we just drop a raised window
+        // to the normal zlevel if it's maximized. we make no provision
+        // for automatically re-raising it when restored.
+        if (modeEvent->mSizeMode == nsSizeMode_Maximized) {
+          PRUint32 zLevel;
+          eventWindow->GetZLevel(&zLevel);
+          if (zLevel > nsIXULWindow::normalZ)
+            eventWindow->SetZLevel(nsIXULWindow::normalZ);
+        }
+
         aEvent->widget->SetSizeMode(modeEvent->mSizeMode);
+
         // persist mode, but not immediately, because in many (all?)
         // cases this will merge with the similar call in NS_SIZE and
         // write the attribute values only once.
-        eventWindow->SetPersistenceTimer(PR_FALSE, PR_FALSE, PR_TRUE);
+        eventWindow->SetPersistenceTimer(PAD_MISC);
         result = nsEventStatus_eConsumeDoDefault;
+
         // Note the current implementation of SetSizeMode just stores
         // the new state; it doesn't actually resize. So here we store
         // the state and pass the event on to the OS. The day is coming
@@ -595,8 +607,11 @@ nsWebShellWindow::HandleEvent(nsGUIEvent *aEvent)
 
             // since the window has been activated, replace persistent size data
             // with the newly activated window's
-            if (eventWindow->mChromeLoaded)
-              eventWindow->PersistPositionAndSize(PR_TRUE, PR_TRUE, PR_TRUE);
+            if (eventWindow->mChromeLoaded) {
+              eventWindow->PersistentAttributesDirty(
+                             PAD_POSITION | PAD_SIZE | PAD_MISC);
+              eventWindow->SavePersistentAttributes();
+            }
 
             break;
           }
@@ -722,7 +737,7 @@ NS_IMETHODIMP nsWebShellWindow::LoadMenuItem(
     // Try to find the key node.
     nsCOMPtr<nsIDocument> document;
     nsCOMPtr<nsIContent> content = do_QueryInterface(domElement);
-    if (NS_FAILED(rv = content->GetDocument(*getter_AddRefs(document)))) {
+    if (NS_FAILED(rv = content->GetDocument(getter_AddRefs(document)))) {
       NS_ERROR("Unable to retrieve the document.");
       return rv;
     }
@@ -957,7 +972,7 @@ void nsWebShellWindow::DynamicLoadMenus(nsIDOMDocument * aDOMDoc, nsIWidget * aP
       }
 
       nsCOMPtr<nsIPresContext> presContext;
-      if (NS_FAILED(rv = docViewer->GetPresContext(*getter_AddRefs(presContext)))) {
+      if (NS_FAILED(rv = docViewer->GetPresContext(getter_AddRefs(presContext)))) {
           NS_ERROR("Unable to retrieve the doc viewer's presentation context.");
           return;
       }
@@ -1157,23 +1172,20 @@ nsWebShellWindow::DestroyModalDialogEvent(PLEvent *aEvent)
 }
 
 void
-nsWebShellWindow::SetPersistenceTimer(PRBool aSize, PRBool aPosition, PRBool aMode)
+nsWebShellWindow::SetPersistenceTimer(PRUint32 aDirtyFlags)
 {
   PR_Lock(mSPTimerLock);
   if (mSPTimer) {
     mSPTimer->SetDelay(SIZE_PERSISTENCE_TIMEOUT);
-    mSPTimerSize |= aSize;
-    mSPTimerPosition |= aPosition;
-    mSPTimerMode |= aMode;
+    PersistentAttributesDirty(aDirtyFlags);
   } else {
     nsresult rv;
     mSPTimer = do_CreateInstance("@mozilla.org/timer;1", &rv);
     if (NS_SUCCEEDED(rv)) {
+      NS_ADDREF_THIS(); // for the timer, which holds a reference to this window
       mSPTimer->InitWithFuncCallback(FirePersistenceTimer, this,
                                      SIZE_PERSISTENCE_TIMEOUT, nsITimer::TYPE_ONE_SHOT);
-      mSPTimerSize = aSize;
-      mSPTimerPosition = aPosition;
-      mSPTimerMode = aMode;
+      PersistentAttributesDirty(aDirtyFlags);
     }
   }
   PR_Unlock(mSPTimerLock);
@@ -1184,11 +1196,7 @@ nsWebShellWindow::FirePersistenceTimer(nsITimer *aTimer, void *aClosure)
 {
   nsWebShellWindow *win = NS_STATIC_CAST(nsWebShellWindow *, aClosure);
   PR_Lock(win->mSPTimerLock);
-  win->PersistPositionAndSize(win->mSPTimerPosition, win->mSPTimerSize,
-                              win->mSPTimerMode);
-  win->mSPTimerSize = PR_FALSE;
-  win->mSPTimerPosition = PR_FALSE;
-  win->mSPTimerMode = PR_FALSE;
+  win->SavePersistentAttributes();
   PR_Unlock(win->mSPTimerLock);
 }
 
@@ -1256,7 +1264,7 @@ nsWebShellWindow::OnStateChange(nsIWebProgress *aProgress,
       return NS_OK;
 
     nsCOMPtr<nsIDocument> doc;
-    docv->GetDocument(*getter_AddRefs(doc));
+    docv->GetDocument(getter_AddRefs(doc));
     if (!doc)
       return NS_OK;
 
@@ -1383,7 +1391,7 @@ nsCOMPtr<nsIDOMDocument> nsWebShellWindow::GetNamedDOMDoc(const nsAString & aWeb
     return domDoc;
 
   nsCOMPtr<nsIDocument> doc;
-  docv->GetDocument(*getter_AddRefs(doc));
+  docv->GetDocument(getter_AddRefs(doc));
   if (doc)
     return nsCOMPtr<nsIDOMDocument>(do_QueryInterface(doc));
 
@@ -1407,9 +1415,8 @@ void nsWebShellWindow::LoadContentAreas() {
     nsCOMPtr<nsIDocumentViewer> docViewer = do_QueryInterface(contentViewer);
     if (docViewer) {
       nsCOMPtr<nsIDocument> doc;
-      docViewer->GetDocument(*getter_AddRefs(doc));
-      nsCOMPtr<nsIURI> mainURL;
-      doc->GetDocumentURL(getter_AddRefs(mainURL));
+      docViewer->GetDocument(getter_AddRefs(doc));
+      nsIURI *mainURL = doc->GetDocumentURL();
       if (mainURL) {
         nsCAutoString search;
         nsCOMPtr<nsIURL> url = do_QueryInterface(mainURL);
@@ -1422,7 +1429,7 @@ void nsWebShellWindow::LoadContentAreas() {
 
   // content URLs are specified in the search part of the URL
   // as <contentareaID>=<escapedURL>[;(repeat)]
-  if (searchSpec.Length() > 0) {
+  if (!searchSpec.IsEmpty()) {
     PRInt32     begPos,
                 eqPos,
                 endPos;
@@ -1491,7 +1498,7 @@ PRBool nsWebShellWindow::ExecuteCloseHandler()
         nsCOMPtr<nsIDocumentViewer> docViewer;
         nsCOMPtr<nsIPresContext> presContext;
         docViewer = do_QueryInterface(contentViewer);
-        if (docViewer && NS_SUCCEEDED(docViewer->GetPresContext(*getter_AddRefs(presContext)))) {
+        if (docViewer && NS_SUCCEEDED(docViewer->GetPresContext(getter_AddRefs(presContext)))) {
           nsEventStatus status = nsEventStatus_eIgnore;
           nsMouseEvent event;
           event.eventStructType = NS_EVENT;
@@ -1534,8 +1541,7 @@ nsWebShellWindow::AttributeChanged(nsIDocument *aDocument,
                                    nsIContent*  aContent,
                                    PRInt32      aNameSpaceID,
                                    nsIAtom*     aAttribute,
-                                   PRInt32      aModType, 
-                                   nsChangeHint aHint)
+                                   PRInt32      aModType)
 {
 #if 0
   //printf("AttributeChanged\n");
@@ -1641,7 +1647,7 @@ NS_IMETHODIMP nsWebShellWindow::Destroy()
    if(docv)
       {
       nsCOMPtr<nsIDocument> doc;
-      docv->GetDocument(*getter_AddRefs(doc));
+      docv->GetDocument(getter_AddRefs(doc));
       if(doc)
          doc->RemoveObserver(NS_STATIC_CAST(nsIDocumentObserver*, this));
       }
@@ -1652,7 +1658,8 @@ NS_IMETHODIMP nsWebShellWindow::Destroy()
   if (mSPTimer) {
     mSPTimer->Cancel();
     mSPTimer = nsnull;
-    PersistPositionAndSize(mSPTimerPosition, mSPTimerSize, mSPTimerMode);
+    SavePersistentAttributes();
+    NS_RELEASE_THIS(); // the timer held a reference to us
   }
   PR_Unlock(mSPTimerLock);
 
