@@ -170,8 +170,6 @@ nsInstallInfo::~nsInstallInfo()
 static NS_DEFINE_IID(kISoftwareUpdateIID, NS_ISOFTWAREUPDATE_IID);
 static NS_DEFINE_IID(kSoftwareUpdateCID,  NS_SoftwareUpdate_CID);
 
-static NS_DEFINE_IID(kIZipReaderIID, NS_IZIPREADER_IID);
-static NS_DEFINE_IID(kZipReaderCID,  NS_ZIPREADER_CID);
 
 MOZ_DECL_CTOR_COUNTER(nsInstall)
 
@@ -1258,7 +1256,7 @@ nsInstall::LoadResources(JSContext* cx, const nsString& aBaseName, jsval* aRetur
             JSString* propValJSStr = JS_NewUCStringCopyZ(cx, NS_REINTERPRET_CAST(const jschar*, pVal.get()));
             jsval propValJSVal = STRING_TO_JSVAL(propValJSStr);
             nsString UCKey = NS_ConvertUTF8toUCS2(pKey);
-            JS_SetUCProperty(cx, res, UCKey.get(), UCKey.Length(), &propValJSVal);
+            JS_SetUCProperty(cx, res, (jschar*)UCKey.get(), UCKey.Length(), &propValJSVal);
         }
     }
 
@@ -2443,7 +2441,7 @@ nsInstall::CurrentUserNode(nsString& userRegNode)
 PRBool
 nsInstall::BadRegName(const nsString& regName)
 {
-    if ( regName.Length() == 0 )
+    if ( regName.IsEmpty() )
         return PR_TRUE;
 
     if ((regName.First() == ' ' ) || (regName.Last() == ' ' ))
